//////////////////////////////////////////
// Generic index-check Array            //
// (NOT dynamically allocated by design //
//////////////////////////////////////////

// LB 6'th dec 2001

// usage: CArray<arraySize, Dataype>name;
// after that it can be used like a normal array

#ifndef __CARRAY_H__
#define __CARRAY_H__

#include "wpconsole.h"

#include "debug.h"    
#include "wpconsole.h"
#include <string>
#undef  DBGFILE
#define DBGFILE "CArray.hpp"

#define INDEXCHECKEDARRAYS

template <const int c, class LB=char>
class CArray
{
	public:
		CArray(std::string gcc="noname"); 
		

		inline LB& operator [](unsigned long int);
	
		inline CArray& prepare(char*, int);
		
		bool geterror() { return error; }
		unsigned long int get_max() { return maxxx; }
		
	private:

		LB internalArray[c];
		const unsigned long int maxxx;
		bool error;
		const std::string arrayName;
		char *fn;
		int source_line;

		inline LB* operator&(); // gcc 3.2 has problems with it ... but don't want to delete it

		// prevent copying array
		CArray(const CArray&); 
		CArray& operator=(const CArray&); 

};


template <const int c, class LB> CArray<c, LB>::CArray(std::string s): maxxx(c), error(0), arrayName(s), fn(NULL)
{
    //printf("c_tor call test %s[%u]\n", arrayName.c_str(), maxxx);	
}


template <const int c, class LB> LB& CArray<c, LB>::operator[](unsigned long int  in)
{
    #ifdef INDEXCHECKEDARRAYS

	if (in>=maxxx)
	{
		char tmp[512];
		error=1;

		if (fn==NULL)  sprintf(tmp, "index out of range error: %s[%lu] (0..%lu)\n",arrayName.c_str(), in, maxxx-1);
		else 		   sprintf(tmp, "index out of range error: %s[%lu] (0..%lu) file:%s %i\n",arrayName.c_str(), in, maxxx-1, fn, source_line);

		/*if (server_data.errors_to_console)			
		{
		   Console.send(tmp); 
		}*/

		LogError(tmp);

		Console.send(tmp); // errors can occur before server_data is read !
		                     // that's why I'm currently always sending those errors to console.
		                     // note that in that case they goto errorlog.0_0_0_0.txt in lw root dir
		                     // no matter what's set in server.scp

		return internalArray[0];
	}
    #endif
	
	return internalArray[in];
}

template <const int c, class LB> LB* CArray<c, LB>::operator&()
{
	return &internalArray[0];
}

template <const int c, class LB> CArray<c, LB>& CArray<c, LB>::prepare(char* f, int l)
{
   fn = f;
   source_line = l;
   return *this;

}


// like CArray, but you set the size in the constructor (allows to use variables to set the size) //Thrawn
template <class T=char>
class CDynArray
{
	public:
		CDynArray(int size, std::string gcc="noname"); 
		~CDynArray(); 
		
		inline T& operator [](unsigned long int);
	
		inline CDynArray& prepare(char*, int);
		
		bool geterror() { return error; }
		unsigned long int GetSize() { return m_dwSize; }
		
	private:

		T* internalArray;
		const unsigned long int m_dwSize;
		bool error;
		const std::string arrayName;
		char *fn;
		int source_line;

		inline T* operator&(); // gcc 3.2 has problems with it ... but don't want to delete it

		// prevent copying array
		CDynArray(const CDynArray&); 
		CDynArray& operator=(const CDynArray&); 

};


template <class T> CDynArray<T>::CDynArray(int size, std::string s):  internalArray((T*)malloc(size*sizeof(T))), m_dwSize(size), error(0),	arrayName(s), fn(NULL)
{
    //printf("c_tor call test %s[%u]\n", arrayName.c_str(), maxxx);	

}

template <class T> CDynArray<T>::~CDynArray()
{
	free(internalArray);
}


template <class T> T& CDynArray<T>::operator[](unsigned long int index)
{
    #ifdef INDEXCHECKEDARRAYS

	if (index>=m_dwSize)
	{
		char tmp[512];
		error=1;

		if (fn==NULL)  sprintf(tmp, "index out of range error: %s[%lu] (0..%lu)\n",arrayName.c_str(), index, m_dwSize-1);
		else 		   sprintf(tmp, "index out of range error: %s[%lu] (0..%lu) file:%s %i\n",arrayName.c_str(), index, m_dwSize-1, fn, source_line);

		LogError(tmp);

//		Console.send(tmp); // errors can occur before server_data is read !
		                     // that's why I'm currently always sending those errors to console.
		                     // note that in that case they goto errorlog.0_0_0_0.txt in lw root dir
		                     // no matter what's set in server.scp

		return internalArray[0];
	}
    #endif
	
	return internalArray[index];
}

template <class T> T* CDynArray<T>::operator&()
{
	return &internalArray[0];
}

template <class T> CDynArray<T>& CDynArray<T>::prepare(char* f, int l)
{
   fn = f;
   source_line = l;
   return *this;

}

/// Index checked vector class (LB 22.4.2003) !
/// well, I know at() of ordianry vectors IS already index checked, 
/// but I dont like to use at() for many reasons.

#include <vector>
using namespace std;

template<class T> class SecVec : public vector<T>
{	
	public:

		SecVec(string gcc="noname"): vector<T>(), fn(NULL), VecName(gcc) 
		{ 
			T dummy2;                            // add a dummy element to the dummy vector to have something to return 
			                                     // in case of invalid positions ([0] doesnt work in case of an empty vector!)
			dummy.push_back(dummy2); 	
		}

		SecVec(int s, string gcc="noname"): vector<T>(s), fn(NULL), VecName(gcc)
		{ 
			T dummy2;  
			dummy.push_back(dummy2); 			
		}

        T& operator[](int i) 
		{ 			
			if (i>=(int)vector<T>::size() || i<0) 
			{ 			
				char tmp[512];
		        if (fn==NULL)  sprintf(tmp, "SVEC-index out of range error: %s [%u] (0..%u)\n", VecName.c_str(), i, vector<T>::size() );
		        else 		   sprintf(tmp, "SVEC-index out of range error: %s [%u] (0..%u) file:%s %i\n",VecName.c_str(),i, vector<T>::size(), fn, source_line);
		        LogError(tmp);
				return dummy[0];   
			}
		    return vector<T>::operator[](i); 
		}

		const T& operator[](int i) const 
		{ 		
			if (i>=(int)vector<T>::size() || i<0) 
			{			
				char tmp[512];
		        if (fn==NULL)  sprintf(tmp, "SVEC-index out of range error: %s [%u] (0..%u)\n",VecName.c_str(), i, vector<T>::size() );
		        else 		   sprintf(tmp, "SVEC-index out of range error: %s [%u] (0..%u) file:%s %i\n",VecName.c_str(),i, vector<T>::size(), fn, source_line);
		        LogError(tmp);
				return dummy[0];   
				
			}
			return vector<T>::operator[](i); 
		}

		inline SecVec& prepare(char* f, int s)
		{
			fn=f;
			source_line=s;
			return *this;
		}

	private:

	  vector<T> dummy;
	  char *fn;
	  int source_line;
	  /*const*/ std::string VecName; // actual constness hits a couple of compiler bugs

};

// note: can only instaciate intrinsic data types (char, float, int)

#ifdef _MSC_VER
#pragma warning(disable : 4183 ) // msvc emmits a false warning compiling this
#endif

template<class T> class dynArr2D
{
	public:

	inline dynArr2D<T>::dynArr2D<T>(unsigned int row, unsigned int col)
	{
	  unsigned int i;
	  ptr = new (T* [row]);      // Allocate row pointers
      for (i = 0; i < row; i++)
	  {
        ptr[i] = new T[col];    // Allocate rows
	  }
	  rows=row;
	  cols=col;
	}

	inline ~dynArr2D<T>()
	{
	  unsigned int i;
	  for (i = 0; i < rows; i++)
	  {
        delete [] ptr[i];            //  Deallocate rows
	  }
      delete [] ptr;                  //  Deallocate row pointers
	}

	inline T& operator() (unsigned int row, unsigned int col)
	{
	  if (row>=rows || col>=cols)
	  {
		char tmp[512];
		sprintf(tmp, "error in dynarr2d, %i/%i %i/%i\n", row, rows, col,cols);
		LogError(tmp);
		return ptr[0][0];
	  }
	  return ptr[row][col];
	}

	inline T** getPtr(void) { return ptr ; }

	private:

	 T** ptr;
	 unsigned int rows;
	 unsigned int cols;
};

#ifdef _MSC_VER
#pragma warning(default:4183)
#endif

#endif
