/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

#include "LoS.h"

#include "mapstuff.h"
#include "rectangle.h"
#include "regions.h"
#include "multi.h"
#include "mapS.h"
#include "tiledata.h"
#include <math.h>

#include "wolfpack.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "LoS.cpp"

/////////////////////
// Name:	cLoS
// Purpose:	new implementation for Line of Sight checking
// History:	by Duke, 26.12.01
//
class cLoS
{
protected:
	cCoord pos1;
	cCoord pos2;
	cRectangle lineRect;	// rectangle bounding the line
	bool noXdelta, noYdelta, noZdelta;
	int Xdir, Ydir, Zdir;
	float incX, incY, incZ;
	float incYhalf;
	short minZ,maxZ;		// just needed for vertikal lines

	bool isInsideRectangle(short x, short y)	{return lineRect.includes(x,y);}
	short getZatXY(short x, short y);
	bool isBlocking(short id);
	bool crossesTile(short x, short y);
	bool isBlockedByHeight(short id, short baseZ, short lineZ);
	bool isBlockedByMulti(PC_ITEM pi);
	bool isBlockedByTile(PC_ITEM pi);
public:
	cLoS(cCoord& posA,cCoord& posB);
	~cLoS() { }
	bool isBlockedByItem(PC_ITEM pi);
	bool isBlockedByLand(short x, short y);
	bool isBlockedByStatic(short x, short y);
	void firstTile(short* x, short* y);
	bool nextTile(short* x, short* y);
};

cLoS::cLoS(cCoord& posA,cCoord& posB) : lineRect(posA.x,posA.y,posB.x,posB.y)
{
	pos1=posA;
	pos2=posB;

	noXdelta=noYdelta=noZdelta=false;
	if (pos1.x == pos2.x)
		noXdelta=true;
	if (pos1.y == pos2.y)
		noYdelta=true;

	Xdir=1, Ydir=1, Zdir=1;
	if (pos1.x > pos2.x)
		Xdir = -1;
	if (pos1.y > pos2.y)
		Ydir = -1;
	if (pos1.z > pos2.z)
		Zdir = -1;

	incX=incY=incZ=1;	// initialize to nonzero values
	if (!noXdelta)		// if there is a x-dimension, both incY and incZ are based on x
	{
		incY= float (pos2.y-pos1.y)/(pos2.x-pos1.x);
		incZ= fabs(float (pos2.z-pos1.z)/(pos2.x-pos1.x));
	}
	else 
	{
		if (!noYdelta)	// if all x are the same, both incX and incZ are based on y
		{
			incX= float (pos2.x-pos1.x)/(pos2.y-pos1.y);
			incZ= fabs(float (pos2.z-pos1.z)/(pos2.y-pos1.y));
		}
	}
	incYhalf = fabs(incY)*0.5;	// just to speed up repeated calculations

	if (noXdelta && noYdelta)
	{
		minZ = min(pos1.z,pos2.z);
		maxZ = max(pos1.z,pos2.z);
	}
}

short cLoS::getZatXY(short x, short y)
{
	if (!noXdelta)
		return (short) ( (float) pos1.z + (float) (abs(x-pos1.x)) *incZ* (float)Zdir );
	else
		return (short) ( (float) pos1.z + (float) (abs(y-pos1.y)) *incZ* (float)Zdir );
	// if both Xdelta and Ydelta are zero, the return value is undefined ie. it's 0, but meaningless !
}

bool cLoS::crossesTile(short x, short y)
{
	if (!lineRect.includes(x,y))
		return false;	// the tile is far from where the line is

	if (noXdelta || noYdelta)	// AND inside the box (see above)
		return true;			// => a vertical line

	// let's think of the tile as a square. The borders have a length of 1.
	// x/y is in the middle of the tile, so the borders are at +/- 0.5

	// determine the y-values of our *line* for the x-values of the square's borders
	float yLine = float(pos1.y + (x-pos1.x)*incY*Ydir);
	if ( (float(yLine + incYhalf)) < (float(y - 0.5)) )
		return false;	// the line goes through below the square
	if ( (float(yLine - incYhalf)) < (float(y + 0.5)) )
		return false;	// the line goes through above

	return true;
}

bool cLoS::isBlocking(short id)
{
	// Walls, Chimneys, ovens, not fences
	if (((id>=   6 && id<= 748))||((id>=  761)&&(id<=  881))||
		((id>= 895 && id<=1006))||((id>= 1057)&&(id<= 1061))||
		( id==1072)||(id==1073) ||((id>= 1080)&&(id<= 1166))||
		((id>=2347)&&(id<=2412))||((id>=16114)&&(id<=16134))||
		((id>=8538)&&(id<=8553))||((id>= 9535)&&(id<= 9555))||
		(id==12583))
		return true;
	
	// Doors, not gates
	if (( id>=1653 && id<=1782 )||( id>=8173 && id<=8188 ))
		return true;
	
	// Floors & Flat Roofing (Attacking through floors Roofs)
	if (((id>=1169 && id<=1413))||((id>=1508)&&(id<=1514))||
		((id>=1579 && id<=1586))||((id>=1591)&&(id<=1598)))
		return true;
	
	return false;
}

bool cLoS::isBlockedByHeight(short id, short baseZ, short zLine)
{
	const tileData *tile;
	tile = TileData->SeekTile(id);
	if ( (noXdelta && noYdelta) &&		// a vertikal line, zLine is undefined, using minZ & maxZ
		!(maxZ < baseZ) && !(minZ > (baseZ + tile->height)) )
	{
		if (isBlocking(id))
			return true;
	}
	else
	{
		float zLineTop    = float (zLine + 0.5*incZ);
		float zLineBottom = float (zLine - 0.5*incZ);
		if ( !(zLineTop < baseZ || zLineBottom > (baseZ + tile->height)) ) // not above or below
		{
			if (tile->height || incZ)
			{
				if (isBlocking(id))
					return true;
			}
		}
	}
	return false;
}

bool cLoS::isBlockedByItem(PC_ITEM pi)
{
//	if (!pi->isMulti())
	if (pi->id1 < 0x40)	// unbelievable, but this significantly helps performance (Duke)
	{
		if (!this->crossesTile(pi->pos.x,pi->pos.y))
			return false;
		if (this->isBlockedByTile(pi))
			return true;
	}
	else
	{
		cRectangle maxRect( pi->pos.x - BUILDRANGE,pi->pos.y - BUILDRANGE,
							pi->pos.x + BUILDRANGE,pi->pos.y + BUILDRANGE);
		if (!maxRect.overlaps(lineRect))
			return false;
		if (this->isBlockedByMulti(pi))
			return true;
	}
	return false;
}

bool cLoS::isBlockedByMulti(PC_ITEM pi)
{
	int LoopExit=0;
	singleMultiPart mult;

	assert(pi->id()>=0x4000);

	Multi->multiIteratorStart(pi->id()-0x4000);

	while (Multi->multiIteratorNext(mult) && ++LoopExit<MAXLOOPS)
	{
	  if (mult.visibility && this->crossesTile(pi->pos.x+mult.x, pi->pos.y+mult.y))
	  {
			short zLine=getZatXY(pi->pos.x+mult.x, pi->pos.y+mult.y);
			if (isBlockedByHeight(mult.tile, pi->pos.z+mult.z, zLine))
			return true;
	  }
	}
	return false;
}

bool cLoS::isBlockedByTile(PC_ITEM pi)
{
	if (pi->visible==0)	// normally visible (!)
	{
		short zLine=getZatXY(pi->pos.x,pi->pos.y);
		
		if (isBlockedByHeight(pi->id(), pi->pos.z, zLine))
			return true;
	}
	return false;
}

bool cLoS::isBlockedByLand(short x, short y)
{
	cellData map1;	
	cCoord c(x,y,0,0); // NOTE: needs to be changed when multiple maps get supported

	AllMaps->SeekMap(c, map1);
	short id = map1.TID;
	if (id != 2)
	{
#if 0
		if (TileData->isLandBreakLos(id))
			return true;
#endif
		if ( // Mountain walls
			((id>= 431)&&(id<= 432))||
			((id>= 467)&&(id<= 475))||
			((id>= 543)&&(id<= 560))||
			((id>=1754)&&(id<=1757))||
			((id>=1787)&&(id<=1789))||
			((id>=1821)&&(id<=1824))||
			((id>=1851)&&(id<=1854))||
			((id>=1881)&&(id<=1884))
			)
		{
			return true;
		}
	}
	return false;
}

bool cLoS::isBlockedByStatic(short x, short y)
{
	short zLine=getZatXY(x,y);
	
	cellData stat;
	cCoord c(x,y,0,0); // NOTE: needs to be changed when implenting muliple maps

    AllMaps->mapStaticIteratorStart(c);

	int loopexit=0;
	while ( (AllMaps->mapStaticIteratorNext(stat)) && (++loopexit < MAXLOOPS) )
	{
		if (isBlockedByHeight(stat.TID, stat.z, zLine))
			return true;
	}
	return false;
}

// firstTile and nextTile will iterate through the tiles *along* the line
void cLoS::firstTile(short* x, short* y)
{
	*x=pos1.x;
	*y=pos1.y;
}

bool cLoS::nextTile(short* x, short* y)
{
	if (noXdelta && noYdelta)
		return false;

	if (noXdelta)
	{
		*y += Ydir;
	}
	else if (noYdelta)
	{
		*x += Xdir;
	}
	else
	{
		// this part determines if our line leaves the tile through a side border or the top border
		// the variable names refer to a line that goes lower left to top right
		// xdir and ydir assure that it also works for other directions
		float nextRightBorderY = pos1.y + (*x - pos1.x + Xdir*0.5)*incY;
		float nextTopBorderY = *y + (Ydir*0.5);
		if ((nextTopBorderY-nextRightBorderY)*Ydir >= 0)
		{
			*x += Xdir;
		}
		if ((nextTopBorderY-nextRightBorderY)*Ydir <= 0)
		{
			*y += Ydir;
		}
	}
	if (Xdir==1)
	{
		if (*x > pos2.x) return false;
	}
	else
	{
		if (*x < pos2.x) return false;
	}
	if (Ydir==1)
	{
		if (*y > pos2.y) return false;
	}
	else
	{
		if (*y < pos2.y) return false;
	}

	return true;
}

int line_of_sight2(cCoord &source, cCoord &target, P_ITEM TargetItem)
{
	cLoS line(source,target);

	// checking map and static items first
	short x,y;
	int loopexit=0;
	line.firstTile(&x,&y);
	do
	{
		if (line.isBlockedByLand(x,y))
			return false;
		if (line.isBlockedByStatic(x,y))
			return false;
	}
	while (line.nextTile(&x,&y) && (++loopexit < MAXLOOPS) ); 
	// note: on corners we get a wrap-around effect and a infite loop  without loop-check!

	// then check the dynamic tiles because they usually don't block
	PC_ITEM pi;
	loopexit=0;
	RegGrid3x3Iterator4Items rg3ii(source);
	while((pi = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (line.isBlockedByItem(pi))
		{
			if (TargetItem)		// checking LoS to an ITEM ??
				if (pi->isSameAs(TargetItem))
					return true;
			return false;
		}
	}
	// later we can add a check for characters here

	return true;
}

int line_of_sight(int s, cCoord &source, cCoord &target, int checkfor)
{
	if (source.distance(target) >18) return 0;
	int los2 = line_of_sight2( source, target);
	return los2;
}

// If we are checking LoS to an item, we need to know it's not the item itself that is blocking
int line_of_sight(int s, P_CHAR pc, P_ITEM pi, int checkfor)
{
	if (pc->pos.distance(pi->pos) >18) return 0;

	cCoord Eyes = pc->pos;
//	Eyes.z += 14;	// you can't see with your feet ;)
	int los2 = line_of_sight2(Eyes , pi->pos, pi);
	return los2;
}


