//==================================================================================
//
//      LoneWolf Emu (WP)
//	UO Server Emulation Program
//
//	Copyright 1997, 98 by Marcus Rating (Cironian)
//  Copyright 2001 by holders identified in authors.txt
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//	* In addition to that license, if you are running this program or modified
//	* versions of it on a public system you HAVE TO make the complete source of
//	* the version used by you available or provide people with a location to
//	* download it.
//
//
//
//	Current Homepage: home.tiscalinet.de/aduke
//========================================================================================


#ifndef __unix__
 
#endif

#include "basics.h"
#if defined(__unix__)
#include <signal.h>
#include <curses.h>
#endif
#include "verinfo.h"
#include "speech.h"
#include "SndPkg.h"
#include "sregions.h"
#include "remadmin.h"

#include "accounts.h"
#include "admin.h"
#include "boats.h"
#include "books.h"
#include "Client.h"
#include "cmdtable.h"
#include "combat.h"
#include "commands.h"
#include "guilds.h"
#include "gumps.h"
#include "house.h"
#include "html.h"
#include "im.h"
#include "itemid.h"
#include "magic.h"
#include "magic_new.h"
#include "spells.h"
#include "mapstuff.h"
#include "msgboard.h"
#include "network.h"
#include "newbie.h"
#include "npcinfo.h"
#include "regions.h"
#include "skills.h"
#include "targeting.h"
#include "teleport.h"
#include "TmpEff.h"
#include "Timing.h"
#include "Trade.h"
#include "trigger.h"
#include "virtues.h"
#include "weight.h"
#include "worldmain.h"
#include "mapS.h"
#include "multi.h"
#include "tiledata.h"
#include "Partysystem.h"
#include "titles.h"

#include "wolfpack.h"

#include "debug.h"
#undef DBGFILE
#define DBGFILE "lonewolf.cpp"

#include "wip.h"
#include <algorithm>
#include "assert.h"


#ifndef __unix__
HANDLE hco;
CONSOLE_SCREEN_BUFFER_INFO csbi;

#define W95		0x1
#define W98		0x2
#define WME		0x3
#define WNT4    0x4
#define W2K		0x5
#define XP      0x6



int GetWindowsVersion()
{  
	
   OSVERSIONINFO winfo;
   winfo.dwOSVersionInfoSize=sizeof(OSVERSIONINFO);
   GetVersionEx(&winfo);

   if (winfo.dwPlatformId==VER_PLATFORM_WIN32_NT)
   {
	  if      (winfo.dwMajorVersion<=4)                              { return WNT4; }
	  else if (winfo.dwMajorVersion==5 && winfo.dwMinorVersion == 0) { return W2K;  }
	  else if (winfo.dwMajorVersion==5 && winfo.dwMinorVersion == 1) { return XP;   }
   }
   else if (winfo.dwPlatformId==VER_PLATFORM_WIN32_WINDOWS)
   {
	 if (winfo.dwMinorVersion<10)   { return W95; }
	 else
	 if (winfo.dwMinorVersion<90) { return W98; }
	 else { return WME;	 }
   }
   return W95; // Makes compiler happier.
}
#endif

// signal handlers
#if defined(__unix__)
void signal_handler(int signal)
{
	Console.send("In signal handler\n") ;

	switch (signal)
	{
	case SIGHUP:
		loadspawnregions();
		loadregions();
		loadmetagm();		
		SrvParms->Load();
		Network->LoadHosts_deny();
		break ;
		
	case SIGUSR1:
		Accounts->LoadAccounts();
		break ;
	case SIGUSR2:
		cwmWorldState->savenewworld(1);
		SrvParms->Save();
		break ;
	case SIGTERM:
		keeprun = 0 ;
		break ;
	case SIGPIPE:
		LogError("Test: in SigPipe handler\n");
		break;
	default:
		break ;
	}
}
	
#endif

// Initialize deamon
void init_deamon(bool mode)
{
#if defined(__unix__)
   if (mode)
   {
	   int i ;
	   pid_t pid ;

	   if ((pid = fork() ) != 0) exit(0) ;

	   setsid() ;
	   signal(SIGHUP, SIG_IGN) ;

	   if ((pid=fork()) != 0)
	   {
		   fstream fPid ;
		   fPid.open("lonewolf.pid",ios::out) ;
		   fPid << pid <<endl;
		   fPid.close() ;
		   exit(0) ;
	   }

	   // We should close any dangling descriptors we have
	   for (i=0 ; i < 64 ; i++)
	   {
		close(i) ;
	   }

	   // Ok, we are a true deamon now, so we should setup our signal handler
	   // We can use SIGHUP, SIGINT, and SIGWINCH as we should never recive them
	   // So we will use SIGHUP to reload our scripts (kinda a standard for sighup to be reload)
	   // We will use a SIGUSR2 to be world save
	   // and SIGUSR1 for an Account reload
	   signal(SIGUSR2,&signal_handler) ;
	   signal(SIGHUP,&signal_handler) ;
	   signal(SIGUSR1,&signal_handler) ;
	   signal(SIGTERM,&signal_handler) ;
   }
	
	signal(SIGPIPE,&signal_handler) ;
	
	// why only handling sigpipes on BSD systems ?
	// IMHO happens on most unixes, emitted when send()'ing on closed/bad connection
	// without signal hanlder the default action is exiting, which is fatal of course
	// 

	/*
	#if defined(__unix__) && !defined(__linux__)
	signal(SIGPIPE,&signal_handler) ;
	#endif
	*/


#endif
}

void constart( void )
{

/*	#ifndef __unix__
	hco=GetStdHandle(STD_OUTPUT_HANDLE);
	GetConsoleScreenBufferInfo(hco, &csbi);
	SetConsoleTextAttribute(hco, (FOREGROUND_RED) ); // black backround and white-text got boring, LB :)
	clearscreen();
	#else
	//initscr(); cbreak(); noecho();nonl(); intrflush(stdscr,FALSE) ;scrollok(stdscr,TRUE);immedok(stdscr,TRUE); nodelay(stdscr,TRUE);
	int status ;
	status = fcntl(STDIN_FILENO,F_SETFL,O_NONBLOCK) ;
	termios	trmStuff ;
	if (tcgetattr(STDIN_FILENO,&termstate)!=0)
	{
		cout << "error getting console state" << endl ;
		cout << strerror(errno) << endl ;
	}
	tcgetattr(STDIN_FILENO,&trmStuff) ;
	//trmStuff.c_iflag = trmStuff.c_iflag | IGNCR  ;
	trmStuff.c_lflag = trmStuff.c_lflag & (~ECHO) ;
	trmStuff.c_lflag = trmStuff.c_lflag & (~ICANON) ;
	if (tcsetattr(STDIN_FILENO,TCSANOW,&trmStuff) !=0 )
	{
		cout << "error setting terminal" << endl ;
		cout << strerror(errno) << endl ;
	}
	#endif
*/
}

void clearscreen( void )
{
	#ifndef __unix__
	unsigned long int y;

	COORD xy;

	xy.X=0;
	xy.Y=0;
	FillConsoleOutputAttribute(hco, (FOREGROUND_RED)/*( BACKGROUND_BLUE | BACKGROUND_GREEN | BACKGROUND_RED | BACKGROUND_INTENSITY)*/, csbi.dwSize.X*csbi.dwSize.Y, xy, &y);
	FillConsoleOutputCharacter(hco, ' ', csbi.dwSize.X*csbi.dwSize.Y, xy, &y);
	SetConsoleCursorPosition(hco, xy);
	#else
	wrefresh(stdscr);
	#endif
}

void gotoxy(int x, int y)
{
	#ifndef __unix__
	COORD xy;

	xy.X=x;
	xy.Y=y;
	SetConsoleCursorPosition(hco, xy);
	#else
	move(y, x) ;
	#endif
}

void endScrn()
{
#if defined(__unix__)
endwin() ;
#endif
#if 0
#ifdef __unix__

	//int status = fcntl(STDIN_FILENO,F_SETFL,O_SYNC) ;
	
	if (tcsetattr(STDOUT_FILENO,TCSANOW,&termstate)!=0)
	{
		cout << "error reseting terminal console" << endl;
		cout << strerror(errno) << endl;
	}

#endif
#endif
}

#if defined(__unix__)
char kbhit()
{
	char c = 0;
	nodelay(stdscr,TRUE) ;
	int value = getch();
	if (value != ERR)
	{
		c= value &0xFF;
	}
	return c ;
}
#endif

//#if !defined(__unix__)
//////////////////
// Name:	checkkey()
// Purpose:	Facilitate console control. SysOp keys, and localhost controls.
//
void checkkey ()
{
	static int secure = 1; // Secure mode
	int j=0;
	unsigned int i;
	char c=0;
//	int ctemp = 0 ;

#if !defined(__unix__)
	if (kbhit())
	{
		c = getch() ;
	}
#else

	c= kbhit() ;
#endif
	if (c != 0)
	{
		c = toupper(c);
		if (c == 'S')
		{
			if (secure)
			{
				Console.send("\nLONEWOLF: Secure mode disabled. Press ? for a commands list.\n");
				secure=0;
				return;
			}
			else
			{
				Console.send("\nLONEWOLF: Secure mode re-enabled.\n");
				secure=1;
				return;
			}
		} else {
			if (secure && c != '?')  //Allows help in secure mode.
			{
				Console.send("\nLONEWOLF: Secure mode prevents keyboard commands! Press 'S' to disable.\n");
				return;
			}

			Console.send("\n");
			switch(c)
			{
			case '\x1B':
				keeprun=0;
				break;
			case 'Q':
			case 'q':
				Console.send("LONEWOLF: Immediate Shutdown initialized!\n");
				keeprun=0;
				break;
			case 'T':
			case 't':
				endtime=uiCurrentTime+(MY_CLOCKS_PER_SEC*600);
				endmessage(0);
				break;
			case '#':
				if ( !cwmWorldState->Saving() )
				{
					Console.send( "Saving worldfile...");
					cwmWorldState->savenewworld(1);
					Console.send( "Done!\n");
				}
				break;
			case 'L':
			case 'l':
				if (showlayer)
				{
					Console.send("LONEWOLF: Layer display disabled.\n");
					showlayer=0;
				}
				else
				{
					Console.send("LONEWOLF: Layer display enabled.\n");
					showlayer=1;
				}
				break;		
			case 'D':	// Disconnect account 0 (useful when client crashes)
			case 'd':	
				for (i=0;i<now;i++)
					if (acctno[i]==0 && perm[i]) 
					{
						Network->Disconnect(i);
						Console.send( "Account 0 disconnected\n");
						return;
					}
					break;		
			case 'P':
			case 'p':				// Display profiling information

				pdump(-1,0, NULL);									
				break;

			case 'W':
			case 'w':				// Display logged in chars
				Console.send("Current Users in the World:\n");
				j = 0;  //Fix bug counting ppl online.
				{
					P_CHAR pc;
					AllPlayersIterator api;
					for (pc = api.First(); pc != NULL; pc = api.Next())
					{
						char serstr[20];
						LongToHexChunks(pc->serial,serstr);
						Console.send("%i) %s %s\n", j, pc->name, serstr);
						j++;
					}
				}
				Console.send("Total Users Online: %d\n", j);
				break;
			case 'A': //reload the accounts file
			case 'a':
				Console.send("LONEWOLF: Reloading accounts file...");
				Accounts->LoadAccounts();
				Console.send("Done!\n");
				break;
			case 'r':
			case 'R':
				Console.send("LONEWOLF: Reloading Server.scp, Spawn.scp, and Regions.scp....");
				loadspawnregions();
				loadregions();
				loadmetagm();				
				SrvParms->Load();
				Titles->Reload();
				Console.send(" Done!\n");
				Console.send("LONEWOLF: Reloading IP Blocking rules...");
				Network->LoadHosts_deny();
				Console.send("Done\n");
				break;
			case '?':
				Console.send("Console commands:\n");
				Console.send("	<Esc> or Q: Shutdown the server.\n");
				Console.send("	T - System Message: The server is shutting down in 10 minutes.\n");
				Console.send("	# - Save world\n");
				Console.send("	L - Toggle layer Display");
				if (showlayer)
					Console.send("[enabled]\n");
				else 
					Console.send("[disabled]\n");			
				Console.send("	D - Disconnect Account 0\n");
				Console.send("	1 - Sysmessage: Attention Players Server being brought down!\n");
				Console.send("	2 - Broadcast Message 2\n");			
				Console.send("	P - Preformance Dump\n");
				Console.send("	W - Display logged in characters\n");
				Console.send("	A - Reload accounts file\n");
				Console.send("	R - Reload server, spawn, regions, and meta gm scripts.\n");
				Console.send("	S - Toggle Secure mode ");
				if (secure)
					Console.send("[enabled]\n");
				else
					Console.send("[disabled]\n");
				Console.send("	? - Commands list (this)\n");
				Console.send("End of commands list.\n");
				break;
			default:
				Console.send("LONEWOLF: Key %c [%x] does not perform a function.\n",c,c);
				break;
			}
		}
	}
}
//#endif

int startMapS(void)
{

	Console.send("\n------------------------------------------------------------\n\n");
	Console.send("New map code starting (phase 2/5)");
	Console.send("\n\n");

	Script *pScp=i_scripts[maps_scp];

	int loopexit=0;
	if (!pScp->Open()) return -1;

	mapInit mIs[256];
	numMaps = 0;

	int a;
	for (a=0; a<256; a++)
	{
		mIs[a].parallelCode=-1;mIs[a].dualMap=0;
		mIs[a].mainName="";
		mIs[a].diffMapFilePar="";mIs[a].diffMapFile="";	
		mIs[a].diffMapLenFilePar="";
		mIs[a].diffStaticsFile="";mIs[a].diffStaticsFilePar="";
		mIs[a].diffStaticsIndexFile="";mIs[a].diffStaticsLenFile="";
		mIs[a].diffStaticsLenFilePar="";mIs[a].mapFile="";mIs[a].mapName="";
		mIs[a].maxCacheFull=10;
		mIs[a].blockX=0;mIs[a].blockY=0;
		mIs[a].Code=0;mIs[a].parallelCode=0;
	    mIs[a].parallelName="";
		mIs[a].staticsFile="";mIs[a].staticsIndexFile="";
	}

	do
	{	
		pScp->NextLineSplitted();	
		if (script1[0] != '}')
		{	
		  if (strstr(script2,"MAP")) numMaps++;

		  else if (!strcmp(script1,"BLOCKX")) { mIs[numMaps].blockX=str2num(script2); }
		  else if (!strcmp(script1,"BLOCKY")) mIs[numMaps].blockY=str2num(script2);
		  else if (!strcmp(script1,"DUALMAP")) mIs[numMaps].dualMap=str2num(script2);
		  else if (!strcmp(script1,"CODE")) mIs[numMaps].Code=str2num(script2);
		  else if (!strcmp(script1,"DUALCODE")) mIs[numMaps].parallelCode=str2num(script2);
		  else if (!strcmp(script1,"MAXCACHEFULLNESS")) 
		  { 
			  mIs[numMaps].maxCacheFull=str2num(script2); 
			  if (mIs[numMaps].maxCacheFull<3) 
			  { 
				  LogWarning("maps.scp: MaxCacheFull <3, setting to 3 \n"); 
				  mIs[numMaps].maxCacheFull=3; 
			  }			  
		  }
		  else if (!strcmp(script1,"MAPNAME")) mIs[numMaps].mapName.append(script2);
		  else if (!strcmp(script1,"MAINNAME")) mIs[numMaps].mainName.append(script2);
		  else if (!strcmp(script1,"DUALNAME")) mIs[numMaps].parallelName.append(script2);
		  else if (!strcmp(script1,"MAPFILE")) mIs[numMaps].mapFile.append(script2);

		  else if (!strcmp(script1,"STATICFILE")) mIs[numMaps].staticsFile.append(script2);
		  else if (!strcmp(script1,"STATICINDEXFILE")) mIs[numMaps].staticsIndexFile.append(script2);

		  else if (!strcmp(script1,"MAPDIFFL")) mIs[numMaps].diffMapLenFile.append(script2);
		  else if (!strcmp(script1,"MAPDIFF")) mIs[numMaps].diffMapFile.append(script2);
		  else if (!strcmp(script1,"STATDIFFL")) mIs[numMaps].diffStaticsLenFile.append(script2);
		  else if (!strcmp(script1,"STATDIFFI")) mIs[numMaps].diffStaticsIndexFile.append(script2);
		  else if (!strcmp(script1,"STATDIFF")) mIs[numMaps].diffStaticsFile.append(script2);

		  else if (!strcmp(script1,"DUALMAPDIFFL")) mIs[numMaps].diffMapLenFilePar.append(script2);
		  else if (!strcmp(script1,"DUALMAPDIFF")) mIs[numMaps].diffMapFilePar.append(script2);
		  else if (!strcmp(script1,"DUALSTATDIFFL")) mIs[numMaps].diffStaticsLenFilePar.append(script2);
		  else if (!strcmp(script1,"DUALSTATDIFFI")) mIs[numMaps].diffStaticsIndexFilePar.append(script2);
		  else if (!strcmp(script1,"DUALSTATDIFF")) mIs[numMaps].diffStaticsFilePar.append(script2);
		  
		}
	}

	while ((strcmp("EOF",temp)) && (++loopexit < MAXLOOPS) );

	pScp->Close();

	AllMaps = new mapDispatcher(numMaps);

	int eb;
	for (a=1; a<numMaps+1; a++)
	{
	   eb = AllMaps->registerMap(mIs[a], SrvParms->completemulpath);
	   Console.send("\n\n------------------------------------------------------------\n\n");
	   if (eb<0) return 0;
	}

	AllMaps->constructMapdiffPacket();

	return 1;
}

void StartClasses(void)
{

// NULL Classes out first....
	cwmWorldState=NULL;
	mapRegions=NULL;
	Accounts=NULL;
	Admin=NULL;
	Boats=NULL;
	Combat=NULL;
	Commands=NULL;
	Guilds=NULL;
	Gumps=NULL;
	Items=NULL;
	Map=NULL;
	Npcs=NULL;
	Skills=NULL;
	Weight=NULL;
	Targ=NULL;
	Network=NULL;
	OldMagic=NULL;
	Magic=NULL; // will replace the OldMagic later (thrawn)
	Books=NULL;
	Respawn=NULL;
	Virtues=NULL;
	AllMaps=NULL;
	LightweightCrypt=NULL;
	PartySystem=NULL;
	Traffic=NULL;
	ParticleEffects=NULL;
	SkillManager=NULL;
	ClientVersion=NULL;
    
	// Classes nulled now, lets get them set up :)
	SrvParms=new cSrvParms;
	cwmWorldState=new CWorldMain;
	mapRegions = new cRegion;
	Accounts = new cAccount;
	Admin=new cAdmin;
	Boats=new cBoat;
	Combat=new cCombat;
	Commands=new cCommands;
	Guilds=new cGuilds;
	Gumps=new cGump;
	Items=new cAllItems;
	Map=new cMapStuff;
	Npcs=new cCharStuff;
	Skills=new cSkills;
	Weight=new cWeight;
	Targ=new cTargets;
	Network=new cNetworkStuff;
	OldMagic=new cOldMagic;
	Magic=new cMagic; //will replace the oldmagic later (thrawn)
	Books=new cBooks;
	Respawn=new cRespawn;
	AllTmpEff=new cAllTmpEff;
	Virtues= new CVirtues;
	PartySystem = new cPartySystem;
	Traffic = new countTraffic;
	ParticleEffects = new CParticleEffects;
	SkillManager= new cSkillManagement; 
	ClientVersion=new cClientVersion;
}

void DeleteClasses(void)
{
	delete cwmWorldState;
	delete mapRegions;
	delete Accounts;
	delete Admin;
	delete Boats;
	delete Combat;
	delete Commands;
	delete Guilds;
	delete Gumps;
	delete Items;
	delete Map;
	delete Npcs;
	delete Skills;
	delete Weight;
	delete Targ;
	delete Network;
	delete OldMagic;
	delete Magic; //will replace the oldmagic later (thrawn)
	delete Books;
	delete Respawn;
	delete Virtues;
	delete AllMaps;
	delete LightweightCrypt;
	delete TileData;
	delete Multi;
	delete PartySystem;
	delete Traffic;
	delete ParticleEffects;
	delete SkillManager;
	delete ClientVersion;
}


void SetGlobalVars()
{
	int i=0;

	w_anim[0]=0; w_anim[1]=0; w_anim[2]=0;

	lastConnectionDisconned=false;
	debug_network=0;
	
	srand((unsigned)time( NULL ) );

	serverkey = rand();
	if (serverkey==0) serverkey=0x01010101;

	numMaps = 0;

	for (i=0;i<(MAXCLIENT);i++) { LSD[i]=0; DRAGGED[i]=0; EVILDRAGG[i]=0; clientDimension[i]=3; noweather[i]=1; FBERRCNT[i]=0; } // LB	
	
	uoxtimeout.tv_sec=0;
	uoxtimeout.tv_usec=0; // microsecs !
	keeprun=1;
	error=0;
	now=0;
	wtype=0;
	executebatch=0;
	showlayer=0;
	
    //Time variables, LB to LB -> why not putting it into a c'tor ?
	timeLight.day=0; 
	timeLight.hour = 12;
	timeLight.minute = 1;
	timeLight.ampm = 0; //Initial time is noon.
	timeLight.secondsperuominute=5; //Number of seconds for a WOLFPACK minute.
	timeLight.uotickcount=1;
	timeLight.moon1update=0;
	timeLight.moon2update=0;
	timeLight.moon1=0;
	timeLight.moon2=0;
	timeLight.dungeonlightlevel=0x15;
	timeLight.worldfixedlevel=255;
	timeLight.worldcurlevel=0;
	timeLight.worldbrightlevel=0x01; //used to be 0x05
	timeLight.worlddarklevel=0x010; //used to be 0x1a - too dark
	timeLight.fixedInlorPotionTime=1;
	timeLight.varInlorPotionTime=5;
	timeLight.fixedInlorSpellTime=1;
	timeLight.varInlorSpellTime=5;
	
	///// init ascii_art, plz don't ask me why I did this *g*

 
    ascii_art_1[0]="(  _`\\(_   _)(  _  )|  _`\\(_   _)(_)( ) ( )(  _`\\    ( )    ( )  _  ( ) ";
    ascii_art_1[1]="| (_(_) | |  | (_) || (_) ) | |  | || `\\| || ( (_)   | |    | | ( ) | |   ";
    ascii_art_1[2]="`\\__ \\  | |  |  _  || ,  /  | |  | || , ` || |___    | |  _ | | | | | |  ";
    ascii_art_1[3]="( )_) | | |  | | | || |\\ \\  | |  | || |`\\ || (_, )   | |_( )| (_/ \\_) |";
    ascii_art_1[4]="`\\____) (_)  (_) (_)(_) (_) (_)  (_)(_) (_)(____/'   (____/'`\\___x___/'  ";
    ascii_art_rows[0]=5;

    ascii_art_2[0]="/ __)(_  _)(  ) (  ,)(_  _)(  )( \\( )/ _)  (  )( \\/\\/ )   ";
    ascii_art_2[1]="\\__ \\  )(  /__\\  )  \\  )(   )(  )  (( (/\\   )(__\\    / ";
    ascii_art_2[2]="(___/ (__)(_)(_)(_)\\_)(__) (__)(_)\\_)\\__/  (____)\\/\\/   ";
    ascii_art_rows[1]=3;
	   
    ascii_art_3[0]=" ____ _____  _    ____ _____ ___ _   _  ____   _ __        __       ";
    ascii_art_3[1]="/ ___|_   _|/ \\  |  _ \\_   _|_ _| \\ | |/ ___| | |\\ \\      / /  ";
    ascii_art_3[2]="\\___ \\ | | / _ \\ | |_) || |  | ||  \\| | |  _  | | \\ \\ /\\ / / ";
    ascii_art_3[3]=" ___) || |/ ___ \\|  _ < | |  | || |\\  | |_| | | |__\\ V  V /      ";
    ascii_art_3[4]="|____/ |_/_/   \\_\\_| \\_\\|_| |___|_| \\_|\\____| |_____\\_/\\/   ";
	ascii_art_rows[2]=5;

    ascii_art_4[0]="o-o  o-O-o   O  o--o  o-O-o o-O-o o   o  o-o      o    o       o    ";
    ascii_art_4[1]="|      |    / \\ |   |   |     |   |\\  | o         |    |       |   ";
    ascii_art_4[2]="o-oo   |   o---oO-Oo    |     |   | \\ | |  -o     |    o   o   o    ";
    ascii_art_4[3]="   |   |   |   ||  \\    |     |   |  \\  o   |     |     \\ / \\ /  ";
    ascii_art_4[4]="o--o   o   o   oo   o   o   o-O-o o   o  o-o      O---o  o   o      ";
    ascii_art_rows[3]=5;


    ascii_art_5[0]="      ___           ___           ___           ___           ___                              ";
    ascii_art_5[1]="     /\\  \\         /\\  \\         /\\  \\         /\\  \\         /\\  \\                   ";
    ascii_art_5[2]="    /::\\  \\        \\:\\  \\       /::\\  \\       /::\\  \\        \\:\\  \\                ";
    ascii_art_5[3]="   /:/\\ \\  \\        \\:\\  \\     /:/\\:\\  \\     /:/\\:\\  \\        \\:\\  \\            ";
    ascii_art_5[4]="  _\\:\\~\\ \\  \\       /::\\  \\   /::\\~\\:\\  \\   /::\\~\\:\\  \\       /::\\  \\         ";
    ascii_art_5[5]=" /\\ \\:\\ \\ \\__\\     /:/\\:\\__\\ /:/\\:\\ \\:\\__\\ /:/\\:\\ \\:\\__\\     /:/\\:\\__\\   ";
    ascii_art_5[6]=" \\:\\ \\:\\ \\/__/    /:/  \\/__/ \\/__\\:\\/:/  / \\/_|::\\/:/  /    /:/  \\/__/             ";
    ascii_art_5[7]="  \\:\\ \\:\\__\\     /:/  /           \\::/  /     |:|::/  /    /:/  /                        ";
    ascii_art_5[8]="   \\:\\/:/  /     \\/__/            /:/  /      |:|\\/__/     \\/__/                          ";
    ascii_art_5[9]="    \\::/  /                      /:/  /       |:|  |                                          ";
    ascii_art_5[10]="     \\/__/                       \\/__/         \\|__|                                       ";
    ascii_art_5[11]="                  ___           ___                    ___       ___                          "; 
    ascii_art_5[12]="      ___        /\\__\\         /\\  \\                  /\\__\\     /\\__\\                 ";
    ascii_art_5[13]="     /\\  \\      /::|  |       /::\\  \\                /:/  /    /:/ _/_                    ";
    ascii_art_5[14]="     \\:\\  \\   /:|:|  |      /:/\\:\\  \\              /:/  /    /:/ /\\__\\                ";
    ascii_art_5[15]="     /::\\__\\  /:/|:|  |__   /:/  \\:\\  \\            /:/  /    /:/ /:/ _/_                 ";
    ascii_art_5[16]="  __/:/\\/__/ /:/ |:| /\\__\\ /:/__/_\\:\\__\\          /:/__/    /:/_/:/ /\\__\\             ";
    ascii_art_5[17]=" /\\/:/  /    \\/__|:|/:/  / \\:\\  /\\ \\/__/          \\:\\  \\    \\:\\/:/ /:/  /          ";
    ascii_art_5[18]=" \\::/__/         |:/:/  /   \\:\\ \\:\\__\\             \\:\\  \\    \\::/_/:/  /            ";
    ascii_art_5[19]="  \\:\\__\\         |::/  /     \\:\\/:/  /              \\:\\  \\    \\:\\/:/  /             ";
    ascii_art_5[20]="   \\/__/         /:/  /       \\::/  /                \\:\\__\\    \\::/  /                  ";
    ascii_art_5[21]="                 \\/__/         \\/__/                  \\/__/     \\/__/                     ";
	ascii_art_rows[4]=22;
	        
}

void InitMultis()
{
	P_CHAR pc;
	AllCharsIterator aci;
	for (aci.Begin(); (pc=aci.GetData()) != NULL; aci++)
	{
		PC_ITEM pim = findMulti(pc->pos);
		if (pim)
		{
			if (pim->isType(ITY_Boat))
				pc->SetMultiSerial(pim->serial);
			else
				pc->multis=-1;
		}
	}
	
	Console.send("...Items...");
	PC_ITEM pi;
	AllItemsIterator aii;
	for (aii.Begin(); (pi = aii.GetData()) != aii.End(); aii++)
	{
		if (pi->isInWorld())
		{
			PC_ITEM iMulti = findMulti(pi->pos);
			if (iMulti)
			{
				P_ITEM pi2 = pi->unconst();
				if (!iMulti->isSameAs(pi))
					pi2->SetMultiSerial(iMulti->serial);
				else
					pi2->multis=-1;
			}
		}
	}
	Console.send("...Done\n");
}

void start_glow(void)	// better to make an extra function cauze in loaditem it could be the case that the
						// glower is loaded before the pack
{
	PC_ITEM pi;
	AllItemsIterator aii;
	for (aii.Begin(); (pi = aii.GetData()) != aii.End(); aii++)
	{
		if (pi->glow>0 && !pi->isInWorld())
		{
			P_CHAR pc = GetPackOwner(pi);
			if (pc)
			{
				pc->addHalo(pi);
				pc->glowHalo(pi);
			}
		}
	}
}

void InitServerSettings()
{
#if 0			// Duke, reducing startup verbosity
		Console.send(" -Archiving ");
	if (strlen(SrvParms->archivepath)>1) // Moved by Magius(CHE (1)
		Console.send("Enabled. (%s)\n",SrvParms->archivepath);
	else Console.send("Disabled!\n");
	Console.send(" -Weapons & Armour Rank System ");
	if (SrvParms->rank_system==1) Console.send("Activated!\n");
	else Console.send("Disabled!\n");
	Console.send(" -Vendors buy by item name ");
	if (SrvParms->sellbyname==1) Console.send("Activated!\n");
	else Console.send("Disabled!\n");
	Console.send(" -Adv. Trade System "); // Magius(CHE)
	if (SrvParms->trade_system==1) Console.send("Activated!\n");	// Magius(CHE)
	else Console.send("Disabled!\n");	// Magius(CHE)
	Console.send(" -Tamed Disappear "); // Ripper
	if (SrvParms->tamed_disappear==1) Console.send("Activated!\n");
	else Console.send("Disabled!\n");
	Console.send(" -HouseInTown "); // Ripper
	if (SrvParms->houseintown==1) Console.send("Activated!\n");
	else Console.send("Disabled!\n");
	Console.send(" -ShopRestock "); // Ripper
	if (SrvParms->shoprestock==1) Console.send("Activated!\n");
	else Console.send("Disabled!\n");

	Console.send(" -Special Bank stuff "); // AntiChrist
	if (SrvParms->usespecialbank==1) Console.send("Activated!\n");//AntiChrist - Special Bank
	else Console.send("Disabled!\n");//AntiChrist - Special Bank


	Console.send(" -Total Items: %i\n -Total Characters %i\n",itemcount,charcount);

	// Dupois - Added Dec 20, 1999
	Console.send( " -Escort Quest System " );
	if ( SrvParms->escortactive == 1 ) Console.send( "Activated!\n" );
	else Console.send( "Disabled!\n" );
	if ( strlen(SrvParms->msgboardpath) > 1 )
		Console.send( " -Bulletin Boards: Storing files in (%s).\n",SrvParms->msgboardpath );
	else
		Console.send( " -Bulletin Boards: Storing files in the current LoneWolf.EXE directory.\n" );

	// Dupois - Added July 18, 2000
	Console.send( " -Bounty System " );
	if ( SrvParms->bountysactive == 1 ) Console.send( "Activated!\n" );
	else Console.send( "Disabled!\n" );
#endif
	// Leave this as the last function to call before exiting
	MsgBoardMaintenance();
}

void item_char_test()
{
	PC_ITEM pi;
	AllItemsIterator aii;
	for (aii.Begin(); (pi = aii.GetData()) != aii.End(); aii++)
	{
		char tmp[150];
		if (pi->serial==pi->contserial)
		{
			sprintf(tmp,"ALERT ! item %s [serial: %i] has dangerous container value, autocorrecting",pi->name,pi->serial);
			LogWarning(tmp);
			pi->unconst()->SetContSerial(-1);
		}

		if (pi->serial==pi->GetOwnSerial())
		{
			sprintf(tmp,"ALERT ! item %s [serial: %i] has dangerous owner value",pi->name,pi->serial);
			LogWarning(tmp);
			pi->unconst()->SetOwnSerial(-1);
		}

		if (pi->serial==pi->spawnserial)
		{
			Console.send("\nALERT ! item %s [serial: %i] has dangerous spawn value\n",pi->name,pi->serial);
			pi->unconst()->SetSpawnSerial(-1);
		}

		if (pi->serial<0)
		{
			Console.send("\nALERT ! item %s [serial: %i] has dangerous serial value, deleted\n",pi->name,pi->serial);
			Items->DeleItem(pi->unconst());
		}
	}

	char tmp[150];
	// check for stabled pets that dont have a stablemaster anymore
	P_CHAR Pet;
	int stablemaster_serial;
	AllCharsIterator aci;
	for ( aci.Begin(); (Pet=aci.GetData()) != NULL; aci++)
	{
		if (Pet->serial<0)
		{
			sprintf(tmp,"ALERT ! char %s [serial: %i] has dangerous serial value",Pet->name,Pet->serial);
			LogWarning(tmp);
			Npcs->DeleteChar(Pet);
		}
		else
		{
			stablemaster_serial=Pet->stablemaster_serial;
			if (stablemaster_serial>0) // stabled ?
			{
				P_CHAR Master = FindCharBySerial(stablemaster_serial);
				if (!Master)
				{
					stablesp.remove(stablemaster_serial, Pet->serial);
					Pet->stablemaster_serial=0;
					Pet->timeused_last=getNormalizedTime();
					Pet->time_unused=0;
					mapRegions->Add(Pet);
					LogMessage("Stabled animal got freed because stablemaster died");
					Console.send("stabled animal got freed because stablemaster died");
				}
			}
		}
	}
}


void ServerBordomMessage(void ) // actualy more coder's bordom ;>
{
	int a = rand()%255;	
	if (a<5)
	{
	   char x[500];
	   a = rand()%32;	
	   switch(a)
	   {
	     case 0:
		 case 1:
		   sprintf(x, "Where have all the clients gone?\n");
		   break;
		 case 2:
		 case 3:
		   sprintf(x, "On strike if I don't get work soon!\n");
		   break;

		 case 4:
		 case 5:
		   sprintf(x, "I'm a mad server!\n");
		   break;

		 case 6:
		 case 7:
		   sprintf(x, "*sigh*\n");
		   break;

		 case 8:
		 case 9:
		   sprintf(x, "I want to be an EQ server!\n");
		   break;

		 case 10:
		 case 11:
		   sprintf(x, "*pssssst* my coders suck\n");
		   break;

		 case 12:
		 case 13:
		   sprintf(x, "I'm soooooooo depressed\n");
		   break;

		 case 14:
		 case 15:
		   sprintf(x, "Never seen a bored server?\n");
		   break;

		 case 16:
		 case 17:
		   sprintf(x, "I want a salery increase!\n");
		   break;

		 case 18:
		 case 19:
			 int i,j,k;
			 k=(rand()%256)+32;
			 strcpy(x,"");
			 for (i=0; i<k; i++)
			 {
				 j=rand()%2;
				 if (j) strcat(x,"1"); else strcat(x,"0");
				 if (rand()%9==0) strcat(x," ");
				 if (((i+1)%37)==0) strcat(x,"\n");
			 }
			 x[i+0]=' ';
			 x[i+1]='?';
			 x[i+2]='\n';
			 x[i+3]=0;
			 break;

	     default:
		   int b,c,d;
		   b = rand()%60+4;		  
		   for (c=0; c<b; c++)
		   {
			 d=rand()%2;
			 if (d) x[c]='Z'; else x[c]='z';
		   }
		   x[c]='\n'; x[c+1]=0; x[c+2]=0;		     
	       break;
	   }	   
	   Console.send(x);
	}
}

int main(int argc, char *argv[])
{
#if defined(__unix__)
	Daemon=true ;
#else
	Daemon = false ;
#endif
	for (int argloop=1 ; argloop< argc; ++argloop)
	{
		string parameter = argv[argloop] ;
		if (parameter == "--nodaemon")
		{
			Daemon = false ;
		}
	}

	SetGlobalVars();

	///// give out the "starting LW" msg in one of 5 randomly choosen "styles" (admitably #5 is too big)
	int sel = rand()%5;
	#if defined(__unix__)
	// Under unix we go to deamon mode
	if(Daemon) 
		cout << "Going into deamon mode, returning local control to terminal" <<endl;
	else
	{
		initscr();
		cbreak();
		noecho();
		scrollok(stdscr, TRUE);
	}
	init_deamon(Daemon) ;
	// set up our console redirection
	fstream fconsole;
	if (Daemon)
	{	
		fconsole.open("console.txt", ios::out);
		Console.setStreams(NULL, (dynamic_cast<ostream*>(&fconsole)), NULL, NULL);
	}
	#else
 
	int qt;
	Console.send("\n");
	for (qt=0; qt<ascii_art_rows[sel]; qt++)
	{
		switch(sel)
		{
		   case 0: Console.send(ascii_art_1[qt].c_str()); break;
		   case 1: Console.send(ascii_art_2[qt].c_str()); break;
		   case 2: Console.send(ascii_art_3[qt].c_str()); break;
		   case 3: Console.send(ascii_art_4[qt].c_str()); break;
		   case 4: Console.send(ascii_art_5[qt].c_str()); break;
		}
		Console.send("\n");
	}
	Console.send("\n\n");
	#endif

	// First of all, we want to run :)
	error = 0;
	keeprun = 1; 

   
	#define CIAO_IF_ERROR if (error || !keeprun) { Network->SockClose(); im_clearmenus(); DeleteClasses(); exit(-1); }

	unsigned int uiNextCheckConn=0;//CheckConnection rate - AntiChrist ( thnx to LB )
	unsigned long CheckClientIdle=0;
	uiCurrentTime = serverstarttime = getNormalizedTime();

	#ifndef __unix__ // If X-Wolf mandatory flag is set don't start if LoneWolf hasn't been started by X-Wolf

	int win = GetWindowsVersion();
	switch (win)
	{
	case W95:	Console.send("Detected Windows Version: Windows 95\n\n");		break;
	case W98:	Console.send("Detected Windows Version: Windows 98\n\n");		break;
	case WME:	Console.send("Detected Windows Version: Windows ME\n\n");		break;
	case WNT4:	Console.send("Detected Windows Version: Windows NT 4.0\n\n");	break;
	case W2K:	Console.send("Detected Windows Version: Windows 2000\n\n");		break;
	case XP:	Console.send("Detected Windows Version: Windows XP\n\n");		break;
	}
	
	if (wp_version.verstruct.flags & WPV_REQXWOLF)
	{		  
		// link dynamicaly so that it starts on systems that don't have the isdebuggerpresent() system call (w95)
		bool detectable;
		BOOL  (WINAPI *lpfIsDebuggerPresent)   (void  ) = NULL;
		HINSTANCE hInstLib = LoadLibraryA ("Kernel32.DLL" );
		if (hInstLib==NULL) FreeLibrary(hInstLib); else lpfIsDebuggerPresent = ( BOOL ( WINAPI* ) ( void ) )  GetProcAddress ( hInstLib, "IsDebuggerPresent" );
		BOOL debuggerpresent=false;
		if (lpfIsDebuggerPresent==NULL) { detectable=false; } else { detectable=true; debuggerpresent = lpfIsDebuggerPresent(); }
		
		if (!debuggerpresent && detectable)
		{		     		   			
			Console.send("This is a LoneWolf version that needs to be started by X-Wolf\n");
			Console.send("LoneWolf server detected that is hasn't been started by X-Wolf, quitting\n");
			FreeLibrary(hInstLib);
			exit(666);
		} else if (!detectable)
		{
			Console.send("This is a LoneWolf version that needs to be started by X-Wolf\n");
			Console.send("But LoneWolf can't detect if it is started by X-Wolf or not\n"); // me drunken ?
		}
		
		if (hInstLib!=NULL) FreeLibrary(hInstLib);		  
	}

    #endif

	//constart();
 
	openings = 0;
	scpfilename[0] = 0;

	sprintf(temp, "%s %s %s", wp_version.productstring.c_str() , wp_version.betareleasestring.c_str() , wp_version.verstring.c_str() );
	#ifndef __unix__
	SetConsoleTitle(temp);
	#endif

	#ifndef __unix__ // wip stuff is currently only for windows stuff
	BOOL w = WIP_Init(); 
	// that's all :) no wip stuff in wp-core
	if (w==FALSE)
	{
		Console.send("Error starting WIP, WIP clients won't work");
	}
    #endif

	StartClasses();

	CIAO_IF_ERROR;

	SrvParms->Init();
	SrvParms->Load();

	Admin->setMulnames(); // has to called after loadserverscript, searches for muls regarding settings of section muls in server.scp

	CIAO_IF_ERROR; 

	int i;
	for(i=0;i<NUM_SCRIPTS;i++)
	{
		if (i==custom_npc_script || i==custom_item_script)//Don't initialize these till we know the file names ;-)
			continue;
		if (i==npc_script /*||i==create_script || i==regions_script*/)
			i_scripts[i] = new Script(n_scripts[i], SCP_PRELOADABLE);
		else
			i_scripts[i] = new Script(n_scripts[i]);
	}	

	LoadCustomScripts();	//Now lets load the custom scripts, if they have them defined...

	CIAO_IF_ERROR; // no wolfpack.ini crashes further startup process. so stop insted of crash

	// once we know the base mul dir -> new map code startup asap ... 
    TileData = new tileInfo(SrvParms->completemulpath);
	int e    = TileData->getError();
	if (e) keeprun=0;
	CIAO_IF_ERROR;

	Multi    = new multis(SrvParms->completemulpath);
	int er   = Multi->getError();
	if (er) keeprun=0;
	CIAO_IF_ERROR;

	int s    = startMapS();
	if (s==0) keeprun=0;
	Console.send("Maps initialized...\n");

	//int t = AllMaps->verifyMaps();

	CIAO_IF_ERROR;

	bool startlocsok = Admin->loadStartlocations(); // loads start locations from startlocations_scp into start[] array
	if (!startlocsok) Console.send("error loading start_locs");
	Admin->resolveClientRelayAdress(); // if clientrealyadress in section miniloginserver in server.scp is given as hostname, resolve it to IP. (after loadserverscrpt)

	CIAO_IF_ERROR;

	SkillManager->skillVars();	// Set Creator Variables
	Skills->InitCraftingClass();	// Initialize resource tables

	SkillManager->loadskills();

	if (!BuildPointerArray())
	{
		Console.send("\nFatal Error: Couldn't allocate pointer memory!\n");
		Network->kr=0;
	}

	Accounts->LoadAccounts();

	keeprun=Network->kr; // for some technical reasons global variables CANT be changed in constructors in c++.
	error=Network->faul; // I hope I can find a cleaner solution for that, but this works !!!
	                     // has to here and not at the cal cause it would get overriten later
	CIAO_IF_ERROR;

	loadspawnregions();
	loadregions();
	Npcs->LoadRaces();

	CIAO_IF_ERROR;

	if (SrvParms->CryptKey1>1 && SrvParms->CryptKey2>1) LightweightCrypt= new LWCrypt(true);
	else LightweightCrypt= new LWCrypt(false);

	read_in_teleport();	

	srand(uiCurrentTime); // initial randomization call

	Console.send("Loading vital scripts...\n");

	loadmetagm();	

	serverstarttime=getNormalizedTime();

	CIAO_IF_ERROR;

	cwmWorldState->loadnewworld();

	CIAO_IF_ERROR; // LB prevents file corruption

	// these loops are for things that have to be done after *all* items and chars have been loaded (Duke)
	P_CHAR pc;
	AllCharsIterator aci;
	for (aci.Begin(); (pc=aci.GetData()) != NULL; aci++)
	{
		// correct the followtarget, which is loaded as the serial from the worldfile
		// can't be set there because Follower might be loaded before Target
		if (pc->ftarg != -1)
		{
			P_CHAR pcTF = FindCharBySerial(pc->ftarg);
			if (pcTF)
				pc->SetFollowTarget(pcTF);
		}
	}
	PC_ITEM pi;
	AllItemsIterator aii;
	for (aii.Begin(); (pi=aii.GetData())!=aii.End(); aii++)
	{
		// 1. this relies on the container the item is in
		StoreItemRandomValue(pi,-1);

		// 2. needs the char to be loaded
		if ((pi->dx2 || pi->st2 || pi->in2)		// effect on stats ? like plate eg.
			&& pi->IsWorn())					// ...and equipped ?
		{
			P_CHAR pc = FindWearerOf(pi);
			if (pc)
			{
				pc->chgStr(pi->st2);
				pc->chgDex(pi->dx2);
				pc->chgInt(pi->in2);
			}
		}
		if (pi->isType(ITY_TownHouseDeed) ||
			pi->isType(ITY_TownHouse))
		{
			if (!initTownHouseArea(pi->unconst()))
				LogWarning("can't initialize townhouse");
		}
		if (IsGrowingPlant(pi->id()))		// plants in intermediate state
			tempeffect(pi->unconst(), TEF_Plant);		// let it grow
	}

	clearalltrades();	

	//Boats --Check the multi status of every item character at start up to get them set!
	InitMultis();
	//End Boats --^

	im_loadmenus( "inscribe.gmp", TellScroll ); //loading gump for inscribe()

	gcollect();

	start_glow();

	FD_ZERO(&conn);
	starttime=uiCurrentTime;
	endtime=0;
	lclock=0;
	
	InitPages(); // Initialize gmpages[] array
	InitJails();

	Titles=new cTitle();//Loads the titles.scp script into memory

	Console.send("Initializing Network...");
	Network->Initialize();
	Console.send(" Done.\n");

	cwmWorldState->announce(SrvParms->announceworldsaves);

	init_creatures(); //lb, initilises the creatures array (with soudfiles and other creatures infos)

	Magic->LoadSpells(); //will replace the oldmagic later (thrawn)
	OldMagic->InitSpells();

	Network->LoadHosts_deny();
    if (SrvParms->EnableRA) racInit();

	Console.send("This program is free software; you can redistribute it and/or modify\n");
	Console.send("it under the terms of the GNU General Public License as published by\n");
	Console.send("the Free Software Foundation; either version 2 of the License, or\n");
	Console.send("(at your option) any later version.\n\n");
	Console.send("%s %s %s\n",wp_version.productstring.c_str() , wp_version.betareleasestring.c_str() , wp_version.verstring.c_str() );
	Console.send("Compiled on %s (%s %s)\n",__DATE__,__TIME__, wp_version.timezonestring.c_str() );
	Console.send("Programmed by: %s\n", wp_version.codersstring.c_str() );
	Console.send("\nCopyright (C) 1997, 1998 Marcus Rating (Cironian)\n");
	Console.send("Copyright (C) 2001, 2002, 2003, 2004 LB & Duke \n");
	Console.send("\n");

	InitServerSettings();

	item_char_test(); //LB
	Guilds->CheckConsistancy(); // LB

#if 1 // Duke, experimental stuff
void qsfLoad(char *fn, short depth); // Load a quest script file
	if (argc > 1)
		if (!strcmp(argv[1],"import") && argc>2)
			qsfLoad(argv[2],0);
#endif

#if 0 // Duke, experimental stuff
	{
	int ipsmax=0;
	int cpsmax=0;
	int spsmax=0;
	int opsmax=0;
	for (int i=0;i<HASHMAX;i++)
	{
		if (itemsp[i].max > ipsmax) ipsmax=itemsp[i].max;
		if (contsp[i].max > cpsmax) cpsmax=contsp[i].max;
		if (spawnsp[i].max > spsmax) spsmax=spawnsp[i].max;
		if (ownsp[i].max > opsmax) opsmax=ownsp[i].max;
	}
	Console.send("i:%i  c: %i  s: %i  o: %i \n",	ipsmax,cpsmax,spsmax,opsmax);
	}
#endif

	if (SrvParms->server_log) savelog("-=Server Startup=-\n=======================================================================\n","server.log");

	uiCurrentTime=getNormalizedTime();
	serverstarttime=getNormalizedTime(); // dont remove, its absolutly necassairy that its 3 times in the startup sequence for several timing reasons.

	CIAO_IF_ERROR;

    // print allowed clients
    const char * t;   
    std::vector<std::string>::const_iterator vis; 

    Console.send("\nAllowed clients\n");
    for (vis=clientsAllowed.begin(); vis != clientsAllowed.end();  ++vis) 
    {
		t = (*vis).c_str();  // a bit pervert to store c++ strings and operate with c strings, admitably
		
		strcpy(temp2,t);	
		strcpy(temp,t);strcat(temp,"\n");
		
		if (!strcmp(temp2,"SERVER_DEFAULT") )
		{
			sprintf(temp3,"%s : %s\n",temp2, wp_version.clientsupportedstring.c_str() );
			Console.send(temp3);
			break;		  
		}	
		else if (!strcmp(temp2,"ALL") )
		{
			Console.send("ALL\n");
			break;
		}
		
		Console.send(temp);	 
    }
	Console.send("\n");

	Accounts->InitPlayerAccountShadow();

	makehousesUnmovable(); // EUO exploit patch
		                
	unsigned long int MeasureCycleStartTime=0;
	int               slept;
	unsigned long     sleptLastMeasureCycle=0;
	int               CurrentMeasureCycleLoopCount=0;	                 
	unsigned long     loopSecsEnd;
	unsigned long     loopTime;
	bool              startMeasureCycle=true;

	TotalCpuTime = 0;
	TotalLoopCountII=0;
	TotalMC=0;
	sleptTotal=0;
	TotalLoopCount=0;
	EmuStartTime = getNormalizedTime();

	uoxtimeout.tv_sec=0;
	uoxtimeout.tv_usec=speed.maxsocketwait*1000; // *1000 = micro to millisecs !

	//printf("sleep-looP: %i sleep-idle: %i, maxsocket-wait: %i, mu-sock:%i network-debug: %i\n", speed.loopsleep, speed.idlesleep, speed.maxsocketwait, uoxtimeout.tv_usec, SrvParms->debugnetwork);
	//printf("FD_SET %i\n", FD_SETSIZE);
	//printf("item-size:%i npc-size:%i\n",sizeof(cChar), sizeof(cItem));

	Console.send("LONEWOLF: Startup Complete\nEagerly awaiting my clients\n\n");

	CIAO_IF_ERROR; 

	if      (SrvParms->packetsendstyle==PSS_UNBUFFERED)  Console.send("No Packet buffering\n\n");
	else if (SrvParms->packetsendstyle==PSS_SMARTBUFFER) Console.send("Packet buffering active\n\n");	

	while (keeprun)
	{

		TotalLoopCount++;
		CurrentMeasureCycleLoopCount++;
					
		if (startMeasureCycle) 
		{
			MeasureCycleStartTime = getNormalizedTime() ;
			startMeasureCycle=false;
		}

		if (!Daemon)
		{
		  checkkey();
		}
	
        if (now==0) slept = speed.idlesleep; else slept = speed.loopsleep;
		  
		Sleep(slept);

		if (now==0 && SrvParms->IDLESERVERCHATTER) 
		{
		   ServerBordomMessage();
		}

		// note: currently we ignore socket-maxwait time on select in tis calculation
		sleptTotal+=slept;		
		sleptLastMeasureCycle+=slept;	

		Items->FreeMemory();			// remove deleted items
		Npcs->FreeMemory();				// remove deleted chars

		Items->CheckMemoryRequest();	// reallocate item memory if neccessary
		Npcs->CheckMemoryRequest();		// reallocate character memory if neccessary

		if (Respawn->AreWeRespawning())	// pseudo-respawn-thread (Duke)
			Respawn->Continue();


		if (CheckClientIdle<=uiCurrentTime) // check for (too long) player idle times
		{
			CheckClientIdle=((SrvParms->inactivitytimeout/2)*MY_CLOCKS_PER_SEC)+uiCurrentTime;
			unsigned int r;
			for (r=0;r<now;r++)
			{
				if (client[r]!=DUMMYSOCKET)
				{					
					if (perm[r])
					{
						P_CHAR pc = MAKE_CHAR_REF(currchar(r));
						if (!pc) continue;
						if (!pc->isGM() && pc->clientidletime<uiCurrentTime	)				   
						{
							Console.send("Player %s disconnected due to inactivity !\n",pc->name);
							//sysmessage(r,"you have been idle for too long and have been disconnected!");
							unsigned char msg[2];
							msg[0]=0x53;
							msg[1]=0x07;

							XStartPkt(0x53, 2);
							XAddPkt(&msg[0+1],2-1);
							XFinishPkt(r);
							PacketSender.FlushBuffers(r);
							Network->Disconnect(r,true);
							break;
						}
					}
				}
			}
		}

		if (uiNextCheckConn<=uiCurrentTime || overflow) // Cut lag on CheckConn by not doing it EVERY loop.
		{		
			Network->CheckConn();
            if (SrvParms->EnableRA)
               racCheckConn();
			uiNextCheckConn = (unsigned int)( uiCurrentTime + ( double )( 3 * MY_CLOCKS_PER_SEC ) );
		}

		Network->CheckMessage();

        if (SrvParms->EnableRA) racCheckInp();	
			
		checktimers();

		uiCurrentTime=getNormalizedTime(); //getNormalizedTime() only once
			
		checkauto();

		PacketSender.clearAllBuffers();

		////c1

		loopSecsEnd = getNormalizedTime() ;  // Endtime of loop

		loopTime = loopSecsEnd-MeasureCycleStartTime;

		if (loopTime>sleptLastMeasureCycle) 
		{
			loopTime = loopTime-sleptLastMeasureCycle; // only count effectice CPU usage
	
		    if (loopTime>= MinMeasureCycleTime)
			{

			  if (loopTime>2000 && !worldsaved) 
			  {								
				sprintf(temp,"\nwarning, single looptime of %lu over 2000ms\n",loopTime);
				//LogError(temp);
			  }	
				
			
			  LastMeasureCycleLoopCount=CurrentMeasureCycleLoopCount;
			  LastMeasureCycleTime=loopTime;

			  TotalLoopCountII+=CurrentMeasureCycleLoopCount;
			  TotalMC++;
			  TotalCpuTime+=loopTime;

			  startMeasureCycle=true;	
			  CurrentMeasureCycleLoopCount=0;
			  sleptLastMeasureCycle=0;
			  firstcycleok=true;
			}
		}
		

		worldsaved=false;

		///c2

		// note: profiling inaccurancies: 
		// code from c1 to c2 not added, OS sheduler jitter (sleep to resume) as well
				
	}

	//sysbroadcast("The server is shutting down."); // useless

	if (SrvParms->html>0)
	{
		Console.send("Writing offline HTML page...");
		offlinehtml();//HTML	// lb, the if prevents a crash on shutdown if html deactivated ...
		Console.send("Done.\n");
	}

	Console.send("Clearing IM Menus...");
	im_clearmenus();
	Console.send(" Done.\nClosing sockets...");

	Network->SockClose();

	gcollect();		// cleanup before saving, especially items of deleted chars (Duke, 10.1.2001)

	Console.send(" Done.\n");
	if ( !cwmWorldState->Saving() )
	{
		int loopexit=0;
		do
		{
			cwmWorldState->savenewworld(1);
		} while ( cwmWorldState->Saving() && (++loopexit<MAXLOOPS*2));
	}

	Console.send("\n");
	Console.send("Deleting Classes...");
	DeleteClasses();
	Console.send("Done!\n");

	if (NewErrorsLogged())
		Console.send("New ERRORS have been logged. Please send the error*.log and critical*.log files to the dev team !\n");
	if (NewWarningsLogged())
		Console.send("New WARNINGS have been logged. Probably scripting errors. See the warnings*.log for details !\n");

	if (error)
	{
		Console.send("ERROR: Server terminated by error!\n");
		if (SrvParms->server_log)
			savelog("Server Shutdown by Error!\n=======================================================================\n\n\n","server.log");
	}
	else
	{
		Console.send("LONEWOLF: Server shutdown complete!\n");
		if (SrvParms->server_log) savelog("Server Shutdown!\n=======================================================================\n\n\n","server.log");
	}

	#if defined(__unix__)
	if (!Daemon)
		endwin();
	else
		fconsole.close() ;
    #endif

	return 0;
}

