/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//


#include "accounts.h"
#include "Client.h"
#include "cmdtable.h"
#include "commands.h"
#include "guilds.h"
#include "gumps.h"
#include "house.h"
#include "itemid.h"
#include "LoS.h"
#include "network.h"
#include "newbie.h"
#include "npcinfo.h"
#include "regions.h"
#include "skills.h"
#include "SndPkg.h"
#include "sregions.h"
#include "TmpEff.h"
#include "trigger.h"
#include "walking.h"
#include "magic.h"
#include "dbl_single_click.h"
#include "targeting.h"
#include "magic_new.h"
#include "spells.h"

#include "wolfpack.h"

#include "debug.h"
#undef DBGFILE
#define DBGFILE "lwmisc.cpp"


///////////
// Name:	inRange
// Purpose:	checks if position 1 and 2 are in given range
// history:	by Duke, 19.11.2000
//			changed to cCoord-interface Duke, 28.08.2003
//
bool inRange(const cCoord& a, const cCoord& b, int range)
{
	return (max(abs(a.x - b.x), abs(a.y - b.y)) <= range);
}

int inrange2 (UOXSOCKET s, PC_ITEM pi) // Is item i in visual range for player on socket s
{
	int vr=VISRANGE;
	P_CHAR Player = MAKE_CHARREF_LRV(currchar(s),0);

	if (pi == NULL) // blackwind Crash fix 
		return 0; 

	if (IsLargeHouse(pi->id()))
		vr=BUILDRANGE;
	return inRange(Player->pos, pi->pos, vr);
}

bool iteminrange (const P_CHAR Player, const PC_ITEM pi, const int distance)
{
	if (!pi || !Player) return false;
	if (Player->isGM()) return true;
	PC_ITEM oc = GetOutmostCont(pi);
	if (Player->Wears(oc)) return true;
	return inRange(Player->pos,oc->pos, distance);
}

bool iteminrange (const UOXSOCKET s, const PC_ITEM pi, const int distance)
{
	P_CHAR pc = MAKE_CHARREF_LRV(currchar(s),false);
	return iteminrange(pc,pi,distance);
}

bool npcinrange (P_CHAR Player, P_CHAR pc, int distance)
{
	if (!pc || !Player) return 0;
	if (Player->isGM()) return 1;
	return inRange(Player->pos, pc->pos, distance);
}

///////////
// Name:	inVisRange
// Purpose:	checks if position 1 and 2 are in visual range
// history:	by Duke, 18.11.2000
//
bool inVisRange(int x1, int y1, int x2, int y2)
{
	return (max(abs(x1-x2), abs(y1-y2)) <= VISRANGE);
}

bool inVisRange(const cCoord &a, const cCoord &b)
{
	return inVisRange(a.x, a.y, b.x, b.y);
}

bool inrange1p (P_CHAR pca, P_CHAR pcb) // Are characters a and b in visual range
{
	if (!pca || !pcb) return false;

	return inVisRange(pca->pos, pcb->pos);
}

bool _inrange1p(P_CHAR pca, CHARACTER b,int cln, char* cfn) // Are characters a and b in visual range
{
	P_CHAR pcb = MAKE_CHARREF_LRV_LOGCALLER(b, false);
	if (pca == NULL) return false;

	return inVisRange(pca->pos, pcb->pos);
}


unsigned int dist(const cCoord &a, const cCoord &b) // Distance between position a and b
{
	return a.distance(b);
}


// LB 1-Sept 2002
// calculates distance between item i and player a
// if item is IN an container, the internal container coords are not taken into account.
// instead, the in worlds coords of the outermost container. (or player it its a bp)

unsigned int itemdistDeluxe(P_CHAR a, const PC_ITEM pi)
{
	if (pi==NULL || a==NULL) return ~0;

	if (pi->isInWorld()) // easiest case, item in world
	{
		return (a->pos.distance(pi->pos));
	}
	else
	{
		PC_ITEM Cont=GetOutmostCont(pi); 
		if (Cont!=NULL)
		{
		  if (!Cont->isInWorld()) 
		  {
			if (Cont->IsWorn())
			{
				cChar* Npc = FindWearerOf(Cont);
				if (Npc) return (Npc->pos.distance(a->pos)); // container is a BP, return pos of "carrier"				
				else
				{                
				  return ~0; // happens when an item gets referred that has been owned by a npc that got deleted (e.g delechar)
				             // this is NO error/warning
				}
			} 
			else
			{
			   LogWarning("error2 in itemdistDeluxe\n");
			   return ~0;
			}
		  }
		  else return (a->pos.distance(Cont->pos)); // outermost contraienr in world !
		}
	}
	return ~0;
}

int bestskill(P_CHAR pc) // Which skill is the highest for character p
{
	if (!pc) return 0;
	int i,a=0,b=0;
	int c=SkillManager->getAllSkillsValue();
	for (i=0;i<c;i++)
	{
		if (pc->getRealSkill(i) > b)
		{
			a=i;
			b=pc->getRealSkill(i);
		}
	}
	return a;
}


void gcollect () // Remove items which were in deleted containers
{
	int removed, rtotal=0;
	int idelete;
	LogMessage("Performing Garbage Collection...");

	rtotal=0;
	int loopexit=0;
	do
	{
		removed=0;
		PC_ITEM pi;
		AllItemsIterator aii;
		for (aii.Begin(); (pi = aii.GetData()) != aii.End(); aii++)
		{
			if (pi->isInWorld()) continue;
			idelete=1;
			// find the container if theres one.
			P_CHAR pc_j=FindWearerOf(pi);
			if (pc_j !=NULL)
			{
				idelete=0;
			}
			if (idelete)
			{
				P_ITEM pi_j=FindItemBySerial(pi->contserial);
				if (pi_j!=NULL)
				{
					idelete=0;
				}
			}

			if (idelete)
			{
				Items->DeleItem(pi->unconst());
				removed++;
			}
		}
		rtotal+=removed;
	} while (removed>0 && (++loopexit < MAXLOOPS) );

	sprintf(temp," gc: Removed %i items",rtotal);
	if (rtotal>0) LogMessage(temp);
}

void savelog(const char *msg, char *logfile)
{
	FILE *file;
	char fulllogfile[400];
	
	if (SrvParms->logging_path[0]!=0) // if loggingpath not set, use lonewolf root dir
	{
		char slash='/'; // linux slash
#ifndef __unix__ 
		slash = '\\'; // windows slash
#endif
		bool slash_at_end = (SrvParms->logging_path[strlen(SrvParms->logging_path)-1]==slash);
		
		if (!slash_at_end) sprintf(fulllogfile, "%s%c%s",SrvParms->logging_path, slash, logfile);
		else  sprintf(fulllogfile, "%s%s",SrvParms->logging_path, logfile);
	} else strcpy(fulllogfile, logfile);
	
	file=fopen(fulllogfile,"a");
	
	if (file==NULL)
	{
		if (SrvParms->errors_to_console)
		{
			Console.send("can't open logfile\n");
			return;
		}		  
	}
	
	char timestamp[60];
	MakeTimeStamp(timestamp);
	fprintf(file,"%s %s",timestamp,msg);
	
#ifdef DEBUG
	Console.send("DEBUG: Logging to %s\n", logfile);
#endif
	
	fclose(file);
}

void splitline() // For putting single words of cline into comm array
{
	int i=0;
	char *s;
	char *d;

	d=" ";
	s=strtok((char*)cline,d);
	int loopexit=0;
	while ( (s!=NULL) && (++loopexit < MAXLOOPS) )
	{
		comm[i]=(unsigned char*)s;
		i++;
		s=strtok(NULL,d);
	}
	tnum=i;
}

int makenumber (int countx) // Converts decimal string comm[count] to int
{
	if (comm[countx] == NULL)
		return 0;

	int i;
	sscanf((char*)comm[countx], "%i", &i);
	return i;
}

int hexnumber(int countx) // Converts hex string comm[count] to int
{
	if (comm[countx] == NULL)
		return 0;

	int i;
	sscanf((char*)comm[countx], "%x", &i);
	return i;
}

bool makestring(int countx, char* str, bool upr) // puts a string comm[count] to into str, memory of str has to be allocated by caller
{
   	if (comm[countx] == NULL) 
	{
		str[0]=0;
		return false;	
	}

	strncpy(str, (char*)comm[countx], 256);
	if (upr==true) strupr(str);

	return true;
}

void wornitems(UOXSOCKET s, P_CHAR pc) // Send worn items of player j
{
	pc->onhorse=false;
	int ci=0,loopexit=0;
	P_ITEM pi;
	while ( ((pi=pc->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
	{
		if (pi->layer==0x19) pc->onhorse=true;
		wearIt(s,pi);
	}
}

void all_items(int s) // Send ALL items to player
{
	PC_ITEM pi;
	AllItemsIterator aii;
	for (aii.Begin(); (pi = aii.GetData()) != aii.End(); aii++)
		senditem(s, pi);
}


/////////////////
// Name:	deathstuff
// History:	by UnKnown (Touched tabstops by Tauriel Dec 28, 1998)
// Purpose:	creates a corpse, moves items to it, take out of war mode, does animation and sound, etc.
//

void deathstuff(P_CHAR Victim)
{
	if (!Victim) return;
	if (Victim->dead || Victim->isAiVendor() || Victim->isInvul()) 
		return;

	AllTmpEff->ExpireOnDeath(Victim);
	Victim->ResetPoisoned();

	// by thrawn: check if player died in an arena
	unsigned char r = calcRegionFromXY(Victim->pos);
	if (region[r].isArena() && !Victim->summontimer)
	{
		region[r].DeathInArena(Victim);
		return; // dont realy "kill" the player in an arena -> return here
	}

	Victim->setXid(Victim->id());
	Victim->xskin = Victim->skin;
	Victim->murdererSer = 0;	// Reset previous murderer serial # to zero

	PlayDeathSound(Victim);

	bool VictimInGuardedRegion=Victim->inGuardedArea(); // important to get it at the beginning before it gets deleted in case of npc
	bool KilledByGuard = false;

	char murderername[50]; //AntiChrist
	P_CHAR Opponent = Victim->getAttacker();
	if (Opponent) 
		strcpy(murderername,Opponent->name); 
	else 
		murderername[0]=0;

	P_CHAR pc_t = NULL;
	AllCharsIterator aci;
	for (aci.Begin();(pc_t = aci.GetData()) != NULL; aci++)
	{
		if (Victim->isSameAs(pc_t->GetTarget()))
		{
			if (pc_t->isAiTeleGuard())
			{
				pc_t->summontimer=(uiCurrentTime+(MY_CLOCKS_PER_SEC*20));
				pc_t->npcWander=2;
				pc_t->setNextMoveTime();
				npctalkall(pc_t,"Thou have suffered thy punishment, scoundrel.",0);
			}
			pc_t->ResetTarget();
			pc_t->timeout=0;
			P_CHAR Attacker = pc_t->getAttacker();
			if (Attacker)
			{
				Attacker->resetAttackFirst();
				Attacker->setAttacker(NULL);
			}
			pc_t->setAttacker(NULL);
			pc_t->resetAttackFirst();

			if (pc_t->isAiTeleGuard() || pc_t->npcaitype==9) // Ripper...so non human npcs disapear if killed by guards.
			{
				if (Victim->npc && !Victim->isHuman())
					KilledByGuard = true;
			}

			if (!pc_t->npc && !pc_t->inGuardedArea())
			{//AntiChrist
				Karma(pc_t, Victim, (0-(Victim->karma)));
				Fame(pc_t,Victim->fame);
				//murder count \/
				if (!Victim->npc && !pc_t->npc)//Player vs Player
				{
					if (Victim->isInnocent() && Guilds->Compare(pc_t,Victim)==0 && pc_t->attackfirst == 1)
					{
						// Ask the victim if they want to place a bounty on the murderer (need gump to be added to
						// BountyAskViction() routine to make this a little nicer ) - no time right now
						// BountyAskVictim( Victim->serial, pc_t->serial );
						Victim->murdererSer = pc_t->serial;

						pc_t->kills++;
						sprintf(temp, "You have killed %i innocent people.", pc_t->kills);
						UOXSOCKET sock = pc_t->calcSocket();
						sysmessage(sock, temp);
						if (pc_t->kills==(unsigned)repsys.maxkills)
							sysmessage(sock, "You are now a murderer!");
						setcharflag(pc_t);//AntiChrist
					}

					if (SrvParms->pvp_log)
					{
						sprintf(temp,"%s was killed by %s!\n",Victim->name,pc_t->name);
						savelog(temp,"PvP.log");
					}
				}
			}//if !npc
			if (pc_t->npc && pc_t->war) npcToggleCombat(pc_t); // ripper
		}
	}

	int z=Victim->calcSocket();
	if (z!=-1) unmounthorse(z);

	int ci=0,loopexit=0;
	P_ITEM TrW;
	while ( ((TrW=Victim->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
	{
		if (TrW->isType(ITY_Container) && TrW->pos.x==26 && TrW->pos.y==0 &&
			TrW->pos.z==0 && TrW->id()==0x1E5E )
		{
			endtrade(TrW);
		}
	}
	if (!Victim->npc)
		Victim->setId(Victim->xid()==0x190 ? 0x0192 : 0x0193);	// Male or Female ghost

	Victim->skin = 0x0000; // Undyed
	Victim->dead = true;	// Dead
	Victim->setHp(0);		// With no hp left
	Victim->SetPoison(0);

	// Make the corpse
	sprintf(temp,"corpse of %s",Victim->name);
	const P_ITEM Corpse = Items->SpawnItem(Victim, 1, temp, 0, 0x2006, Victim->xskin, 0);
	if (Corpse==NULL) return;//AntiChrist to preview crashes

	// Corpse highlighting.. Ripper
	if (!Victim->npc)
	{
	    if (Victim->isInnocent()) Corpse->more2=1;
	    else if (Victim->isCriminal()) Corpse->more2=2;
	    else if (Victim->isMurderer()) Corpse->more2=3;
        Corpse->ownserial=Victim->serial;
	}// end corpse highlighting.

	Corpse->amount=Victim->xid(); // Amount == corpse type
	Corpse->morey=isTrueHuman(Victim);
	Corpse->carve=Victim->carve;//store carve section - AntiChrist
	strcpy(Corpse->name2,Victim->name);

	Corpse->setType(ITY_Container);
	Corpse->more1=0;
	Corpse->dir=Victim->dir;
	Corpse->corpse=1;
	Corpse->setNotMovable();
	Corpse->startDecay();
	Corpse->MoveTo(Victim->pos);
	
	//JustMichael -- If it was a player set the ownerserial to the player's
	if ( Victim->npc != 1 )
	{
		Corpse->SetOwnSerial(Victim->serial);
		Corpse->more4 = char( SrvParms->playercorpsedecaymultiplier&0xff ); // how many times longer for the player's corpse to decay
	}

	strcpy(Corpse->murderer,murderername);	//AntiChrist -- stores the time and the murderer's name
	Corpse->murdertime=uiCurrentTime;

	int q;

	if (( !VictimInGuardedRegion && Victim->npc && Victim->account==-1) || !Victim->npc )
		// no loot for npcs in guarded areas !!!
	{

	  // note: though it's a huge optimization to generate loot only when the mobile really dies,
	  // it's a bit stupid to copy loot into BP for a couple of nano-secs and delete afterward,
      // after coping into corpse. well, true.
	  // main reason for doing it that way: code reuse.

	  if (Victim->npc) // if it's a npc, add loot to backpack !!!
	  {
		  Npcs->ApplyDeathLootTags(Victim);
	  }

	  // Put objects on corpse
	  P_ITEM Pack = Victim->getBackpack();
	  P_ITEM LootE;	// for equipped items
	  ci=0,loopexit=0;
	  while ( ((LootE=Victim->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
	  {
		Victim->removeItemBonus(LootE);	// for BONUS ITEMS - remove bonus
		if ((LootE->trigon==1) && (LootE->layer >0) && (LootE->layer!=15) && (LootE->layer<19))// -Frazurbluu- Trigger Type 2 is my new trigger type *-
		{
			triggerwitem(z, LootE, 1); // trigger is fired when item destroyed
		}
		if (LootE->layer!=0x0B && LootE->layer!=0x10)	//Let's check all items, except HAIRS and BEARD
		{
			if (IsChaosOrOrderShield(LootE->id()))		// Ripper...so order/chaos shields disappear when on corpse backpack.
			{
				Guilds->RemoveGuildShield(Victim,LootE);
			}
			if (LootE->isType(ITY_Container) && LootE->layer!=0x1A && LootE->layer!=0x1B &&
				LootE->layer!=0x1C && LootE->layer!=0x1D)
			{//if this is a pack but it's not a VendorContainer(like the buy container) or a bankbox
				int ci1=0;
				int loopexit1=0;
				P_ITEM LootP;	// for loot from the packs
				while ( ((LootP=LootE->Search(&ci1)) != NULL) && (++loopexit1 < MAXLOOPS) )
				{//let's search for all the items in the pack ( now that we now this' the character's backpack )
					if ( !LootP->newbie() && !LootP->isType(ITY_SpellBook))//Morrolan spellbook disappearance fix
					{//put the item in the corpse only of we're sure it's not a newbie item or a spellbook
						LootP->layer=0;
						LootP->SetContSerial(Corpse->serial);
						LootP->SetRandPosInCont(Corpse);
						// Ripper...so order/chaos shields disappear when on corpse backpack.
						if (IsChaosOrOrderShield(LootP->id()))
						{
							Guilds->RemoveGuildShield(Victim,LootP);
						}
						LootP->refresh();
					}
				}
			}
			else if (LootE->layer==0x1A)
			{//else if it's a NPC vendor special container, clear potential sell/buy windows (Duke)
				P_CHAR pc;
				AllPlayersInRangeIterator api(Victim);
				for (pc = api.First(); pc != NULL; pc = api.Next())
				{
					SndClearTrade(api.getSocket(), Victim->serial);
				}
			}//else if it's a normal item but ( not newbie and not bank items )
			else if (!LootE->newbie() && LootE->layer!=0x1D)
			{
				if (!LootE->isSameAs(Pack))
				{
					//LootE->layer=0; // only un-rem if you want naked corpses.
					LootE->SetContSerial(Corpse->serial);
				}
			}
			else if (Pack && LootE->layer!=0x1D)
			{//else if the item is newbie put it into char's backpack
				if (!LootE->isSameAs(Pack))
				{
					LootE->layer=0;
					LootE->SetContSerial(Pack->serial);
				}
			}
			if (LootE->layer==0x15 && Victim->shop==0) LootE->layer=0x1A;
			LootE->pos.x = RandomNum(20, 70);
			LootE->pos.y = RandomNum(85, 160);
			LootE->pos.z=9;
			SndRemoveItemToAll(LootE);
			LootE->refresh();
		}
		if (Victim->Wears(LootE) && ( LootE->layer==0x0B || LootE->layer==0x10 ))
		{
			strcpy(LootE->name,"Hair/Beard");
			LootE->pos.x=0x47;
			LootE->pos.y=0x93;
			LootE->pos.z=0;
//			LootE->SetContSerial(Corpse->serial);
//			LootE->refresh();
		}
	}
	}


	if (!Victim->npc)
	{
		const P_ITEM Shroud = Items->SpawnItem(Victim, 1, "a Death Shroud", 0, 0x204E, 0, 0);
		if (!Shroud) return;
		Victim->robe = Shroud->serial; 
		Shroud->def=1;
		Victim->Wear(Shroud,0x16);
	}

	if (SrvParms->showdeathanim) deathaction(Victim, Corpse);

	if (Victim->account!=-1) // LB
	{
		teleport(Victim, Victim->pos); // coords NOT changed here !
		q = Victim->calcSocket();
		if (q>-1) deathmenu(q);
	}

	Corpse->refresh();
	if (Victim->summoned || KilledByGuard) Items->DeleItem(Corpse);	// no corpse for summoned creatures
	if (Victim->npc) Npcs->DeleteChar(Victim);
}


void explodeitem(P_CHAR Player, P_ITEM pi)
{
	unsigned int dmg=0,len=0;
	unsigned int dx,dy,dz;

	if (!Player || !pi) return;

	// - send the effect (visual and sound)
	if (!pi->isInWorld()) //bugfix LB
	{	
		soundeffect2(Player, 0x0207);
	}
	else
	{
		staticeffect2(pi, (unsigned char)0x36, (unsigned char)0xB0, (unsigned char)0x10, (unsigned char)0x80, (unsigned char)0x00); 
		soundeffect3(pi, 0x0207);
	}

	len=pi->morex/250; //4 square max damage at 100 alchemy
	switch (pi->morez)
	{
	case 1:dmg=RandomNum( 5,10) ;break;
	case 2:dmg=RandomNum(10,20) ;break;
	case 3:dmg=RandomNum(20,40) ;break;
	default:
		LogErrorVar("switch reached default for %i",pi->morez);
		dmg=RandomNum(5,10);
	}

	if (dmg<5) dmg=RandomNum(5,10);	// 5 points minimum damage
	if (len<2) len=2;	// 2 square min damage range

	cChar* pc;
	int loopexit=0;

	RegGrid3x3Iterator4Chars rg3ic(pi->pos);
	while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (pc->isInvul() || pc->isAiVendor() || pc->isDummy()) // don't affect vendors and dummy storage npcs
			continue;
		if (pc->isGM() || (!pc->npc && !pc->InGame()) )
			continue;
		dx=abs(pc->pos.x-pi->pos.x);
		dy=abs(pc->pos.y-pi->pos.y);
		dz=abs(pc->pos.z-pi->pos.z);
		if ((dx<=len)&&(dy<=len)&&(dz<=len))
		{
			pc->reduceHp(dmg+(2-min(dx,dy)));
			if (pc->Hp() > 0)
			{
				npcattacktarget(pc,Player);
				updatechar(pc);
			}
		}
	}

	if (pi->isInWorld()) // no chain reactions if its exploding inside BP, even if more potions in BP
	{
		int chain=0;
		PC_ITEM piMap;
		loopexit=0;
		RegGrid3x3Iterator4Items rg3ii(pi->pos);
		while((piMap = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
		{
			if (piMap->id()==0x0F0D && piMap->isType(ITY_Potion) && !piMap->free && !pi->isSameAs(piMap) && piMap->morey!=POTION_EXPLODING) // check for expl-potions
			{
				dz=abs(pi->pos.z - piMap->pos.z);
				if (inRange(pi->pos,piMap->pos,2) && dz<=2 && chain==0) // only trigger if in 2*2*2 cube
				{						
					if (rand()%2==1) chain=1; // LB - more aggressive - :)
					tempeffect(Player, piMap->unconst(), TEF_Explode, 0, (rand()%4)+1, 0); // trigger ...
				}			
			}
		}
	}
	Items->DeleItem(pi);
}

// Dupois - added doorsfx() to be used with dooruse()
// Added Oct 8, 1998
// Plays the proper door sfx for doors/gates/secretdoors
static void doorsfx(P_ITEM pi, int x, int y)
{
	const int OPENWOOD = 0x00EA;
	const int OPENGATE = 0x00EB;
	const int OPENSTEEL = 0x00EC;
	const int OPENSECRET = 0x00ED;
	const int CLOSEWOOD = 0x00F1;
	const int CLOSEGATE = 0x00F2;
	const int CLOSESTEEL = 0x00F3;
	const int CLOSESECRET = 0x00F4;

	if (y==0) // Request open door sfx
	{
		if (((x>=0x0695)&&(x<0x06C5))|| // Open wooden / ratan door
			((x>=0x06D5)&&(x<=0x06F4)))
			soundeffect3(pi,OPENWOOD);

		if (((x>=0x0839)&&(x<=0x0848))|| // Open gate
			((x>=0x084C)&&(x<=0x085B))||
			((x>=0x0866)&&(x<=0x0875)))
			soundeffect3(pi,OPENGATE);

		if (((x>=0x0675)&&(x<0x0695))|| // Open metal
			((x>=0x06C5)&&(x<0x06D5)))
			soundeffect3(pi,OPENSTEEL);

		if ((x>=0x0314)&&(x<=0x0365)) // Open secret
			soundeffect3(pi,OPENSECRET);
	}
	else if (y==1) // Request close door sfx
	{
		if (((x>=0x0695)&&(x<0x06C5))|| // close wooden / ratan door
			((x>=0x06D5)&&(x<=0x06F4)))
			soundeffect3(pi,CLOSEWOOD);

		if (((x>=0x0839)&&(x<=0x0848))|| // close gate
			((x>=0x084C)&&(x<=0x085B))||
			((x>=0x0866)&&(x<=0x0875)))
			soundeffect3(pi,CLOSEGATE);

		if (((x>=0x0675)&&(x<0x0695))|| // close metal
			((x>=0x06C5)&&(x<0x06D5)))
			soundeffect3(pi,CLOSESTEEL);

		if ((x>=0x0314)&&(x<=0x0365)) // close secret
			soundeffect3(pi,CLOSESECRET);
	}
} // doorsfx() END

#define DOORTYPES 17
const unsigned short doorbase[DOORTYPES]={
0x0675, 0x0685, 0x0695, 0x06A5, 0x06B5, 0x06C5, 0x06D5, 0x06E5, 0x0839, 0x084C, 
0x0866, 0x00E8, 0x0314, 0x0324, 0x0334, 0x0344, 0x0354};

static short GetDoorBase(short id)
{
	int i,db;
	for (i=0;i<DOORTYPES;i++)
	{
		db=doorbase[i];
		if (id >= db && id <= db+15)
			return db;
	}
	return 0;
}

bool IsDoor(short id)
{
	if (GetDoorBase(id))
		return true;
	else
		return false;
}

static void ChangeDoorPos(short doorflags, cCoord &doorpos)
{
	short x=0;
	short y=0;
	switch (doorflags)
	{
	case 0:	x--;y++;break;
	case 1:	x++;y--;break;
	case 2:	x++;y++;break;
	case 3:	x--;y--;break;
	case 4:	x--;	break;
	case 5:	x++;	break;
	case 6:	x++;y--;break;
	case 7:	x--;y++;break;
	case 8:	x++;y++;break;
	case 9:	x--;y--;break;
	case 10:x++;y--;break;
	case 11:x--;y++;break;
	case 12:		break;
	case 13:		break;
	case 14:	y--;break;
	case 15:	y++;break;
	}
	doorpos.Add(x,y);
}

bool DoorIsBlockedByChar(P_ITEM pi)
{
	if (!pi) return false;

	short id = pi->id();
	short db = GetDoorBase(id);
	if (!db) return false;

	short flags = id-db;
	cCoord newPos = pi->pos;
	ChangeDoorPos(flags,newPos);

	P_CHAR pc;
	int loopexit=0;
	RegCellIterator4Chars rcic(newPos);
	while((pc = rcic.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (!pc->pos.IsSameXY(newPos))
			continue;
		if (!pc->dead && !pc->isDummy() && (pc->InGame() || pc->npc))		// ghosts don't block ;) //thrawn: dummys and offline players shouldn't as well
			return true;
	}
	return false;
}

void dooruse(UOXSOCKET s, P_ITEM pi)
{
	if (!pi) return;

	if (s>-1) if (!iteminrange(s,pi,2)) {sysmessage(s, "You cannot reach the handle from here");return;}

	short id = pi->id();
	short db = GetDoorBase(id);
	if (!db)
	{
		if (s>-1)
			sysmessage(s, "This doesnt seem to be a valid door type. Contact a GM.");
		return;
	}
	if (DoorIsBlockedByChar(pi))
	{
		if (s>-1)
			sysmessage(s, "There is something in the way!");
		return;
	}
	short flags = id-db;
	if (flags & 1)	// is it an open door ?
	{
		pi->setId(id-1);			// so close it
		doorsfx(pi, id, 1);
		pi->dooropen=0;
	}
	else
	{
		pi->setId(id+1);			// open it
		doorsfx(pi, id, 0);
		tempeffect(pi, TEF_Door);
	}

	ChangeDoorPos(flags,pi->pos);
	pi->refresh();
	

	if (s>-1)
	{
		P_CHAR Player = MAKE_CHARREF_LR(currchar(s));
		PC_ITEM House = findHouseAt(pi->pos);
		if ( House )
		{
			int j=on_hlist(House, Player->serial);
			if ( j==H_FRIEND || Player->Owns(House) ) // house_refresh stuff, LB, friends of the house can do.
			{
				refreshhouse(s,Player,House->unconst());
			}
		}
	}
}

int validhair(short ID) // Is selected hair type valid
{	
	switch( ID )
	{
	case 0x203B:
	case 0x203C:
	case 0x203D:
	case 0x2044:
	case 0x2045:
	case 0x2046:
	case 0x2047:
	case 0x2048:
	case 0x2049:
	case 0x204A:
		return 1;
	default:
		return 0;
	}
	return 1;
}

int validbeard(short b) // Is selected beard type valid
{
	switch( b )
	{
	case 0x203E:
	case 0x203F:
	case 0x2040:
	case 0x2041:
	case 0x204B:
	case 0x204C:
	case 0x204D:
		return 1;
	}
	return 0;
}

void charcreate( UOXSOCKET s ) // All the character creation stuff
{
	signed int ii ;
	int totalstats,totalskills;
	P_CHAR pc = Npcs->MakeBaseChar();

	strncpy(pc->name, (char*)(&buffer[s][10]), 50);
	pc->account = acctno[s];
	
	if (SrvParms->char_log)
	{
		char logtxt[512];
		sprintf(logtxt, "%s (%i) was created on account %i\n", pc->name, pc->serial, pc->account);
		savelog(logtxt, "characters.log");
	}
	
	//	Code to support the new GuildType, and GuildTraitor members of the
	//	character strcture.
	pc->GuildType=-1;				//	Default to no guild
	pc->GuildTraitor=false;	//	Also default to non trator, or no guild
	pc->setRace(1);

	if (buffer[s][0x46]!='\x00')
	{
		pc->setId(0x191);
		pc->setXid(0x191);
	}


	pc->skin = ( ( (buffer[s][0x50]) << 8) ) + buffer[s][0x51];
	pc->xskin = pc->skin;

	pc->setPriv(SrvParms->defaultpriv1);
	pc->SetPriv2(SrvParms->defaultpriv2);

	if (acctno[s]==0)
	{
		pc->setPriv(0xE7);
		pc->priv3[0]=0xFFFFFFFF; // lb, meta gm
		pc->priv3[1]=0xFFFFFFFF;
		pc->priv3[2]=0xFFFFFFFF;
		pc->priv3[3]=0xFFFFFFFF;
		pc->priv3[4]=0xFFFFFFFF;
		pc->priv3[5]=0xFFFFFFFF;
		pc->priv3[6]=0xFFFFFFFF;
	}

	int start_loc = (buffer[s][0x5B]);
	if (start_loc<=9)
	{
		cCoord nPos(makenum2(start[start_loc][2]), makenum2(start[start_loc][3]), makenum2(start[start_loc][4]),DEFAULT_MAP);
		pc->MoveTo(nPos);
	}
	else
	{
		cCoord nPos(1000,100,0,DEFAULT_MAP);
		pc->MoveTo(nPos);
	}


	pc->dir=4;

	// max/min start stats
	int max_total=80; // 65 in pre 3.0 clients
	int max_indiv=60; // 45 in pre 3.0 clients
	int min_indiv=10;

	/// note: we are not checking if pre or post 3.0 clients and set numbers to 65/80 accordingly.
	/// not worth it :] 
	/// if users decide to hack pre 3.0 clients they can get the damn 15 plus start stat points more they get anyway with 3.0 clients.

	short st = buffer[s][0x47];
	if (st>max_indiv) st=max_indiv;		// fix for hack exploit
	if (st<min_indiv) st=min_indiv;
	totalstats=st;

	short d=buffer[s][0x48];
	if (d>max_indiv) d=max_indiv;		// fix for hack exploit
	if (d<min_indiv) d=min_indiv;
	if (d+totalstats>max_total) d=max_total-totalstats;
	totalstats+=d;

	short in=buffer[s][0x49];
	if (in>max_indiv) in=max_indiv;		// fix for hack exploit
	if (in<min_indiv) in=min_indiv;
	if (in+totalstats>max_total) in=max_total-totalstats;
	totalstats+=in;

	if (SrvParms->thehardway & 1)	// bit x01 means: NO stats above 10, regardless what they selected
	{
		st=10;
		d=10;
		in=10;
	}

	pc->setStr(st);
	pc->fullHp();
	pc->setDex(d);
	pc->fullStm();
	pc->setInt(in);
	pc->mn=in;

	
	// max/min start skills
    int maxskill_indiv=50;
	int maxskill_total=100;

	if (buffer[s][0x4b]>maxskill_indiv) buffer[s][0x4b]=maxskill_indiv; // fixes for hack exploit
	totalskills=buffer[s][0x4b];
	if (buffer[s][0x4d]>maxskill_indiv) buffer[s][0x4d]=maxskill_indiv;
	if (buffer[s][0x4d]+totalskills>maxskill_total) buffer[s][0x4d]=maxskill_total-totalskills;
	totalskills+=buffer[s][0x4d];
	if (buffer[s][0x4f]>maxskill_indiv) buffer[s][0x4f]=maxskill_indiv;
	if (buffer[s][0x4f]+totalskills>maxskill_total) buffer[s][0x4f]=maxskill_total-totalskills;

	int sk=SkillManager->getAllSkillsValue();
	for (ii=0;ii<sk;ii++)
	{
		pc->setRealSkill(ii,0);
		if (!(SrvParms->thehardway & 2))	// bit x02 means: NO skills, regardless what they selected
		{
			if (ii==buffer[s][0x4a]) pc->setRealSkill(ii,buffer[s][0x4b]*10);
			if (ii==buffer[s][0x4c]) pc->setRealSkill(ii,buffer[s][0x4d]*10);
			if (ii==buffer[s][0x4e]) pc->setRealSkill(ii,buffer[s][0x4f]*10);
			SkillManager->updateSkillLevel(pc, ii);
		}
	}


	short HairId=ShortFromCharPtr(&buffer[s][0x52]);
	if (validhair(HairId))
	{
		const P_ITEM pi = Items->SpawnItem(pc,1, "#", 0, HairId, ShortFromCharPtr(&buffer[s][0x54]),0);
		if (!pi) 
		{		
			return;
		}	
		if (pi->color() < 0x044E || pi->color() > 0x04AD)
		{
			pi->setColor(0x044E);
		}
		pi->setNotMovable(); // anti EUO exploit 
		pc->Wear(pi,0x0B);
	} 

	short beardId=ShortFromCharPtr(&buffer[s][0x56]);
	if ( validbeard(beardId) && pc->id()==0x0190 )
	{
		const P_ITEM pi = Items->SpawnItem(pc,1, "#", 0, beardId, ShortFromCharPtr(&buffer[s][0x58]),0);
		if (!pi) return;
		if (pi->color() < 0x044E || pi->color() > 0x04AD)
		{
			pi->setColor(0x044E);
		}
		pi->setNotMovable(); // anti EUO exploit
		pc->Wear(pi,0x10);
	}

	// - create the backpack
	const P_ITEM Pack = pc->addBackpack();
	if (!Pack) return;



	const P_ITEM pi_pants = Items->SpawnItem(pc,1,"#",0,0x0915,0,0); // spawn pants
	if (!pi_pants) return;
	short pantlayer;
	
	if (pc->id()==0x190 && pc->xid()==0x190)
	{
		pi_pants->setId(rand()%2 ? 0x1539 : 0x152E);
		pantlayer=0x04; // pant
	}
	else
	{
		pi_pants->setId(rand()%2 ? 0x1516 : 0x1537);
		pantlayer=23; //skirt
	}
	
	// pant/skirt color -> old client code, random color
	pi_pants->setColor(ShortFromCharPtr(&buffer[s][102]));
	pi_pants->setType(ITY_None);
	pi_pants->dye=1;
	pi_pants->hp=10;
	pc->Wear(pi_pants,pantlayer);

	//pi_pants->SetOwnSerial(pc->serial);
	//pi_pants->setOwnerMovable();

	const P_ITEM pi_shirt = Items->SpawnItem(pc,1,"#",0,0x0915,0,0); // spawn shirt
	if (!pi_shirt) return;
    pi_shirt->setId(rand()%2 ? 0x1EFD : 0x1517);
	pi_shirt->setColor(ShortFromCharPtr(&buffer[s][100]));
	pi_shirt->dye=1;
	pi_shirt->hp=10;
	pi_shirt->def=1;
	pc->Wear(pi_shirt,0x05);

	//pi_shirt->SetOwnSerial(pc->serial);
	//pi_shirt->setOwnerMovable();

	const P_ITEM pi_shoes = Items->SpawnItem(pc,1,"#",0,0x170F,0x0287,0); // shoes
	if (!pi_shoes) return;
	pi_shoes->dye=1;
	pi_shoes->hp=10;
	pi_shoes->def=1;
	pc->Wear(pi_shoes,0x03);

	//pi_shoes->SetOwnSerial(pc->serial);
	//pi_shoes->setOwnerMovable();

	const P_ITEM pi_dagger = Items->SpawnItem(pc,1,"#",0,0x0F51,0,0); // dagger
	if (!pi_dagger) return;
	pi_dagger->hp=10;
	pi_dagger->spd=50;
	pi_dagger->lodamage=3;
	pi_dagger->hidamage=15;
	pi_dagger->itmhand=1;
	pc->Wear(pi_dagger,0x01);


#ifdef SPECIAL

	n=Items->SpawnItem(s,1,"#",0,0x09,0x15,0,0,0,0); 
	if (n==-1) return;//AntiChrist to preview crashes
	const P_ITEM pi_special=MAKE_ITEMREF_LR(n);	// on error return
	
	switch (RandomNum(0, 6))
	{
	case 0:	pi_special->setId(0x154b); break;
	case 1:	pi_special->setId(0x1545); break;
	case 2:	pi_special->setId(0x1547); break;
	case 3:	pi_special->setId(0x1549); break;
	case 4:	pi_special->setId(0x171c); break;
	case 5:	pi_special->setId(0x1f0b); break;
	case 6:	pi_special->setId(0x1451); break;
	}
	
	pi_special->SetContSerial(pc->serial);
	pi_special->layer=0x06;

	//pi_special->SetOwnSerial(pc->serial);
	//pi_special->setOwnerMovable();
	
#endif

	// Give the character some gold
	if (SrvParms->startgold > 0)	// no gold, no item ;)
	{
		P_ITEM Gold=Items->SpawnItem(pc,SrvParms->startgold,"#",1,0x0EED,0,1);
		if (!Gold) return;
	}

	setcurrchar(s,pc->GetIndex());
	if (!(SrvParms->thehardway & 4))	// bit x04 means: no additional items
		newbieitems(s,pc);

	perm[s]=1;

	Accounts->AddPlayerToAccountShadow(pc); // Account/player-list housekeeping
	Network->startchar(s);
}

int unmounthorse(int s) // Get off a horse (Remove horse item and spawn new horse) 
{ 
	P_CHAR pc = MAKE_CHARREF_LRV(currchar(s), -1); 
	if (!pc) return -1;
	
	pc->onhorse = false;
	
	P_ITEM pi = pc->GetItemOnLayer(0x19);
	if (pi)
	{
		P_CHAR Pet = FindCharBySerial(pi->morex);
		if ( (Pet != NULL) && (pc->Owns(Pet)) )
		{
			Pet->doppelgaenger_serial=0;
			Pet->MoveTo(pc->pos);
			updatechar(Pet);
		}
		else
		{
			Console.send("Makeing a new horse cause the other somehow got out/lost :(\n"); 
			int m;	// script # of mount to be spawned
			switch (pi->id() & 0x00FF) 
			{ 
				case 0xA0:	m=SNC_MountA0;	break; 
				case 0xA1:	m=SNC_MountA1;	break; 
				case 0xA2:	m=SNC_MountA2;	break; 
				case 0xA3:	m=SNC_MountA3;	break; 
				case 0xA4:	m=SNC_MountA4;	break; 
				case 0xA5:	m=SNC_MountA5;	break; 
				case 0xA6:	m=SNC_MountA6;	break; 
				case 0x9F:	m= (pi->color() == 0x0455) ? SNC_Mount9Fa : SNC_Mount9Fb;	break; 
				default: 
					LogCritical("No horse type in unmounthorse(), avoiding crash and removing mount"); 
					Items->DeleItem(pi); // bugfix: cant get off the hors if its not found (Thrawn)
					return -1; 
			} 
			Pet = Npcs->AddNPC(s, m, pc->pos);
			strcpy(Pet->name, pi->name); 
			Pet->setXid(Pet->id()); 
			Pet->skin  = Pet->xskin = pi->color(); 
			Pet->npcWander          = pi->moreb1; 
			Pet->poisoned           = pi->Poisoned(); 
			Pet->dispz = Pet->pos.z = pc->pos.z; 
			Pet->dir                = pc->dir; 
			Pet->SetOwnSerial( pc->serial ); 
			if (pi->decaytimer.IsRunning()) 
				Pet->summontimer = pi->decaytimer.getValue(); 
			Pet->setAi(0); 
			updatechar(Pet); 
		}
		Items->DeleItem(pi); 
		return 0; 
	}
	return -1; 
}

void endmessage(int x) // If shutdown is initialized
{
	char temp[1024];
	x = 0;
	unsigned int igetclock = uiCurrentTime;
	if (endtime < igetclock)
		endtime = igetclock;
	
	if ((((endtime - igetclock)/MY_CLOCKS_PER_SEC)/60) < 1) 	
		sprintf(temp, "Server going down in %i secs.", (endtime - igetclock)/MY_CLOCKS_PER_SEC); // blackwind %i Secs fix..
	else
		sprintf(temp, "Server going down in %i minutes.",
		((endtime - igetclock)/MY_CLOCKS_PER_SEC)/60);
	sysbroadcast(temp);
	Console.send(temp);
}

short scriptCommandtoNum(char *sc)
{
	if (!strcmp("GMMENU", sc))				return 0;	
	else if (!strcmp("ITEMMENU", sc))		return 1;	
	else if (!strcmp("WEBLINK", sc))		return 2;	
	else if (!strcmp("SYSMESSAGE", sc))		return 3;	
	else if (!strcmp("GMPAGE", sc))			return 4;	
	else if (!strcmp("CPAGE", sc))			return 5;
	else if (!strcmp("VERSION", sc))		return 6;		
	else if (!strcmp("ADDITEM", sc))		return 7;	
	else if (!strcmp("BATCH", sc))			return 8;	
	else if (!strcmp("INFORMATION", sc))	return 9;
	else if (!strcmp("NPC", sc))			return 10;	
	else if (!strcmp("POLY", sc))			return 11;	
	else if (!strcmp("SKIN", sc))			return 12;	
	else if (!strcmp("LIGHT", sc))			return 13;	
	else if (!strcmp("DRY", sc))			return 14;	
	else if (!strcmp("RAIN", sc))			return 15;	
	else if (!strcmp("SNOW", sc))			return 16;	
	else if (!strcmp("GCOLLECT", sc))		return 17;	
	else if (!strcmp("GOPLACE", sc))		return 18;	
	else if (!strcmp("MAKEMENU", sc))		return 19;
	else if (!strcmp("CREATETRACKINGMENU", sc)) return 20;	
	else if (!strcmp("TRACKINGMENU", sc))	return 21;
	return 99;	
}

void numtoScriptCommand(short num, char* com)
{
	switch (num)
	{
	   case 0: strcpy(com,"GMMENU"); break;
	   case 1: strcpy(com,"ITEMMENU"); break;
	   case 2: strcpy(com,"WEBLINK"); break;
	   case 3: strcpy(com,"SYSMESSAGE"); break;
	   case 4: strcpy(com,"GMPAGE"); break;
	   case 5: strcpy(com,"CPAGE"); break;
	   case 6: strcpy(com,"VERSION"); break;
	   case 7: strcpy(com,"ADDITEM"); break;
	   case 8: strcpy(com,"BATCH"); break;
	   case 9: strcpy(com,"INFORMATION"); break;
	   case 10: strcpy(com,"NPC"); break;
	   case 11: strcpy(com,"POLY"); break;
	   case 12: strcpy(com,"SKIN"); break;
	   case 13: strcpy(com,"LIGHT"); break;
	   case 14: strcpy(com,"DRY"); break;
	   case 15: strcpy(com,"RAIN"); break;
	   case 16: strcpy(com,"SNOW"); break;
	   case 17: strcpy(com,"GCOLLECT"); break;
	   case 18: strcpy(com,"GOPLACE"); break;
	   case 19: strcpy(com,"MAKEMENU"); break;
	   case 20: strcpy(com,"CREATETRACKINGMENU"); break;
	   case 21: strcpy(com,"TRACKINGMENU"); break;	  	   
       default: strcpy(com,"INVALID"); break;
	}
}

void scriptcommand (int s, char *script1, char *script2) // Execute command from script
{
	char tstring[1024];
	int total, ho, mi, se;
	int tmp ;
	int c;
	P_CHAR Player = MAKE_CHARREF_LR(currchar(s));
	strupr(script1);
	strupr(script2);

	if (!strcmp("GMMENU", script1))
	{
		gmmenu(s, str2num(script2));
		return;
	}
	if (!strcmp("ITEMMENU", script1))
	{
		itemmenu(s, str2num(script2));
		return;
	}
	if (!strcmp("WEBLINK", script1))
	{
		weblaunch(s, script2);
		return;
	}
	if (!strcmp("SYSMESSAGE", script1))
	{
		sysmessage(s, script2);
		return;
	}
	if (!strcmp("GMPAGE", script1))
	{
		Commands->GMPage(s, script2);
		return;
	}
	if (!strcmp("CPAGE", script1))
	{
		Commands->CPage(s, script2);
		return;
	}
	if (!strcmp("VERSION", script1))
	{
		sysmessage(s, idname);
		return;
	}
	//AntiChrist - no need of skill type check
	if (!strcmp("ADDITEM", script1))
	{
		addmitem[s]=str2num(script2);
		Skills->MakeMenuTarget(s,addmitem[s],Player->making);
		return;
	}
	if (!strcmp("BATCH", script1))
	{
		executebatch=str2num(script2);
		return;
	}
	if (!strcmp("INFORMATION", script1))
	{
		total=(uiCurrentTime-starttime)/MY_CLOCKS_PER_SEC;
		ho=total/3600;
		total-=ho*3600;
		mi=total/60;
		total-=mi*60;
		se=total;
		total=0;
		c=0;
		int totaltotal=0;
		int cc=0;
		PC_ITEM pi;
		AllItemsIterator aii;
		for (aii.Begin(); (pi = aii.GetData()) != aii.End(); aii++)
			{ total++; totaltotal++; }

		P_CHAR pc;
		AllCharsIterator aci;
		for (aci.Begin(); (pc=aci.GetData()) != NULL; aci++) 
		{ 
			if (!pc->free) 
				c++; 
			cc++; 
		}
		sprintf(tstring, "Time up [%i:%i:%i] Connected players [%i out of %i accounts] Items [%i] Characters [%i]",
			ho,mi,se,now,Accounts->Count(),total,c);

		sysmessage(s, tstring);
		sprintf(tstring, "Runtime lag factors: Corpses not freed from memory [%i] Decayed items still in memory [%i]",
			cc-c,totaltotal-total);
		sysmessage(s,tstring);
		return;
	}
	if (!strcmp("NPC", script1))
	{
		addmitem[s]=str2num(script2);
		sprintf(tstring, "Select location for NPC. [Number: %i]", addmitem[s]);
		target(s, 27, tstring);
		return;
	}
	if (!strcmp("POLY", script1))
	{
		tmp = hstr2num(script2);
		Player->setId(tmp);
		Player->setXid(tmp);
		updatechar(Player);
		return;
	}
	if (!strcmp("SKIN", script1))
	{
		Player->skin = Player->xskin = static_cast<UI16>(hstr2num(script2));
		return;
	}
	if (!strcmp("LIGHT", script1))
	{
		timeLight.worldfixedlevel=hstr2num(script2);
		if (timeLight.worldfixedlevel!=255) setabovelight(timeLight.worldfixedlevel,-1);
		else setabovelight(timeLight.worldcurlevel,-1);
		return;
	}

	if (!strcmp("DRY", script1))
	{
		return;
	}

	if (!strcmp("RAIN", script1))
	{
		return;
	}
	if (!strcmp("SNOW", script1))
	{
		return;
	}

	if (!strcmp("GCOLLECT", script1))
	{
		gcollect();
		return;
	}
	if (!strcmp("GOPLACE", script1))
	{
		tmp=str2num(script2);
		Commands->MakePlace (s, tmp);
		if (addx[s]!=0)
		{
			cCoord newPos(addx[s],addy[s],addz[s],DEFAULT_MAP);
			Player->teleTo(newPos);
		}
		return;
	}
	if (!strcmp("MAKEMENU", script1))
	{
		Skills->MakeMenu(s, str2num(script2), Player->making);
		return;
	}
	if (!strcmp("CREATETRACKINGMENU", script1))
	{
		Skills->CreateTrackingMenu(s, str2num(script2));
		return;
	}
	if (!strcmp("TRACKINGMENU", script1))
	{
		Skills->TrackingMenu(s, str2num(script2));
		return;
	}
}

void batchcheck(int s) // Do we have to run a batch file
{
	sprintf(temp, "BATCH %i", executebatch);
	openscript("menus.scp");
	if (!i_scripts[menus_script]->find(temp))
	{
		closescript();
		return;
	}

	int loopexit=0;
	do
	{
		read2();
		if (script1[0]!='}')
		{
			if (!strcmp("ADDITEM", script1))	// bughunting, Duke, 5.6.2003
				LogAssert(itemmake[s].maxskill!=0);

			scriptcommand(s, script1, script2);
		}
	}
	while ( (script1[0]!='}') && (++loopexit < MAXLOOPS) );
	closescript();
	executebatch=0;
}

void npctalkall_runic(P_CHAR Speaker, char *txt,char antispam)
{
	if (!Speaker) return;

	bool machwas = false;

	if (antispam)
	{
		if (Speaker->antispamtimer<uiCurrentTime)
		{
			Speaker->antispamtimer=uiCurrentTime+MY_CLOCKS_PER_SEC*10;
			machwas=true;
		} else machwas=false;
	} else machwas=true;

	if (machwas)
	{
		P_CHAR pc;
		AllPlayersInRangeIterator api(Speaker);
		for (pc = api.First(); pc != NULL; pc = api.Next())
		{
			npctalk_runic(api.getSocket(), Speaker, txt,antispam);
		}
	}
}

void npcemoteall(P_CHAR Speaker, char *txt,unsigned char antispam) // NPC speech to all in range.
{
	if (!Speaker) return;

	bool machwas = false;

	if (antispam)
	{
		if (Speaker->antispamtimer<uiCurrentTime)
		{
			Speaker->antispamtimer=uiCurrentTime+MY_CLOCKS_PER_SEC*10;
			machwas=true;
		} else machwas=false;
	} else machwas=true;

	if (machwas)
	{
		P_CHAR pc;
		AllPlayersInRangeIterator api(Speaker);
		for (pc = api.First(); pc != NULL; pc = api.Next())
		{
			npcemote(api.getSocket(), Speaker, txt,0);
		}
	}
}

//taken from 6904t2(5/10/99) - AntiChrist
void callguards(P_CHAR Caller)
{
	if (!Caller) return;

	//AntiChrist - anti "GUARDS" spawn timer
	if (Caller->antiguardstimer<uiCurrentTime)
	{
		Caller->antiguardstimer=uiCurrentTime+(MY_CLOCKS_PER_SEC*10);
	} else return;

	if (!Caller->inGuardedArea() || !SrvParms->guardsactive )
		return;

	cChar* pc;
	int loopexit=0;
	RegGrid3x3Iterator4Chars rg3ic(Caller->pos);
	while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (Caller->dist(pc) < 15)
		{
			if ( !pc->dead && !pc->isInnocent() )
			{
				Npcs->SpawnGuard(pc, pc, pc->pos);
			}
		}
	}
}

void mounthorse(int s, P_CHAR Mount) // hide horse char and give player a horse item (Thrawn)
{ 
	P_CHAR Player = MAKE_CHARREF_LR(currchar(s));

	if (!Player || !Mount) return;

	if (npcinrange(Player, Mount, 2) == 0 && !Player->isGM()) return; 
	if (Player->Owns(Mount) || Player->isGM()) 
	{ 
		if (Player->onhorse) 
		{ 
			sysmessage(s, "You are already on a mount."); 
			return; 
		}

		if (Player->isGM())
		{ 
			Mount->SetOwnSerial( Player->serial ); 
			Mount->setAi(0); 
		} 

		strcpy(temp, Mount->name); 
		Player->onhorse = true; 
		const P_ITEM pi = Items->SpawnItem(Player, 1, temp, 0, 0x0915, Mount->skin, 0); 
		if (!pi) return;
		
		pi->id1 = 0x3E; 
		
		switch (Mount->id1)
		{
		case 0:
			{
				switch (Mount->id2) 
				{ 
				case 0xC8: 
					pi->setId(0x3E9F);
					if (!strcmp(Mount->name,"MYSTIC HORSE")) 
					{
						if (clientDimension[s]==2) pi->setId(0x3ECF);
						else pi->setId(0x3E00);
					}
					break; // Horse or Mare (based on color currently) 
				case 0xE2:	pi->setId(0x3EA0);	break; // Horse 
				case 0xE4:	pi->setId(0x3EA1);	break; // Horse 
				case 0xCC:	pi->setId(0x3EA2);	break; // Horse 
				case 0xD2:	pi->setId(0x3EA3);	break; // Desert Ostard 
				case 0xDA:	pi->setId(0x3EA4);	break; // Frenzied Ostard 
				case 0xDB:	pi->setId(0x3EA5);	break; // Forest Ostard 
				case 0xDC:	pi->setId(0x3EA6);	break; // llama 
				case 0x72:	pi->setId(0x3EA9);	break; // Dark Steed
				case 0x73:	pi->setId(0x3EAA);	break; // Etheral Horse
				case 0xBB:	pi->setId(0x3EB8);	break; // Ridgeback
				case 0x75:	pi->setId(0x3EA8);	break; // Silver Steed
				case 0x77:	pi->setId(0x3EB1);	break; // Faction Horse
				case 0x76:	pi->setId(0x3EB2);	break; // Faction Horse
				case 0x78:	pi->setId(0x3EAF);	break; // Faction Horse
				case 0x90:	pi->setId(0x3EB3);	break; // Sea Horse
				case 0x79:	pi->setId(0x3EB0);	break; // Faction Horse
				case 0x7A:	pi->setId(0x3EB4);	break; // Unicorn
				case 0x74:	pi->setId(0x3EB5);	break; // Nightmare
				case 0x84:	pi->setId(0x3EAD);	break; // Kiirin
				case 0xAA:	pi->setId(0x3EAB);	break; // Etheral Llama
				case 0xAB:	pi->setId(0x3EAC);	break; // Etheral Ostard
			
				case 0xb1:  pi->setId(0x3EB5);	break; // Nightmare
				case 0xb2:  pi->setId(0x3EB6);	break; // Nightmare
				case 0xb3:  pi->setId(0x3EB7);	break; // Nightmare

				case 0xbc:  pi->setId(0x3EB8);	break; // Yet another Ridgeback
				case 0xbe:  pi->setId(0x3E9E);	break; // Fire ethernal

				} // end switch id2 of case 0
				break; // case 0
			}
		case 1: break;
		case 2: break;
		case 3: // lbr mounts, currently (1-april 2002) only those 4 work, no idea if osi has further plans (LB)
			{
				switch (Mount->id2) 
				{
				case 0x17:	pi->setId(0x3Ebc);	break; // giant beetle				
				case 0x18: // no mount, but my test NPC
					pi->setId(0x3E00 + w_anim[0]);
					if (w_anim[1]==0xff) pi->id1=0x3f;
					break;
				case 0x19:	pi->setId(0x3Ebb);	break; // skeletal mount
				case 0x1a:	pi->setId(0x3Ebd);	break; // swamp dragon
				case 0x1f:	pi->setId(0x3Ebe);	break; // armor dragon
				}
				break;
			}
			
		default: break;
		} // end switch id1
		
		
		Player->Wear(pi,0x19);
		
		pi->MoveTo(Mount->fx1, Mount->fy1, Mount->fz1); 
		
		pi->moreb1 = Mount->npcWander; 
		pi->att = Mount->fx2; 
		pi->def = Mount->fy2; 
		
		// AntiChrist bugfixes - 11/10/99 
		pi->moreb2   = Mount->realStr(); 
		pi->moreb3   = Mount->realDex(); 
		pi->moreb4   = Mount->realInt(); 
		pi->lodamage = Mount->fame; 
		pi->hidamage = Mount->karma; 
		pi->poisoned = Mount->poisoned;
		pi->morex    = Mount->serial;  //using morex to store pet serial (thrawn)
		if (Mount->summontimer != 0) 
			pi->decaytimer.setValue(Mount->summontimer);
		
		SndWornItemsToAllInRange(Player);
		
		/////////
		/// make npc invisble (to regular players), but don't delete it
		/////////

		Mount->doppelgaenger_serial=-3;		// -3: marker for npc in mount status
		Mount->MoveToOneOff(Player->pos);	// put it beside him and get it out of old mapregions !
		Mount->war = 0; 
		Mount->setAttacker(NULL); 
		updatechar(Mount);
	} 
	else sysmessage(s, "You dont own that creature."); 
}

int isTrueHuman(P_CHAR pc)
{
	if (!pc) return 0;
	// Check if the Player or Npc is human! -- by Magius(CHE)
	if (pc->xid()==0x0190 || pc->xid()==0x0191) return 1;
	else return 0;
}

void npcToggleCombat(P_CHAR pc)
{
	if (!pc) return;
	pc->war=(!(pc->war));
	walking2(pc);
}

/// region: -1, set/change global light
/// region >=0: set/change regional light
void setabovelight(unsigned char lightchar, int reg)
{
	unsigned int i;

	if (reg<0 || reg>255)
	{
		if (lightchar != timeLight.worldcurlevel)
		{
			timeLight.worldcurlevel=lightchar;
			for (i=0;i<now;i++)
			{
				if (perm[i]) dolight(i);
			}
		}
	} else
	{
		if (lightchar != region[reg].curlight)
		{
			region[reg].curlight=lightchar;
			for (i=0; i<now;i++)
			{
				if (perm[i]) dolight(i);
			}

		}
	}

}

void telltime( UOXSOCKET s )
{
	char tstring[60];
	char tstring2[60];
	int lhour;
	lhour=timeLight.hour;
	
	if ((timeLight.minute>=0)&&(timeLight.minute<=14)) strcpy(tstring,"It is");
	else if ((timeLight.minute>=15)&&(timeLight.minute<=30)) strcpy(tstring,"It is a quarter past");
	else if ((timeLight.minute>=30)&&(timeLight.minute<=45)) strcpy(tstring,"It is half past");
	else
	{
		strcpy(tstring,"It is a quarter till");
		lhour++;
		if (lhour==0) lhour=12;
	}
	switch( lhour )
	{
	case 1: sprintf( tstring2, "%s one o'clock", tstring );		break;
	case 2: sprintf( tstring2, "%s two o'clock", tstring );		break;
	case 3: sprintf( tstring2, "%s three o'clock", tstring );	break;
	case 4: sprintf( tstring2, "%s four o'clock", tstring );	break;
	case 5: sprintf( tstring2, "%s five o'clock", tstring );	break;
	case 6: sprintf( tstring2, "%s six o'clock", tstring );		break;
	case 7: sprintf( tstring2, "%s seven o'clock", tstring );	break;
	case 8: sprintf( tstring2, "%s eight o'clock", tstring );	break;
	case 9: sprintf( tstring2, "%s nine o'clock", tstring );	break;
	case 10: sprintf( tstring2, "%s ten o'clock", tstring );	break;
	case 11: sprintf( tstring2, "%s eleven o'clock", tstring );	break;
	case 12:
		if ( timeLight.ampm ) 
			sprintf( tstring2, "%s midnight.", tstring );
		else 
			sprintf( tstring2, "%s noon.", tstring );
		break;
	}
	
	if (lhour==12) strcpy(tstring, tstring2);
	else if (timeLight.ampm)
	{
		if ((lhour>=1)&&(lhour<6)) sprintf(tstring,"%s in the afternoon.",tstring2);
		else if ((lhour>=6)&&(lhour<9)) sprintf(tstring,"%s in the evening.",tstring2);
		else sprintf(tstring,"%s at night.",tstring2);
	}
	else
	{
		if ((lhour>=1)&&(lhour<5)) sprintf(tstring,"%s at night.",tstring2);
		else sprintf(tstring,"%s in the morning.",tstring2);
	}
	
	sysmessage(s,tstring);
}

void impaction(int s, int act)
{
	P_CHAR Player = MAKE_CHARREF_LR(currchar(s));
	if (Player->onhorse && (act==0x10 || act==0x11))
	{
		Player->DoAction(0x1B);
		return;
	}
	if ((Player->onhorse || !Player->isHuman()) && act==0x22)
	{
		return;
	}
	Player->DoAction(act);
}

int chardir(P_CHAR pca, P_CHAR pcb)	// direction from character a to char b
{
	if (!pca || !pcb) return 0;
	int xdif = pcb->pos.x - pca->pos.x;
	int ydif = pcb->pos.y - pca->pos.y;

	int dir;
	if ((xdif==0)&&(ydif<0)) dir=0;
	else if ((xdif>0)&&(ydif<0)) dir=1;
	else if ((xdif>0)&&(ydif==0)) dir=2;
	else if ((xdif>0)&&(ydif>0)) dir=3;
	else if ((xdif==0)&&(ydif>0)) dir=4;
	else if ((xdif<0)&&(ydif>0)) dir=5;
	else if ((xdif<0)&&(ydif==0)) dir=6;
	else if ((xdif<0)&&(ydif<0)) dir=7;
	else dir=-1;

	return dir;
}

char indungeon(P_CHAR pc)
{
	if (!pc) return 0;
	int reg=pc->region;
	int override=777;

	if (reg>=0 && reg<256)
	{
		override=region[reg].dungeon;
	}

	if (override==0 || override==1) return override;

	if (pc->pos.x<5119) 
		return 0;

	int x1 = (pc->pos.x-5119)>>8;
	int y1 = (pc->pos.y>>8);

	switch (y1)
	{
	case 5:
	case 0:	return 1;
	case 1:
		if (x1 != 0) return 1;
		return 0;
	case 2:
	case 3:
		if (x1 < 3) return 1;
		else return 0;
	case 4:
	case 6:
		if (x1 == 0) return 1;
		else return 0;
	case 7:
		if (x1 < 2) return 1;
		else return 0;
	}
	return 0;
}

// 17.3.2002 Duke trying to figure out what this function *really* does
void npcattacktarget(P_CHAR Attacker, P_CHAR Target)
{
	if (!Attacker || !Target) return;
	if (Attacker->isSameAs(Target)) return;
	if (Target->dead || Attacker->dead) return;
	if ((Target->doppelgaenger_serial<=-1) || (Attacker->doppelgaenger_serial<=-1)) return; //invis storage npcs should not interact with other chars

	if (!line_of_sight(-1,Attacker->pos, Target->pos, LOS_STD_BLOCKERS)) return; //From Leviathan - Morrolan

	playmonstersound(Target, SND_STARTATTACK);

	unsigned int olddist=30 ;
	if (Target->HasTarget())		// Victim is already fighting
	{
		P_CHAR CurrTarg = Target->GetTarget();
		olddist=Target->dist(CurrTarg);
	}
	if (olddist > Target->dist(Attacker))	// if current attacker is closer ...
	{
		Target->SetTarget(Attacker);
		Target->setAttacker(Attacker);
		Target->setAttackFirst();		// BS (Duke)
	}

	olddist=30;
	if (Attacker->HasTarget())
	{
		P_CHAR CurrTarg = Attacker->GetTarget();
		olddist=Attacker->dist(CurrTarg);
	}
	if (olddist > Target->dist(Attacker) &&
		(!Attacker->isAiTeleGuard() || Attacker->HasTarget() ))
	{
		Attacker->SetTarget(Target);
		Attacker->setAttacker(Target);
		Attacker->resetAttackFirst();
	}

	// unhide and disturb both
	Target->unhide();
	Target->disturbMed();
	Attacker->unhide();
	Attacker->disturbMed();

	if (Target->npc)
	{
		if (!Target->war)
			npcToggleCombat(Target);
		Target->setNextMoveTime();		// ???
	}
	if (Attacker->npc && !Attacker->isAiTeleGuard())
	{
		if (!Attacker->war)
			npcToggleCombat(Attacker);
		Attacker->setNextMoveTime();
	}

	sprintf(temp, "You see %s attacking %s!", Attacker->name, Target->name);
	P_CHAR pc;
	AllPlayersInRangeIterator api(Target);
	for (pc = api.First(); pc != NULL; pc = api.Next())
	{
		Target->emotecolor=0x0026;
		npcemote(api.getSocket(), Attacker, temp,1);
	}
}

void npcsimpleattacktarget(P_CHAR Attacker, P_CHAR Target)
{
	if (!Attacker || !Target) return;

	if (Target->isSameAs(Attacker->GetTarget()) &&		// are they already fighting each other ?
		Attacker->isSameAs(Target->GetTarget()))
		return;

	if (Attacker->dead || Target->dead) return;

	Attacker->fight(Target);
	Attacker->setAttackFirst();
	Target->fight(Attacker);
	Target->resetAttackFirst();
}

void openbank(int s, P_CHAR pc)
{
	if (!pc) return;

	int ci=0,loopexit=0;
	P_ITEM Box;
	int banks=0;
	while ( ((Box=SearchOwnedItems(pc->serial,&ci)) != NULL) && (++loopexit < MAXLOOPS) )
	{
		if (Box->isType(ITY_Container) && Box->morex==1)
		{
			banks++;
			if (SrvParms->usespecialbank)//if using specialbank
			{
				if (Box->morey==0 && Box->morez==0) //if not initialized yet for the special bank
					Box->morey=123;//convert to new special bank
				
				if (Box->morey==123)//check if a goldbank
				{				
					wearIt(s,Box);
					backpack(s,Box);
				}
				else // prevent a bad exploit, namely having a bank box that doesnt close on running away 
					 // UO client automatically closes LAST bankbox, with specialbank we can open two, in the end players could
					 // have one permanenlty opened. also note that client does NOT notify on container close, so we always have to send 
					 // this clear packet no matter if the dangerous case happened or not.
				{
					//SndRemoveItem(s,Box);	// close the dual bankbox
					// exploit fix withdrawn for now because it causes weird client crashes (Duke & LB, 31.7.03)
				}

			}
			else
			{//don't check for goldbank		
				if (Box->morey!=123)
				{
					wearIt(s,Box);
					backpack(s,Box);
				}
			}
		}
	}

	if (!SrvParms->usespecialbank && banks > 1)
	{
		char tmp[256];
		sprintf(tmp,"Player %s: More than one bankbox (%i)\n",pc->name, banks); 
		LogWarning(tmp);
	}

	if (!banks)
	{
		sprintf(temp, "%s's bank box.", pc->name);
		const P_ITEM pic = Items->SpawnItem(pc,1,temp,0,0x09AB,0,0);
		if (!pic) return;
		pic->SetOwnSerial(pc->serial);
		pic->setNotMovable();
		pic->morex=1;
		if (SrvParms->usespecialbank)//AntiChrist - Special Bank
			pic->morey=123;//gold only bank
		pic->setType(ITY_Container);
		pc->Wear(pic,0x1D);
		wearIt(s,pic);
		backpack(s,pic);
	}
}

//
//special bank - AntiChrist
//
//If activated, you can only put golds into normal banks
//and there are special banks (for now we still use normal bankers,
//but u have to say the SPECIALBANKTRIGGER word to open it)
//where u can put all the items: one notice: the special bank
//is caracteristic of regions....so in Britain you don't find
//the items you leaved in Minoc!
//All this for increasing pk-work and commerce! :)
//(and surely the Mercenary work, so now have to pay strong
//warriors to escort u during your travels!)
//
void openspecialbank(int s, P_CHAR pc)
{
	P_CHAR Player = MAKE_CHARREF_LR(currchar(s));

	int ci=0,loopexit=0;
	P_ITEM Box;
	bool hasbankhere=false;

	while ( ((Box=SearchOwnedItems(pc->serial,&ci)) != NULL) && (++loopexit < MAXLOOPS) )
	{
		if (Box->isType(ITY_Container) && Box->morex==1)	// some sort of bank box
		{
			if (Box->morey!=123 )	//specialbank and the current region - AntiChrist
			{
				if (Box->morez==0)	//convert old banks into new banks
					Box->morez=Player->region;
				
				if (Box->morez==Player->region)
				{			
					wearIt(s,Box);
					backpack(s,Box);						
					hasbankhere=true;
				}
			}
			else					// so it's a gold only box
			{							// anti exploit (see comment in openbank, it's symmetrical)
				//SndRemoveItem(s,Box);	// close the dual goldbankbox
				// exploit fix withdrawn for now because it causes weird client crashes (Duke & LB, 31.7.03)
			}
		}
	}

	if (!hasbankhere)
	{
		sprintf(temp, "%s's items bank box.", pc->name);
		const P_ITEM pic = Items->SpawnItem(pc,1,temp,0,0x09AB,0,0);
		if (!pic) return;
		pic->SetOwnSerial(pc->serial);
		pic->setNotMovable();
		pic->morex=1;
		pic->morey=0;//this's a all-items bank
		pic->morez=Player->region;//let's store the region
		pic->setType(ITY_Container);
		pc->Wear(pic,0x1D);
		wearIt(s,pic);
		backpack(s,pic);
	}
}

void bankcheck(cChar* Banker, cChar* Player, int amount, UOXSOCKET s)
{
	int gold;
	gold = Player->CountBankGold();
	if (amount < 5000 || amount > 1000000)
	{
		sprintf(temp, "%s you can only get checks worth 5000gp to 1000000gp.", Player->name);
		npctalkall(Banker, temp, 1);
	}
	else
	{
		if (gold < amount)
		{
			npctalkall(Banker,"Sorry you don't have enough gold in your bank account for that!",1);	
		}
		else
		{
			const P_ITEM pi = Items->SpawnItem(Player, 1, "a check", 0, 0x14F0, 0, 0); // bank check
			if (pi != NULL)
			pi->setType(ITY_BankDeed);
			pi->setId(0x14F0);
			pi->setColor(0x0099);
			pi->newbie();
			pi->morex = amount;
			Player->deleteBankGold(amount);
			P_ITEM Bankbox = Player->GetBankBox();
			Bankbox->AddItem(pi);
			statwindow(s, Player);
			sprintf(temp, "%s your check has been placed in your bankbox, it is worth %i.", Player->name, amount);
			npctalk(s, Banker, temp, 1);
		}
	}
}

void bankwithdrawal(cChar* Banker, cChar* pc, int amount, UOXSOCKET s)
{
	int bgold;
	P_ITEM gold;
	bgold = pc->CountBankGold();
	if (bgold < amount)
	{
		npctalkall(Banker,"Sorry you don't have enough gold in your bank account for that!",1);
	}
	else
	{
		pc->deleteBankGold(amount);
		gold = Items->SpawnItem(pc, amount, "gold coin", 1, 0x0EED, 0, 1);
		soundeffect(pc, 0x0037);
		npctalkall(Banker,"There you go sir!",1);
	}
}

PC_ITEM GetOutmostCont(PC_ITEM pItem, short rec)
{
	if ( rec<0		// too many recursions
		|| !pItem							// bad parm
		|| pItem->IsWorn()					// a character
		|| pItem->isInWorld() )				// in the world
		return pItem;
	P_ITEM pOut=FindItemBySerial(pItem->contserial);	// up one level
	if (!pOut)
	{
		//LogErrorVar("container of item %i not found",pItem->serial);
		//LogErrorVar("ID of that item is %x",pItem->id());
	}
	return GetOutmostCont(pOut,--rec);
}

P_CHAR GetPackOwner(PC_ITEM pItem, short rec)
{
	PC_ITEM pio=GetOutmostCont(pItem,--rec);
	if (!pio || pio->isInWorld())
		return NULL;
	return FindWearerOf(pio);
}

void goldsfx(int s, int goldtotal)
{
	if (goldtotal==1) soundeffect(s, 0x00, 0x35);
	if ((goldtotal>1)&&(goldtotal<6)) soundeffect(s, 0x00, 0x36);
	else soundeffect(s, 0x00, 0x37);
	return;
}

void addgold(int s, P_CHAR Player, int totgold)
{
	Items->SpawnItem(s,Player,totgold,"#",1,0x0EED,0,1,1);
}

int calcValue(P_ITEM pi, int value)
{
	if (!pi) return value;
	int mod=10;

	if (pi->isType(ITY_Potion))
	{
		if (pi->morex>500) mod=mod+1;
		if (pi->morex>900) mod=mod+1;
		if (pi->morex>1000) mod=mod+1;
		if (pi->morez>1) mod=mod+(3*(pi->morez-1));
		value=(value*mod)/10;
	}

	// Lines added for Rank System by Magius(CHE)
	if (pi->rank>0 && pi->rank<10 && SrvParms->rank_system==1)
	{
		value=(int) (pi->rank*value)/10;
	}
	if (value<1) value=1;

	if (pi->rndvaluerate<0) pi->rndvaluerate=0;
	if (pi->rndvaluerate!=0 && SrvParms->trade_system==1)
	{
		value+=(int) (value*pi->rndvaluerate)/1000;
	}
	if (value<1) value=1;
	// end addon

	return value;
}

int calcGoodValue(int actreg, P_ITEM pi, int value,int goodtype)
{ // Function Created by Magius(CHE) for trade System
	if (!pi) return value;
	int regvalue=0;
	int x;
	int good=pi->good;

	if (good<=-1 || good >255 || actreg<=-1 || actreg>255) return value;

	if (goodtype==1) regvalue=region[actreg].goodsell[pi->good]; // Vendor SELL
	if (goodtype==0) regvalue=region[actreg].goodbuy[pi->good]; // Vendor BUY

	x=(int) (value*abs(regvalue))/1000;

	if (regvalue<0)	value-=x;
	else value+=x;

	if (value<=0) value=1; // Added by Magius(CHE) (2)

	return value;
}

void StoreItemRandomValue(PC_ITEM pi,int tmpreg)
{ // Function Created by Magius(CHE) for trade System
	if (!pi) return;	// on error return

	if (pi->good<0) return;
	if (tmpreg<0)
	{
		PC_ITEM pio=GetOutmostCont(pi);
		if (!pio) return;
		if (pio->isInWorld())
			tmpreg=calcRegionFromXY(pio->pos);
		else
		{
			P_CHAR pc=FindWearerOf(pio);
			if (!pc) return;
			tmpreg=calcRegionFromXY(pc->pos);
		}
	}

	if (tmpreg<0 || tmpreg>255 || pi->good<0 || pi->good>255) return;

	int min=region[tmpreg].goodrnd1[pi->good];
	int max=region[tmpreg].goodrnd2[pi->good];

	if (max!=0 || min!=0)
	{
		pi->unconst()->rndvaluerate=(int) RandomNum(min,max);
	}
}

void loadmetagm() // LORD BINARY
{
	char sect[512];
	int i,k,mode,n;
	unsigned int const plus=1;
	int pm,ss,y;

	openscript("metagm.scp");

	for (i=0;i<256;i++)
	{
		for (int a=0;a<7;a++) metagm[i][a]=0;
	}

	i=-1;k=0;
	do
	{
		i++;
		sprintf(sect, "COMMAND_CLEARANCE %i", i);
		if (i_scripts[metagm_script]->find(sect))
		{
			k++;n=0;mode=-1;pm=-1;
			int loopexit=0;
			do
			{
				read2();
				if (script1[0]!='}')
				{
					if (!strcmp("MODE+",script1)) { mode=1;n=1;pm=1;} // plus mode
					if (!strcmp("MODE-",script1)) { mode=2;n=1;pm=0;}	// minus mode

					if (pm>-1 && n==0) // only check for commnad words AFTER the mode token
					{
						ss=0; y=-1;int loopexit=0;
						while((command_table[ss].cmd_name)&&(y==-1)&& (++loopexit < MAXLOOPS) ) // search for the command
						{
							if (!strcmp(command_table[ss].cmd_name, script1)) y=ss;
							ss++;
						}

						if (y==-1) // not found ?
						{
							Console.send("warning: found unknown command %s in meta gm script\n",script1);
							Console.send("press any key to continue. . .\n");//AntiChrist-so we know if something's wrong
							//getchar();//AntiChrist-so we know if something's wrong
						} else // found it!
						{
							if (command_table[y].cmd_priv_m!=255)
							{
								           metagm[i][command_table[y].cmd_priv_m] =  (metagm[i][command_table[y].cmd_priv_m]) | (plus << (command_table[y].cmd_priv_b) ) ;
								if (pm==0) metagm[i][command_table[y].cmd_priv_m] = ~(metagm[i][command_table[y].cmd_priv_m]);
							}
						}
					}
					n=0;
				}
			}
			while ( (script1[0]!='}') && (++loopexit < MAXLOOPS) );

			if (mode==-1)
			{
				error=1;
				keeprun=0;
				Console.send("\n Meta-Gm script parsing error, mode keyword missing, section# %i - closing LoneWolf\n",i);
			}

			if (pm==0)
			{
				for (int aa=0;aa<7;aa++) if (metagm[i][aa]==0) metagm[i][aa]=0xFFFFFFFF;
			
			}
		}

	} while (i<255);

	closescript();
	Console.send("Meta Gm script loaded... %i priv3 shortcut/s\n",k-3);
}

void LoadMapScript(void)
{


}

void dosocketmidi(int s)
{
	Script *pScp=i_scripts[regions_script];
	if (!pScp->Open()) return;

	char sect[512];
	P_CHAR Player = MAKE_CHARREF_LR(currchar(s));

	if (Player->war)
	{
		strcpy(sect, "MIDILIST COMBAT");
	}
	else
	{
		sprintf(sect, "MIDILIST %i", region[Player->region].midilist);
	}
	if (region[Player->region].midilist!=0 && !pScp->find(sect))
	{
		pScp->Close();
		return;
	}

	char midiarray[50];
	int i=0;
	int loopexit=0;
	do
	{
		pScp->NextLineSplitted();
		if (script1[0]!='}')
		{
			if (!strcmp("MIDI",script1))
			{
				midiarray[i]=str2num(script2);
				i++;
			}
		}
	}
	while ((script1[0]!='}') && (++loopexit < MAXLOOPS) );
	pScp->Close();
	if (i!=0)
	{
		i=rand()%(i);
		playmidi(s, 0, midiarray[i]);
	}
}

// Dupois - added to do easy item sound effects based on an
//			items id1 and id2 fields in struct items. Then just define the CASE statement
//			with the proper sound function to play for a certain item as shown.
//			Use the DEFAULT case for ranges of items (like all ingots make the same thump).
// Sounds - coins dropping (all the same no matter what amount because all id's equal 0x0EED
//			gems dropping (two type broke them in half to make some sound different then others)
// NOTE	  - I wasn't sure what the different soundeffect() func's did so I just used
//			soundeffect() and it seemed to work fairly well.
// Added Oct 09, 1998
// new interface, Duke, 25.3.2001
// removed ingots, Duke, 30.11.02
void itemsfx(UOXSOCKET s, short id)
{
	if (id==0x0EED)
	{
		goldsfx(s, 2);
	}
	else if (id>=0x0F0F && id<=0x0F20 )	// Any gem stone (typically smaller)
	{
		soundeffect(s, 0x00, 0x32);
	}
	else if (id>=0x0F21 && id<=0x0F30)	// Any gem stone (typically larger)
	{
		soundeffect(s, 0x00, 0x34);
	}
	soundeffect(s, 0x00, 0x42);					// play default item move sfx // 00 48
}

void Karma(P_CHAR pc, P_CHAR Victim, int nKarma)
{	// nEffect = 1 positive karma effect
	int nCurKarma=0, nChange=0, nEffect=0;

	nCurKarma = pc->karma;

	if ((nCurKarma>10000)||(nCurKarma<-10000))
		if (nCurKarma>10000)
			pc->karma=10000;
		else
			pc->karma=-10000;

	if (nCurKarma<nKarma && nKarma>0)
	{
		nChange=((nKarma-nCurKarma)/75);
		pc->karma=(nCurKarma+nChange);
		nEffect=1;
	}

	//AntiChrist - modified to avoid crashes
	//the nKilledID==-1 check and the chars[nKilledID] check were in the same line
	//That may cause some crash with some compilator caus there's no a defined
	//order in executing these if checks
	if (nCurKarma>nKarma && !Victim)
	{
		nChange=((nCurKarma-nKarma)/50);
		pc->karma=(nCurKarma-nChange);
		nEffect=0;
	}
	else if (nCurKarma>nKarma && Victim->karma>0)
	{
		nChange=((nCurKarma-nKarma)/50);
		pc->karma=(nCurKarma-nChange);
		nEffect=0;
	}

	if (nChange==0 || pc->npc==1)
		return;
	UOXSOCKET s = pc->calcSocket();
	if (nChange<=25)
	{
		if (nEffect)
			sysmessage(s,"You have gained a little karma.");
		else
			sysmessage(s,"You have lost a little karma.");
		return;
	}
	if (nChange<=75)
	{
		if (nEffect)
			sysmessage(s,"You have gained some karma.");
		else
			sysmessage(s,"You have lost some karma.");
		return;
	}
	if (nChange<=100)
	{
		if (nEffect)
			sysmessage(s,"You have gained alot of karma.");
		else
			sysmessage(s,"You have lost alot of karma.");
		return;
	}
	if (nChange>100)
	{
		if (nEffect)
			sysmessage(s,"You have gained a huge amount of karma.");
		else
			sysmessage(s,"You have lost a huge amount of karma.");
		return;
	}
}

//added by Genesis 11-8-98
void Fame(P_CHAR pc, int nFame)
{
	if (!pc) return;

	if (pc->npc) //NPCs don't gain fame.
		return;

	int nCurFame= pc->fame;
	int nChange=0, nEffect=0;
	if (nCurFame>nFame) // if player fame greater abort function
	{
		if (nCurFame>10000)
			pc->fame=10000;
		return;
	}
	if (nCurFame<nFame)
	{
		nChange=(nFame-nCurFame)/75;
		pc->fame=(nCurFame+nChange);
		nEffect=1;
	}
	if (pc->dead)
	{
		if (nCurFame<=0)
			pc->fame=0;
		else
		{
			nChange=(nCurFame-0)/25;
			pc->fame=(nCurFame-nChange);
		}
		pc->deaths++;
		nEffect=0;
	}
	if (nChange==0)
		return;
	UOXSOCKET s = pc->calcSocket();
	if (nChange<=25)
	{
		if (nEffect)
			sysmessage(s,"You have gained a little fame.");
		else
			sysmessage(s,"You have lost a little fame.");
		return;
	}
	if (nChange<=75)
	{
		if (nEffect)
			sysmessage(s,"You have gained some fame.");
		else
			sysmessage(s,"You have lost some fame.");
		return;
	}
	if (nChange<=100)
	{
		if (nEffect)
			sysmessage(s,"You have gained alot of fame.");
		else
			sysmessage(s,"You have lost alot of fame.");
		return;
	}
	if (nChange>100)
	{
		if (nEffect)
			sysmessage(s,"You have gained a huge amount of fame.");
		else
			sysmessage(s,"You have lost a huge amount of fame.");
		return;
	}
}

//Purpose: Spawn all items in itemlist (pointed by item's morex), for item type 103 (enarmy objet).
void enlist(int s, int listnum) 
{
	char sect[50];
	Script * Scp=i_scripts[items_script];
	sprintf(sect, "ITEMLIST %i", listnum);
	Scp=Scp->Select(sect,custom_item_script);
	if (!Scp) return;
	int loopexit=0,item_num;
	scpMark pos;
	Scp->NextLine();
	while( !Scp->EndSect() && (++loopexit < MAXLOOPS) )
	{
		item_num=str2num(script1);
		pos=Scp->Suspend();
		P_ITEM pi=Items->SpawnItemBackpack2(s, item_num);
		if (!pi) return;
		Scp->Resume(pos);
		Scp->NextLine();
	}
	Scp->Close();
}


void criminal(P_CHAR pc)//Repsys ....Ripper
{
	if (pc == NULL)	return;

	unsigned char r = calcRegionFromXY(pc->pos);
	if (region[r].isArena()) return;	// cant get criminal inside an arena

	if ((!pc->npc)&&(!pc->isCriminal() || !pc->isMurderer()))
	{//Not an npc, not grey, not red
		
		pc->crimflag=(repsys.crimtime*MY_CLOCKS_PER_SEC)+uiCurrentTime;
		//printw(" Seeting Crimflag to %d \n",chars[c].crimflag) ;
		sysmessage(pc->calcSocket(),"You are now a criminal!");
		setcharflag(pc);
		if (pc->inGuardedArea() && SrvParms->guardsactive)//guarded
			Npcs->SpawnGuard(pc, pc, pc->pos);
	}
}

void setcharflag(P_CHAR pc)// repsys ...Ripper
{
	if (!pc->npc)
	{
		if (pc->kills >= (unsigned) repsys.maxkills)
		{
			pc->setMurderer();
		}	
		else if (pc->crimflag==-1 || pc->crimflag == 0)
		{
			pc->setInnocent();
		}
		else if (pc->crimflag>0)
		{
			pc->setCriminal();
		}		
		else
		{
			pc->setCriminal();
		}
	}
	else
	{
		switch (pc->npcaitype)
		{
			case 2: // bad npcs
			case 3: // bad healers
			case 50: // EV & BS
				pc->setMurderer();
				break;
			case 1: // good healers
			case 4: // teleport guard
			case 5: // beggars
			case 6: // chaos guard
			case 7: // order guard
			case 8: // banker
			case 9: // town guard
			case 12: // decorative blue npcs
			case 17: // player vendor
			case 30: // ?
			case 40: // ? old teleport guards?
				pc->setInnocent();
				break;
			default:
				if (pc->isHuman())
				{
					pc->setInnocent();
					return;
				}
				if (SrvParms->animals_guarded == 1 && pc->npcaitype == 0 && !pc->tamed)
				{
					if (pc->inGuardedArea())	// in a guarded region, with guarded animals, animals == blue
						pc->setInnocent();
					else				// if the region's not guarded, they're gray
						pc->setCriminal();
				}
				else if (pc->ownserial>-1 && pc->tamed)
				{
					P_CHAR Owner = FindCharBySerial(pc->ownserial);
					if (Owner)
					{
						pc->flag = Owner->flag;
					}
				}
				else 
					pc->setCriminal();
				break;
		}
	}
}


//Written by AntiChrist - 4/11/1999
//
//This is very useful for bandwidth i think....i replaced all the
//stupid loops in the entire code sending a item to ALL the
//sockets...without checking if the socket's character was in
//range.....very baaad!
//
//I also added the inpack check and the worned check....
//
void RefreshItem(P_ITEM pi)//Send this item to all online people in range
{//check if item is in a pack or on the ground, then use different methods
	if (!pi) return;


	if (pi->isInWorld())	//first check: let's check if it's on the ground....
	{//yeah, it's on ground!
		int a;
		for(a=0;a<(int)now;a++)//send this item to all the sockets in range
		{
			if (perm[a] && iteminrange(a,pi,VISRANGE))
				senditem(a,pi);
		}
		return;
	}

	//if not, let's check if it's on a char or in a pack

	if (pi->IsWorn())//container is a player...it means it's equipped on a character!
	{
		unsigned char wearitem[15]={0,};

		LongToCharPtr(pi->serial,&wearitem[1]);
		ShortToCharPtr(pi->id(),&wearitem[5]);

		wearitem[8]=pi->layer;

		LongToCharPtr(pi->contserial,&wearitem[9]);
		ShortToCharPtr(pi->color(),&wearitem[13]);
		P_CHAR PackOwner=FindWearerOf(pi);
		if (PackOwner)
		{
			P_CHAR pc;
			AllPlayersInRangeIterator api(PackOwner);
			for (pc = api.First(); pc != NULL; pc = api.Next())
			{
				XStartPkt(0x2e,15);
				XAddPkt(&wearitem[0+1], 15-1);
				XFinishPkt(api.getSocket());
			}
		}
	}
	else//container is an item...it means we have to use sendbpitem()!!
	{
		unsigned int aa;
		for (aa=0;aa<now;aa++)		//send this item to all the sockets in range
		{
			if (perm[aa])
				sendbpitem(aa, pi);	//NOTE: there's already the inrange check
									//in the sendbpitem() function, so it's unuseful
									//to do a double check!!
		}
	}
}

// if we can find new effects they can be added here and will be active 
// for 'go 'goiter 'goplace 'whilst and 'tell for gm's and counselors 

////////////////////////////////// 
// Function for the different gm movement effects 
// 0 = none 
// 1 = flamestrike 
// 2 - 6 = different sparkles 
// Aldur
// 
// 

void doGmMoveEff(UOXSOCKET s)
{ 
	if (s == -1) // Just to make sure ;)
		return;
	
	P_CHAR Player = MAKE_CHARREF_LR(currchar(s));
	if (!Player->IsPermaHidden())
	{ 
		cCoord effPos(Player->pos);
		effPos.Add(1,1,10);
		switch (Player->gmMoveEff)
		{
		case 1:
			// flamestrike 
			staticeffect3(effPos, 0x37, 0x09, 0x09, 0x19, 0); 
			soundeffect(Player, 0x0208); 
			break;
			// sparklies (fireworks wand style) 
		case 2:	staticeffect3(effPos, 0x37, 0x3A, 0x09, 0x19, 0);	break;
		case 3:	staticeffect3(effPos, 0x37, 0x4A, 0x09, 0x19, 0);	break;
		case 4:	staticeffect3(effPos, 0x37, 0x5A, 0x09, 0x19, 0);	break;
		case 5:	staticeffect3(effPos, 0x37, 0x6A, 0x09, 0x19, 0);	break;
		case 6:	staticeffect3(effPos, 0x37, 0x7A, 0x09, 0x19, 0);	break;
		}
	}
}

// processing external command packet from client !
void ExternalCommand(UOXSOCKET s, P_CHAR Player)
{
	int loopexit=0,i;

	switch (buffer[s][3])
	{
	case 0xC7: // Action
		if (Player->onhorse) // Ripper
			return; // Ripper
		
		if (!strcmp((char*)&buffer[s][4],"bow")) Player->DoAction(0x20);
		if (!strcmp((char*)&buffer[s][4],"salute")) Player->DoAction(0x21);
		return;
	case 0x24: // Skill
		i=4;	   
		while ( (buffer[s][i]!=' ') && (++loopexit < MAXLOOPS) ) i++;
		buffer[s][i]=0;
		SkillManager->SkillUse(s, str2num((char*)&buffer[s][4]));
		return;
		
	case 0x27:
	case 0x56:	// Spell
	{
		P_ITEM Book = Magic->GetSpellBook(Player,SBT_REGULAR);

		int spn = buffer[s][4]-0x30;
		if (buffer[s][5]>0x20) spn = (spn*10)+(buffer[s][5]-0x30);
 
		cSpell* spell = Magic->GetSpell(spn);
		if (!spell) return;

		if (spell->IsInBook(Book))
		{
			if (Player->IsFrozen()) // REAL cant cast while frozen bugfix, lord binary
			{
				if (Player->IsCasting()) sysmessage(s, "You are already casting a spell.");
				else sysmessage(s, "You cannot cast spells while frozen.");
			}
			else 
			{ 
//				currentSpellType[s]=0;
				Magic->BeginCasting(Player, spn, STY_BOOK); 
			}
		}
		else
			sysmessage(s, "You don't have that spell.");
		break;
	}
	case 0x43:	// Open spell book
		if (buffer[s][2]=='\x05')  // Open spell book
		{
			Magic->OpenSpellBook(Player,NULL,SBT_REGULAR);
			return;
		} 
		if (buffer[s][2]=='\x06')  // Open spell book
		{
			switch (buffer[s][4])
			{
			case '\x31':
				Magic->OpenSpellBook(Player,NULL,SBT_REGULAR);
				break;
			case '\x32':
				Magic->OpenSpellBook(Player,NULL,SBT_NECRO);
				break;
			case '\x33':
				Magic->OpenSpellBook(Player,NULL,SBT_PALADIN);
				break;
			}
			return;
		}
		break;
	case 0x58:	// Door macro
		if (buffer[s][2]=='\x05')
		{
			// search for closest door and open it ...										
			return;
		}
		break;
	}
}

/// client version controll system.
/// true: client ok, false: client disallowed
bool CVCS(UOXSOCKET s)
{
   string cpps;
   vector<string>::const_iterator viter;

   strcpy(temp3,(char*) &buffer[s][3]); // copy client version data	

   clientData[s].clientVersion=temp3; // copy it to client data structre
   string cv=clientData[s].clientVersion;

   if (ClientVersion->get3DNess(cv)) clientDimension[s]=3; else  clientDimension[s] = 2;

   sprintf(temp,"You are using a %iD client, version %s", clientDimension[s], temp3);
   sysmessage(s,temp);

   if (ClientVersion->getClientNr(cv)==-1) return false; // client not found in database, dont allow it
   if (SrvParms->AOS_Mode && !ClientVersion->isAOSClient(cv)) return false;

   viter = find(clientsAllowed.begin(), clientsAllowed.end(), "ALL");
   if ( viter != clientsAllowed.end() ) return true; // ALL mode found/activated -> quit						
				
   viter = find(clientsAllowed.begin(), clientsAllowed.end(), "SERVER_DEFAULT");
   if ( viter != clientsAllowed.end() )  // server_default mode ?
   {
	  if ( strcmp( temp3, wp_version.clientsupportedstring.c_str() ) ) // check if client version matches
	  {         
		 return false;
	  }
	  return true;
   }
   else
   {
	  cpps.assign(temp3);				
	  viter = find(clientsAllowed.begin(), clientsAllowed.end(), cpps);
	  if (viter != clientsAllowed.end() ) return true; 
	  else 
	  {
		Console.send("Client %i disconnected by Client Version Control System\n", s);	    
	    return false;
	  }
	}

   return true;
}

void doNPCPopupMenueAction(UOXSOCKET s)
{
	unsigned int ID = LongFromCharPtr(&buffer[s][5]);
	unsigned char menuChosen=buffer[s][10];
	
	switch(menuChosen)
	{
	case 0: 								  
		dbl_click_character(s, ID);
		break;
		
	case 1: 
		Targ->BuyShop(s, FindCharBySerial(ID));
		break;
		
	case 2: 
		sellstuff(s, FindCharBySerial(ID));
		break;								 
	}						  
}

void DeleteItemOnLayer(P_CHAR pc, int ly)
{
	int ci=0, loopexit=0;
	P_ITEM pi;
	while ( (( pi=pc->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
	{
		if (pi->layer==ly)
		{
			Items->DeleItem(pi);
		}
	}
}

//////////////////////////////////////////////////////
// type 0: output to console, socket+file ignored
// type 1: output to client, socket has to be valid socket-number, file ignored
// type 2: output to remote adminstrtion, socket has to be valid,  file ignored
// type 3: output to htmlfile(long),   file has to valid, socket ignored
// type 4: output to htmlfile (short), file has to valid, socket ignored
//////////////////////////////////////////////////////

void pdump(UOXSOCKET s, char type, FILE* file)
{
	if (type>4) return;

	char tmp[1024];

	string tmpLine("");
	string term;

	if      (type==0 || type==1 || type==2) term="\n";
	else if (type==3 || type==4)            term="<br>";

	unsigned long int totTime;
	totTime = getNormalizedTime()-EmuStartTime;						

	if (type==0 || type==3)
	{
	  if (firstcycleok)
	  {
	    tmpLine+=term; tmpLine+="Performance Dump:"; tmpLine+=term; tmpLine+=term;		 

	    sprintf(tmp,"Characters: %i/%i (Dynamic)", Npcs->GetCount() , Npcs->GetCapacity());
		tmpLine+=tmp; tmpLine+=term;
        
        sprintf(tmp,"Items: %i/%i (Dynamic)" , Items->GetCount() , Items->GetCapacity());
		tmpLine+=tmp; tmpLine+=term; tmpLine+=term;		
	  
	    sprintf(tmp,"Total loops:%lu", TotalLoopCount);
		tmpLine+=tmp; tmpLine+=term; 
	    
	    sprintf(tmp,"Total CPU time:%lums", TotalCpuTime);
		tmpLine+=tmp; tmpLine+=term; 
	    
	    if (totTime!=0) 
		{ 
		   sprintf(tmp,"CPU usage:%.02f%%", (float) ( (float) TotalCpuTime / (float) totTime ) * 100.0);
		   tmpLine+=tmp; tmpLine+=term; 
		  
		}

	    sprintf(tmp,"Measurementcycle triggertime:%lu ms", MinMeasureCycleTime);
		tmpLine+=tmp; tmpLine+=term; tmpLine+=term; 
	 				 
	    tmpLine+="Last measurementcycle stats:";

        sprintf(tmp, "Time:%lu ms", LastMeasureCycleTime);
		tmpLine+=tmp; tmpLine+=term;
	 
	    sprintf(tmp,"Loopcount: %lu", LastMeasureCycleLoopCount);
		tmpLine+=tmp; tmpLine+=term;
	  
	    if (LastMeasureCycleTime!=0) 
		{
		   sprintf(tmp, "Simulationcycles per sec: %.02f", (float) ( (float)LastMeasureCycleLoopCount/(float)LastMeasureCycleTime) * 1000.0 );
		   tmpLine+=tmp; tmpLine+=term; tmpLine+=term; 		   
		}

	    tmpLine+="Average stats:"; tmpLine+=term;
	  
	    sprintf(tmp, "Measurement cycles:%i",TotalMC);
		tmpLine+=tmp; tmpLine+=term;
	   
	    sprintf(tmp, "Loopcount:%.02f", (float) ( (float) TotalLoopCountII / (float) TotalMC ) );
		tmpLine+=tmp; tmpLine+=term;
	 
	    sprintf(tmp, "Simulationcycles per sec: %.02f", (float) ((float) TotalLoopCountII / (float)TotalCpuTime)*1000.0);
		tmpLine+=tmp; tmpLine+=term; tmpLine+=term;
	    
	  } else
	  {
	    tmpLine += "First measurementcycle not completed, plz retry later"; tmpLine+=term;
	 
	  }
	}

	if (type==1)
	{
		if (firstcycleok)
		{
		   if (LastMeasureCycleTime!=0) 
		   {
		      sprintf(tmp, "Last Simulationcycles per sec: %.02f", (float) ( (float)LastMeasureCycleLoopCount/(float)LastMeasureCycleTime) * 1000.0 );
		      sysmessage(s, tmp);		   
		   }
		   if (TotalCpuTime!=0)
		   {
			  sprintf(tmp, "Average Simulationcycles per sec: %.02f", (float) ((float) TotalLoopCountII / (float)TotalCpuTime)*1000.0);
			  sysmessage(s, tmp);
		   }
		}
		else
		{
		  sprintf(tmp, "First measurementcycle not completed, plz retry later");
		}
		return;
	}

	switch(type)
	{
	   case 0:
		   Console.send(tmpLine.c_str());
		   break;
	   case 2:
		   break;
	   case 3:
		   fprintf(file,tmpLine.c_str());
		   break;
	   case 4:
		   if (TotalCpuTime!=0)
		   {
			   if (firstcycleok)
			   {
		          sprintf(tmp, "%.02f", (float) ((float) TotalLoopCountII / (float)TotalCpuTime)*1000.0);
		          fprintf(file,tmp);
			   }
		   }
		   break;
	}
}


void makehousesUnmovable(void) // EUO exploit patch (makes houses not movable, new houses already arnt)
{
	//////////////////////////////////////
	///////// secure houses /////////////
	/////////////////////////////////////

	PC_ITEM pi;
	AllItemsIterator aii;
	int secs=0;
	for (aii.Begin(); (pi = aii.GetData()) != aii.End(); aii++)
	{
		if (ishouse(pi->id()))
		{
			if (!pi->isNotMovable())
			{
				pi->unconst()->setNotMovable();
				secs++;
			}
		}		
	}

	if (secs>0) 
	{ 
		char tmp[512];
		sprintf(tmp,"updated %i houses to be unmoveable\n",secs);
	    Console.send(tmp);
	}

	/////////////////////////////////////////////////////////////////////////
	/// make everything players wears (in particlar BP+hair) EUO secure !!!
	////////////////////////////////////////////////////////////////////////

	int cnt1=0; int cnt2=0;
	for (aii.Begin(); (pi = aii.GetData()) != aii.End(); aii++)
	{
		P_CHAR pc=NULL;
	    PC_ITEM pio=GetOutmostCont(pi);
	    if (pio) pc=FindWearerOf(pio);
		if (pc)
		{
			switch(pi->layer)
			{
			   case 0: break;
			   case 0x0b: // hair
			   case 0x10: // beard
			   case 0x15: // backpack
			   case 0x19: // mount
			   case 0x1a: // shoppacks
			   case 0x1b:
			   case 0x1c:
			   case 0x1d: // bankbox

				 if (!pi->isNotMovable()) // hair+bp should never be movable !!!
				 {
				   pi->unconst()->setNotMovable();
				   cnt1++;
				 }				 			 
				 break;
			} // end switch
		}
					
	}
	
	if ((cnt1+cnt2)>0)
	{
	  char tmp[512];
	  sprintf(tmp,"%i worn items made unmovable (%i+%i)\n",cnt1+cnt2,cnt1,cnt2);
	  Console.send(tmp);
	}
}

void mpf_helper(FILE* f, FILE* f2, P_CHAR pc, int online, bool dobackup)
{
  int acct=pc->account;
  int serial=pc->serial;
  int type=0; if (pc->isCounselor()) type=2; if (pc->isGM()) type=1; 
  char sex=1; if (pc->id()==0x191) sex=0; // female = 0, anything else = 1 = male
  int guildnumber=pc->guildnumber;
  int str=pc->effStr();
  int dex=pc->effDex();
  int inte=pc->effInt();
  int hp=pc->Hp();
  int stam=pc->Stm();
  int mana=pc->Mana();
  int karma=pc->karma;
  int fame=pc->fame;
  int kills=pc->kills;
  int deaths=pc->deaths;
  int ghost=pc->dead;
  int war=pc->war;
  int hidden=pc->isHidden();
  int invul=pc->isInvul();
  int onhorse=pc->onhorse;
  int hunger=pc->Hunger();
  int flag=0; if (pc->isCriminal()) flag=1; if (pc->isMurderer()) flag=2;
  int gold=pc->CountGold();
  int bankgold=pc->CountBankGold();

  unsigned short x=pc->pos.x;
  unsigned short y=pc->pos.y;
  signed char z=pc->pos.z;
  char map=pc->pos.map;
  char direction=pc->dir;

  char tmp[6666];

  sprintf(tmp,"%s\n%i %i %i %i %i %i\n%i %i %i %i %i %i %i %i %i %i %i %i %i %i %i %i %i %i %i\n",pc->name, online, acct, serial, type, sex, guildnumber, str,dex,inte, hp, stam,mana, karma,fame, kills,deaths, ghost, war, hidden, invul, onhorse, hunger, flag, gold, bankgold);
  fprintf(f,"%s",tmp); 
  if (dobackup) fprintf(f2,"%s",tmp);

  int a;
  int m=SkillManager->getAllSkillsValue();
  for (a=0;a<m; a++) 
  {
	  if (pc->getRealSkill(a)!=0)
	  {
	    sprintf(tmp,"%i %i %i  ", a, pc->getRealSkill(a), pc->Skill(a));
		fprintf(f,"%s",tmp); 
		if (dobackup) fprintf(f2,"%s",tmp);
	  }
  }

  sprintf(tmp, "\n0\n%i %i %i %i %i\n\n", x,y,z,map, direction);
  fprintf(f,"%s",tmp); 
  if (dobackup) fprintf(f2,"%s",tmp);
		  		   
}


void makePlayerPosFile(void)
{
  static int calls=0;

  calls++;
  FILE *pposfile=NULL;
  FILE *pposfilebackup=NULL;
  bool dobackup=false;
  pposfile = fopen(SrvParms->playerpositionsfile, "w+");

  if (SrvParms->playerposfilebackupratio>0) 
  {	
     // create unique backup filename
	 if ((calls % SrvParms->playerposfilebackupratio)==0)
	 {
	   char fn[300];
	   time_t currentTime; tm *ct;
	   time(&currentTime); ct = localtime(&currentTime); 
	   if (ct->tm_year>99) ct->tm_year-=100; // y2k patch
	   sprintf(fn, "%s_%02d%02d%02d%02d%02d%02d_%i", SrvParms->playerpositionsfile, ct->tm_mday, ct->tm_mon+1, ct->tm_year, ct->tm_hour, ct->tm_min, ct->tm_sec, calls);
	   pposfilebackup = fopen(fn, "w+");	 

	   if (pposfilebackup==NULL)
	   {
	     char tmp[512];
	     sprintf(tmp,"PlayerPosition backup file %s couldn't be created\n",fn);
	     LogWarning(tmp);
	   } else dobackup=true;
	     
	 }
  }

  if (pposfile==NULL)
  {
	  char tmp[512];
	  sprintf(tmp,"PlayerPosition file %s couldn't be created\n",SrvParms->playerpositionsfile );
	  LogWarning(tmp);
	  return;
  }

  int players=0;
  int online=0;
  P_CHAR pc;
  AllCharsIterator aci;
  for (aci.Begin(); (pc=aci.GetData()) != NULL; aci++)
  {
	  if (pc->account>=0 && !pc->npc) 
	  {
		  players++;		  
		  if (pc->InGame()) online++;		  
	  }
  }

   // prolog, date, time
  time_t currentTime;
  time(&currentTime); 

  char tmp[3024];
  sprintf(tmp, "\nLW Player position file version 4.0\n\nLayout:\nName\nOnline Account# Serial# Playertype Sex Guildnumber\nStr Dex Int HP Stamina Mana Karma Fame Kills Deaths Ghost War Hidden Invul OnHorse Hunger Flag Gold Bankgold\nSkillvector\n0\nX Y Z Map# Direction\n\nUpdate#: %i\nDate: %sTotal Players: %i\nOnline: %i\n\nDATA START\n\n", calls, ctime(&currentTime), players, online);
  fprintf(pposfile, "%s", tmp);
  
  if (dobackup) fprintf(pposfilebackup, "%s",tmp);

  for (aci.Begin(); (pc=aci.GetData()) != NULL; aci++) // online players first
  {
	  char online=pc->InGame();
	  if (pc->account>=0 && !pc->npc &&online) 
	  {
		  mpf_helper(pposfile, pposfilebackup, pc, online, dobackup);
	
	  }
  }

  for (aci.Begin(); (pc=aci.GetData()) != NULL; aci++) // then offliners
  {
	  char online=pc->InGame();
	  if (pc->account>=0 && !pc->npc &&!online) 
	  {
		 mpf_helper(pposfile,  pposfilebackup, pc, online, dobackup);		 
	  }
  }

  sprintf(tmp,"\nDATA END\n\n");
  fprintf(pposfile, "%s", tmp);  
  if (dobackup) fprintf(pposfilebackup, "%s",tmp);

  fflush(pposfile);
  fclose(pposfile);

  if (dobackup) 
  {
    fflush(pposfilebackup);
    fclose(pposfilebackup);
  }

}

