/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

#include "SrvParms.h"

#include "mapstuff.h"
#include "worldmain.h"

#include "wolfpack.h"
#include "network.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "SrvParms.cpp"

////////////////////
// name:	cKeyWord, cSection etc.
// purpose:	handle the key=value stuff in server.scp
// history:	by Duke, Oct 2003
// remarks:	I'm still not content with the way the default values are handled, so do not *deploy* these classes !
//
enum eKeyWordTypes
{
	KWT_dec_uint = 1,	// decimal stored as an unsigned int
	KWT_dec_int,		// decimal stored as an int
	KWT_flo_flo,		// float
	KWT_str_str,		// a string ;)
	KWT_str_sp1			// special handler 1: for defaultfood
};

class cKeyWord
{
	char _key[50];
	eKeyWordTypes _type;
	void *_data;
public:
	cKeyWord(char* key, eKeyWordTypes type, unsigned int* data, const unsigned int def);
	cKeyWord(char* key, eKeyWordTypes type, int* data, const int def);
	cKeyWord(char* key, eKeyWordTypes type, float*data, const double def);
	cKeyWord(char* key, eKeyWordTypes type, char*data, const char* def);
	cKeyWord(char* key, eKeyWordTypes type, unsigned char* data, const unsigned char def);
	void Save(FILE* fp);
	bool Load(char* key, char* parm);
};

cKeyWord::cKeyWord(char* key, eKeyWordTypes type, unsigned int* data, const unsigned int def)
{
	strcpy(_key,key);
	_type=type;
	_data=data;
	switch(_type)
	{
	case KWT_dec_uint: *((unsigned int*)_data) = def; break;
	default:	LogErrorVar("invalid type of keyword <%i>",_type);
	}
}

cKeyWord::cKeyWord(char* key, eKeyWordTypes type, int* data, const int def)
{
	strcpy(_key,key);
	_type=type;
	_data=data;
	switch(_type)
	{
	case KWT_dec_int: *((int*)_data) = (int)def; break;
	default:	LogErrorVar("invalid type of keyword <%i>",_type);
	}
}

// float constants can't be converted to void*s, so we need another constructor
// update: forget it, results in 'ambigous call'
cKeyWord::cKeyWord(char* key, eKeyWordTypes type, float*data, const double def)
{
	strcpy(_key,key);
	_type=type;
	_data=data;
	switch(_type)
	{
	case KWT_flo_flo: *((float*)_data) = def; break;
	default:	LogErrorVar("invalid type of keyword <%i>",_type);
	}
}

cKeyWord::cKeyWord(char* key, eKeyWordTypes type, char* data, const char* def)
{
	strcpy(_key,key);
	_type=type;
	_data=data;
	switch(_type)
	{
	case KWT_str_str: strcpy((char*)_data, def); break;
	default:	LogErrorVar("invalid type of keyword <%i>",_type);
	}
}

cKeyWord::cKeyWord(char* key, eKeyWordTypes type, unsigned char* data, const unsigned char def)
{
	strcpy(_key,key);
	_type=type;
	_data=data;
	switch(_type)
	{
	case KWT_str_sp1: *data = def; break;
	default:	LogErrorVar("invalid type of keyword <%i>",_type);
	}
}

bool cKeyWord::Load(char* key, char* parm)
{
	if (strcmp(_key,key)) return false;	// not what we're looking for

	switch(_type)
	{
	case KWT_dec_uint:
	case KWT_dec_int: *((int*)_data) = str2num(parm);	break;
	case KWT_str_str: strcpy((char*)_data, parm);		break;
	case KWT_flo_flo: *((float*)_data) = atof(parm);	break;
	case KWT_str_sp1:
		*((unsigned char*)_data) = 0;
		if (strstr(parm,"BAKED"))		SrvParms->defaultfood |= FOOD_BAKED;
		if (strstr(parm,"CHEESE"))		SrvParms->defaultfood |= FOOD_CHEESE;
		if (strstr(parm,"FISH"))		SrvParms->defaultfood |= FOOD_FISH;
		if (strstr(parm,"FRUITS"))		SrvParms->defaultfood |= FOOD_FRUIT;
		if (strstr(parm,"MAGIC"))		SrvParms->defaultfood |= FOOD_MAGIC;
		if (strstr(parm,"MEAT"))		SrvParms->defaultfood |= FOOD_MEAT;
		if (strstr(parm,"RAW"))			SrvParms->defaultfood |= FOOD_RAW;
		if (strstr(parm,"VEGETABLES"))	SrvParms->defaultfood |= FOOD_VEGETABLE;
		break;
	default:	LogErrorVar("invalid type of keyword <%i>",_type);
	}
	return true;
}

void cKeyWord::Save(FILE* fp)
{
	switch(_type)
	{
	case KWT_dec_uint:
	case KWT_dec_int: fprintf(fp,"%s %i\n", _key, *((int*)_data));	break;
	case KWT_str_str: fprintf(fp,"%s %s\n", _key, (char*)_data);	break;
	case KWT_flo_flo: fprintf(fp,"%s %f\n", _key, *((float*)_data));	break;
	case KWT_str_sp1:
		{
		char tempfood[255] = "DEFAULTFOOD ";
		if ((SrvParms->defaultfood&FOOD_MEAT) == FOOD_MEAT) strcat(tempfood,"MEAT+");
		if ((SrvParms->defaultfood&FOOD_FISH) == FOOD_FISH) strcat(tempfood,"FISH+");
		if ((SrvParms->defaultfood&FOOD_CHEESE) == FOOD_CHEESE) strcat(tempfood,"CHEESE+");
		if ((SrvParms->defaultfood&FOOD_FRUIT) == FOOD_FRUIT) strcat(tempfood,"FRUITS+");
		if ((SrvParms->defaultfood&FOOD_VEGETABLE) == FOOD_VEGETABLE) strcat(tempfood,"VEGETABLES+");
		if ((SrvParms->defaultfood&FOOD_BAKED) == FOOD_BAKED) strcat(tempfood,"BAKED+");
		if ((SrvParms->defaultfood&FOOD_RAW) == FOOD_RAW) strcat(tempfood,"RAW+");
		if ((SrvParms->defaultfood&FOOD_MAGIC) == FOOD_MAGIC) strcat(tempfood,"MAGIC+");

		int len = strlen(tempfood);
		if (len>12) tempfood[len-1] = 0; //remove the last "+"
		else strcpy(tempfood,"DEFAULTFOOD MEAT+FISH+CHEESE+FRUITS+VEGETABLES+BAKED+RAW+MAGIC"); //eat everything by default
		fprintf(fp,"%s\n", tempfood);
		}
		break;
	default:	LogErrorVar("invalid type of keyword <%i>",_type);
	}
}

const int KPS=15;	// keywords per section

class cSection
{
protected:
	char _secname[50];
	cKeyWord* _keys[KPS];	// I know, an unnecessary limitation, will be changed later ;)
public:
	cSection(char* sname);
	void Add(cKeyWord* kw);
	bool Load(char* key);
	bool LoadKeywords(char* key, char* parm);
	void SaveKeywords(FILE* fp);
	void Save(FILE* fp);
};

cSection::cSection(char* sname)
{
	int i;
	for (i=0;i<KPS;i++) _keys[i]=NULL;
	strcpy(_secname,sname);
}

void cSection::Add(cKeyWord* kw)
{
	if (!kw) return;
	int i;
	for (i=0;i<KPS;i++)
	{
		if (_keys[i]==NULL)
		{
			_keys[i]=kw;
			break;
		}
	}
	if (i >= KPS)
		LogErrorVar("Too many keywords in section %s", _secname);
}

bool cSection::LoadKeywords(char* key, char* parm)
{
	int i;
	for (i=0;i<KPS;i++)
	{
		if (_keys[i]!=NULL)
		{
			if (_keys[i]->Load(key,parm))
				return true;
		}
	}
	return false;
}

bool cSection::Load(char* key)
{
	if (strcmp(_secname,key)) return false;	// not what we're looking for

	int loopexit=0;
	do
	{
		readw2();
		if (!LoadKeywords(script1,script2) && !(script1[0]=='}'))
			LogWarningVar("Invalid keyword <%s> in server script",script1);
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
	return true;
}

void cSection::SaveKeywords(FILE* fp)
{
	int i;
	for (i=0;i<KPS;i++)
	{
		if (_keys[i]!=NULL)
		{
			_keys[i]->Save(fp);
		}
	}
}

void cSection::Save(FILE* fp)
{
	fprintf(fp, "SECTION %s\n",_secname);
	fprintf(fp, "{\n");
	SaveKeywords(fp);
	fprintf(fp, "}\n\n");
}

const int SPS=10;	// sections per script

class cParmScript
{
protected:
	cSection* _sects[SPS];	// I know, an unnecessary limitation, will be changed later ;)
public:
	cParmScript();
	void Add(cSection* sect);
	void Load(FILE* fp);
	bool LoadSections(char* key);
	void SaveSections(FILE* fp);
	void Save();
};

cParmScript::cParmScript()
{
	int i;
	for (i=0;i<SPS;i++) _sects[i]=NULL;
}

void cParmScript::Add(cSection* kw)
{
	if (!kw) return;
	int i;
	for (i=0;i<SPS;i++)
	{
		if (_sects[i]==NULL)
		{
			_sects[i]=kw;
			break;
		}
	}
	if (i >= SPS)
		LogError("Too many sections in parmscript");
}

bool cParmScript::LoadSections(char* key)
{
	int i;
	for (i=0;i<SPS;i++)
	{
		if (_sects[i]!=NULL)
		{
			if (_sects[i]->Load(key))
				return true;
		}
	}
	return false;
}

void cParmScript::Load(FILE* fp)
{
	int loopexit=0;
	do
	{
		readw2();
		if (!strcmp(script1,"SECTION"))
			LoadSections(script2);
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}

void cParmScript::SaveSections(FILE* fp)
{
	int i;
	for (i=0;i<SPS;i++)
	{
		if (_sects[i]!=NULL)
		{
			_sects[i]->Save(fp);
		}
	}
}

void cParmScript::Save()
{
	FILE *file;
	file = fopen("server.scp", "w");
	
	fprintf(file, "// LONEWOLF Server Script\n");
	fprintf(file, "// Generated by %s Version %s\n\n",  wp_version.betareleasestring.c_str() , wp_version.verstring.c_str() );

	SaveSections(file);

	fprintf(file, "EOF\n\n");
	fclose(file);
}

// important: loaddefault has to be called before. 
// currently it is. makes no sense to change that too.

void loadclientsallowed(void)
{
	int num_c=0, s=0, loopexit=0;
	vector<string> cls;    

	do
	{
		readFullLine();
		
		if (!strcmp(script1,"SERVER_DEFAULT")) { s=1; break; }
		else if (!strcmp(script1,"ALL")) { s=2; break; }
		else if (strlen(script1)>3) 
		{ 		
			num_c++;
			cls.push_back(script1);
		}
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );  

	if (num_c==0 && s==0) s=1; // no entry == pick server_default

	switch(s)
	{
	case 0: 
		clientsAllowed.clear();  // delete the entry from loaddefaults
		copy(cls.begin(), cls.end(), back_inserter(clientsAllowed) ); // copy temp vector to perm vector
		break;

	case 1: 
		break; // already in vector from loaddefaults

	case 2: 
		clientsAllowed.clear();    // delete server_defualt entry from loaddefaults
		clientsAllowed.push_back("ALL"); // now add the correct one
		break;
	}
}


void cSrvParms::Init()
{
	ParmScript = new cParmScript();

	SrvParms->saveinterval=300;
    clientsAllowed.push_back("SERVER_DEFAULT");
	SrvParms->poisontimer=180; // lb

	SrvParms->decaytimer=300;  // 5 minute decay rate;
	SrvParms->archivepath[0]=0;
	SrvParms->logging_path[0]=0;
	SrvParms->htmlfilepath[0]=0;

	SrvParms->alwaysrelogin=0;
	SrvParms->antifastwalk=0;
	SrvParms->antiinjection=0;

	SrvParms->CryptKey1=0;
	SrvParms->CryptKey2=0;

	SrvParms->defaultpriv1=0;
    SrvParms->defaultpriv2=0;

	serv.sServer =  "A LoneWolf shard" ;
	serv.sIP="127.0.0.1";
	serv.uiFull=0;
	serv.uiIndex=0;
	serv.uiPort=2593;
	serv.uiTime=0;
	serv.AliasServer="NONE";
	serv.AliasIP="192.168.0.1";

	SrvParms->IDLESERVERCHATTER=0;

	// Should we check character age for delete?	

	SrvParms->skillcap=999999;	// default=no cap
	SrvParms->statcap=999;		// default=no cap
	SrvParms->logmode=0;
	SrvParms->packetstats=0;

	SrvParms->packetsendstyle=PSS_SMARTBUFFER;

	//taken from 6904t2(5/10/99) - AntiChrist
	SrvParms->playercorpsedecaymultiplier=3;	// Player's corpse decays 3x the normal decay rate
	SrvParms->lootdecayswithcorpse=1;			// JM - 0 Loot disappears with corpse, 1 loot gets left when corpse decays
	
	SrvParms->auto_a_create = 0;                     // for auto accounts
	SrvParms->auto_a_reload = 0;
	SrvParms->checkcharage = 0;

	SrvParms->antimacro_exceptions=0;
	SrvParms->antimacro_inputtype=0;
	SrvParms->antimacro_queryfrequency=20; // default = 5*20=100minutes between queries
	SrvParms->antimacro_on=0;
	SrvParms->antimacro_querytype=0;
	
	SrvParms->invisibiliytimer=60;			// invisibility spell lasts 1 minute
	SrvParms->snoopdelay=7;
	SrvParms->skilldelay=5;					// Skill usage delay (5 seconds);
	SrvParms->objectdelay=1;					// in seconds
	SrvParms->bandagedelay=6;					// in seconds
	SrvParms->bandageincombat=1;				// 0=no, 1=yes; can use bandages while healer and/or patient are in combat ?
	SrvParms->bandagecanres=1;				// 0=no, 1=yes; can resurrect with bandages
	SrvParms->inactivitytimeout=300;			// seconds of inactivity until player will be disconnected

	cSection* sp = new cSection("BOOKS");
	sp->Add(new cKeyWord("PATH",		KWT_str_str,	&bookpath[0],	""));
	ParmScript->Add(sp);

	sp = new cSection("CONTAINERS");
	sp->Add(new cKeyWord("BANKBOXLIMIT",			KWT_dec_uint,	&BankboxLimit,			125));
	sp->Add(new cKeyWord("STRONGBOXLIMIT",			KWT_dec_uint,	&StrongboxLimit,		25));
	sp->Add(new cKeyWord("SECURECONTLIMIT",			KWT_dec_uint,	&SecurecontLimit,		125));
	sp->Add(new cKeyWord("CONTAINERLIMIT",			KWT_dec_uint,	&ContainerLimit,		125));
	sp->Add(new cKeyWord("CONTAINERWEIGHT",			KWT_dec_uint,	&ContainerWeight,		400));
	sp->Add(new cKeyWord("BOATHOLDWEIGHT",			KWT_dec_uint,	&BoatHoldWeight,		1200));
	ParmScript->Add(sp);

	sp = new cSection("MONEY");
	sp->Add(new cKeyWord("STARTGOLD",			KWT_dec_uint,	&startgold,			100));
	sp->Add(new cKeyWord("GOLDWEIGHT",			KWT_flo_flo,	&goldweight,		0.005f));//what a goldpiece weighs this is in hundreths of a stone! AntiChrist
	sp->Add(new cKeyWord("STABLING_FEE",		KWT_flo_flo,	&stablingfee,		0.25));
	sp->Add(new cKeyWord("USESPECIALBANK",		KWT_dec_int,	&usespecialbank,	0));	//AntiChrist - specialbank
	sp->Add(new cKeyWord("SPECIALBANKTRIGGER",	KWT_str_str,	&specialbanktrigger[0],"WARE"));
	ParmScript->Add(sp);

/*	SrvParms->startgold=100;
	SrvParms->goldweight=0.005f;	
	SrvParms->stablingfee=0.25;
	strcpy(SrvParms->specialbanktrigger,"WARE"); //AntiChrist - Special Bank word trigger
	SrvParms->usespecialbank=0;	//AntiChrist - 1= Special Bank enabled */

	sp = new cSection("PLAYERPOSITIONFILE");
	sp->Add(new cKeyWord("FILENAME",	KWT_str_str,	&playerpositionsfile[0],	""));
	sp->Add(new cKeyWord("REFRESHTIME",	KWT_dec_int,	&playerpositionsrefresh,	0));
	sp->Add(new cKeyWord("BACKUPRATIO",	KWT_dec_int,	&playerposfilebackupratio,	0));
	ParmScript->Add(sp);

	sp = new cSection("SPEED");
	sp->Add(new cKeyWord("ANTINPCAGGREGATIONS",	KWT_dec_int,	&speed.antinpcaggregation,	0));
	sp->Add(new cKeyWord("SLEEP_IDLE",			KWT_dec_int,	&speed.idlesleep,			1000));	// 1 sec
    sp->Add(new cKeyWord("SLEEP_PERLOOP",		KWT_dec_int,	&speed.loopsleep,			10));
	sp->Add(new cKeyWord("MAXSOCKETWAIT",		KWT_dec_int,	&speed.maxsocketwait,		10));
	sp->Add(new cKeyWord("CHECK_ITEMS",			KWT_flo_flo,	&speed.itemtime,			1.5));
	sp->Add(new cKeyWord("CHECK_NPCS",			KWT_flo_flo,	&speed.npctime,				1.0));
	sp->Add(new cKeyWord("CHECK_NPCAI",			KWT_flo_flo,	&speed.npcaitime,			1.2));
	sp->Add(new cKeyWord("CHECK_SPAWNREGIONS",	KWT_dec_uint,	&speed.srtime,				30));	
	sp->Add(new cKeyWord("BOAT_SPEED",			KWT_flo_flo,	&boatspeed,					0.75));	
	ParmScript->Add(sp);
	
	sp = new cSection("COMBAT");
	sp->Add(new cKeyWord("COMBAT_HIT_MESSAGE",		KWT_dec_uint,	&combathitmessage,		1));
	sp->Add(new cKeyWord("MAX_ABSORBTION",			KWT_dec_uint,	&maxabsorbtion,			20));	// 20 Arm (single armour piece) -- Magius(CHE)
	sp->Add(new cKeyWord("MAX_NON_HUMAN_ABSORBTION",KWT_dec_uint,	&maxnohabsorbtion,		100));	// 100 Arm (total armour) -- Magius(CHE)
	sp->Add(new cKeyWord("MONSTERS_VS_ANIMALS",		KWT_dec_uint,	&monsters_vs_animals,	0));	// By default monsters won't attack animals
	sp->Add(new cKeyWord("ANIMALS_ATTACK_CHANCE",	KWT_dec_uint,	&animals_attack_chance,	15));	// a 15% chance
	sp->Add(new cKeyWord("ANIMALS_GUARDED",			KWT_dec_uint,	&animals_guarded,		0)); 	// By default players can attack animals without summoning guards
	sp->Add(new cKeyWord("NPC_DAMAGE_RATE",			KWT_dec_uint,	&npcdamage,				2));
	sp->Add(new cKeyWord("NPC_BASE_FLEEAT",			KWT_dec_uint,	&npc_base_fleeat,		20));	// 20% of hp
	sp->Add(new cKeyWord("NPC_BASE_REATTACKAT",		KWT_dec_uint,	&npc_base_reattackat,	40));
	sp->Add(new cKeyWord("ATTACKSTAMINA",			KWT_dec_int,	&attackstamina,			-2));	// AntiChrist - attacker looses stamina when hits
	sp->Add(new cKeyWord("ATTACK_DISTANCE",			KWT_dec_uint,	&attack_distance,		13));	// blackwind 
	ParmScript->Add(sp);

	sp = new cSection("VENDOR");
	sp->Add(new cKeyWord("SELLBYNAME",		KWT_dec_uint,	&sellbyname,		1));
	sp->Add(new cKeyWord("SELLFROMSUBPACKS",KWT_dec_int,	&sellfromsubpacks,	0));
	sp->Add(new cKeyWord("SELLMAXITEM",		KWT_dec_uint,	&sellmaxitem,		5));
	sp->Add(new cKeyWord("TRADESYSTEM",		KWT_dec_uint,	&trade_system,		1));
	sp->Add(new cKeyWord("RANKSYSTEM",		KWT_dec_uint,	&rank_system,		1));
	sp->Add(new cKeyWord("CHECKBANK",		KWT_dec_int,	&CheckBank,			2000)); // Ripper..2000 default
	sp->Add(new cKeyWord("VENDORGREET",		KWT_dec_int,	&VendorGreet,		1));	// Ripper..do greet by default.
	ParmScript->Add(sp);

	sp = new cSection("REGENERATE");
	sp->Add(new cKeyWord("HITPOINTS_REGENRATE",	KWT_dec_uint, &hitpointrate,	8));	// Seconds to heal ONE hp
	sp->Add(new cKeyWord("STAMINA_REGENRATE",	KWT_dec_uint, &staminarate,		3));	// Seconds to heal ONE stm
	sp->Add(new cKeyWord("MANA_REGENRATE",		KWT_dec_uint, &manarate,		5));	// Seconds to heal ONE mn
	sp->Add(new cKeyWord("ARMOR_AFFECT_MANA_REGEN",KWT_dec_int,&armoraffectmana,1));	// meditation allowed
	ParmScript->Add(sp);

	sp = new cSection("HUNGER");
	sp->Add(new cKeyWord("DEFAULTFOOD",			KWT_str_sp1,	&defaultfood,	0xFF));	// all chars can eat every food by default (thrawn)
	sp->Add(new cKeyWord("HUNGERRATE",			KWT_dec_uint,	&hungerrate,	6000));	// seconds
	sp->Add(new cKeyWord("HUNGER_DAMAGE",		KWT_dec_uint,	&hungerdamage,	2));	// 2 Hitpoints
	sp->Add(new cKeyWord("HUNGER_DAMAGE_RATE",	KWT_dec_uint,	&hungerdamagerate,	10));	// seconds
	ParmScript->Add(sp);

	sp = new cSection("RESOURCE");
	sp->Add(new cKeyWord("MINECHECK",			KWT_dec_uint,	&minecheck,				1));
	sp->Add(new cKeyWord("ORE_PER_AREA",		KWT_dec_uint,	&resource.ore,			10));
	sp->Add(new cKeyWord("ORE_RESPAWN_TIME",	KWT_dec_uint,	&resource.oretime,		600));
	sp->Add(new cKeyWord("ORE_RESPAWN_RATE",	KWT_dec_uint,	&resource.orerate,		3));
	sp->Add(new cKeyWord("ORE_RESPAWN_AREA",	KWT_dec_uint,	&resource.orearea,		10));
	sp->Add(new cKeyWord("ORE_TRIGGER",			KWT_dec_int,	&resource.miningtrigger,0));
	sp->Add(new cKeyWord("ORE_STAMINA",			KWT_dec_int,	&resource.miningstamina,-5));
	sp->Add(new cKeyWord("LOGS_PER_AREA",		KWT_dec_uint,	&resource.logs,			3));
	sp->Add(new cKeyWord("LOG_RESPAWN_TIME",	KWT_dec_uint,	&resource.logtime,		600));
	sp->Add(new cKeyWord("LOG_RESPAWN_RATE",	KWT_dec_uint,	&resource.lograte,		3));
	sp->Add(new cKeyWord("LOG_RESPAWN_AREA",	KWT_dec_uint,	&resource.logarea,		10));
	sp->Add(new cKeyWord("LOG_TRIGGER",			KWT_dec_int,	&resource.logtrigger,	0));
	sp->Add(new cKeyWord("LOG_STAMINA",			KWT_dec_int,	&resource.logstamina,	-5));
	ParmScript->Add(sp);
	
	SrvParms->skilladvancemodifier=1000;		// Default Modulo val for skill advance
    SrvParms->statsadvancemodifier=500;		// Default Modulo val for stats advance
	SrvParms->gatetimer=30;
	SrvParms->showdeathanim=1;				// 1 = yes/true
	SrvParms->engraving=1;		// Values= 0(Disabled) or 1(Enabled) - allow engravin of weapons, etc.. when enabled
	SrvParms->smeltableitems=1;		// Values= 0(Disabled) or 1(Enabled) - allow smelting of weapons, etc.. when enabled
	SrvParms->skilllevel=5;		// Range from 1 to 10 - This value if the difficulty to create an item using a make command: 1 is easy, 10 is difficult! - Magius(CHE)
	SrvParms->guardsactive=1;
	SrvParms->errors_to_console=0;
	SrvParms->bg_sounds=2;
	SrvParms->announceworldsaves=1;
	SrvParms->joinmsg=1;
	SrvParms->partmsg=1;
	SrvParms->savespawns=1;
	SrvParms->npcpopmenues=0;
	SrvParms->AOS_Mode=0;
	SrvParms->newmagictest=0;

	SrvParms->gm_log=0;
	SrvParms->pvp_log=0;
	SrvParms->speech_log=0;
	SrvParms->server_log=0;
	SrvParms->Key1=new cKeyWord("CHAR_LOG", KWT_dec_int, &char_log, 0);

	SectionIT=new cSection("IT");
	SectionIT->Add(new cKeyWord("THEHARDWAY", KWT_dec_int, &thehardway, 0));

	SrvParms->rogue=1;
	SrvParms->quittime=300;//Instalog

	SrvParms->msgboardpath[0] = 0;	   // Dupois - Added Dec 20, 1999 for message boards (current dir)
	SrvParms->backup_save_ratio=1; //LB
	
	SrvParms->msgpostaccess=0;           // Dupois - Added Dec 20, 1999 for message boards (GM only)
	SrvParms->msgpostremove=0;           // Dupois - Added Dec 20, 1999 for message boards (GM only)
	SrvParms->msgretention=30;           // Dupois - Added Dec 20, 1999 for message boards (30 Days)
	SrvParms->escortactive=1;            // Dupois - Added Dec 20, 1999 for message boards (Active)
	SrvParms->escortinitexpire=86400;    // Dupois - Added Dec 20, 1999 for message boards (24 hours)
	SrvParms->escortactiveexpire=1800;   // Dupois - Added Dec 20, 1999 for message boards (1/2 Hour)
	SrvParms->escortdoneexpire=1800;     // Dupois - Added Dec 20, 1999 for message boards (1/2 Hour)
		
	SrvParms->bountysactive=1;           // Dupois - Added July 18, 2000 for bountys (1-Active)
	SrvParms->bountysexpire=0;           // Dupois - Added July 18, 2000 for bountys (0-Never - in days)
	
	
	SrvParms->maxstealthsteps=10;// AntiChrist - stealth=100.0 --> 10 steps allowed
	SrvParms->runningstaminasteps=15;// AntiChrist - how often ( in steps ) stamina decreases while running
	
	SrvParms->persecute=1;//AntiChrist

	SrvParms->html=-1;//HTML

	SrvParms->tamed_disappear=1; // Ripper
	SrvParms->houseintown=0; // Ripper
	SrvParms->shoprestock=1; //Ripper
	SrvParms->commandPrefix = '\''; //Ripper
	SrvParms->defaultgmview = 2;

	SrvParms->housedecay_secs=604800; // LB, one week

    SrvParms->eclipsemode=0; // Light to dark..
	SrvParms->ra_port = 2594;
	SrvParms->EnableRA = false;

	SrvParms->showCVCS = 1; // blackwind 
	SrvParms->default_jail_time=86400; // 

	SrvParms->mul_autodetect = 1;
	SrvParms->mulpath[0]=0;

/*
	resource.logs=3;
	resource.logtime=600;
	resource.logarea=10;
	resource.lograte=3;
	resource.logtrigger=0;
	resource.logstamina=-5;
	resource.ore=10;
	resource.oretime=600;
	resource.orerate=3;
	resource.orearea=10;
	resource.miningtrigger=0;
	resource.miningstamina=-5; */
	//REPSYS
	repsys.crimtime=120;
	repsys.maxkills=4;
	repsys.murderdecay=420;
	//RepSys ---^
	tracking_data.baserange=TRACKINGRANGE;
	tracking_data.maxtargets=MAXTRACKINGTARGETS;
	tracking_data.basetimer=TRACKINGTIMER;
	tracking_data.redisplaytime=TRACKINGDISPLAYTIME;
	begging_data.range=3;
	begging_data.timer=300;
	strcpy(begging_data.text[0],"Could thou spare a few coins?");
	strcpy(begging_data.text[1],"Hey buddy can you spare some gold?");
	strcpy(begging_data.text[2],"I have a family to feed, think of the children.");
	fishing_data.basetime=10;		// it takes at least FISHINGTIMEBASE sec to fish;
	fishing_data.randomtime=5;
	fishing_data.loot_probability=12;
	spiritspeak_data.spiritspeaktimer=30; // spiritspeak lasts at least 30 seconds, additional time is calculated by Skill & INT;
}

void loadanticheat()
{
	int loopexit=0;
	do
	{
	  readw2();		

	  if (!strcmp(script1,"ANTIFASTWALKHACKTOOLS")) SrvParms->antifastwalk =  (char) str2num(script2);
	  else if (!strcmp(script1,"ANTIINJECTION"))    SrvParms->antiinjection = (char) str2num(script2);
	  else if (!strcmp(script1,"CRYPTKEY1"))        SrvParms->CryptKey1 =     str2num(script2);
	  else if (!strcmp(script1,"CRYPTKEY2"))        SrvParms->CryptKey2 =     str2num(script2);
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}


void loadnetworking()
{
	int loopexit=0;
	do
	{
	  readw2();	
	  if (!strcmp(script1,"GAMESERVERPORT"))  serv.uiPort = (short) str2num(script2);     
	  else if (!strcmp(script1,"PACKETSENDSTYLE:")) 
	  {	   
		 if ( (!strcmp ( script2, "UNBUFFERED") ) )  SrvParms->packetsendstyle  = PSS_UNBUFFERED;
	     else if ( (!strcmp ( script2, "SMARTBUFFER") ) ) SrvParms->packetsendstyle  = PSS_SMARTBUFFER;		 
	     else SrvParms->packetsendstyle = PSS_SMARTBUFFER;
	     	
	  }
	  //else if (!strcmp(script1,"POCKEYSFASTLOGIN"))) POKEY=str2num(script2);

	}
	while (  (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}



void loadminiloginserver()
{
	int loopexit=0;
	do
	{
		readw2();
		if (!strcmp(script1,"SHARDNAME"))               serv.sServer = script2;
		else if (!strcmp(script1,"CLIENTRELAYADDRESS")) serv.sIP = script2;
		else if (!strcmp(script1,"ALIASADDRESS"))       serv.AliasIP = script2;
		else if (!strcmp(script1,"ALIASNAME"))          serv.AliasServer = script2;
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}


void loadantimacro()
{
	int loopexit=0;
	do
	{
		readw2();
		if (!strcmp(script1,     "ENABLED"))              SrvParms->antimacro_on=str2num(script2);	 
		else if (!strcmp(script1,"QUERYCHECKINTERVALL"))  SrvParms->antimacro_queryfrequency=str2num(script2);
		else if (!strcmp(script1,"QUERYTYPE"))            SrvParms->antimacro_querytype=str2num(script2);	 
		else if (!strcmp(script1,"INPUTTYPE"))            SrvParms->antimacro_inputtype=str2num(script2);
		else if (!strcmp(script1,"EXCEPTIONSTYPE"))       SrvParms->antimacro_exceptions=str2num(script2);
	}
	while (  (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}

void loadbackup()
{
	int loopexit=0;
	do
	{
	  readw2();
	  if (!strcmp(script1,"ARCHIVEPATH")) strcpy(SrvParms->archivepath,script2);
	  else if (!strcmp(script1,"BACKUP_SAVE_RATIO")) 
	  { //LB
	    SrvParms->backup_save_ratio=str2num(script2);
	    if (SrvParms->backup_save_ratio<1) SrvParms->backup_save_ratio=1;
	  }

	}
	while (  (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );

}

void cSrvParms::LoadLogging()
{
	int loopexit=0;
	do
	{
		readw2();
		if (!strcmp(script1,"LOGGINGPATH")) strcpy(SrvParms->logging_path,script2);
		else if (!strcmp(script1,"SERVER_LOG")) SrvParms->server_log=str2num(script2);
		else if (!strcmp(script1,"SPEECH_LOG")) SrvParms->speech_log=str2num(script2);
		else if (!strcmp(script1,"PVP_LOG")) SrvParms->pvp_log=str2num(script2);
		else if (SrvParms->Key1->Load(script1,script2)) continue;
		else if (!strcmp(script1,"GM_LOG")) SrvParms->gm_log=str2num(script2);		
	}
	while (  (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}

void loadworldsaveparams()
{
	int loopexit=0;
	do
	{
		readw2();
		if (!strcmp(script1,"SAVEINTERVAL")) SrvParms->saveinterval=str2num(script2); 	  
	}
	while ( strcmp(script1, "}") && (++loopexit < MAXLOOPS) );
}

void loadmulparams()
{
	int loopexit=0;
	do
	{
	  readw2();
	  if (!strcmp(script1,"MULPATH")) strcpy(SrvParms->mulpath,script2);
	  else if (!strcmp(script1,"AUTODETECT")) SrvParms->mul_autodetect=str2num(script2);
	}
	while (  (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}

void loadhtmlparams()
{
	int loopexit=0;
	do
	{
	  readw2();
	 // if (!strcmp(script1,"HTMLFILEPATH"))) strcpy(SrvParms->htmlfilepath,script2);
	 /* else*/ if (!strcmp(script1,"HTMLSAVEINTERVAL")) SrvParms->html=str2num(script2);//HTML

	}
	while (  (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}

void loadmsgboardparams()
{
	int loopexit=0;
	do
	{
	  readw2();
	  
	  if (!strcmp(script1,"MSGBOARDPATH")) strcpy(SrvParms->msgboardpath,script2);               // Dupois - Added Dec 20, 1999 for message boards
	  else if (!strcmp(script1,"MSGPOSTACCESS")) SrvParms->msgpostaccess=str2num(script2);            // Dupois - Added Dec 20, 1999 for message boards
	  else if (!strcmp(script1,"MSGPOSTREMOVE")) SrvParms->msgpostremove=str2num(script2);            // Dupois - Added Dec 20, 1999 for message boards
	  else if (!strcmp(script1,"MSGRETENTION")) SrvParms->msgretention=str2num(script2);              // Dupois - Added Dec 20, 1999 for message boards
	  else if (!strcmp(script1,"ESCORTACTIVE")) SrvParms->escortactive=str2num(script2);              // Dupois - Added Dec 20, 1999 for escorts
	  else if (!strcmp(script1,"ESCORTINITEXPIRE")) SrvParms->escortinitexpire=str2num(script2);      // Dupois - Added Dec 20, 1999 for escorts
	  else if (!strcmp(script1,"ESCORTACTIVEEXPIRE")) SrvParms->escortactiveexpire=str2num(script2);  // Dupois - Added Dec 20, 1999 for escorts
	  else if (!strcmp(script1,"ESCORTDONEEXPIRE")) SrvParms->escortdoneexpire=str2num(script2);      // Dupois - Added Dec 20, 1999 for escorts
	  else if (!strcmp(script1,"BOUNTYSACTIVE")) SrvParms->bountysactive=str2num(script2);            // Dupois - Added July 18, 2000 for bountys
	  else if (!strcmp(script1,"BOUNTYSEXPIRE")) SrvParms->bountysexpire=str2num(script2);            // Dupois - Added July 18, 2000 for bountys

	}
	while (  (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );

}

void loadshowinfos()
{
	int loopexit=0;
	do
	{
	  readw2();
	  if (!strcmp(script1,"ANNOUNCE_WORLDSAVES")) SrvParms->announceworldsaves=str2num(script2);
	  else if (!strcmp(script1,"JOINMSG")) SrvParms->joinmsg=str2num(script2);
	  else if (!strcmp(script1,"PARTMSG")) SrvParms->partmsg=str2num(script2);
	  else if (!strcmp(script1,"SHOWDEATHANIM")) SrvParms->showdeathanim=str2num(script2);
	  else if (!strcmp(script1,"ERRORS_TO_CONSOLE")) SrvParms->errors_to_console=str2num( script2 ); // LB
	  else if (!strcmp(script1,"SHOW_CVCS_INFO_AT_LOGIN")) SrvParms->showCVCS=str2num(script2); //blackwind 
	  else if (!strcmp(script1,"LOGMODE")) { SrvParms->logmode=str2num(script2); debug_network=SrvParms->logmode&1; }
	  else if (!strcmp(script1,"PACKETSTATS")) { SrvParms->packetstats=str2num(script2); }
	}
	while (  (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );

}

void loadaccountparams()
{
	int loopexit=0;
	do
	{
		readw2();
		if (!strcmp(script1, "AUTO_CREATE_ACCTS"))		SrvParms->auto_a_create = str2num( script2 );
		else if (!strcmp(script1, "AUTO_RELOAD_ACCTS"))	SrvParms->auto_a_reload = str2num( script2 );
		else if (!strcmp(script1, "CHECKCHAR_AGE"))		SrvParms->checkcharage = str2num( script2 );	  
	}
	while (strcmp(script1, "}") && (++loopexit < MAXLOOPS) );
}
/*
void loadcontainers() // by Ripper
{
	int loopexit=0;
	do
	{
		readw2();
		if (!strcmp(script1,"BANKBOXLIMIT")) SrvParms->BankboxLimit=str2num(script2);
		else if (!strcmp(script1,"STRONGBOXLIMIT")) SrvParms->StrongboxLimit=str2num(script2);
		else if (!strcmp(script1,"SECURECONTLIMIT")) SrvParms->SecurecontLimit=str2num(script2);
		else if (!strcmp(script1,"CONTAINERLIMIT")) SrvParms->ContainerLimit=str2num(script2);
		else if (!strcmp(script1,"CONTAINERWEIGHT")) SrvParms->ContainerWeight=str2num(script2);
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}*/
/*
void loadresources()
{
	int loopexit=0;
	do
	{
		readw2();
		if (!strcmp(script1,"MINECHECK")) SrvParms->minecheck=str2num(script2); // Moved by Magius(CHE)
		else if (!strcmp(script1,"LOGS_PER_AREA")) resource.logs=str2num(script2);
		else if (!strcmp(script1,"LOG_RESPAWN_TIME")) resource.logtime=str2num(script2);
		else if (!strcmp(script1,"LOG_RESPAWN_RATE")) resource.lograte=str2num(script2);
		else if (!strcmp(script1,"LOG_RESPAWN_AREA")) resource.logarea=str2num(script2);
		else if (!strcmp(script1,"LOG_TRIGGER")) resource.logtrigger=str2num(script2);
		else if (!strcmp(script1,"LOG_STAMINA")) resource.logstamina=str2num(script2);
		else if (!strcmp(script1,"ORE_PER_AREA")) resource.ore=str2num(script2);
		else if (!strcmp(script1,"ORE_RESPAWN_TIME")) resource.oretime=str2num(script2);
		else if (!strcmp(script1,"ORE_RESPAWN_RATE")) resource.orerate=str2num(script2);
		else if (!strcmp(script1,"ORE_RESPAWN_AREA")) resource.logarea=str2num(script2);
		else if (!strcmp(script1,"ORE_TRIGGER")) resource.miningtrigger=str2num(script2);
		else if (!strcmp(script1,"ORE_STAMINA")) resource.miningstamina=str2num(script2);
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}*/

void loadrepsys() //Repsys
{
	int loopexit=0;
	do
	{
		readw2();
		if (!strcmp(script1,"MURDER_DECAY")) repsys.murderdecay=str2num(script2);
		else if (!strcmp(script1,"MAXKILLS")) repsys.maxkills=str2num(script2);
		else if (!strcmp(script1,"CRIMINAL_TIME")) repsys.crimtime=str2num(script2);
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );

	if (!repsys.murderdecay) repsys.murderdecay=420;
	if (!repsys.maxkills) repsys.maxkills=4;
	if (!repsys.crimtime) repsys.crimtime=120;
}

void cSrvParms::loadserver()
{
	// Server.scp completly reorganized by Magius(CHE) 3 September 1999
	int loopexit=0;
	do
	{
		readw2();
	
		if (!strcmp(script1,"SKILLCAP")) SrvParms->skillcap=str2num(script2);
		else if (!strcmp(script1,"STATCAP")) SrvParms->statcap=str2num(script2);
		else if (!strcmp(script1,"SKILLADVANCEMODIFIER"))
		{
            int mod=str2num(script2);
            SrvParms->skilladvancemodifier=(mod>=1 ? mod : 1);    // must be nonzero && positive
		}
        else if (!strcmp(script1,"STATSADVANCEMODIFIER"))
		{
            int mod=str2num(script2);
            SrvParms->statsadvancemodifier=(mod>=1 ? mod : 1);    // must be nonzero && positive
		}
		else if (!strcmp(script1,"SKILLLEVEL")) 
		{
			SrvParms->skilllevel=str2num(script2); //MAgius(CHE)
			if (SrvParms->skilllevel<2 || SrvParms->skilllevel>10) SrvParms->skilllevel=5;
		}

		else if (!strcmp(script1,"DECAYTIMER")) SrvParms->decaytimer=str2num(script2);
		//taken from 6904t2(5/10/99) - AntiChrist
		else if (!strcmp(script1,"PLAYERCORPSEDECAYMULTIPLIER")) SrvParms->playercorpsedecaymultiplier=str2num(script2);//JM
		else if (!strcmp(script1,"LOOTDECAYSWITHCORPSE")) SrvParms->lootdecayswithcorpse=str2num(script2);
		else if (!strcmp(script1,"INVISTIMER")) SrvParms->invisibiliytimer=str2num(script2);
		else if (!strcmp(script1,"SKILLDELAY")) SrvParms->skilldelay=str2num(script2);
		else if (!strcmp(script1,"OBJECTDELAY")) SrvParms->objectdelay=str2num(script2);
		else if (!strcmp(script1,"BANDAGEDELAY")) SrvParms->bandagedelay=str2num(script2);
		else if (!strcmp(script1,"BANDAGEINCOMBAT")) SrvParms->bandageincombat=str2num(script2);
		else if (!strcmp(script1,"BANDAGECANRES")) SrvParms->bandagecanres=str2num(script2);
		else if (!strcmp(script1,"INACTIVITYTIMEOUT")) SrvParms->inactivitytimeout=str2num(script2);
		else if (!strcmp(script1,"GATETIMER")) SrvParms->gatetimer=str2num(script2);	
		else if (!strcmp(script1,"GUARDSACTIVE")) SrvParms->guardsactive=str2num(script2);			
		else if (!strcmp(script1,"BG_SOUNDS")) SrvParms->bg_sounds=str2num(script2);	
        else if (!strcmp(script1,"POISONTIMER")) SrvParms->poisontimer=str2num(script2); // lb
		else if (!strcmp(script1,"SAVESPAWNREGIONS")) SrvParms->savespawns=str2num(script2);	
		else if (!strcmp(script1,"ROGUE")) SrvParms->rogue = str2num(script2);
		else if (!strcmp(script1,"CHAR_TIME_OUT")) SrvParms->quittime = str2num(script2);//Instalog
        else if (!strcmp(script1,"ALWAYSRELOGIN")) SrvParms->alwaysrelogin = str2num(script2);
		else if (!strcmp(script1,"MAXSTEALTHSTEPS")) SrvParms->maxstealthsteps = str2num(script2);//AntiChrist
		else if (!strcmp(script1,"RUNNINGSTAMINASTEPS")) SrvParms->runningstaminasteps=str2num(script2);//AntiChrist
        else if (!strcmp(script1,"PERSECUTION")) SrvParms->persecute=str2num(script2);//AntiChrist			
		else if (!strcmp(script1,"TAMED_DISAPPEAR")) SrvParms->tamed_disappear=str2num(script2); // Ripper
		else if (!strcmp(script1,"HOUSEINTOWN")) SrvParms->houseintown=str2num(script2); // Ripper
		else if (!strcmp(script1,"SHOPRESTOCK")) SrvParms->shoprestock=str2num(script2); // Ripper
		else if (!strcmp(script1,"COMMANDPREFIX")) SrvParms->commandPrefix=script2[0]; //Ripper	 	
		else if (!strcmp(script1,"HOUSEDECAY_SECS")) SrvParms->housedecay_secs=str2num( script2 ); // LB	
		else if (!strcmp(script1,"DEFAULT_JAIL_TIME")) SrvParms->default_jail_time=str2num(script2);// blackwind 
	    else if (!strcmp(script1,"DEFAULTPRIVONE")) SrvParms->defaultpriv1=str2num(script2); 
		else if (!strcmp(script1,"DEFAULTPRIVTWO")) SrvParms->defaultpriv2=str2num(script2); 
		else if (!strcmp(script1,"NPCPOPUPMENUES")) SrvParms->npcpopmenues=str2num(script2); 	
		else if (!strcmp(script1,"AOSMODE")) SrvParms->AOS_Mode=str2num(script2); 	
		else if (!strcmp(script1,"NEWMAGICTEST")) SrvParms->newmagictest=str2num(script2); 	
		else if (!strcmp(script1,"ENGRAVING")) SrvParms->engraving=str2num(script2); // Thrawn
		else if (!strcmp(script1,"SMELTABLEITEMS")) SrvParms->smeltableitems=str2num(script2); // Thrawn
		else if (!strcmp(script1,"DEFAULTGMVIEW")) SrvParms->defaultgmview=str2num(script2); //the viewingdepth default (thrawn)
		else if (!strcmp(script1,"IDLESERVERCHATTER")) SrvParms->IDLESERVERCHATTER=str2num(script2);

		// compatibility/conversion mechanism for moving keys to a different section here. (Duke, 17.2.04)
		// works as follows:
		// 1. from an OLD server.scp, the keywords a read in here.
		// 2. the values are NOT overwritten by the subsequent NEW section because it doesn't exist in an old file
		// 3. the values are saved to the new section and not saved to section IT anymore.
		// SECTION 'MONEY' (created in 13.0.9 beta)
		else if (!strcmp(script1,"STARTGOLD")) SrvParms->startgold=str2num(script2); 
		else if (!strcmp(script1,"GOLDWEIGHT")) SrvParms->goldweight=atof(script2);
		else if (!strcmp(script1,"STABLING_FEE")) SrvParms->stablingfee = atof(script2);
		else if (!strcmp(script1,"USESPECIALBANK")) SrvParms->usespecialbank=str2num(script2); //AntiChrist - Special Bank
		else if (!strcmp(script1,"SPECIALBANKTRIGGER"))  //AntiChrist - Special Bank
		{
			strcpy(SrvParms->specialbanktrigger,script2);
			for(int unsigned i = 0; i < strlen(SrvParms->specialbanktrigger); i++)
				SrvParms->specialbanktrigger[i] = toupper(SrvParms->specialbanktrigger[i]);
		}

		else if (SrvParms->SectionIT->LoadKeywords(script1,script2)) continue;
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}


void loadtracking() // Load scriptable tracking data
{
	int loopexit=0;
	do
	{
		readw2();
		if (!strcmp(script1,"BASE_TRACKING_RANGE")) tracking_data.baserange=str2num(script2);
		else if (!strcmp(script1,"MAX_TRACKING_TARGETS"))
		{
			tracking_data.maxtargets=str2num(script2);
			if (tracking_data.maxtargets>MAXTRACKINGTARGETS)
				tracking_data.maxtargets=MAXTRACKINGTARGETS;
		}
		else if (!strcmp(script1,"BASE_TRACKING_TIME")) tracking_data.basetimer=str2num(script2);
		else if (!strcmp(script1,"TRACKING_MESSAGE_REDISPLAY_TIME")) tracking_data.redisplaytime=str2num(script2);
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}

void loadbegging()
{
	int loopexit=0;
	do
	{
		readw2();

		if (!strcmp(script1,"BEGGING_TIME")) begging_data.timer=str2num(script2);
		else if (!strcmp(script1,"BEGGING_RANGE")) begging_data.range=str2num(script2);
		else if (!strcmp(script1,"BEGGING_TEXT0")) strcpy(begging_data.text[0],script2);
		else if (!strcmp(script1,"BEGGING_TEXT1")) strcpy(begging_data.text[1],script2);
		else if (!strcmp(script1,"BEGGING_TEXT2")) strcpy(begging_data.text[2],script2);
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}

void loadfishing()
{
	int loopexit=0;
	do
	{
		readw2();
		if (!strcmp(script1,"BASE_FISHING_TIME")) fishing_data.basetime=str2num(script2);
		else if (!strcmp(script1,"RANDOM_FISHING_TIME")) fishing_data.randomtime=str2num(script2);
		else if (!strcmp(script1,"FISHING_LOOT_PROBABILITY")) fishing_data.loot_probability=str2num(script2); 
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}

void loadspiritspeak()
{
	int loopexit=0;
	do
	{
		readw2();
		if (!strcmp(script1,"SPIRITSPEAKTIMER")) spiritspeak_data.spiritspeaktimer=str2num(script2);
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS) );
}

void loadtime_light()
{
	int loopexit=0;
	do
	{
		readw2();
		if (!strcmp(script1,"DAY")) timeLight.day=str2num(script2);
		else if (!strcmp(script1,"HOUR")) timeLight.hour=str2num(script2);
		else if (!strcmp(script1,"MINUTE")) timeLight.minute=str2num(script2);
		else if (!strcmp(script1,"AMPM")) timeLight.ampm=str2num(script2);
		else if (!strcmp(script1,"MOON1UPDATE")) timeLight.moon1update=str2num(script2);
		else if (!strcmp(script1,"MOON2UPDATE")) timeLight.moon2update=str2num(script2);
		else if (!strcmp(script1,"MOON1")) timeLight.moon1=str2num(script2);
		else if (!strcmp(script1,"MOON2")) timeLight.moon2=str2num(script2);
		else if (!strcmp(script1,"DUNGEONLIGHTLEVEL")) timeLight.dungeonlightlevel=str2num(script2);
		else if (!strcmp(script1,"WORLDFIXEDLEVEL")) timeLight.worldfixedlevel=str2num(script2);
		else if (!strcmp(script1,"WORLDCURLEVEL")) timeLight.worldcurlevel=str2num(script2);
		else if (!strcmp(script1,"WORLDBRIGHTLEVEL")) timeLight.worldbrightlevel=str2num(script2);
		else if (!strcmp(script1,"WORLDDARKLEVEL")) timeLight.worlddarklevel=str2num(script2);
		else if (!strcmp(script1,"SECONDSPERUOMINUTE")) timeLight.secondsperuominute=str2num(script2);
		else if (!strcmp(script1,"SEASON")) timeLight.season=str2num(script2);
		else if (!strcmp(script1,"INLORPOTIONFIXEDTIME")) timeLight.fixedInlorPotionTime=str2num(script2);
		else if (!strcmp(script1,"INLORPOTIONVARTIME")) timeLight.varInlorPotionTime=str2num(script2);
		else if (!strcmp(script1,"INLORSPELLFIXEDTIME")) timeLight.fixedInlorSpellTime=str2num(script2);
		else if (!strcmp(script1,"INLORSPELLVARTIME")) timeLight.varInlorSpellTime=str2num(script2);

	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS));
}

void loadremote_admin()
{
	int loopexit=0;
	do
	{
		readw2();
		if (!strcmp(script1,"ENABLE")) SrvParms->EnableRA = true;
		else if (!strcmp(script1,"DISABLE")) SrvParms->EnableRA = false;
		else if (!strcmp(script1,"PORT")) SrvParms->ra_port = str2num(script2);
	}
	while ( (strcmp(script1, "}")) && (++loopexit < MAXLOOPS));
}

void cSrvParms::LoadScript(char *fn) // Load a server script
{
	int loopexit=0;
	wscfile = fopen(fn, "r");
	if (wscfile==NULL)
	{
		LogMessageVar("<%s> not found...defaults are used",fn);
		return;
	}
	do
	{
		readw2();
		if (!strcmp(script1, "SECTION"))
		{
			if (!strcmp(script2, "IT")) loadserver();
	        else if (!strcmp(script2, "GAMESERVERNETWORKING")) loadnetworking();
			else if (!strcmp(script2, "ANTICHEAT")) loadanticheat();
			else if (!strcmp(script2, "MINILOGINSERVER")) loadminiloginserver();
			else if (!strcmp(script2, "BACKUP")) loadbackup();
			else if (!strcmp(script2, "LOGGING")) LoadLogging();			
			else if (!strcmp(script2, "WORLDSAVE")) loadworldsaveparams();
			else if (!strcmp(script2, "MULS")) loadmulparams();
			else if (!strcmp(script2, "HTML")) loadhtmlparams();
			else if (!strcmp(script2, "ANTIMACRO")) loadantimacro();
			else if (!strcmp(script2, "MSGBOARDS")) loadmsgboardparams();
			else if (!strcmp(script2, "SHOWINFOS")) loadshowinfos();
			else if (!strcmp(script2, "ACCOUNTS")) loadaccountparams();				        
			else if (!strcmp(script2, "CLIENTS_ALLOWED")) loadclientsallowed();
//			else if (!strcmp(script2, "RESOURCE")) loadresources();
			else if (!strcmp(script2, "REPSYS")) loadrepsys();
			else if (!strcmp(script2, "TRACKING")) loadtracking();
			else if (!strcmp(script2, "BEGGING")) loadbegging();
			else if (!strcmp(script2, "FISHING")) loadfishing();
			else if (!strcmp(script2, "SPIRITSPEAK")) loadspiritspeak();
			else if (!strcmp(script2, "TIME_LIGHT")) loadtime_light();
			else if (ParmScript->LoadSections(script2)) continue;
			else if (!strcmp(script2, "REMOTE_ADMIN")) loadremote_admin();
//			else if (!strcmp(script2, "CONTAINERS")) loadcontainers();
		}
	}
	while ( (strcmp(script1, "EOF")) && (++loopexit < MAXLOOPS) );
	fclose(wscfile);
	wscfile = NULL;
}

void cSrvParms::Load() // Load server script
{
	LoadScript("server.scp");
	LoadScript("shards_server.scp");
}

void cSrvParms::Save(void)
{
	FILE *file;
	file = fopen("server.scp", "w");
    vector<string>::const_iterator vis; 
	
	fprintf(file, "// LONEWOLF Server Script\n");
	fprintf(file, "// Generated by %s Version %s\n\n",  wp_version.betareleasestring.c_str() , wp_version.verstring.c_str() );

	fprintf(file, "SECTION CLIENTS_ALLOWED\n");
	fprintf(file, "{\n");	
   
    for (vis=clientsAllowed.begin(); vis != clientsAllowed.end();  ++vis) 
    {
	  fprintf(file, "%s\n", vis->c_str()); 
    }  

	fprintf(file, "}\n\n");

    fprintf(file, "SECTION GAMESERVERNETWORKING\n");		
	fprintf(file, "{\n");
	fprintf(file, "GAMESERVERPORT %i\n", serv.uiPort);	// First entry, make it more visible.

	switch(SrvParms->packetsendstyle)
	{
	case PSS_UNBUFFERED:
		{
			fprintf(file, "PACKETSENDSTYLE: UNBUFFERED\n");
		}
		break;
		
	case PSS_SMARTBUFFER:
		{
			fprintf(file, "PACKETSENDSTYLE: SMARTBUFFER\n");
		}
		break;	
		
	default:
		{
			fprintf(file, "PACKETSENDSTYLE: SMARTBUFFER\n");
		}
		break;
	}
	
	//fprintf(file, "POCKEYSFASTLOGIN %i\n", POKEY);
	fprintf(file, "}\n\n");

	fprintf(file, "SECTION REMOTE_ADMIN\n");
	fprintf(file, "{\n");
	if (SrvParms->EnableRA)
		fprintf(file, "ENABLE\n");
	else
		fprintf(file, "DISABLE\n");
	
	fprintf(file, "PORT %i\n", SrvParms->ra_port);
	fprintf(file, "}\n\n");

	fprintf(file, "SECTION ANTICHEAT\n");
	fprintf(file, "{\n");
	fprintf(file, "ANTIFASTWALKHACKTOOLS %i\n",SrvParms->antifastwalk);
	fprintf(file, "ANTIINJECTION %i\n",SrvParms->antiinjection);
	fprintf(file, "CRYPTKEY1 %lu\n",SrvParms->CryptKey1);
	fprintf(file, "CRYPTKEY2 %lu\n",SrvParms->CryptKey2);
	fprintf(file, "}\n\n");

	fprintf(file, "SECTION ANTIMACRO\n");
	fprintf(file, "{\n");
	fprintf(file, "ENABLED %i\n", SrvParms->antimacro_on);
	fprintf(file, "QUERYCHECKINTERVALL %i\n", SrvParms->antimacro_queryfrequency); 
	fprintf(file, "QUERYTYPE %i\n", SrvParms->antimacro_querytype); 
	fprintf(file, "INPUTTYPE %i\n", SrvParms->antimacro_inputtype); 
	fprintf(file, "EXCEPTIONSTYPE %i\n", SrvParms->antimacro_exceptions); 
	fprintf(file, "}\n\n");

	fprintf(file, "SECTION MINILOGINSERVER\n");
	fprintf(file, "{\n");
	fprintf(file, "SHARDNAME %s\n",serv.sServer.c_str() );
	fprintf(file, "CLIENTRELAYADDRESS %s\n",serv.sIP.c_str() );		 
	fprintf(file, "ALIASNAME %s\n",serv.AliasServer.c_str() );		 
	fprintf(file, "ALIASADDRESS %s\n",serv.AliasIP.c_str() );		 	
	fprintf(file, "}\n\n");

	fprintf(file, "SECTION MULS\n");
	fprintf(file, "{\n");
	fprintf(file, "MULPATH %s\n",SrvParms->mulpath);
	fprintf(file, "AUTODETECT %i\n",SrvParms->mul_autodetect);//LB
	fprintf(file, "}\n\n");

	fprintf(file, "SECTION BACKUP\n");
	fprintf(file, "{\n");
	fprintf(file, "ARCHIVEPATH %s\n",SrvParms->archivepath);
	fprintf(file, "BACKUP_SAVE_RATIO %i\n",SrvParms->backup_save_ratio);//LB
	fprintf(file, "}\n\n");
	
	fprintf(file, "SECTION LOGGING\n");
	fprintf(file, "{\n");
	fprintf(file, "LOGGINGPATH %s\n",SrvParms->logging_path);
	fprintf(file, "SERVER_LOG %i\n",SrvParms->server_log); //Lb, split log to those 4
	fprintf(file, "SPEECH_LOG %i\n",SrvParms->speech_log);
	fprintf(file, "PVP_LOG %i\n",SrvParms->pvp_log);
	fprintf(file, "GM_LOG %i\n",SrvParms->gm_log);
	Key1->Save(file);	// CHARS_LOG
	fprintf(file, "}\n\n");
	
	fprintf(file, "SECTION WORLDSAVE\n");
	fprintf(file, "{\n");
	fprintf(file, "SAVEINTERVAL %i\n",SrvParms->saveinterval);	
	fprintf(file, "}\n\n");

	fprintf(file, "SECTION HTML\n");
	fprintf(file, "{\n");
	//fprintf(file, "HTMLFILEPATH %s\n",SrvParms->htmlfilepath);
	fprintf(file, "HTMLSAVEINTERVAL %i\n",SrvParms->html);
	fprintf(file, "}\n\n");

	fprintf(file, "SECTION MSGBOARDS\n");
	fprintf(file, "{\n");
	fprintf(file, "MSGBOARDPATH %s\n",SrvParms->msgboardpath);              // Dupois - Added Dec 20, 1999 for msgboard.cpp
	fprintf(file, "MSGPOSTACCESS %i\n",SrvParms->msgpostaccess);            // Dupois - Added Dec 20, 1999 for msgboard.cpp
	fprintf(file, "MSGPOSTREMOVE %i\n",SrvParms->msgpostremove);            // Dupois - Added Dec 20, 1999 for msgboard.cpp
	fprintf(file, "MSGRETENTION %i\n",SrvParms->msgretention);              // Dupois - Added Dec 20, 1999 for msgboard.cpp
	fprintf(file, "ESCORTACTIVE %i\n",SrvParms->escortactive);              // Dupois - Added Dec 20, 1999 for escorts
	fprintf(file, "ESCORTINITEXPIRE %i\n",SrvParms->escortinitexpire);      // Dupois - Added Dec 20, 1999 for escorts
	fprintf(file, "ESCORTACTIVEEXPIRE %i\n",SrvParms->escortactiveexpire);  // Dupois - Added Dec 20, 1999 for escorts
	fprintf(file, "ESCORTDONEEXPIRE %i\n",SrvParms->escortdoneexpire);      // Dupois - Added Dec 20, 1999 for escorts	
	fprintf(file, "BOUNTYSACTIVE %i\n",SrvParms->bountysactive);            // Dupois - Added July 18, 2000 for bountys
	fprintf(file, "BOUNTYSEXPIRE %i\n",SrvParms->bountysexpire);            // Dupois - Added July 18, 2000 for bountys
	fprintf(file, "}\n\n");

	fprintf(file, "SECTION SHOWINFOS\n");
	fprintf(file, "{\n");
	fprintf(file, "ANNOUNCE_WORLDSAVES %i\n",SrvParms->announceworldsaves);
	fprintf(file, "JOINMSG %i\n",SrvParms->joinmsg);
	fprintf(file, "PARTMSG %i\n",SrvParms->partmsg);
	fprintf(file, "SHOWDEATHANIM %i\n",SrvParms->showdeathanim);
	fprintf(file, "SHOW_CVCS_INFO_AT_LOGIN %i\n",SrvParms->showCVCS); // blackwind 
	fprintf(file, "ERRORS_TO_CONSOLE %i\n",SrvParms->errors_to_console);
	fprintf(file, "LOGMODE %i\n",SrvParms->logmode);
	fprintf(file, "PACKETSTATS %i\n",SrvParms->packetstats);
	fprintf(file, "}\n\n");

	fprintf(file, "SECTION ACCOUNTS\n");
	fprintf(file, "{\n");
	fprintf(file, "AUTO_CREATE_ACCTS %i\n", SrvParms->auto_a_create);
	fprintf(file, "AUTO_RELOAD_ACCTS %i\n", SrvParms->auto_a_reload); 
	fprintf(file, "CHECKCHAR_AGE %i\n", SrvParms->checkcharage); 
	fprintf(file, "}\n\n");

/*	fprintf(file, "SECTION CONTAINERS\n");
	fprintf(file, "{\n");
	fprintf(file, "BANKBOXLIMIT %i\n",SrvParms->BankboxLimit);
	fprintf(file, "STRONGBOXLIMIT %i\n",SrvParms->StrongboxLimit);
	fprintf(file, "SECURECONTLIMIT %i\n",SrvParms->SecurecontLimit);
    fprintf(file, "CONTAINERLIMIT %i\n",SrvParms->ContainerLimit);
	fprintf(file, "CONTAINERWEIGHT %i\n",SrvParms->ContainerWeight);
	fprintf(file, "}\n\n");*/

	fprintf(file, "SECTION IT\n");
	fprintf(file, "{\n");		
	fprintf(file, "COMMANDPREFIX %c\n", SrvParms->commandPrefix ); // Ripper	
	fprintf(file, "SKILLCAP %i\n",SrvParms->skillcap);
	fprintf(file, "STATCAP %i\n",SrvParms->statcap);
	fprintf(file, "SKILLADVANCEMODIFIER %i\n",SrvParms->skilladvancemodifier);
    fprintf(file, "STATSADVANCEMODIFIER %i\n",SrvParms->statsadvancemodifier);
	fprintf(file, "DECAYTIMER %i\n",SrvParms->decaytimer);
	//taken from 6904t2(5/10/99) - AntiChrist
	fprintf(file, "PLAYERCORPSEDECAYMULTIPLIER %i\n", SrvParms->playercorpsedecaymultiplier);
	fprintf(file, "LOOTDECAYSWITHCORPSE %i\n", SrvParms->lootdecayswithcorpse);
	fprintf(file, "INVISTIMER %i\n",SrvParms->invisibiliytimer);
	fprintf(file, "SKILLDELAY %i\n",SrvParms->skilldelay);
	fprintf(file, "SKILLLEVEL %i\n",SrvParms->skilllevel); // By Magius(CHE)
	fprintf(file, "OBJECTDELAY %i\n",SrvParms->objectdelay);
	fprintf(file, "BANDAGEDELAY %i\n",SrvParms->bandagedelay);
	fprintf(file, "BANDAGEINCOMBAT %i\n",SrvParms->bandageincombat);
	fprintf(file, "BANDAGECANRES %i\n",SrvParms->bandagecanres);
	fprintf(file, "GATETIMER %i\n",SrvParms->gatetimer);
	fprintf(file, "GUARDSACTIVE %i\n",SrvParms->guardsactive);
	fprintf(file, "BG_SOUNDS %i\n",SrvParms->bg_sounds);
	fprintf(file, "POISONTIMER %i\n",SrvParms->poisontimer); // lb
 	fprintf(file, "SAVESPAWNREGIONS %i\n",SrvParms->savespawns); 	
	fprintf(file, "DEFAULTPRIVONE %i\n",SrvParms->defaultpriv1);	
	fprintf(file, "DEFAULTPRIVTWO %i\n",SrvParms->defaultpriv2);	
	fprintf(file, "ROGUE %i\n",SrvParms->rogue);
	fprintf(file, "INACTIVITYTIMEOUT %i\n",SrvParms->inactivitytimeout);
	fprintf(file, "CHAR_TIME_OUT %i\n",SrvParms->quittime);//Instalog
	fprintf(file, "ALWAYSRELOGIN %i\n",SrvParms->alwaysrelogin);
	fprintf(file, "MAXSTEALTHSTEPS %i\n",SrvParms->maxstealthsteps);//Instalog
	fprintf(file, "RUNNINGSTAMINASTEPS %i\n",SrvParms->runningstaminasteps);//Instalog
    fprintf(file, "PERSECUTION %i\n",SrvParms->persecute);//AntiChrist	
	fprintf(file, "TAMED_DISAPPEAR %i\n",SrvParms->tamed_disappear); // Ripper
	fprintf(file, "HOUSEINTOWN %i\n",SrvParms->houseintown); // Ripper
	fprintf(file, "SHOPRESTOCK %i\n",SrvParms->shoprestock); // Ripper
	fprintf(file, "HOUSEDECAY_SECS %lu\n",SrvParms->housedecay_secs);
	fprintf(file, "DEFAULT_JAIL_TIME %i\n",SrvParms->default_jail_time);	
	fprintf(file, "NPCPOPUPMENUES %i\n",SrvParms->npcpopmenues);	
	fprintf(file, "AOSMODE %i\n",SrvParms->AOS_Mode);	
	fprintf(file, "NEWMAGICTEST %i\n",SrvParms->newmagictest);	
	fprintf(file, "ENGRAVING %i\n",SrvParms->engraving);
	fprintf(file, "SMELTABLEITEMS %i\n",SrvParms->smeltableitems);
	fprintf(file, "DEFAULTGMVIEW %i\n",SrvParms->defaultgmview);
	fprintf(file, "IDLESERVERCHATTER %i\n",SrvParms->IDLESERVERCHATTER);
	SectionIT->SaveKeywords(file);
/*	fprintf(file, "STARTGOLD %i\n",SrvParms->startgold);
	fprintf(file, "STABLING_FEE %f\n",SrvParms->stablingfee);
	fprintf(file, "USESPECIALBANK %i\n",SrvParms->usespecialbank);	//AntiChrist - specialbank
	fprintf(file, "SPECIALBANKTRIGGER %s\n",SrvParms->specialbanktrigger);
	fprintf(file, "GOLDWEIGHT %f\n",SrvParms->goldweight);//AntiChrist - gold weight */
	fprintf(file, "}\n\n");

	ParmScript->SaveSections(file);

/*	fprintf(file, "SECTION RESOURCE\n");
	fprintf(file, "{\n");
	fprintf(file, "MINECHECK %i\n",SrvParms->minecheck);
	fprintf(file, "ORE_PER_AREA %u\n",resource.ore);
	fprintf(file, "ORE_RESPAWN_TIME %lu\n",resource.oretime);
	fprintf(file, "ORE_RESPAWN_RATE %lu\n",resource.orerate);
	fprintf(file, "ORE_RESPAWN_AREA %u\n",resource.orearea);
	fprintf(file, "ORE_TRIGGER %i\n",resource.miningtrigger);
	fprintf(file, "ORE_STAMINA %i\n",resource.miningstamina);
	fprintf(file, "LOGS_PER_AREA %i\n",resource.logs);
	fprintf(file, "LOG_RESPAWN_TIME %lu\n",resource.logtime);
	fprintf(file, "LOG_RESPAWN_RATE %lu\n",resource.lograte);
	fprintf(file, "LOG_RESPAWN_AREA %u\n",resource.logarea);
	fprintf(file, "LOG_TRIGGER %i\n",resource.logtrigger);
	fprintf(file, "LOG_STAMINA %i\n",resource.logstamina);
	fprintf(file, "}\n\n"); */
	
	fprintf(file, "SECTION REPSYS\n");
	fprintf(file, "{\n");
	fprintf(file, "MURDER_DECAY %li\n", repsys.murderdecay);
	fprintf(file, "MAXKILLS %i\n", repsys.maxkills);
	fprintf(file, "CRIMINAL_TIME %i\n", repsys.crimtime);
	fprintf(file, "}\n\n");
	
	fprintf(file, "SECTION TRACKING\n");
	fprintf(file, "{\n");
	fprintf(file, "BASE_TRACKING_RANGE %i\n",tracking_data.baserange);
	fprintf(file, "MAX_TRACKING_TARGETS %i\n",tracking_data.maxtargets);
	fprintf(file, "BASE_TRACKING_TIME %i\n",tracking_data.basetimer);
	fprintf(file, "TRACKING_MESSAGE_REDISPLAY_TIME %i\n",tracking_data.redisplaytime);
	fprintf(file, "}\n\n");
	
	fprintf(file, "SECTION BEGGING\n");
	fprintf(file, "{\n");

	fprintf(file, "BEGGING_RANGE %i\n",begging_data.range);
	fprintf(file, "BEGGING_TIME %i\n",begging_data.timer);
	fprintf(file, "BEGGING_TEXT0 %s\n",begging_data.text[0]);
	fprintf(file, "BEGGING_TEXT1 %s\n",begging_data.text[1]);
	fprintf(file, "BEGGING_TEXT2 %s\n",begging_data.text[2]);
	fprintf(file, "}\n\n");
	
	fprintf(file, "SECTION FISHING\n");
	fprintf(file, "{\n");
	fprintf(file, "BASE_FISHING_TIME %i\n",fishing_data.basetime);
	fprintf(file, "RANDOM_FISHING_TIME %i\n",fishing_data.randomtime);
	fprintf(file, "FISHING_LOOT_PROBABILITY %i\n",fishing_data.loot_probability);
	fprintf(file, "}\n\n");
	
	fprintf(file, "SECTION SPIRITSPEAK\n");
	fprintf(file, "{\n");
	fprintf(file, "SPIRITSPEAKTIMER %i\n",spiritspeak_data.spiritspeaktimer);
	fprintf(file, "}\n\n");
	
	fprintf(file, "SECTION TIME_LIGHT\n");
	fprintf(file, "{\n");
	fprintf(file, "DAY %i\n", timeLight.day);
	fprintf(file, "HOUR %i\n", timeLight.hour);
	fprintf(file, "MINUTE %i\n", timeLight.minute);
	fprintf(file, "AMPM %i\n", timeLight.ampm);
	fprintf(file, "MOON1UPDATE %i\n", timeLight.moon1update);
	fprintf(file, "MOON2UPDATE %i\n", timeLight.moon2update);
	fprintf(file, "MOON1 %i\n", timeLight.moon1);
	fprintf(file, "MOON2 %i\n", timeLight.moon2);
	fprintf(file, "DUNGEONLIGHTLEVEL %i\n", timeLight.dungeonlightlevel);
	fprintf(file, "WORLDFIXEDLEVEL %i\n", timeLight.worldfixedlevel);
	fprintf(file, "WORLDCURLEVEL %i\n", timeLight.worldcurlevel);
	fprintf(file, "WORLDBRIGHTLEVEL %i\n", timeLight.worldbrightlevel);
	fprintf(file, "WORLDDARKLEVEL %i\n", timeLight.worlddarklevel);
	fprintf(file, "SECONDSPERUOMINUTE %i\n", timeLight.secondsperuominute); // lb
	fprintf(file, "SEASON %i\n", timeLight.season); // lb	
	fprintf(file, "INLORPOTIONFIXEDTIME %i\n", timeLight.fixedInlorPotionTime); // lb
	fprintf(file, "INLORPOTIONVARTIME %i\n", timeLight.varInlorPotionTime); // lb
	fprintf(file, "INLORSPELLFIXEDTIME %i\n", timeLight.fixedInlorSpellTime); // lb
	fprintf(file, "INLORSPELLVARTIME %i\n", timeLight.varInlorSpellTime); // lb
	fprintf(file, "}\n\n");	

	fprintf(file, "EOF\n\n");
	fclose(file);
	file = NULL;
}

