/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

//////////////////////////////////////////////////////////////////////
// TmpEff.cpp: implementation of temporary effects
//				cut from Wolfpack.cpp by Duke, 25.10.2000


#include "TmpEff.h"

#include "Client.h"
#include "magic.h"
#include "regions.h"
#include "skills.h"
#include "SndPkg.h"
//#include "targeting.h"

#include "wolfpack.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "TmpEff.cpp"


/////////////
// Name:	reverseIncognito
// Purpose:	undo the effects of the incognito spell
// History: by AntiChrist
//			isolated from two functions by Duke, 10.6.2001
//
static void reverseIncognito(P_CHAR pc)
{
	if (pc->incognito)//let's ensure it's under incognito effect!
	{
		pc->setId(pc->xid());	// SEX
		pc->checkEquipment(); // also checks for males wearing female armor now

		pc->skin = pc->xskin;	// SKIN COLOR
		
		strcpy(pc->name,pc->orgname);	// NAME
		
		if (pc->hairserial>-1)//if hairs exist, restore hair style/color
		{
			P_ITEM pHair = FindItemBySerial(pc->hairserial);
			if (pHair)
			{
				if (pHair->incognito) //let's ensure it was marked as under incognito effect
				{
					//stores old hair values
					pHair->setColor(pHair->getMoreB1_2());
					pHair->setId(pHair->getMoreB3_4());
					pHair->incognito=false;
				}
			}
		}
		
		if (pc->beardserial>-1)//if beard exists, restore beard style/color
		{
			P_ITEM pBeard = FindItemBySerial(pc->beardserial);
			if (pBeard)
			{
				if (pBeard->incognito) //let's ensure it was marked as under incognito effect
				{
					//restores old beard values
					pBeard->setColor(pBeard->getMoreB1_2());
					pBeard->setId(pBeard->getMoreB3_4());
					pBeard->incognito=false;
				}
			}
		}
		
		//only refresh once, when poly finished
		updatechar(pc);
		int socket=pc->calcSocket();//calculate only once
		if (socket!=-1)
		{
			unsigned int j;
			for (j=0;j<now;j++)
			{//and to all inrange sockets
				if (perm[j] && inrange1pl(pc, currchar(j)))
					wornitems(j, pc);
			}
		}
		pc->incognito=false;//AntiChrist
	}
}

void reversePolymorph(P_CHAR pc)
{
	if (pc->polymorph) // be shure that the char is polymorphed
	{
		pc->setId(pc->xid());

		pc->checkEquipment(); // also checks for males wearing female armor now

		pc->polymorph=false;
		updatechar(pc);
	}
}

void cTmpEff::Init(int number)
{
	sourSer = -1;
	destSer = -1;
	expiretime=0;
	num=number;
	more1=255;
	more2=255;
	more3=255;
	dispellable=0;
}

int cTmpEff::getDest()			{ return destSer;}
void cTmpEff::setDest(int ser)	{ destSer=ser;}
int cTmpEff::getSour()			{ return sourSer;}
void cTmpEff::setSour(int ser)	{ sourSer=ser;}

void cTmpEff::Prolong(int seconds)
{
	cTmpEff *TE=new cTmpEff();
	TE->sourSer = this->sourSer;
	TE->destSer = this->destSer;
	TE->setExpiretime_s(seconds);
	TE->num = this->num;
	TE->more1 = this->more1;
	TE->more2 = this->more2;
	TE->more3 = this->more3;
	TE->dispellable = this->dispellable;
	AllTmpEff->Insert(TE);
}

void cTmpEff::setExpiretime_s(int seconds)
{
	expiretime=uiCurrentTime+(seconds*MY_CLOCKS_PER_SEC);
}

void cTmpEff::On(P_CHAR pc)
{
	if (!pc)
		return;
	
	switch(this->num)
	{
	case TEF_Paralyze:	    pc->Freeze();		break;
	case TEF_NightSight:	pc->fixedlight=timeLight.worldbrightlevel;break;
	case TEF_Clumsy:	    pc->chgDex(-1 * this->more1);	break;
	case TEF_FeebleMind:	pc->chgInt(-1 * this->more1);	break;
	case TEF_Weaken:	    pc->chgStr(-1 * this->more1);	break;
	case TEF_Agility:	    pc->chgDex(this->more1);		break;
	case TEF_Cunning:	    pc->chgInt(this->more1);		break;
	case TEF_Strength:	    pc->chgStr(this->more1);		break;
	case TEF_Bless:
		pc->chgStr(this->more1);
		pc->chgDex(this->more2);
		pc->chgInt(this->more3);
		break;
	case TEF_Curse:
		pc->chgStr(-1 * this->more1);
		pc->chgDex(-1 * this->more2);
		pc->chgInt(-1 * this->more3);
		break;
	}
}

void cTmpEff::Off(P_CHAR pc)
{
	if (!pc)
		return;

	switch(this->num)
	{
	case TEF_Paralyze:	    pc->Unfreeze();			 break;
	case TEF_NightSight:	pc->fixedlight='\xFF';	 break;
	case TEF_Clumsy:	    pc->chgDex(this->more1); break;
	case TEF_FeebleMind:	pc->chgInt(this->more1); break;
	case TEF_Weaken:	    pc->chgStr(this->more1); break;
	case TEF_Agility:	    pc->chgDex(-1 * this->more1);break;
	case TEF_Cunning:	    pc->chgInt(-1 * this->more1);break;
	case TEF_Strength:	    pc->chgStr(-1 * this->more1);break;
	case TEF_Bless:
		pc->chgStr(-1 * this->more1);
		pc->chgDex(-1 * this->more2);
		pc->chgInt(-1 * this->more3);
		break;
	case TEF_Curse:
		pc->chgStr(this->more1);
		pc->chgDex(this->more2);
		pc->chgInt(this->more3);
		break;
	}
}

void cTmpEff::Reverse()
{
	P_CHAR pc = FindCharBySerial(getDest());
	if (!pc) return;

	switch(num)
	{
	case TEF_Paralyze:	    pc->Unfreeze();	break;
	case TEF_NightSight:	pc->fixedlight='\xFF';break;
	case TEF_Clumsy:	    pc->chgDex(more1);	break;
	case TEF_FeebleMind:	pc->chgInt(more1);	break;
	case TEF_Weaken:	    pc->chgStr(more1);break;
	case TEF_Agility:	    pc->chgDex(-1 * more1);break;
	case TEF_Cunning:	    pc->chgInt(-1 * more1);break;
	case TEF_Strength:	    pc->chgStr(-1 * more1);break;
	case TEF_Bless:
		pc->chgStr(-1 * more1);
		pc->chgDex(-1 * more2);
		pc->chgInt(-1 * more3);
		break;
	case TEF_Curse:
		pc->chgStr(more1);
		pc->chgDex(more2);
		pc->chgInt(more3);
		break;
	case TEF_Polymorph: //Polymorph spell by AntiChrist
		reversePolymorph(pc);
		break;
	case TEF_Incognito: //Incognito spell by AntiChrist
		reverseIncognito(pc);
		break;

	case TEF_Protection:
		int toDrop;
		toDrop = more1; //Effect->more1;
		if ( ( pc->getRealSkill(PARRYING) - toDrop ) < 0 )
			pc->setRealSkill(PARRYING,0);
		else
			pc->chgRealSkill(PARRYING, -toDrop);
		break;
		
	default:
		LogError("Fallout of switch");
		return;
	}//switch
	
	if (pc)
		pc->checkEquipment(); //AntiChrist - checks equipments for stats requirements
}

void cTmpEff::Expire()
{
	bool DestIsMobile = true; // not all effects have a npc/player as destination. (no destination or item)
	
	
	switch (num) // effects whose primary(!) target is no player/npc !
	{
	  case TEF_Mortar: 
	  case TEF_TDummy:	   
	  case TEF_Explode:
		DestIsMobile=false;
	}
	  
	P_CHAR pc_s = FindCharBySerial(getDest());
	if (!pc_s && DestIsMobile) 
	{
		char temp[1024];
		sprintf(temp,"Error 0815 in TmpEffect::Expire, effn:%i dest:%i",num, getDest());
		LogError(temp);
		return;
	}

	UOXSOCKET sk;
	if (pc_s) sk=pc_s->calcSocket(); else sk=-1;

	switch(num)
	{

	case TEF_Paralyze:
		if (pc_s->IsFrozen())
		{
			pc_s->Unfreeze();
			if (sk!=-1) sysmessage(sk, "You are no longer frozen.");
			OldMagic->afterParticles(38, pc_s); 			
		}
		break;

	case TEF_NightSight:
		pc_s->fixedlight='\xFF';
		dolight(sk);		
		break;

	case TEF_Clumsy:
		pc_s->chgDex(more1);
		statwindow(sk, pc_s);
		break;

	case TEF_FeebleMind:
		pc_s->chgInt(more1);
		statwindow(sk, pc_s);
		break;

	case TEF_Weaken:
		pc_s->chgStr(more1);
		statwindow(sk, pc_s);
		break;

	case TEF_Agility:
		pc_s->chgDex(-1 * more1);
		pc_s->setStm(min(pc_s->Stm(), (int)pc_s->effDex()));
		statwindow(sk, pc_s);
		break;

	case TEF_Cunning:
		pc_s->chgInt(-1 * more1);
		pc_s->mn=min(pc_s->mn, (int)pc_s->effInt());
		statwindow(sk, pc_s);
		break;

	case TEF_Strength:
		pc_s->chgStr(-1 * more1);
		pc_s->setHp(min(pc_s->Hp(), static_cast<int>(pc_s->effStr())));
		statwindow(sk, pc_s);
		break;

	case TEF_Grinding:
		if (more1 == 0)
		{
			if (more2!=0)
			{
				sprintf(temp, "*%s continues grinding.*", pc_s->name);
				npcemoteall(pc_s, temp,0);
			}
			soundeffect2(pc_s, 0x0242);
		}
		break;

	case TEF_Mortar:
		P_ITEM Mortar;
		Mortar=FindItemBySerial(getDest());
		if (Mortar)
		{
			P_CHAR pc = FindCharBySerial(getSour());
			if (pc!=NULL) Skills->CreatePotion(pc, more1, more2, Mortar);
		}
		break;

	case TEF_Bless:
		pc_s->chgStr(-1 * more1);
		pc_s->setHp(min(pc_s->Hp(), static_cast<int>(pc_s->effStr())));
		pc_s->chgDex(-1 * more2);
		pc_s->setStm(min(pc_s->Stm(), (int)pc_s->effDex()));
		pc_s->chgInt(-1 * more3);
		pc_s->mn=min(pc_s->mn, (int)pc_s->effInt());
		statwindow(sk, pc_s);
		break;

	case TEF_Curse:
		pc_s->chgStr(more1);
		pc_s->chgDex(more2);
		pc_s->chgInt(more3);
		statwindow(sk, pc_s);
		break;

	case TEF_TDummy: //- training dummies Tauriel check to see if item moved or not before searching for it
		{
			P_ITEM pTrainDummy = FindItemBySerial(getDest());
			if (pTrainDummy)
			{
				if (pTrainDummy->id()==0x1071)
				{
					pTrainDummy->setId(0x1070);
					pTrainDummy->gatetime=0;
					pTrainDummy->refresh();
				}
				else if (pTrainDummy->id()==0x1075)
				{
					pTrainDummy->setId(0x1074);
					pTrainDummy->gatetime=0;
					pTrainDummy->refresh();
				}
			}
		}
		break;

	case TEF_ReactiveArmor: //reactive armor
		pc_s->ra=0;
		break;

	case TEF_ExplMsg: //Explosion potion messages	Tauriel
		sprintf(temp, "%i", more3);
		sysmessage(sk, temp); // crashfix, LB
		break;

	case TEF_Explode: //Explosion potion explosion	Tauriel			
		P_CHAR pc;
		P_ITEM it;
		it=FindItemBySerial(getDest());
		pc = FindCharBySerial(getSour());;
		explodeitem(pc, it); //explode this item
		break;

	case TEF_Polymorph: //Polymorph spell by AntiChrist 9/99
		if (pc_s->polymorph)//let's ensure it's under polymorph effect!
		{
			pc_s->setId(pc_s->xid());
			pc_s->polymorph=false;
			updatechar(pc_s);
		}
		break;

	case TEF_Incognito: //Incognito spell by AntiChrist 12/99
		reverseIncognito(pc_s);
		break;
		
	case TEF_LSD: // LSD potions, LB 5'th nov 1999
		{
		if (sk==-1) return;
		LSD[sk]=0;
		sysmessage(sk,"LSD has worn off");
		pc_s->setStm(3); // stamina near 0
		pc_s->mn=3;
		pc_s->setHp(pc_s->Hp()/7);
		impowncreate(sk,pc_s,0);
		all_items(sk); // absolutely necassairy here !!!

		cChar* pc;
		int loopexit=0;
		RegGrid3x3Iterator4Chars rg3ic(pc_s->pos);
		while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
		{
			if (pc_s->dist(pc)<15 && ( pc->InGame() || pc->npc ) ) updatechar(pc);
		}
		}
		break;
		
	case TEF_Protection:
		int toDrop;
		toDrop = more1; //Effect->more1;
		if ( ( pc_s->getRealSkill(PARRYING) - toDrop ) < 0 )
			pc_s->setRealSkill(PARRYING,0);
		else
			pc_s->chgRealSkill(PARRYING, -toDrop);
		break;
		
	case TEF_HideGM: // delayed hiding for gms after flamestrike effect
		sysmessage(sk,"You have hidden yourself well.");
		pc_s->hide();
		SndRemoveNpcToAll(pc_s);
		updatechar(pc_s);
		break;
		
	case TEF_UnHideGM: // delayed unhide for gms
		// Changed to be uniform with delayed hideing  (Aldur)
		sysmessage(sk, "You are now visible."); 
		pc_s->hidden = 0; 
		updatechar(pc_s); 
		break;
		
	case TEF_HealMsg: //heals some pf - solarin
		pc_s->increaseHp(more1);
		if (!more2)		// starting ?
			tempeffect(pc_s,TEF_HealMsg,more1+1,1,more3,0);
		else
			sysmessage(pc_s->calcSocket(), "You finished healing.");
		break;

	case TEF_Camping: // camp secure message
		{
		sysmessage(pc_s->calcSocket(), "Your camp is now secure.");

		PC_ITEM pj;
		int loopexit=0;
		RegGrid3x3Iterator4Items rg3ii(pc_s->pos);
		while ((pj = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
		{
			if (pj->isType(ITY_Campfire) && pc_s->dist(pj) < 3)
			{
				pj->unconst()->more1=1;
			}
		}
		}
		break;

	case TEF_Doppelgaenger: // doppelgaenger effect
			
		Npcs->TempDupeNpcRevert(pc_s);
		break;
		
	default:
		LogErrorVar("Fallout of switch (num = %i).", num);
		break;
	}

	if (pc_s) pc_s->checkEquipment(); //AntiChrist - checks equipments for stats requirements
	// note: if sourcePCrequ false it would crash without the  if (pc_s) ...
}

class cTmpEffSheep : public cTmpEff	// return naked sheep to 'woolish'
{
public:
	cTmpEffSheep(P_CHAR Sheep)
	{
		Init(TEF_Sheep);
		setDest(Sheep->serial);
		setExpiretime_s(3600);
		On(Sheep);
		updatechar(Sheep);
	}
	virtual void Expire()
	{
		P_CHAR Sheep = FindCharBySerial(getDest());
		if (Sheep)
		{
			Off(Sheep);
			updatechar(Sheep);
		}
	}
	virtual void On(P_CHAR Sheep)	{Sheep->setId(0xDF);}
	virtual void Off(P_CHAR Sheep)	{Sheep->setId(0xCF);}
};

class cTmpEffPlant : public cTmpEff	// make a plant grow
{
public:
	cTmpEffPlant(P_ITEM Plant,unsigned char adv)
	{
		Init(TEF_Plant);
		setDest(Plant->serial);
		if (adv)					// change appearance of the plant (after harvest) ?
		{
			Advance(Plant);
			Plant->refresh();
		}
		setExpiretime_s(3600);		// let it grow
	}
	virtual void Expire()
	{
		P_ITEM Plant = FindItemBySerial(getDest());
		if (Plant)
		{
			Advance(Plant);
			Plant->refresh();
		}
	}
private:
	virtual void Advance(P_ITEM Plant)
	{
		bool prolong = false;
		short id = Plant->id();
		switch (id)
		{
		case 0x0C4F:	Plant->setId(0x0C51);					break;	// ripe cotton
		case 0x0C50:	Plant->setId(0x0C52);					break;	// ripe cotton
		case 0x0C51:	Plant->setId(0x0C53);	prolong=true;	break;
		case 0x0C52:	Plant->setId(0x0C54);	prolong=true;	break;
		case 0x0C53:	Plant->setId(0x0C4F);					break;
		case 0x0C54:	Plant->setId(0x0C50);					break;

		case 0x0C5B:	Plant->setId(0x1449);					break;	// ripe wheat
		case 0x1449:	Plant->setId(0x1EBF);	prolong=true;	break;
		case 0x1EBF:	Plant->setId(0x1EBE);	prolong=true;	break;
		case 0x1EBE:	Plant->setId(0x0DAE);	prolong=true;	break;
		case 0x0DAE:	Plant->setId(0x0DAF);	prolong=true;	break;
		case 0x0DAF:	Plant->setId(0x0C55);	prolong=true;	break;
		case 0x0C55:	Plant->setId(0x0C56);	prolong=true;	break;
		case 0x0C56:	Plant->setId(0x0C57);	prolong=true;	break;
		case 0x0C57:	Plant->setId(0x0C59);	prolong=true;	break;
		case 0x0C59:	Plant->setId(0x0C58);	prolong=true;	break;	// this is NOT a typo! looks better :)
		case 0x0C58:	Plant->setId(0x0C5A);	prolong=true;	break;
		case 0x0C5A:	Plant->setId(0x0C5B);					break;

		default:	LogWarningVar("unknown plant id <%x>",id);
		}
		if (prolong)
			tempeffect(Plant, TEF_Plant);	// let it grow
	}
};

// this is used to animate the flour mill
class cTmpEffMill : public cTmpEff	// flour mill
{
public:
	cTmpEffMill(P_ITEM MillIn, int dir, int phase)
	{
		Init(TEF_Mill);
		setDest(MillIn->serial);
		this->more1=dir;	// direction of mill; 1==WE 2==NS
		P_ITEM MillOut = FindPartTwo(MillIn);
		if (!MillOut)
		{
			LogWarning("incomplete mill!");
			return;
		}
		setExpiretime_s(5);
		this->more2=phase;	// phase one of this effect; fill in and start to move
		if (phase == 1)
		{
			if (dir==1)
			{
				MillIn->setId(0x1921);
				MillOut->setId(0x1926);
			}
			else
			{
				MillIn->setId(0x192D);
				MillOut->setId(0x1932);
			}
			MillOut->refresh();
		}
		else	// this is phase two
		{
			MillIn->setId(dir==1 ? 0x1925 : 0x1931);	// input is half full
		}
		MillIn->refresh();
	}
	virtual void Expire()
	{
		P_ITEM MillIn = FindItemBySerial(getDest());
		if (!MillIn) return;
		P_ITEM MillOut = FindPartTwo(MillIn);
		if (!MillOut) return;
		if (this->more2==1)		// end of first phase of the effect
		{
			tempeffect(MillIn, TEF_Mill, more1, 2);	// start second phase
		}
		else
		{
			if (this->more1==1)		// W/E mill
			{
				MillIn->setId(0x1920);	// input is empty again
				MillOut->setId(0x1923);	// output is full and calm
			}
			else
			{
				MillIn->setId(0x192C);	// input is empty again
				MillOut->setId(0x192F);	// output is full and calm
			}
			MillIn->refresh();
			MillOut->refresh();
		}
	}
private:
	P_ITEM FindPartTwo(P_ITEM MillIn)	// get the other part of the mill
	{
		short id = MillIn->id();
		cCoord Opos(MillIn->pos);		// pos of other part
		if (id==0x1920 || id==0x1921 || id==0x1925)
			Opos.Add(1);
		else
			Opos.Add(0,1);

		P_ITEM pi;
		int loopexit=0;
		RegCellIterator4Items rcii(Opos);
		while((pi = rcii.Next()) != NULL && (++loopexit < MAXLOOPS) )
		{
			if (!pi->pos.IsSameXY(Opos)) continue;
			short id = pi->id();
			if (this->more1==1 && (id==0x1922 || id==0x1923 || id==0x1926))
				return pi;
			if (this->more1==2 && (id==0x192E || id==0x192F || id==0x1932))
				return pi;
		}
		return NULL;
	}
};

class cTmpEffDoor : public cTmpEff	// former tempeff 13
{
public:
	cTmpEffDoor(int num2, P_ITEM Door)
	{
		Init(num2);
		setDest(Door->serial);
		if (Door->dooropen)
		{
			Door->dooropen=0;
			return;
		}
		setExpiretime_s(10);
		Door->dooropen=1;
	}
	virtual void Expire()
	{
		P_ITEM Door = FindItemBySerial(getDest());
		if (Door)
		{
			if (Door->dooropen==0)
				return;				// they must have closed it manually
			if (DoorIsBlockedByChar(Door))
			{
				Door->dooropen=0;	// must be closed to apply another tempeffect	
				tempeffect(Door, TEF_Door);	// wait some more time
				return;
			}
			else
				dooruse(-1, Door);
		}
	}
};

class cTmpEffTaming : public cTmpEff
{
public:
	cTmpEffTaming(P_CHAR Tamer, P_CHAR Pet, int rounds, int secs)
	{
		if (!Tamer || !Pet)	return;
		Init(TEF_Taming);
		setSour(Tamer->serial);
		setDest(Pet->serial);
		more1=rounds;
		more2=secs;
		setExpiretime_s(secs);
		Skills->TameTargetPart2(Tamer,Pet);					// issue taming message
	}
	virtual void Expire()
	{
		P_CHAR Tamer = FindCharBySerial(getSour());
		P_CHAR Pet = FindCharBySerial(getDest());
		if (!Tamer || !Pet)	return;

		more1--;
		if (more1 > 0)
		{
			tempeffect(Tamer, Pet, TEF_Taming, more1,more2);	// wait some more time
			return;
		}
		else
			Skills->TameTargetPart3(Tamer,Pet);
	}
};

void cAllTmpEff::Off()
{
	unsigned int i;
	for (i=0;i<teffectcount;i++)
	{
		cTmpEff *pTE = teffects[i];
		P_CHAR pc = FindCharBySerial(pTE->getDest());
		if (pc)
			pTE->Off(pc);
	}
}

void cAllTmpEff::On()
{
	unsigned int i;
	for (i=0;i<teffectcount;i++)
	{
		cTmpEff *pTE = teffects[i];
		P_CHAR pc = FindCharBySerial(pTE->getDest());
		if (pc)
			pTE->On(pc);
	}
}

void cAllTmpEff::ExpireOnDeath(P_CHAR pc)
{
	unsigned int i;
	for(i=0;i<teffectcount;i++)
	{
		cTmpEff *pTE = teffects[i];
		if (pTE->getDest() == pc->serial)
		{
			pTE->Expire();
			AllTmpEff->Remove(i);
			i--;
		}
	}
}

void cAllTmpEff::Check()
{
	unsigned int i;
	for(i=0;i<teffectcount;i++)
	{
		cTmpEff *pTE = teffects[i];
		if (pTE->expiretime <= uiCurrentTime)
		{
			pTE->Expire();
			AllTmpEff->Remove(i);
			i--;
		}
	}
}

bool cAllTmpEff::Add(P_CHAR pc_source, P_CHAR pc_dest, int num, unsigned char more1, unsigned char more2, unsigned char more3, short dur)
{
	if (!pc_source || !pc_dest) return 0;
	unsigned int ic; // antichrist' changes
	int color; //used for incognito spell
	int	loopexit=0;

	if (teffectcount >= (unsigned) (Npcs->GetCapacity()*5) )
		return 0;

	cTmpEff *pTE;
	for (ic=0;ic<teffectcount;ic++)	// If there is already an effect of the same or similar kind, reverse it first (Duke)
	{
		pTE = teffects[ic];
		if (pTE->getDest() == pc_dest->serial)
		{
			if ((pTE->num==3 && num==3)||
				(pTE->num==4 && num==4)||
				(pTE->num==5 && num==5)||
				(pTE->num==6 && num==6)||
				(pTE->num==7 && num==7)||
				(pTE->num==8 && num==8)||
				(pTE->num==11&& num==11)||
				(pTE->num==12&& num==12)||
				(pTE->num==18&& num==18)|| //added Poly reverse - AntiChrist (9/99)
				(pTE->num==21&& num==21)||
				(pTE->num==19&& num==19)|| //added Incognito reverse - AntiChrist (12/99)
				(pTE->num==18&& num==19)|| //reverse poly effect if we have to use incognito - AntiChrist (12/99)
				(pTE->num==19&& num==18) )  //reverse incognito effect if we have to use poly - AntiChrist (12/99)
			{
				pTE->Reverse();
				AllTmpEff->Remove(ic);
			}
		}
	}

	pTE=new cTmpEff;
	pTE->Init(num);
	pTE->setSour(pc_source->serial);
	pTE->setDest(pc_dest->serial);

	switch (num)
	{
	case TEF_Paralyze:
		pc_dest->Freeze();
		if (dur > 0)		// if a duration is given (fields), use that
			pTE->setExpiretime_s(dur);
		else
			pTE->setExpiretime_s(pc_source->Skill(MAGERY)/100);
		pTE->more1=0;
		pTE->more2=0;
		pTE->dispellable=1;
		break;

	case TEF_NightSight:	// night sight
		pc_dest->fixedlight=timeLight.worldbrightlevel;
		dolight(pc_dest->calcSocket());
		OldMagic->afterParticles(6, pc_dest); // shows particles for UO:3D clients, like On OSI servers

		if (dur > 0)		// if a duration is given (potions), use that (Duke, 30.12.2000)
			 pTE->setExpiretime_s(dur);
		else pTE->setExpiretime_s(10);

		pTE->more1=0;
		pTE->more2=0;
		pTE->dispellable=1;
		break;

	case TEF_Clumsy:
		if (pc_dest->effDex()<more1)
			more1=pc_dest->effDex();
		pc_dest->chgDex(-1 * more1);
		pc_dest->setStm(min(pc_dest->Stm(), (int)pc_dest->effDex()));
		statwindow(pc_dest->calcSocket(), pc_dest);
		pTE->setExpiretime_s(pc_source->Skill(MAGERY)/10);
		pTE->more1=more1;
		pTE->more2=0;
		pTE->dispellable=1;
		break;

	case TEF_FeebleMind:
		if (pc_dest->effInt()<more1)
			more1=pc_dest->effInt();
		pc_dest->chgInt(-1 * more1);
		pc_dest->mn=min(pc_dest->mn, (int)pc_dest->effInt());
		statwindow(pc_dest->calcSocket(), pc_dest);
		pTE->setExpiretime_s(pc_source->Skill(MAGERY)/10);
		pTE->more1=more1;
		pTE->more2=0;
		pTE->dispellable=1;
		break;
	case TEF_Weaken:
		if (pc_dest->effStr()<more1)
			more1=pc_dest->effStr();
		pc_dest->chgStr(-1 * more1);
		pc_dest->setHp(min(pc_dest->Hp(), static_cast<int>(pc_dest->effStr())));
		statwindow(pc_dest->calcSocket(), pc_dest);
		pTE->setExpiretime_s(pc_source->Skill(MAGERY)/10);
		pTE->more1=more1;
		pTE->more2=0;
		pTE->dispellable=1;
		break;

	case TEF_Agility:
		if (pc_dest->effDex()+more1>250)
			more1=250-pc_dest->effDex();
		pc_dest->chgDex(more1);
		statwindow(pc_dest->calcSocket(), pc_dest);
		if (dur > 0)		// if a duration is given (potions), use that (Duke, 31.10.2000)
			pTE->setExpiretime_s(dur);
		else
			pTE->setExpiretime_s(pc_source->Skill(MAGERY)/10);
		pTE->more1=more1;
		pTE->more2=0;
		pTE->dispellable=1;
		break;

	case TEF_Cunning:
		if (pc_dest->effInt()+more1>255)
			more1=pc_dest->effInt()-255;
		pc_dest->chgInt(more1);
		statwindow(pc_dest->calcSocket(), pc_dest);
		pTE->setExpiretime_s(pc_source->Skill(MAGERY)/10);
		pTE->more1=more1;
		pTE->more2=0;
		pTE->dispellable=1;
		break;

	case TEF_Strength:
		if (pc_dest->effStr()+more1>255)
			more1=pc_dest->effStr()-255;
		pc_dest->chgStr(more1);
		statwindow(pc_dest->calcSocket(), pc_dest);
		if (dur > 0)		// if a duration is given (potions), use that (Duke, 31.10.2000)
			pTE->setExpiretime_s(dur);
		else			// else use caster's skill
			pTE->setExpiretime_s(pc_source->Skill(MAGERY)/10);
		pTE->more1=more1;
		pTE->more2=0;
		pTE->dispellable=1;
		break;

	case TEF_Grinding:
		pTE->setExpiretime_s(more2);
		pTE->more1=more1;
		pTE->more2=more2;
		break;

	case TEF_Mortar:
		pTE->setExpiretime_s(12);
		pTE->more1=more1;
		pTE->more2=more2;
		break;

	case TEF_Bless: // Bless
		if (pc_dest->effStr()+more1>255)
			more1=pc_dest->effStr()-255;
		if (pc_dest->effDex()+more2>250)
			more2=250-pc_dest->effDex();
		if (pc_dest->effInt()+more3>255)
			more3=pc_dest->effInt()-255;
		pc_dest->chgStr(more1);
		pc_dest->chgDex(more2);
		pc_dest->chgInt(more3);
		statwindow(pc_dest->calcSocket(), pc_dest);
		pTE->setExpiretime_s(pc_source->Skill(MAGERY)/10);
		pTE->more1=more1;
		pTE->more2=more2;
		pTE->more3=more3;
		pTE->dispellable=1;
		break;

	case TEF_Curse:
		if (pc_dest->effStr()<more1)
			more1=pc_dest->effStr();
		if (pc_dest->effDex()<more2)
			more2=pc_dest->effDex();
		if (pc_dest->effInt()<more3)
			more3=pc_dest->effInt();
		pc_dest->chgStr(-1 * more1);
		pc_dest->chgDex(-1 * more2);
		pc_dest->chgInt(-1 * more3);
		statwindow(pc_dest->calcSocket(), pc_dest);
		pTE->setExpiretime_s(pc_source->Skill(MAGERY)/10);
		pTE->more1=more1;
		pTE->more2=more2;
		pTE->more3=more3;
		pTE->dispellable=1;
		break;

	case TEF_ReactiveArmor: // Reactive armor
		pTE->setExpiretime_s(pc_source->Skill(MAGERY)/10);
		pTE->dispellable=1;
		break;

	case TEF_ExplMsg: //Explosion potions	Tauriel
		pTE->setExpiretime_s(more2);
		pTE->more1=more1; //item/potion
		pTE->more2=more2; //seconds
		pTE->more3=more3; //countdown#
		break;

	case TEF_Polymorph: //Polymorph - AntiChrist 09/99
		pTE->setExpiretime_s(polyduration);

		int k;
		//Grey flag when polymorphed - AntiChrist (9/99)
		pc_dest->crimflag=(polyduration*MY_CLOCKS_PER_SEC)+uiCurrentTime;
		if (pc_dest->onhorse) unmounthorse(pc_dest->calcSocket());
		k=(more1<<8)+more2;

		pc_dest->setXid(pc_dest->id());//let's backup previous id

		if (k>=0x000 && k<=0x3e1) // lord binary, body-values >0x3e crash the client
		{
			pc_dest->setId(k); // allow only non crashing ones

			unsigned short color = pc_dest->skin; // transparency for monsters allowed, not for players,
			// if polymorphing from monster to player we have to switch from transparent to semi-transparent
			if ((color&0x4000) && (k >=0x0190 && k<=0x03e1))
			{
				if (color!=0x8000)
				{
					pc_dest->skin = pc_dest->xskin = 0xF000;
				}
			}
		}
		pc_dest->polymorph=true;
		break;

	case TEF_Incognito://incognito spell - AntiChrist (10/99)//revised by AntiChrist - 9/12/99
		pTE->setExpiretime_s(90);

		//AntiChrist 11/11/99
		//If char is already under polymorph effect, let's reverse the
		//polymorph effect to avoid problems
		if (pc_dest->polymorph)
		{
			pc_dest->setId(pc_dest->xid());
			pc_dest->polymorph=false;
			updatechar(pc_dest);
		}

		//first: let's search for beard and hair serial
		//(we could use alredy saved serials...but it's better
		//to recalculate them)
		pc_dest->hairserial=-1;
		pc_dest->beardserial=-1;

		int ci;
		ci=0;
		loopexit=0;
		P_ITEM pi;
		while ( ((pi=pc_dest->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
		{
			if (pi->layer==0x10)//beard
				pc_dest->beardserial=pi->serial;
			if (pi->layer==0x0B)//hairs
				pc_dest->hairserial=pi->serial;
		}

		// ------ SEX ------
		pc_dest->setXid(pc_dest->id());
		pc_dest->setId(0x0190);
		// only if we don't already have a beard we can also turn to female
		if (pc_dest->beardserial == -1 && rand()%2)
			pc_dest->setId(0x0191);

		// --- SKINCOLOR ---
		pc_dest->xskin = pc_dest->skin;
		color=rand()%6;
		switch(color)
		{
			case 0:				pc_dest->skin = 0x83EA;				break;
			case 1:				pc_dest->skin = 0x8405;				break;
			case 2:				pc_dest->skin = 0x83EF;				break;
			case 3:				pc_dest->skin = 0x83F5;				break;
			case 4:				pc_dest->skin = 0x841C;				break;
			case 5:				pc_dest->skin = 0x83FB;				break;
			default:												break;
		}

		// ------ NAME -----
		strcpy(pc_dest->orgname,pc_dest->name);

		if (pc_dest->id()==0x0190) setrandomname(pc_dest,"1");//get a name from male list
		else setrandomname(pc_dest,"2");//get a name from female list

		//
		//damn..this formula seems to include also some bad color...
		//i'll test this later
		//AntiChrist
		//
		//use unique color for hair&beard
		//color=0x044E+(rand()%(0x04AD-0x044E));

		//i had to track down some valid value
		//for hair/beard colors, cause that
		//formula contained some bad value =(
		//but now it works perfectly :)
		//AntiChrist-11/11/99
		color=rand()%8;
		switch(color)
		{
			case 0:	color=0x044e;	break;
			case 1:	color=0x0455;	break;
			case 2:	color=0x045e;	break;
			case 3:	color=0x0466;	break;
			case 4:	color=0x046a;	break;
			case 5:	color=0x0476;	break;
			case 6:	color=0x0473;	break;
			case 7:	color=0x047c;	break;
			default://it should not go here...but..who nows =P
				color=0x044e;
		}

		// ------ HAIR -----
		if (pc_dest->hairserial>-1)//if hairs exist
		{//change hair style/color
			P_ITEM Hair = FindItemBySerial(pc_dest->hairserial);
			if (Hair)
			{
				//stores old hair values...
				Hair->setMoreB1_2(Hair->color());
				Hair->setMoreB3_4(Hair->id());

				//and change them with random ones
				short h;
				switch(rand()%10)
				{
					case 0: h = 0x203B; break;
					case 1: h = 0x203C; break;
					case 2:	h = 0x203D; break;
					case 3:	h = 0x2044; break;
					case 4:	h = 0x2045; break;
					case 5:	h = 0x2046; break;
					case 6:	h = 0x2047; break;
					case 7:	h = 0x2048; break;
					case 8:	h = 0x2049; break;
					case 9:	h = 0x204A; break;
					default:h = 0x204A; break;
				}
				Hair->setId(h);

				//random color
				Hair->setColor(color);
				//let's check for invalid values
				if (Hair->color() < 0x044E || Hair->color() > 0x04AD)
				{
					Hair->setColor(0x044E);
				}

				Hair->incognito=true;//AntiChrist
			}//if j!=-1
		}//if hairserial!=-1


		// -------- BEARD --------
		if (pc_dest->id()==0x0190)// only if a man
		if (pc_dest->beardserial>-1)//if beard exist
		{//change beard style/color
			P_ITEM Beard = FindItemBySerial(pc_dest->beardserial);
			if (Beard)
			{
				//clConsole.send("BEARD FOUND!!\n");
				//stores old beard values
				Beard->setMoreB1_2(Beard->color());
				Beard->setMoreB3_4(Beard->id());

				//changes them with random ones
				short b;
				switch(rand()%7)
				{
					case 0: b = 0x203E; break;
					case 1: b = 0x203F; break;
					case 2:	b = 0x2040; break;
					case 3:	b = 0x2041; break;
					case 4:	b = 0x204B; break;
					case 5:	b = 0x204C; break;
					case 6:	b = 0x204D; break;
					default:b = 0x204D; break;
				}
				Beard->setId(b);

				//random color
				Beard->setColor(color);

				if (Beard->color() < 0x044E || Beard->color() > 0x04AD)
				{
					Beard->setColor(0x044E);
				}

				Beard->incognito=true;//AntiChrist
			}
		}//if beardserial!=-1
		
		//only refresh once
		updatechar(pc_dest);
		SndWornItemsToAllInRange(pc_dest);
		pc_dest->incognito=true;//AntiChrist
		break;

	case TEF_LSD: // LSD potions, LB 5'th nov 1999
		k=pc_source->calcSocket();
		if (k==-1) return 0;
		sysmessage(k,"Hmmm, tasty, LSD");
		LSD[k]=1;
		pc_source->fullHp();
		pc_source->mn=pc_source->effInt();
		impowncreate(k,pc_source,0);
		pTE->setExpiretime_s(90);
		break;

	case TEF_Protection:		// protection
		pTE->setExpiretime_s(12);
		pTE->dispellable=1;
		pTE->more1=more1;
		pc_dest->chgRealSkill(PARRYING, more1);
		break;

	case TEF_HideGM: // gm hiding
		pTE->setExpiretime_s(more1);
		pTE->more1=more1;
		break;

	case TEF_UnHideGM: // gm un hiding
		pTE->setExpiretime_s(more1);
		pTE->more1=more1;
		break;

	case TEF_HealMsg:
	    //heals some hp in 9 seconds - Solarin
		k=pc_source->calcSocket();
		pTE->setExpiretime_s(more3);
		if (!more2)
			sysmessage(k,"You start healing yourself...");
		else
			sysmessage(k,"You continue to heal...");

		pTE->more1=more1; //hp to give back
		pTE->more2=more2; //# of call to decide which string to display on checktempeffect
		pTE->more3=more3; //delay#
		break;

	case TEF_FencingPara: // special fencing paralyzation -Frazurbluu-
		pc_dest->Freeze();
		pTE->setExpiretime_s(5);
		pTE->num=1;
		pTE->more1=0;
		pTE->more2=0;
		break;

	case TEF_Concussion: // special sword use concussion intelligence loss -Frazurbluu-
		if (pc_dest->effInt()<more1)
			more1=pc_dest->effInt();
		pc_dest->chgInt(-1 * more1);
		pc_dest->mn=min(pc_dest->mn, (int)pc_dest->effInt());
		statwindow(pc_dest->calcSocket(), pc_dest);
		pTE->setExpiretime_s(30);
		pTE->num=4;
		pTE->more1=more1;
		pTE->more2=0;
		break;

	case TEF_Camping: // Camping fire message
		k=pc_source->calcSocket();
		pTE->setExpiretime_s(30);
		sysmessage(k,"You feel it will take a few moments to secure your camp.");
		break;

	case TEF_Doppelgaenger: // doppelgaenger effect
	
		pTE->setExpiretime_s(dur);
		Npcs->TempDupeNpc(pc_source, pc_dest, more1);
		break;

	default:
		LogErrorVar("Fallout of switch! Value (%d)",num);
		return 0;
	}
	AllTmpEff->Insert(pTE);
	pc_dest->checkEquipment();	// checks equipments for stats requirements
	return 1;
}

bool cAllTmpEff::Add(P_CHAR pcSource, P_ITEM piDest, int num, unsigned char more1, unsigned char more2, unsigned char more3)
{
	if (!pcSource) return 0;
	if (teffectcount > (unsigned) (Npcs->GetCapacity()*5)) return 0;

	cTmpEff *pTE=new cTmpEff;
	pTE->Init(num);
	pTE->setSour(pcSource->serial);
	pTE->setDest(piDest->serial);

	switch (num)
	{

	case TEF_Mortar:
		pTE->setExpiretime_s(12);
		pTE->more1=more1;
		pTE->more2=more2;		
		break;

	case TEF_TDummy: //Tauriel training dummies swing for 5(?) seconds
		pTE->setExpiretime_s(5);
		pTE->more2=0;
		break;

	case TEF_Explode: //Explosion potion (explosion)	Tauriel (explode in 4 seconds)
		pTE->setExpiretime_s(4);
		pTE->more1=more1;
		pTE->more2=more2;
		piDest->morey=POTION_EXPLODING;
		break;

	default:
		LogError("Fallout of switch");
		return 0;
	}

	AllTmpEff->Insert(pTE);
	return 1;
}

cAllTmpEff::cAllTmpEff()
{
	teffects=NULL; // No temp effects to start with
	teffectcount=0;
}

bool cAllTmpEff::Alloc(int count)
{
	teffects = (cTmpEff **)malloc(count*sizeof(cTmpEff*));	//MAXEFFECTS = 5*MAXCHARS
	return teffects==NULL ? false : true;
}

bool cAllTmpEff::ReAlloc(int newcount)
{
	teffects = (cTmpEff **)realloc(teffects, (newcount)*sizeof(cTmpEff*));
	return teffects==NULL ? false : true;
}

void cAllTmpEff::Insert(cTmpEff* pTE)
{
	teffects[teffectcount] = pTE;
	teffectcount++;
}

void cAllTmpEff::Remove(int i)
{
	delete teffects[i];
	teffects[i] = teffects[teffectcount-1];
	teffectcount--;
}


unsigned char tempeffect(P_CHAR Source, P_CHAR Dest, eTmpEffTypes num, unsigned char more1, unsigned char more2, unsigned char more3, short dur)
{
	if (num==TEF_Taming)
	{
		cTmpEff *pTE=new cTmpEffTaming(Source,Dest,more1,more2);
		AllTmpEff->Insert(pTE);
		return 1;
	}
	else
		return AllTmpEff->Add(Source, Dest, num, more1, more2, more3, dur);
}

unsigned char tempeffect(P_CHAR pcBoth, eTmpEffTypes num, unsigned char more1, unsigned char more2, unsigned char more3, short dur)
{
	if (num==TEF_Sheep)
	{
		cTmpEff *pTE=new cTmpEffSheep(pcBoth);
		AllTmpEff->Insert(pTE);
		return 1;
	}
	else
		return AllTmpEff->Add(pcBoth, pcBoth, num, more1, more2, more3, dur);
}

unsigned char tempeffect(P_ITEM piDest, eTmpEffTypes num, unsigned char more1, unsigned char more2, unsigned char more3)
{
	return tempeffect(0, piDest, num, more1, more2, more3);
}

unsigned char tempeffect(P_CHAR pcSource, P_ITEM piDest, eTmpEffTypes num, unsigned char more1, unsigned char more2, unsigned char more3)
{
	cTmpEff *pTE=NULL;
	switch (num)
	{
	case TEF_Door:	pTE=new cTmpEffDoor(num,piDest);		break;
	case TEF_Plant:	pTE=new cTmpEffPlant(piDest,more1);		break;
	case TEF_Mill:	pTE=new cTmpEffMill(piDest,more1,more2);break;
	default:		return AllTmpEff->Add(pcSource, piDest, num, more1, more2, more3);
	}
	AllTmpEff->Insert(pTE);
	return 1;
}


