/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

#include "accounts.h"
#include "utilsys.h"
#include <time.h>

#include "wolfpack.h"

#include "debug.h"
#undef DBGFILE
#define DBGFILE "accounts.cpp"

cAccount::cAccount(void)
{
	unsavedaccounts = 0;
	saveratio = 0;         // Save everyaccount
}

cAccount::~cAccount(void)
{
	if (unsavedaccounts > 0)
		SaveAccounts();
}

void cAccount::LoadAccount( int acctnumb )
{
	int loopexit=0;
	account_st account;
	account.name.erase();
	account.pass.erase();
	account.ban = false;
	account.remoteadmin = false;
	account.number = acctnumb;
	account.lastused = 0;
	do
	{
		read2();
		
		if (!strcmp(script1, "NAME"))				account.name =  script2;
		else if (!strcmp(script1, "PASS"))			account.pass = script2;
		else if (!strcmp(script1, "BAN"))			account.ban = true;
		else if (!strcmp(script1, "REMOTEADMIN"))	account.remoteadmin = true;
		else if (!strcmp(script1, "LASTUSED"))		account.lastused = str2num(script2);
	}

	while ( (strcmp(script1, "}")) && (strcmp(script1, "EOF")) && (++loopexit < MAXLOOPS) );

	acctlist.insert(make_pair(account.name, account));
	acctnumbers_sp.insert(make_pair(account.number, account.name));
	lasttimecheck = uiCurrentTime;
}

int cAccount::Count()
{
	return acctlist.size();
}

void cAccount::LoadAccounts( void )
{
	int b,c,ac, account,loopexit=0;

	CArray<128, char>accnumb("accnumb"); 		//// 64 badly crashed
	CArray<128, char>acc("acc");

	char *t;
	lastusedacctnum = 0;

	openscript("accounts.adm");
	acctlist.clear();
	//int x=0;
	do
	{
		read2();	
		if (!strcmp(script1, "SECTION"))
		{
			c = strlen(script2);
			for (b=0; b<9; b++) acc[b]=script2[b]; 
			for (b=8; b<c; b++) accnumb[b-8]=script2[b];
			accnumb[b-8]=0; acc[8]=0;		
			ac = strtol(&accnumb[0], &t, 10);
			if (strlen(t)!=0) ac=-1;

			if (strcmp(&acc[0],"ACCOUNT ") || ac < 0 )
			{
				LogWarning("Error loading accounts, skipping invalid account entry!");
			}
			else
			{
			   account=ac;
			   LoadAccount(account);
			   if (account > lastusedacctnum)
				   lastusedacctnum = account;
			}
		}
	}
	while ( (strcmp(script1, "EOF")) && (++loopexit < MAXLOOPS) );

	closescript();	
}

void cAccount::SaveAccounts( void )
{
	map<string, account_st>::iterator iter_account;

	account_st account;
	fstream faccount ;
	string  line ;

	faccount.open("accounts.adm",ios::out) ;

	unsigned int maxacctnumb = 0;     // Saving the number of loaded accounts
	if (faccount.is_open())
	{
		for (iter_account = acctlist.begin(); iter_account != acctlist.end(); iter_account++)
		{
			account = iter_account->second;    
			faccount << "SECTION ACCOUNT " << account.number << endl ;
			faccount << "{" << endl ;
			faccount << "NAME " << account.name << endl ;
			faccount << "PASS " << account.pass << endl ;
			faccount << "LASTUSED " << account.lastused << endl ;
			if (account.ban)
				faccount << "BAN" << endl ;
			if (account.remoteadmin)
				faccount << "REMOTEADMIN" << endl;
			faccount << "}" << endl << endl;
			if (maxacctnumb < account.number)
				maxacctnumb = account.number;
		}
		faccount << endl ;
		faccount << "// Note: Last day saved: " << getPlatformDay() << endl;
		faccount << "// Note: Last used Account Number was: " << maxacctnumb << endl;
		faccount << "EOF" << endl ;
	}
	faccount.close();
	unsavedaccounts = 0;

	return;
}

bool cAccount::RemoteAdmin(int acctnum)
{
	if (acctnum < 0) return false;

	map<string, account_st>::iterator iter_account;
	map<int, string>::iterator iter_acctnumber = acctnumbers_sp.find(acctnum);

	if (iter_acctnumber == acctnumbers_sp.end()) return false;

	if ((iter_account = acctlist.find(iter_acctnumber->second)) != acctlist.end()) return iter_account->second.remoteadmin;

	else return false;
}
	

signed int cAccount::Authenticate(string username, string password)
{
//	if (username.length() < 1 || password.length() < 1)
//		return LOGIN_NOT_FOUND;
// @Pokey: this doesn't really help :( makes me hang at login (Duke,28.2.02)
	account_st account;
	map<string, account_st>::iterator iter_account;

	if ((iter_account = acctlist.find(username)) != acctlist.end())
	{
		account = iter_account->second;
		if (account.pass == password)
		{
			if (account.ban)
				return ACCOUNT_BANNED;
			else
				return account.number;
		} else
			return BAD_PASSWORD;
	} else
		return LOGIN_NOT_FOUND;
}


bool cAccount::CheckExistence(char* username)
{
	account_st account;
	map<string, account_st>::iterator iter_account;

	if ((iter_account = acctlist.find(username)) != acctlist.end()) return true;
	else return false;
}

void cAccount::CheckAccountFile(void)
{
	struct stat filestatus;
	static time_t lastchecked;
	
	stat("accounts.adm", &filestatus);

	if (difftime(filestatus.st_mtime, lastchecked) > 0.0)
		LoadAccounts();

	lastchecked = filestatus.st_mtime;
	lasttimecheck = uiCurrentTime;
}

bool cAccount::IsOnline( int acctnum )
{
	if (acctnum < 0) return false;

	map<int, acctman_st>::iterator iter_acctman;

	if ((iter_acctman = acctman.find(acctnum)) != acctman.end())
	{
		acctman_st dummy = iter_acctman->second;
		if (dummy.online) return true;
		P_CHAR pc = MAKE_CHARREF_LRV(dummy.character,false);
		if (pc->logout > uiCurrentTime) return true; // POTENTIAL SIGNED/UNSIGNED problem
	}
	return false;
}


CHARACTER cAccount::GetInWorld( int acctnum )
{
	if (acctnum < 0) return -1;

	map<int, acctman_st>::iterator iter_acctman;

	if ((iter_acctman = acctman.find(acctnum)) != acctman.end())
	{
		acctman_st dummy = iter_acctman->second;
		return dummy.character;
	} else return -1;


}

void cAccount::SetOnline( int acctnum, P_CHAR Player)
{
	acctman_st dummy;
	dummy.online = true;
	dummy.character = Player->GetIndex();
	acctman.insert(make_pair(acctnum, dummy));
}

void cAccount::SetOffline( int acctnum )
{
	map<int, acctman_st>::iterator iter_acctman;
	if ((iter_acctman = acctman.find(acctnum)) != acctman.end())
	{
		acctman.erase(iter_acctman);
	}
}

bool cAccount::ChangePassword(unsigned int number, string password)
{
	account_st account;
	map<string, account_st>::iterator iter_account;
	
	for (iter_account = acctlist.begin(); iter_account != acctlist.end(); iter_account++)
	{
		if (iter_account->second.number == number)
		{
			iter_account->second.pass = password;
			unsavedaccounts++;
			if (unsavedaccounts >= saveratio)
				SaveAccounts();
			return true;
		}
	}
	return false;
}

void cAccount::SetLastUsed(unsigned int number, int day)
{
	account_st account;
	map<string, account_st>::iterator aai;
	
	for (aai = acctlist.begin(); aai != acctlist.end(); aai++)
	{
		if (aai->second.number == number)
		{
			aai->second.lastused = day;
			unsavedaccounts++;
			if (unsavedaccounts >= saveratio)
				SaveAccounts();
			return;
		}
	}
}


void cAccount::AddGift(int gift_scp_num)
{
	map<string, account_st>::iterator aai;	// all account iterator
	
	for (aai = acctlist.begin(); aai != acctlist.end(); aai++)
	{
		int accno = aai->second.number;
		P_CHAR pc;
		AllCharsIterator aci;
		for (aci.Begin(); (pc=aci.GetData()) != NULL; aci++)
		{
			if (pc->account == accno)
			{
				if (pc->AddGift(gift_scp_num))
					break;	// if success, proceed with next account
			}
		}
	}
	return;
}

unsigned int cAccount::CreateAccount(string username, string password)
{
	lastusedacctnum++;
	account_st account;
	account.name = username;
	account.pass = password;
	account.ban = false;
	account.remoteadmin = false;
	account.number = lastusedacctnum;
	acctlist.insert(make_pair(username, account));
	unsavedaccounts++;
	if (unsavedaccounts >= saveratio)
		SaveAccounts();

	acctnumbers_sp.insert(make_pair(lastusedacctnum, username) );
	account_playerlist *dummy = new account_playerlist;	// add a new playerlist/account shadow as well
	acctplayer.insert(make_pair(lastusedacctnum, dummy));	
	
	return account.number;
}


void cAccount::RemoveAccount(int number)
{
	account_st account;
	map<string, account_st>::iterator aai;
	
	for (aai = acctlist.begin(); aai != acctlist.end(); aai++)
	{
		if (aai->second.number == number)
		{
			//acctlist.erase(aai->first); // equivalent to not rem'ed one. maps can be deleted by key or by iter-position

			acctlist.erase(aai); 
			acctnumbers_sp.erase(number);
			acctplayer.erase(number); 

			unsavedaccounts++;
			if (unsavedaccounts >= saveratio) SaveAccounts();

			return;
		}
	}
}


//////////////////////////////////////////////////
// maintaining the per account based player list functions
// note that they are logical private 
// (they just maintain an account-class internal list other real public ones need)
// but need to be public for technical resaons
//////////////////////////////////////////////////

void cAccount::InitPlayerAccountShadow(void) // call once after worldloading
{
	map<int, string>::iterator iter_acctnumber;
	map<int, account_playerlist* >::iterator iter_acctplayer;

	int accts=0;
	for (iter_acctnumber=acctnumbers_sp.begin(); iter_acctnumber!=acctnumbers_sp.end(); ++iter_acctnumber )
	{
		account_playerlist *dummy = new account_playerlist;	
		acctplayer.insert(make_pair(iter_acctnumber->first, dummy));
		accts++;	
	}

	P_CHAR pc;
	AllCharsIterator aci;
	for (aci.Begin(); (pc=aci.GetData()) != NULL; aci++)
	{
		if (!pc->npc && pc->account>-1)
		{			 
			iter_acctplayer=acctplayer.find(pc->account);
			if (iter_acctplayer!=acctplayer.end()) // account found !
			{
				player_struct pl;
				pl.serial=pc->serial;
				pl.stat=OFF; // on serverstart every one is off

				iter_acctplayer->second->addPlayerInitialy(pl); // add player to per account player list
			}
		}	    
	}
	
	int mex=0;
	for (iter_acctplayer=acctplayer.begin(); iter_acctplayer!=acctplayer.end(); ++iter_acctplayer )
	{	
		bool end;
		player_struct ps;

		iter_acctplayer->second->FirstPlayerListIter(ps, end);
	    while (!end)
		{	
			P_CHAR pc = FindCharBySerial(ps.serial);		
			if (pc!=NULL) 
			{
				//printf("I-AN:%i %s\n", iter_acctplayer->first, pc->name);
				mex++;
			}

			iter_acctplayer->second->NextPlayerListIter(ps, end);		
		}			
	}
	
	if ((accts-mex)>0)
	{
		sprintf(temp,"Warning: %i players in accountfile that are not in worldfile (total %i) \n\n", accts-mex,accts);
		Console.send(temp);
	}
}

void cAccount::RemovePlayerFromAccountShadow(P_CHAR Player) // called after player deletation
{

	if (Player==NULL) return;
	map<int, account_playerlist*>::iterator iter_acctplayer;

	iter_acctplayer=acctplayer.find(Player->account);
	if (iter_acctplayer!=acctplayer.end()) // account found !
	{
		int serial=Player->serial;
		
		bool end;
		player_struct ps;
		iter_acctplayer->second->FirstPlayerListIter(ps, end);
	    while (!end)
		{	
		  if (ps.serial==serial)
		  {
			iter_acctplayer->second->EraseCurrentIteratedPlayer(); // and delete him/her
			break;
		  }		

		  iter_acctplayer->second->NextPlayerListIter(ps, end);
		}
	} else
	{
		sprintf(temp,"Error in RemovePlayerToAccountShadow after trying to delete player %s\n", Player->name);
		LogError(temp);
	}

	///////////////////// debug ///////////	

	/*
	bool end;
	player_struct ps;

	iter_acctplayer=acctplayer.find(Player->account);
	iter_acctplayer->second->FirstPlayerListIter(ps, end);
    while (!end)
	{	
		P_CHAR pc = FindCharBySerial(ps.serial);		
		if (pc!=NULL) 
		{
			printf("I-AN:%i %s\n", iter_acctplayer->first, pc->name);			
		}

		iter_acctplayer->second->NextPlayerListIter(ps, end);		
	}
	*/
}

void cAccount::AddPlayerToAccountShadow(P_CHAR Player) // called after player creation
{
	if (Player==NULL) return;
	map<int, account_playerlist* >::iterator iter_acctplayer;

	iter_acctplayer=acctplayer.find(Player->account);
	if (iter_acctplayer!=acctplayer.end()) // account found !
	{
		player_struct pl;
		pl.serial=Player->serial;
		pl.stat=OFF; // though player is connected, he/she's not logged in yet (just created char, going to be logged in a few moments/packets)

		iter_acctplayer->second->addPlayerInitialy(pl);  // add player to per account player list
	
	} else
	{
		sprintf(temp,"Error in AddPlayerToAccountShadow after trying to add player %s\n", Player->name);
		LogError(temp);	
	}

	/////////////////////////////// debug ///////////////////////

	/*
	bool end;
	player_struct ps;

	iter_acctplayer=acctplayer.find(Player->account);
	iter_acctplayer->second->FirstPlayerListIter(ps, end);
	while (!end)
	{	
	  P_CHAR pc = FindCharBySerial(ps.serial);		
	  if (pc!=NULL) 
	  {
		printf("I-AN:%i %s\n", iter_acctplayer->first, pc->name);			
	  }

	  iter_acctplayer->second->NextPlayerListIter(ps, end);		
	}
	*/

}

/////////////////////////////////////////////////////////////

P_CHAR cAccount::GetCurrentPlayer(int acct)
{
  return NULL;
}



