/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

// LoneWolf - Bounty System
// FILE - bounty.cpp
// DATE - July 17, 2000
//   By - Dupois (dupois@home.com)
//
#include "bounty.h"

#include "msgboard.h"
#include <iostream>

using namespace std ;

#include "wolfpack.h"
#undef  DBGFILE
#define DBGFILE "bounty.cpp"

////////////////
// FUNCTION:    BountyAskVictim( int nVictimSerial, int nMurdererSerial )
//
// PURPOSE:     Used to ask the murder victim whether they would like to place
//              a bounty on the head of the murderer.
// 
// PARAMETERS:  nVictimSerial   Victim characters serial number
//              nMurdererSerial Murderer characters serial number
//
// RETURNS:     void
//
void BountyAskVictim( int nVictimSerial, int nMurdererSerial )
{
	int nAmount     = 0;
	
	P_CHAR Victim=FindCharBySerial(nVictimSerial);
	P_CHAR Murderer=FindCharBySerial(nMurdererSerial);
	if (!Victim || !Murderer) return;
	
	// Indicate that the victim is being asked if they want to place a bounty
	sprintf(temp, "BountyAskVictim():  %s is attempting to place a bounty of %i on %s\n",
		Victim->name, nAmount, Murderer->name );
	LogMessage(temp);
	
	// If the amount of the bounty is larger than zero, create a bounty posting
	if (nAmount > 0)
	{
		if (BountyCreate(Murderer, nAmount))
		{
			sprintf(temp, "BountyAskVictim():  %s has placed a bounty of %i on %s",
                Victim->name,
                nAmount,
                Murderer->name );
			LogMessage(temp);
		}
		else
		{
			sprintf(temp, "BountyAskVictim():  %s FAILED to place a bounty of %i on %s",
				Victim->name,
				nAmount,
				Murderer->name );
			LogMessage(temp);
		}
	}
	return;
}


//////////////////////////////////////////////////////////////////////////////
// FUNCTION:    BountyCreate( int nMurdererSerial, int nRewardAmount )
//
// PURPOSE:     Used to create the bounty posting message on the global
//              bulletin board for all to see.
// 
// PARAMETERS:  nMurdererSerial Murderer characters serial number
//              nRewardAmount   Bounty placed on the murderers head
//
// RETURNS:     TRUE  Bounty post successfully created
//              FALSE Bounty post could not be created
//////////////////////////////////////////////////////////////////////////////
bool BountyCreate( P_CHAR Murderer, int nRewardAmount )
{
	if (!Murderer) return false;
	int   nPostSerial = 0;
	
	// Check that we have a reward amount greater than zero
	if (nRewardAmount > 0)
	{
		// Check that this murderer doesn't already have a bounty on them
		if (Murderer->questBountyReward > 0)
		{
			// This murderer already has a bounty on them because they 
			// have a reward amount on their head, so delete old bounty
			// and add the new (updated) one
			nRewardAmount += Murderer->questBountyReward;
			BountyDelete( Murderer->serial );
		}
		
		// Attempt to post the message first
		Murderer->questBountyReward = nRewardAmount;
		nPostSerial = MsgBoardPostQuest( Murderer->serial, BOUNTYQUEST );
		
		// If we received a valid serial number then the post was successfull
		if (nPostSerial > 0)
		{
			Murderer->questBountyPostSerial = nPostSerial;
			return true;
		}
	}
	
	// Failed to post bounty
	sprintf(temp, "BountyCreate():  FAILED to place a bounty of %i on %s (PostSerial=%x)\n",
		nRewardAmount,
		Murderer->name,
		nPostSerial );
	LogWarning(temp);
	
	// Post must have failed
	return false;
	
} // BountyCreate()


bool BountyDelete( int nMurdererSerial )
{
	P_CHAR Murderer=FindCharBySerial(nMurdererSerial);
	if (!Murderer) return false;
	
	
	// Find and mark the post associated with this bounty as deleted
	// so that the bulletin board maintenance routine can clean it up
	bool bReturn = MsgBoardRemoveGlobalPostBySerial( Murderer->questBountyPostSerial );
	
	// Reset all bounty values for this character
	Murderer->questBountyReward     = 0;
	Murderer->questBountyPostSerial = 0;
	
	return bReturn;
	
} // BountyDelete()


bool BountyWithdrawGold( P_CHAR pVictim, int nAmount )
{
	int has = pVictim->CountBankGold();
	if (has < nAmount)
		return false;

	P_ITEM pBox = pVictim->GetBankBox();
	if (!pBox)
		return false;	// shouldn't happen coz it's needed in CountBankGold...

	pBox->DeleteAmount(nAmount,0x0EED);
	return true;
}
