#ifndef gumps__
#define gumps__

#include <vector>
#include "platform.h"
#include "CArray.h"
#include "network.h"

#include "wolfpack.h"
#include "debug.h"
#undef  DBGFILE
#define DBGFILE "cGumps.hpp"

using namespace std;

//********************
//* Gumps class by LB
//********************

// purpose: going to modern OSI style gumps ...
// can't be done with current gump class reasonably
// will enable better gump scriping etc,etc


template <const int d=0x4000>
class CGumps
{
   public:	  

	   CGumps(unsigned int gumpid, string name="a gump"):rawdatasizemax(d) , gumpId(gumpid)
	   { 
		   GumpRawdata=NULL;
		   GumpRawdata= new CArray<d, unsigned char>(name); 

		   GumpDecl.clear();
		   GumpText.clear();

		   // set fix points of raw-array (0, 7-10)

		   for (int a=0; a<rawdatasizemax; a++) (*GumpRawdata)[a]=0x0;

		   (*GumpRawdata)[0]=0xb0;	
		   
		   int a0=gumpid;

		   unsigned char a1 = ( ( a0 & 0x000000ff )       );
		   unsigned char a2 = ( ( a0 & 0x0000ff00 ) >> 8  );
		   unsigned char a3 = ( ( a0 & 0x00ff0000 ) >> 16 );
		   unsigned char a4 = ( ( a0 & 0xff000000 ) >> 24 );

		   (*GumpRawdata)[10]=a1;	
		   (*GumpRawdata)[9] =a2;	
		   (*GumpRawdata)[8] =a3;	
		   (*GumpRawdata)[7] =a4;	
		   
		   real_length=0;
		   fixed_length=0;

	   }
	   
	   virtual ~CGumps()
	   {
		   if (GumpRawdata!=NULL) delete GumpRawdata;
	   }

	   inline unsigned int myGumpId() { return gumpId; }

	   inline void SendGump(UOXSOCKET s);

	   inline void addGumpDescr(string s)     { string str; str=str+"{ "; str=str+s; str=str+" }"; GumpDecl.push_back(str); }   
	   inline void addGumpText(string s)      { GumpText.push_back(s);	}   
	   inline void deleteGumpDescrComplete()  { GumpDecl.clear();	        }
	   inline void deleteGumpTextComplete()   { GumpText.clear();           }

	   void toRawDataDynOnly(void);
	   void toRawData(int x=0, int y=0);

   private:
	   
	    const unsigned short rawdatasizemax;	
		const unsigned int   gumpId;

		int real_length;
		int fixed_length;
			   
	    CArray<d, unsigned char>*GumpRawdata;
		vector<std::string>GumpDecl;
		vector<std::string>GumpText;	
   
public: // just for testing public, it's private
		void showrawdatadump();

};

template<const int d> void CGumps<d>::SendGump(UOXSOCKET s)
{
   int serial,ind;

   ind = currchar(s);
   P_CHAR Player = MAKE_CHARREF_LR(ind);

   serial = Player->serial;

   unsigned char a1 = ( ( serial & 0x000000ff )       );
   unsigned char a2 = ( ( serial & 0x0000ff00 ) >> 8  );
   unsigned char a3 = ( ( serial & 0x00ff0000 ) >> 16 );
   unsigned char a4 = ( ( serial & 0xff000000 ) >> 24 );
   
   (*GumpRawdata)[6] = a1;	
   (*GumpRawdata)[5] = a2;	
   (*GumpRawdata)[4] = a3;	
   (*GumpRawdata)[3] = a4;	

   // now send
   XStartPkt(0xb0, real_length);
   XAddPkt (&(*GumpRawdata)[0+3], real_length-3 ); // note that &(*Gumpdata) is NOT the same as Gumpdata in this case !!!
   XFinishPkt(s);
	
}

template<const int d>void CGumps<d>::toRawData(int x, int y)
{
   unsigned short textLines, declLength,txtsize;
   unsigned char  a1,a2,a3,a4;
   int a,pos;

   a1 = ( ( x & 0x000000ff )       );
   a2 = ( ( x & 0x0000ff00 ) >> 8  );
   a3 = ( ( x & 0x00ff0000 ) >> 16 );
   a4 = ( ( x & 0xff000000 ) >> 24 );
   
   (*GumpRawdata)[14] = a1;	
   (*GumpRawdata)[13] = a2;	
   (*GumpRawdata)[12] = a3;	
   (*GumpRawdata)[11] = a4;	

   a1 = ( ( y & 0x000000ff )       );
   a2 = ( ( y & 0x0000ff00 ) >> 8  );
   a3 = ( ( y & 0x00ff0000 ) >> 16 );
   a4 = ( ( y & 0xff000000 ) >> 24 );
   
   (*GumpRawdata)[18] = a1;	
   (*GumpRawdata)[17] = a2;	
   (*GumpRawdata)[16] = a3;	
   (*GumpRawdata)[15] = a4;	
    
   vector<string>::iterator txtIter;
   vector<string>::iterator decIter;
   string str;

   declLength=0;
   pos=21;

   for(decIter=GumpDecl.begin(); decIter!=GumpDecl.end(); ++decIter)
   {
	   str=*decIter;
       txtsize=str.size();
	   declLength+=txtsize;

	   for (a=0; a<txtsize; a++)
	   {
           (*GumpRawdata)[pos]=str[a]; 
		   pos++;
	   }
   }

   declLength++;
   a1 = ( ( declLength & 0x00ff )       );
   a2 = ( ( declLength & 0xff00 ) >> 8  );

   (*GumpRawdata)[20] = a1;	
   (*GumpRawdata)[19] = a2;	

   fixed_length=declLength+20;

   textLines = GumpText.size();

   (*GumpRawdata)[pos]=0;

   a1 = ( ( textLines & 0x00ff )       );
   a2 = ( ( textLines & 0xff00 ) >> 8  );

   (*GumpRawdata)[pos+2] = a1;
   (*GumpRawdata)[pos+1] = a2;
   
   pos+=3;

   for(txtIter=GumpText.begin(); txtIter!=GumpText.end(); ++txtIter)
   {
	   str=*txtIter;

	   txtsize=str.size();
	   a1 = ( ( txtsize & 0x00ff )       );
       a2 = ( ( txtsize & 0xff00 ) >> 8  );

	   (*GumpRawdata)[pos+1] = a1;
	   (*GumpRawdata)[pos]   = a2;

	   pos+=2;

	   for (a=0; a<txtsize; a++)
	   {
           (*GumpRawdata)[pos+1]   = str[a];
		   (*GumpRawdata)[pos] = 0;
		   pos+=2;
	   }
   }
   
   real_length=pos;

   a1 = ( ( real_length & 0x00ff )       );
   a2 = ( ( real_length & 0xff00 ) >> 8  );

   (*GumpRawdata)[2]=a1;
   (*GumpRawdata)[1]=a2;

}

// pre: (complete) torawdata(x,y) called once !
// only changes dynamic part of the gumps (text)
// input/pre: internal vectors of text set
template<const int d>void CGumps<d>::toRawDataDynOnly(void)
{
   unsigned short textLines;
   unsigned char a1,a2;
   int pos, txtsize,a;

   vector<string>::iterator txtIter;
   string str;

   pos=fixed_length;
   textLines = GumpText.size();

   (*GumpRawdata)[pos]=0;

   a1 = ( ( textLines & 0x00ff )       );
   a2 = ( ( textLines & 0xff00 ) >> 8  );

   (*GumpRawdata)[pos+2] = a1;
   (*GumpRawdata)[pos+1] = a2;

   pos+=3;
   
   for(txtIter=GumpText.begin(); txtIter!=GumpText.end(); ++txtIter)
   {
	   str=*txtIter;

	   txtsize=str.size();
	   a1 = ( ( txtsize & 0x00ff )       );
       a2 = ( ( txtsize & 0xff00 ) >> 8  );

	   (*GumpRawdata)[pos+1] = a1;
	   (*GumpRawdata)[pos]   = a2;

	   pos+=2;

	   for (a=0; a<txtsize; a++)
	   {
           (*GumpRawdata)[pos+1]   = str[a];
		   (*GumpRawdata)[pos] = 0;
		   pos+=2;
	   }
   }
   
   real_length=pos;

   a1 = ( ( real_length & 0x00ff )       );
   a2 = ( ( real_length & 0xff00 ) >> 8  );

   (*GumpRawdata)[2]=a1;
   (*GumpRawdata)[1]=a2;

}

template <const int d> void CGumps<d>::showrawdatadump()
{
	   int actRow ;
	   for(int actLine = 0; actLine < (real_length / 16) + 1; actLine++)
	   {
		  printf("%04x: ", actLine * 16);

		  for( actRow = 0; actRow < 16; actRow++)
		  {	
			if (actLine * 16 + actRow < real_length) printf("%02x ", (unsigned int)((*GumpRawdata)[actLine * 16 + actRow]));		  
			else printf("-- ");
		  }

		  printf(": ");

		  for(actRow = 0; actRow < 16; actRow++)
		  {
			  if (actLine * 16 + actRow < real_length) printf("%c", isprint((*GumpRawdata)[actLine * 16 + actRow]) ? (*GumpRawdata)[actLine * 16 + actRow] : '.');
		  }

		  printf("\r\n");
		}
	   	printf("\r\n");	

}

#endif
