/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//
#include "commands.h"

#include "Client.h"
#include "cmdtable.h"
#include "gumps.h"
#include "mapstuff.h"
#include "network.h"
#include "SndPkg.h"
#include "sregions.h"
#include "tiledata.h"

#include "wolfpack.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "commands.cpp"

// inline command to do targeting - coulda made this a
// macro but this is pretty much as fast...
static inline void _do_target(int s, TARGET_S *ts)
{
	target(s, ts->a1, ts->txt);
	return;
}


struct gmpage_st
{
	string reason;
	SERIAL serial;
	char timeofcall[9];
	string name;
	int handled;
};

CArray<MAXPAGES, gmpage_st>gmpages_CArray("gmpages");
CArray<MAXPAGES, gmpage_st>counspages_CArray("counspages");

#define gmpages gmpages_CArray.prepare(DBGFILE,__LINE__)
#define counspages counspages_CArray.prepare(DBGFILE,__LINE__)

void InitPages() // Initilizes the gmpages[] and counspages[] arrays
{
	int i;
	for(i=1;i<MAXPAGES;i++)
	{
		gmpages[i].name.erase();
		gmpages[i].reason.erase();
		gmpages[i].serial = 0;
		gmpages[i].timeofcall[0]=0;
		gmpages[i].handled=1;
	}
	for(i=1;i<MAXPAGES;i++)
	{
		counspages[i].name.erase();
		counspages[i].reason.erase();
		counspages[i].serial = 0;
		counspages[i].timeofcall[0]=0;
		counspages[i].handled=1;
	}
}

void donewithcall(int s, int type)
{
	P_CHAR Player = MAKE_CHARREF_LR(currchar(s));
	int cn = Player->callnum;
	if (cn > 0) //Player is on a call
	{
		if (type==1) //Player is a GM
		{
			gmpages[cn].handled=1;
			gmpages[cn].name.erase();
			gmpages[cn].reason.erase();
			gmpages[cn].serial = 0;
			sysmessage(s,"Call removed from the GM queue.");
		}
		else //Player is a counselor
		{
			counspages[cn].handled=1;
			counspages[cn].name.erase();
			counspages[cn].reason.erase();
			counspages[cn].serial = 0;
			sysmessage(s,"Call removed from the Counselor queue.");
		}
		Player->callnum=0;
	}
	else
	{
		sysmessage(s,"You are currently not on a call");
	}
}

/* extensively modified 8/2/99 crackerjack@crackerjack.net -
 * see cmdtable.cpp for more details */
void cCommands::Command(UOXSOCKET s, string speech) // Client entred a '/' command like /ADD
{
	int i=9;
	unsigned char *comm;
	unsigned char nonuni[512];
	int y,loopexit=0;

	P_CHAR Player = MAKE_CHARREF_LR(currchar(s));

	if (Player->unicode)
		cCommands::cmd_offset = 1;
	else
		cCommands::cmd_offset = 1;

	cCommands::command_line = speech;
	cCommands::params = cCommands::command_line.split(" ");
	strcpy((char*)nonuni, speech.c_str());
	strcpy((char*)tbuffer, (char*)nonuni);

//	strupr((char*)nonuni); //dont strupr the whole command with all its parameters
	cline = (char*) &nonuni[0];
	splitline();
	if (tnum<1)
		return;
	// Let's ignore the command prefix;
	comm = &nonuni[1];
	strupr((char*)comm); // only strupr the "comamnd word" :) (Thrawn)

	i=0; y=-1;loopexit=0;
	while((command_table[i].cmd_name)&&(y==-1) && (++loopexit < MAXLOOPS))
	{
		if (!strcmp((char*)command_table[i].cmd_name, (char*)comm)) y=i;
		i++;
	}

	if (y==-1)
	{
		sysmessage(s, "Unrecognized command.");
		return;
	}
	else
	{
		if ((Player->isTrueGM() && !Player->isGM()) ||		// a restricted GM outside his region(s)
			(Player->account!=0)&&(command_table[y].cmd_priv_m!=255)&&
			(!(Player->priv3[command_table[y].cmd_priv_m]&
			(0-0xFFFFFFFF<<command_table[y].cmd_priv_b))))
		{
			sysmessage(s, "Access denied.");
			return;
		}

		switch(command_table[y].cmd_type)
		{
		case CMD_FUNC:
			(*((CMD_EXEC)command_table[y].cmd_extra)) (s);
			break;
		case CMD_ITEMMENU:
			itemmenu(s, (int)command_table[y].cmd_extra);
			break;
		case CMD_TARGET:
			_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			break;
		case CMD_TARGETX:
			if (tnum==2)
			{
				addx[s]=makenumber(1);
				_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			}
			else
				sysmessage(s, "This command takes one number as an argument.");
			break;
		case CMD_TARGETXY:
			if (tnum==3)
			{
				addx[s]=makenumber(1);
				addy[s]=makenumber(2);
				_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			}
			else
				sysmessage(s, "This command takes two numbers as arguments.");
			break;
		case CMD_TARGETXYZ:
			if (tnum==4)
			{
				addx[s]=makenumber(1);
				addy[s]=makenumber(2);
				addz[s]=makenumber(3);
				_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			}
			else
				sysmessage(s, "This command takes three numbers as arguments.");
			break;
		case CMD_TARGETHX:
			if (tnum==2)
			{
				addx[s]=hexnumber(1);
				_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			}
			else
				sysmessage(s, "This command takes one hex number as an argument.");
			break;
		case CMD_TARGETHXY:
			if (tnum==3)
			{
				addx[s]=hexnumber(1);
				addy[s]=hexnumber(2);
				_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			}
			else
				sysmessage(s, "This command takes two hex numbers as arguments.");
			break;
		case CMD_TARGETHXYZ:
			if (tnum==4)
			{
				addx[s]=hexnumber(1);
				addy[s]=hexnumber(2);
				addz[s]=hexnumber(3);
				_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			}
			else
				sysmessage(s, "This command takes three hex numbers as arguments.");
			break;
		case CMD_TARGETID1:
			if (tnum==2)
			{
				addid1[s]=makenumber(1);
				_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			}
			else
				sysmessage(s, "This command takes one number as an argument.");
			break;
		case CMD_TARGETID2:
			if (tnum==3)
			{
				addid1[s] = static_cast<unsigned char>(makenumber(1));
				addid2[s] = static_cast<unsigned char>(makenumber(2));
				_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			}
			else
				sysmessage(s, "This command takes two numbers as arguments.");
			break;
		case CMD_TARGETID3:
			if (tnum==4)
			{
				addid1[s] = static_cast<unsigned char>(makenumber(1));
				addid2[s] = static_cast<unsigned char>(makenumber(2));
				addid3[s] = static_cast<unsigned char>(makenumber(3));
				_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			}
			else
				sysmessage(s, "This command takes three numbers as arguments.");
			break;
		case CMD_TARGETID4:
			if (tnum==5)
			{
				addid1[s] = static_cast<unsigned char>(makenumber(1));
				addid2[s] = static_cast<unsigned char>(makenumber(2));
				addid3[s] = static_cast<unsigned char>(makenumber(3));
				addid4[s] = static_cast<unsigned char>(makenumber(4));
				//clConsole.send("1: %i 2: %i 3: %i 4: %i\n",addid1[s],addid2[s],addid2[s],addid3[s],addid4[s]);
				_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			}
			else
				sysmessage(s, "This command takes four numbers as arguments.");
			break;
		case CMD_TARGETHID1:
			if (tnum==2)
			{
				addid1[s] = static_cast<unsigned char>(hexnumber(1));
				_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			}
			else
				sysmessage(s, "This command takes one hex number as an argument.");
			break;
		case CMD_TARGETHID2:
			if (tnum==3)
			{
				addid1[s] = static_cast<unsigned char>(hexnumber(1));
				addid2[s] = static_cast<unsigned char>(hexnumber(2));
				_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			}
			else
				sysmessage(s, "This command takes two hex numbers as arguments.");
			break;
		case CMD_TARGETHID3:
			if (tnum==4)
			{
				addid1[s] = static_cast<unsigned char>(hexnumber(1));
				addid2[s] = static_cast<unsigned char>(hexnumber(2));
				addid3[s] = static_cast<unsigned char>(hexnumber(3));
				_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			}
			else
				sysmessage(s, "This command takes three hex numbers as arguments.");
			break;
		case CMD_TARGETHID4:
			if (tnum==5)
			{
				addid1[s] = static_cast<unsigned char>(hexnumber(1));
				addid2[s] = static_cast<unsigned char>(hexnumber(2));
				addid3[s] = static_cast<unsigned char>(hexnumber(3));
				addid4[s] = static_cast<unsigned char>(hexnumber(4));
				_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			}
			else
				sysmessage(s, "This command takes four hex numbers as arguments.");
			break;
		case CMD_TARGETTMP:
			if (tnum==2)
			{
				tempint[s]=makenumber(1);
				_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			}
			else
				sysmessage(s, "This command takes a number as an argument.");
			break;
		case CMD_TARGETHTMP:
			if (tnum==2)
			{
				tempint[s]=hexnumber(1);
				_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			}
			else
				sysmessage(s, "This command takes a hex number as an argument.");
			break;

		case CMD_TARGET4OPT:
			addx[s]		= hexnumber(1);
			addy[s]		= hexnumber(2);
			addz[s]		= hexnumber(3);
			tempint[s]	= hexnumber(4);
			addid1[s]	= static_cast<unsigned char>(tnum);
			_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			break;

		case CMD_TARGETSTR:
			if (strlen(Commands->GetAllParams().c_str()) <= 120)
				strcpy(xtext[s], Commands->GetAllParams().c_str());
			_do_target(s, (TARGET_S *)command_table[y].cmd_extra);
			break;

		default:
			sysmessage(s, "BUG: Command has a bad command type set!");
			break;
		}
		return;
	}

	sysmessage(s, "BUG: Should never reach end of command() function!");
}

mstring cCommands::GetAllParams(void)
{
	unsigned int pos = 0;
	mstring dummy;
	pos = command_line.find_first_of(" ");
	if (pos != mstring::npos)
	{
		dummy = command_line.substr(pos);
		dummy.trim();
	}
	return dummy;
}


void cCommands::MakeShop(P_CHAR pc)
{
	if (!pc) return;

	pc->shop=1;
	if (pc->GetItemOnLayer(0x1A) == NULL)
	{
		P_ITEM p1A=Items->SpawnItem(pc,1,"#",0,0x2AF8,0,0);
		if (p1A)
		{
			p1A->setType(ITY_Container);
			p1A->newbieOn();
			pc->Wear(p1A,0x1A);
		}
	}
	
	if (pc->GetItemOnLayer(0x1B) == NULL)
	{
		P_ITEM p1B=Items->SpawnItem(pc,1,"#",0,0x2AF8,0,0);
		if (p1B)
		{
			p1B->setType(ITY_Container);
			p1B->newbieOn();
			pc->Wear(p1B,0x1B);
		}
	}
	
	if (pc->GetItemOnLayer(0x1C) == NULL)
	{
		P_ITEM p1C=Items->SpawnItem(pc,1,"#",0,0x2AF8,0,0);
		if (p1C)
		{
			p1C->setType(ITY_Container);
			p1C->newbieOn();
			pc->Wear(p1C,0x1C);
		}
	}
}

void cCommands::NextCall(int s, int type)
{
	// Type is the same as it is in showgmqueue()
	
	int i;
	int x = 0;
	
	P_CHAR Player = MAKE_CHARREF_LR(currchar(s));
	
	if (Player->callnum != 0)
	{
		donewithcall(s, type);
	}
	if (type == 1) // Player is a GM
	{
		for (i = 1; i < MAXPAGES; i++)
		{
			if (gmpages[i].handled == 0)
			{
				P_CHAR pc = FindCharBySerial(gmpages[i].serial);
				if (pc)
				{
					sysmessage(s, "");
					sprintf(temp, "Transporting to next call: %s", gmpages[i].name.c_str());
					sysmessage(s, temp);
					sprintf(temp, "Problem: %s.", gmpages[i].reason.c_str());
					sysmessage(s, temp);
					sprintf(temp, "Serial number %x", gmpages[i].serial);
					sysmessage(s, temp);
					sprintf(temp, "Paged at %s.", gmpages[i].timeofcall);
					sysmessage(s, temp);
					gmpages[i].handled = 1;
					Player->teleTo(pc->pos);
					Player->callnum = i;
					x++;
				}// if
				if (x > 0)
					break;
			}// if
		}// for
		if (x == 0)
			sysmessage(s, "The GM queue is currently empty");
	} // end first IF
	else // Player is only a counselor
	{
		x = 0;
		for (i = 1; i < MAXPAGES; i++)
		{
			if (counspages[i].handled == 0)
			{
				P_CHAR pc = FindCharBySerial(counspages[i].serial);
				if (pc)
				{
					sysmessage(s, "");
					sprintf(temp, "Transporting to next call: %s", counspages[i].name.c_str());
					sysmessage(s, temp);
					sprintf(temp, "Problem: %s.", counspages[i].reason.c_str());
					sysmessage(s, temp);
					sprintf(temp, "Serial number %x", counspages[i].serial);
					sysmessage(s, temp);
					sprintf(temp, "Paged at %s.", counspages[i].timeofcall);
					sysmessage(s, temp);
					gmpages[i].handled = 1;
					Player->teleTo(pc->pos);
					Player->callnum = i;
					x++;
					break;
				}// if
			}// else
			if (x > 0)
				break;
		}// for
		if (x == 0)
			sysmessage(s, "The Counselor queue is currently empty");
	}// if
}

void cCommands::KillSpawn(int s, int r)  //courtesy of Revana
{
	int killed=0;
	char temp[512];
	r++; // synch with 1-indexed real storage, casue 0 is no region indicator, LB

	if (r<=0 || r>=255) return;

	sysmessage(s,"Killing spawn, this may cause lag...");

	P_CHAR pc;
	AllCharsIterator aci;
	for (aci.Begin(); (pc=aci.GetData()) != NULL; aci++)
	{
		if (pc->spawnregion==r)
		{
			bolteffect(pc, true);
			soundeffect2(pc, 0x0029);
			Npcs->DeleteChar(pc);
			killed++;
		}
	}

	PC_ITEM pi;
	AllItemsIterator aii;
	for (aii.Begin(); (pi = aii.GetData()) != aii.End(); aii++)
	{
		if (pi->spawnregion==r)
		{			
			Items->DeleItem(pi->unconst());
			killed++;
		}
	}

	gcollect();
	sysmessage(s, "Done.");
	sprintf(temp, "%i of Spawn %i have been killed.",killed,r-1);
	sysmessage(s, temp);
}

void cCommands::RegSpawnMax (int s, int r) // rewrite LB
{
//	sysmessage(s, "Command is disabled. Use respawn instead.");	//might cause allocation problems (Duke)
//	return;

	char temps[100];

	spawnregion_st *SR = GetSpawnRegion(r);
	if (!SR) return;

	int spawn = GetSpawnDelta(SR);

	sprintf(temps, "Region %d is Spawning %d items/NPCs, this might cause some lag.", r, spawn);
	sysbroadcast(temps);

	SpawnSomeMore(SR, spawn);

	sprintf(temps, "Done. %d total NPCs/items spawned in Spawnregion %d.",spawn,r);
	sysmessage(s, temps);
}

void cCommands::RegSpawnNum (int s, int r, int n) // rewrite by LB
{
//	sysmessage(s, "Command is disabled. Use respawn instead.");	//might cause allocation problems (Duke)
//	return;

	char temps[100];
	
	spawnregion_st *SR = GetSpawnRegion(r);
	if (!SR) return;
	
	int spawn = GetSpawnDelta(SR);
	if (n > spawn) 
	{		
		sprintf(temps, "%d too many for region %d, spawning %d to reach MAX instead.",n, r, spawn);
		sysmessage(s, temps);
		n=spawn;
	}		
	sprintf(temps, "Region %d is Spawning: %d NPCs/items, this will cause some lag.", r, spawn);
	sysbroadcast(temps);
	
	SpawnSomeMore(SR, spawn);
	
	sprintf(temps, "Done. %d total NPCs/items spawned in Spawnregion %d.",spawn,r);
	sysmessage(s, temps);		
}

void cCommands::KillAll(int s, int percent, const char* sysmsg)
{
	sysmessage(s,"Killing all characters, this may cause some lag...");
	sysbroadcast(sysmsg);

	P_CHAR pc;
	AllCharsIterator aci;
	for (aci.Begin(); (pc=aci.GetData()) != NULL; aci++)
	{
		if (!pc->isGM())
		{
			if (rand()%100+1<=percent)
			{
				bolteffect(pc, true);
				soundeffect2(pc, 0x0029);
				deathstuff(pc);
			}
		}
	}
	sysmessage(s, "Done.");
}

//////////////
// Name:	CPage
// History:	UnKnown
// Purpose:	handle help button (Calls Counselor Call Menus up)
//
void cCommands::CPage(int s, char *reason) // 
{
	int i, x;
	int x2=0;

	P_CHAR Player = MAKE_CHARREF_LR(currchar(s));
	
	x=0;
	
	for(i=1;i<MAXPAGES;i++)
	{
		if (counspages[i].handled==1)
		{
			counspages[i].handled=0;
			counspages[i].name = Player->name;
			counspages[i].reason = reason;
			counspages[i].serial = Player->serial;
			time_t current_time = time(0);
			struct tm *local = localtime(&current_time);
			sprintf(counspages[i].timeofcall, "%02d:%02d:%02d", local->tm_hour, local->tm_min, local->tm_sec);
		
			Player->playercallnum=i;
			Player->pagegm=2;
			x2++;
			break;
		}
	}
	if (x2==0)
	{
		sysmessage(s,"The Counselor Queue is currently full. Contact the shard operator");
		sysmessage(s,"and ask them to increase the size of the queue.");
	}
	else
	{
		if (strcmp(reason,"OTHER"))
		{
			Player->pagegm=0;
			char serstring[33];
			LongToHexChunks(Player->serial,serstring);
			sprintf(temp, "Counselor Page from %s %s: %s", Player->name, serstring, reason);

			P_CHAR pc;
			AllPlayersIterator api;
			for (pc = api.First(); pc != NULL; pc = api.Next())
			{
				if (pc->isCounselor())
				{
					x=1;
					sysmessage(api.getSocket(), temp);
				}				
				
				if (x==1)
				{
					sysmessage(s, "Available Counselors have been notified of your request.");
				}
				else sysmessage(s, "There was no Counselor available to take your call.");
			}
		}
		else sysmessage(s,"Please enter the reason for your Counselor request");
	}
}

////////////
// Name:	GMPage
// History:	by unknown
// Purpose:	Help button (Calls GM Call Menus up)
//
void cCommands::GMPage(int s, char *reason)
{
	int i, x=0;
	int x2=0;
	
	P_CHAR Player = MAKE_CHARREF_LR(currchar(s));

	for(i=1;i<MAXPAGES;i++)
	{
		if (gmpages[i].handled==1)
		{
			gmpages[i].handled=0;
			gmpages[i].name = Player->name;
			gmpages[i].reason = reason;
			gmpages[i].serial = Player->serial;
			time_t current_time = time(0);
			struct tm *local = localtime(&current_time);
			sprintf(gmpages[i].timeofcall, "%02d:%02d:%02d", local->tm_hour, local->tm_min, local->tm_sec);
		
			Player->playercallnum=i;
			Player->pagegm=1;
			x2++;
			break;
		}
	}
	if (x2==0)
	{
		sysmessage(s,"The GM Queue is currently full. Contact the shard operator");
		sysmessage(s,"and ask them to increase the size of the queue.");
	}
	else
	{
		if (strcmp(reason,"OTHER"))
		{
			Player->pagegm=0;
			char serstring[33];
			LongToHexChunks(Player->serial,serstring);
			sprintf(temp, "Page from %s %s: %s", Player->name, serstring, reason);
			
			P_CHAR pc;
			AllPlayersIterator api;
			for (pc = api.First(); pc != NULL; pc = api.Next())
			{
				if (pc->isGM())				
				{
					x=1;
					sysmessage(api.getSocket(), temp);
				}
				if (x==1)
				{
					sysmessage(s, "Available Game Masters have been notified of your request.");
				}
				else sysmessage(s, "There was no Game Master available to take your call.");
			}
		}
		else sysmessage(s,"Please enter the reason for your GM request");
	}
}

void cCommands::DyeItem(int s) // Rehue an item
{
	int body,b,k;

	P_ITEM pi = FindItemBySerPtr( &buffer[s][1] );
	if (pi)
	{
		unsigned short color = ShortFromCharPtr(&buffer[s][7]);
		
		if (!dyeall[s])
		{
			if (color<0x0002 || color>0x03E9)
			{
				color=0x03E9;
			}
		}
		
		if (!(color&0xC000))
		{
			pi->setColor(color);
		}
		
		if (color==17969)
		{
			pi->setColor(color);
		}
		pi->refresh();//AntiChrist
		
		soundeffect( s, 0x02, 0x3e ); // plays the dye sound, LB
		return;
	}
	
	P_CHAR pc = FindCharBySerPtr(&buffer[s][7]);
	if (pc)
	{
		P_CHAR Player = MAKE_CHARREF_LR(currchar(s));
		if ( !(Player->isGM() ) ) return; // Only gms dye characters
		k=(buffer[s][7]<<8)+buffer[s][8];
		
		body=pc->id();
		b=k&0x4000; 
		
		if ( ( ( k>>8 ) < 0x80 ) && body >= 0x0190 && body <= 0x0193 ) k+= 0x8000;
		
		if (b==16384 && (body >=0x0190 && body<=0x03e1)) k=0xf000; // but assigning the only "transparent" value that works, namly semi-trasnparency.
		
		if (k!=0x8000) 
		{	
			pc->skin = pc->xskin = k;
			updatechar(pc);
		}
	}
	soundeffect(s, 0x02, 0x3E); // plays the dye sound, LB
}


void cCommands::SetItemTrigger(int s, P_ITEM pi)
{
	if (!pi) return;

	sysmessage(s,"Item triggered");
	pi->trigger=addx[s];
}

void cCommands::SetTriggerType(int s, P_ITEM pi)
{
	if (!pi) return;

	sysmessage(s,"Trigger type set");
	pi->trigtype=addx[s];
}

void cCommands::SetTriggerWord(int s, P_CHAR pc)
{
	if (pc)
	{
		sysmessage(s,"Trigger word set");
		strcpy(pc->trigword,xtext[s]);
	}
}

void cCommands::AddHere(int s, char z)
{
	int pileable=0;
	short id = (addid1[s]<<8)+addid2[s];
	const tileData *tile;
	
	tile = TileData->SeekTile(id);
	if (TileData->isStackable(tile->flags)) pileable=1;
	
	P_CHAR Player = MAKE_CHARREF_LR(currchar(s));
	
	P_ITEM pi=Items->SpawnItem(Player, 1, "#", pileable, id, 0, 0);
	if (pi)
	{
		pi->doordir=0;
		pi->resetPriv();		

		pi->MoveTo(Player->pos.x,Player->pos.y,z);
		pi->refresh();//AntiChrist
	}
	addid1[s]=0;
	addid2[s]=0;
}


void cCommands::SetNPCTrigger(int s, P_CHAR pc)
{
	if (pc)
	{
		//   if (pc->npc)
		//   {
		sysmessage(s,"NPC triggered");
		pc->trigger=addx[s];
		//   }else{
		//    sysmessage(s,"You can not trigger Player Characters");
		//   }
	}
}


void cCommands::WhoCommand(int s, int type,int buttonnum)
{
	char sect[512];
	int length, length2, textlines;
	unsigned int line, i;
	char menuarray[7*(MAXCLIENT)+50][50];  /** lord binary **/
	char menuarray1[7*(MAXCLIENT)+50][50]; /** the ( IS important !!! **/
	unsigned int linecount=0;
	unsigned int linecount1=0;

	if (buttonnum < 0)
		return;


	int serial=whomenudata[buttonnum];
	P_CHAR pc_c = FindCharBySerial( serial ); // find selected char ...
	if (!pc_c) 
	{
		sysmessage(s,"selected character not found");
		return;
	}
	
	//--static pages
	strcpy(menuarray[linecount++], "nomove");
	strcpy(menuarray[linecount++], "noclose");
	strcpy(menuarray[linecount++], "page 0");
	strcpy(menuarray[linecount++], "resizepic 0 0 5120 260 280");    //The background
	strcpy(menuarray[linecount++], "button 20 240 2130 2129 1 0 1"); //OKAY
	strcpy(menuarray[linecount++], "text 20 10 300 0");           //text <Spaces from Left> <Space from top> <Length, Color?> <# in order>
	strcpy(menuarray[linecount++], "text 20 30 300 1");
	
	//--Command Button Page 
	strcpy(menuarray[linecount++], "page 1");
	strcpy(menuarray[linecount++], "text 20 60 300 2");	//goto text
	strcpy(menuarray[linecount++], "button 150 60 1209 1210 1 0 200"); //goto button
	strcpy(menuarray[linecount++], "text 20 80 300 3");	//gettext
	strcpy(menuarray[linecount++], "button 150 80 1209 1210 1 0 201"); //get button
	strcpy(menuarray[linecount++], "text 20 100 300 4");	//Jail text
	strcpy(menuarray[linecount++], "button 150 100 1209 1210 1 0 202"); //Jail button
	strcpy(menuarray[linecount++], "text 20 120 300 5");	//Release text
	strcpy(menuarray[linecount++], "button 150 120 1209 1210 1 0 203"); //Release button
	strcpy(menuarray[linecount++], "text 20 140 300 6");	//Kick user text
	strcpy(menuarray[linecount++], "button 150 140 1209 1210 1 0 204"); //kick button
	strcpy(menuarray[linecount++], "text 20 180 300 7");

	length=21;
	length2=1;
	
	for(line=0;line<linecount;line++)
	{
		if (strlen(menuarray[line])==0)
			break;
		{
			length+=strlen(menuarray[line])+4;
			length2+=strlen(menuarray[line])+4;
		}
	}
	
	length+=3;
	textlines=0;
	line=0;
	
	sprintf(menuarray1[linecount1++], "User %i selected (account %i)",buttonnum,pc_c->account);
	sprintf(menuarray1[linecount1++], "Name: %s",pc_c->name);   
	sprintf(menuarray1[linecount1++], "Goto Character:");
	sprintf(menuarray1[linecount1++], "Get Character:");
	sprintf(menuarray1[linecount1++], "Jail Character:");
	sprintf(menuarray1[linecount1++], "Release Character:");
	sprintf(menuarray1[linecount1++], "Kick Character:");
	sprintf(menuarray1[linecount1++], "Serial# [%i]",pc_c->serial);   
	
	
	for(line=0;line<linecount1;line++)
	{
		if (strlen(menuarray1[line])==0)
			break;
		length+=strlen(menuarray1[line])*2 +2;
		textlines++;
	}

	unsigned char gump1[21]={0,};
	unsigned char gump2[3]={0,};
	unsigned char gump3[2]={0,};

	LongToCharPtr(1, &gump1[3]); // can be anything except 0		
	gump1[10]=type; // Gump Number
	gump1[19]=length2>>8;
	gump1[20]=length2%256;

	if (length>=32000)
	{
		sysmessage(s, "Whomenu gump data too huge, plz try wholist filtering");
		return;
	}

	XStartPkt(0xb0, length);
	XAddPkt(&gump1[0+3], 21-3);
	
	for(line=0;line<linecount;line++)
	{
		sprintf(sect, "{ %s }", menuarray[line]);
		XAddPkt((unsigned char*)&sect[0], strlen(sect));	
	}
	
	gump2[1]=textlines>>8;
	gump2[2]=textlines%256;
	
	XAddPkt(&gump2[0], 3);
	
	for(line=0;line<linecount1;line++)
	{
		if (strlen(menuarray1[line])==0)
			break;
		{
			gump3[0]=strlen(menuarray1[line])>>8;
			gump3[1]=strlen(menuarray1[line])%256;
			XAddPkt(&gump3[0], 2);
			gump3[0]=0;
			for (i=0;i<strlen(menuarray1[line]);i++)
			{
				gump3[1]=menuarray1[line][i];
				XAddPkt(&gump3[0], 2);
			}
		}
	}

	XFinishPkt(s);
}

void cCommands::MakePlace(int s, int i) // Decode a teleport location number into X/Y/Z
{
	int x = 0, y = 0, z = 0, loopexit = 0;

	Script* Scp = findSection(location_script,"LOCATION",i);
	if (Scp==NULL) // avoid crashes if not found
	{
		sysmessage(s,"invalid place");
		return;
	}
	while (Scp->NextLineSplitted() && (++loopexit < MAXLOOPS) )
	{
		if (Scp->getKeyVal("X",	&x))	continue;
		if (Scp->getKeyVal("Y",	&y))	continue;
		if (Scp->getKeyVal("Z",	&z))	continue;
	}
	addx[s] = x;
	addy[s] = y;
	addz[s] = z;
	Scp->Close();
}


void cCommands::DupeItem(P_CLIENT ps, P_ITEM pi, int amount)
{
	if (!pi) return;
	if (pi->corpse) return;

	P_CHAR Player = ps->getPlayer();
	if (!Player) return;
	P_ITEM Pack = Player->getBackpack();
	if (!Pack) return;

    P_ITEM Ic = Items->CloneItem(pi);
	if (!Ic) return;
	
	Ic->SetContSerial(Pack->serial);
	Ic->SetOwnSerial(pi->ownserial);
	Ic->SetSpawnSerial(pi->spawnserial);
	Ic->layer=0;	// it's created in a backpack
	Ic->amount=amount;
	
	Ic->refresh();//AntiChrist
}

void cCommands::ShowGMQue(int s, int type) // Shows next unhandled call in the GM queue
{
	
	// Type is 0 if it is a Counselor doing the command (or a GM doing /cq) and 1 if it is a GM
	
	int i;
	int x=0;
	
	if (type==1) //Player is a GM
	{
		for(i=1;i<MAXPAGES;i++)
		{
			if (gmpages[i].handled==0)
			{
				if (x==0)
				{
					sysmessage(s,"");
					sprintf(temp,"Next unhandled page from %s", gmpages[i].name.c_str());
					sysmessage(s,temp);
					sprintf(temp,"Problem: %s.", gmpages[i].reason.c_str());
					sysmessage(s,temp);
					sprintf(temp,"Serial number %x", gmpages[i].serial);
					sysmessage(s,temp);
					sprintf(temp,"Paged at %s.", gmpages[i].timeofcall);
					sysmessage(s,temp);
				}
				x++;
			}
		}
		if (x>0)
		{
			sprintf(temp,"Total pages in queue: %i",x);
			sysmessage(s,"");
			sysmessage(s,temp);
		}
		else sysmessage(s,"The GM queue is currently empty");
	} //end of first if
	else //Player is a counselor so show counselor queue
	{
		for(i=1;i<MAXPAGES;i++)
		{
			if (counspages[i].handled==0)
			{
				if (x==0)
				{
					sysmessage(s,"");
					sprintf(temp,"Next unhandled page from %s", counspages[i].name.c_str());
					sysmessage(s,temp);
					sprintf(temp,"Problem: %s.", counspages[i].reason.c_str());
					sysmessage(s,temp);
					sprintf(temp,"Serial number %x", counspages[i].serial);
					sysmessage(s,temp);
					sprintf(temp,"Paged at %s.", counspages[i].timeofcall);
					sysmessage(s,temp);
				}
				x++;
			}
		}
		if (x>0)
		{
			sprintf(temp,"Total pages in queue: %i",x);
			sysmessage(s,"");
			sysmessage(s,temp);
		}
		else sysmessage(s,"The Counselor queue is currently empty");
	}
}
// new wipe function, basically it prints output on the console when someone wipes so that
// if a malicious GM wipes the world you know who to blame

void cCommands::Wipe(int s)
{
	P_CHAR Player = MAKE_CHARREF_LR(currchar(s));
	
	LogMessageVar("LoneWolf: %s has initiated an item wipe",Player->name);
	
	PC_ITEM pi;
	AllItemsIterator aii;
	for (aii.Begin(); (pi=aii.GetData())!=aii.End(); aii++)
	{
		if (pi->isInWorld() && pi->wipe==0)
		{
			Items->DeleItem(pi->unconst());
		}
	}
	sysbroadcast("All items have been wiped."); 
}

void cCommands::PartialWipe(int s, const cRectangle& rect, bool inverse)
{
	P_CHAR Player = MAKE_CHARREF_LR(currchar(s));
	
	LogMessageVar("LoneWolf: %s has initiated a partial item wipe",Player->name);
	
	PC_ITEM pi;
	AllItemsIterator aii;
	for (aii.Begin(); (pi=aii.GetData())!=aii.End(); aii++)
	{
		if (!pi->isInWorld() || pi->wipe)
			continue;
		if (pi->pos.IsIn(rect)) 
		{
			if (!inverse)
				Items->DeleItem(pi->unconst());
		}
		else
		{
			if (inverse)
				Items->DeleItem(pi->unconst());
		}
	}
}

void cCommands::Possess(int s) 
{
	sysmessage(s, "wiping wordfile, hold on...");
	sysmessage(s, "just kidding");
	sysmessage(s, "command currently disabled");
}

void cCommands::GotoCur(UOXSOCKET s)
// Goes to the current call in the GM/Counsellor Queue
{
	int x=0;
	P_CHAR Player = MAKE_CHARREF_LR(currchar(s));
	
	if (Player->callnum<=0)
	{
		sysmessage(s,"You are not currently on a call.");
	}
	else
	{  
		P_CHAR pc = FindCharBySerial( gmpages[Player->callnum].serial );
		if (pc)
		{
			Player->teleTo(pc->pos);
			sysmessage(s,"Transporting to your current call.");
			x++;
		}
	}
}

void cCommands::GMtransfer(UOXSOCKET s)
// Escilate a Counsellor Page into the GM Queue
{
	int i;
	int x2=0;
	P_CHAR Player = MAKE_CHARREF_LR(currchar(s));
	
	if (Player->callnum>=0)
	{
		if (!Player->isGM()) //Char is a counselor
		{
			for(i=1;i<MAXPAGES;i++)
			{
				if (gmpages[i].handled==1)
				{
					gmpages[i].handled=0;
					gmpages[i].name = counspages[Player->callnum].name;
					gmpages[i].reason = counspages[Player->callnum].reason;
					gmpages[i].serial = counspages[Player->callnum].serial;
					time_t current_time = time(0);
					struct tm *local = localtime(&current_time);
					sprintf(gmpages[i].timeofcall, "%02d:%02d:%02d", local->tm_hour, local->tm_min, local->tm_sec);
					x2++;
					break;
				}
			}
			if (x2==0)
			{
				sysmessage(s,"The GM Queue is currently full. Contact the shard operator");
				sysmessage(s,"and ask them to increase the size of the queue.");
			}
			else
			{
				sysmessage(s,"Call successfully transferred to the GM queue.");
				donewithcall(s,1);
			}
		}
		else
		{
			sysmessage(s,"Only Counselors may use this command.");
		}
	}
	else
	{
		sysmessage(s,"You are not currently on a call");
	}
}

bool cCommands::GetPageReason(UOXSOCKET s, P_CHAR Player, char* comm)
{
	P_CHAR pc;
	AllPlayersIterator api;
	if (Player->pagegm == 1)
	{
		gmpages[Player->playercallnum].reason = comm;
		sprintf(temp, "GM Page from %s [%x]: %s",Player->name, Player->serial, comm);
		int x = 0;
		for (pc = api.First(); pc != NULL; pc = api.Next())
		{
			if (pc->isGM())
			{
				x = 1;
				sysmessage(api.getSocket(), temp);
			}
		}
		if (x == 1)
			sysmessage(s, "Available Game Masters have been notified of your request.");
		else 
			sysmessage(s, "There was no Game Master available to take your call.");
		Player->pagegm = 0;
		return true;
	}
	if (Player->pagegm == 2) // Counselor page
	{
		counspages[Player->playercallnum].reason = comm;
		sprintf(temp, "Counselor Page from %s [%x]: %s", Player->name, Player->serial, comm);
		int x = 0;
		for (pc = api.First(); pc != NULL; pc = api.Next())
		{
			if (pc->isCounselor())
			{
				x = 1;
				sysmessage(api.getSocket(), temp);
			}	
		}
		if (x == 1)
			sysmessage(s, "Available Counselors have been notified of your request.");
		else 
			sysmessage(s, "There was no Counselor available to take your call.");
		Player->pagegm = 0;
		return true;
	}
	return false;
}







