/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//


// Include files

#include "coord.h"

// System Includes
#include <algorithm>
#include <cmath>
using namespace std;
#include "rectangle.h"

#define COORD_ALL_PLANES 255
#define COORD_HIDDEN_PLANE 254


cCoord::cCoord(void)
{
	x = y = z = map  = 0;
}

cCoord::cCoord(const cCoord& clCoord)
{
	(*this) = clCoord ;
}

cCoord::cCoord(UI16 uiX, UI16 uiY, SI08 siZ, UI08 uiMap)
{
	x = uiX;
	y = uiY;
	z = siZ;
	map = uiMap;
}


/*!
 * Calculates the distance between 2 coordinates. If they are in different planes and maps, the
 * return value is infinite (0xFFFFFFFF). The exception is if one of the planes is COORD_ALL_PLANES
 *
 * @param &src : Coordinate to compare
 *
 * @return UI32  : distance result
 */
UI32 cCoord::distance(const cCoord &src) const
{
	UI32 uiResult = ~0; // Initialize with *infinite*
	if (map == src.map)
	{
		RF64 rfResult = sqrt(static_cast<RF64>((x - src.x)*(x - src.x) + (y - src.y)*(y - src.y)));
		//		RF64 rfResult = sqrt(static_cast<RF64>((x * src.x + y * src.y + z * src.z)));
		uiResult = static_cast<UI32>(floor(rfResult)); // truncate
	}
	return uiResult;
}

// Operators
cCoord& cCoord::operator=(const cCoord& clCoord)
{
	x = clCoord.x;
	y = clCoord.y;
	z = clCoord.z;
	map = clCoord.map;	
	return (*this);
}

bool cCoord::operator ==(const cCoord& src) const
{
	return (x == src.x && y == src.y && z == src.z && map == src.map);
}

cCoord cCoord::operator+(const cCoord& src) const
{
	return cCoord(this->x + src.x, this->y + src.y, this->z + src.z);
}

bool cCoord::IsSameXY(const cCoord &other) const
{
	return (x==other.x && y==other.y);
}
bool cCoord::IsSameXY(short ox, short oy) const
{
	return (x==ox && y==oy);
}
bool cCoord::IsIn(const cRectangle& rect) const
{
	return rect.includes(x,y);
}
void cCoord::Add(short dx, short dy, short dz)
{
	x+=dx;
	y+=dy;
	z+=dz;
}

