/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

/* House code for deed creation by Tal Strake, revised by Cironian */

#include "house.h"

#include "boats.h"
#include "Client.h"
#include "itemid.h"
#include "mapstuff.h"
#include "rectangle.h"
#include "regions.h"
#include "SndPkg.h"
#include "utilsys.h"
#include "walking.h"
#include "multi.h"
#include "network.h"

#include "wolfpack.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "house.cpp"

bool CheckBuildSite(const cCoord& pos, int sx, int sy);

void mtarget(int s, int targnum, char b1, char b2, char *txt)
{
	unsigned char multitarcrs[27]="\x99\x01\x40\x01\x02\x03\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x02\x00\x00\x00\x00\x00\x00";

	targetok[s]=1; 
	LongToCharPtr(targnum,&multitarcrs[2]);
	multitarcrs[18]=b1;
	multitarcrs[19]=b2;
	sysmessage(s, txt);

	XStartPkt(0x99, 26);
	XAddPkt(&multitarcrs[0+1], 26-1);
	XFinishPkt(s);
}

bool isBoat(short id) {return (id>=0x4000 && id<=0x4018);}

void makeKeysForHouse(P_CHAR Player, P_ITEM House, short houseid)
{
	short id;
	char* p;
	if (isTent(houseid))	{p="a tent key"; id=0x1010;}	//iron key for tents
	else if (isBoat(houseid)){p="a ship key"; id=0x1013;}	//Boats -Rusty Iron Key
	else					{p="a house key";id=0x100F;}	//gold key for everything else
	P_ITEM Key=Items->SpawnItem(Player, 1, p, 0, id, 0,1);
	Key->setMore1_4(House->serial);//use the house's serial for the more on the key to keep it unique
	Key->setType(ITY_Key);
	Key->newbieOn();

	// create a second key and put it in the Player's bankbox
	P_ITEM Bankbox = Player->GetBankBox();
	if (Bankbox)
	{
		Key=Items->SpawnItem(Player, 1, p, 0, id, 0,1);
		Key->setMore1_4(House->serial);
		Key->setType(ITY_Key);
		Key->newbieOn();
		Bankbox->AddItem(Key);
	}
}

// prototype code for 'houses in town'
// not usable yet. Still working on it (Duke,19.03.2002)
bool initTownHouseArea(P_ITEM House)
{
	Script *Scp=findSection(townhouse_scp,"TOWNHOUSE",House->morex);
	if (!Scp) return false;

	House->thi = new cItem::cTownHouseInfo;

	int loopexit=0;
	while (Scp->NextLineSplitted() && (++loopexit < 50))	// no more than 50 lines in one section
	{
		if (Scp->getKeyVal("RECT",			&(House->thi->area)))		continue;
		if (Scp->getKeyVal("DOOR1",			&(House->thi->door1)))		continue;
		if (Scp->getKeyVal("DOOR2",			&(House->thi->door2)))		continue;
		if (Scp->getKeyVal("SIGN",			&(House->thi->sign)))		continue;
		if (Scp->getKeyVal("LOCKDOWNMAX",	&(House->lockdownmax)))		continue;
		if (Scp->getKeyVal("SECUREMAX",		&(House->securemax)))		continue;
	}
	Scp->Close();
	return true;
}

PC_ITEM findBigItemAt(cCoord& pos, int bits)
{
	int lastdist=30;
	int ret;

	PC_ITEM pi;
	int loopexit=0;
	RegGrid3x3Iterator4Items rg3ii(pos);
	while((pi = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (((bits & 1) &&	// boats
			(pi->id() >= 0x4000 && pi->id() <= 0x4018)) ||
			((bits & 2) &&	// multi-houses
			ishouse(pi)) )
		{
			ret = pos.distance(pi->pos);
			if (ret<=lastdist)
			{
				lastdist=ret;
				if (Boats->inmulti(pos, pi))
					return pi;
			}
		}
		
		if ((bits & 4) &&	// static houses
			(pi->isType(ITY_TownHouse)))
		{
			if (!pi->thi) continue;
			if (!pi->thi->area) continue;
			if (pos.IsIn(*(pi->thi->area)))
				return pi;
		}
	}
	return NULL;
}

PC_ITEM findBoatAt(cCoord& pos)
{
	return findBigItemAt(pos, 1);
}

PC_ITEM findHouseAt(cCoord& pos)
{
	return findBigItemAt(pos, 2+4);
}

P_ITEM getTownHouseItem(cCoord& pos, int lookingfor)
{
	PC_ITEM pi;
	int loopexit=0;
	RegGrid3x3Iterator4Items rg3ii(pos);
	while((pi = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (pi->pos.IsSameXY(pos))
		{
			if (lookingfor == 1 && IsDoor(pi->id()))
			{
				return pi->unconst();
			}
			if (lookingfor == 2 && IsSign(pi->id()))
			{
				return pi->unconst();
			}
		}
	}
	return NULL;
}
void deedTownHouse(UOXSOCKET s,P_ITEM pi)
{
	sysmessage(s, "Sorry, redeeding is not yet implemented!");
}
void placeTownHouse(P_CLIENT ps, cCoord& pos)
{
	P_CHAR Player = ps->getPlayer();
	P_ITEM pi = Player->getDclickedItem();
	if (!pi) return;

	if (!pi->isType(ITY_TownHouseDeed)) return;

	if (!pi->thi)		// make sure houseinfo is initialized
	{
		if (!initTownHouseArea(pi))
		{
			ps->SysMsg("You have been betrayed! This deed will never work!");
			return;
		}
	}
	if (!pi->thi->area) return;
	if (!pi->thi->door1) return;
	if (!pi->thi->sign) return;

	if (!pos.IsIn(*(pi->thi->area)))
	{
		ps->SysMsg("You have to place the deed inside your new home!");
		return;
	}

	P_ITEM Door1, Door2=NULL;
	bool missingDoor = false;
	Door1 = getTownHouseItem(*(pi->thi->door1),1);
	if (!Door1) missingDoor=true;

	if (pi->thi->door2)					// should there be a second door ?
	{
		Door2 = getTownHouseItem(*(pi->thi->door2),1);
		if (!Door2) missingDoor=true;

	}
	if (missingDoor)	// either the GM didn't place them at all or they are out of place because they are OPEN
	{
		ps->SysMsg("Please close the door!");
		return;
	}
	
	pi->setType(ITY_TownHouse);
	pi->setNotMovable();
	pi->visible=2;			// only GMs can see it
	
	pi->SetOwnSerial(Player->serial);
	pi->SetContSerial(-1);
	pi->MoveTo2(pos);
	
	makeKeysForHouse(Player, pi, pi->id());
	
	if (Door1)
		Door1->setMore1_4(pi->serial);	// make it lockable with the house key
	if (Door2)							// is there a second door ?
		Door2->setMore1_4(pi->serial);	// make it lockable too
	
	P_ITEM Sign = getTownHouseItem(*(pi->thi->sign),2);
	if (Sign)
	{
		Sign->setMore1_4(pi->serial);
		Sign->SetOwnSerial(Player->serial);
		Sign->setType(ITY_Sign);
		Sign->morex=4;					// 4 is # of house gump menu
	}
	
	ps->SysMsg("Welcome to your new home!");
}

/////////////
// Name:	buildhouse
// Purpose:	Triggered by double clicking a deed-> the deed's moreX is read
//			for the house section in house.scp. Extra items can be added
//			using HOUSE ITEM, (this includes all doors!) and locked "LOCK"
//			Space around the house with SPACEX/Y and CHAR offset CHARX/Y/Z
// History:	by UnKnown
//			Rewrite by Zippy (onlynow@earthlink.net) Date 1/24/99
//			Lots of client crash fixes by LB, 25-dec 1999
//
void buildHousePass1(int s, int i) {buildhouse(s,i,1);}
void buildHousePass2(int s, int i) {buildhouse(s,i,2);}
void buildhouse(int s, int scpno, int pass)
{
	int x,y,loopexit=0;//where they click, and the house/key items
	signed char z;
	int sx=0,sy=0;                                  //space around the house needed
	int k,ll=0,tmp;                                            //Temps
	int hitem[100],icount=0;//extra "house items" (up to 100)
	char sect[512];                         //file reading
	unsigned char id1=0,id2=0;                                   //house ID
	char itemsdecay = 0;            // set to 1 to make stuff decay in houses
	long int pos;                                   //for files...
	int cx=0,cy=0,cz=8;             //where the char is moved to when they place the house (Inside, on the steps.. etc...)(Offset)
	int boat=0;//Boats
	int hdeed=0;//deed id #		
	int norealmulti=0,nokey=0,othername=0;
	char name[512];
	unsigned short id=0;
	int sm=2, ldm=100;

	P_CHAR Player = MAKE_CHARREF_LR(currchar(s));

	if (buffer[s][11]==0xFF && buffer[s][12]==0xFF && buffer[s][13]==0xFF && buffer[s][14]==0xFF) return; // do nothing if user cancels, avoids CRASH!
	
	hitem[0]=0;//avoid problems if there are no HOUSE_ITEMs by initializing the first one as 0
	if (scpno)
	{
		Script* Scp = findSection(house_script,"HOUSE",scpno);
		if (!Scp) return;
		while (Scp->NextLineSplitted() && (++loopexit < MAXLOOPS) )
		{
			if (!strcmp(script1,"ID"))
			{
				tmp=hstr2num(script2);
				id1 = (unsigned char)(tmp>>8);
				id2 = (unsigned char)(tmp%256);
				id=tmp;
			}
			else if (!strcmp(script1,"LOCKDOWNMAX"))	{	ldm=str2num(script2);	}
			else if (!strcmp(script1,"SECUREMAX"))		{	sm=str2num(script2);	}
			else if (!strcmp(script1,"SPACEX"))			{	sx=str2num(script2)+1;	}
			else if (!strcmp(script1,"SPACEY"))			{	sy=str2num(script2)+1;	}
			else if (!strcmp(script1,"CHARX"))			{	cx=str2num(script2);	}
			else if (!strcmp(script1,"CHARY"))			{	cy=str2num(script2);	}
			else if (!strcmp(script1,"CHARZ"))			{	cz=str2num(script2);	}
			else if (!strcmp(script1,"ITEMSDECAY"))		{	itemsdecay=str2num(script2);}
			else if (!strcmp(script1,"HOUSE_ITEM"))
			{
				hitem[icount]=str2num(script2);
				icount++;
			}
			else if (!strcmp(script1,"HOUSE_DEED"))		hdeed=str2num(script2);
			else if (!strcmp(script1,"BOAT"))			boat=1;
			else if (!strcmp(script1,"NOREALMULTI"))	norealmulti=1;	// LB bugfix for pentas crashing client
			else if (!strcmp(script1,"NOKEY"))			nokey=1;
			else if (!strcmp(script1,"NAME")) 
			{ 
				strcpy(name,script2);
				othername=1;
			}
		}
		Scp->Close();
		
		if (!id1 && !norealmulti)
		{
			LogWarningVar("Bad house script # %i!",scpno);
			return;
		}
	}
	
	if (pass==1)
	{
		if (scpno)
		{
			addid1[s]=0x40;addid2[s]=100;//Used in addtarget
			if (norealmulti)
				target(s, 245, "Select a place for your structure: ");
			else
				mtarget(s, 246, id1-0x40, id2, "Select location for building.");
		}
		else
		{
			mtarget(s, 246, addid1[s]-0x40, addid2[s], "Select location for building.");
		}
		return;
	}

	if (pass==2)//for when we come back after they target something
	{

		P_ITEM pHouse=NULL;
		x=(buffer[s][11]<<8)+buffer[s][12];//where they targeted
		y=(buffer[s][13]<<8)+buffer[s][14];
		z=buffer[s][16]+Map->TileHeight((buffer[s][17]<<8)+buffer[s][18]);
		cCoord BuildPos(x,y,z,Player->pos.map);

		if (!norealmulti || (norealmulti && (id1!=0 && id<0x4000) ) ) // only create the multi object, check if target is ok for building, etc,etc if it's a REAL multi !!! (or nonrealmulti with defined mainitem thats no real multi)
		{
			
			if (!Player->isGM() && SrvParms->houseintown==0)
			{
				if (Player->inGuardedArea() && ishouse(id) ) // popy
				{
					sysmessage(s," You cannot build houses in town!");
					return;
				}
			}
			
			/*
			if (ishouse(id1, id2)) // strict checking only for houses ! LB
			{
			if (!(CheckBuildSite(BuildPos,sx,sy)))
			{
			sysmessage(s,"Can not build a house at that location (CBS)!");
			return;
			}
			}*/
			
			if (!Player->isGM())	// disable checking for GMs. Experimental for Sayro (Duke, 22.3.2002)
			{
				for (k=-sx;k<sx;k++)//check the SPACEX and SPACEY to make sure they are valid locations....
				{
					for (ll=-sy;ll<sy;ll++)
					{
						if ((!validNPCMove(x+k,y+ll,z,Player))&&
							((Player->pos.x!=x+k)&&(Player->pos.y!=y+ll)))
							/*This will take the char making the house out of the space check, be careful 
							you don't build a house on top of your self..... this had to be done So you 
							could extra space around houses, (12+) and they would still be buildable.*/
						{
							sysmessage(s, "You cannot build your stucture there.");
							return;
							//clConsole.send("Invalid %i,%i [%i,%i]\n",k,l,x+k,y+l);
						} //else clConsole.send("DEBUG: Valid at %i,%i [%i,%i]\n",k,l,x+k,y+l);
						
						cCoord po(x+k,y+ll,z);
						if (findHouseAt(po) && !(norealmulti))
						{
							sysmessage(s,"You cant build structures inside structures");
							return;
						}
					}
				}
			}
			
			//Boats ->
			if (id2>=18) sprintf(temp,"%s's house",Player->name);//This will make the little deed item you see when you have showhs on say the person's name, thought it might be helpful for GMs.
			else strcpy(temp, "a mast");
//			strcpy(temp, name);
			//--^
			
			if (othername) strcpy(temp,name);		
			
			pHouse=Items->SpawnItem(Player, 1,temp,0,id,0,0);
			if (!pHouse) return;		
			
			pHouse->MoveTo(x,y,z);
			pHouse->resetPriv();
			pHouse->lockdownmax = ldm; // Lockdown items max. Ripper
			pHouse->securemax = sm;  // Secure chests max. Ripper
			pHouse->more4 = itemsdecay; // set to 1 to make items in houses decay
			pHouse->morex=hdeed; // crackerjack 8/9/99 - for converting back *into* deeds
			pHouse->SetOwnSerial(Player->serial);
			pHouse->setNotMovable(); // client doesnt allow you, but cheat programs do !
			
			if (!hitem[0] && !boat)
			{
				teleport(Player, Player->pos); // player coords NOT changed here !!! -> old_pos=curr_pos, indication of false use !!!!
				all_items(s);
				return;//If there's no extra items, we don't really need a key, or anything else do we? ;-)
			}
			
			if (boat) //Boats
			{
				if (!Boats->Build(s,Player,pHouse, id2, scpno))
				{
					Items->DeleItem(pHouse);
					return;
				} 
			}
		} // end of realmulti only
		
		if (scpno)	//Boats->.. Moved from up there ^ // delete deed object
			Items->DeleItem(Player->getDclickedItem()); // this will del the deed no matter where it is
		
		//Key...
		if (!nokey) 
			makeKeysForHouse(Player, pHouse, id);
		
		// realmulti+norealmulti
		for (k=0;k<icount;k++)//Loop through the HOUSE_ITEMs
		{
			openscript("house.scp");
			sprintf(sect,"HOUSE ITEM %i",hitem[k]);
			if (!i_scripts[house_script]->find(sect))
			{
				closescript();
			}
			else
			{
				P_ITEM pi=NULL;
				loopexit=0;
				do 
				{
					read2();
					if (script1[0]!='}')
					{
						if (!strcmp(script1,"ITEM"))
						{
							pos=ftell(scpfile);// To prevent accidental exit of loop.
							closescript();
							pi=Items->CreateScriptItem(s,str2num(script2),0);//This opens the item script... so we gotta keep track of where we are with the other script.
							openscript("house.scp");
							fseek(scpfile, pos, SEEK_SET);
							strcpy(script1, "ITEM");
							if (pi!=NULL)
							{
							  if (!pHouse && !norealmulti)
							  {
								LogError("Error in house.scp, ITEM\n");
								closescript();
								return;
							  } 
							  if (pHouse)
							  {							  
							    pi->pos=pHouse->pos;							   
							  }	else pi->pos=Player->pos;
							  pi->setNotMovable();//Non-Movebale by default
							  pi->decayOff();//since even things in houses decay, no-decay by default							  
							  pi->SetOwnSerial(Player->serial);
							}
						}

						if (pi!=NULL)
						{
							if (!strcmp(script1,"DECAY"))			pi->decayOn();
							if (!strcmp(script1,"NODECAY"))			pi->decayOff();							
							if (!strcmp(script1,"PACK"))//put the item in the Builder's Backpack
							{						
								P_ITEM iITY=Player->getBackpack();
								if (iITY!=NULL)
								{
									pi->SetContSerial(iITY->serial);
									pi->SetRandPosInCont(iITY);
								}
							}
							
							if (!strcmp(script1,"MOVEABLE"))		pi->setAllwaysMovable();
							if (!strcmp(script1,"LOCK"))//lock it with the house key
							{
								if (!pHouse && !norealmulti)
								{
									LogError("Error in house.scp, LOCK\n");
									closescript();
									return;
								}
								if (pHouse) pi->setMore1_4(pHouse->serial);
							}
							//offsets + or - from the center of the house:
							if (!strcmp(script1,"X"))				pi->pos.Add(str2num(script2));
							if (!strcmp(script1,"Y"))				pi->pos.Add(0,str2num(script2));
							if (!strcmp(script1,"Z"))				pi->pos.Add(0,0,str2num(script2));
						}
					}
				}
				while ( (strcmp(script1,"}")) && (++loopexit < MAXLOOPS) );

				if (pi==NULL)
				{ 
					LogError("Error in house.scp");
					closescript();
					return;
				}

				P_ITEM iLl = MAKE_ITEM_REF(ll);
				if (iLl!=NULL) if (pi->isInWorld())  mapRegions->Add(iLl);  //add to mapRegions

				closescript();
			}
		}

		all_items(s);//make sure they have all the items Sent....

		if (!norealmulti)
		{
			cCoord iPos(x+cx,y+cy,z+cz,Player->pos.map);
			Player->teleTo(iPos); //move char inside house
			Player->dispz=Player->pos.z;
		}
	}
}

// turn a house into a deed if possible. - crackerjack 8/9/99
// s = socket of player
// i = house item # in items[]
// morex on the house item must be set to deed item # in items.scp.
void deedhouse(UOXSOCKET s, P_ITEM House) // Ripper & AB
{
	if (!House)
	{
		LogError("Invalid house to deed");
		return;
	}
	cChar* Player = MAKE_CHARREF_LR(currchar(s));

	if (House->isType(ITY_TownHouse))
	{
		deedTownHouse(s,House);
		return;
	}

	if (Player->Owns(House) || Player->isGM())
	{
		cRectangle rect(0,0,0,0);
		Map->MultiArea(House, rect);
		
		P_ITEM iII = Items->SpawnItemBackpack2(s, House->morex);        // need to make before delete
		if ( iII == NULL ) return;
		sprintf(temp, "Demolishing House %s", House->name);
		sysmessage( s, temp );
		sprintf(temp, "Converted into a %s.", iII->name);
		sysmessage(s, temp); 
		// door/sign delete

		P_CHAR Vendor;
		int loopexit=0;
		RegGrid3x3Iterator4Chars rg3ic(House->pos);
		while((Vendor = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
		{
			if (Vendor->pos.IsIn(rect))
			{
				if ( Vendor->isAiVendor() ) // player vendor in right place
				{
					sprintf(temp, "A vendor deed for %s", Vendor->name );
					P_ITEM pPvDeed = Items->SpawnItem(Player, 1, temp, 0, 0x14F0, 0, 1);
					if (Vendor->GetPriv4()==0) pPvDeed->setType(ITY_VendorDeed); else pPvDeed->setType(ITY_VendorDeedWithoutHouse);
					pPvDeed->value = 2000;
					pPvDeed->refresh();
					sprintf(temp, "Packed up vendor %s.", Vendor->name);
					Npcs->DeleteChar(Vendor);
					sysmessage(s, temp);
				}
			}
		}
		
		PC_ITEM pi;
		loopexit=0;
		RegGrid3x3Iterator4Items rg3ii(House->pos);
		while((pi = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
		{
			if (pi->pos.IsIn(rect))
			{
				Items->DeleItem(pi->unconst());
			}
		}

		killkeys(House);
		sysmessage(s,"All house items and keys removed.");
		Items->DeleItem(House);
		
		Player->pos.z = Player->dispz = Map->MapElevation(Player->pos);
		teleport(Player, Player->pos); // only x/y used for old_pos param, those didnt change, hence curr pos can be used
		return;
	}
}

// removes houses - without regions. slow but necassairy for house decay
// LB 19-September 2000
void killhouse(P_ITEM House)
{
	if (!House) return;

	cRectangle rect(0,0,0,0);
	Map->MultiArea(House, rect);
		
	P_CHAR pc;
	AllCharsIterator aci;
	for (aci.Begin(); (pc = aci.GetData()) != aci.End(); aci++) // deleting npc-vendors attched to the decying house
	{
		if (pc->pos.IsIn(rect))
		{
			if (pc->isAiVendor()) // player vendor in right place, delete !
			{
				Npcs->DeleteChar(pc);
			}
		}                                           
	}
	
	PC_ITEM pi;
	AllItemsIterator aii;
	for (aii.Begin(); (pi = aii.GetData()) != aii.End(); aii++) // deleting itmes inside house
	{
		if (pi->pos.IsIn(rect))
		{
			if (!pi->isType(ITY_GuildStone)) // dont delete guild stones !
			{
				Items->DeleItem(pi->unconst());        
			}
		}
	}

	killkeys(House);	// deleting house keys   
}

// helper functiion for houde-decay.
// cause houses have no special type or more value we have to check all
// house item'ids
// LB 19-September 2000

bool ishouse(short id)
{ 
	if (!(id&0x4000)) return false; // no multi
	id-=0x4000;     
	return (Multi->ishouse(id));
}

bool ishouse(PC_ITEM pi)
{
	return ishouse(pi->id());
}

bool isTent(short id)
{
	return (id >= 0x4070 && id <= 0x4073);
}

// does all the work for house decay.
// checks all items if they are houses 
// if so, check its time stamp. if its too old remove it
// LB 19-September 2000
int check_house_decay()
{
	int houses=0;   
	int decayed_houses=0;
	unsigned long int timediff;
	unsigned long int ct=getNormalizedTime();
	
	PC_ITEM pi;	 
	AllItemsIterator aii;
	for (aii.Begin(); (pi=aii.GetData()) != aii.End(); aii++) 
	{   
		if (ishouse(pi) && !pi->free)
		{
			// its a house -> check its unused time

			//clConsole.send("id2: %x time_unused: %i max: %i\n",pi->id2,pi->time_unused,server_data.housedecay_secs);
			
			if (pi->time_unused>SrvParms->housedecay_secs) // not used longer than max_unused time ? delete the house
			{          
				decayed_houses++;
				sprintf(temp,"%s decayed! not refreshed for > %lu seconds!\n",pi->name,SrvParms->housedecay_secs);
				LogMessage(temp);
				killhouse(pi->unconst());
			}
			else // house ok -> update unused-time-attribute
			{
				P_ITEM pi2 = pi->unconst();
				timediff=(ct-pi2->timeused_last)/MY_CLOCKS_PER_SEC;
				pi2->time_unused+=timediff; // might be over limit now, but it will be cought next check anyway
				
				pi2->timeused_last=ct;	// if we don't do that and housedecay is checked every 11 minutes,
				// it would add 11,22,33,... minutes. So now timeused_last should in fact
				// be called timeCHECKED_last. but as there is a new timer system coming up
				// that will make things like this much easier, I'm too lazy now to rename
				// it (Duke, 16.2.2001)
			}
			houses++;
		}
	}
	return decayed_houses;
}

void killkeys(P_ITEM House) // Crackerjack 8/11/99
{
	if (!House) return;
	
	PC_ITEM pi;
	AllItemsIterator aii;
	for (aii.Begin(); (pi=aii.GetData()) != aii.End(); aii++)
	{
		if (pi->isType(ITY_Key) && pi->getMore1_4() == House->serial)
		{
			Items->DeleItem(pi->unconst());
		}
	}
	return;
}

////////////
// name:	GetEstate
// purpose:	return the rectangle of the house's area
// history:	by Duke, 16.1.2003
// remark:	the rectangle passed on entry is NOT allways set !! Use the ptr returned !!
//
cRectangle* GetEstate(PC_ITEM House, cRectangle& rect)
{
	cRectangle *pr;
	if (House->isType(ITY_TownHouse))
		pr=House->thi->area;
	else
	{
		Map->MultiArea(House, rect);
		pr=&rect;
	}
	return pr;
}

// Crackerjack 8/12/99 - House List Functions

// Checks if somebody is on the house list.
// on_hlist(int h (items[] index for house), int serial (char serial),
//		int *li (pointer to variable to put items[] index of list item in or NULL))
// Returns:
// 0 - Character is not on house list
// Anything else - Character is on house list, type # is returned.
static PC_ITEM get_hlist(PC_ITEM House, int serial)
{
	PC_ITEM pi;
	int loopexit=0;
	RegGrid3x3Iterator4Items rg3ii(House->pos);
	while((pi = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (pi->morey==(unsigned)House->serial &&	pi->getMore1_4()==serial) 
		{
			return pi;
		}		          
	}								
	return NULL;
}

int on_hlist(PC_ITEM House, int serial)
{
	if (!House) return 0;
	
	PC_ITEM pi = get_hlist(House,serial);
	if (pi)
	{
		return pi->morex;
	}		          
	return 0;
}

// Adds somebody to a house list.
// add_hlist(int c (chars[] index), int h (items[] index for house), int t (list type))
// Returns:
// 1 - Successful addition to house list
// 2 - Character is already on a house list
// 3 - Character is not on property
int add_hlist(P_CHAR pc, P_ITEM House, int t)
{
	if (!House) return 0;

	if (on_hlist(House, pc->serial))
		return 2;

	cRectangle *pr, rect(0,0,0,0);
	pr = GetEstate(House, rect);
	// Make an object with the character's serial & the list type
	// and put it "inside" the house item.
	if (pc->pos.IsIn(*pr))
	{
		P_ITEM pi = Items->MakeBaseItem();
		if (!pi) return 0;

		pi->morex=t;
		pi->setMore1_4(pc->serial);
		pi->morey=House->serial;

		pi->decayOff(); // no decay !!
		pi->visible=0;
		strcpy(pi->name,"friend of house");

		pi->pos = House->pos;

		mapRegions->Add(pi);
		return 1;
	}

	return 3;
}
// Removes somebody from a house list.
// del_hlist(int c (chars[] index), int h (items[] index for house))
// Returns:
// 0 - Player was not on a list
// # - What list the player was on if any.
bool del_hlist(P_CHAR pc, P_ITEM House)
{
	PC_ITEM pi = get_hlist(House,pc->serial);
	if (pi)
	{
		mapRegions->Remove(pi);
		Items->DeleItem(pi->unconst());
		return true;
	}
	return false;
}

// Handles house commands from friends of the house. - Crackerjack 8/12/99
void house_speech(int s, char *msg)	// msg must already be capitalized
{
	if (s<0 || s>MAXCLIENT) return;

	P_CHAR Player = MAKE_CHARREF_LR(currchar(s));
	PC_ITEM House = findHouseAt(Player->pos);
	if (!House) return; // not in a house, so we don't care.

	if (strstr(msg,"I WISH TO LEAVE THIS BUILDING"))	// allow escape from locked houses (Duke, idea by Sayro)
	{
		ejectChar(House,Player);
	}

	int fr=on_hlist(House, Player->serial);
	if (fr!=H_FRIEND && !Player->Owns( House ) )
		return; // not a friend or owner, so we don't care.

	if (strstr(msg, "I BAN THEE"))	// house ban
	{
		setAddid1_4(s,House->serial);
		target(s, 229, "Select person to ban from house.");
		return;
	}
	if (strstr(msg, "REMOVE THYSELF"))	// kick out of house
	{
		setAddid1_4(s,House->serial);
		target(s, 228, "Select person to eject from house.");
		return;
	}
	if (strstr(msg,"I WISH TO LOCK THIS DOWN")) 
	{ // lock down code AB/LB
		target(s, 232, "Select item to lock down");
	}
	   
	if (strstr(msg,"I WISH TO RELEASE THIS")) 
	{ // lock down code AB/LB
		target(s, 233, "Select item to release");
	}
	if (strstr(msg,"I WISH TO SECURE THIS")) 
	{ // lock down code AB/LB
		target(s, 234, "Select item to secure");
	}
}

bool CheckBuildSite(const cCoord& pos, int sx, int sy)
{
signed int checkz;
//char statc;
int checkx;
int checky;
int ycount=0;
checkx = pos.x-(sx/2);
for (;checkx<(pos.x+(sx/2));checkx++)
{
	checky=pos.y-(sy/2);
	for ( ;checky<(pos.y+(sy/2));checky++)
	{
		cCoord chkpos(checkx,checky,pos.z,pos.map);
		checkz=Map->MapElevation(chkpos);
		if ((checkz>(pos.z-2))&&(checkz<(pos.z+2)))
		{
			ycount++;
		}
	//	statc=Map->StaHeight(checkx,checky,checkz);
	//	if (statc>0)
	//		statb=true;
	}
}
if (ycount==(sx*sy)) //&& (statb==false))
	return true;
else
	return false;
}

void refreshhouse(UOXSOCKET s, P_CHAR Player, P_ITEM House)
// house refreshment when a house owner or friend of a house opens the house door
{
	if (House->type == ITY_TownHouse)
		return;
	if (s!=-1) 
	{
		int ds;
		if (SrvParms->housedecay_secs!=0)
			ds=((House->time_unused)*100)/(SrvParms->housedecay_secs);
		else ds=-1;
		
		if (ds>=50) // sysmessage if decay status >=50%
		{
			if (!Player->Owns(House))
				sysmessage(s,"You refreshed your friend's house");
			else
				sysmessage(s,"You refreshed the house");
		}
	}
				
	House->time_unused=0;
	House->timeused_last=getNormalizedTime();
}

// eject a char from a house
// return true if sucessfully ejected
bool ejectChar(PC_ITEM House, P_CHAR pc)
{
	if (!House) return false;

	cRectangle *pr, rect(0,0,0,0);
	pr = GetEstate(House,rect);

	if (pc->pos.IsIn(*pr))
	{
		cCoord ePos(pr->upperRightX, pr->upperRightY+1, pc->pos.z, pc->pos.map);
		pc->teleTo(ePos);
		return true;
	}
	return false;
}

// check if player is co-owner of the house (Duke, 28.10.2002)
// 'House' can also be an item that is locked down in a house
bool IsCoOwner(PC_ITEM House, P_CHAR Player)
{
	if (House && Player)
	{
		P_CHAR Owner = FindCharBySerial(House->ownserial);
		if (Owner)
		{
			if (Owner->account == Player->account)
				return true;
		}
	}
	return false;
}

