
#include "magic_new.h"
#include "magic.h"
#include "spells.h"
#include "chars.h"
#include "items.h"
#include "defines.h"
#include "pointer.h"
#include "globals.h"
#include "SndPkg.h"
#include "basics.h"
#include "network.h"
#include "itemid.h"
#include "gumps.h"
#include "mapstuff.h"
#include "scriptc.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "magic_new.cpp"

cMagic::cMagic()
{
	TithingGump=NULL;
	TithingGump = new CTithing;		
	spells=NULL;
}

cMagic::~cMagic()
{
	int i;

	if (spells!=NULL)
	{
	  for (i=0;i<this->numspells;++i) delete (*spells)[i];
	  delete spells;
	}

	if (TithingGump!=NULL) delete TithingGump;

}

bool cMagic::LoadSpells()
{
	cSpell* spell = 0; // current spell
	cSpell  defaultspell(0);

	SecVec<cSpell*> vSpells("vSpells"); // first use vectors, later we use normal arrays (better performance)

	int currspell = -1;
	
	Script *pScp=i_scripts[spells_script];
	if (!pScp->Open()) return false;

	int loopexit = 0;
	do
	{
		pScp->NextLine();	// reads a line of the scripts without splitting the parameters
		if (!strncmp("SPELL",script1,5))
		{
			LogError("CAUTION: spells_new.scp seems to be in old spells format. Magic stuff might not work correctly!");
			return false;
		}
		if (strncmp("SECTION SPELL",script1,13)) continue; // we're looking for SECTION SPELL here..

		currspell = str2num(&script1[13]);
		switch (currspell) // now decide which kind of spell we have
		{
		// START REGULAR SPELLS
		
		case 0:		spell = &defaultspell;				break;
		// first circle
		case 1:		spell = new cClumsySpell();			break;
		case 2:		spell = new cCreateFoodSpell();		break;
		case 3:		spell = new cFeeblemindSpell();		break;
		case 4:		spell = new cHealSpell();			break;
		case 5:		spell = new cMagicArrowSpell();		break;
		case 6:		spell = new cNightsightSpell();		break;
		case 7:		spell = new cReactiveArmorSpell();	break;
		case 8:		spell = new cWeakenSpell();			break;
		// second circle
		case 9:		spell = new cAgilitySpell();		break;
		case 10:	spell = new cCunningSpell();		break;
		case 11:	spell = new cCureSpell();			break;
		case 12:	spell = new cHarmSpell();			break;
		case 13:	spell = new cMagicTrapSpell();		break;
		case 14:	spell = new cMagicUntrapSpell();	break;
		case 15:	spell = new cProtectionSpell();		break;
		case 16:	spell = new cStrengthSpell();		break;
		// third circle
		case 17:	spell = new cBlessSpell();			break;
		case 18:	spell = new cFireballSpell();		break;
		case 19:	spell = new cMagicLockSpell();		break;
		case 20:	spell = new cPoisonSpell();			break;
		case 21:	spell = new cTelekinesisSpell();	break;
		case 22:	spell = new cTeleportSpell();		break;
		case 23:	spell = new cUnlockSpell();			break;
		case 24:	spell = new cWallOfStoneSpell();	break;
		// fourth circle
		case 25:	spell = new cArchCureSpell();		break;
		case 26:	spell = new cArchProtectionSpell();	break;
		case 27:	spell = new cCurseSpell();			break;
		case 28:	spell = new cFireFieldSpell();		break;
		case 29:	spell = new cGreaterHealSpell();	break;
		case 30:	spell = new cLightningSpell();		break;
		case 31:	spell = new cManaDrainSpell();		break;
		case 32:	spell = new cRecallSpell();			break;
		// fifth circle
		case 33:	spell = new cBladeSpiritSpell();	break;
		case 34:	spell = new cDispelFieldSpell();	break;
		case 35:	spell = new cIncognitoSpell();		break;
		case 36:	spell = new cMagicReflectSpell();	break;
		case 37:	spell = new cMindBlastSpell();		break;
		case 38:	spell = new cParalyzeSpell();		break;
		case 39:	spell = new cPoisonFieldSpell();	break;
		case 40:	spell = new cSummonCreatureSpell();	break;
		// sixth circle
		case 41:	spell = new cDispelSpell();			break;
		case 42:	spell = new cEnergyBoltSpell();		break;
		case 43:	spell = new cExplosionSpell();		break;
		case 44:	spell = new cInvisibilitySpell();	break;
		case 45:	spell = new cMarkSpell();			break;
		case 46:	spell = new cMassCurseSpell();		break;
		case 47:	spell = new cParalyzeFieldSpell();	break;
		case 48:	spell = new cRevealSpell();			break;
		// seventh circle
		case 49:	spell = new cChainLightningSpell();	break;
		case 50:	spell = new cEnergyFieldSpell();	break;
		case 51:	spell = new cFlamestrikeSpell();	break;
		case 52:	spell = new cGateTravelSpell();		break;
		case 53:	spell = new cManaVampireSpell();	break;
		case 54:	spell = new cMassDispelSpell();		break;
		case 55:	spell = new cMeteorSwarmSpell();	break;
		case 56:	spell = new cPolymorphSpell();		break;
		// eight circle
		case 57:	spell = new cEarthquakeSpell();		break;
		case 58:	spell = new cEnergyVortexSpell();	break;
		case 59:	spell = new cResurrectionSpell();	break;
		case 60:	spell = new cAirElementalSpell();	break;
		case 61:	spell = new cEarthElementalSpell();	break;
		case 62:	spell = new cWaterElementalSpell();	break;
		case 63:	spell = new cFireElementalSpell();	break;
		case 64:	spell = new cSummonDaemonSpell();	break;
		// END REGULAR SPELLS

		// START NECRO SPELLS
		case 101:	spell = new cAnimateDeadSpell();	break;
		case 102:	spell = new cBloodOathSpell();		break;
		case 103:	spell = new cCorpseSkinSpell();		break;
		case 104:	spell = new cCurseWeaponSpell();	break;
		case 105:	spell = new cEvilOmenSpell();		break;
		case 106:	spell = new cHorrificBeastSpell();	break;
		case 107:	spell = new cLichFormSpell();		break;
		case 108:	spell = new cMindRotSpell();		break;
		case 109:	spell = new cPainSpikeSpell();		break;
		case 110:	spell = new cPoisonStrikeSpell();	break;
		case 111:	spell = new cStrangleSpell();		break;
		case 112:	spell = new cSummonFamiliarSpell();	break;
		case 113:	spell = new cVampiricEmbraceSpell();break;
		case 114:	spell = new cVengefulSpiritSpell();	break;
		case 115:	spell = new cWitherSpell();			break;
		case 116:	spell = new cWraithFormSpell();		break;
		// END NECRO SPELLS

		// START PALADIN SPELLS
		case 201:	spell = new cCleanseByFireSpell();	break;
		case 202:	spell = new cCloseWoundsSpell();	break;
		case 203:	spell = new cConsecrateWeaponSpell();break;
		case 204:	spell = new cDispelEvilSpell();		break;
		case 205:	spell = new cDivineFurySpell();		break;
		case 206:	spell = new cEnemyOfOneSpell();		break;
		case 207:	spell = new cHolyLightSpell();		break;
		case 208:	spell = new cNobleSacrificeSpell();	break;
		case 209:	spell = new cRemoveCurseSpell();	break;
		case 210:	spell = new cSacredJourneySpell();	break;
		// END PALADIN SPELLS

		// alle other spells are custom spells
		default:	spell = new cSpell(currspell);		break;
		};

		if (spell->GetSpellnum())		// don't touch spell #0
		{
			*spell = defaultspell;		// setting scripted default values
			vSpells.push_back(spell);	// and add to spell list
		}

		do
		{
			pScp->NextLineSplitted();		// now we need the parameters => splitt the line
			if (!strcmp("MANA", script1))
			{
				cline = &script2[0];
				splitline();
				spell->SetMana(makenumber(0),0);
				spell->SetMana(makenumber(1),1);
				spell->SetMana(makenumber(2),2);
			}
			else if (!strcmp("TITHING", script1))
			{
				cline = &script2[0];
				splitline();
				spell->SetTithing(makenumber(0),0);
				spell->SetTithing(makenumber(1),1);
				spell->SetTithing(makenumber(2),2);
			}
			else if (!strcmp("LOSKILL", script1))
			{
				cline = &script2[0];
				splitline();
				spell->SetLoskill(makenumber(0),0);
				spell->SetLoskill(makenumber(1),1);
				spell->SetLoskill(makenumber(2),2);
			}
			else if (!strcmp("HISKILL", script1))
			{
				cline = &script2[0];
				splitline();
				spell->SetHiskill(makenumber(0),0);
				spell->SetHiskill(makenumber(1),1);
				spell->SetHiskill(makenumber(2),2);
			}
			else if (!strcmp("DELAY", script1))
			{
				cline = &script2[0];
				splitline();
				spell->SetDelay(makenumber(0),0);
				spell->SetDelay(makenumber(1),1);
				spell->SetDelay(makenumber(2),2);
			}
			else if (!strcmp("MANTRA", script1))	spell->SetMantra(script2);
			else if (!strcmp("TARGMSG", script1))	spell->SetTargMsg(script2);

			else if (!strcmp("STARTTRIG", script1))	spell->SetTrigger(str2num(script2),0);
			else if (!strcmp("TARGTRIG", script1))	spell->SetTrigger(str2num(script2),1);
			else if (!strcmp("ENDTRIG", script1))	spell->SetTrigger(str2num(script2),2);

			else if (!strcmp("ACTION", script1))	spell->SetAction(hstr2num(script2));

			else if (!strcmp("REFLECTABLE", script1))	spell->SetReflectable(true);
			else if (!strcmp("DISABLED", script1))		spell->SetDisabled(true);
			else if (!strcmp("RUNIC", script1))			spell->SetRunic(true);

			else if (!strcmp("AFFECTCASTER", script1))	spell->SetAffectCaster(true);
			else if (!strcmp("AFFECTPARTY", script1))	spell->SetAffectParty(true);

			else if (!strcmp("REAG", script1))
			{
				splitline();
				spell->AddReag(makenumber(0),hexnumber(1));
			}
		} while ((strcmp(script1, "EOF")) && (script1[0]!='}') && (++loopexit < MAXLOOPS));

	} while ((strcmp(script1, "EOF")) &&(++loopexit < MAXLOOPS));

	pScp->Close();

	// now we know the number of spells and can create our array
	spells = new CDynArray<cSpell*>(vSpells.size());
	numspells = vSpells.size();

	int i;
	for (i=0;i<numspells;++i) (*spells)[i] = vSpells[i]; // and copy all spells into the array

	return true;
}

cSpell* cMagic::GetSpell(int spellnum)
{
	if (!spellnum) return NULL; // return for spell number 0

	int i;
	for (i=0;i<this->numspells;++i)
	{
		if ((*spells)[i]->GetSpellnum() == spellnum) return (*spells)[i];
	}

	LogWarningVar("Spell number %d not found in scripts - please check spells_new.scp",spellnum);
	return NULL;
}

int cMagic::CountSpells(P_ITEM Book)
{
	if (!Book) return 0;

	unsigned int ui = 1;

	int numspells = 0;

	int more  = Book->getMore1_4();
	int more2 = Book->getMoreB1_4();

	int i;
	for (i=0;i<32;++i) // check all 32 bits of each variable
	{
		ui = (1<<i);
		if ((more  & ui) == ui) ++numspells;
		if ((more2 & ui) == ui) ++numspells;
	}
	return numspells;
}

P_ITEM cMagic::GetSpellBook(P_CHAR Player,int booktype)
{
	int type = 0;

	switch (booktype)
	{
	case SBT_REGULAR:
		type = ITY_SpellBook;
		break;
	case SBT_NECRO:
		type = ITY_NecroBook;
		break;
	case SBT_PALADIN:
		type = ITY_PaladinBook;
		break;
	}

	int ci=0, loopexit=0;
	P_ITEM pi;
	while ( (( pi=Player->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
	{
		if (pi->isType(type)) return pi;
	}

	// not equipped ? so let's search the primary backpack
	P_ITEM Pack = Player->getBackpack();
	if (Pack)
	{
		ci=0; loopexit=0;
		while ( (( pi=Pack->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
		{
			if (pi->isType(type)) return pi;
		}
	}
	return NULL;
}

void cMagic::OpenSpellBook(P_CHAR Player, P_ITEM pi, int booktype)
{
	P_ITEM Book;
	int s = Player->calcSocket();
	
	// we need to be shure if client understands the new packet
	if (!SrvParms->AOS_Mode) // client is not running in aos mode
	{
		if (booktype == SBT_REGULAR) // regular spellbook, send old packed
		{
			OpenSpellBook_old(Player,pi);
			return;
		}
		else
		{
			sysmessage(s, "Server must run in AOS mode to open this spellbook");
			return;
		}
	}

	// here starts the real function
	if (!pi) Book = FindItemBySerPtr(&buffer[s][1]); // item send by client?
	else	 Book = pi;

	P_ITEM Pack = Player->getBackpack();

	if (!Book) Book = GetSpellBook(Player,booktype);

	if (!Book ||	// no book at all
		(Pack && !Pack->ContainsSub(Book) &&	// not in primary pack
		!Player->Wears(Book) ))		// not equipped
	{
		sysmessage(s, "In order to open spellbook, it must be equipped in your hand or in the first layer of your backpack.");
		return;
	}

	if (Book->layer!=1) senditem(s,Book); // prevents crash if pack not open


	// now, everythin is clear and we can send the packets :)
	unsigned char sbookstart[8]="\x24\x00\x00\x00\x00\xFF\xFF"; //0x24 //spellbookserial //0xff 0xff
	LongToCharPtr(Book->serial, &sbookstart[1]);

	XStartPkt(0x24,7);
	XAddPkt(&sbookstart[0+1],7-1);
	XFinishPkt(s);

	/*
	BYTE 0xbf
	BYTE[2] total packet length
	BYTE[2] 0x1b (sub-command open spellbook new)
	BYTE[2] unknown, always 1
	BYTE[4] Spellbook serial
	BYTE[2] Item Id (book)
	BYTE[2] scroll offset     // 1=regular, 101=necro, 201=paladin
	BYTE[8] spellbook content // first bit of first byte = spell #1, second bit
	*/
	unsigned char bookdata[24] = "\xBF\x00\x17\x00\x1B\x00\x01\x00\x00\x00\x00\x0E\xFA\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00";
	LongToCharPtr(Book->serial, &bookdata[7]);

	// setting scroll offset
	switch (booktype)
	{
	case 0: // regualr book
		bookdata[11] = 0x0E; // book id
		bookdata[12] = 0xFA; // book id
		bookdata[13] = 0x00; // spell offset
		bookdata[14] = 0x01; // spell offset
		break;
	case 1: // necro book
		bookdata[11] = 0x22; // book id
		bookdata[12] = 0x53; // book id
		bookdata[13] = 0x00; // spell offset
		bookdata[14] = 0x65; // spell offset
		break;
	case 2: // paladin book
		bookdata[11] = 0x22; // book id
		bookdata[12] = 0x52; // book id
		bookdata[13] = 0x00; // spell offset
		bookdata[14] = 0xC9; // spell offset

		statwindow(s, Player, 4); // client needs tithing points info via 0x11 in mode 4, otherwise they are not displayed correctly (LB) 
		break;
	}

	// the more variables store our spells
	bookdata[15] = Book->more4;
	bookdata[16] = Book->more3;
	bookdata[17] = Book->more2;
	bookdata[18] = Book->more1;
	bookdata[19] = Book->moreb4;
	bookdata[20] = Book->moreb3;
	bookdata[21] = Book->moreb2;
	bookdata[22] = Book->moreb1;

	// send the packet
	XStartPkt(0xbf,23);
	XAddPkt(&bookdata[0+3],23-3);
	XFinishPkt(s);
}

//OpenSpellBook_old uses the new spell storing system, but the old 3.x client packages
void cMagic::OpenSpellBook_old(P_CHAR Player, P_ITEM pi) // we need this for compatiblity with 3.x clients
{
	if (!Player) return;
	int s = Player->calcSocket();

	P_ITEM Book;

	if (!pi) // item send by client?
		Book=FindItemBySerPtr(&buffer[s][1]);
	else 
		Book=pi;


	P_ITEM Pack = Player->getBackpack();
	if (!Book)
	{
		Book = GetSpellBook(Player,SBT_REGULAR);
	}

	if (!Book ||	// no book at all
		(Pack && !Pack->Contains(Book) &&	// not in primary pack
		!Player->Wears(Book) ))		// not equipped
	{
		sysmessage(s, "In order to open spellbook, it must be equipped in your hand or in the first layer of your backpack.");
		return;
	}

	if (Book->layer!=1) senditem(s,Book); // prevents crash if pack not open

	unsigned char sbookstart[8]="\x24\x40\x01\x02\x03\xFF\xFF";
	LongToCharPtr(Book->serial, &sbookstart[1]);

	XStartPkt(0x24, 7);
	XAddPkt(&sbookstart[0+1], 7-1);
	XFinishPkt(s);

	int a;
	int scount=0;

	CArray<70, int>SiB("SIB");
	for (a=0; a<70; a++) SiB[a]=0;

	int i;
	for (i=1;i<65;++i)
	{
		cSpell* spell = GetSpell(i);
		if (!spell) continue;

		if (spell->IsInBook(Book))
		{
			SiB[i-1] = 1;
			++scount;
		}
	}

	unsigned char sbookinit[6]="\x3C\x00\x3E\x00\x03";
	int l=(scount*19)+5;
	sbookinit[1]=(l)>>8;
	sbookinit[2]=(l)%256;
	sbookinit[3]=scount>>8;
	sbookinit[4]=scount%256;
	if (scount>0) 
	{
		XStartPkt(0x3c, l);
		XAddPkt(&sbookinit[0+3], 5-3);
	}

	unsigned char sbookspell[20]="\x40\x01\x02\x03\x1F\x2E\x00\x00\x01\x00\x48\x00\x7D\x40\x01\x02\x03\x00\x00";
	for (i=0;i<70;i++)
	{
		if (SiB[i])
		{
			sbookspell[0]=0x41;
			sbookspell[1]=0x00;
			sbookspell[2]=0x00;
			sbookspell[3]=i+1;
			sbookspell[8]=i+1;
			LongToCharPtr(Book->serial, &sbookspell[13]);

			XAddPkt(&sbookspell[0], 19);
		}
	}
	XFinishPkt(s);
}

void cMagic::ShowTithingGump(P_CHAR Player)
{
	TithingGump->setGoldandTithing(Player->CountGold(),Player->GetTithingPoints());
	TithingGump->showTithingGump(Player->calcSocket());			
}

bool cMagic::BeginCasting(P_CHAR Player,short spellnum,char type,void* preTarget)
{
	if (!Player) return false;
	int s = Player->calcSocket();

	cSpell* spell = GetSpell(spellnum);
	if (!spell) return false;

	spell->CheckTrigger(s,Player,0);

	if (!spell->CheckRequirements(Player,NULL,type)) return false;	// char lacks the requirements
																	// sysmessages will be given in CheckRequirements()

	ParticleEffects->SpellCastingStart(spell->GetSpellnum(),Player, s); // do some visual effects on spellstart

	Player->SetCastingSpell(spell,type,preTarget);	// sets all necessary data for the player
	Player->StartSpellDelay();						// and finaly start the delay

	if (Player->isHidden() && Player->isGM()) return true; // no wop for hidden gms // Added by Selos  
	if (spell->IsRunic() || Player->Skill(MAGERY)/10 > 99 )	// Add runic talk to gm mages (Blackwind)
	{		
		npctalkall_runic(Player,strupr(spell->GetMantra()),0);
	}
	else
	{		
		npctalkall(Player, spell->GetMantra(),0);
	}
	return true;
}

void cMagic::AfterSpellDelay(P_CHAR Player)
{
	if (!Player) return;
	int s = Player->calcSocket();

	Player->endCasting(); // unfreeze, etc

	cSpell* spell = Player->GetCastingSpell();
	if (!spell) return;

	if (spell->GetTargetType() == NO_TARGET)	this->FinnishCasting(Player,NULL,NO_TARGET); // no target required
	else if (Player->GetPreTarget())			this->FinnishCasting(Player,Player->GetPreTarget(),ITEM_TARGET); // currently only for runebooks, but I want it to be more universal later! (Thrawn)
	else target(s,101, spell->GetTargMsg()); // when target required, send target cursor
}

void cMagic::FinnishCasting(P_CHAR Caster,void* Target, char targtype) // called to finnish casting process..
{
	cSpell* spell = Caster->GetCastingSpell(); // first look for the spell we want to cast
	char	type  = Caster->GetCastingSpellType();
	Caster->SetCastingSpell(NULL,STY_BOOK); // reset players "currentspell"
	
	if (!spell) return;

	cCoord* targpos = NULL;
	switch (targtype)
	{
	case NO_TARGET:
		break;
	case CHAR_TARGET:
		targpos = &((P_CHAR)Target)->pos;
		break;
	case ITEM_TARGET:
		targpos = &((P_ITEM)Target)->pos;
		break;
	case  MAP_TARGET:
		targpos = (cCoord*)Target;
		break;
	default:
		break;
	}

	if (!spell->CheckRequirements(Caster,targpos,type)) return;	// we need to check all requirements again

	// Check casters magery Skill
	LogAssert(spell->GetHiskill(type)!=0);

	int s = Caster->calcSocket();

	bool succeeded = true;
	if (!Caster->checkSkill(MAGERY, spell->GetLoskill(type), spell->GetHiskill(type))) succeeded = false;

	spell->AccomplishRequirements(Caster,type,succeeded);
	if (!succeeded)
	{
		OldMagic->SpellFail(s);
		return;
	}

	if (!SrvParms->newmagictest) // redirection to old casting functions
	{
		Caster->spell = spell->GetSpellnum();
		int s = Caster->calcSocket();

		if		(targtype == NO_TARGET)	 OldMagic->NewCastSpell(s,0);
		else if (targtype == MAP_TARGET) OldMagic->NewCastSpell(s,0);
		else if (targtype == CHAR_TARGET)OldMagic->NewCastSpell(s,((P_CHAR)Target)->serial);
		else if (targtype == ITEM_TARGET)OldMagic->NewCastSpell(s,((P_ITEM)Target)->serial);
		return;
	}

	if (targtype==NO_TARGET) // no effects for "map target" and "no target" spells
	{
		spell->DoEffect(Caster); // now we can apply the effect
	}
	else if (targtype==CHAR_TARGET)
	{
		int s = Caster->calcSocket(); // calc the players socket
 		ParticleEffects->SpellCastingDestination(spell->GetSpellnum(),Caster,(P_CHAR)Target,NULL,s); // some visual effects on target and caster
		spell->DoEffect(Caster,(P_CHAR)Target); // now we can apply the effect
	}
	else if (targtype==ITEM_TARGET)
	{
		int s = Caster->calcSocket(); // calc the players socket
 		ParticleEffects->SpellCastingDestination(spell->GetSpellnum(),Caster,NULL,(P_ITEM)Target,s); // some visual effects on target and caster
		spell->DoEffect(Caster,(P_ITEM)Target); // now we can apply the effect
	}
	else if (targtype==MAP_TARGET) // no effects for "map target" and "no target" spells
	{
		spell->DoEffect(Caster,(cCoord*)Target); // now we can apply the effect
	}
}

// HERE GO SOME HELPER FUNCTIONS FOR ALL STUFF AROUND MAGERY
char cMagic::CalcFieldDirection(cCoord* casterpos,cCoord* destpos) // return values: 0 = x direction; i = y direction
{
	if (abs(casterpos->x - destpos->x) < abs(casterpos->y - destpos->y)) // calculate the direction of the wall
	{
		return 0;
	}
	else
	{
		return 1;
	}
}

void cMagic::AddMagicField(P_CHAR Player,cCoord* Position,short xid, short yid,int numItems, short time, short effNum)
{
	int s = Player->calcSocket();
	char direction = CalcFieldDirection(&Player->pos,Position); // 0 = wall has x direction; 1 = wall has y direction

	int i;
	for (i=(int)numItems/(-2);i<=(int)numItems/2;++i) //make "numItems" wall items
	{
		cCoord pos = *Position;
		if (!direction) pos.x += i;
		else			pos.y += i;

		if (Map->IsBlockedByCharOrItem(&pos)) continue; //no wall item if pos is blocked by char or item

		P_ITEM pi;
		if (!direction) pi = Items->SpawnItem(Player,1,"#",0,xid,0,0); //wall in x direction
		else			pi = Items->SpawnItem(Player,1,"#",0,yid,0,0); //wall in y direction
		if (!pi) continue;

		pi->dispellableOn();
		pi->decayOn();
		pi->decaytimer.setMilliSecs(time*MY_CLOCKS_PER_SEC+rand()%MY_CLOCKS_PER_SEC);
		pi->setNotMovable();
		pi->MoveTo2(pos); //move wallitem to the right place

		ParticleEffects->SpellCastingDestination(effNum,Player,NULL,pi,s); // some visual effects on the wall items and caster
	}
}


///////////////////
// Name:	MagicDamage
// History:	Unknown
// Purpose:	Calculate and inflict magic damage.
//
void cMagic::MagicDamage(P_CHAR pc, int amount)
{
	UOXSOCKET s = pc->calcSocket();
	if ( pc->IsFrozen()  &&  pc->effDex() > 0 )
	{
		pc->Unfreeze(); // unfreeze
		if (s != -1) sysmessage(s, "You are no longer frozen.");
	}
	if ( !pc->isInvul() && (region[pc->region].isMagicDamageOn()) && !pc->isDummy()) // LB magic-region change // Thrawn: dont damage dummy npcs
	{
		if (pc->npc) amount *= 2;			// double damage against non-players
		pc->reduceHp(amount);
		pc->disturbCasting(s);
	}
}

///////////////////
// Name:	PoisonDamage
// History:	Unknown
// Purpose:	Apply the poison to the character.
//
void cMagic::PoisonDamage(P_CHAR pc, int poison) // new functionality, lb !!!
{
	if (!pc) return;
	UOXSOCKET s = pc->calcSocket();

	if (pc->IsFrozen())
	{
		pc->Unfreeze();
		if (s!=-1) sysmessage(s, "You are no longer frozen.");
	}
	if ( !pc->isInvul() && (region[pc->region].isMagicDamageOn()) && !pc->isDummy()) // LB magic-region change //Thrawn: dont damage dummy npcs
	{
		if (poison>5) poison = 5;
		else if (poison<0) poison = 1;
		pc->PoisonStart(s,poison);
	}
}

