/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

///////////////////////////////////////////////
///////// Author: Lord Binary
///////// August 2002
///////// Low level access to map+static data
////////////////////////////////////////////

#include "mapS.h"
#include "wolfpack.h"
#include "assert.h"

#include <string>
#include <iostream>
#include <sstream>

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "mapS.cpp"

std::string convertToString(double x)
{
   std::ostringstream o;
   if (o << x) return o.str();
   
   return "conversion error";
} 


int mapS::initMapCache(mapInit &mI)
{
	unsigned char *madifl0=NULL, *madifl1=NULL, *madif0=NULL, *madif1=NULL;
	unsigned long int lenMadifl0=0, lenMadifl1=0, lenMadif0=0,lenMadif1=0;
	int eC;

	std::string outStri, fileStr;

	outStri = "\nInit Map Cache ...\n";
	Console.send(outStri.c_str());
	outStri = "Preloading mapdiffs into Cache ...\n";
	Console.send(outStri.c_str());

	outStri = "opening " + baseDir+mI.mapFile+"\n";
	Console.send(outStri.c_str());
	fileStr = baseDir+mI.mapFile;

	mF = fopen(fileStr.c_str(), "rb"); // open it ONCE !!!
	if (mF==NULL)
	{
		outStri = "fatal error: can't open map file\n";
		Console.send(outStri.c_str());
		return -1;
	}

	assert (mF!=NULL);

	///////////////////////////////////////////////////////////
	////
	//// Copy Mapdiff files into RAM
	////
	/////////////////////////////////////////////////////////

	bool mapdiffs=true;
	bool mapdiffsPar=true;

	if (!Dual) mapdiffsPar=false;

	outStri = baseDir+mI.diffMapLenFile+"\n";
	Console.send(outStri.c_str());
	fileStr = baseDir + mI.diffMapLenFile;
	unsigned long int len=0;

	if (!diffs) return 1;
	if (Dual && ! diffsPar) return 1;

	eC = getFileLen(fileStr, len);
	if (eC<=0 || len<=0) 
	{
		outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n"; Console.send(outStri.c_str());
		mapdiffs=false;
	}

    if (mapdiffs)
    {
	  madifl0 = new unsigned char[len];
	  eC = fileToRAM(fileStr, madifl0, len);
	  if (eC<=0) 
	  {       
		outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n";Console.send(outStri.c_str());		
	    mapdiffs=false;
	  }
    }

	if (mapdiffs) lenMadifl0=len;

	len =0;
	outStri = baseDir+mI.diffMapFile+"\n";
	Console.send(outStri.c_str());
	fileStr = baseDir + mI.diffMapFile;

	if (mapdiffs)
	{
	  eC = getFileLen(fileStr, len);
	  if (eC<=0 || len<=0) 
	  {
		outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n"; Console.send(outStri.c_str());
		mapdiffs=false;
	  }
	}

    if (mapdiffs)
    {
	  madif0 = new unsigned char[len];
	  eC = fileToRAM(fileStr, madif0, len);
	  if (eC<=0) 
	  {       
		outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n";Console.send(outStri.c_str());		
	    mapdiffs=false;
	  }
    }

	if (mapdiffs) lenMadif0=len;

	/////

	len=0;
	if (mapdiffsPar)
	{
	   outStri = baseDir+mI.diffMapLenFilePar+"\n";
	   Console.send(outStri.c_str());
	   fileStr = baseDir + mI.diffMapLenFilePar;

	   eC = getFileLen(fileStr, len);
	   if (eC<=0 || len<=0) 
	   {
		outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n"; Console.send(outStri.c_str());
		mapdiffsPar=false;
	   }
	}

    if (mapdiffsPar)
    {
	  madifl1 = new unsigned char[len];
	  eC = fileToRAM(fileStr, madifl1, len);
	  if (eC<=0) 
	  {       
		outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n";Console.send(outStri.c_str());		
	    mapdiffsPar=false;
	  }
    }

	if (mapdiffsPar) lenMadifl1=len;


	len=0;
	if (mapdiffsPar)
	{
	  outStri = baseDir+mI.diffMapFilePar+"\n";
	  Console.send(outStri.c_str());
	  fileStr = baseDir + mI.diffMapFilePar;

	  eC = getFileLen(fileStr, len);
	  if (eC<=0 || len<=0) 
	  {
		outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n"; Console.send(outStri.c_str());
		mapdiffsPar=false;
	  }
	}

    if (mapdiffsPar)
    {
	  madif1 = new unsigned char[len];
	  eC = fileToRAM(fileStr, madif1, len);
	  if (eC<=0) 
	  {       
		outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n";Console.send(outStri.c_str());		
	    mapdiffsPar=false;
	  }
    }

	if (mapdiffsPar) lenMadif1=len;

	if (!mapdiffsPar && Dual) { diffsPar = false; return 1; }
	if (!mapdiffs)            { diffs = false;	 return 1; }

	NumMapPatches    = lenMadifl0/4;
	NumMapPatchesPar = lenMadifl1/4;

	if (!mapdiffs)
	{
	   outStri = "warning, mapdiffs not used for " + mainName + "\n"; //outStri = outStri + "\n";
	   Console.send(outStri.c_str());			      
	}

	if (Dual && !mapdiffsPar)
	{
	   outStri = "warning, mapdiffs not used for " + parallelName + "\n"; //outStri = outStri + "\n";
	   Console.send(outStri.c_str());			 
	}

	if (mapdiffs)
	{	
	   if (Dual) outStri = "number of patched map blocks in " + mainName + " " + convertToString(lenMadifl0/4) + "\n";
	   else      outStri = "number of patched map blocks in " + mapName + " " + convertToString(lenMadifl0/4) + "\n";
	   Console.send(outStri.c_str());

	}

	if (mapdiffsPar && Dual)
	{	
	   if (Dual) outStri = "number of patched map blocks in " + parallelName + " " + convertToString(lenMadifl1/4) + "\n";
	   else      outStri = "number of patched map blocks in " + mapName + " " + convertToString(lenMadifl1/4) + "\n";
	   Console.send(outStri.c_str());
	}


	//////////////////////////////////////////////
	/////////
	///////// add the mapdiffs
	/////////
	//////////////////////////////////////////////

	short bX=0, bY=0;
	tmp *mds0, *mds1;
	int blockID, ctr;
	unsigned int c,a;

	mds0 = new tmp [blockX*blockY];
	mds1 = new tmp [blockX*blockY];

	for (a=0; a<(unsigned) (blockX*blockY); a++)
	{
		mds0[a].on=false;
		mds1[a].on=false;
	}

	if (mapdiffs)
	{
		ctr=0;
		for (c=0; c < lenMadifl0; c+=4)
		{		
			blockID = LongFromCharPtrRev(madifl0+c);		
			assert (blockID <  blockY*blockX );
			mds0[blockID].on=true;
			mds0[blockID].len=ctr;
			ctr++;
		}
	}

	if (mapdiffsPar)
	{
		ctr=0;
		for (c=0; c < lenMadifl1; c+=4)
		{		
			blockID = LongFromCharPtrRev(madifl1+c);
			assert (blockID <  blockY*blockX );
			mds1[blockID].on=true;
			mds1[blockID].len=ctr;
			ctr++;
		}
	}

	//// note: raw data stored here in reversed byte order, in contrast to statics data
	//// (due to performance cosideratations)

	bool wp, dwp;
	int l, l2, offs, offs2, offsP, r;
    
	//short TID;
	//signed char z;

	for (a=0; a<(unsigned)(blockX*blockY); a++)
	{
		wp=false;
		dwp=false;
		l=-1; l2=-1;

		bX = a / blockY;
		bY = a - (blockY*bX);

		assert (bX < blockX);
		assert (bY < blockY);

		if (mds0[a].on) 
		{ 
			wp=true; 
			l = mds0[a].len; 
		}

		if (mds1[a].on) 
		{
			dwp=true;
			l2 = mds1[a].len; 
		}
	

		if (wp && !dwp)
		{
			assert(l>=0 && (unsigned)l<(lenMadif0/4));
			offs  = 196*l+4; // offset into madif "file"
			assert( (unsigned)(offs+192)<=(lenMadif0));

			offs2 = 196*a+4; // offset into map-file
			assert( (offs2+192)<=(blockX*blockY*196));

			r = fseek(mF, offs2, SEEK_SET);

			if (r) 
			{
				outStri = "error: cant seek into mapfile1\n";
	            Console.send(outStri.c_str());
				return -3;
			}		

			mapCache[bX][bY].data = new unsigned char [192*2];

			r = fread(mapCache[bX][bY].data, sizeof(char), 192, mF);
			if (r!=192)
			{
				outStri = "error: cant read mapfile0\n";
	            Console.send(outStri.c_str());
				return -4;
			}

			memcpy(mapCache[bX][bY].data+192, madif0+offs, 192);
			mapCache[bX][bY].memLayout=1;

		}

		if (!wp && dwp)
		{
			assert(l2>=0 && (unsigned)l2<(lenMadif1/4));
			offs  = 196*l2+4; // offset into madif "file"
			assert( (unsigned)(offs+192)<=(lenMadif1));

			offs2 = 196*a+4; // offset into map-file
			assert( (offs2+192)<=(blockX*blockY*196));

			r = fseek(mF, offs2, SEEK_SET);

			if (r) 
			{
				outStri = "error: cant seek into mapfile2\n";
	            Console.send(outStri.c_str());
				return -5;
			}		

			mapCache[bX][bY].data = new unsigned char [192*2];

			r = fread(mapCache[bX][bY].data, sizeof(char), 192, mF);
			if (r!=192)
			{
				outStri = "error: cant read mapfile1\n";
	            Console.send(outStri.c_str());
				return -6;
			}

			memcpy(mapCache[bX][bY].data+192, madif1+offs, 192);
			mapCache[bX][bY].memLayout=2;

		}

		if (wp && dwp)
		{
			assert(l>=0 && (unsigned)l<(lenMadif0/4));
			offs  = 196*l+4; // offset into madif "file"
			assert( (unsigned)(offs+192)<=(lenMadif0));

			assert(l2>=0 && (unsigned)l2<(lenMadif1/4));
			offsP  = 196*l2+4; // offset into madif "file"
			assert( (unsigned)(offsP+192)<=(lenMadif1));

			offs2 = 196*a+4; // offset into map-file
			assert( (offs2+192)<=(blockX*blockY*196));

			r = fseek(mF, offs2, SEEK_SET);

			if (r) 
			{
				outStri = "error: cant seek into mapfile22\n";
	            Console.send(outStri.c_str());
				return -7;
			}		

			mapCache[bX][bY].data = new unsigned char [192*3];

			r = fread(mapCache[bX][bY].data, sizeof(char), 192, mF);
			if (r!=192)
			{
				outStri = "error: cant read mapfile11\n";
	            Console.send(outStri.c_str());
				return -8;
			}

			memcpy(mapCache[bX][bY].data+192, madif0+offs, 192);
			memcpy(mapCache[bX][bY].data+(192*2), madif1+offsP, 192);

			mapCache[bX][bY].memLayout=3;

		}


	}

	if (maxCacheFullness==101)
	{
	   int memLayout;
	   outStri = "Preloading complete map-data into RAM, plz hold on ...";
	   Console.send(outStri.c_str());

	   int x,y;
	   for (x=0; x<blockX; x++)
	   {
		   for (y=0; y<blockY; y++)
		   {
			   memLayout = mapCache[x][y].memLayout;
			   if (memLayout==0)
			   {
				    blockID = blockY*x + y ;
				   	offs2 = 196*blockID+4; 
			        assert( (offs2+192)<=(blockX*blockY*196));

			        r = fseek(mF, offs2, SEEK_SET);

			        if (r) 
					{
				       outStri = "error: cant seek into mapfile1234\n";
	                   Console.send(outStri.c_str());
				       return -10;
					}		

			        mapCache[bX][bY].data = new unsigned char [192];

			        r = fread(mapCache[bX][bY].data, sizeof(char), 192, mF);
			        if (r!=192)
					{
				      outStri = "error: cant read mapfile1234\n";
	                  Console.send(outStri.c_str());
				      return -11;
					}

					mapCache[x][y].tag=true;

			   }
			  
		   }
	   }

	   outStri = "\ndone\n";
	   Console.send(outStri.c_str());
	}

	if (mapdiffs)
	{		
		delete[] mds0;
		delete[] madifl0;
		delete[] madif0;
	}

	if (Dual && mapdiffsPar)
	{	
		delete[] mds1;
		delete[] madifl1;
		delete[] madif1;
	}


	return 1;
}

int mapS::verifyFileZ(mapInit & mI)
{
	std::string outStri;
	std::string fileStr;
	unsigned long int len;
	int eC;

	outStri = "Verifying map related files ...\n";
	Console.send(outStri.c_str());

	fileStr = baseDir + mI.staticsIndexFile;
	eC = getFileLen(fileStr, len);

	if (eC<=0) 
	{
		outStri = "VF: error getting filelength for " + fileStr +"\n";
		Console.send(outStri.c_str());
		return -1;
	}

	if (  (len / 12) != (unsigned) (blockX*blockY) ) 
	{ 
		outStri = "VF: wrong filesize " + fileStr+"\n";
		Console.send(outStri.c_str());
		return -2;
	}

	fileStr = baseDir + mI.mapFile;
	eC = getFileLen(fileStr, len);

	if (eC<=0) 
	{
		outStri = "VF: error getting filelength for " + fileStr+"\n";
		Console.send(outStri.c_str());
		return -3;
	}

	if (( len / 196 ) != (unsigned) (blockX*blockY) ) 
	{
		outStri = "VF: wrong filesize " + fileStr+"\n";
		Console.send(outStri.c_str());
		return -4;
	}

	return 1;
}

int mapS::getFileLen(std::string Dir, unsigned long int & len)
{
	 FILE *f;
	 unsigned long int length,s;
	 std::string outStri;

	 len = 0;

	 f = fopen(Dir.c_str(), "rb"); 
	 if (f==NULL) 
	 {
		 outStri = "GL: opening file failed: "+Dir+"\n";
		 Console.send(outStri.c_str());
		 return -1;
	 }

	 // set filepointer to end
	 s = fseek(f, 0, SEEK_END);
     if (s!=0) 
	 {
		outStri = "GL: seeking failed: "+Dir+"\n";
		Console.send(outStri.c_str());
		return -2;
	 }

	 // to get filelength
	 length = ftell(f);

	 if (length<=0) 
	 {
		 outStri = "GL: getting filesize failed: "+Dir+"\n";
		 Console.send(outStri.c_str());
		 return -3;
	 }

	 // set filepointer back to start
	 s=fseek(f,0, SEEK_SET); 
     if (s!=0) 
	 {
		outStri = "GL: seeking2 failed: "+Dir+"\n";
		Console.send(outStri.c_str());
		return -4;
	 }

	 fclose(f);

	 len = length;

	 return 1;

}

////////////////////////////////////
int mapS::fileToRAM(std::string Dir, unsigned char *buffer, unsigned long int len)
{
	 FILE *f;
	 unsigned long int pos;
	 std::string outStri;

	 // open
	 f = fopen(Dir.c_str(), "rb"); 
	 if (f==NULL) 
	 {
		 outStri = "FR: opening failed"+Dir+"\n";
		 Console.send(outStri.c_str());	
		 return -10;
	 }

     // now transfer to ram
	 pos = fread(buffer, sizeof(char), len, f);
     if (pos!=len) 
	 {
		 outStri = "FR: transfter into RAM failed "+Dir+"\n";
		 Console.send(outStri.c_str());	
		 return -11;
	 }

	 fclose(f);

	 return 1;
}

int mapS::doReordering(int StaLen, int StaOffs, const unsigned char *StaIn, short bX, short bY, int WriteOffs, int &Len)
{
	int a;
	static cellData tempSta[1024][64];
	short offs[64];
	unsigned char staX,staY;
	signed char staZ;
	short staDat, subcellID, d;
	int intraOffs, intraOffs2,b,c;

	for (a=0;a<64; a++) tempSta[0][a].TID=0; // 0'th element: #statics per block ... in .tid.  1...x: actual staData

	for (a=0; a<StaLen; a+=7)
	{
		staDat = ShortFromCharPtrRev(StaIn+StaOffs+a);
	    staX   = StaIn[StaOffs+a+2];
		staY   = StaIn[StaOffs+a+3];
		staZ   = StaIn[StaOffs+a+4];

		assert (staX<8);
		assert (staY<8);

		subcellID = 8*staY + staX;

		tempSta[0][subcellID].TID++;
		d = tempSta[0][subcellID].TID;

		assert(d<1024 && d>0);

		tempSta[d][subcellID].TID=staDat;
		tempSta[d][subcellID].z=staZ;

	} // end of statics per block loop
				

	// write statics data			
	intraOffs = 196 + WriteOffs;
	for (a=0; a<64; a++)
	{
	  assert (intraOffs <= 196+WriteOffs+(StaLen/7)*3);
	  offs[a]=intraOffs;
	  b = tempSta[0][a].TID;
	  if (b!=0)
	  {
		for (c=0; c<b; c++)
		{
		   assert (c<1024 && c>=0);

		   staDat = tempSta[c+1][a].TID;
		   staZ   = tempSta[c+1][a].z;

		   ShortToCharPtr(staDat, staticsShadow[bX][bY].data+intraOffs);
		   staticsShadow[bX][bY].data[intraOffs+2]=staZ;
		   intraOffs+=3;					  
		 }
	  }
	}

	// write index

	Len = intraOffs;	
	assert(intraOffs == (196 + WriteOffs + (StaLen/7)*3));
			
	intraOffs2=WriteOffs;
	short bl;
	for (a=0; a<64; a++)
	{
		assert (intraOffs2<=WriteOffs+192);
		bl = tempSta[0][a].TID;	

		ShortToCharPtr(offs[a], staticsShadow[bX][bY].data+intraOffs2);
		staticsShadow[bX][bY].data[intraOffs2+2]=bl;

		intraOffs2+=3;
	}

	return 1;
}

int mapS::loadStatics(mapInit & mI)
{
	std::string outStri;
	std::string fileStr;

	int eC,a=0;
	unsigned int c=0;

	unsigned char *staI=NULL, *sta=NULL, *sdifl0=NULL, *sdifl1=NULL, *sdifi0=NULL, *sdifi1=NULL, *sdif0=NULL, *sdif1=NULL;

	unsigned long int lenStaI=0, lenSta=0, lenSdifl0=0, lenSdifl1=0, lenSdifi0=0, lenSdifi1=0, lenSdif0=0,lenSdif1=0;


	outStri = "Loading Statics Shadow ...\n";
	Console.send(outStri.c_str());

	outStri = "Loading raw data into RAM\n";
	Console.send(outStri.c_str());

	/////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////
	///
	///   first of all load raw data into RAM
	///
	///////////////////////////////////////////////////////////////////////////////////////////////////////////
    /////////

	outStri = baseDir+mI.staticsIndexFile +"\n";
	Console.send(outStri.c_str());

	fileStr = baseDir + mI.staticsIndexFile;

	unsigned long int len=0;

	eC = getFileLen(fileStr, len);
	if (eC<=0 || len<=0) 
	{
		outStri = "error loading " + convertToString(eC);outStri = outStri + "\n"; Console.send(outStri.c_str());
		return eC;
	}

	staI = new unsigned char[len];
	eC = fileToRAM(fileStr, staI, len);
	if (eC<=0) 
	{
		outStri = "error loading " + convertToString(eC);outStri = outStri + "\n";Console.send(outStri.c_str());
		delete[] staI;
		return eC;
	}

	lenStaI=len;

	len=0;
	outStri = baseDir+mI.staticsFile+"\n"; Console.send(outStri.c_str());
	fileStr = baseDir + mI.staticsFile;

	eC = getFileLen(fileStr, len);
	if (eC<=0 || len<=0) 
	{
		delete[] staI;
		outStri = "error loading " + convertToString(eC);outStri = outStri + "\n"; Console.send(outStri.c_str());
		return eC;
	}

    sta = new unsigned char[len];
	eC = fileToRAM(fileStr, sta, len);
	if (eC<=0) 
	{
		delete[] staI;
		delete[] sta;
		outStri = "error loading " + convertToString(eC);outStri = outStri + "\n";Console.send(outStri.c_str());		
		return eC;
	}

    lenSta=len;

	bool mapdiffs=true;
	bool mapdiffsPar=true;

	// remark: mapdiffs not found is treated as warning !

	len=0;
	outStri = baseDir + mI.diffStaticsLenFile+"\n"; Console.send(outStri.c_str());
	fileStr = baseDir + mI.diffStaticsLenFile;

	eC = getFileLen(fileStr, len);

	if (eC<=0 || len<=0) 
	{
		outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n"; Console.send(outStri.c_str());
		mapdiffs=false;
	}

    if (mapdiffs)
    {
	  sdifl0 = new unsigned char[len];
	  eC = fileToRAM(fileStr, sdifl0, len);
	  if (eC<=0) 
	  {
        delete[] sdifl0;
		outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n";Console.send(outStri.c_str());		
	    mapdiffs=false;
	  }
    }

	if (mapdiffs) lenSdifl0=len;

	len=0;
	outStri = baseDir+mI.diffStaticsIndexFile+"\n";
	Console.send(outStri.c_str());
	fileStr = baseDir + mI.diffStaticsIndexFile;

	if (mapdiffs)
	{
	   eC = getFileLen(fileStr, len);

	   if (eC<=0 || len<=0) 
	   {
		  outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n"; Console.send(outStri.c_str());
		  mapdiffs=false;
	   }

       if (mapdiffs)
	   {
	     sdifi0 = new unsigned char[len];
	     eC = fileToRAM(fileStr, sdifi0, len);
	     if (eC<=0) 
		 {
           delete[] sdifi0;
		   outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n";Console.send(outStri.c_str());		
	       mapdiffs=false;
		 }
	   }
	}
	
	if (mapdiffs) lenSdifi0=len;

	len=0;
	outStri = baseDir+mI.diffStaticsFile+"\n";
	Console.send(outStri.c_str());
	fileStr = baseDir + mI.diffStaticsFile;

	if (mapdiffs)
	{
	   eC = getFileLen(fileStr, len);

	   if (eC<=0 || len<=0) 
	   {
		  outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n"; Console.send(outStri.c_str());
		  mapdiffs=false;
	   }

       if (mapdiffs)
	   {
	     sdif0 = new unsigned char[len];
	     eC = fileToRAM(fileStr, sdif0, len);
	     if (eC<=0) 
		 {
           delete[] sdif0;
		   outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n";Console.send(outStri.c_str());		
	       mapdiffs=false;
		 }
	   }
	}

	if (mapdiffs) lenSdif0=len;

	if (!Dual) mapdiffsPar=false;

	if (Dual)
	{
	   len=0;
	   outStri = baseDir+mI.diffStaticsLenFilePar+"\n";Console.send(outStri.c_str());
	   fileStr = baseDir + mI.diffStaticsLenFilePar;

	   if (mapdiffsPar)
	   {
	     eC = getFileLen(fileStr, len);

	     if (eC<=0 || len<=0) 
		 {
		    outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n"; Console.send(outStri.c_str());
		    mapdiffsPar=false;
		 }

         if (mapdiffsPar)
		 {
	       sdifl1 = new unsigned char[len];
	       eC = fileToRAM(fileStr, sdifl1, len);
	       if (eC<=0) 
		   {
             delete[] sdifl1;
		     outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n";Console.send(outStri.c_str());		
	         mapdiffsPar=false;
		   }
		 }
	   }


	   if (mapdiffsPar) lenSdifl1=len;

	   len=0;
	   outStri = baseDir+mI.diffStaticsIndexFilePar+"\n";Console.send(outStri.c_str());
	   fileStr = baseDir + mI.diffStaticsIndexFilePar;

	   if (mapdiffsPar)
	   {
	     eC = getFileLen(fileStr, len);

	     if (eC<=0 || len<=0) 
		 {
		    outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n"; Console.send(outStri.c_str());
		    mapdiffsPar=false;
		 }

         if (mapdiffsPar)
		 {
	       sdifi1 = new unsigned char[len];
	       eC = fileToRAM(fileStr, sdifi1, len);
	       if (eC<=0) 
		   {
             delete[] sdifi1;
		     outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n";Console.send(outStri.c_str());		
	         mapdiffsPar=false;
		   }
		 }
	   }

	   if (mapdiffsPar) lenSdifi1=len;

	   len=0;
	   outStri = baseDir+mI.diffStaticsFilePar+"\n";Console.send(outStri.c_str());
	   fileStr = baseDir + mI.diffStaticsFilePar;

	   if (mapdiffsPar)
	   {
	     eC = getFileLen(fileStr, len);

	     if (eC<=0 || len<=0) 
		 {
		    outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n"; Console.send(outStri.c_str());
		    mapdiffsPar=false;
		 }

         if (mapdiffsPar)
		 {
	       sdif1 = new unsigned char[len];
	       eC = fileToRAM(fileStr, sdif1, len);
	       if (eC<=0) 
		   {
             delete[] sdif1;
		     outStri = "warning, file not found, not used " + convertToString(eC);outStri = outStri + "\n";Console.send(outStri.c_str());		
	         mapdiffsPar=false;
		   }
		 }
	   }	  
	}

	if (mapdiffsPar) lenSdif1=len;
	len=0;

	if (!mapdiffs)
	{
	   outStri = "warning, mapdiffs not used for " + mainName + "\n"; //outStri = outStri + "\n";
	   Console.send(outStri.c_str());			      
	}

	if (Dual && !mapdiffsPar)
	{
	   outStri = "warning, mapdiffs not used for " + parallelName + "\n"; //outStri = outStri + "\n";
	   Console.send(outStri.c_str());			 
	}

	NumStatics = lenSta/7;

	outStri = "Number of Statics: " + convertToString(NumStatics) +"\n";
	Console.send(outStri.c_str());

	if (mapdiffs)
	{	
	   if (Dual) outStri = "number of patched statics blocks in " + mainName + " " + convertToString(lenSdifl0/4) + "\n";
	   else      outStri = "number of patched statics blocks in " + mapName + " " + convertToString(lenSdifl0/4) + "\n";
	   Console.send(outStri.c_str());

	   	NumStaticPatches=lenSdifl0/4;

	}

	if (mapdiffsPar && Dual)
	{	
	   if (Dual) outStri = "number of patched statics blocks in " + parallelName + " " + convertToString(lenSdifl1/4) + "\n";
	   else      outStri = "number of patched statics blocks in " + mapName + " " + convertToString(lenSdifl1/4) + "\n";
	   Console.send(outStri.c_str());

	   NumStaticPatchesPar=lenSdifl1/4;

	}


	diffs = mapdiffs;
	diffsPar = mapdiffsPar;

	///////////////
	////////////////////////////////////////////////////////////////////////////////////////////
	////////////////// now we have statics data 1:1 in RAM, reorder 1:1 copy,
	////               store reordered data in map object, free 1:1 copy
	//////////////////////////////////////////////////////////////////////////////
	//////////////

	outStri = "Reordering static tiles for fast access...\n" ;
	Console.send(outStri.c_str());

	signed long int x,y,blockID,  staLen,  offset=0, per;
	unsigned long int offsetI=0;
	int ctr=0, l,d;

	short th=0;
					
	for (x=0; x<blockX; x++)
	{
		for (y=0;y < blockY; y++)
		{
			per =  (long) (((float(x*y)))/((float)(blockX*blockY))*100.0);
			if (per>th)
			{
				outStri = convertToString(th) +"%...";
			    Console.send(outStri.c_str());
				if (per<99) th+=10;
			}

			// blocks are stored top/down then left/right in file (cells left/right then top/down)

			blockID = blockY*x + y ;
			offsetI = blockID * 12 ;

			assert (offsetI<lenStaI);

			offset  = LongFromCharPtrRev(staI+offsetI);
			staLen  = LongFromCharPtrRev(staI+offsetI+4);
		
			if (offset==-1) // no statics in this block
			{
				// basicly redundant because already init' to this values, but I want to make this explizit
				staticsShadow[x][y].data=NULL;
				staticsShadow[x][y].memLayout=0;
				staticsShadow[x][y].totLength=0;
			}
			else
			{
				// first put in unpatched data

				staticsShadow[x][y].memLayout=0;

				assert ( ((unsigned)offset < lenSta) && (offset>=0));
				assert ( (staLen < 500000) && (staLen>=0));

				l = (staLen/7)*3 + 192 + 4;
			
				staticsShadow[x][y].data = new unsigned char[l];
				staticsShadow[x][y].totLength=l;

				//do the reordering

				int le;
				doReordering(staLen, offset, sta, x, y, 0, le);
				assert (le==l);
																
			} // if !=-1
		
		} // end y
	} // end x

	outStri = "100%\n";
	Console.send(outStri.c_str());


	outStri = "Adding statics mapdiffs\n";
	Console.send(outStri.c_str());

	///////////////////////////////////////////////////////////////////
	//////////////
	///////                        Add the mapdiffs
	//////////////
	////////////////////////////////////////////////////////////////////

	short bX, bY;
	tmp *mds0, *mds1;
	mds0  = new tmp[ blockY*blockX ];
	mds1  = new tmp[ blockY*blockX ];
	int ctr2;

	for (a=0; a<(blockY*blockX); a++)
	{
		mds0[a].on=false;
		mds1[a].on=false;
	}

	if (mapdiffs)
	{
		d=0; ctr=0;
		for (c=0; c < lenSdifl0; c+=4)
		{		
			blockID = LongFromCharPtrRev(sdifl0+c);		

			bX = blockID / blockY;
			bY = blockID - (blockY*bX);

			assert (blockID <  blockY*blockX );
			assert (bX<blockX);
			assert (bY<blockY);		

			ctr++;

			offset =  LongFromCharPtrRev(sdifi0+d);
			len    =  LongFromCharPtrRev(sdifi0+d+4);

			mds0[blockID].on=true;
			mds0[blockID].len=len;
			mds0[blockID].offset=offset;

			d+=12;
		}
	}

	if (mapdiffsPar)
	{
		assert (Dual);

		d=0; ctr2=0;
		for (c=0; c < lenSdifl1; c+=4)
		{		
			blockID = LongFromCharPtrRev(sdifl1+c);		

			bX = blockID / blockY;
			bY = blockID - (blockY*bX);

			assert (blockID <  blockY*blockX );
			assert (bX<blockX);
			assert (bY<blockY);
	
			ctr2++;		

			offset =  LongFromCharPtrRev(sdifi1+d);
			len    =  LongFromCharPtrRev(sdifi1+d+4);

			mds1[blockID].on=true;
			mds1[blockID].len=len;
			mds1[blockID].offset=offset;

			d+=12;
		}

	}

	int offset2=0, len2=0, origStaLen, origOffsI, origOffset;
	int l1,l2,l3,lTot;
	bool wp,dwp;
	unsigned char *ptr;
	int ctrs[17];
	for (a=0;a<17;a++) ctrs[a]=0;

	for (a=0; a<(blockY*blockX); a++)
	{
		
		wp=false;
		if (mds0[a].on)
		{
			wp=true;
			len    = mds0[a].len;
			offset = mds0[a].offset;
		}

		dwp=false;
		if (mds1[a].on)
		{
			dwp=true;
			len2    = mds1[a].len;
			offset2 = mds1[a].offset;
		}
		
		bX = a / blockY;
		bY = a - (blockY*bX);

		assert (bX < blockX);
		assert (bY < blockY);

		ptr = staticsShadow[bX][bY].data;


		origOffsI   = a*12;	// we need to redo original for patched ons!	
		origOffset  = LongFromCharPtrRev(staI+origOffsI);
	    origStaLen  = LongFromCharPtrRev(staI+origOffsI+4);

		int le;

		if (!wp && !dwp) // memlayout 0
		{
			assert(staticsShadow[bX][bY].memLayout==0);
			ctrs[0]++;

		} else 
		
		/// world patch only
		if (wp && !dwp && offset!=-1 && ptr==NULL)
		{
			ctrs[1]++;

			///// no statics @ original world 
			///// patched statics data @ map
			///// no patched statics @ mirrormap
			///// sum: one datablock containing patches for map

			staticsShadow[bX][bY].memLayout=1;
		
			lTot = (len/7)*3 + 192 + 4;
			
			staticsShadow[bX][bY].data = new unsigned char[lTot];
			staticsShadow[bX][bY].totLength = lTot;
			
			doReordering(len, offset, sdif0, bX, bY, 0, le);
		    assert (le==lTot);

		} else

		if (wp && !dwp && offset!=-1 && ptr!=NULL)  
		{
			ctrs[2]++;

			///// statics @ original world 
			///// patched statics data @ map
			///// no patched statics @ mirrormap
			///// sum: two datablocks containing patches for map and original map

			staticsShadow[bX][bY].memLayout=2;

			delete [] staticsShadow[bX][bY].data;

			assert(origOffset!=-1);

			l1 = (origStaLen/7)*3 + 192 + 4;
			l2 = (len/7)*3 + 192 + 4;
			lTot =  l1+l2;

			staticsShadow[bX][bY].data = new unsigned char[lTot];
			staticsShadow[bX][bY].totLength = lTot;

			doReordering(origStaLen, origOffset, sta, bX, bY, 0, le);
			assert(le==l1);
			doReordering(len, offset, sdif0, bX, bY, l1, le);
			assert(le==lTot);

			ShortToCharPtr(l1, staticsShadow[bX][bY].data+192); // write index to 2'nd data block


		} else

		if (wp && !dwp && offset==-1 && ptr==NULL) // patched reverted case = no real patch
		{
			ctrs[3]++;	

			///// no statics @ original world 
			///// no patched statics data @ map (actually data at mapdiff files, but invalidated)
			///// no patched statics @ mirrormap
			///// sum: no data at all

			assert(staticsShadow[bX][bY].memLayout==0);	

		} else

		if (wp && !dwp && offset==-1 && ptr!=NULL)  
		{
			ctrs[4]++;

			///// statics @ original world 
			///// patched statics data @ map to no statics
			///// no patched statics @ mirrormap
			///// sum: one datablock containing orginal map
			
			assert(origOffset!=-1 && staticsShadow[bX][bY].memLayout==0);

		} else

		// Dualmap patch only

		if (!wp && dwp && offset2!=-1 && ptr==NULL)
		{
		   ctrs[5]++;

		   	///// no statics @ original world 
			///// no patched statics data @ map
			///// patched statics @ mirrormap
			///// sum: one datablock containing mirror map data

		   	staticsShadow[bX][bY].memLayout=5;

			assert (len2!=-1);
		
			lTot = (len2/7)*3 + 192 + 4;
			
			staticsShadow[bX][bY].data = new unsigned char[lTot];
			staticsShadow[bX][bY].totLength = lTot;
			
			doReordering(len2, offset2, sdif1, bX, bY, 0, le);
		    assert (le==lTot);


		} else

		if (!wp && dwp && offset2!=-1 && ptr!=NULL)  
		{
		   ctrs[6]++;

		    ///// statics @ original world 
			///// no patched statics data @ map
			///// patched statics @ mirrormap
			///// sum: two datablocks containing mirror map data and original world

		   	staticsShadow[bX][bY].memLayout=6;

			delete [] staticsShadow[bX][bY].data;

			assert(origOffset!=-1);

			l1 = (origStaLen/7)*3 + 192 + 4;
			l2 = (len2/7)*3 + 192 + 4;
			lTot =  l1+l2;

			staticsShadow[bX][bY].data = new unsigned char[lTot];
			staticsShadow[bX][bY].totLength = lTot;

			doReordering(origStaLen, origOffset, sta, bX, bY, 0, le);
			assert(le==l1);
			doReordering(len2, offset2, sdif1, bX, bY, l1, le);
			assert(le==lTot);

			ShortToCharPtr(l1, staticsShadow[bX][bY].data+192); // write index to 2'nd data block


		} else

		if (!wp && dwp && offset2==-1 && ptr==NULL)  // patch reverted case
		{
		   ctrs[7]++;

		    ///// no statics @ original world 
			///// no patched statics data @ map
			///// no patched statics @ mirrormap (but data in diff file that says data invalidated)
			///// sum: no data

		   assert(staticsShadow[bX][bY].memLayout==0);	

		} else

		if (!wp && dwp && offset2==-1 && ptr!=NULL)  
		{
		   ctrs[8]++;

		   	///// statics @ original world 
			///// no patched statics data @ map
			///// patched statics @ mirrormap (pathced to nom statics)
			///// sum: one datablock containg original world

		   	staticsShadow[bX][bY].memLayout=8;
			assert(origOffset!=-1);

		} else

		///////// both maps patched
	
		if (wp && dwp && offset!=-1 && offset2!=-1 && ptr==NULL)
		{

			// original map: no data
			// patched statics @ map
			// patched statics @ mirrormap
			// sum: 2 data blocks containing map+mirrormap patches

		    staticsShadow[bX][bY].memLayout=9;

			l1 = (len/7)*3 + 192 + 4;
			l2 = (len2/7)*3 + 192 + 4;
			lTot =  l1+l2;

			staticsShadow[bX][bY].data = new unsigned char[lTot];
			staticsShadow[bX][bY].totLength = lTot;

			doReordering(len, offset, sdif0, bX, bY, 0, le);
			assert(le==l1);
			doReordering(len2, offset2, sdif1, bX, bY, l1, le);
			assert(le==lTot);

			ShortToCharPtr(l1, staticsShadow[bX][bY].data+192); // write index to 2'nd data block

		} else

		if (wp && dwp && offset!=-1 && offset2==-1 && ptr==NULL)  // dual patch reverted case 
		{
		   ctrs[10]++;

		   	// original map: no data
		   	// patched statics @ map
			// no statics @ mirrormap (actually in data files, but reverted/invlaidated)
			// sum: 1 data block containing map patches

		   // warning: cut and paste from case 1, because equivalent // (changing if at 1 makes ot harder to maintain!)
		    staticsShadow[bX][bY].memLayout=1;
		
		    lTot = (len/7)*3 + 192 + 4;
			
		    staticsShadow[bX][bY].data = new unsigned char[lTot];			
			staticsShadow[bX][bY].totLength = lTot;
			
			doReordering(len, offset, sdif0, bX, bY, 0, le);
		    assert (le==lTot);

		} else

		if (wp && dwp && offset==-1 && offset2!=-1 && ptr==NULL)  // world patch reverted case
		{
			ctrs[11]++;

			// original map: no data
		   	// no patched statics @ map (actually in data files, but reverted/invlaidated)
			// patched statics @ mirrormap 
			// sum: 1 data block containing mirrormap patches

		   // warning: cut and paste from case 5, because equivalent // (changing if at 5 makes ot harder to maintain!)

			staticsShadow[bX][bY].memLayout=5;

			assert (len2!=-1);
		
			lTot = (len2/7)*3 + 192 + 4;
			
			staticsShadow[bX][bY].data = new unsigned char[lTot];
			staticsShadow[bX][bY].totLength = lTot;
			
			doReordering(len2, offset2, sdif1, bX, bY, 0, le);
		    assert (le==lTot);


		} else

		if (wp && dwp && offset==-1 && offset2==-1 && ptr==NULL)  // both patches reverted case 
		{			
			// no map data, both no patched (invalidated)
			ctrs[12]++;
			assert (staticsShadow[bX][bY].memLayout==0);

		} else

		if (wp && dwp && offset!=-1 && offset2!=-1 && ptr!=NULL)
		{
		   ctrs[13]++;

		   // original map: data
		   // patched statics @ map
		   // patched statics @ mirrormap 
		   // sum: 3 data block containing mirrormap patches, map patches and original world

		   staticsShadow[bX][bY].memLayout=13;

		   delete[] staticsShadow[bX][bY].data;

		   assert(origOffset!=-1);
		   assert(len!=(unsigned)-1 && len2!=-1);

		   l1 = (origStaLen/7)*3 + 192 + 4;
		   l2 = (len/7)*3 + 192 + 4;
		   l3 = (len2/7)*3 + 192 + 4;
		   lTot =  l1+l2+l3;

		   staticsShadow[bX][bY].data = new unsigned char[lTot];
		   staticsShadow[bX][bY].totLength = lTot;

		   doReordering(origStaLen, origOffset, sta, bX, bY, 0, le);
		   assert(le==l1);
		   doReordering(len, offset, sdif0, bX, bY, l1, le);
		   assert(le==l1+l2);
		   doReordering(len2, offset2, sdif1, bX, bY, l1+l2, le);
		   assert(le==lTot);

		   ShortToCharPtr(l1, staticsShadow[bX][bY].data+192);    // write index to 2'nd data block
		   ShortToCharPtr(l1+l2, staticsShadow[bX][bY].data+194); // write index to 3'nd data block

		} else

		if (wp && dwp && offset!=-1 && offset2==-1 && ptr!=NULL)  
		{
		   ctrs[14]++;

		   // original map: data
		   // patched statics @ map 
		   // no patched statics @ mirrormap (in map files but reverted)
		   // sum: 2 data block containing map patches and original world

		   // equivalent to 2, cut and past from 2

		   	staticsShadow[bX][bY].memLayout=2;

			delete [] staticsShadow[bX][bY].data;

			assert(origOffset!=-1);

			l1 = (origStaLen/7)*3 + 192 + 4;
			l2 = (len/7)*3 + 192 + 4;
			lTot =  l1+l2;

			staticsShadow[bX][bY].data = new unsigned char[lTot];
			staticsShadow[bX][bY].totLength = lTot;

			doReordering(origStaLen, origOffset, sta, bX, bY, 0, le);
			assert(le==l1);
			doReordering(len, offset, sdif0, bX, bY, l1, le);
			assert(le==lTot);

			ShortToCharPtr(l1, staticsShadow[bX][bY].data+192); // write index to 2'nd data block

		} else

		if (wp && dwp && offset==-1 && offset2!=-1 && ptr!=NULL)
		{
			ctrs[15]++;

			// original map: data
		    // no patched statics @ map (in map files but reverted)
		    // patched statics @ mirrormap 
		    // sum: 2 data block containing mirrormap patches and original world

		    // equivalen to 6, cut and past from 6

			staticsShadow[bX][bY].memLayout=6;

			delete [] staticsShadow[bX][bY].data;

			assert(origOffset!=-1);

			l1 = (origStaLen/7)*3 + 192 + 4;
			l2 = (len2/7)*3 + 192 + 4;
			lTot =  l1+l2;

			staticsShadow[bX][bY].data = new unsigned char[lTot];
			staticsShadow[bX][bY].totLength = lTot;

			doReordering(origStaLen, origOffset, sta, bX, bY, 0, le);
			assert(le==l1);
			doReordering(len2, offset2, sdif1, bX, bY, l1, le);
			assert(le==lTot);

			ShortToCharPtr(l1, staticsShadow[bX][bY].data+192); // write index to 2'nd data block

		} else

		if (wp && dwp && offset==-1 && offset2==-1 && ptr!=NULL)  
		{
			ctrs[16]++;

			// original map: data
		    // no patched statics @ map (in map files but reverted)
		    // no patched statics @ mirrormap (in map files but reverted)
		    // sum: 1 data block containing original map

			assert(staticsShadow[bX][bY].memLayout==0);

		}

		assert (staticsShadow[bX][bY].memLayout!=3 && staticsShadow[bX][bY].memLayout!=7 && staticsShadow[bX][bY].memLayout!=12
             && staticsShadow[bX][bY].memLayout!=16 && staticsShadow[bX][bY].memLayout!=10 && staticsShadow[bX][bY].memLayout!=11
			 && staticsShadow[bX][bY].memLayout!=14 && staticsShadow[bX][bY].memLayout!=15 && staticsShadow[bX][bY].memLayout!=4);

	}

	//////////
	// note: equivalent states: 3,7,12,16,4 -> 0
	//                          10 ->1
	//                          11 ->5
	//                          14 ->2
	//                          15 ->6
	//
	// hence used states: 0,1,2,5,6,8,9,13
	// in current OSI mapdiffs 9 not used
	// dense code possible (0..7), but not used
	//////////

	int deads=ctrs[3]+ctrs[7]+ctrs[10]+ctrs[11]+ctrs[12];
	outStri="number of reverted(dead) statics-patches: "+convertToString(deads)+"\n";
	Console.send(outStri.c_str());

	//for (a=0; a<17; a++) printf("ML:%i %i\n",a,ctrs[a]);
	//printf("\n");

	delete[] mds0;
	delete[] mds1;

	if (mapdiffs)
	{
	  delete[] sdif0;
	  delete[] sdifl0;
	  delete[] sdifi0;
	}

	if (mapdiffsPar && Dual)
	{
	  delete[] sdif1;
	  delete[] sdifl1;
	  delete[] sdifi1;
	}

	delete[] staI;
	delete[] sta;
	
	return 1;
}

mapS::mapS(mapInit &mI, std::string Dir):Code(mI.Code), parallelCode(mI.parallelCode), blockX(mI.blockX), blockY(mI.blockY), maxCacheFullness(mI.maxCacheFull), Dual(mI.dualMap)
{

  std::string outStri;
  int x,y;
  int f;

  errorCode  = 0;

  mF = NULL;

  posQLRU     = 0;
  maxLRUStack = 5000;
  blocksUsed  = 0;
  
  cacheHits = 0;
  cacheMisses = 0;
  flushes = 0;

  mapName =mI.mapName;
  mainName=mI.mainName;
  parallelName=mI.parallelName;
  baseDir = Dir;

  NumMapPatches=0;
  NumStaticPatches=0;
  NumMapPatchesPar=0;
  NumStaticPatchesPar=0;

  // check if mul-files ok
  f = verifyFileZ(mI);
  if (f!=1) 
  {	  
	  outStri ="invalid map-files size\n";
	  Console.send(outStri.c_str());
	  errorCode=-1;
	  return;
  }

  if (Dual) outStri = "Constructing: " + mapName +" ( " + mainName + " + " + parallelName +" )\n";
  else                         outStri = "Constructing: " + mapName +"\n";

  Console.send(outStri.c_str());

  // create dynamic 2D array ([blockx][blocky])
  mapCache = new mapTag* [blockX];
  for (x=0; x<blockX; x++)
  {
	  mapCache[x] = new mapTag[blockY];
  }

  // same for statics-shadow
  staticsShadow = new statShadow* [blockX];
  for (x=0; x<blockX; x++)
  {
	  staticsShadow[x] = new statShadow[blockY];
  }

  // init both
  for (x=0; x<blockX; x++)
  {
	  for (y=0; y<blockY; y++)
	  {
		  mapCache[x][y].data=NULL;
		  mapCache[x][y].memLayout=0;		
		  mapCache[x][y].tag=0;	
		  

		  staticsShadow[x][y].data=0;
		  staticsShadow[x][y].memLayout=0;
		  staticsShadow[x][y].totLength=0;
	  }
  }

  QuasiLRU         = new block[maxLRUStack];
  
  f = loadStatics(mI);
  if (f!=1) 
  {
	  outStri = "fatal error in loadStatics\n";
	  Console.send(outStri.c_str());
	  errorCode=-2;
	  return;
  }

  f = initMapCache(mI);
  if (f!=1) 
  {
	  outStri = "fatal error in initMapCache\n";
	  Console.send(outStri.c_str());
	  errorCode=-3;
	  return;
  }

}

/////
///// in: x,y,dual,usemapdiffs
///// dual: request for map or dualmap ? false: map, true: dualmap
///// out: bx,by, len(=num of statics in this cell), offs
///// access to data/usage: getRawStatics(x,y,?,bx,by,?, len,offs,?); if (len>0) ptr = staticShadow[bx][by].data+offs else ptr = NULL;
////////
int mapS::getRawStatics(short x, short y, bool dualrequ,  short &bX, short &bY, unsigned char &SCID, unsigned char &len, short &off , bool usemapdiffs)
{
	unsigned char const* p;
	int memLayout, intraOffs;
	short rX,rY;
	short start;	

	bX = x / 8;
	bY = y / 8;

	//assert(x>=0 && y>=0);
	// note: unfortunately there are too many sources of invalid map-coords
	// eg loops that dont check for bounds, hence you cant use it with assertion on, only enable for debugging/killing that sources
	if (x<0 || y<0) 
	{
		len=0;
		return -1;
	}

    //assert(bX<blockX && bY<blockY);
	if (bX>=blockX || bY>=blockY) 
	{
		len=0;
		return -1;
	}

	rX = x - bX*8;
	rY = y - bY*8;

    assert (rX>=0 && rX<8);
	assert (rY>=0 && rY<8);
		   
    SCID = (8*rY + rX);

    memLayout = staticsShadow[bX][bY].memLayout;
	p = staticsShadow[bX][bY].data;
		
	if ( memLayout==0 || !usemapdiffs )
	{
		if (p==NULL || memLayout==1 || memLayout==5 || memLayout==9) // 1,5,9: no data in unpatched-world, but data patched
		{
			off = 0;
			len = 0;
			return 1;

		} else
		{

		   assert (p!=NULL);
  		   assert (staticsShadow[bX][bY].totLength>=199);
         
		   intraOffs = SCID*3;			   
		   off       = ShortFromCharPtr(p+intraOffs);
		   len       = p[intraOffs+2];

		   if (len!=0) assert(staticsShadow[bX][bY].totLength>off+2);
		   return 1;

		}
	}
	else 
	{
		assert (p!=NULL);
		assert (staticsShadow[bX][bY].totLength>=199);

		switch(memLayout)
		{
		   case 1: // 0/1/0, 1 Datablock

			   if (dualrequ) // no data for par-map
			   {
				  off = 0;
			      len = 0;
			      return 1;

			   } else       // patched map for normal map
			   {				  
		           intraOffs = SCID*3;		           
		           off       = ShortFromCharPtr(p+intraOffs);
		           len       = p[intraOffs+2];

		           if (len!=0) assert(staticsShadow[bX][bY].totLength>off+2);
				   return 1;
			   }

			   break;

		   case 2: // 1/1/0, 2 Datablocks

			   if (dualrequ) // orig data for par-map
			   {				  
		           intraOffs = SCID*3;		         
		           off       = ShortFromCharPtr(p+intraOffs);
		           len       = p[intraOffs+2];

		           if (len!=0) assert(staticsShadow[bX][bY].totLength>off+2);
				   return 1;


			   } else // patched map for normal map
			   {			
				   start     = ShortFromCharPtr(p+192);
				   intraOffs = SCID*3+start;
				   off       = ShortFromCharPtr(p+intraOffs);
				   len       = p[intraOffs+2];

				   if (len!=0) assert(staticsShadow[bX][bY].totLength>start);
				   if (len!=0) assert(staticsShadow[bX][bY].totLength>intraOffs);
		           if (len!=0) assert(staticsShadow[bX][bY].totLength>off+2);
				   return 1;
			   }
			   break;

		   case 5: // 0/0/1, 1 Datablock

			   if (dualrequ)
			   {
				   intraOffs = SCID*3;		           
		           off       = ShortFromCharPtr(p+intraOffs);
		           len       = p[intraOffs+2];

		           if (len!=0) assert(staticsShadow[bX][bY].totLength>off+2);
				   return 1;

			   }
			   else
			   {
				  off = 0;
			      len = 0;
			      return 1;

			   }
			   break;

		   case 6: // 1/0/1, 2 datablocks

			   if (dualrequ)
			   {
				   start     = ShortFromCharPtr(p+192);
				   intraOffs = SCID*3+start;
				   off       = ShortFromCharPtr(p+intraOffs);
				   len       = p[intraOffs+2];

				   if (len!=0) assert(staticsShadow[bX][bY].totLength>start);
				   if (len!=0) assert(staticsShadow[bX][bY].totLength>intraOffs);
		           if (len!=0) assert(staticsShadow[bX][bY].totLength>off+2);
				   return 1;
			   } 
			   else
			   {
				   intraOffs = SCID*3;		         
		           off       = ShortFromCharPtr(p+intraOffs);
		           len       = p[intraOffs+2];

		           if (len!=0) assert(staticsShadow[bX][bY].totLength>off+2);
				   return 1;
			   }
			   break;

		   case 8: 

			    intraOffs = SCID*3;		         
		        off       = ShortFromCharPtr(p+intraOffs);
		        len       = p[intraOffs+2];
		        if (len!=0) assert(staticsShadow[bX][bY].totLength>off+2);
				// actually #8 redundant, could be set to ML 0 ...
				return -1;
		
			    break;

		   case 9: // 0/1/1 , 2 datablocks containg patches for both worlds

			   if (dualrequ)
			   {
				   start     = ShortFromCharPtr(p+192);
				   intraOffs = SCID*3+start;
				   off       = ShortFromCharPtr(p+intraOffs);
				   len       = p[intraOffs+2];

				   if (len!=0) assert(staticsShadow[bX][bY].totLength>start);
				   if (len!=0) assert(staticsShadow[bX][bY].totLength>intraOffs);
		           if (len!=0) assert(staticsShadow[bX][bY].totLength>off+2);
				   return 1;

			   }
			   else
			   {
				   intraOffs = SCID*3;		         
		           off       = ShortFromCharPtr(p+intraOffs);
		           len       = p[intraOffs+2];

		           if (len!=0) assert(staticsShadow[bX][bY].totLength>off+2);
				   return 1;

			   }

			   break;

		   case 13: // 1/1/1, 3 data blocks contiang normal worlds + both patched worlds

			   if (dualrequ)
			   {
				   start     = ShortFromCharPtr(p+194);
				   intraOffs = SCID*3+start;
				   off       = ShortFromCharPtr(p+intraOffs);
				   len       = p[intraOffs+2];

				   if (len!=0) assert(staticsShadow[bX][bY].totLength>start);
				   if (len!=0) assert(staticsShadow[bX][bY].totLength>intraOffs);
		           if (len!=0) assert(staticsShadow[bX][bY].totLength>off+2);
				   return 1;

			   }
			   else
			   {
				   start     = ShortFromCharPtr(p+192);
				   intraOffs = SCID*3+start;
				   off       = ShortFromCharPtr(p+intraOffs);
				   len       = p[intraOffs+2];

				   if (len!=0) assert(staticsShadow[bX][bY].totLength>start);
				   if (len!=0) assert(staticsShadow[bX][bY].totLength>intraOffs);
		           if (len!=0) assert(staticsShadow[bX][bY].totLength>off+2);
				   return 1;

			   }

			   break;

		   default:
			   // error/warning !
			   return -2;
			   break;

		   return 1;

		}
	}


	return 1;
}

int mapS::getRawMap(short x, short y, bool dual, short & TID, signed char &z, bool usemapdiffs)
{
	std::string outStri;
	short bX, bY, rX, rY, SCID;
	int memLayout, blockID, offs,offs2,r;
	unsigned char *p;	

	bX = x / 8;
	bY = y / 8;

	//assert(x>=0 && y>=0);
	// note: unfortunately there are too many sources of invalid map-coords
	// eg loops that dont check for bounds, hence you cant use it with assertion on, only for real debugging
	if (x<0 || y<0) 
	{
		TID=-1;
		return -1;
	}

	//assert(bX<blockX && bY<blockY);
	if (bX>=blockX || bY>=blockY) 
	{
		TID=-1;
		return -1;
	}

	rX = x - bX*8;
	rY = y - bY*8;

    assert (rX>=0 && rX<8);
	assert (rY>=0 && rY<8);
		   
    SCID = (8*rY + rX);

    memLayout = mapCache[bX][bY].memLayout;
	p = mapCache[bX][bY].data;

	// subset of cells that have memLayout 1-3 (mapdiffs) are no cache !
	if (!usemapdiffs && memLayout>0) // perma-cachehit by design (even if mapdiff no request, it's there)
	{
		cacheHits++;
		assert (p!=NULL);
		assert(memLayout<=3);
	
		offs = SCID*3;
		assert(offs<=(192-3));
		TID = ShortFromCharPtrRev(p+offs);
		z   = p[offs+2];
		return 1;
	}
	
	if (usemapdiffs && memLayout>0) // perma-cachehit by design
	{
		cacheHits++;
		assert(memLayout<=3);
	    assert (p!=NULL);

		switch(memLayout)
		{
		   case 1:

			   if (dual)
			   {
				  offs = SCID*3;
		          assert(offs<=(192-3));
		          TID = ShortFromCharPtrRev(p+offs);
		          z   = p[offs+2];
		          return 1;
			   }
			   else
			   {
				   offs = 192+SCID*3;
				   assert(offs<=(2*192-3));
				   TID = ShortFromCharPtrRev(p+offs);
		           z   = p[offs+2];
		           return 1;
			   }
			   break;

		   case 2:

			   if (dual)
			   {
				   offs = 192+SCID*3;
				   assert(offs<=(2*192-3));
				   TID = ShortFromCharPtrRev(p+offs);
		           z   = p[offs+2];
		           return 1;
			   }
			   else
			   {
				  offs = SCID*3;
		          assert(offs<=(192-3));
		          TID = ShortFromCharPtrRev(p+offs);
		          z   = p[offs+2];
		          return 1;

			   }
			   break;

		   case 3:

			   if (dual)
			   {
				   offs = 2*192+SCID*3;
				   assert(offs<=(3*192-3));
				   TID = ShortFromCharPtrRev(p+offs);
		           z   = p[offs+2];
		           return 1;
			   }
			   else
			   {
				   offs = 192+SCID*3;
				   assert(offs<=(2*192-3));
				   TID = ShortFromCharPtrRev(p+offs);
		           z   = p[offs+2];
		           return 1;
			   }
			   break;			 
		}

		return 1;
	}

	assert(memLayout==0);

	//////////////////// Cache Mamagement /////////////////

	if ( mapCache[bX][bY].tag ) // cachehit
	{
		cacheHits++;
		assert (p!=NULL);
		// get data and return;

		offs = SCID*3;
		assert(offs<=(192-3));
		TID = ShortFromCharPtrRev(p+offs);
		z   = p[offs+2];		        
		return 1;

	}
	else // cache miss, lots of things to do ...
	{
		// first of all, get data from file

		cacheMisses++;

		blockID = blockY*bX + bY ;
		offs2 = 196*blockID+4;
		assert( (offs2+192)<=(blockX*blockY*196));

		r = fseek(mF, offs2, SEEK_SET);

		if (r) 
		{
		   outStri = "error: cant seek into mapfile33\n";
	       Console.send(outStri.c_str());
		   return -2;
		}		

		mapCache[bX][bY].data = new unsigned char [192];

		r = fread(mapCache[bX][bY].data, sizeof(char), 192, mF);
		if (r!=192)
		{
		   outStri = "error: cant read mapfile99\n";
	       Console.send(outStri.c_str());
		   return -3;
		}

		mapCache[bX][bY].tag=true; // now it's a cachehit next time of couse ...

		// return result
		p = mapCache[bX][bY].data;
		offs = SCID*3;
		assert(offs<=(192-3));
		TID = ShortFromCharPtrRev(p+offs);
		z   = p[offs+2];

		/// update fullnesscounter
		blocksUsed++;

		//// update LRU-stack
		if (posQLRU<maxLRUStack) // note: no dupes cauz only added on miss
		{
			QuasiLRU[posQLRU].x=x;
			QuasiLRU[posQLRU].y=y;
			posQLRU++;		
		}

		//// cache replacement necassairy ?

		int per;
		per = int(((float) blocksUsed / (float)(blockX*blockY))*100.0);	

        if (per<=maxCacheFullness) return 1; // we are done if cache not full

		// cache full, replace old cells (at once to get < per for a while)
		// due to rounding maxLRU should be >1%, in case of 768*512 approx 4000
		// hence 5000, shouldn't get below. you might try to incerase, but dont decrease

		flushes++;

		int a;
		short xx,yy,bx,by;
		for (a=0; a<posQLRU; a++)
		{
			xx = QuasiLRU[a].x;
			yy = QuasiLRU[a].y;

			bx = xx / 8;
			by = yy / 8;

			assert (bx<=blockX);
			assert (by<=blockY);

			assert (mapCache[bx][by].data!=NULL);
			assert (mapCache[bx][by].tag==true);

			delete [] mapCache[bx][by].data;
			mapCache[bx][by].tag=false;	
		}

		blocksUsed-=posQLRU;
		posQLRU = 0;

		return 1;
		
	}
	
	return 1;

}

#include "mapstuff.h"

int mapS::verifyMap(void)
{
	/*
	int x,y;

	unsigned char len, cid;
	int loopexit,a;
	short n_ID, o_ID, bx,by, off;
	signed char o_z, n_z;
	staticrecord *stat;
	bool hit;
	unsigned char* p;*/



	return 1;

	printf("verifying maps\n");

	/*
	for (x=0; x<blockX*8; x++)
	{
		for (y=0; y<blockY*8; y++)
		{		  
           getRawStatics(x,y, false, bx,by, cid, len,off, false);

		   assert (bx<=blockX && bx>=0);
		   assert (by<=blockY && by>=0);
		  
		   if (len>0) p = staticsShadow[bx][by].data+off; else p=NULL;
         
		   		 
		   for (a=0; a<len; a++)
		   {

			   assert((off+3*a+2)<staticsShadow[bx][by].totLength);
			   assert(p!=NULL);
			   			 			 
			   n_ID = ShortFromCharPtr(p+3*a);
			   n_z  = p[3*a+2];


			   MapStaticIterator msi(x, y);
			   loopexit=0;
			   hit =false;
		       while ( (stat = msi.Next()) && (++loopexit < MAXLOOPS) )
			   {
			     o_ID = stat->itemid;
				 o_z  = stat->zoff;

				 if ( (o_ID==n_ID) && (o_z==n_z) )
				 { 
					 hit=true;
					 break;
				 }
			   }

			   if (hit==false) 
			   {
				   printf("validation failed @(%i %i) M:%i\n",x,y, staticsShadow[bx][by].memLayout);
				   return -1;
			   }

		   }		  		  
		
		}
	}

	printf("done\n");

	printf("start of new code:\n");

	for (x=0; x<blockX*8; x++)
	{
		for (y=0; y<blockY*8; y++)
		{		  
           getRawStatics(x,y, false, bx,by, cid, len,off, false);
		   assert (bx<=blockX && bx>=0);
		   assert (by<=blockY && by>=0);
		   
		   getRawStatics(x,y, false, bx,by, cid, len,off, true);
		   assert (bx<=blockX && bx>=0);
		   assert (by<=blockY && by>=0);
		   getRawStatics(x,y, true, bx,by, cid, len,off, false);
		   assert (bx<=blockX && bx>=0);
		   assert (by<=blockY && by>=0);
		   getRawStatics(x,y, true, bx,by, cid, len,off, true);
		   assert (bx<=blockX && bx>=0);
		   assert (by<=blockY && by>=0);
		  
		   if (len>0) p = staticsShadow[bx][by].data+off; else p=NULL;		   
		   		 
		   for (a=0; a<len; a++)
		   {

			   assert((off+3*a+2)<staticsShadow[bx][by].totLength);
			   assert(p!=NULL);
			   			 			 
			   n_ID = ShortFromCharPtr(p+3*a);
			   n_z  = p[3*a+2];
		   }
		}
	}

	printf("end of new code:\n");

	printf("start of old code:\n");

	for (x=0; x<blockX*8; x++)
	{
		for (y=0; y<blockY*8; y++)
		{		  

			MapStaticIterator msi(x, y);
			loopexit=0;			 
		    while ( (stat = msi.Next()) && (++loopexit < MAXLOOPS) )
			   {
			     o_ID = stat->itemid;
				 o_z  = stat->zoff;
			}					   		  		  
		
		}
	}

	*/

/*
	printf("start veri of map\n");

	short TID;
	signed char z;
*/

	/*
	//// covering all codepaths
	for (x=0; x<blockX*8; x++)
	{
	  for (y=0; y<blockY*8; y++)
	  {
		  getRawMap(x,y, true, TID, z, true);
		  getRawMap(x,y, true, TID, z, false);
		  getRawMap(x,y, false, TID, z, true);
		  getRawMap(x,y, false, TID, z, false);
	  }

	}
 */

	/*

	printf("new M code start\n");
	for (x=0; x<blockX*8; x++)
	{
	  for (y=0; y<blockY*8; y++)
	  {
		  getRawMap(x,y, false, TID, z, false);
	  }

	}
	printf("done\n");

	printf("old M code start\n");
	for (x=0; x<blockX*8; x++)
	{
	  for (y=0; y<blockY*8; y++)
	  {
		  Map->SeekMap0(x,y);
	  }

	}
	printf("done\n");

	printf("comparing\n");

	// comparing
	map_st M;
	bool ok;
	for (x=0; x<blockX*8; x++)
	{
	  for (y=0; y<blockY*8; y++)
	  {
		  getRawMap(x,y, false, TID, z, false);
		  M = Map->SeekMap0(x,y);

		  if (M.id==TID && M.z==z) ok=true; else 
		  {
			  ok=false;
			  printf("veri failed: (%i / %i) M:%i %i NM:%i %i\n",x,y,M.id, M.z, TID, z);
			  //return -91;
		  }
	  }

	}


	printf("end veri of map\n");*/


	return 1;
}

mapS::~mapS()
{
	
   int x,y;
   unsigned char* ptr;

   if (errorCode==0)
   {
     for (x=0; x<blockX; x++)
	 {
	  for (y=0; y<blockY; y++)
	  {
		  ptr = staticsShadow[x][y].data;
		  if (ptr!=NULL) delete [] ptr;

		  ptr = mapCache[x][y].data;
		  if (ptr!=NULL) delete [] ptr;

	  }
	 }

	 delete[] QuasiLRU;
	 if (mF!=NULL) fclose(mF);

   }

   
}
