/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//


#include <string>
#include "stdio.h"
#include "coord.h"
#include "assert.h"
#include "basics.h"
#include "typedefs.h"
#include "network.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "mapS.h"

struct tmp
{
	bool on;
	int offset;
	int len;
};

struct block
{
	short x;
	short y;
};

struct cellData
{
    unsigned short TID;
    signed char z;
};


struct mapTag
{
	unsigned char *data;
	bool tag;
	unsigned char memLayout;
};

struct statShadow
{
	unsigned char *data;
	unsigned char memLayout;
	short totLength;

};

struct mapInit
{
 
	short Code;
	short parallelCode; // map CODES client(!) understands

	short blockX;
	short blockY;

	unsigned char maxCacheFull;

	char dualMap;

	// all associated file names
	std::string mainName;
	std::string parallelName;
	std::string mapName;

	std::string mapFile;
	std::string staticsIndexFile;
	std::string staticsFile;

	std::string diffMapLenFile;
	std::string diffMapFile;

	std::string diffStaticsIndexFile;
	std::string diffStaticsLenFile;
	std::string diffStaticsFile;

	std::string diffMapLenFilePar;
	std::string diffMapFilePar;

	std::string diffStaticsIndexFilePar;
	std::string diffStaticsLenFilePar;
	std::string diffStaticsFilePar;

};

class mapS
{
   private:

	 const short Code;
	 const short parallelCode;

	 const short blockX;
	 const short blockY;

	 const short maxCacheFullness;

	 const char Dual;

	 int NumMapPatches;
	 int NumStaticPatches;
	 int NumMapPatchesPar;
	 int NumStaticPatchesPar;
	 int NumStatics;

	 bool diffs;
	 bool diffsPar;

	 unsigned int cacheHits;
	 unsigned int cacheMisses;
	 unsigned int flushes;
	 short maxLRUStack; // quasi const
	
     mapTag         **mapCache;	 // already support for par inside !
	 statShadow     **staticsShadow;
	 block           *QuasiLRU;	 

	 short posQLRU;
	 int blocksUsed;

	 FILE* mF;

	 short errorCode;

	 std::string mapName;
	 std::string mainName;
	 std::string parallelName;
	 std::string baseDir;

	 // staticIter	
	 int currIterPos;
	 int IterEnd;
	 unsigned char* currIterPtr;
	 /// for verification of iterator
	 int offset_saved;
	 int bx_saved, by_saved;

	 //////////////////////////////

	 int verifyFileZ(mapInit &);
	 int loadStatics(mapInit &);
	 int initMapCache(mapInit &);

	 // helper for constructing memory	for statics
	 int fileToRAM(std::string Dir,  unsigned char*, unsigned long int len);
	 int getFileLen(std::string Dir,  unsigned long int & len);
	 inline int doReordering(int StaLength, int Staffset, const unsigned char* in, short bX, short bY, int WriteOffs, int &Len);

     // helper getting actual data
	 int getRawStatics(short x, short y, bool dual, short &bX, short &bY, unsigned char &SCID, unsigned char &len, short &off , bool usemapdiffs);
	 int getRawMap(short x, short y, bool dual, short &TID, signed char &z, bool usemapdiffs);

   public:

	   mapS( mapInit& mI, std::string baseDir);
	   short getErrorCode() { return errorCode; }

	   int verifyMap(void);

	   void StaticIteratorStart(unsigned short x, unsigned short y, bool dual)
	   {
		   short bx,by, offs;
		   unsigned char len,ignored;
		   bool useDUAL=false; // change if dual maps implemented (phase 4)

		   getRawStatics(x, y, dual, bx, by, ignored, len, offs, useDUAL);

		   if (len>0) 
		   {
			   offset_saved = offs;
			   bx_saved     = bx;
			   by_saved     = by;

			   currIterPos  = 0;
			   IterEnd      = len;
			   currIterPtr  = staticsShadow[bx][by].data+offs;
		   }
		   else
		   {
			   currIterPos = 0;
			   IterEnd     = 0;
			   currIterPtr = NULL;
		   }
	   }

	   bool StaticIteratorNext(cellData &data)
	   {	
		  if ( (currIterPos<IterEnd) && (currIterPtr!=NULL) )
		  {
			 assert((offset_saved+3*currIterPos+2)<staticsShadow[bx_saved][by_saved].totLength);

			 data.TID = ShortFromCharPtr(currIterPtr+3*currIterPos);
			 data.z   = currIterPtr[3*currIterPos+2];

             currIterPos++;
			 return true;	

		   } else return false;		 
	   }


	   void seekMap(unsigned short x, unsigned short y, bool dual, cellData &data)
	   {
		   short _TID_;
		   signed char _z_;
		   bool useDUAL=false; // change if dual maps implemented (phase 4)

		   getRawMap(x, y, dual, _TID_, _z_, useDUAL);
		   
		   data.TID = _TID_;
		   data.z   = _z_;
	   }

	   inline int getBX(void) { return blockX; }
	   inline int getBY(void) { return blockY; }	   

	   inline int getCode(void)     { return Code; }
	   inline int getParCode(void)  { return parallelCode; }
	   inline bool getDualness(void) { if (Dual) return true ; else return false; }

	   inline int getNumMapDifPatches(void)        { return NumMapPatches; }
	   inline int getNumMapDifPatchesPar(void)     { return NumMapPatchesPar; }
	   inline int getNumStaticDifPatches(void)     { return NumStaticPatches; }
	   inline int getNumStaticDifPatchesPar(void)  { return NumStaticPatchesPar; }
	  			   
	   void getCachstats(int & currFullness, int & maxFullness, unsigned int& misses, unsigned int &hits )
	   {
		   int per = int(((float) blocksUsed / (float)(blockX*blockY))*100.0);	
		   currFullness = per;
		   maxFullness  = maxCacheFullness;
		   misses = cacheMisses;
		   hits = cacheHits;
	   }

	   ~mapS(void);

};


class mapDispatcher
{

	private:
        mapS **allMaps;
		const unsigned char maxMaps;
		short registeredMaps;

		unsigned char *mapdiffPacket;
		short diffpacketLen;
		unsigned char switchmapPacket[10];

		// iter data;
		int  IterMap;
		bool IterDual;

		inline int toInternalMapNum(int map)
		{		
			int a;
			for (a=0; a<registeredMaps; a++)
			{
				if (allMaps[a]->getCode()==map || (allMaps[a]->getParCode()==map && allMaps[a]->getDualness() ) ) return a;
			}
			return (a+128);
		}
				
	public:

		inline bool isValidMap(int clientMapNum) 
		{ 
			int n=toInternalMapNum(clientMapNum); 
			if (n>=128) return false; else return true; 
		}
		
		mapDispatcher(unsigned char actualMaps):maxMaps(actualMaps), registeredMaps(0)
		{
			allMaps = new mapS*[maxMaps];			
		}

		int registerMap( mapInit& mI, std::string baseDir)
		{
			if (registeredMaps<0 || registeredMaps>=maxMaps) 
			{			
				return -1;
			}

			mapS *newMap = new mapS(mI, baseDir);

			if (newMap->getErrorCode()<0)
			{
				delete newMap;
				return -2;
			}
			
			allMaps[registeredMaps]=newMap;

			registeredMaps++;
			return 1;
		}

		void constructMapdiffPacket(void)
		{
		 
		  int a;		  
		  int mps=0;
		  short len;
		  for (a=0; a<registeredMaps; a++) 
		  {
			mps++;
			if (allMaps[a]->getDualness()) mps++;			 
		  }

		  len=mps*8+9;
		  diffpacketLen=len;
		  mapdiffPacket = new unsigned char[len+2]; // +2: just being schizo
	
		  mapdiffPacket[0]=0xBF;
		  ShortToCharPtr(len,  &mapdiffPacket[1]);
		  ShortToCharPtr(0x18, &mapdiffPacket[3]);
		  LongToCharPtr(mps,   &mapdiffPacket[5]);

		  int difs;
		  int ctr=9;
		  for (a=0; a<registeredMaps; a++)
		  {
            difs=allMaps[a]->getNumMapDifPatches();
			LongToCharPtr(difs,   &mapdiffPacket[ctr]);
			ctr+=4;

			difs=allMaps[a]->getNumStaticDifPatches();
			LongToCharPtr(difs,   &mapdiffPacket[ctr]);
			ctr+=4;

			if (allMaps[a]->getDualness())
			{
			  difs=allMaps[a]->getNumMapDifPatchesPar();
			  LongToCharPtr(difs,   &mapdiffPacket[ctr]);
			  ctr+=4;

			  difs=allMaps[a]->getNumStaticDifPatchesPar();
			  LongToCharPtr(difs,   &mapdiffPacket[ctr]);
			  ctr+=4;
			}
		  }

		  assert(ctr==diffpacketLen);
		}

		void SendMapDifPacket(UOXSOCKET s)
		{
		   //printf("TT: %i\n", diffpacketLen);
		   XStartPkt(0xBF, diffpacketLen);
		   XAddPkt(&mapdiffPacket[0+3], diffpacketLen-3);
		   XFinishPkt(s);
		}


		~mapDispatcher(void)
		{
			int a;
			for (a=0; a<registeredMaps; a++) 
			{
			   delete allMaps[a];
			}

			delete[] allMaps;
			delete[] mapdiffPacket;
		}

		//// copy of all public mapS methods with additional map# param here (inlinded)

		int verifyMaps(void)
		{
			int a;
			for (a=0; a<registeredMaps; a++)
			{
				allMaps[a]->verifyMap();
			}

			return 1;
		}

	   void mapStaticIteratorStart(const cCoord &coord) 
	   {
		   bool dual;
		   int  map ;
		   map = toInternalMapNum(coord.map);		  
		   if (map>=registeredMaps || map<0) 
		   {
			   char temp[1024];
			   sprintf(temp, "error: invalid map:%i\n", map);
			   LogError(temp);
			   return;
		   }

		   dual = allMaps[map]->getDualness();

		   IterMap  = map;
		   IterDual = dual;

		   allMaps[map]->StaticIteratorStart(coord.x, coord.y, dual);		   		
	   }

	   bool mapStaticIteratorNext(cellData &D)
	   {		 		  
		   if (IterMap>=registeredMaps || IterMap<0) 
		   {
			   char temp[1024];
			   sprintf(temp, "error: invalid map:%i\n", IterMap);
			   LogError(temp);
			   return false;
		   }
		   
		   return allMaps[IterMap]->StaticIteratorNext(D);   		
	   }

	   void SeekMap(const cCoord &coord, cellData &D)
	   {
		   bool dual;
		   int  map ;
		   map  = toInternalMapNum(coord.map);		   
		   if (map>=registeredMaps || map<0) 
		   {
			   char temp[1024];
			   sprintf(temp, "error: invalid map:%i\n", map);
			   LogError(temp);
			   return;
		   }

		   dual = allMaps[map]->getDualness();

		   allMaps[map]->seekMap(coord.x, coord.y, dual, D);
	   }

	   int GetMapHeight(int num)
	   {
		   int map = toInternalMapNum(num);
		   if (map<0 || map>=registeredMaps) return 0;
		   return allMaps[map]->getBY();
	   }

	   int GetMapWidth(int num)
	   {
		   int map = toInternalMapNum(num);
		   if (map<0 || map>=registeredMaps) return 0;
		   return allMaps[map]->getBX();		   
	   }

	   int getCachstats(int num, int &maxF, int &currF, unsigned int& miss, unsigned int& hit)
	   {
		    int map = toInternalMapNum(num);
		    if (map<0 || map>=registeredMaps) return 0;
			allMaps[map]->getCachstats(currF, maxF, miss, hit );		   		 
			return 1;
	   }

	   int getNumMaps(void) { return registeredMaps; }

};
