/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

#include "mapstuff.h"

#include <assert.h>
#include "basics.h"
#include "regions.h"
#include "multi.h"
#include "mapS.h"
#include "tiledata.h"

#include "wolfpack.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "mapstuff.cpp"

//#define DEBUG_MAP_STUFF	1

#ifdef DEBUG_MAP_STUFF
void bitprint(FILE *fp, unsigned char x)
{
	for (int i = 7; i >= 0; --i)
	{
		if ((x & 0x80) == 0)
			fprintf(fp, "0");
		else
			fprintf(fp, "1");
		if (4 == i)
			fprintf(fp, " ");
		x = x << 1;
	}
}
#endif


signed char cMapStuff::TileHeight(int tilenum)
{
	const tileData *tile;	
	tile = TileData->SeekTile(tilenum);
	
	// For Stairs+Ladders
	if (TileData->isClimbable(tile->flags)) return (signed char)(tile->height/2);

    /// note1 old code: flag2 &0x4 -> that's climbale ... should it be stairs, not sure ...
	/// left old one for now

	return (tile->height);
}


//////////////
// Name:	StaticTop
// Purpose:	Top of statics at/above given coordinates
//

signed char cMapStuff::StaticTop(const cCoord& pos, signed char oldz)
{
	signed char top = illegal_z;
	int loopexit=0;
	cellData stat;
//	cCoord c(x,y,0,0); // NOTE: needs to be changed when mulitple maps get implemented

	AllMaps->mapStaticIteratorStart(pos);
	while ( (AllMaps->mapStaticIteratorNext(stat)) && (++loopexit < MAXLOOPS) )
	{
		signed char tempTop = stat.z + TileHeight(stat.TID);
		if ((tempTop <= oldz + MaxZstep) && (tempTop > top))
		{
			top = tempTop;
		}
	}
	return top;
}	

///////////////////////////////////////////////////////////////////
// author  : Lord Binary 17/8/99
// purpose : check if something is under a (static) roof.
//           I wrote this to check if a player is inside a static building (with roof)
//           to disable rain/snow when they enter buildings 
//           this only works for static buildings.
//           Its perfect for weather stecks, if you want make a in_building() 
//           you need to call it with x,y x+1,y x,y+1,x-1,y,x,y-1 
//           if they all return 1 AND the player isnt running under a underpass or bridge 
//           he/she is really in a building. though idunno yet how to check the later one (underpassing checks)
//           probably with the floor-bit
///////////////////////////////////////////////////////////////////////

bool cMapStuff::IsUnderRoof(short int x, short int y, signed char z)
{
	cellData stat;
	cCoord c(x,y,0,0); // NOTE: needs to be changed when mulitple maps get implemented

	AllMaps->mapStaticIteratorStart(c);

	int loopexit=0;
	while ( (AllMaps->mapStaticIteratorNext(stat)) && (++loopexit < MAXLOOPS) )
	{
		const tileData *tile;	
		tile = TileData->SeekTile(stat.TID);
		
		if (TileData->isFloor(tile->flags) && (tile->height+stat.z)>z )
			return false; // check the floor bit
		   // if set -> this must be a underpassing/bridge
		
		if ((tile->height+stat.z)>z) // a roof  must be higher than player's z !
		{
			if (strstr("roof", (char *) tile->name) || strstr("shingle", (char *) tile->name)) 
				return true;			
			
			// now why would not want to check the z value of wooden boards first??
			// this was after the if (.. >z), i'm moving this up inside of it
			if (!strcmp((char *) tile->name,"wooden boards")) return true;
			// ok, well going by the commented out section above we've likely got some bugs
			// because we've left out checking for "stone pavern" and "wooden board"
			// i'll stick these back in. even if these were bogus tile names it can't hurt
			if (!strcmp((char *) tile->name,"wooden board") ||
				!strcmp( (char *) tile->name, "stone pavern") ||
				!strcmp( (char *) tile->name, "stone pavers"))
				return true;
		}
	}
	return false;
}

//
// i guess this function returns where the tile is a 'blocker' meaning you can't pass through it
bool cMapStuff::DoesTileBlock(int tilenum)
{
	const tileData *tile;
	
	tile = TileData->SeekTile(tilenum);	
	return (TileData->isBlocking(tile->flags));
}



// crackerjack 8/9/1999 - finds the "corners" of a multitile object. I use
// this for when houses are converted into deeds.
void cMapStuff::MultiArea(PC_ITEM iMulti, cRectangle& rect)
{
	int x1,y1,x2,y2;
	Map->MultiArea(iMulti, &x1,&y1,&x2,&y2);
	rect.init(x1,y1,x2,y2);
}

void cMapStuff::MultiArea(PC_ITEM iMulti, int *x1, int *y1, int *x2, int *y2)
{
	if (!iMulti) return;

	int LoopExit=0;
	singleMultiPart mult;
	
	*x1 = *y1 = *x2 = *y2 = 0;	

	assert(iMulti->id()>=0x4000);

	Multi->multiIteratorStart(iMulti->id()-0x4000);

	while (Multi->multiIteratorNext(mult) && ++LoopExit<MAXLOOPS)
	{       
	  if (mult.x<*x1) *x1=mult.x;
	  if (mult.x>*x2) *x2=mult.x;
	  if (mult.y<*y1) *y1=mult.y;
	  if (mult.y>*y2) *y2=mult.y;
	}

	*x1+=iMulti->pos.x;
	*x2+=iMulti->pos.x;
	*y1+=iMulti->pos.y;
	*y2+=iMulti->pos.y;
}

// return the height of a multi item at the given x,y. this seems to actually return a height

signed char cMapStuff::MultiHeight(P_ITEM iMulti, const cCoord& pos, signed char oldz)
{  
	if (!iMulti) return 0;

	int LoopExit=0;
	singleMultiPart mult;
	assert(iMulti->id()>=0x4000);

	Multi->multiIteratorStart(iMulti->id()-0x4000);
	while (Multi->multiIteratorNext(mult) && ++LoopExit<MAXLOOPS)
	{
		if (mult.visibility && (iMulti->pos.x+mult.x == pos.x) && (iMulti->pos.y+mult.y == pos.y))
		{
			int tmpTop = iMulti->pos.z + mult.z;
			if ((tmpTop<=oldz+MaxZstep)&& (tmpTop>=oldz-1))
			{
				//clConsole.send("At or above=%i\n",multi.z);
				return mult.z;
			}
			else if ((tmpTop>=oldz-MaxZstep)&& (tmpTop<oldz-1))
			{
				//clConsole.send("Below=%i\n",multi.z);
				return mult.z;
			}
		}                                                                                                                 
	}
	return 0;                                                                                                                     
} 

// This was fixed to actually return the *elevation* of dynamic items at/above given coordinates
signed char cMapStuff::DynamicElevation(const cCoord& pos, signed char oldz)
{
	//int z = illegal_z;
	signed char z = illegal_z;
	signed char tempVal = illegal_z;

	RegionIterator ri(pos, ItemsOnly);
	for (int mapitem = ri.First(); mapitem != -1; mapitem = ri.Next())
	{
		P_ITEM iMapitem = MAKE_ITEM_REF(mapitem);
		if (iMapitem==NULL) return illegal_z;

		if (ri.IsMulti())
		{
			z = MultiHeight(iMapitem, pos, oldz);
			// this used to do a z++, but that doesn't take into account the fact that
			// the itemp[] the multi was based on has its own elevation		

			tempVal = z + iMapitem->pos.z + 1;
			z = tempVal;
		}
		else if (iMapitem->pos.IsSameXY(pos))
		{
			signed char ztemp = iMapitem->pos.z + TileHeight( iMapitem->id() );
			if ((ztemp <= oldz + MaxZstep) && (ztemp > z))
			{
				z=ztemp;
			}
		}
	}

	return z;
}


int cMapStuff::MultiTile(P_ITEM iMulti, cCoord& pos, signed char oldz)
{
	if (!iMulti) return 0;

    int LoopExit=0;
	singleMultiPart mult;
	assert(iMulti->id()>=0x4000);

	Multi->multiIteratorStart(iMulti->id()-0x4000);
	while (Multi->multiIteratorNext(mult) && ++LoopExit<MAXLOOPS)
	{
	   if ((mult.visibility && (iMulti->pos.x+mult.x == pos.x) && (iMulti->pos.y+mult.y == pos.y)
			&& (abs(iMulti->pos.z+mult.z-oldz)<=1)))
		{
			int mt=mult.tile;
			/*if (DoesTileBlock(mt)) return mt;
			else return 0;*/
			return mt;
		}
	}
	return 0;
}

////////////////////////////////////
// returns which dynamic tile is present at (x,y) or -1 if no tile exists
// originally by LB & just michael
////////////////////////////////////

int cMapStuff::DynTile(cCoord& pos, signed char oldz)
{
	RegionIterator ri(pos, ItemsOnly);

	for (int mapitem = ri.First(); mapitem != -1; mapitem = ri.Next())
	{
		P_ITEM iMapitem = MAKE_ITEM_REF(mapitem);
		if (iMapitem==NULL) return -1;

		if (ri.IsMulti())
		{
			return MultiTile(iMapitem, pos, oldz);
		}
		else if (iMapitem->pos.IsSameXY(pos))
		{
			return iMapitem->id();
		}
	}

	return -1;
}

////////////////////////////////
// return the elevation of MAP0.MUL at given coordinates, we'll assume since its land
// the height is inherently 0
/////////////////////////////////

signed char cMapStuff::MapElevation(const cCoord& pos)
{
	cellData map;
	AllMaps->SeekMap(pos, map);

	// make sure nothing can move into black areas
	if (430 == map.TID || 475 == map.TID || 580 == map.TID || 610 == map.TID ||
		611 == map.TID || 612 == map.TID || 613 == map.TID)
		return illegal_z;
	// more partial black areas
	//if ((map.id >= 586 && map.id <= 601) || (map.id >= 610 && map.id <= 613))
	//	return illegal_z;
	return map.z;
}

//////////////////////////
// compute the 'average' map height by looking at three adjacent cells
//int cMapStuff::AverageMapElevation(int x, int y, int &id)
///////////////////////////

signed char cMapStuff::AverageMapElevation(const cCoord& pos, int &id)
{
	// first thing is to get the map where we are standing
	cellData map1;

	AllMaps->SeekMap(pos, map1);
	id = map1.TID;
	// if this appears to be a valid land id, <= 2 is invalid
	if (map1.TID > 2 && illegal_z != MapElevation(pos))
	{
		// get three other nearby maps to decide on an average z?
		cCoord tpos(pos);
		tpos.Add(1);	// x+1
		signed char map2z = Map->MapElevation(tpos);
		tpos=pos;
		tpos.Add(0,1);	// y+1
		signed char map3z = Map->MapElevation(tpos);
		tpos.Add(1);	// x+1, y+1
		signed char map4z = Map->MapElevation(tpos);
		
		signed char testz = 0;
		if (abs(map1.z - map4z) <= abs(map2z - map3z))
		{
			if (illegal_z == map4z)
				testz = map1.z;
			else
			{
				testz = (signed char)((map1.z + map4z) >> 1);
				if (testz%2<0) --testz;
				// ^^^ Fix to make it round DOWN, not just in the direction of zero
			}
		} else {
			if (illegal_z == map2z || illegal_z == map3z)
				testz = map1.z;
			else
			{
				testz = (signed char)((map2z + map3z) >> 1);
				if (testz%2<0) --testz;
				// ^^^ Fix to make it round DOWN, not just in the direction of zero
			}
		}
		return testz;
	}
//	clConsole.send("Uhh.. Someone's walking on something funny..\n");
	return illegal_z;
}


// Blocking statics at/above given coordinates?
bool cMapStuff::DoesStaticBlock( short int x, short int y, signed char oldz )
{
	cellData stat;
	cCoord c(x,y,0,0); // NOTE: needs to be changed when implenting muliple maps

	AllMaps->mapStaticIteratorStart(c);
	int loopexit=0;
	while ( (AllMaps->mapStaticIteratorNext(stat)) && (++loopexit < MAXLOOPS) )
	{
		const int elev = stat.z + TileHeight(stat.TID);	
		if ( (elev >= oldz) && (stat.z <= oldz ) )
		{		
			bool btemp=TileData->isTileBlocking(stat.TID);					
			if (btemp) return true;
		}
	}
	return false;
}

// Return new height of player who walked to X/Y but from OLDZ
//int cMapStuff::Height(int x, int y, int oldz)
signed char cMapStuff::Height(cCoord& pos, signed char oldz)
{
	// let's check in this order.. dynamic, static, then the map
	signed char dynz = DynamicElevation(pos, oldz);
	if (illegal_z != dynz)
		return dynz;

	signed char staticz = StaticTop(pos, oldz);
	if (illegal_z != staticz)
		return staticz;

	return MapElevation(pos);
}

// can the monster move here from an adjacent cell at elevation 'oldz'
// use illegal_z if they are teleporting from an unknown z
bool cMapStuff::CanMonsterMoveHere(cCoord& pos, signed char oldz)
{
	int mWidth  = AllMaps->GetMapWidth(pos.map);
	int mHeight = AllMaps->GetMapHeight(pos.map);

	if ( pos.x >= ( mWidth * 8 ) || pos.y >= ( mHeight * 8 ) ) return false;

    const signed char elev = Height(pos, oldz);
	if (illegal_z == elev)
		return false;

	// is it too great of a difference z-value wise?
	if (oldz != illegal_z)
	{
		// you can climb MaxZstep, but fall up to 15
		if (elev - oldz > MaxZstep)
			return false;
		else if (oldz - elev > 15)
			return false;
	}

    // get the tile id of any dynamic tiles at this spot
    const int dt= DynTile(pos,elev);

	/**************************************************************/
	/// note: not quite correct
	/// returns ONE item, even if >=1 present.
	/// if this one is not blocking, the remaining ones might be !
	/*****************************************************************/
	
    // if there is a dynamic tile at this spot, check to see if its a blocker
    // if it does block, might as well short-circuit and return right away
    if (dt >= 0 && DoesTileBlock(dt))
		return false;
	

	/////////////////////////////////////////////////////////////////
    // if there's a static block here in our way, return false
	////////////////////////////////////////////////////////////////

	cellData stat;
	AllMaps->mapStaticIteratorStart(pos);
	int loopexit=0;
	while ( (AllMaps->mapStaticIteratorNext(stat)) && (++loopexit < MAXLOOPS) )
	{
		const int el = stat.z + TileHeight(stat.TID);	
		if (el >= elev && stat.z <= elev)
		{		
			if ( TileData->isTileBlocking(stat.TID) || TileData->isTileWet(stat.TID) ) return false;
		}
	}


	/////////////////////////////////////////////
	// check if base map blocks or is water!!
	/////////////////////////////////////////////

	cellData map1;
	AllMaps->SeekMap(pos, map1);
	if ( TileData->isLandBlocking(map1.TID)  || TileData->isLandWet(map1.TID) ) return false;	
	
    return true;
}


bool cMapStuff::IsRoofOrFloorTile( tileData *tile )
// checks to see if the tile is either a roof or floor tile
{
	if (TileData->isFloor(tile->flags)) return true; // check the floor bit
	
	if (strstr("roof", (char *) tile->name) || strstr("shingle", (char *) tile->name)) 
		return true;

	if (strstr( "floor", (char *)tile->name ))
		return true;
	// now why would not want to check the z value of wooden boards first??
	// this was after the if (.. >z), i'm moving this up inside of it
	if (!strcmp((char *) tile->name,"wooden boards"))
		return true;
	// i'll stick these back in. even if these were bogus tile names it can't hurt
	if (!strcmp((char *) tile->name,"wooden board") ||
		!strcmp( (char *) tile->name, "stone pavern") ||
		!strcmp( (char *) tile->name, "stone pavers"))
		return true;

	return false;
}

bool cMapStuff::IsRoofOrFloorTile( unitile_st *tile )
// checks to see if the tile is either a roof or floor tile
{
	const tileData *newTile;
	newTile = TileData->SeekTile( tile->id);
	return IsRoofOrFloorTile( (tileData*)newTile );
}

bool cMapStuff::IsBlockedByChar(cCoord* pos)
{
  cChar* pc;
  int loopexit=0;
  int p;
  RegGrid3x3Iterator4Chars rg3ic(*pos);
  while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
  {
	p = pos->distance(pc->pos);		
    if (p==0) return true;   
  }
  return false;

}

bool cMapStuff::IsBlockedByItem(cCoord* pos)
{
	return !CanMonsterMoveHere(*pos, pos->z);
}

bool cMapStuff::IsBlockedByCharOrItem(cCoord* pos)
{
	return (IsBlockedByChar(pos) || IsBlockedByItem(pos));
}

