/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

/////////////////////////////////////////
/// rewrite: LB Sept 2002
/////////////////////////////////////////

//////////////////////////////////////////
////
//// this class covers access to multi.mul 
////
//////////////////////////////////////////

#include "multi.h"
#include "assert.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "multi.cpp"


multis::multis(std::string basedir)
{  
   std::string mulDir, mulIdxDir,t;
   std::string verDir; 
   FILE *mif=NULL, *mf=NULL, *vf=NULL;
   int s,r;
   unsigned long int length;

   error=0;

   mulIdxDir = basedir + "multi.idx";
   mulDir    = basedir + "multi.mul";
   verDir    = basedir + "verdata.mul";

   t = "\n-----------------------------------------------------------\n\nLoading " + mulDir +"\n";
   Console.send(t.c_str());

   mif = fopen(mulIdxDir.c_str(), "rb");
   if (mif==NULL)
   {
     t = "fatal error: " + mulIdxDir + " not found, quitting\n";
	 Console.send(t.c_str());
	 error=1;
	 return;
   }   

   mf = fopen(mulDir.c_str(), "rb");
   if (mf==NULL)
   {
     t = "fatal error: " + mulDir + " not found, quitting\n";
	 Console.send(t.c_str());
	 error=1;
	 return;
   }   

   ///////////////////////
   /// getmulticount;
   ///////////////////////

   // get multi length;
   s = fseek(mif, 0, SEEK_END);
   if (s!=0) 
   {
	 t = "fatal error seking1: " + mulIdxDir + "\n";Console.send(t.c_str());
	 error=1;return;
   }
	
   length = ftell(mif);

   if (length<=0) 
   {	
	 t = "fatal error seking3: " + mulIdxDir + "\n";Console.send(t.c_str());
	 error=1;return;
   }

   s=fseek(mif,0, SEEK_SET); 
   if (s!=0) 
   {
	 t = "fatal error seeking2: " + mulIdxDir + "\n";Console.send(t.c_str());
	 error=1;return;	   
   }

   length=length/12;

   //// load multi index

   int realnum=0;
   unsigned int a,st;
   for (a=0; a<length; a++)
   {
	   r = fread(&st, sizeof(int), 1, mif);
	   if (r!=1)
	   {
		  t = "fatal error reading1: " + mulIdxDir + "\n";Console.send(t.c_str());
	      error=1;return;	   
	   }

	   if (st!=0xffffffff)
	   {
		   realnum++;
	   }

	   r = fseek(mif, 8, SEEK_CUR); // skip over remaing 8 bytes
	   if (r!=0)
	   {
		  t = "fatal error seeking4: " + mulIdxDir + "\n";Console.send(t.c_str());
	      error=1;return;	   
	   }
   }

   assert(realnum>0);

   s=fseek(mif,0, SEEK_SET);  // reset fp to start
   if (s!=0) 
   {
	 t = "fatal error seeking2: " + mulIdxDir + "\n";Console.send(t.c_str());
	 error=1;return;	   
   }

   multiCount = realnum;

   ///////////////////////////
   //// load multi data
   ///////////////////////////

   assert(length<=8196);
   if (length>=8196)
   {
	   Console.send("patched muldata.idx with too many (>8195) entries\n");
	   error=1;
	   return;
   }

   for (a=0; a<8196; a++)
   {
	   theMultis[a].multiParts=NULL;
	   theMultis[a].count=0;
	   theMultis[a].ishouse=false;
   }

   unsigned int start, size, dummy,chi=0, d, tot=0;
   for (a=0; a<length; a++)
   {	   
	   r = fread(&start, sizeof(int),1, mif);
	   if (r!=1)
	   {
		  t = "fatal error reading10: " + mulIdxDir + "\n";Console.send(t.c_str());
	      error=1;return;	   
	   }

	   r = fread(&size, sizeof(int),1, mif);
	   if (r!=1)
	   {
		  t = "fatal error reading11: " + mulIdxDir + "\n";Console.send(t.c_str());
	      error=1;return;	   
	   }

	   r = fread(&dummy, sizeof(int),1, mif);
	   if (r!=1)
	   {
		  t = "fatal error reading12: " + mulIdxDir + "\n";Console.send(t.c_str());
	      error=1;return;	   
	   }
	 
	   if (start!=0xffffffff)
	   {
		   int mc = size/12;
		   theMultis[tot].multiParts = new singleMultiPart[mc];
		   theMultis[tot].count=mc;		

		   //printf("tot:%04x\n",tot);

		   unsigned short block, x,y,z;
		   unsigned int flag;

		   r = fseek(mf, start, SEEK_SET);
		   if (r!=0)
		   {
			 t = "fatal error seeking12: " + mulDir + "\n";Console.send(t.c_str());
	         error=1;return;
		   }

		   for (d=0; d<(unsigned)mc; d++)
		   {		    

		     r = fread(&block, sizeof(short), 1, mf);
		     if (r!=1)
			 {
			   t = "fatal error reading12: " + mulDir + "\n";Console.send(t.c_str());
	           error=1;return;
			 }

		     r = fread(&x, sizeof(short), 1, mf);
		     if (r!=1)
			 {
			   t = "fatal error reading13: " + mulDir + "\n";Console.send(t.c_str());
	           error=1;return;
			 }

		     r = fread(&y, sizeof(short), 1, mf);
		     if (r!=1)
			 {
			   t = "fatal error reading14: " + mulDir + "\n";Console.send(t.c_str());
	           error=1;return;
			 }

		     r = fread(&z, sizeof(short), 1, mf);
		     if (r!=1)
			 {
			   t = "fatal error reading15: " + mulDir + "\n";Console.send(t.c_str());
	           error=1;return;
			 }

		     r = fread(&flag, sizeof(int), 1, mf);
		     if (r!=1)
			 {
			   t = "fatal error reading16: " + mulDir + "\n";Console.send(t.c_str());
	           error=1;return;
			 }

			 theMultis[tot].multiParts[d].tile=block;			 

			 //if (tot<3) printf("multi-tile:%04x %04x\n",block, tot);

			 theMultis[tot].multiParts[d].x=x;
			 theMultis[tot].multiParts[d].y=y;
			 theMultis[tot].multiParts[d].z=(signed char)z;

			 if      (flag==0) theMultis[tot].multiParts[d].visibility=false;
			 else if (flag==1) theMultis[tot].multiParts[d].visibility=true;
			 else
			 {
				 printf("warning: strange multi flag: %i\n", flag);
				 theMultis[tot].multiParts[d].visibility=true;
			 }

		   } // end of multi-blocks loop
		   chi++;
	   } // end if valid multi index entry
	   tot++;
   }

   assert(chi==(unsigned)realnum);

   ////////////////////////////////////
   /// verdata.mul present ?
   // if yes load its patches
   ////////////////////////////////////

    bool verdata;
   	vf = fopen(verDir.c_str(), "rb");
	if (vf==NULL) verdata=false; else verdata=true;

	if (verdata)
	{
	   Console.send("Verdata found\n");
	   int numpatch;		

	   r = fread(&numpatch, sizeof(int), 1, vf);
	   if (r!=1) 
	   {
			Console.send("but invalid, not using it\n");		
			return;
	   }
		
	   int b=0; unsigned long int pos;	
	   int type, block, extra, size, position;

	   for (a=0; a<(unsigned)numpatch; a++)
	   {
			pos = 4+a*20;

			r = fseek(vf, pos, SEEK_SET);
			if (r!=0)
			{
			   t = "error seeking1: " + verDir + "\n";Console.send(t.c_str());		  			  
		       return;
			}

			r = fread(&type, sizeof(int), 1, vf);
			if (r!=1)
			{
			   t = "error reading1: " + verDir + "\n";Console.send(t.c_str());		  			  
		       return;
			}

			if (type==0x0e) // 0x0e multi.mul patch magic number
			{		
			  b++;			
			  
		      r = fread(&block, sizeof(int),1, vf);
			  if (r!=1)
			  {
			    t = "error reading2: " + verDir + "\n";Console.send(t.c_str());		  			  
		        return;
			  }

			  r = fread(&position, sizeof(int),1, vf);
			  if (r!=1)
			  {
			    t = "error reading3: " + verDir + "\n";Console.send(t.c_str());		  			  
		        return;
			  }

			  r = fread(&size, sizeof(int),1, vf);
			  if (r!=1)
			  {
			    t = "error reading4: " + verDir + "\n";Console.send(t.c_str());		  			  
		        return;
			  }

			  r = fread(&extra, sizeof(int),1, vf);
			  if (r!=1)
			  {
			    t = "error reading4: " + verDir + "\n";Console.send(t.c_str());		  			  
		        return;
			  }

			} // end of multi patch specific if
			else
			{					
			  fseek(vf, 16, SEEK_CUR); // skip over next 16 bytes if type mismatching
			}			
	   } // end of verdata patches loop
	   
	   if (b==0) sprintf(temp,"number of verdata multi patches: 0\n");
	   else sprintf(temp, "We have a problem, %i multi patches in verdata, but verdata multis not supported yet\nplz inform developers\n",b);

	   //// unfortunatly I've never seen a verdata multi patch so I can't implement yet.
	   //// put in code for it here

	   Console.send(temp);

	} // end of verdata if

	d=0;
	for (a=0; a<8196; a++)
	{
		if (theMultis[a].count>0 && theMultis[a].multiParts)
		{
			d+=theMultis[a].count;
		}
	}

	sprintf(temp, "%i tiles in %i multis\n", d, multiCount);
	Console.send(temp);

   Console.send("Done\n");

   if (mif) fclose(mif);
   if (mf)  fclose(mf);
   if (vf)  fclose(vf);

   ////////////////// check house.scp to see which multis are houses //////////////

  // old harcoded list
  for (a=0x64; a<=0x7f; a++) theMultis[a].ishouse=true;
  theMultis[0x87].ishouse=true;
  theMultis[0x8d].ishouse=true;
  theMultis[0x96].ishouse=true;
  theMultis[0x98].ishouse=true;
  theMultis[0x9a].ishouse=true;
  theMultis[0x9c].ishouse=true;
  theMultis[0x9e].ishouse=true;
  theMultis[0xa0].ishouse=true;
  theMultis[0xa2].ishouse=true;
  theMultis[0xbb8].ishouse=true;
  theMultis[0x1388].ishouse=true;

	// search in house.scp token truehouse for scriptable exceptions
	int loopexit=0;
	int tmp,id=0,i;

	for (i=0;i<512;i++)
	{
		Script* Scp = findSection(house_script,"HOUSE",i);
		if (!Scp) continue;
		while (Scp->NextLineSplitted() && (++loopexit < MAXLOOPS) );
		{
			if (!strcmp(script1,"ID")) /// note -> ID has to come before any other tags !!!
			{		   
				id=hstr2num(script2);
			}
			else if (!strcmp(script1,"TRUEHOUSE"))
			{
				tmp=str2num(script2);
				id-=0x4000;
				if (id<0 || id>=8196)
				{
					sprintf(temp,"build house list error:invalid multi ID:%i\n",id);
					LogError(temp);
					continue;
				}
				if (tmp) theMultis[id].ishouse=true; else theMultis[id].ishouse=false;
			}	
		}
		Scp->Close();
	}
	
	int hctr=0;
	for (a=0; a<8196; a++)
	{
		if (theMultis[a].ishouse) hctr++;
	}
	
	sprintf(temp,"Number of houses: %i\n",hctr);
	Console.send(temp);
}
