/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

#if !defined(__MULTI_H__)
#define __MULTI_H__

#include "platform.h"
#include "typedefs.h"
#include "wolfpack.h"
#include "assert.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "multi.h"

struct singleMultiPart
{
	unsigned short tile;
	signed short x;
	signed short y;
	signed char z;
	bool visibility;
};

struct multi
{
	int count;
	singleMultiPart* multiParts;
	bool ishouse;
};

class multis
{
  private:
	int error;
	int multiCount;
	multi theMultis[8196];

	// needed for iterator
	int currIterPos;
	int currIterEnd;
	int currIterId;

  public:
	multis::multis(std::string basedir);

	multis::~multis(void)
	{
		assert(multiCount>0);
		int a;
		for (a=0; a<8196; a++) if (theMultis[a].multiParts) delete[] (theMultis[a].multiParts);	
	}

	int getError(void) { return error; }

	bool ishouse(unsigned short id) 
	{ 
		if (id>=8196)
		{
			sprintf(temp,"ishouse: Invalid multi num:%i\n", id);
			LogError(temp);	
			return false;
		}
		
		return theMultis[id].ishouse;
	}

  
	bool multiIteratorStart(int id)
	{
		if (id<0 || id>=8196)
		{
			sprintf(temp,"Invalid multi num:%i\n", id);
			LogError(temp);
			currIterPos=0;
			currIterEnd=0;
			currIterId=0;
			return false;
		}

		if ( theMultis[id].count==0 || theMultis[id].multiParts==NULL)
		{
			sprintf(temp,"Empty multi num:%i\n", id);
			LogError(temp);
			currIterPos=0;
			currIterEnd=0;
			currIterId=0;
			return false;
		}

		currIterId  = id;
		currIterPos = 0;
		currIterEnd = theMultis[id].count;

		return true;
	}

    bool multiIteratorNext(singleMultiPart &mData)
	{
		assert(currIterId < 8196 && currIterId>=0);
		assert(theMultis[currIterId].multiParts!=NULL);

		if (currIterPos<currIterEnd && currIterEnd>0)
		{
		   mData = theMultis[currIterId].multiParts[currIterPos]; // use default (compiler provided) copy constructor which copies all member variables

		   assert(mData.tile==theMultis[currIterId].multiParts[currIterPos].tile && mData.x==theMultis[currIterId].multiParts[currIterPos].x);
		   assert(mData.y==theMultis[currIterId].multiParts[currIterPos].y && mData.z==theMultis[currIterId].multiParts[currIterPos].z);
		   assert(mData.visibility==theMultis[currIterId].multiParts[currIterPos].visibility);

		   currIterPos++;
		   return true;

		} else return false;
	}

};

#endif

