/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

#include "coord.h"
#include "itemid.h"
#include "LwMisc.h"
#include "mapS.h"
#include "mapstuff.h"
#include "regions.h"
#include "worldmain.h"

#include "wolfpack.h"

#include "debug.h"
#undef DBGFILE
#define DBGFILE "qsf.cpp"

union uQsfid
{
	struct qsfbits
	{
		//Byte 1:
		int mapnum:3;
		int blocknum:5;	// 1024er block# 0-23 x0-x18
		
		//Byte 2:
		int unused1:3;
		int CompleteBlock:1;	// indicator (if set, next 4 bits are zero)
		int subblock:4;			// 256x256 
		
		//Byte 3:
		bool Every:1;
		bool Doors:1;
		bool Signs:1;
		bool Deco:1;
		bool AllSpawns:1;
		bool VendorSpawns:1;
		bool HarmlessSpawns:1;	// (eg. item spawns)
		bool MonsterSpawns:1;
		
		//Byte 4:
		int version:8;
	} bits;
	int id;
};

static char tmsg[200] ;
void readqline(FILE* fp) // Read a line from a .qsf file
{
	int i, valid=0;
	char c;
	temp[0]=0;
	while (!valid)
	{
		i=0;
		if (feof(fp)) return;
		c=(char)fgetc(fp);
		while (c!=10)
		{
			if (c!=13)
			{
				temp[i]=c;
				i++;
			}
			if (feof(fp) || i>1024) return;
			c=(char)fgetc(fp);
		}
		temp[i]=0;
		valid=1;
		if (temp[0]=='/' && temp[1]=='/') valid=0;
		if (temp[0]=='{') valid=0;
		if (temp[0]==0) valid=0;
		if (temp[0]==10) valid=0;
		if (temp[0]==13) valid=0;
	}
}

void readq2(FILE* fp)
{
	int i=0;
	
	readqline(fp);
	script1[0]=0;
	script2[0]=0;
	script3[0]=0;
	while(temp[i]!=0 && temp[i]!=' ' && temp[i]!='=' && i<1024) i++;
	strncpy(script1, temp, i);
	script1[i]=0;
	if (script1[0]!='}' && temp[i]!=0) strcpy(script2, (char*)(temp+i+1));
	return;
}

void readq3(FILE* fp)
{
	int i=0,j;
	
	readqline(fp);
	script1[0]=0;
	script2[0]=0;
	script3[0]=0;
	while(temp[i]!=0 && temp[i]!=' ' && temp[i]!='=' && i<1024) i++;
	strncpy(script1,(char*) temp, i);
	script1[i]=0;
	if (script1[0]=='}' || temp[i]==0) return;
	i++;
	j=i;
	while(temp[i]!=0 && temp[i]!=' ' && temp[i]!='=' && i<1024) i++;
	strncpy(script2, (char*)(temp+j), i-j);
	script2[i-j]=0;
	strcpy((char*)script3, (char*)(temp+i+1));
}

#define save_txt(txt)	sp+=sprintf(sp,"%s\n",txt)

void ExportItem(FILE *fp, PC_ITEM pi, P_ITEM Default)
{
	char sectbuf[4000];
	char* sp=sectbuf;
	
	save_txt("SECTION ITEM");
	save_txt("{");
	
	cwmWorldState->SaveTags(pi, Default, sp);
	
	save_txt("}\n");

	fputs(sectbuf,fp);	// write the complete section to file
}

void LoadItem(FILE *fp,char *fn,int qsfid)
{
	const P_ITEM pi=Items->MakeBaseItem();
	if (!pi) return;

	int bad = 0;

	int loopexit=0;
	do
	{
		readq2(fp);
		if (!strcmp(script1,"}")) continue;

		int rc = Items->ApplyCommonTags(pi);				// tags used in both scp and wsc
		if (rc) {if (rc==1) continue; else bad=1;}

		rc = Items->ApplyWscTags(pi);						// tags used only in wsc
		if (rc) {if (rc==1) continue; else bad=1;}

		sprintf(tmsg,"invalid keyword <%s> detected in file <%s>. Ignored!",script2,fn);
		LogWarning(tmsg);
	}
	while (strcmp(script1, "}") && ++loopexit<MAXLOOPS);

	pi->qsf=qsfid;

	if (bad)
		Items->DeleItem(pi);
	else
	{
		if (pi->isInWorld())
		{
			int MapNum = 0; // of course, this needs to replaced with real map num when multi maps implemented
			int max_x = AllMaps->GetMapWidth(MapNum)  * 8;
			int max_y = AllMaps->GetMapHeight(MapNum) * 8;
			
			//printf("maxx:%i maxy:%i\n",max_x, max_y);
			
			mapRegions->Add(pi); // it reurns 1 if invalid, if invalid it DOESNT get added !!!
			if (pi->pos.x>=max_x || pi->pos.y>=max_y)	// lord bianry
			{
				sprintf(temp, "INVALID COORDS3 !!!! maxX:%i maxY:%i x:%i y:%i name:%s", max_x, max_y, pi->pos.x, pi->pos.y, pi->name);
				LogError(temp);
				Items->DeleItem(pi);	//these are invalid locations, delete them!
			}
			pi->refresh();
		}
	}
}

void LoadQitem(FILE *fp,char *fn,int qsfid)
{
	int item_scp_no=0;
	cCoord pos;
	int loopexit=0;
	do
	{
		readq2(fp);
		if		(!strcmp(script1,"}")) continue;
		else if	(!strcmp(script1,"ITEM_SCP")) item_scp_no=str2num(script2);
		else if (!strcmp(script1,"MAP")) pos.map=str2num(script2);
		else if (!strcmp(script1,"X")) pos.x=str2num(script2);
		else if (!strcmp(script1,"Y")) pos.y=str2num(script2);
		else if (!strcmp(script1,"Z")) pos.z=str2num(script2);
		else
		{
			sprintf(tmsg,"invalid keyword <%s> detected in file <%s>. Ignored!",script2,fn);
			LogWarning(tmsg);
		}
	}
	while (strcmp(script1, "}") && ++loopexit<1024);

	if (item_scp_no > 0 && pos.x > 0 && pos.y > 0)	// seems to be a valid entry :)
	{
		P_ITEM pi=Items->CreateFromScript(-1,item_scp_no);
		if (pi)
		{
			pi->MoveTo2(pos);
			pi->qsf=qsfid;
		}
	}
}

void LoadQnpc(FILE *fp,char *fn,int qsfid)
{
	int npc_scp_no=0;
	cCoord pos;
	int loopexit=0;
	do
	{
		readq2(fp);
		if		(!strcmp(script1,"}")) continue;
		else if	(!strcmp(script1,"NPC_SCP")) npc_scp_no=str2num(script2);
		else if (!strcmp(script1,"MAP")) pos.map=str2num(script2);
		else if (!strcmp(script1,"X")) pos.x=str2num(script2);
		else if (!strcmp(script1,"Y")) pos.y=str2num(script2);
		else if (!strcmp(script1,"Z")) pos.z=str2num(script2);
		else
		{
			sprintf(tmsg,"invalid keyword <%s> detected in file <%s>. Ignored!",script2,fn);
			LogWarning(tmsg);
		}
	}
	while (strcmp(script1, "}") && ++loopexit<MAXLOOPS);

	if (npc_scp_no > 0 && pos.x > 0 && pos.y > 0)	// seems to be a valid entry :)
	{
		pos.z = Map->MapElevation(pos);
		P_CHAR pc = Npcs->AddNPC(-1,npc_scp_no,pos);
		if (pc)
			pc->qsf = qsfid;
	}
}

int GetQsfidFromFilename(char *fn)
{
	int qsfid=-1;
	int len = strlen(fn);
	if (len==8 || (len>8 && (fn[8]=='.' || fn[8]=='_')))
	{
		int i;
		for (i=0;i<8;i++)
		{
			if (!isxdigit(fn[i]))
				break;
		}
		if (i==8)
			sscanf(fn,"%x",&qsfid);
	}
	return qsfid;
}

void qsfLoad(char *fn, short depth) // Load a quest script file
{
	if (depth>9)
	{
		LogWarning("nesting level has reached 10! Probably recursive includes! include aborted!");
		return;
	}

	int qsfid=GetQsfidFromFilename(fn);
	if (qsfid==-1)
	{
		LogWarningVar("Filename <%s> invalid for QSF. Skipped.",fn);
		return;
	}

	FILE *fp=fopen(fn, "r");
	if (fp==NULL)
	{
		sprintf(tmsg,"<%s> not found...nothing to load",fn);
		LogWarning(tmsg);
		return;
	}
	else
	{
		sprintf(tmsg,"including <%s>",fn);
		LogMessage(tmsg);
	}

	int loopexit=0;
	do
	{
		readq2(fp);
		if (!strcmp(script1, "SECTION"))
		{
			char *p = strchr(script2,' ');
			if (p) *p=0;	// truncate after section id to support punt's toolset
			if		(!strcmp(script2, "ITEM"))	LoadItem(fp,fn,qsfid);
			else if	(!strcmp(script2, "QITEM")) LoadQitem(fp,fn,qsfid);
			else if	(!strcmp(script2, "QNPC"))	LoadQnpc(fp,fn,qsfid);
			else
			{
				sprintf(tmsg,"invalid keyword <%s> detected in file <%s>",script2,fn);
				LogWarning(tmsg);
			}
		}
		else
			if (!strcmp(script1, "#include"))
				qsfLoad(script2,depth+1);
	}
	while (strcmp(script1, "EOF") && !feof(fp) && ++loopexit<MAXLOOPS);
	fclose(fp);
}

void qsfRemove(char *fn, short depth) // Remove a quest script file (simple version)
{
	int qsfid=GetQsfidFromFilename(fn);
	if (qsfid==-1)
	{
		LogWarningVar("Filename <%s> invalid for QSF. Skipped.",fn);
		return;
	}

	P_CHAR pc;
	AllCharsIterator aci;
	for (aci.Begin(); (pc=aci.GetData()) != NULL; aci++)
	{
		if (pc->qsf==qsfid)
			Npcs->DeleteChar(pc);
	}

	PC_ITEM pi;
	AllItemsIterator aii;
	for (aii.Begin(); (pi = aii.GetData()) != aii.End(); aii++)
	{
		if (pi->qsf==qsfid)
			Items->DeleItem(pi->unconst());
	}
}

bool IsSpawner(PC_ITEM pi)
{
	return(pi->isType(ITY_ItemSpawner)
		|| pi->isType(ITY_NpcSpawner)
		|| pi->isType(ITY_SpawnChest)
		|| pi->isType(ITY_LockedSpawnChest)
		|| pi->isType(ITY_NpcAreaSpawner)
		|| pi->isType(ITY_QuestSpawn) );
}

void qsfExport(char *fn) // Export to quest script file (simple version)
{
	uQsfid qid;
	qid.id = GetQsfidFromFilename(fn);
	if (qid.id==-1)
	{
		LogWarningVar("Filename <%s> invalid for QSF. Skipped.",fn);
		return;
	}

	FILE *fp=fopen(fn, "w");
	if (fp==NULL)
	{
		sprintf(tmsg,"<%s> not found...nothing to load",fn);
		LogWarning(tmsg);
		return;
	}

//	short bn = qid.bits.blocknum;
	short bn = (qid.id>>24) & 0x1F;		// isolate block #
	short baseX = (bn/4)*1024; 
	short baseY = (bn%4)*1024;
	short side = 1023;
	if (!((qid.id>>16) & 0x10))			// not the complete block
	{
		short sbn = (qid.id>>16) & 0x000F;// isolate subblock #
		short subX = (sbn/4)*256; 
		short subY = (sbn%4)*256;
		baseX += subX;
		baseY += subY;
		side = 255;
	}
	cRectangle Block(baseX,baseY,baseX+side,baseY+side);

	int sel = (qid.id & 0x0000FF00) >> 8;	// isolate selection bits
	P_ITEM Default = new cItem;	// create an item with the default values
	Default->Init(0);

	PC_ITEM pi;
	AllItemsIterator aii;
	for (aii.Begin(); (pi = aii.GetData()) != aii.End(); aii++)
	{
		if (!pi->isInWorld()) continue;
		if (!pi->pos.IsIn(Block)) continue;

		if ( (sel&0x80)		// every item
			|| ((sel&0x40) && IsDoor(pi->id()))
			|| ((sel&0x20) && IsSign(pi->id()))
			|| ((sel&0x08) && IsSpawner(pi)) )
		{
			ExportItem(fp, pi, Default);
		}
	}
	fclose(fp);
}

short qsfWhere(cCoord pos)
{
	short bx = pos.x/1024, by = pos.y/1024;
	short sx = pos.x - bx*1024, sy = pos.y - by*1024;	// relative x/y inside the block
	short sbx = sx/256, sby = sy/256;
	return (((bx*4) + by)<<8) + (sbx*4)+sby + (pos.map << 13);
}

