/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

// -- Region class  Added by Tauriel 3/6/1999 To lookup items by region
//									This should help for now. BTW- my first attempt at C++
//									So forgive any newbie mistakes :)
//									-- Side note, I wanted regions to be more generic, but
//									now that I have to do this, time doesn't allow for it yet.

#include "regions.h"

#include "wolfpack.h"


#include "debug.h"
#undef  DBGFILE
#define DBGFILE "regions.cpp"

const int initialSize = 5;
cRegion::cRegion() //constructor
{
	int i=0;
	GridSize=32;
	ColSize=128;

	// allow for deferred allocation (Duke, 18.12.01)
	for (i=0;i<33000;i++)
	{
		MapCells4Items[i].pointer=NULL;
		MapCells4Items[i].max=0;
	}
	for (i=0;i<33000;i++)
	{
		MapCells4Chars[i].pointer=NULL;
		MapCells4Chars[i].max=0;
	}
}

cRegion::~cRegion() //destructor
{
	for (int i = 0; i < 33000; ++i)
	{
		delete [] MapCells4Items[i].pointer;
		delete [] MapCells4Chars[i].pointer;
	}
}

void cRegion::Add(P_ITEM pi)
{
	if (pi)
	{
		unsigned int uiCell = GetCell(pi->pos.x,pi->pos.y);
	    if (uiCell > 32999)
			return;
		setptr(&MapCells4Items[uiCell], pi); //set item in pointer array
	}
}

void cRegion::Add(P_CHAR pc)
{
	int c = pc->GetIndex();
	if (c != -1)
	{
		unsigned int uiCell = GetCell(pc->pos.x,pc->pos.y);
	    if (uiCell > 32999)
			return;
		setptr(&MapCells4Chars[uiCell], c+CharacterOffset); //set item in pointer array
	}
}

void cRegion::Remove(PC_ITEM pi)
{
	if (pi)
	{
		unsigned int uiCell = GetCell(pi->pos.x,pi->pos.y);
	    if (uiCell > 32999)
			return;
		removefromptr(&MapCells4Items[uiCell], pi);
	}
}

void cRegion::Remove(P_CHAR pc)
{
	int c = pc->GetIndex();
	if (c != -1)
	{
		unsigned int uiCell = GetCell(pc->pos.x,pc->pos.y);
	    if (uiCell > 32999)
			return;
		removefromptr(&MapCells4Chars[uiCell], c+CharacterOffset);
	}
}


//- Returns the cell the character/item is in
unsigned int cRegion::GetCell(unsigned int x, unsigned int y)
{
	int cell = myGridx(x) + myGridy(y) + (myGridx(x) * (ColSize-1));
	return (unsigned int) ((cell<0) ? 0 : cell);  // - Return 0 if negative otherwise cell #
}

// - Get the next item pointer in a cell return -1 if done
long cRegion::GetNextItem(unsigned int cell, unsigned int Last)
{
	lookuptr_st mapcell;
	unsigned int i,max;

	if (Last==(unsigned int)-1) Last=0;else Last++;
	if (cell>32998 ) return -1;
	mapcell=MapCells4Items[cell];
	if (mapcell.pointer==NULL) return -1;
	max=mapcell.max;

	for (i=Last;i<max;i++)
	{
		if (mapcell.pointer[i] != -1) return i;
	}
	return -1; //didn't find another one
}

long cRegion::GetNextChar(unsigned int cell, unsigned int Last)
{
	lookuptr_st mapcell;
	unsigned int i,max;

	if (Last==(unsigned)-1) Last=0; else Last++;
	if (cell>32998 ) return -1;
	mapcell=MapCells4Chars[cell];
	if (mapcell.pointer==NULL) return -1;
	max=mapcell.max;

	for (i=Last;i<max;i++)
	{
		if (mapcell.pointer[i] != -1) return i;
	}
	return -1; //didn't find another one
}

// - Get an item in a cell return -1 if done
long cRegion::GetItem(unsigned int cell, unsigned int item)
{
	if (cell>32999 ) return -1;
	if (item>(unsigned)MapCells4Items[cell].max ) return -1;
	return MapCells4Items[cell].pointer[item];
}

long cRegion::GetChar(unsigned int cell, unsigned int item)
{
	if (cell>32999 ) return -1;
	if (item>(unsigned)MapCells4Chars[cell].max ) return -1;
	return MapCells4Chars[cell].pointer[item];
}



RegionIterator::RegionIterator(const cCoord& pos, RegionIteratorDomain dm) :
	x(pos.x), y(pos.y), mapitemptr(-1), mapitem(-1), domain(dm)
{
	StartPos = pos;
	getcell = mapRegions->GetCell(x,y);
}

int RegionIterator::First()
{
	mapitemptr = mapitem = -1;
	return Next();
}

int RegionIterator::Next()
{
	if (domain == ItemsOnly)
		mapitemptr = mapRegions->GetNextItem(getcell, mapitemptr);
	else
		mapitemptr = mapRegions->GetNextChar(getcell, mapitemptr);
	if (mapitemptr==-1) 
		return -1;

	if (domain == ItemsOnly)
		mapitem = mapRegions->GetItem(getcell, mapitemptr);
	else
		mapitem = mapRegions->GetChar(getcell, mapitemptr);
	if (domain == ItemsOnly && !IsItem())
		return Next();
	if (domain == CharsOnly && !IsChar())
		return Next();
	return mapitem;
}

bool RegionIterator::End()
{
	return (mapitem == -1);// || mapitemptr == -1 );
}

bool RegionIterator::IsItem() const
{
	return mapitem >= 0 && mapitem < CharacterOffset;
}

bool RegionIterator::IsChar() const
{
	return mapitem >= CharacterOffset;
}

bool RegionIterator::IsMulti() const
{
	P_ITEM Imapmulti=MAKE_ITEM_REF(mapitem);
	if (!Imapmulti) return false;
	return Imapmulti->id() >= 0x4000;
}

////////////////////
// name:	RegCellIterator4Items
// history:	by Duke, 28.10.2000
// Purpose:	iterates through one cell of the region grid,
//			returning only items (as a pointer, not an index)
//
P_ITEM RegCellIterator4Items::First()
{
	mapitemptr = mapitem = -1;
	return Next();
}

P_ITEM RegCellIterator4Items::Next()
{
	mapitemptr = mapRegions->GetNextItem(getcell, mapitemptr);
	if (mapitemptr==-1) 
		return NULL;

	mapitem = mapRegions->GetItem(getcell, mapitemptr);
	if (domain == ItemsOnly && !IsItem())
		return Next();

	P_ITEM pi = MAKE_ITEM_REF(mapitem);
	if (!pi) return NULL;
	if (pi->pos.map != StartPos.map)
		return Next();
	return pi;
}

////////////////////
// name:	RegCellIterator4Chars
// history:	by Duke, 18.11.2002
// Purpose:	iterates through one cell of the region grid,
//			returning only chars (as a pointer, not an index)
//
P_CHAR RegCellIterator4Chars::First()
{
	mapitemptr = mapitem = -1;
	return Next();
}

P_CHAR RegCellIterator4Chars::Next()
{
	mapitemptr = mapRegions->GetNextChar(getcell, mapitemptr);
	if (mapitemptr==-1) 
		return NULL;

	mapitem = mapRegions->GetChar(getcell, mapitemptr);
	if (domain == CharsOnly && !IsChar())
		return Next();

	P_CHAR pc = MAKE_CHARREF_LRV(mapitem-CharacterOffset,NULL);
	if (pc->pos.map != StartPos.map)
		return Next();
	return pc;
}

////////////////////
// name:	RegGrid3x3Iterator4Items
// history:	by Duke, 28.10.2000
// Purpose:	iterates through one cell plus the eight surroundung cells of the region grid,
//			returning only items (as a pointer, not an index)
// Remark:	I know there is a problem with the edges & borders of the grid,
//			but as in the normal UO map there's only water there ....
//
RegGrid3x3Iterator4Items::RegGrid3x3Iterator4Items(cCoord pos) : RegionIterator(pos,ItemsOnly)
{
	GridColSize=128;
	orgCell=getcell;	// remember it
	if (orgCell >= GridColSize)
		getcell-=GridColSize;
	if (getcell >= 1) 
		getcell-=1;
}

P_ITEM RegGrid3x3Iterator4Items::NextCell()
{
	getcell++;	//adjacent cell
	if (getcell%GridColSize > orgCell%GridColSize+1)
		getcell+=GridColSize-3;	//next col
	if (getcell > orgCell+GridColSize+1)
		return NULL;	// upper right corner of the box reached
	return Next();
}

P_ITEM RegGrid3x3Iterator4Items::Next()
{
	mapitemptr = mapRegions->GetNextItem(getcell, mapitemptr);
	if (mapitemptr==-1) 
		return NextCell();

	mapitem = mapRegions->GetItem(getcell, mapitemptr);
	if (domain == ItemsOnly && !IsItem())
		return Next();

	P_ITEM pi = MAKE_ITEM_REF(mapitem);
	if (!pi) return NULL;
	if (pi->pos.map != StartPos.map)
		return Next();
	return pi;
}

////////////////////
// name:	RegGrid3x3Iterator4Chars
// history:	by Duke, 18.11.2000
// Purpose:	iterates through one cell plus the eight surroundung cells of the region grid,
//			returning only items (as a pointer, not an index)
// Remark:	I know there is a problem with the edges & borders of the grid,
//			but as in the normal UO map there's only water there ....
//
RegGrid3x3Iterator4Chars::RegGrid3x3Iterator4Chars(cCoord pos) : RegionIterator(pos,CharsOnly)
{
	GridColSize=128;
	orgCell=getcell;	// remember it
	if (orgCell >= GridColSize)
		getcell-=GridColSize;
	if (getcell >= 1) 
		getcell-=1;
}

P_CHAR RegGrid3x3Iterator4Chars::NextCell()
{
	getcell++;	//adjacent cell
	if (getcell%GridColSize > orgCell%GridColSize+1)
		getcell+=GridColSize-3;	//next col
	if (getcell > orgCell+GridColSize+1)
		return NULL;	// upper right corner of the box reached
	return Next();
}

P_CHAR RegGrid3x3Iterator4Chars::Next()
{
	mapitemptr = mapRegions->GetNextChar(getcell, mapitemptr);
	if (mapitemptr==-1) 
		return NextCell();

	mapitem = mapRegions->GetChar(getcell, mapitemptr);
	if (domain == CharsOnly && !IsChar())
		return Next();

	P_CHAR pc = MAKE_CHARREF_LRV(mapitem-CharacterOffset,NULL);
	if (pc->pos.map != StartPos.map)
		return Next();
	return pc;
}
