/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

/*
 Cache script section locations by <erwin@andreasen.com>
 Reads through the contents of the file and saves location of each
 SECTION XYZ entry

 Calling Script::find() will then seek to that location directly rather
 than having to parse through all of the script
 */

#include "scriptc.h"

#include "rectangle.h"
#include "scripts.h"

#include <fstream>
#include <iostream>
#include <string>

using namespace std;
#include <sys/stat.h>
#include <errno.h>

#include "mstring.h"
#include "wolfpack.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "scriptc.cpp"

extern FILE* scpfile;

char get_modification_date(const char *filename, time_t* mod_time) 
{
    struct stat stat_buf;
    
    if ((stat(filename, &stat_buf)))
	{
		char tmperr[1000];
        sprintf(tmperr, "Cannot stat %s: %s", filename, strerror(errno));
		LogWarning(tmperr);
        fprintf(stderr, tmperr);
        exit(1);
    }

    *mod_time = stat_buf.st_mtime;
    return 1;
}

bool Script::preload()	// load the complete script into memory (Duke,6.1.2001)
{
    FILE *fp;
    struct stat stat_buf;
    
    if (stat(filename, &stat_buf))
        return false;

    scpsize = stat_buf.st_size;

	if (flags & SCP_PRELOADED)
		delete img;
	img = new char[scpsize];
	curmempos=img;
    if (!(fp = fopen(filename, "rb")))
	{
        fprintf(stderr, "Cannot open %s: %s", filename, strerror(errno));
        exit(1);
	}
	int r=fread(img, sizeof( char ), scpsize, fp);	// read in the whole file
	if (r!=scpsize)
	{
		LogErrorVar("problem preloading %s",filename);
	}
	else
		flags |= SCP_PRELOADED;
	fclose(fp);
    return true;

}

void Script::reload()
{
	if (flags & SCP_PRELOADABLE)
	{
		preload();
		MakeIndexForMem();
	}
	else
		MakeIndexForFile();
}

#if 1
void Script::MakeIndexForFile()
{
	FILE *fp;
    char buf[1024], section_name[256];
//    int count = 0;
    
    entries.clear();
    
    if (!(fp = fopen(filename, "r")))
	{
        fprintf(stderr, "Cannot open %s: %s", filename, strerror(errno));
        exit(1);
    }
    Console.send("Reloading %-15s: ", filename); fflush (stdout);

    // Snarf the part of SECTION... until EOL
	int loopexit=0;
    while( (fgets(buf, sizeof(buf), fp)) && (++loopexit < MAXLOOPS*3) )	// up to 200,000 lines
	{
        if (sscanf(buf, "SECTION %256[^\n]", section_name) == 1)
		{
			
			if (section_name[0] == 'x')
			{
				std::string sScript1 = section_name, sToken1;
				int sEnd = sScript1.rfind("x", 0), lastpos;
				int loopexit2=0;
				while (sEnd != -1 && (++loopexit2<MAXLOOPS))
				{
					lastpos = sEnd;
					sEnd = sScript1.find("x", sEnd+1);
					sToken1 = sScript1.substr(lastpos, sEnd-lastpos);
					ScriptEntry section;
					section.name = sToken1;
					section.offset = ftell(fp);
					entries.insert(make_pair(section.name, section));
				}
			} else
			{
				ScriptEntry section;
				section.name = section_name;
				section.offset =  ftell(fp);
				entries.insert(make_pair(section.name, section));
			}
        }
	}

    Console.send("%6d sections found.\n", entries.size());
    
    fclose(fp);
}

#elif
void Script::MakeIndexForFile()
{
	fstream myscpfile;
	mstring sInput;
	entries.clear();
	vector<mstring>	vecLine ;
	myscpfile.open(filename, ios::in);
	if (myscpfile.is_open())
	{
		clConsole.send("Reloading %-15s: ", filename); fflush (stdout);
		int loopexit=0;
		while(getline(myscpfile,sInput)&& ++loopexit<MAXLOOPS*20) 
		{
			sInput.trim();
			// Ok, unfortunately, as we have no standard syntax for our SECTIONS, we just want the 
			// the first string
			mstring sFirst;
			SI32 siEnd = 0;
			siEnd = sInput.find(" ");
			if ((siEnd != string::npos)&&(siEnd!=0))
			{
				sFirst = sInput.substr(0,siEnd);
				sInput = sInput.substr(siEnd+1);
			}
			else
			{
				// No first word
				sFirst = sInput;
				sInput.erase();
			}
			if (!sFirst.empty())
			{

				if (sFirst == "SECTION")
				{
					// We are in a section, need to parse the section name
					
					
					if (sInput.find("x")==0)
					{
						vector<mstring> vecEntries ;
						mstring sEntries ;
						// strip off the leading x
						//sEntries = sInput.substr(1) ;
						vecEntries = sEntries.split("x") ;
						int i;
						for (i = 0; i< vecEntries.size(); i++)
						{
							
							ScriptEntry section;
							section.name = vecEntries[i] ;
							section.offset = myscpfile.tellg() ;
							entries.insert(make_pair(section.name,section)) ;
						}
						vecEntries.clear() ;
					}
					else
					{
						ScriptEntry section;
						section.name = sInput;
						section.offset = myscpfile.tellg();
						entries.insert(make_pair(section.name,section)) ;
					}
					vecLine.clear() ;
				}
			}
		}
		clConsole.send("%6d sections found.\n", entries.size()) ;
		myscpfile.close() ;
	}
	else
	{
		cerr<< "Cannot open " << filename << ": " << strerror(errno) <<endl;
		exit(1) ;
	}



/*
	FILE *fp;
    char buf[1024], section_name[256];
    int count = 0;
    
    entries.clear();
    
    if (!(fp = fopen(filename, "r")))
	{
        fprintf(stderr, "Cannot open %s: %s", filename, strerror(errno));
        exit(1);
    }
    clConsole.send("Reloading %-15s: ", filename); fflush (stdout);

    // Snarf the part of SECTION... until EOL
	int loopexit=0;
    while( (fgets(buf, sizeof(buf), fp)) && (++loopexit < MAXLOOPS) )
        if (sscanf(buf, "SECTION %256[^\n]", section_name) == 1)
		{

			if (section_name[0] == 'x')
			{
				mstring sScript1(section_name);
				sScript1.trim();
				vector<mstring> Tokens = sScript1.split("x");
				int i;
				for (i = 0; i < Tokens.size(); i++)
				{
					ScriptEntry section;
					section.name = Tokens[i];
					section.offset = ftell(fp);
					entries.insert(make_pair(section.name, section));
				}
			} else {
				ScriptEntry section;
				section.name = section_name;
				section.offset =  ftell(fp);
				entries.insert(make_pair(section.name, section));
			}
        }

    clConsole.send("%6d sections found.\n", entries.size());
    
    fclose(fp);
*/
}
#endif

void Script::MakeIndexForMem()
{
    char section_name[256];
    
    entries.clear();   // we need to figure out why this was crashing on Mingw32

    if (flags & SCP_PRELOADED)
	{
		int loopexit=0;
		while( NextLine() && (++loopexit < MAXLOOPS) )
		{
			if (sscanf((char*)temp, "SECTION %256[^\n]", section_name) == 1)
			{
				if (section_name[0] == 'x')
				{
					mstring sScript1(section_name);
					vector<mstring> Tokens = sScript1.split("x");
					unsigned int i;
					for (i = 0; i < Tokens.size(); i++)
					{
						ScriptEntry section;
						section.name = Tokens[i];
						section.offset = (unsigned long) curmempos;
						entries.insert(make_pair(section.name, section));
					}
				} 
				else 
				{
					ScriptEntry section;
					section.name = section_name;
					section.offset =  (unsigned long) curmempos;
					entries.insert(make_pair(section.name, section));
				}
			}
		}
	}
}

// Parse this script, caching section positions
Script::Script(const char *_filename, short _flags)
{
	flags = _flags;
	filename = NULL;
    filename = strdup(_filename);
    
    get_modification_date(filename, &last_modification);

    reload();
}

Script::~Script()
{
	if ( filename != NULL )
		delete filename;
}

// Look for that section in this previously parsed script file
bool Script::find(const char *secname, const int secID)
{
	char sect[512];
	sprintf(sect, "%s %i", secname, secID);
	return this->find(sect);
}

// Look for that section in this previously parsed script file
bool Script::find(const char *section)
{
    time_t current;
    
    get_modification_date(filename, &current);

    if (current > last_modification)
	{
        reload();
        last_modification = current;
    }
	std::string dummy = section;
	std::map<std::string, ScriptEntry>::iterator iter_entry = entries.find(dummy);

    //lp = iter_entry->second.offset; // save last position, for debugging only

    if (iter_entry == entries.end())
        return 0;

	if (flags & SCP_PRELOADED)
		curmempos=(char*)(iter_entry->second.offset);
	else
		fseek(scpfile, iter_entry->second.offset, SEEK_SET);

    return 1;
}

bool Script::Open()
{
	if (flags & SCP_PRELOADED)
	{
		curmempos=img;
	}
	else
		openscript(filename);
	return true;
}

void Script::Close()
{
	if (!(flags & SCP_PRELOADED))
		closescript();
}

Script* Script::Select(char* sect, short custom)
{
	Script * pScp;
	this->Open();
	if (this->find(sect))
		return this;

	this->Close();
	if (n_scripts[custom][0]!=0)	// do we have a name for the alternate script ?
	{
		pScp=i_scripts[custom];
		pScp->Open();
		if (pScp->find(sect))
			return pScp;
		
		pScp->Close();
	}
	return NULL;
}

scpMark Script::Suspend()
{
	scpMark pos;
	if (flags & SCP_PRELOADED)
		pos=(scpMark)curmempos;
	else
		pos=ftell(scpfile);
	this->Close();
	return pos;
}

void Script::Resume(scpMark mark)
{
	this->Open();
	if (flags & SCP_PRELOADED)
		curmempos=(char *)mark;
	else
		fseek(scpfile, mark, SEEK_SET);
}

bool Script::ReadMemLine()	// 
{
	char *p=curmempos;
	int i=0, valid=0;

	int loopexit=0;
	int loopexit2=0;
	while(!valid && (++loopexit<MAXLOOPS*17))
	{
		i=0;
		while (p<img+scpsize && (++loopexit2<MAXLOOPS*17))	// virtual EOF
		{
			if (*p==0x0A || *p==0x0D)
			{
				temp[i]=0;
				p++;
				break;
			}
			else
			{
				temp[i]=*p;
				p++;i++;
			}
		}
		curmempos=p;
		valid=1;
		if (temp[0]=='/' && temp[1]=='/') valid=0;
		if (temp[0]=='{') valid=0;
		if (temp[0]==0) valid=0;

		if (p>=img+scpsize && !valid)	// virtual EOF
			return false;
	}
	return true;
}

bool Script::NextLineInSection()
{
	if (!this->NextLine())
		return false;
	if (EndSect())
		return false;
	return true;
}

bool Script::NextLine()
{
	if (flags & SCP_PRELOADED)
	{
		if (!ReadMemLine())
			return false;
		strcpy(script1,(char*)temp);
	}
	else
		read1();

	return true;
}

bool Script::NextLineSplitted()
{
	if (flags & SCP_PRELOADED)
	{
		ReadMemLine();
		int i=0;
		script1[0]=0;
		script2[0]=0;
	
		while(temp[i]!=0 && temp[i]!=' ' && temp[i]!='=' && i<1024 )
		{
			i++;
		}
	
		strncpy(script1,temp, i);
		script1[i]=0;
		if (!EndSect() && temp[i]!=0)
			strcpy(script2, (temp+i+1));
	}
	else
		read2();

	if (EndSect())
		return false;
	return true;
}


// Look for that section in this previously parsed script file
char Script::isin(const char *section)
{
    time_t current;
    
    get_modification_date(filename, &current);

    if (current > last_modification)
	{
        reload();
        last_modification = current;
    }

	std::map<std::string, ScriptEntry>::iterator iter_entry = entries.find(section);

	if (iter_entry == entries.end())
		return 0;
	else
	{
		fseek(scpfile, iter_entry->second.offset, SEEK_SET);
		return 1;
	}
}

/////////////////
// name:	getKeyVal
// purpose:	if the current line has the given key, read the values
// history:	by Duke, 24.03.2002
//			added support for int values. Duke, 6.01.2003
//
bool Script::getKeyVal(const char* key, int* num)
{
	if (strcmp(script1,key))
		return false;

	sscanf(script2,"%i",num);
	return true;
}
bool Script::getKeyVal(const char* key, cCoord** pos)
{
	if (strcmp(script1,key))
		return false;

	int x,y,z;
	if (sscanf(script2,"%i %i %i",&x,&y,&z) != 3)
		LogWarning("Invalid position given");
	else
		*pos = new cCoord(x,y,z);

	return true;
}
bool Script::getKeyVal(const char* key, cRectangle** rect)
{
	if (strcmp(script1,key))
		return false;

	int x1,y1,x2,y2;
	if (sscanf(script2,"%i %i %i %i",&x1,&y1,&x2,&y2) != 4)
		LogWarning("Invalid rectangle given");
	else
		*rect = new cRectangle(x1,y1,x2,y2);

	return true;
}
bool Script::getKeyVal(const char* key, cMapRectangle* rect)
{
	if (strcmp(script1,key))
		return false;

	int x1,y1,x2,y2,map=0;
	int p = sscanf(script2,"%i %i %i %i %i",&x1,&y1,&x2,&y2,&map);
	if (p<4 || p>5)
		LogWarning("Invalid rectangle given");
	else
		rect->init(x1,y1,x2,y2,map);

	return true;
}
bool Script::getKeyVal(const char* key, char* name)
{
	if (strcmp(script1,key))
		return false;

	sscanf(script2,"%s", name);
	return true;
}
bool Script::getKeyVal(const char* key, short* Value)
{
	if (strcmp(script1,key))
		return false;

	sscanf(script2,"%hi",Value);
	return true;
}
/////////////////
// name:	findSection
// purpose:	a *free* constructor for Script, positioning to the given section
// history:	by Duke, 25.03.2002
//
Script* findSection(enScripts scpID, char* sectName, int sectID)
{
	Script *Scp=i_scripts[scpID];
	if (!Scp) return NULL;
	if (!Scp->Open()) return NULL;
	if (!Scp->find(sectName, sectID))
	{
		Scp->Close();
		return NULL;
	}
	return Scp;
}

Script* findSection(enScripts scpID, char* sectName)
{
	Script *Scp=i_scripts[scpID];
	if (!Scp) return NULL;
	if (!Scp->Open()) return NULL;
	if (!Scp->find(sectName))
	{
		Scp->Close();
		return NULL;
	}
	return Scp;
}
/////////////////
// name:	PickFromList
// purpose:	randomly picks a line from a list of numbers, weightened by the second numbers
// history:	by Duke, 03.01.2003
//
int Script::PickFromList()
{
	int i=0, loopexit=0;
	int iList[512];		//-- no more than 512 items in a single item list (changed by Magius(CHE))
	int draws[512];		// a multiplier for that entry (works as if they had repeated this line n times)
	int totaldraws=0;
	while ( this->NextLine() && i<512 && (++loopexit < MAXLOOPS) )  // -- count items storing item #'s in iList[]
	{
		if (this->EndSect()) break;
		draws[i]=-1;
		sscanf(script1,"%i %i",&iList[i],&draws[i]);
		if (draws[i] == -1) draws[i] = 1;
		totaldraws += draws[i];				// summarize all draw chances
		i++;
	}
	this->Close();
	if (i==0) 
		return -1;

	int picked = rand()%totaldraws;
	totaldraws=0;
	for (int j=0;j<i;j++)
	{
		totaldraws += draws[j];
		if (totaldraws > picked)
			return(iList[j]);
	}
	LogError("should not happen");
	return -1;
}

