/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//
//
//
//Header file for spells.cpp
//holds:	all spell classes

#if !defined(__SPELLS_H__)
#define __SPELLS_H__

#include "CArray.h"
#include "typedefs.h"

#include <vector>

class cCoord; //forward declaration


static const unsigned char   NO_TARGET = 0;
static const unsigned char CHAR_TARGET = 1;
static const unsigned char ITEM_TARGET = 2;
static const unsigned char	MAP_TARGET = 4;

class cReag
{
public:
	SecVec<char>  reagamount;//("reagamount");
	SecVec<short> reagid;//("reagid");
};

class cSpell
{
protected:
	unsigned char spellnum;		//number of the spell
	unsigned char mana[3];		// mana requirements
	unsigned char tithing[3];	// tithing points required
	unsigned char delay[3];		// spell delay
	unsigned short loskill[3];	// low magery skill req.
	unsigned short hiskill[3];	// high magery skill req.

	unsigned char targtype;

	short action;			// character action
	cReag reags;			// holds the all required reagents
	
	char mantra[25];	// words of power
	char targmsg[100];// string visualized with targ. system

	int trig[3];

	unsigned char flags;	// bit 0: disabled
							// bit 1: reflactable
							// bit 2: runic
							// bit 3: affect caster
							// bit 4: affect party

protected:
	bool IsEnabled();

	bool CheckAvailible		(P_CHAR Player, char type);	// checks if the spell is availible in the spellbook, ar as scroll/item
	bool CheckReags			(P_CHAR Player, char type);	// checks if player has all reagents he needs
	bool CheckMana			(P_CHAR Player, char type);	// checks if player has enough mana
	bool CheckTithing		(P_CHAR Player, char type);	// checks if player has enough mana
	bool CheckRegion		(P_CHAR Player, cCoord* targpos);		// checks if spell is allowed in players region
	bool CheckPlayersItems	(P_CHAR Player);						// checks if caster wields any restricted items

	void DeleteReags	(P_CHAR Player, char type);		// deletes all necessary reags
	void DeleteMana		(P_CHAR Player, char type);		// deletes required mana
	void DeleteTithing	(P_CHAR Player, char type);		// deletes required mana

public:
	cSpell(unsigned char num);
	virtual ~cSpell();

	cSpell& operator= (cSpell& b);

	unsigned char GetSpellnum() {return spellnum;};

	void SetMana   (unsigned char value, unsigned char type)  {mana[type]	= value;};
	void SetTithing(unsigned char value, unsigned char type)  {tithing[type]	= value;};
	void SetDelay  (unsigned char value, unsigned char type)  {delay[type]	= value;};
	void SetLoskill(unsigned short value, unsigned char type) {loskill[type]	= value;};
	void SetHiskill(unsigned short value, unsigned char type) {hiskill[type]	= value;};
	void SetTrigger(unsigned int value, unsigned char pos) {trig[pos]	= value;};

	unsigned char GetMana   (unsigned char type)  {return mana[type];};
	unsigned char GetTithing(unsigned char type)  {return tithing[type];};
	unsigned char GetDelay  (unsigned char type)  {return delay[type];};
	unsigned short GetLoskill(unsigned char type) {return loskill[type];};
	unsigned short GetHiskill(unsigned char type) {return hiskill[type];};
	unsigned int GetTrigger(unsigned char pos) {return trig[pos];};

	void SetAction (short value) {action = value;};
	void SetMantra (char* str) {strcpy(mantra,str);};
	void SetTargMsg(char* str) {strcpy(targmsg,str);};

	short GetAction () {return action;};
	char* GetMantra () {return mantra;};
	char* GetTargMsg() {return targmsg;};

	void SetDisabled    (bool b) {b ? flags |= 0x01 : flags &= 0xFE;};
	void SetReflectable (bool b) {b ? flags |= 0x02 : flags &= 0xFD;};
	void SetRunic       (bool b) {b ? flags |= 0x04 : flags &= 0xFB;};
	void SetAffectCaster(bool b) {b ? flags |= 0x08 : flags &= 0xF7;};
	void SetAffectParty (bool b) {b ? flags |= 0x10 : flags &= 0xEF;};

	bool IsDisabled   () {return (flags&0x01 ? true : false);};
	bool IsReflectable() {return (flags&0x02 ? true : false);};
	bool IsRunic      () {return (flags&0x04 ? true : false);};
	bool DoesAffectCaster() {return (flags&0x08 ? true : false);};
	bool DoesAffectParty () {return (flags&0x10 ? true : false);};

	void AddReag(char amount, short id);

	bool CheckRequirements(P_CHAR Player, cCoord* targpos, char type); //checks the given requirements
	void AccomplishRequirements(P_CHAR Player, char type, bool succeeded); //accomplish the given requirements
	char GetTargetType();	// what kind of target do we need?

	char CheckTrigger(UOXSOCKET s, P_CHAR Player, char pos);

	virtual void DoEffect(P_CHAR Caster);
	virtual void DoEffect(P_CHAR Caster, P_CHAR CharTarg);
	virtual void DoEffect(P_CHAR Caster, P_ITEM ItemTarg);
	virtual void DoEffect(P_CHAR Caster, cCoord* MapTarg);

	bool IsInBook (P_ITEM Book);
	void AddToBook(P_ITEM Book);
};

// LVL 1 SPELLS
class cClumsySpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster, P_CHAR CharTarg);
public:
	cClumsySpell();
};

class cCreateFoodSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cCreateFoodSpell();
};

class cFeeblemindSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster, P_CHAR CharTarg);
public:
	cFeeblemindSpell();
};

class cHealSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster, P_CHAR CharTarg);
public:
	cHealSpell();
};

class cMagicArrowSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cMagicArrowSpell();
};

class cNightsightSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cNightsightSpell();
};

class cReactiveArmorSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cReactiveArmorSpell();
};

class cWeakenSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster, P_CHAR CharTarg);
public:
	cWeakenSpell();
};


// LVL 2 SPELLS
class cAgilitySpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster, P_CHAR CharTarg);
public:
	cAgilitySpell();
};

class cCunningSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster, P_CHAR CharTarg);
public:
	cCunningSpell();
};

class cCureSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cCureSpell();
};

class cHarmSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cHarmSpell();
};

class cMagicTrapSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cMagicTrapSpell();
};

class cMagicUntrapSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cMagicUntrapSpell();
};

class cProtectionSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cProtectionSpell();
};

class cStrengthSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster, P_CHAR CharTarg);
public:
	cStrengthSpell();
};


// LVL 3 SPELLS
class cBlessSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster, P_CHAR CharTarg);
public:
	cBlessSpell();
};

class cFireballSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cFireballSpell();
};

class cMagicLockSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cMagicLockSpell();
};

class cPoisonSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cPoisonSpell();
};

class cTelekinesisSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cTelekinesisSpell();
};

class cTeleportSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cTeleportSpell();
};

class cUnlockSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cUnlockSpell();
};

class cWallOfStoneSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster, P_CHAR CharTarg);
	void DoEffect(P_CHAR Caster, P_ITEM ItemTarg);
	void DoEffect(P_CHAR Caster, cCoord* MapTarg);
public:
	cWallOfStoneSpell();
};


// LVL 4 SPELLS
class cArchCureSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cArchCureSpell();
};

class cArchProtectionSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cArchProtectionSpell();
};

class cCurseSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cCurseSpell();
};

class cFireFieldSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster, P_CHAR CharTarg);
	void DoEffect(P_CHAR Caster, P_ITEM ItemTarg);
	void DoEffect(P_CHAR Caster, cCoord* MapTarg);
public:
	cFireFieldSpell();
};

class cGreaterHealSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cGreaterHealSpell();
};

class cLightningSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cLightningSpell();
};

class cManaDrainSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cManaDrainSpell();
};

class cRecallSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cRecallSpell();
};


// LVL 5 SPELLS
class cBladeSpiritSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cBladeSpiritSpell();
};

class cDispelFieldSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cDispelFieldSpell();
};

class cIncognitoSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cIncognitoSpell();
};

class cMagicReflectSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cMagicReflectSpell();
};

class cMindBlastSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cMindBlastSpell();
};

class cParalyzeSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cParalyzeSpell();
};

class cPoisonFieldSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cPoisonFieldSpell();
};

class cSummonCreatureSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cSummonCreatureSpell();
};


// LVL 6 SPELLS
class cDispelSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cDispelSpell();
};

class cEnergyBoltSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cEnergyBoltSpell();
};

class cExplosionSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cExplosionSpell();
};

class cInvisibilitySpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cInvisibilitySpell();
};

class cMarkSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cMarkSpell();
};

class cMassCurseSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cMassCurseSpell();
};

class cParalyzeFieldSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cParalyzeFieldSpell();
};

class cRevealSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cRevealSpell();
};


// LVL 7 SPELLS
class cChainLightningSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cChainLightningSpell();
};

class cEnergyFieldSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cEnergyFieldSpell();
};

class cFlamestrikeSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cFlamestrikeSpell();
};

class cGateTravelSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cGateTravelSpell();
};

class cManaVampireSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cManaVampireSpell();
};

class cMassDispelSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cMassDispelSpell();
};

class cMeteorSwarmSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cMeteorSwarmSpell();
};

class cPolymorphSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cPolymorphSpell();
};


// LVL 8 SPELLS
class cEarthquakeSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cEarthquakeSpell();
};

class cEnergyVortexSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cEnergyVortexSpell();
};

class cResurrectionSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cResurrectionSpell();
};

class cAirElementalSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cAirElementalSpell();
};

class cEarthElementalSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cEarthElementalSpell();
};

class cWaterElementalSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cWaterElementalSpell();
};

class cFireElementalSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cFireElementalSpell();
};

class cSummonDaemonSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cSummonDaemonSpell();
};
// END REGULAR SPELLS

// START NECRO SPELLS
class cAnimateDeadSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cAnimateDeadSpell();
};

class cBloodOathSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cBloodOathSpell();
};

class cCorpseSkinSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cCorpseSkinSpell();
};

class cCurseWeaponSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cCurseWeaponSpell();
};

class cEvilOmenSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cEvilOmenSpell();
};

class cHorrificBeastSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cHorrificBeastSpell();
};

class cLichFormSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cLichFormSpell();
};

class cMindRotSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cMindRotSpell();
};

class cPainSpikeSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cPainSpikeSpell();
};

class cPoisonStrikeSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cPoisonStrikeSpell();
};

class cStrangleSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cStrangleSpell();
};

class cSummonFamiliarSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cSummonFamiliarSpell();
};

class cVampiricEmbraceSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cVampiricEmbraceSpell();
};

class cVengefulSpiritSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cVengefulSpiritSpell();
};

class cWitherSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cWitherSpell();
};

class cWraithFormSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cWraithFormSpell();
};
// END NECRO SPELLS

// START PALADIN SPELLS
class cCleanseByFireSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster, P_CHAR CharTarg);
public:
	cCleanseByFireSpell();
};

class cCloseWoundsSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster, P_CHAR CharTarg);
public:
	cCloseWoundsSpell();
};

class cConsecrateWeaponSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cConsecrateWeaponSpell();
};

class cDispelEvilSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cDispelEvilSpell();
};

class cDivineFurySpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cDivineFurySpell();
};

class cEnemyOfOneSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cEnemyOfOneSpell();
};

class cHolyLightSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cHolyLightSpell();
};

class cNobleSacrificeSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cNobleSacrificeSpell();
};

class cRemoveCurseSpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cRemoveCurseSpell();
};

class cSacredJourneySpell : public cSpell
{
protected:
	void DoEffect(P_CHAR Caster);
public:
	cSacredJourneySpell();
};
// END PALADIN SPELLS

#endif

/*
cAnimateDeadSpell
cBloodOathSpell
cCorpseSkinSpell
cCurseWeaponSpell
cEvilOmenSpell
cHorrificBeastSpell
cLichFormSpell
cMindRotSpell
cPainSpikeSpell
cPoisonStrikeSpell
cStrangleSpell
cSummonFamiliarSpell
cVampiricEmbraceSpell
cVengefulSpiritSpell
cWitherSpell
cWraithFormSpell
*/

/*
cCleanseByFireSpell
cCloseWoundsSpell
cConsecrateWeaponSpell
cDispelEvilSpell
cDivineFurySpell
cEnemyOfOneSpell
cHolyLightSpell
cNobleSacrificeSpell
cRemoveCurseSpell
cSacredJourneySpell
*/
