/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

#ifndef __SREGIONS_H__
#define __SREGIONS_H__

#include "platform.h"
#include "typedefs.h"
#include "coord.h"
#include "items.h"
#include "defines.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "sregions.h"

class region_st
{
public:
	
	char name[50];
	int midilist;
	
	char guardowner[50];
	unsigned char snowchance;
	unsigned char rainchance;
	int guardnum[10];
	int goodsell[256]; // Magius(CHE)
	int goodbuy[256]; // Magius(CHE)
	int goodrnd1[256]; // Magius(CHE) (2)
	int goodrnd2[256]; // Magius(CHE) (2)
	int dungeonlevel;
	int brightlevel;
	int darklevel;
	int fixedlight;
	int curlight;
	unsigned char dungeon;

	bool arena;			// if true: region is defined as an arena. no player can die here
	cCoord arenaexit;	// if the region is an arena, the "dead" player gets teleported to this coords healed there
	
	void ResetPriv(void)                 
	{ 
		priv=0; 
		priv2=0; 
		int a; 
		for (a=1;a<MAXSPELLS+1;++a) setSpellAllowed(a); 
	}
	
	inline void setGuarded(void)         { priv|=0x01; }
	inline void setRaining(void)         { priv|=0x10; }
	inline void setSnowing(void)         { priv|=0x20; }
	inline void setMagicDamage(void)     { priv|=0x40; }
	inline void setRuneBookAllowed(void) { priv2|=0x01;}
	
	inline void setSpellAllowed(int spellnum)
	{
		if (spellnum<=0) return; // spellnum must be greater null, non negative value

		unsigned char spellid = 1;
		unsigned char octnum  = 0;

		octnum = ((spellnum-1)/8);
		spellid <<= ((spellnum-1)%8); // calc the spell in binary position

		if (octnum >= ((MAXSPELLS)/8)) // stay in array range of spellsallowed[27]
		{
			char t[512];
			sprintf(t,"Invalid spell# in IsSpellallowed:%i\n",spellnum);
			LogError(t);
			return;			
		}

		spellsallowed[octnum] |= spellid;
	}
	
	inline bool isGuarded(void)         { return (priv&0x01 ? true:false);}
	inline bool isRainingOn(void)       { return (priv&0x10 ? true:false);}
	inline bool isSnowingOn(void)       { return (priv&0x20 ? true:false);}
	inline bool isMagicDamageOn(void)   { return (priv&0x40 ? true:false);}
	inline bool isRuneBookAllowed(void) { return (priv2&0x1 ? true:false);}
	
	inline bool isSpellAllowed(int spellnum)
	{
		if (spellnum<=0) return false; // spellnum must be greater null, non negative value

		unsigned char spellid = 1;
		unsigned char octnum  = 0;

		octnum = ((spellnum-1)/8);
		spellid <<= ((spellnum-1)%8); // calc the spell in binary position

		if (octnum >= ((MAXSPELLS)/8)) // stay in array range of spellsallowed[27]
		{
			char t[512];
			sprintf(t,"Invalid spell# in IsSpellallowed:%i\n",spellnum);
			LogError(t);
			return false;			
		}

		return ((spellsallowed[octnum] & spellid) == spellid) ? true : false;
	}

	bool isArena()    {return arena;}
	void setArena()   {arena = true;}

	void DeathInArena(P_CHAR Player);
	
private:
	unsigned char priv;
	unsigned char priv2;
	unsigned char spellsallowed[((MAXSPELLS)/8)];
};

class cRespawn
{
private:
	bool respawning;
	int currentSpawnRegion;
//	unsigned int currentSpawnItem;
	AllItemsIterator currentSpawnItem;
public:
	cRespawn() {respawning=false;}
	~cRespawn() {}
	bool AreWeRespawning() {return respawning;}
	void Start() {respawning=true; currentSpawnRegion=0; currentSpawnItem.Begin();}
	void Continue();
};

extern cRespawn	*Respawn;

struct spawnregion_st;	// forward decl
class cCoord;


spawnregion_st* GetSpawnRegion(int r);
bool doregionspawn(spawnregion_st *SR);
void loadspawnregions();//Zippy
void loadregions();
void checkregion(P_CHAR pc);
unsigned char calcRegionFromXY(const cCoord& pos);
bool IsInInstantLogoutArea(const cCoord& pos);
int NumberOfSpawnRegions();
int IncSpawnCount(int r);
void DecSpawnCount(int r);
int AllSpawnMax();
void SpawnSomeMore(spawnregion_st *SR, int amount);
int GetSpawnDelta(spawnregion_st *SR);
bool DoTimedSpawn();

struct jail_st
{
	cCoord oldpos;
	cCoord pos;
	bool occupied;
};
void InitJails();

#endif

