/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

#include "targeting.h"

#include "basics.h"
#include "boats.h"
#include "Client.h"
#include "cmdtable.h"
#include "commands.h"
#include "guilds.h"
#include "gumps.h"
#include "house.h"
#include "itemid.h"
#include "LoS.h"
#include "magic.h"
#include "magic_new.h"
#include "mapstuff.h"
#include "necro.h"
#include "network.h"
#include "regions.h"
#include "skills.h"
#include "SndPkg.h"
#include "TmpEff.h"
#include "trigger.h"
#include "weight.h"
#include "globals.h"
#include "mapS.h"
#include "tiledata.h"
#include "Partysystem.h"
#include "npcinfo.h"
#include "LwMisc.h"
#include "spells.h"


#include "wolfpack.h"

#include "debug.h"
#undef DBGFILE
#define DBGFILE "targeting.cpp"

class cPacket6C
{
public:
//Clicking Commands (19 bytes) 
//* BYTE cmd 
//* BYTE type 
//* 0x00 = server set cursor to pointer 
//* 0x01 = client return pointer position 
//* BYTE[4] charID
	short Tnum;		// we only use the low byte anyway, so short
//* BYTE Cursor Type 
//* 0x00 - Select Object 
//* 0x01 - Choose x, y, z 
//* The following are always sent but are only valid if sent by client 
//* BYTE[4] Clicked On ID
	long Tserial;
//* BYTE[2] click xLoc
	short TxLoc;
//* BYTE[2] click yLoc 
	short TyLoc;
//* BYTE unknown2 (0x00) 
//* BYTE click zLoc
	signed char TzLoc;
//* BYTE[2] model # (if a static tile, 0 if a map/landscape tile)
	short model;
//Note: the model # and charID should NEVER be trusted.

	cPacket6C(unsigned char *pbuf)
	{
		Tnum=ShortFromCharPtr(pbuf+4); // changed from byte to short ... actually we have 4 bytes, but 65kb ought to be enough ... LB
		Tserial=LongFromCharPtr(pbuf+7);
		TxLoc=ShortFromCharPtr(pbuf+11);
		TyLoc=ShortFromCharPtr(pbuf+13);
		TzLoc=*(pbuf+16);
		model=ShortFromCharPtr(pbuf+17);
	}
};
	
///////////////////
// class:	cTarget
// purpose:	base class of all Target classes
//			encapsulates the basic functions of target processing
// remarks:	intended to become a member of the 'transaction' class in the future
//			the cTargets::multitarget shall become the ctarget::factory
//			Duke, 07/16/00
//			Changed my mind. These classes are now considered 'experimental'
//			Duke, 7.11.2000
//
class cTarget
{
protected:
	int s,serial;
	P_ITEM pi;
	P_CHAR pc;
//	P_CLIENT ps;
	void makeSerial()		{serial=LongFromCharPtr(&buffer[s][7]);}
	void FindTargetChar()	{pc=FindCharBySerial(serial);}
	void FindTargetItem()	{pi=FindItemBySerial(serial);}
public:
	cTarget(P_CLIENT pCli)	{s=pCli->GetSocket();}
	virtual void process() = 0;
};

class cCharTarget : public cTarget
{
public:
	cCharTarget(P_CLIENT pCli) : cTarget(pCli) {}
	virtual void CharSpecific(P_CHAR pc) = 0;
	virtual void process()
	{
		makeSerial();
		FindTargetChar();
		if (pc)
			CharSpecific(pc);
		else
			sysmessage(s,"That is not a character.");
	}
};

class cItemTarget : public cTarget
{
public:
	cItemTarget(P_CLIENT pCli) : cTarget(pCli) {}
	virtual void ItemSpecific() = 0;
	virtual void process()
	{
		makeSerial();
		FindTargetItem();
		if (pi)
			ItemSpecific();
		else
			sysmessage(s,"That is not an item.");
	}
};

class cWpObjTarget : public cTarget
{
public:
	cWpObjTarget(P_CLIENT pCli) : cTarget(pCli) {}
	virtual void CharSpecific(P_CHAR pc) = 0;
	virtual void ItemSpecific() = 0;
	virtual void process()
	{
		makeSerial();
		if (isItemSerial(serial)) // an item's serial ?
		{
			FindTargetItem();
			if (pi)
				ItemSpecific();
			else
				sysmessage(s,"That is not a valid item.");
		}
		else
		{
			FindTargetChar();
			if (pc)
				CharSpecific(pc);
			else
				sysmessage(s,"That is not a valid character.");
		}
	}
};


void cTargets::PlVBuy(int s, P_CHAR Player, P_ITEM pi)//PlayerVendors
{
	if (s == -1 || !Player) return;

	P_CHAR Vendor = FindCharBySerial(addx[s]);
	if (!Vendor) return;

	P_ITEM Pack = Packitem(Player);
	if (!Pack) {sysmessage(s,"Time to buy a backpack"); return; } //LB

	if (!pi) return;
	if (pi->isInWorld()) return;

	P_ITEM SellCont=FindItemBySerial(pi->contserial);	// the pack
	P_CHAR Vendor2=GetPackOwner(SellCont);				// the vendor
	if (!Vendor2) return;
	if (!Vendor2->isSameAs(Vendor) || !Vendor->isAiVendor()) return;

	if (Player->Owns(Vendor))
	{
		npctalk(s, Vendor, "I work for you, you need not buy things from me!",0);
		return;
	}

	if (!Player->canAfford(pi->value))
	{
		npctalk(s, Vendor, "You cannot afford that.",0);
		return;
	}
	Player->pay(pi->value);	// take gold from player

	npctalk(s, Vendor, "Thank you.",0);
	Vendor->holdg+=pi->value; // putting the gold to the vendor's "pocket"

	// sends item to the proud new owner's pack
	pi->SetContSerial(Pack->serial);
	pi->refresh();

}

////////////////
// name:		triggertarget
// history:		by Magius(CHE),24 August 1999
// Purpose:		Select an item to set with new trigger.
//
void cTargets::triggertarget(int s, P_ITEM pi)
{
	if (pi)
		triggerwitem(s,pi,1);
}

void cTargets::BanTarg(int s)
{

}

////////////////
// name:		addtarget
// history:		UnKnown (Touched tabstops by Tauriel Dec 29, 1998)
// Purpose:		Adds an item when using /add # # .
//
static void AddTarget(int s, cPacket6C *pp)
{
	if (pp->TxLoc==-1 || pp->TyLoc==-1) return;

	if (addid1[s]>=0x40) // LB 25-dec-199, changed == to >= for chest multis, probably not really necassairy
	{
		switch (addid2[s])
		{
		case 100:
		case 102:
		case 104:
		case 106:
		case 108:
		case 110:
		case 112:
		case 114:
		case 116:
		case 118:
		case 120:
		case 122:
		case 124:
		case 126:
		case 140:
			buildHousePass2(s,addx[s]);//If its a valid house, send it to buildhouse!
			return; // Morrolan, here we WANT fall-thru, don't mess with this switch
		}
	}
	int pileable=0;
	short id=(addid1[s]<<8)+addid2[s];
	const tileData *tile;
	tile = TileData->SeekTile(id);
	if (TileData->isStackable(tile->flags)) pileable=1;

	P_ITEM pi=Items->SpawnItem(currchar(s), 1, "#", pileable, id, 0,0);
	if (!pi) return;
	pi->resetPriv();	//Make them not decay
	pi->MoveTo(pp->TxLoc,pp->TyLoc,pp->TzLoc+Map->TileHeight(pp->model));

	pi->refresh();
	addid1[s]=0;
	addid2[s]=0;
}

class cRenameTarget : public cWpObjTarget
{
public:
	cRenameTarget(P_CLIENT pCli) : cWpObjTarget(pCli) {}
	void CharSpecific(P_CHAR pc)
	{
		strcpy(pc->name,xtext[s]);
	}
	void ItemSpecific()
	{
		if (addx[s]==1) //rename2 //New -- Zippy
			strcpy(pi->name2,xtext[s]);
		else
			strcpy(pi->name,xtext[s]);
	}
};

//we don't need this anymore - AntiChrist (9/99)
// hehe, yes we do
// what about the /tele command ???
// modified for command only purpose and uncommented by LB...

// Changed to have a gmmove effect at origin and at target point 
// Aldur
// 
static void TeleTarget(P_CLIENT ps, cPacket6C *pp) 
{ 
	if (pp->TxLoc==-1 || pp->TyLoc==-1) return;
	P_CHAR Player = ps->getPlayer();
	UOXSOCKET s = ps->GetSocket();

	int x=pp->TxLoc; 
	int y=pp->TyLoc; 
	signed char z=pp->TzLoc; 
	cCoord newPos(x,y,z+Map->TileHeight(pp->model),Player->pos.map);	

	if (x<0 || y<0) return;

	if (line_of_sight( s, Player->pos, (cCoord&) newPos,LOS_STD_BLOCKERS) || Player->isGM()) 
	{ 
		doGmMoveEff(s); 

		Player->teleTo(newPos); 

		doGmMoveEff(s); 
	} 
}

class cRemoveTarget : public cWpObjTarget
{
public:
	cRemoveTarget(P_CLIENT pCli) : cWpObjTarget(pCli) {}
	void CharSpecific(P_CHAR pc)
	{
		if (pc->account>-1 && !pc->npc) // player check added by LB
		{
			sysmessage(s,"You cant delete players");
			return;
		}
		sysmessage(s, "Removing character.");
		Npcs->DeleteChar(pc);
	}
	void ItemSpecific()
	{
		sysmessage(s, "Removing item.");
		Items->DeleItem(pi);
	}
};

void DyeTarget(int s, int serial)
{
	int body,b,k;

	if (addid1[s]==255 && addid2[s]==255)
	{
		P_ITEM pi=FindItemBySerial(serial);
		if (pi!=NULL)
		{
			SndDyevat(s,pi->serial, pi->id());
			pi->refresh();
		}
		P_CHAR pc = FindCharBySerial(serial);
		if (pc)
		{
			SndDyevat(s,pc->serial,0x2106);
		}
	}
	else
	{
		P_ITEM pi=FindItemBySerial(serial);
		if (pi!=NULL)
		{
			unsigned short color = ((addid1[s])<<8)+addid2[s];

			if (!dyeall[s])
			{
				if (color<0x0002 || color>0x03E9)
				{
					color=0x03E9;
				}
			}

			b=((color&0x4000)>>14)+((color&0x8000)>>15);
			if (!b)
			{
				pi->setColor(color);
			}
			pi->refresh();
		}

		P_CHAR pc = FindCharBySerial(serial);
		if (pc)
		{
			body=pc->id();
			k=(addid1[s]<<8)+addid2[s];
			if ( ( (k>>8) < 0x80 ) && body >= 0x0190 && body <= 0x0193 )
				k += 0x8000;

			b=k&0x4000;
			if (b==16384 && (body >=0x0190 && body<=0x03e1))
				k=0xf000; // but assigning the only "transparent" value that works, namly semi-trasnparency.

			if (k!=0x8000) // 0x8000 also crashes client ...
			{
				pc->skin = pc->xskin = k;
				updatechar(pc);
			}
		}
	}
}

class cNewzTarget : public cWpObjTarget
{
public:
	cNewzTarget(P_CLIENT pCli) : cWpObjTarget(pCli) {}
	void CharSpecific(P_CHAR pc)
	{
		pc->dispz=pc->pos.z=addx[s];
		updatechar(pc);
	}
	void ItemSpecific()
	{
		pi->pos.z=addx[s];
		pi->refresh();
	}
};

class cInczTarget : public cWpObjTarget
{
public:
	cInczTarget(P_CLIENT pCli) : cWpObjTarget(pCli) {}
	void CharSpecific(P_CHAR pc)
	{
		pc->dispz=pc->pos.z=addx[s]+pc->pos.z;
		updatechar(pc);
	}
	void ItemSpecific()
	{
		pi->pos.z=addx[s]+pi->pos.z;
		pi->refresh();
	}
};


void cTargets::IDtarget(int s, P_CHAR pc, P_ITEM pi)
{
	if (!pc && !pi) return;
	short id=(addx[s]<<8)+addy[s];
	if (pi)
	{
		pi->setId(id);
		pi->refresh();
	}
	else if (pc)
	{
		pc->setId(id);
		pc->setXid(id);
		updatechar(pc);
	}
}

//public !!
void cTargets::XTeleport(int s, P_CHAR Player, int x)
{
	if (!Player) return;
	int serial = -1;
	P_CHAR pc = NULL;
	switch (x)
	{
	case 0:	// get by target
		serial = LongFromCharPtr(&buffer[s][7]);
		pc = FindCharBySerial(serial);
		break;
	case 2:	// get by socket
		if (perm[makenumber(1)])
		{
			pc = MAKE_CHAR_REF(currchar(makenumber(1)));
		}
		else 
			return;
		break;
	case 5:	// get by serial
		serial = calcserial(hexnumber(1), hexnumber(2), hexnumber(3), hexnumber(4));
		pc = FindCharBySerial(serial);
		break;
	}
	
	if (pc)
	{
		pc->teleTo(Player->pos);
		return;// Zippy
	}

	P_ITEM pi = FindItemBySerial(serial);
	if (pi)
	{
		pi->MoveTo2(Player->pos);
	}
}

void XgoTarget(int s, P_CHAR pc)
{
	if (pc)
	{
		cCoord newPos(addx[s],addy[s],addz[s],DEFAULT_MAP);
		pc->teleTo(newPos); //teleTo automatically updates the other clients
	}
}

static void PrivTarget(P_CLIENT ps, P_CHAR pc)
{
	P_CHAR Player = ps->getPlayer();
	UOXSOCKET s = ps->GetSocket();
	if (!Player) return;
	if (SrvParms->gm_log)	//Logging
	{
		sprintf(temp, "%s.gm_log",Player->name);
		sprintf(temp2, "%s as given %s Priv [%x][%x]\n",Player->name,pc->name,addid1[s],addid2[s]);
		savelog(temp2, temp);
	}
	pc->setPriv(addid1[s]);
	pc->SetPriv2(addid2[s]);
}

static void KeyTarget(int s, P_CHAR Player, P_ITEM pi) // new keytarget by Morollan
{
	if (!Player || !pi) return;
	
	if (pi->getMore1_4()==0)
	{
		if ( pi->isType(ITY_Key) && (iteminrange(Player,pi,2) || (!pi->isInWorld()) ) )
		{
			if (!Player->checkSkill(TINKERING, 400))
			{
				sysmessage(s,"You fail and destroy the key blank.");
				Items->DeleItem(pi);
			}
			else
			{
				pi->setMore1_4(getAddid1_4(s));
				sysmessage(s, "You copy the key."); //Morrolan can copy keys
			}
		}
		return;
	}
	
	else if (pi->getMore1_4()==getAddid1_4(s) || addid1[s]==0xFF)
	{
		if (IsSign(pi->id()))
		{
			sysmessage(s, "What do you wish the sign to say?");
			Player->setInput(6,pi);
			return;
		}
		
		//Boats ->
		if (pi->isType(ITY_Boat) && pi->type2==3)
		{
			Boats->OpenPlank(pi);
			return;
		}
		//End Boats --^

		if (!iteminrange(Player,pi,2)) // check if item is in range
		{
			sysmessage(s, "You can't reach the item!");
			return; // return if not
		}

		if (pi->isType(ITY_Container) || pi->isType(ITY_SpawnChest))
		{
			if (pi->isType(ITY_Container)) pi->setType(ITY_LockedContainer);
			if (pi->isType(ITY_SpawnChest)) pi->setType(ITY_LockedSpawnChest);
			sysmessage(s, "You lock the container.");
			SndRemoveItemToAll(pi);	// close any open gumps
			pi->refresh();
			return;
		}
		if (pi->isType(ITY_Key))
		{
			Player->setInput(6,pi);
			sysmessage(s,"Enter new name for key.");//morrolan rename keys
			return;
		}
		if (pi->isType(ITY_LockedContainer) || pi->isType(ITY_LockedSpawnChest))
		{
			if (pi->isType(ITY_LockedContainer)) pi->setType(ITY_Container);
			if (pi->isType(ITY_LockedSpawnChest)) pi->setType(ITY_SpawnChest);
			sysmessage(s, "You unlock the container.");
			return;
		}
		if (pi->isType(ITY_Door))
		{
			pi->setType(ITY_LockedDoor);
			sysmessage(s, "You lock the door.");
			return;
		}
		if (pi->isType(ITY_LockedDoor))
		{
			pi->setType(ITY_Door);
			sysmessage(s, "You unlock the door.");
			return;
		}
	}//else if
	else
	{
		if (pi->isType(ITY_Key)) sysmessage (s, "That key is not blank!");
		else if (pi->more1=='\x00') sysmessage(s, "That does not have a lock.");
		else sysmessage(s, "The key does not fit into that lock.");
		return;
	}
	return;
}//keytarget()

void cTargets::IstatsTarget(int s, cPacket6C *pt)
{
	if (pt->Tserial==0)
	{
		const tileData *tile = TileData->SeekTile(pt->model);
		sprintf(temp, "Item [Static] ID [%x]",pt->model);
		sysmessage(s, temp);
		sprintf(temp, "ID2 [%i], Height [%i]",pt->model, tile->height);
		sysmessage(s, temp);
	}
	else
	{
		P_ITEM pi=FindItemBySerial(pt->Tserial);
		if (pi!=NULL)
		{
			char serstr[20];
			LongToHexChunks(pi->serial,serstr);
			char contstr[20];
			LongToHexChunks(pi->contserial,contstr);
			sprintf(temp, "Item [Dynamic] Ser %s ID [%x] Name [%s] Name2 [%s] Color [%x] Cont %s Layer [%x] Type [%d] Magic [%x] More [%x %x %x %x] Position [%i %i %i] Amount [%i] Priv [%x]",
				serstr,pi->id(),
				pi->name,pi->name2,pi->color(),
				contstr,
				pi->layer,pi->type,pi->getMovability(),
				pi->more1,pi->more2,pi->more3,pi->more4,
				pi->pos.x,pi->pos.y,pi->pos.z,pi->amount, pi->getPriv());
			sysmessage(s, temp);
			sprintf(temp,"STR [%d] HP/MAX [%d/%d] Damage [%d-%d] Defence [%d] Rank [%d] SecureIt [%d] MoreXYZ [%i %i %i] Poisoned [%i] RaceHate [%i] Weight [%d] Owner [%x] Creator [%s] MadeValue [%i] Value [%i] Decaytime[%i] Decay [%i] GoodType[%i] RandomValueRate[%i]",
				pi->st, pi->hp,pi->maxhp, pi->lodamage, pi->hidamage,pi->def,pi->rank,pi->secureIt,
				pi->morex, pi->morey, pi->morez,pi->Poisoned(),pi->racehate,
				pi->weight, pi->ownserial, // Ison 2-20-99
				pi->creator,pi->madewith,pi->value,
				pi->decaytimer.getRemainingSecs(),pi->decays(),
				pi->good,pi->rndvaluerate); // Magius(CHE) (2)
			sysmessage(s,temp); // Ison 2-20-99
		}
	}
}
void cTargets::TargIdTarget(int s, P_ITEM pi) // Fraz
{
	if (pi)
	{
		if (!pi->isLockedDown())
		{
			if (pi->name2 &&(strcmp(pi->name2, "#")))
				strcpy(pi->name, pi->name2);
			if (pi->name[0] == '#')
				pi->getName(temp2);
			else 
				strcpy(temp2, pi->name);
			sprintf(temp, "You found that this item appears to be called: %s", temp2);
			sysmessage(s, (char*) temp);
		}
		if (!pi->isType(ITY_CastingItem))
		{
			if (!pi->isType(ITY_IDwand))
			{
				sysmessage(s, "This item has no hidden magical properties.");
			}
			else
			{
				sprintf(temp, "It is enchanted with item identification, and has %d charges remaining.", pi->morex);
				sysmessage(s, temp);
			}
		}
		else
		{
			sprintf(temp, "It is enchanted with the spell %s, and has %d charges remaining.", spellname[(8*(pi->morex - 1)) + pi->morey - 1], pi->morez);
			sysmessage(s, temp);
		}
	}
}
static void CstatsTarget(P_CLIENT ps, P_CHAR pc)
{
	UOXSOCKET s = ps->GetSocket();

	char serstr[20];
	LongToHexChunks(pc->serial,serstr);
	sprintf(temp, "Ser %s ID [%x] Name [%s] Skin [%x] Account [%x] Priv [%x %x] Position [%i %i %i] CTimeout [%i] Fame [%i] Karma [%i] Deaths [%i] Kills [%i] NPCAI [%x] NPCWANDER [%d] WEIGHT [%.2f]",
		serstr,pc->id(),pc->name,pc->skin,
		pc->account,pc->getPriv(),pc->GetPriv2(),
		pc->pos.x,pc->pos.y,pc->pos.z, pc->timeout,
		pc->fame,pc->karma,pc->deaths,pc->kills,
		pc->npcaitype, pc->npcWander, (float)pc->weight);
	sysmessage(s, temp);

	char rname[50];
	if (!pc->getRaceName(rname,50)) strcpy(rname,"None");

	sprintf(temp, "Other Info: Poisoned [%i] Poison [%i] Hunger [%i] Carve[%i] Race[%s]",
		pc->Poisoned(),pc->Poison(),pc->Hunger(),pc->carve,rname);
	sysmessage(s, temp);

	//Gumps->Open(s, c, 0, 8); // sending statuswindows agian if already open crash 3.0.7 clients
	statwindow(s, pc);
}

static void MoveBelongingsToBp(P_CHAR pc)
{
	if (!pc) return;
	P_ITEM Pack = Packitem(pc);
	if (Pack == NULL)
	{
		Pack = pc->addBackpack();
		if (!Pack) return;
	}

	int ci=0, loopexit=0;
	P_ITEM pi;
	while ( (( pi=pc->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
	{
		if (pi->layer!=0x15 && pi->layer!=0x1D &&
			pi->layer!=0x10 && pi->layer!=0x0B)
		{
			if (pi->trigon==1 && pi->trigtype==2 && pi->layer<19)// -Frazurbluu- Trigger Type 2 is my new trigger type *-
			{
				triggerwitem(pc->calcSocket(), pi, 1); // trigger is fired
			}
			pi->SetRandPosInCont(Pack);
			pi->SetContSerial(Pack->serial);
			pi->layer=0x00;
			SndRemoveItemToAll(pi);
			pi->refresh();
		}
		else if (pc->Wears(pi) &&
			(pi->layer==0x0B || pi->layer==0x10))	// hair & beard (Duke)
		{
			Items->DeleItem(pi);
		}
	}
	updatechar(pc);
}

static void GMTarget(P_CLIENT ps, P_CHAR pc)
{
	//UOXSOCKET s = ps->GetSocket();
	P_CHAR Player = ps->getPlayer();

	if (SrvParms->gm_log)
	{
		sprintf(temp, "%s.gm_log",Player->name);
		sprintf(temp2, "%s as made %s a GM.\n",Player->name,pc->name);
		savelog(temp2, temp);
	}
	unmounthorse(pc->calcSocket());
	
	pc->setId(0x03DB);
	pc->skin = 0x8021;
	pc->setXid(0x03DB);
	pc->xskin = 0x8021;
	pc->setPriv(0xF7);
	pc->SetPriv2(0xD9);
	pc->gmrestrict = 0; // By default, let's not restrict them.
	
	int i;	
	for (i = 0; i < 7; i++) // this overwrites all previous settings !
	{
		pc->priv3[i]=metagm[0][i]; // gm defaults
		if (pc->account==0) pc->priv3[i]=0xffffffff;
	}

	int sk=SkillManager->getAllSkillsValue();
	for (i = 0; i < sk; i++)
	{
		pc->setRealSkill(i,1000);
		pc->skill[i]=1000;
	}
	
	// All stats to 100
	pc->setStr(100);
	pc->fullHp();
	pc->setDex(100);
	pc->fullStm();
	pc->setInt(100);
	pc->fullMana();

	short off = 0;
	if (!strncmp(pc->name, "GM ",3)) off=3;
	if (!strncmp(pc->name, "Counselor ",10)) off=10;
	sprintf(temp, "GM %s", pc->name+off);
	strcpy(pc->name,temp);

	MoveBelongingsToBp(pc);
}

static void CnsTarget(P_CLIENT ps, P_CHAR pc)
{
	//UOXSOCKET s = ps->GetSocket();
	P_CHAR Player = ps->getPlayer();

	if (SrvParms->gm_log)
	{
		// logging
		sprintf(temp, "%s.gm_log",Player->name);
		sprintf(temp2, "%s as made %s a Counselor.\n",Player->name,pc->name);
		savelog(temp2, temp);
	}
	pc->setId(0x03DB);
	pc->skin=0x8003;
	pc->setXid(0x03DB);
	pc->xskin=0x8002;
	pc->setPriv(0xB6);
	pc->SetPriv2(0x8D);

	short off = 0;
	if (!strncmp(pc->name, "GM ",3)) off=3;
	if (!strncmp(pc->name, "Counselor ",10)) off=10;
	sprintf(temp, "Counselor %s", pc->name+off);
	strcpy(pc->name,temp);

	for (int u=0;u<7;u++) // this overwrites all previous settigns !!!
	{
		pc->priv3[u]=metagm[1][u]; // counselor defaults
		if (pc->account==0) pc->priv3[u]=0xffffffff;
	}
	MoveBelongingsToBp(pc);
}

void cTargets::GhostTarget(P_CLIENT ps, P_CHAR pc)
{
	if (!pc->dead)
	{
		pc->setAttacker(ps->getPlayer()); // for forensics ev
		bolteffect(pc, true);
		soundeffect2(pc, 0x0029);
		deathstuff(pc);
	}
	else
		ps->SysMsg("That player is already dead.");
}

class cBoltTarget : public cCharTarget
{
public:
	cBoltTarget(P_CLIENT pCli) : cCharTarget(pCli) {}
	void CharSpecific(P_CHAR pc)
	{
		if (addid1[s]!=3) // no param or more than 2 given ? default behaviour
		{
			if (w_anim[0]==0 && w_anim[1]==0)
			{
				bolteffect(pc, true);
				soundeffect2(pc, 0x0029);
			}
			else
			{
				for (int j=0;j<=333;j++) 
				{
					bolteffect2(pc,w_anim[0],w_anim[1]);
				}
			}
		}
		else // 2 params given ... "play" individual animation NOT global
		{
			for (int j=0;j<=333;j++) 
			{
				bolteffect2(pc,addx[s],addy[s]);
			}
		}
	}
};

class cSetAmountTarget : public cItemTarget
{
public:
	cSetAmountTarget(P_CLIENT pCli) : cItemTarget(pCli) {}
	void ItemSpecific()
	{
		if (addx[s] > 64000) //Ripper..to fix a client bug for over 64k.
		{
			sysmessage(s, "No amounts over 64k in a pile!");
			return;
		}
		this->pi->amount=addx[s];
		pi->refresh();
	}
};

void cTargets::KickTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	UOXSOCKET sk=pc->calcSocket();
	if (sk>-1)
	{
		sysmessage(s, "Kicking player");
		sysmessage(sk, "You have been kicked!"); //New -- Zippy
		Network->Disconnect(sk);
	}
}

void cTargets::AddMenuTarget(int s, int addmitem)
{
	LogAssert(addmitem!=-1);
	P_ITEM pi=Items->CreateScriptItem(s, addmitem, 0);
	if (!pi) return;
	pi->refresh();
}

void cTargets::NpcMenuTarget(int s)
{
	Npcs->AddRespawnNPC(s,addmitem[s]);
}

void cTargets::VisibleTarget(int s, P_CHAR pc, P_ITEM pi)
{
	if (!pc && !pi) return;
	
	if (pi)
	{
		pi->visible=addx[s];
		pi->refresh();
	}
	else if (pc)
	{
		pc->hidden=addx[s];
		updatechar(pc);
	}
}

///////////////
// Name:	ContainerEmptyTarget1&2
// Purpose:	used by the '#empty' user-command
// history: by Duke, 29.12.2000
//
static void ContainerEmptyTarget1(P_CLIENT ps, P_ITEM pi)
{
	UOXSOCKET s = ps->GetSocket();
	if (isItemSerial(pi->serial) && pi->isType(ITY_Container))
	{
		addx[s]=pi->serial;
		target(s, 72, "Select container to fill:");
	}
	else
		sysmessage(s,"That is not a valid container!");
}
static void ContainerEmptyTarget2(P_CLIENT ps, P_ITEM pNewCont)
{
	UOXSOCKET s = ps->GetSocket();
	if (pNewCont->isType(ITY_Container))
	{
		P_ITEM pi;	// item to move from old container
		int ci=0,loopexit=0;
		while ( ((pi=ContainerSearch(addx[s],&ci)) != NULL) && (++loopexit < MAXLOOPS) )
			pNewCont->AddItem(pi);
	}
	else
		sysmessage(s,"That is not a valid container!");
}

static void OwnerTarget(P_CLIENT ps, P_CHAR pc)
{
	UOXSOCKET s = ps->GetSocket();
	
	int addser=getAddid1_4(s);
	pc->SetOwnSerial(addser);
	if (addser==-1)
	{
		pc->tamed=false;
	}
	else
	{
		pc->tamed=true;
	}
}

static void OwnerTarget(P_CLIENT ps, P_ITEM pi)
{
	UOXSOCKET s = ps->GetSocket();
	
	int os=getAddid1_4(s);
	pi->SetOwnSerial(os);

	if (pi->visible==1)
	{
		for (unsigned int j=0;j<now;j++) if (perm[j]) senditem(j,pi); // necassairy for items with visible value of 1
	}
}

void cTargets::DvatTarget(int s, P_CHAR Player, P_ITEM pi)
{
	if (!Player) return;
	if (pi && pi->dye==1)//if dyeable
	{
		P_CHAR pc=GetPackOwner(pi);
		if (Player->isSameAs(pc) || pi->isInWorld())
		{//if on ground or currchar is owning the item - AntiChrist
			pi->setColor((addid1[s]<<8)+addid2[s]);
			pi->refresh();
			soundeffect2(Player,0x023E); // plays the dye sound, LB
		} else
		{
			sysmessage(s,"That is not yours!!");
		}
	}
	else
	{
		sysmessage(s, "You can only dye cloth with this.");
	}
}

static void AddNpcTarget(int s, cPacket6C *pp)
{
	if (pp->TxLoc==-1 || pp->TyLoc==-1) return;
	P_CHAR pc = Npcs->MakeBaseChar();
	if (!pc) return;
	strcpy(pc->name, "Dummy");
	short id=(addid1[s]<<8)+addid2[s];
	pc->setId(id);
	pc->setXid(id);
	pc->skin = 0;
	pc->xskin = 0;
	pc->setPriv(0x10);
	cCoord newpos(pp->TxLoc, pp->TyLoc, pp->TzLoc+Map->TileHeight(pp->model));
	pc->teleTo(newpos);
	pc->npc=1;
	pc->dispz=pc->pos.z;
}

void cTargets::AllSetTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	
	int j;
	int k=pc->calcSocket();

	int sk=SkillManager->getAllSkillsValue();
	if (addx[s]<sk)
	{
		pc->setRealSkill(addx[s],addy[s]);
		SkillManager->updateSkillLevel(pc, addx[s]);
		if (k!=-1) updateskillPacket(k, addx[s]);
	}
	else if (addx[s]==ALL)
	{
		for (j=0;j<sk;j++)
		{
			pc->setRealSkill(j,addy[s]);
			SkillManager->updateSkillLevel(pc, j);
			if (k!=-1) updateskillPacket(k,j);
		}
	}
	else if (addx[s]==STR)
	{
		pc->setStr(addy[s]);
		for (j=0;j<sk;j++)
		{
			SkillManager->updateSkillLevel(pc,j);
			if (k!=-1) updateskillPacket(k,j);
		}
		if (k!=-1) statwindow(k,pc);
	}
	else if (addx[s]==DEX)
	{
		pc->setDex(addy[s]);
		for (j=0;j<sk;j++)
		{
			SkillManager->updateSkillLevel(pc,j);
			if (k!=-1) updateskillPacket(k,j);
		}
		if (k!=-1) statwindow(k,pc);
	}
	else if (addx[s]==INTELL)
	{
		pc->setInt(addy[s]);
		for (j=0;j<sk;j++)
		{
			SkillManager->updateSkillLevel(pc,j);
			if (k!=-1) updateskillPacket(k,j);
		}
		if (k!=-1) statwindow(k,pc);
	}
	else if (addx[s]==FAME)
	{
		pc->fame=addy[s];
	}
	else if (addx[s]==KARMA)
	{
		pc->karma=addy[s];
	}
}

static void DevtestTarget(int s, P_CHAR Player, cPacket6C *pp)
{
	;	// do nothing in production
#if 1
	cCoord Tpos(pp->TxLoc,pp->TyLoc,pp->TzLoc);
	if (line_of_sight(s, Player->pos, Tpos, LOS_STD_BLOCKERS))
		sysmessage(s,"I see you !!");
	else
		sysmessage(s,"No LoS !!");
#else
	Player->craftAI = 1;
	Player->nextCraft.setSecs(1);
#endif
}

static void InfoTarget(int s, cPacket6C *pp) // rewritten to work also with map-tiles, not only static ones by LB
{
	int tilenum,x,y;
	signed char z;	
	const tileData* tile;
	cellData map1;
	const landData *land;

	if (pp->TxLoc==-1 || pp->TyLoc==-1) return;
	x=pp->TxLoc;
	y=pp->TyLoc;
	z=pp->TzLoc;	

	if (pp->model==0)	// damn osi not me why the tilenum is only send for static tiles, LB
	{	// calculating the ID's if it's a maptype

		cCoord c(x,y,z,0); // note: needs to be changed when multiple maps get implemented
	
		AllMaps->SeekMap(c, map1);

		land = TileData->SeekLand(map1.TID);

		Console.send("type: map-tile\n");
		Console.send("tilenum: %i\n",map1.TID);
		Console.send("Flags:%08x\n", land->flags);	
		Console.send("TextureId:%04x\n", land->textureId);		
		Console.send("Name:%s\n", land->name);
	}
	else
	{
		tilenum=pp->model; // lb, bugfix
		tile = TileData->SeekTile(tilenum);
		Console.send("type: static-tile\n");
		Console.send("tilenum: %i\n",tilenum);
		Console.send("Flags:%08x\n", tile->flags);	
		Console.send("Weight:%x\n", tile->weight);
		Console.send("Layer:%x\n", tile->quality);
		Console.send("Anim:%lx\n", tile->appearance);
		Console.send("Unknown1:%x\n", tile->uk1);
		Console.send("Unknown2:%x\n", tile->uk2);
		Console.send("Unknown3:%x\n", tile->uk3);
		Console.send("Unknown4:%x\n", tile->uk4);
		Console.send("Height:%x\n", tile->height);
		Console.send("Name:%s\n", tile->name);
	}
	sysmessage(s, "Item info has been dumped to the console.");
	Console.send("\n");
}

void cTargets::TweakTarget(int s, P_CHAR pc, P_ITEM pi)
{
	if (!pc && !pi) return;

	if (pc)
		tweakmenu(s, pc->serial);
	else
		tweakmenu(s, pi->serial);
}

static void SetInvulFlag(P_CLIENT ps, P_CHAR pc)
{
	if (addx[ps->GetSocket()]==1)
		pc->makeInvulnerable();
	else
		pc->makeVulnerable();
}

static void Tiling(int s, cPacket6C *pp) // Clicking the corners of tiling calls this function - Crwth 01/11/1999
{
	if (pp->TxLoc==-1 || pp->TyLoc==-1) return;
	if (clickx[s]==-1 && clicky[s]==-1)
	{
		clickx[s]=pp->TxLoc;
		clicky[s]=pp->TyLoc;
		target(s,198, "Select second corner of bounding box.");
		return;
	}

	int c,pileable=0;
	const tileData *tile;
	int x1=clickx[s],x2=pp->TxLoc;
	int y1=clicky[s],y2=pp->TyLoc;

	clickx[s]=-1;clicky[s]=-1;

	if (x1>x2) {c=x1;x1=x2;x2=c;}
	if (y1>y2) {c=y1;y1=y2;y2=c;}

	if (addid1[s]==0x40)
	{
		switch (addid2[s])
		{
		case 100:
		case 102:
		case 104:
		case 106:
		case 108:
		case 110:
		case 112:
		case 114:
		case 116:
		case 118:
		case 120:
		case 122:
		case 124:
		case 126:
		case 140:
			AddTarget(s,pp);
			return;
		}
	}

	
	int x,y;
	unsigned short id=0;
	P_ITEM pi;

	id=(addid1[s]<<8)+addid2[s];
	tile = TileData->SeekTile(id);
	if (TileData->isStackable(tile->flags)) pileable=1;	

	for (x=x1;x<=x2;x++)
	{
		for (y=y1;y<=y2;y++)
		{
			pi=Items->SpawnItem(currchar(s), 1, "#", pileable, id, 0, 0);		
			if (!pi) return;
			pi->decayOff();	//Make them not decay
			pi->MoveTo(x,y,pp->TzLoc+Map->TileHeight(pp->model));
			pi->refresh();
		}
	}

	addid1[s]=0;
	addid2[s]=0;
}

static void TilingScript(int s, cPacket6C *pp) // Clicking the corners of tiling calls this function - Crwth 01/11/1999
{
	if (pp->TxLoc==-1 || pp->TyLoc==-1) return;
	if (clickx[s]==-1 && clicky[s]==-1)
	{
		clickx[s]=pp->TxLoc;
		clicky[s]=pp->TyLoc;
		target(s,299, "Select second corner of bounding box for tilescript.");
		return;
	}

	int c; //pileable=0;
	int x1=clickx[s],x2=pp->TxLoc;
	int y1=clicky[s],y2=pp->TyLoc;

	clickx[s]=-1;clicky[s]=-1;

	if (x1>x2) {c=x1;x1=x2;x2=c;}
	if (y1>y2) {c=y1;y1=y2;y2=c;}
	
	int x,y;	
	P_ITEM pi;

	for (x=x1;x<=x2;x++)
	{
		for (y=y1;y<=y2;y++)
		{		
			pi = (Items->CreateFromScript(s, addx2[s] ) );
			if (!pi) return;
			pi->resetPriv();	//Make them not decay
			pi->MoveTo(x,y,pp->TzLoc+Map->TileHeight(pp->model));
			pi->refresh();
		}
	}
	addid1[s]=0;
	addid2[s]=0;
}

static void KillNpcs(int s, cPacket6C *pp) // Clicking the corners of tiling calls this function - C
{
	if (pp->TxLoc==-1 || pp->TyLoc==-1) return;
	if (clickx[s]==-1 && clicky[s]==-1)
	{
		clickx[s]=pp->TxLoc;
		clicky[s]=pp->TyLoc;
		target(s,300, "Select second corner of bounding box for killnpcs.");
		return;
	}

	int x1=clickx[s],x2=pp->TxLoc;
	int y1=clicky[s],y2=pp->TyLoc;

	clickx[s]=-1;clicky[s]=-1;

	// charcount loop cauz they are not guaranteed to be in regions.
	// and it won't be executed frequently.

	cRectangle rect(x1,y1,x2,y2);
	int cnt=0;
	P_CHAR pc;
	AllCharsIterator aci;
	for (aci.Begin(); (pc = aci.GetData()) != aci.End(); aci++)
	{
		if ( pc->npc && pc->account<0 )
		{
			if (pc->pos.IsIn(rect))
			{
				cnt++;						
				Npcs->DeleteChar(pc);
			}
			
		}
	}
	
	sprintf(temp, "%i npcs killed (%i %i %i %i)\n",cnt, x1,x2,y1,y2);
	sysmessage(s, temp);

	addid1[s]=0;
	addid2[s]=0;
}

void MakeNpcItem(P_CLIENT ps, P_CHAR Npc)
{
	P_CHAR Player = ps->getPlayer();
	if (Player==NULL) return;
	
	P_ITEM NpcItem = Player->getDclickedItem();
	if (NpcItem==NULL) return;
	
	if (Npc==NULL) return;
	
	if (!Npc->npc) return; // don't put players into packs ;)
	assert (Npc->account==-1);
	
	/////////
	/// make npc invisble (to regular players), but don't delete it
	/////////
	
	Npc->doppelgaenger_serial=-2;	// -2: marker for npc in pack status
	
	Npc->makeInvulnerable();		// they should be neither killable nor visible. without them no later restoring possible	
	Npc->hide();
	cCoord NpcPos(Player->pos);
	NpcPos.Add(1,1);
	Npc->MoveTo(NpcPos);			// get it out of old mapregions !
	updatechar(Npc);
	
	///////
	/// set npc-item values
	///////
	
	NpcItem->morez=NpcItem->id();	// save old item id
	short icon = NpcInfo_getIcon( Npc->id() ); // set it to icon-item best fitting to npc-id
	NpcItem->setId(icon); 
	NpcItem->morex=Npc->serial;		// save serial for retrival (remeber: it still exists, just hidden)
	NpcItem->morey=1;				// enable it
	NpcItem->refresh();
}

static void SwapNpcsTarget(int s, cPacket6C *pp) 
{
	// first run = second targeting cursor
	if (pp->Tserial==-1) return;
	if (clickx[s]==-1)
	{
		clickx[s]=pp->Tserial;
		target(s,301, "Select source creature");
		return;
	}

	// second run, now we have all data
	
	int targetSerial=clickx[s];
	int sourceSerial=pp->Tserial;
	int duration = addx2[s];

	P_CHAR targetNPC, sourceNPC;
	targetNPC = FindCharBySerial(targetSerial);
	sourceNPC = FindCharBySerial(sourceSerial);
	
	if (targetNPC == NULL || sourceNPC ==NULL) // extra care/check, shouldn't happen
	{
		sysmessage(s,"error, invalid npc data, command not executed");
		return;
	}

	// actual code coming here
	// using it for testing doppelgaenger AI (helper) code
	// but might be a usefull command as well, let's see


	AllTmpEff->Add(sourceNPC, targetNPC, 47, 1, 0, 0, duration); //1: skill & stats multiplier
		
	// clear temp storage memory
	clickx[s]=-1;
	addid1[s]=0;
	addid2[s]=0;

}

void cTargets::Wiping(int s, cPacket6C *pt) // Clicking the corners of wiping calls this function - Crwth 01/11/1999
{
	if (clickx[s]==-1 && clicky[s]==-1)
	{
		clickx[s]=pt->TxLoc;
		clicky[s]=pt->TyLoc;
		if (addid1[s])
			target(s,199,"Select second corner of inverse wiping box.");
		else
			target(s,199,"Select second corner of wiping box.");
		return;
	}

	cRectangle rect(clickx[s],clicky[s],pt->TxLoc,pt->TyLoc);
	clickx[s]=-1;clicky[s]=-1;

	if (addid1[s]==1) // addid1[s]==1 means to inverse wipe
		Commands->PartialWipe(s,rect,true);
	else
		Commands->PartialWipe(s,rect,false);
}

static void ExpPotionTarget(int s, P_CHAR Player, cPacket6C *pp) //Throws the potion and places it (unmovable) at that spot
{
	if (!Player || pp->TxLoc==-1 || pp->TyLoc==-1) return;

	// ANTICHRIST -- CHECKS LINE OF SIGHT!
	cCoord puntcoord(pp->TxLoc,pp->TyLoc,pp->TzLoc) ;
	if (line_of_sight(s, Player->pos, (cCoord&) puntcoord , WALLS_CHIMNEYS + DOORS + ROOFING_SLANTED))
	{
		P_ITEM pi = FindItemBySerial(getAddid1_4(s));
		if (pi != NULL) // crashfix LB
		{
			pi->MoveTo(puntcoord);
			pi->SetContSerial(-1);
			pi->setNotMovable(); //make item unmovable once thrown
			movingeffect2(Player, pi, 0x0F, 0x0D, 0x11, 0x00, 0x00);
			pi->refresh();
		}
	}
	else sysmessage(s,"You cannot throw the potion there!");
}

static void Priv3Target(UOXSOCKET s, P_CHAR pc)
{
	Console.send("setpriv3target: %s\n", pc->name);
	pc->priv3[0]=priv3a[s];
	pc->priv3[1]=priv3b[s];
	pc->priv3[2]=priv3c[s];
	pc->priv3[3]=priv3d[s];
	pc->priv3[4]=priv3e[s];
	pc->priv3[5]=priv3f[s];
	pc->priv3[6]=priv3g[s];
}

void cTargets::SquelchTarg(int s, P_CHAR pc)
{
	if (!pc) return;

	if (pc->isGM())
	{
		sysmessage(s, "You cannot squelch GMs.");
		return;
	}

	UOXSOCKET sq = pc->calcSocket();
	if (pc->squelched)
	{
		pc->squelched=0;
		sysmessage(s, "Un-squelching...");
		sysmessage(sq, "You have been unsquelched!");
		pc->mutetime=-1;
	}
	else
	{
		pc->mutetime=-1;
		pc->squelched=1;
		sysmessage(s, "Squelching...");
		sysmessage(sq, "You have been squelched!");
		
		if (addid1[s]!=255 || addid1[s]!=0)
		{
			pc->mutetime=(unsigned int) (uiCurrentTime+(addid1[s]*MY_CLOCKS_PER_SEC));
			addid1[s]=255;
			pc->squelched=2;
		}
	}
}

static void TeleStuff1(UOXSOCKET s, P_CHAR Player, cPacket6C *pp)
{
	if (!IsValidSerial(pp->Tserial)) return;

	if (isCharSerial(pp->Tserial))
		target(s,215,"Select location to put this character.");
	else
		target(s,215,"Select location to put this item.");
}

static void TeleStuff2(UOXSOCKET s, P_CHAR Player, cPacket6C *pp)
{
	int serial = Player->target1serial;
	if (pp->TxLoc==-1 || pp->TyLoc==-1) return;
	if (!IsValidSerial(serial)) return;

	cCoord newpos(pp->TxLoc, pp->TyLoc, pp->TzLoc+Map->TileHeight(pp->model));

	if (isCharSerial(serial))
	{
		P_CHAR pc = FindCharBySerial(serial);
		if (!pc) return;
		sysmessage(s, "Moving character...");
		pc->teleTo(newpos);
	}
	else
	{
		P_ITEM pi = FindItemBySerial(serial);
		if (!pi) return;
		pi->MoveTo(newpos);
		sysmessage(s, "Moving item...");
		pi->refresh();
	}
}

void CarveTarget(int s, P_CHAR Player, P_ITEM Corpse, int feat, int ribs, int hides, int fur, int wool, int bird)
{
	P_ITEM Blood=Items->SpawnItem(Player,1,"#",0,0x122A,0,0);	//add the blood puddle
	if (!Blood) return;
	Blood->pos = Corpse->pos;
	Blood->setNotMovable();
	Blood->startDecay();
	Blood->refresh();

	P_ITEM pi;
	if (feat)
	{
		pi=Items->SpawnItem(s,Player,feat,"feather",1,0x1BD1,0,1,1);
		if (!pi) return;
		sysmessage(s,"You pluck the bird and get some feathers.");
	}
	if (ribs)
	{
		pi=Items->SpawnItem(s,Player,ribs,"raw rib",1,0x09F1,0,1,1);
		if (!pi) return;
		sysmessage(s,"You carve away some meat.");
	}

	if (hides)
	{
		pi=Items->SpawnItem(s,Player,hides,"hide",1,0x1078,0,1,1);
		if (!pi) return;
		sysmessage(s,"You skin the corpse and get the hides.");
	}

	if (fur)
	{	// animals with fur now yield hides (OSI). Duke, 7/17/00
		pi=Items->SpawnItem(s,Player,fur,"hide",1,0x1078,0,1,1);
		if (!pi) return;
		sysmessage(s,"You skin the corpse and get the hides.");
	}

	if (wool)
	{
		pi=Items->SpawnItem(s,Player,wool,"unspun wool",1,0x0DF8,0,1,1);
		if (!pi) return;
		sysmessage(s, "You skin the corpse and get some unspun wool.");
	}
	if (bird)
	{
		pi=Items->SpawnItem(s,Player,bird,"raw bird",1,0x09B9,0,1,1);
		if (!pi) return;
		sysmessage(s, "You carve away some raw bird.");
	}
	Weight->NewCalc(Player);
}

//AntiChrist - new carving system - 3/11/99
//Human-corpse carving code added
//Scriptable carving product added
static void newCarveTarget(UOXSOCKET s, P_CHAR Player, P_ITEM Corpse)
{
	if (!Player || !Corpse) return;

	bool deletecorpse=false;

	P_ITEM Blood=Items->SpawnItem(Player,1,"#",0,0x122A,0,0);	//add the blood puddle
	if (!Blood) return;
	Blood->setNotMovable();
	Blood->startDecay();
	Blood->MoveTo2(Corpse->pos);


	if (Corpse->morey)	//if it's a human corpse
	{
		Player->karma-=100;
		sysmessage(s,"You lost some karma!");
		criminal(Player);
		//create the Head
		sprintf(temp,"the head of %s",Corpse->name2);
		P_ITEM pi=Items->SpawnItem(Player,1,temp,0,0x1DA0,0,0);
		if (!pi) return;
		pi->SetContSerial(Corpse->serial);
		pi->layer=0x01;
		pi->att=5;

		//AntiChrist & Magius(CHE) - store item's owner, so that lately
		//if we want to know the owner we can do it
		pi->setOwnSerialOnly(Corpse->ownserial);

		//create the Body
		sprintf(temp,"the heart of %s",Corpse->name2);
		P_ITEM pi4=Items->SpawnItem(Player,1,temp,0,0x1CED,0,0);
		if (!pi4) return;
		pi4->SetContSerial(Corpse->serial);
		pi4->layer=0x01;
		pi4->att=5;
		pi4->setOwnSerialOnly(Corpse->ownserial);	// see above

		//create the Heart
		sprintf(temp,"the body of %s",Corpse->name2);
		P_ITEM pi5=Items->SpawnItem(Player,1,temp,0,0x1DAD,0,0);
		if (!pi5) return;
		pi5->SetContSerial(Corpse->serial);
		pi5->layer=0x01;
		pi5->att=5;
		pi5->setOwnSerialOnly(Corpse->ownserial);	// see above

		//create the Left Arm
		sprintf(temp,"the left arm of %s",Corpse->name2);
		P_ITEM pi6=Items->SpawnItem(Player,1,temp,0,0x1DA1,0,0);
		if (!pi6) return;
		pi6->SetContSerial(Corpse->serial);
		pi6->layer=0x01;
		pi6->att=5;
		pi6->setOwnSerialOnly(Corpse->ownserial);	// see above

		//create the Right Arm
		sprintf(temp,"the right arm of %s",Corpse->name2);
		P_ITEM pi7=Items->SpawnItem(Player,1,temp,0,0x1DA2,0,0);
		if (!pi7) return;
		pi7->SetContSerial(Corpse->serial);
		pi7->layer=0x01;
		pi7->att=5;
		pi7->setOwnSerialOnly(Corpse->ownserial);	// see above

		//create the Left Leg
		sprintf(temp,"the left leg of %s",Corpse->name2);
		P_ITEM pi8=Items->SpawnItem(Player,1,temp,0,0x1DA3,0,0);
		if (!pi8) return;
		pi8->SetContSerial(Corpse->serial);
		pi8->layer=0x01;
		pi8->att=5;
		pi8->setOwnSerialOnly(Corpse->ownserial);	// see above

		//create the Rigth Leg
		sprintf(temp,"the right leg of %s",Corpse->name2);
		P_ITEM pi9=Items->SpawnItem(Player,1,temp,0,0x1DA4,0,0);
		if (!pi9) return;
		pi9->SetContSerial(Corpse->serial);
		pi9->layer=0x01;
		pi9->att=5;
		pi9->setOwnSerialOnly(Corpse->ownserial);	// see above

		//human: always delete corpse!
		deletecorpse=true;
	}
	else
	{
		Script* Scp = findSection(carve_script,"CARVE",Corpse->carve);
		if (!Scp) return;

		int loopexit=0;
		while (Scp->NextLineSplitted() && (++loopexit < MAXLOOPS))
		{
			int inum = 0;
			if (Scp->getKeyVal("ADDITEM",&inum))
			{
				scpMark m = Scp->Suspend();
				P_ITEM pi10=Items->CreateScriptItem(s,inum,0);
				if (pi10)
				{
					pi10->layer=0;
					Corpse->AddItem(pi10);
				}
				Scp->Resume(m);
			}
		}
		Scp->Close();
	}

	if (deletecorpse)//if corpse has to be deleted
	{
		//let's empty it
		int ci=0, loopexit=0;
		P_ITEM pj;
		while ( (( pj=Corpse->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
		{
			pj->startDecay();
			pj->SetContSerial(-1);
			pj->MoveTo2(Corpse->pos);
		}
		Items->DeleItem(Corpse);	//and then delete the corpse
	}
}

static void CorpseTarget(const P_CLIENT pC, P_ITEM pi)
{
	if (!pi) return;
	UOXSOCKET s = pC->GetSocket();
	P_CHAR Player = pC->getPlayer();

	if (!iteminrange(Player,pi,1))
	{
		sysmessage(s, "That is too far away.");
		return;
	}
	
	Player->DoAction(0x20);

	if (pi->more1)
	{
		sysmessage(s, "You carve the corpse but find nothing usefull.");
		return;
	}
	
	P_ITEM Tool = Player->getDclickedItem();
	if (!Tool) return;
	if (Player->ToolWearOut(s,Tool))
		return;

	pi->more1=1;//corpse being carved...can't carve it anymore

	if (pi->morey || pi->carve>-1)
	{//if specified, use enhanced carving system!
		newCarveTarget(s,Player,pi);
	}
	else
	{//else use standard carving
		short id = pi->amount;
		CarveTarget(s, Player, pi,
			NpcInfo_GetFeathers(id),
			NpcInfo_GetRibs(id),
			NpcInfo_GetHides(id),
			NpcInfo_GetFur(id),
			NpcInfo_GetWool(id),
			NpcInfo_GetBird(id) );
	}
}

static void BladeTarget(P_CLIENT pC, cPacket6C *pp)
{
	UOXSOCKET s = pC->GetSocket();
	short id=pp->model;

	if (isCharSerial(pp->Tserial))
	{
		const P_CHAR Sheep=FindCharBySerial(pp->Tserial);
		if (!Sheep) return;
		if (Sheep->id() == 0xDF)	// a shorn sheep ?
		{
			sysmessage(s,"This sheep doesn't have enough wool.");
			return;
		}
		if (Sheep->id() == 0xCF)	// an ushorn sheep ?
		{
			if (Sheep->summoned) // summoned animals shouldn't give wool (Thrawn)
			{
				sysmessage(s,"You are unable to shear this sheep.");
				return;
			}
			P_ITEM pi=Items->SpawnItem(pC->getPlayer(),1,"#",1,0x0DF8,0,1);
			if (!pi) return;
			pi->refresh();
			tempeffect(Sheep, TEF_Sheep, 0, 0, 0); // change to 'naked'
			sysmessage(s,"You shear the sheep and get some unspun wool.");
			return;
		}
	}
	else if (IsCorpse(id))
	{
		const P_ITEM pi=FindItemBySerial(pp->Tserial);
		if (!pi) return;
		CorpseTarget(pC,pi);
	}
	else if (IsFish(id))
	{
		const P_ITEM rpi=FindItemBySerial(pp->Tserial);
		if (!rpi) return;
		short amt=0;
		if (rpi->amount>1)
			amt = rpi->amount*4;
		else
			amt=4; 
		soundeffect(s,0x00,0x56);
		P_ITEM pi=Items->SpawnItem(pC->getPlayer(),amt,"#",1,0x097A,0,1);
		if (!pi) return;
		pi->refresh();
		Items->DeleItem(rpi);
	}
	else
		sysmessage(s,"You can't think of a way to use your blade on that.");
}

void cTargets::SwordTarget(const P_CLIENT pC, cPacket6C *pp)
{
	UOXSOCKET s = pC->GetSocket();

	if (IsTree(pp->model))
	{
		P_CHAR pc = pC->getPlayer();
		pc->DoAction((!pc->onhorse) ? 0x0D : 0x1D);
		soundeffect2(pc,0x013E);
		P_ITEM pi=Items->SpawnItem(pc,1,"#",1,0x0DE1,0,0); //Kindling
		if (!pi)
			return;
		pi->MoveTo2(pc->pos);
		sysmessage(s, "You hack at the tree and produce some kindling.");
	}
	else if (IsLog(pp->model) || IsBoard(pp->model))
	{
		const P_ITEM pi=FindItemBySerial(pp->Tserial);
		Skills->BowCraft(s,pC->getPlayer(),pi);
	}
	else
		BladeTarget(pC,pp);
}

static void AxeTarget(P_CLIENT pC, cPacket6C *pp)
{
	if (IsTree(pp->model))
		Skills->TreeTarget(pC->GetSocket(),pC->getPlayer());
	else
		BladeTarget(pC,pp);
}

void cTargets::NpcTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	setAddid1_4(s,pc->serial);
	target(s, 57, "Select NPC to follow this player.");
}

void cTargets::NpcTarget2(int s, P_CHAR pc)
{
	if (!pc) return;
	if (pc->npc)
	{
		P_CHAR ToFollow = FindCharBySerial(getAddid1_4(s));
		if (!ToFollow) return;
		pc->SetFollowTarget(ToFollow);
		pc->npcWander=1;
	}
}

void cTargets::NpcRectTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	if (pc->npc)
	{
		pc->fx1=addx[s];
		pc->fy1=addy[s];
		pc->fz1=-1;
		pc->fx2=addx2[s];
		pc->fy2=addy2[s];
		pc->npcWander=3;
	}
}

void cTargets::NpcCircleTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	if (pc->npc)
	{
		pc->fx1=addx[s];
		pc->fy1=addy[s];
		pc->fz1=-1;
		pc->fx2=addx2[s];
		pc->npcWander=2; // bugfix, LB
	}
}

void cTargets::NpcWanderTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	if (pc->npc)
	{
		pc->npcWander=addx[s];
	}
}

//taken from 6904t2(5/10/99) - AntiChrist
void cTargets::NpcAITarget(int s, P_CHAR pc)
{
	if (!pc) return;
	if (pc->npc)
	{
		pc->setAi(addx[s]);
		sysmessage(s, "Npc AI changed.");//AntiChrist
	}
}

void cTargets::xBankTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	openbank(s, pc);
}

void cTargets::xSpecialBankTarget(int s, P_CHAR pc)//AntiChrist
{
	if (!pc) return;
	openspecialbank(s, pc);
}

void cTargets::SellStuffTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	sellstuff(s, pc);
}

void cTargets::ReleaseTarget(int s, P_CHAR pc) 
{ 
	if (pc) 
	{ 
		if (pc->cell == 0) 
		{ 
			sysmessage(s, "That player is not in jail!"); 
		} 
		else 
		{ 
			jails[pc->cell].occupied = false; 
			pc->teleTo(jails[pc->cell].oldpos);
			pc->cell = 0; 
			pc->SetPriv2(0);
			pc->jailtimer.reset(); 
			UOXSOCKET s2=pc->calcSocket();
			soundeffect2(pc, 0x01FD); // Play sound effect for player 
			sysmessage(s2, "You are released.."); 
			sprintf(temp,"Player %s released.", pc->name);
			sysmessage(s, temp); 
		} 
	} 
}

void cTargets::GmOpenTarget(int s, P_CHAR pc)
{
	int ci=0, loopexit=0;
	P_ITEM pi;
	while ( (( pi=ContainerSearch(pc->serial,&ci)) != NULL) && (++loopexit < MAXLOOPS) )
	{
		if (pi->layer==addmitem[s])
		{
			backpack(s,pi);
			return;
		}
	}
	sysmessage(s,"No object was found at that layer on that character");
}

void cTargets::StaminaTarget(int s, P_CHAR pc)
{
	if (pc)
	{
		soundeffect2(pc, 0x01F2);
		staticeffect(pc, 0x376A, 0x09, 0x06);
		pc->fullStm();
		updatestats(pc, 2);
		return;
	}
	sysmessage(s,"That is not a person.");
}

void cTargets::ManaTarget(int s, P_CHAR pc)
{
	if (pc)
	{
		soundeffect2(pc, 0x01F2);
		staticeffect(pc, 0x376A, 0x09, 0x06);
		pc->fullMana();
		updatestats(pc, 1);
		return;
	}
	sysmessage(s,"That is not a person.");
}

void cTargets::MakeShopTarget(int s, P_CHAR pc)
{
	Commands->MakeShop(pc);
	updatechar(pc);
	sysmessage(s, "The buy containers have been added.");
}

void cTargets::JailTarget(int s, P_CHAR pc) 
{ 
	if (!pc) return;
	
	if (pc->cell>0) 
	{ 
		sysmessage(s, "That player is already in jail!"); 
		return; 
	} 
	
	int i, x = 0; 
	for (i = 1; i < 11; i++) 
	{ 
		if (!jails[i].occupied) 
		{ 
			jails[i].oldpos = pc->pos;
			pc->teleTo(jails[i].pos); 
			pc->cell = i; 
			pc->Freeze(); // freeze them  Ripper 
			
			
			// blackwinds jail
			pc->jailtimer.setSecs(addmitem[s]); // Additem array used for jail time here.. 
			addmitem[s] = 0; // clear it 
			UOXSOCKET prisoner = pc->calcSocket(); 
			jails[i].occupied = 1; 
			sysmessage(prisoner, "You are jailed !"); 
			sysmessage(prisoner, "You notice you just got something new at your backpack.."); 
			sprintf(temp,"Player %s has been jailed in cell %i.", pc->name, i);
			sysmessage(s, temp); 

			Items->SpawnItemBackpack2(prisoner, SNI_BallOfJustice); // spawn it to prisoner. 

			// end blackwinds jail 
			
			x++; 
			break; 
		} 
	} 
	if (x == 0) 
		sysmessage(s, "All jails are currently full!"); 
}

void cTargets::AttackTarget(int s, P_CHAR Target2)
{
	P_CHAR pc = FindCharBySerial(addx[s]);
	if (!pc) return;

	if (pc->inGuardedArea()) // Ripper..No pet attacking in town.
	{
		sysmessage(s,"You cant have pets attack in town!");
		return;
	}
	npcattacktarget(Target2, pc);
}

void cTargets::FollowTarget(int s, P_CHAR pc2)
{
	P_CHAR pc = FindCharBySerial(addx[s]);
	if (!pc) return;

	pc->SetFollowTarget(pc2);
	pc->npcWander=1;
}

void cTargets::TransferTarget(int s, P_CHAR NewOwner)
{
	P_CHAR Pet = FindCharBySerial(addx[s]);
	if (!Pet) return;

	bool controlOk=false;
	if (Pet->isMount()) controlOk=true;
	else if (Pet->totame<300) controlOk=true;
	else
	{
		int skillhave =((NewOwner->Skill(TAMING))*4 + NewOwner->Skill(ANIMALLORE))/5;
		if (skillhave > Pet->totame) controlOk=true;
	}

	char t[120];
	if (!controlOk)
	{
		sprintf(t,"* %s does not accept %s as his master *",Pet->name,NewOwner->name);
		npctalkall(Pet,t,0);
		return;
	}

	sprintf(t,"* %s will now take %s as his master *",Pet->name,NewOwner->name);
	npctalkall(Pet,t,0);

	Pet->SetOwnSerial(NewOwner->serial);
	Pet->npcWander=1;

	Pet->ResetFollowTarget();
	Pet->npcWander=0;
}

void cTargets::BuyShopTarget(int s, P_CHAR pc)
{
	if (!pc) return;

	Targ->BuyShop(s, pc);
}

int cTargets::BuyShop(int s, P_CHAR pc)
{
	if (!pc) return 0;
	P_CHAR Player = MAKE_CHARREF_LRV(currchar(s),0);
	P_ITEM Cont1A=NULL, Cont1B=NULL;

	int ci=0, loopexit=0;
	P_ITEM pi;
	while ( (( pi=pc->Search(&ci)) != NULL) && (++loopexit < MAXLOOPS) )
	{
		if (pi->layer==0x1A)
		{
			Cont1A=pi;
		}
		else if (pi->layer==0x1B)
		{
			Cont1B=pi;
		}
	}

	if (!Cont1A || !Cont1B)
	{
		return 0;
	}

	impowncreate(s, pc, 0);	// Send the NPC again to make sure info is current. (OSI does this we might not have to)

	bool i1 = sendshopinfo(s, pc, Cont1A); // Send normal shop items
	bool i2 = sendshopinfo(s, pc, Cont1B); // Send items sold to shop by players

	if (!i1 && i2)	// workaround for client side hardcoded behaviour we don't want
					// namly sending nothing when 0x1a layer container empty but 0x1b not
	{
		senddummyshopinfo(s,pc,Cont1A);
	}

	if (i1 || i2)  // only open buy gump if vendor actually has something to sell 
	{
		SndShopgumpopen(s,pc->serial);
		statwindow(s,Player); // Make sure the gold total has been sent.
		return 1;
	}

	npctalk(s, pc, "I have nothing to sell", 1);
	return 0;
}

////////////////////////////////// 
// Changed hideing to make flamestrike and hide work better 
// (Aldur) 
// 
void cTargets::permHideTarget(int s, P_CHAR pc, P_CHAR Player) 
{ 
	if (!pc) return;
	
	if (pc->isHiddenBySkill()) 
	{ 
		if (pc->isSameAs(Player))
			sysmessage(s, "You are already hiding."); 
		else 
			sysmessage(s, "He is already hiding."); 
		return; 
	} 
	pc->MakePermaHidden(); 
	cCoord effPos(pc->pos);
	effPos.Add(1,1,10);
	staticeffect3(effPos, 0x37, 0x09, 0x09, 0x19, 0); 
	soundeffect2(pc, 0x0208); 
	tempeffect(pc, TEF_HideGM, 1, 0, 0); 
}

////////////////////////////////// 
// Changed unhideing to make flamestrike and unhide work better 
// (Aldur) 
// 
void cTargets::unHideTarget(int s, P_CHAR pc, P_CHAR Player) 
{ 
	if (!pc) return;
	
	if (pc->hidden == 0) 
	{ 
		if (pc->isSameAs(Player))
			sysmessage(s, "You are not hiding."); 
		else 
			sysmessage(s, "He is not hiding."); 
		return; 
	} 
	pc->RemovePermaHidden(); // unhide - AntiChrist 
	// we cant use staticeffect cause the char is invis and 
	// regular chars could see it. Instead we will use the staticeffect3 
	// which takes the char coords. 
	// staticeffect(i, 0x37, 0x09, 0x09, 0x19); 
	cCoord effPos(pc->pos);
	effPos.Add(1,1,10);
	staticeffect3(effPos, 0x37, 0x09, 0x09, 0x19, 0); 
	soundeffect2(pc, 0x0208); 
	tempeffect(pc, TEF_UnHideGM, 1, 0, 0); 
} 

void cTargets::SetSpeechTarget(int s, P_CHAR pc)
{
	if (!pc) return;

	if (!pc->npc)
		sysmessage(s,"You can only change speech for npcs.");
	else
		pc->speech=addx[s];
}

static void SetSpAttackTarget(int s, P_CHAR pc)
{
	if (!pc) return;

	pc->spattack=tempint[s];
}

void cTargets::SetSpaDelayTarget(int s, P_CHAR pc)
{
	if (!pc) return;

	pc->spadelay=tempint[s];
}

void cTargets::SetPoisonTarget(int s, P_CHAR pc)
{
	if (!pc) return;

	pc->SetPoison(tempint[s]);
}

void cTargets::SetPoisonedTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	
	pc->PoisonStart(pc->calcSocket(),tempint[s]);
}

void cTargets::FullStatsTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	
	soundeffect2(pc, 0x01F2);
	staticeffect(pc, 0x376A, 0x09, 0x06);
	pc->fullMana();
	pc->fullHp();
	pc->fullStm();
	
	updatestats(pc, 0);
	updatestats(pc, 1);
	updatestats(pc, 2);
}

void cTargets::SetAdvObjTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	pc->advobj=tempint[s];
}

////////////////
// name:		CanTrainTarget
// history:		by Antrhacks 1-3-99
// Purpose:		Used for training by NPC's
//
void cTargets::CanTrainTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	
	if (!pc->npc)
		sysmessage(s, "Only NPC's may train.");
	else
		pc->cantrain = !pc->cantrain;	//turn on if off, off if on
}

void cTargets::SetSplitTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	pc->split=tempint[s];
}

void cTargets::SetSplitChanceTarget(int s, P_CHAR pc)
{
	if (!pc) return;
	pc->splitchnc=tempint[s];
}

void cTargets::SetDirTarget(int s, P_CHAR pc, P_ITEM pi)
{
	if (!pc && !pi) return;
	
	if (pi)
	{
		pi->dir=addx[s];
		pi->refresh();
	}
	else if (pc)
	{
		pc->dir=addx[s];
		updatechar(pc);
	}
}

////////////////
// name:		NpcResurrectTarget
// history:		by UnKnown (Touched tabstops by Tauriel Dec 28, 1998)
// Purpose:		Resurrects a character
//
bool cTargets::NpcResurrectTarget(P_CHAR pc)
{
	if (pc)
	{
		if (pc->dead)
		{//Shouldn' be a validNPCMove inside a door, might fix house break in. -- from zippy code
			pc->resurrect();
			return true;
		}
		int k=pc->calcSocket();
		if (k > -1)
			sysmessage(k,"That person isn't dead");
	}
	return false;
}

void cTargets::NewXTarget(int s, P_CHAR pc, P_ITEM pi) // Notice a high similarity to th function above? Wonder why. - Gandalf
{
	if (!pc && !pi) return;

	if (pi)
	{
		pi->MoveTo(addx[s],pi->pos.y,pi->pos.z);
		pi->refresh();
	}
	else if (pc)
	{
		cCoord newPos(addx[s],pc->pos.y,pc->pos.z, pc->pos.map);
		pc->teleTo(newPos);
	}
}

void cTargets::NewYTarget(int s, P_CHAR pc, P_ITEM pi)
{
	if (!pc && !pi) return;

	if (pi)
	{
		pi->MoveTo(pi->pos.x,addx[s],pi->pos.z);
		pi->refresh();
	}
	else if (pc)
	{
		cCoord newPos(pc->pos.x,addx[s],pc->pos.z, pc->pos.map);
		pc->teleTo(newPos);
	}
}

void cTargets::IncXTarget(int s, P_CHAR pc, P_ITEM pi)
{
	if (!pc && !pi) return;

	if (pi)
	{
		pi->MoveTo(pi->pos, addx[s]);
		pi->refresh();
	}
	else if (pc)
	{
		cCoord newPos(pc->pos.x + addx[s],pc->pos.y,pc->pos.z, pc->pos.map);
		pc->teleTo(newPos);
	}
}

void cTargets::IncYTarget(int s, P_CHAR pc, P_ITEM pi)
{
	if (!pc && !pi) return;

	if (pi)
	{
		pi->MoveTo(pi->pos, 0, addx[s]);
		pi->refresh();
	}
	if (pc)
	{
		cCoord newPos(pc->pos.x, pc->pos.y + addx[s], pc->pos.z, pc->pos.map);
		pc->teleTo(newPos);
	}
}

void cTargets::Priv3XTarget(int s, P_CHAR pc) // crackerjack's addition, jul.24/99
{
	if (!pc) return;
	
	Console.send("setpriv3target: %s\n", pc->name);
	struct cmdtable_s *pct=&command_table[addx[s]];
	if (addy[s])
	{
		pc->priv3[pct->cmd_priv_m] |= (0-0xFFFFFFFF<<pct->cmd_priv_b);
		sprintf(temp,"%s has been granted access to the %s command.",pc->name, pct->cmd_name);
		sysmessage(s, temp);
	}
	else
	{
		pc->priv3[pct->cmd_priv_m] -= (0-0xFFFFFFFF<<pct->cmd_priv_b);
		sprintf(temp,"%s has been revoked access to the %s command.",pc->name, pct->cmd_name);
		sysmessage(s, temp);
	}
}

void cTargets::ShowPriv3Target(int s, P_CHAR pc) // crackerjack, jul 25/99
{
	if (!pc) return;
	
	char priv_info[10248];
	int i;
	sprintf(priv_info, "%s can execute the following commands:\n", pc->name);
	i=0; int loopexit=0;
	while(command_table[i].cmd_name && (++loopexit < MAXLOOPS) )
	{
		if (command_table[i].cmd_priv_m==255||
			(pc->priv3[command_table[i].cmd_priv_m]&
			(0-0xFFFFFFFF<<command_table[i].cmd_priv_b)))
		{
			sprintf(temp, " %s", command_table[i].cmd_name);
			strcpy(&priv_info[strlen(priv_info)], temp);
		}
		i++;
	}
	SndUpdscroll(s, strlen(priv_info), priv_info);
}

///////////
// name:	HouseOwnerTarget
// purpose:	transfer ownership of a house
// history:	by crackerjack 8/10/99
//			more or less rewritten by Duke, 16.1.2003
//
void cTargets::HouseOwnerTarget(int s, P_CHAR Player, P_CHAR NewOwner) //  - change house owner
{
	if (!Player || !NewOwner) return;

	if (NewOwner->isSameAs(Player))
	{
		sysmessage(s, "you already own this house!");
		return;
	}
	
	P_ITEM Sign = FindItemBySerial(getAddid1_4(s));
	if (!Sign) return;

	P_ITEM House = FindItemBySerial(Sign->getMore1_4());
	if (!House) return;

	int sno = NewOwner->calcSocket();
	if (sno==-1) return;

	killkeys(House);
	Sign->SetOwnSerial(NewOwner->serial);
	House->SetOwnSerial(NewOwner->serial);
	makeKeysForHouse(NewOwner, House, House->id());
	
	cRectangle *Estate, rect(0,0,0,0);
	Estate = GetEstate(House,rect);

	PC_ITEM pi;
	int loopexit=0;
	RegGrid3x3Iterator4Items rg3ii(House->pos);
	while((pi = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (   Player->Owns(pi)
			&& pi->isLockedDown()
			&& pi->pos.IsIn(*(Estate)))
		{
			pi->unconst()->TransferLockedDown(NewOwner->serial);
		}
	}

	sprintf(temp,"You have transferred your house to %s.", NewOwner->name);
	sysmessage(s, temp);

	sprintf(temp,"%s has transferred the house to you.", Player->name);
	sysmessage(sno, temp);

	sprintf(temp, "%s has transferred a house to %s.", Player->name, NewOwner->name);
	P_CHAR pc;
	AllPlayersInRangeIterator api(Player);
	for (pc = api.First(); pc != NULL; pc = api.Next())
	{
		UOXSOCKET si = api.getSocket();
		if (si!=s && si!=sno)
			sysmessage(si, temp);
	}
}

void cTargets::HouseEjectTarget(int s, P_CHAR Player, P_CHAR pc) // crackerjack 8/11/99 - kick someone out of house
{
	if (!Player || !pc) return;
	
	if (Player->isSameAs(pc))
	{
		sysmessage(s,"Do you really want to do that?!");
		return;
	}

	P_ITEM House = FindItemBySerial(getAddid1_4(s));
	if (ejectChar(House,pc))
		sysmessage(s, "Player ejected.");
	else
		sysmessage(s, "That is not inside the house.");
}

void cTargets::HouseBanTarget(int s, P_CHAR Player, P_CHAR pc) // crackerjack 8/12/99 - ban someobdy from the house
{
	if (!Player || !pc) return;

	P_ITEM House = FindItemBySerial(getAddid1_4(s));
	if (!House) return;

	if (pc->isSameAs(Player)) return;
	int r=add_hlist(pc, House, H_BAN);
	if (r==1)
	{
		sprintf(temp,"%s has been banned from this house.", pc->name);
		sysmessage(s, temp);
		Targ->HouseEjectTarget(s,Player,pc);
	}
	else if (r==2)
	{
		sysmessage(s, "That player is already on a house register.");
	}
	else
		sysmessage(s, "That player is not on the property.");
}

void cTargets::HouseFriendTarget(int s, P_CHAR Player, P_CHAR Friend) // crackerjack 8/12/99 - add somebody to friends list
{
	if (!Player || !Friend) return;
	
	P_ITEM House = FindItemBySerial(getAddid1_4(s));
	if (!House) return;
	
	if (Friend->isSameAs(Player))
	{
		sysmessage(s,"You cant do that!");
		return;
	}
	int r=add_hlist(Friend, House, H_FRIEND);
	if (r==1)
	{
		sprintf(temp,"%s has been made a friend of the house.", Friend->name);
		sysmessage(s, temp);
	}
	else if (r==2)
	{
		sysmessage(s, "That player is already on a house register.");
	}
	else 
	{
		sysmessage(s, "That player is not on the property.");
	}
}

void cTargets::HouseUnlistTarget(int s,P_CHAR pc) // crackerjack 8/12/99 - remove somebody from a list
{
	if (!pc) return;
	
	P_ITEM House = FindItemBySerial(getAddid1_4(s));
	if (!House) return;
	
	if (del_hlist(pc, House))
	{
		sprintf(temp,"%s has been removed from the house registry.", pc->name);
		sysmessage(s,temp);
	}
	else
		sysmessage(s, "That player is not on the house registry.");
}

void cTargets::HouseLockdown(UOXSOCKET s, P_CHAR Player, P_ITEM pi) // Abaddon
// PRE:		S is the socket of a valid owner/coowner and is in a valid house
// POST:	either locks down the item, or puts a message to the owner saying he's a moron
// CODER:	Abaddon
// DATE:	17th December, 1999
// revamped by Duke, 25.3.2002
{
	if (!Player) return;
	if (!pi)
	{
		sysmessage( s, "Invalid item!" );
		return;
	}
	short id = pi->id();
	/*houseSer = calcserial( addid1[s], addid2[s], addid3[s], addid4[s] );	// let's find our house
	house = calcItemFromSer(houseSer);*/
	
	// not needed anymore, cause called from house_sped that already checks that ...
	
	if (Items->isFieldSpellItem(pi))
	{
		sysmessage(s,"you cannot lock this down!");
		return;
	}
	if (pi->isType(ITY_Door) || pi->isType(ITY_LockedDoor) || pi->isType(ITY_Sign))
	{
		sysmessage(s, "You cant lockdown doors or signs!");
		return;
	}
	if ( IsAnvil(id) )
	{
		sysmessage(s, "You cant lockdown anvils!");
		return;
	}
	if ( IsForge(id) )
	{
		sysmessage(s, "You cant lockdown forges!");
		return;
	}
	if (pi->isLockedDown())
	{
		sysmessage(s,"That item is already locked down, release it first!");
		return;
	}

	PC_ITEM House = findHouseAt( pi->pos );
	if (House && Player->Owns(House))
	{
		cRectangle *Estate, rect(0,0,0,0);
		Estate = GetEstate(House,rect);
		
		PC_ITEM pj;
		int loopexit=0, amt=0;
		RegGrid3x3Iterator4Items rg3ii(House->pos);
		while((pj = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
		{
			if (   Player->Owns(pj)
				&& pj->isLockedDown()
				&& pj->pos.IsIn(*(Estate)))
			{
				amt++;
			}
		}
		// time to lock it down!
		if (amt < House->lockdownmax)
		{
			pi->LockDown(Player->serial);
		}
		else
		{
			sprintf(temp,"this house has a %i lockdown limit.", House->lockdownmax);
			sysmessage(s,temp);
		}
	}
	else
		sysmessage( s, "That item is not in your house!" );
}

void cTargets::HouseSecureDown(UOXSOCKET s, P_CHAR Player, P_ITEM pi) // Ripper
// For locked down and secure chests
{
	if (!Player) return;
	if (!pi)
	{
		sysmessage( s, "Invalid item!" );
		return;
	}
	
	if (Items->isFieldSpellItem(pi))
	{
		sysmessage(s,"you cannot lock this down!");
		return;
	}
	if (pi->isType(ITY_Door) || pi->isType(ITY_LockedDoor) || pi->isType(ITY_Sign))
	{
		sysmessage(s, "You cant lockdown doors or signs!");
		return;
	}
	if (pi->isLockedDown())
	{
		sysmessage(s,"That item is already locked down, release it first!");
		return;
	}
	if (!pi->isType(ITY_Container))
	{
		sysmessage(s,"You can only secure chests!");
		return;
	}
	
	PC_ITEM House = findHouseAt( pi->pos );
	if (!(House && Player->Owns(House)))
	{
		sysmessage( s, "That item is not in your house!" );
		return;
	}

	cRectangle *Estate, rect(0,0,0,0);
	Estate = GetEstate(House,rect);
	
	PC_ITEM pj;
	int loopexit=0, amt=0;
	RegGrid3x3Iterator4Items rg3ii(House->pos);
	while((pj = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (   Player->Owns(pj)
			&& pj->isLockedDown()
			&& pj->secureIt == 1
			&& pj->pos.IsIn(*(Estate)))
		{
			amt++;
		}
	}
	// time to lock it down!
	if (amt < House->securemax)
	{
		pi->LockDown(Player->serial);
		pi->secureIt = 1;
	}
	else
	{
		sprintf(temp,"this house has a %i lockdown limit.", House->securemax);
		sysmessage(s,temp);
	}
}


void cTargets::HouseRelease(UOXSOCKET s, P_CHAR Player, P_ITEM pi) // Abaddon & Ripper
// PRE:		S is the socket of a valid owner/coowner and is in a valid house, the item is locked down
// POST:	either releases the item from lockdown, or puts a message to the owner saying he's a moron
// CODER:	Abaddon
// DATE:	17th December, 1999
// update: 5-8-00
// revamped by Duke, 25.3.2002
{
	if (!Player) return;
	if (!pi)
	{
		sysmessage( s, "Invalid item!" );
		return;
	}
	
	if (!Player->Owns(pi))
	{
		sysmessage(s,"This is not your item!");
		return;
	}
	if (Items->isFieldSpellItem(pi))
	{
		sysmessage(s,"you cannot release this!");
		return;
	}
	if (pi->isType(ITY_Door) || pi->isType(ITY_LockedDoor) || pi->isType(ITY_Sign))
	{
		sysmessage(s, "You cant release doors or signs!");
		return;
	}

	PC_ITEM House = findHouseAt( pi->pos );
	if (!(House && Player->Owns(House)))
	{
		sysmessage( s, "That item is not in your house!" );
		return;
	}

	// ok, release it
	if ( pi->isLockedDown() || pi->isType(ITY_Container))
	{
		pi->ReleaseLockedDown();
	}
}

void cTargets::SetMurderCount(int s, P_CHAR pc)
{
	if (!pc) return;
	
	pc->kills = addmitem[s];
	setcharflag(pc);
}

void cTargets::GlowTarget(int s, P_CHAR Player, P_ITEM pi1) // LB 4/9/99, makes items glow
{
	if (!Player) return;
	if (!pi1)
	{
		sysmessage(s,"Item not found.");
		return;
	}

	if (!pi1->isInWorld())
	{
		P_CHAR pc = GetPackOwner(pi1);
		if (!Player->isSameAs(pc))	// creation only allowed in the creators pack/char otherwise things could go wrong
		{
			sysmessage(s,"you can't create glowing items in other persons packs or hands");
			return;
		}
	}

	if (pi1->glow!=0)
	{
		sysmessage(s,"that object already glows!\n");
		return;
	}

	pi1->glow_color=pi1->color(); // backup old color
	pi1->setColor(0x99); // set new color to yellow

	P_ITEM Halo = Items->SpawnItem(s,Player,1,"glower",0,0x1647,0,0,1); // new client 1.26.2 glower object

	if (Halo == NULL) return;
	Halo->dir=29; // set light radius maximal
	Halo->visible=0;

	//Halo->setOwnerMovable();

	mapRegions->Remove(Halo); // remove if add in spawnitem
	Halo->layer=pi1->layer;
	if (Halo->layer==0) // if not equipped -> coords of the light-object = coords of the
	{
		Halo->pos=pi1->pos;
	}
	else // if equipped -> place lightsource at player ( height= approx hand level )
	{
		Halo->pos=Player->pos;
		Halo->pos.z+=4;
	}

	//mapRegions->AddItem(c);
	Halo->resetPriv(); // doesnt decay

	pi1->glow=Halo->serial; // set glow-identifier


	pi1->refresh();
	Halo->refresh();
	//setptr(&glowsp[Player->serial%HASHMAX],i);

	impowncreate(s,Player,0); // if equipped send new color too
}

void cTargets::UnglowTarget(int s, P_CHAR Player, P_ITEM pi) // LB 4/9/99, removes the glow-effect from items
{
	if (!Player) return;
	if (!pi)
	{
		sysmessage(s,"no item found");
		return;
	}

	if (!pi->isInWorld())
	{
		P_CHAR pc = GetPackOwner(pi);
		if (!Player->isSameAs(pc))	// creation only allowed in the creators pack/char otherwise things could go wrong
		{
			sysmessage(s,"you can't unglow items in other perons packs or hands");
			return;
		}
	}

	int c=pi->glow;
	if (c==-1) return;
	P_ITEM Halo=FindItemBySerial(c);

	if (pi->glow==0 || !Halo )
	{
		sysmessage(s,"that object doesnt glow!\n");
		return;
	}

	pi->setColor(pi->glow_color);	// restore old color

	Items->DeleItem(Halo); // delete glowing object

	pi->glow=0; // remove glow-identifier
	pi->refresh();

	impowncreate(s,Player,0); // if equipped send new old color too

	Player->removeHalo(pi);
}

void cTargets::RevertSwapTarget(UOXSOCKET s,P_CHAR pc)
{
	if (!pc) return;
	Npcs->TempDupeNpcRevert(pc);
}

void cTargets::ShowSkillTarget(int s,P_CHAR pc) // LB's showskills
{
	// the gump doesn't support '\n', so we must use blank padding
	int a,j,k,b=0,c,z,zz,ges=0;
	char skill_info[(ALLSKILLSMAX+1)*40];
	
	if (!pc) return;
	
	z=addx[s];
	if (z<0 || z>3) z=0;
	if (z==2 || z==3)
		sprintf(skill_info, "%s's skills:", pc->name);
	else
		sprintf(skill_info, "%s's baseskills:", pc->name);
	
	b=strlen(pc->name)+11;
	if (b>23) b=23;
	
	for (c=b;c<=30;c++)
		strcpy(&skill_info[strlen(skill_info)], " ");
	
	char ttt[100];
	int sk=SkillManager->getAllSkillsValue();
	for (a=0;a<sk;a++)
	{
		if (z==0 || z==1) k=pc->getRealSkill(a); else k=pc->Skill(a);
		if (z==0 || z==2) zz=9; else zz=-1;
		
		if (k>zz) // show only if skills >=1
		{
			j=k/10;
			ges+=j;
			sprintf(temp, "%s %i", SkillManager->getSkillName(a),j);
			sprintf(ttt, "%-30.30s", temp);
			strcat(skill_info,ttt);
		}
	}
	sprintf(temp,"sum: %i  ",ges);
	strcat(skill_info,temp);
	
	SndUpdscroll(s, strlen(skill_info), skill_info);
}

void cTargets::FetchTarget(UOXSOCKET s) // Ripper
{
	sysmessage(s,"Fetch is not available at this time.");
}

void cTargets::GuardTarget(UOXSOCKET s, P_CHAR pToGuard)
{
	P_CHAR Pet = FindCharBySerial(addx[s]);
	if (Pet == NULL)
	{
		sysmessage(s, "Currently can't guard anyone but yourself.."); 
		LogError("Lost pet serial");
		return;
	}

	if ( !pToGuard || !pToGuard->Owns(Pet) )
	{
		sysmessage( s, "Currently can't guard anyone but yourself!" );
		return;
	}
	Pet->setAi(32); // 32 is guard mode
	Pet->SetFollowTarget(pToGuard);
	Pet->npcWander=1;
	sysmessage(s, "Your pet is now guarding you.");
	pToGuard->guarded = true;
}

void cTargets::ResurrectionTarget(UOXSOCKET s, P_CHAR pc)
{
	if (!pc) return;
	
	if (pc->dead)
		Targ->NpcResurrectTarget(pc);
}


static void ItemTarget(P_CLIENT ps, cPacket6C *pt)
{
	UOXSOCKET s=ps->GetSocket();
	P_ITEM pi=FindItemBySerial(pt->Tserial);
	if (pi==NULL) return;
	switch(pt->Tnum)
	{
	case 10://MoreTarget
		pi->setMore1_4(getAddid1_4(s));
		pi->refresh();
		break;
	case 90://MoreTarget
		pi->setMoreB1_4(getAddid1_4(s));
		pi->refresh();
		break;
	case  28://MovableTarget
		pi->setMovability(addx[s]);
		pi->refresh();
		break;
	case 31://ColorsTarget
		if (pi->id()==0x0FAB ||						//dye vat
			pi->id()==0x0EFF || pi->id()==0x0E27 )	//hair dye
			SndDyevat(s,pi->serial,pi->id());
		else
			sysmessage(s, "You can only use this item on a dye vat.");
		break;
	case 63://MoreXTarget
		pi->morex=addx[s];
		break;
	case 64://MoreYTarget
		pi->morey=addx[s];
		break;
	case 65://MoreZTarget
		pi->morez=addx[s];
		break;
	case 66://MoreXYZTarget
		pi->morex=addx[s];
		pi->morey=addy[s];
		pi->morez=addz[s];
		break;
	case 89://ObjPrivTarget
		if (addid1[s]==0) pi->decayOff();
		if (addid1[s]==1) pi->decayOn();
		if (addid1[s]==3) pi->setPriv(addid2[s]);
		break;
	case 122://SetValueTarget
		pi->value=addx[s];
		break;
	case 123://SetRestockTarget
		pi->restock=addx[s];
		break;
	case 129://SetAmount2Target
		if (addx[s] > 64000) //Ripper..to fix a client bug for over 64k.
		{
			sysmessage(s, "No amounts over 64k in a pile!");
			return;
		}
		pi->amount2=addx[s];
		pi->refresh();
		break;
	case 133://SetWipeTarget
		pi->wipe=addid1[s];
		pi->refresh();
		break;
	}
}

void cTargets::LoadCannon(int s, P_CHAR Player, P_ITEM pi)
{
	if (!pi || !Player) return;
	
	//if ((pi->id1==0x0E && pi->id2==0x91) && pi->morez==0)
	if ((pi->getMore1_4()==getAddid1_4(s))||
		(addid1[s]==0xFF))
	{
		if (pi->morez==0 && iteminrange(Player,pi,2))
		{
			if (pi->morez==0)
				pi->setType(ITY_CastingItem);
			pi->morex=8;
			pi->morey=10;
			pi->morez=1;
			sysmessage(s, "You load the cannon.");
		}
		else
		{
			if (pi->more1=='\x00') sysmessage(s, "That doesn't work in cannon.");
			else sysmessage(s, "That object doesn't fit into cannon.");
		}
	}
}

void cTargets::DupeTarget(P_CLIENT ps, P_ITEM pi)
{
	int s = ps->GetSocket();
	if (addid1[s]>=1)
	{
		if (pi)
		{
			for (int j=0;j<addid1[s];j++)
			{
				Commands->DupeItem(ps, pi, 1); // lb bugfix
				sysmessage(s,"DupeItem done.");//AntiChrist
			}
		}
	}
}

static void placeTownHouseTarget(P_CLIENT ps, cPacket6C *pt)
{
	cCoord pos(pt->TxLoc,pt->TyLoc,pt->TzLoc);
	placeTownHouse(ps,pos);
}

static bool SetValFromKey(char* p, P_ITEM pi)
{
	char *pv = strpbrk(p,"=");
	if (pv==NULL) return false; // no "=" = NULL=crash
	*pv=0;		// split string here
	pv++;		// skip the equal sign

	bool ok=false;
	if (!strcmp(p,"AMOUNT"))	{ pi->amount=str2num(pv); ok=true; }
	if (!strcmp(p,"DECAY"))		{ if (str2num(pv)) pi->decayOn(); else pi->decayOff(); ok=true; }
	if (!strcmp(p,"ID"))		{ pi->setId(str2num(pv)); ok=true; }
	if (!strcmp(p,"MOREX"))		{ pi->morex=str2num(pv); ok=true; }
	if (!strcmp(p,"MOREY"))		{ pi->morey=str2num(pv); ok=true; }
	if (!strcmp(p,"MOREZ"))		{ pi->morez=str2num(pv); ok=true; }
	if (!strcmp(p,"MOVABLE"))	{ pi->setMovability(str2num(pv)); ok=true; }
	if (!strcmp(p,"NAME"))		{ strcpy(pi->name,pv); ok=true; }
	if (!strcmp(p,"TYPE"))		{ pi->setType(str2num(pv));ok=true; }

	return ok;
}

static void SetiTarget(P_CLIENT ps, P_ITEM pi)
{
	char para[150];
	strcpy(para,xtext[ps->GetSocket()]);	// get the parm string
	char *pk=para,*pnk;	// ptr to keywords
	
	if (pk==NULL) return;

	int loopexit=0;
	while ( (pk && *pk && pk <= para+120) && ++loopexit<MAXLOOPS )
	{
		pnk = strpbrk(pk,",");
		if (pnk)
		{
			*pnk=0;		// split string here
			pnk++;		// skip the comma
		}
		if (! SetValFromKey(pk,pi) ) sysmessage(ps->GetSocket(),"Malformed seti command error");
		pk=pnk;			// proceed to next key
	}
	pi->refresh();
}

static void XsayTarget(P_CLIENT ps, P_CHAR pc)
{
	if (pc) npctalkall(pc,xtext[ps->GetSocket()],0);
}


static void npcact(int s, P_CHAR pc)
{
	if (pc)
		pc->DoAction(addid1[s]);
}

void CloneNPCTarget(UOXSOCKET s, P_CHAR source)
{
	if (source==NULL) return;
	
	if (!source->npc) 
	{
		sysmessage(s,"No clone babies allowed in Britannia");
		return;
	}
	
	P_CHAR clone=NULL;
	
	int a,b,c,d;
	int dx=0,dy=0,e,f;
	signed char nz=0;
	cCoord p(source->pos);
	bool o=false,ct;
	
	b=1;
	e=6;
	ct=true;
	
	if (addid1[s]>=2) b=addx[s]; else b=1;
	if (addid1[s]>=3) if (!addy[s]) ct=false; else ct=true;
	if (addid1[s]>=4) e = addz[s];
	
	f = e*e;
	if (f>6666) f = 6666;
	
	for (c=0; c<b;c++)
	{
		clone = Npcs->CloneNpc(source,ct); // create clone
		if (clone==NULL)
		{
			sysmessage(s,"Sorry, cloning didn't work");
			return;
		}
		
		for (a=0; a<f; a++) // try to place it in a random +-4*+-4 area around original
		{
			
			dy = rand()%e;
			dx = rand()%e;
			d  = rand()%5;
			
			switch(d)
			{
			case 0: dy=dy*-1; break;
			case 1: dx=dx*-1; break;
			case 2: dx=dx*-1; dy=dy*-1; break;
			}
			cCoord tryPos(source->pos);
			tryPos.Add(dx,dy);
			nz = Map->Height(tryPos, p.z);
			o = Map->CanMonsterMoveHere(tryPos, nz);
			if (o) break;
		}
		
		if (!o) 
		{  
			dx=0; 
			dy=0;
			nz = Map->Height(p, p.z);
		}
		cCoord nPos(p.x+dx, p.y+dy, nz,source->pos.map);
		clone->MoveTo(nPos);
		updatechar(clone);
	}
}

void PermActTarget(UOXSOCKET s, P_CHAR pc)
{
	unsigned short actId;
	unsigned short repeat;
	unsigned char speed;
	int params;
	unsigned char direction;

	if (pc==NULL) return;

	params=addid1[s];
	if (params!=5)
	{
	  sysmessage(s,"usage: permact actionId(0-0xff) repetations(0=inf,0-xffff) speed(0-xff) direction(0-7)" );
	  return;
	}

	actId=addx[s];
	repeat=addy[s];
	speed=addz[s];
	direction=tempint[s];

	SendPermActSinglePlayer(s, pc, actId, repeat, speed, direction);

}

void DclickedTriggerTarget(UOXSOCKET s, int serial)
{
	P_ITEM Ti = FindItemBySerial(serial);
	if (Ti)
	{
		triggerwitem(s,Ti,0);
		return;
	}

	P_CHAR Tc = FindCharBySerial(serial);
	if (Tc)
	{
		triggernpc(s,Tc,0);
		return;
	}

	triggerwitem(s,NULL,0);
}

void cTargets::MultiTarget(P_CLIENT ps) // If player clicks on something with the targetting cursor
{
	UOXSOCKET s=ps->GetSocket();
	P_CHAR Player = ps->getPlayer();
	targetok[s]=0;
	
	
	cPacket6C tbuf(&buffer[s][0]), *pt=&tbuf;
	
	if (pt->TxLoc==-1 && pt->TyLoc==-1) // do nothing if user cancelled
	{
		if (pt->Tserial==0 && pt->model==0) // this seems to be the complete 'cancel'-criteria (Duke)
			return;
	}
	if (Player->dead && !Player->isGM())
	{
		sysmessage(s,"You can't target while you are dead");
		return;
	}
	bool Iready=false, Cready=false;
	P_ITEM pi = NULL;
	P_CHAR pc = NULL;
	if (isItemSerial(pt->Tserial))
	{
		pi=FindItemBySerial(pt->Tserial);
		if (pi!=NULL)
			Iready=true;
	}
	else
	{
		pc=FindCharBySerial(pt->Tserial);
		if (pc!=NULL)
		{
			Cready=true;
			if (pc->stablemaster_serial)
				return;		// can't target stabled/mounted pets
		}
	}
	cCoord targpos(pt->TxLoc,pt->TyLoc,pt->TzLoc,Player->pos.map);
	
	switch(pt->Tnum)
	{
	case 0: AddTarget(s,pt); break;
	case 1: { cRenameTarget		T(ps);		T.process();} break;
	case 2: TeleTarget(ps,pt); break; // LB, bugfix, we need it for the /tele command
	case 3: { cRemoveTarget		T(ps);		T.process();} break;
	case 4: DyeTarget(s, pt->Tserial); break;
	case 5: { cNewzTarget		T(ps);		T.process();} break;
	case 6: if (Iready) pi->setType(addx[s]); break; //Typetarget
	case 7: Targ->IDtarget(s,pc,pi); break;
	case 8:	if (Cready) XgoTarget(s,pc); break;
	case 9: if (Cready) PrivTarget(ps,pc); break;
	case 10: ItemTarget(ps,pt); break;//MoreTarget
	case 11: if (Iready) KeyTarget(s,Player,pi); break;
	case 12: Targ->IstatsTarget(s,pt); break;
	case 13: if (Cready) CstatsTarget(ps,pc); break;
	case 14: if (Cready) GMTarget(ps,pc); break;
	case 15: if (Cready) CnsTarget(ps,pc); break;
	case 16: if (Cready) DeleteItemOnLayer(pc, 0x0b); break;
	case 17: if (Cready) DeleteItemOnLayer(pc, 0x10); break;
	case 18: if (Cready) DeleteItemOnLayer(pc, 0x15); break;
	case 19: if (Cready) pc->fonttype=addid1[s]; break;
	case 20: if (Cready) Targ->GhostTarget(ps,pc); break;
	case 21: if (Cready) Targ->ResurrectionTarget(s,pc); break; // needed for /resurrect command
	case 22: { cBoltTarget		T(ps);	T.process();} break;
	case 23: { cSetAmountTarget	T(ps);	T.process();} break;
	case 24: DclickedTriggerTarget(s, pt->Tserial);	break;
	case 25: if (Cready) Targ->KickTarget(s,pc); break;
	case 26: Targ->AddMenuTarget(s, addmitem[s]); break;
	case 27: Targ->NpcMenuTarget(s); break;
	case 28: ItemTarget(ps,pt); break;//MovableTarget
	case 29: Skills->ArmsLoreTarget(s,Player,pi); break;
	case 30: if (Cready) OwnerTarget(ps,pc); else if (Iready) OwnerTarget(ps,pi); break;
	case 31: ItemTarget(ps,pt); break;//ColorsTarget
	case 32: Targ->DvatTarget(s,Player,pi); break;
	case 33: AddNpcTarget(s,pt); break;
	case 34: if (Cready) pc->Freeze(); break;
	case 35: if (Cready) pc->Unfreeze(); break; // unfreeze, AntiChris used LB bugfix
	case 36: if (Cready) Targ->AllSetTarget(s,pc); break;
	case 37: Skills->AnatomyTarget(s,Player,pc); break;
	case 38: OldMagic->Recall(s,Player,pi); break;
	case 39: OldMagic->Mark(s,Player,pi); break;
	case 40: Skills->ItemIdTarget(s,Player,pi); break;
	case 41: Skills->EvaluateIntTarget(s,Player,pc); break;
	case 42: Skills->TameTarget(s,Player,pc); break;
	case 43: OldMagic->Gate(s); break;
	case 44: OldMagic->Heal(s,Player,pc); break; // we need this for /heal command
	case 45: FishTarget(ps,pt->TxLoc,pt->TyLoc,pt->model); break;
	case 46: InfoTarget(s,pt); break;
	case 47: if (Cready) strcpy(pc->title,xtext[s]); break;//TitleTarget

	case 49: Skills->CookOnFire(s,Player,pi,0x097B,"fish steaks"); break;
	case 50: Skills->Smith(s,pi); break;
	case 51: Skills->Mine(s,Player); break;
	case 52: Skills->SmeltOre(s,Player,pi); break;
	case 53: if (Cready) npcact(s,pc); break;
	case 54: Skills->CookOnFire(s,Player,pi,0x09B7,"bird"); break;
	case 55: Skills->CookOnFire(s,Player,pi,0x160A,"lamb"); break;
	case 56: if (Cready) Targ->NpcTarget(s,pc); break;
	case 57: if (Cready) Targ->NpcTarget2(s,pc); break;
	case 58: DevtestTarget(s,Player,pt); break;
	case 59: if (Cready) Targ->NpcCircleTarget(s,pc); break;
	case 60: if (Cready) Targ->NpcWanderTarget(s,pc); break;
	case 61: Targ->VisibleTarget(s,pc,pi); break;
	case 62: Targ->TweakTarget(s,pc,pi); break;
	case 63: //MoreXTarget
	case 64: //MoreYTarget
	case 65: //MoreZTarget
	case 66: ItemTarget(ps,pt); break;//MoreXYZTarget
	case 67: if (Cready) Targ->NpcRectTarget(s,pc); break;
	case 68: Skills->CookOnFire(s,Player,pi,0x09F2,"ribs"); break;
	case 69: Skills->CookOnFire(s,Player,pi,0x1608,"chicken legs"); break;
	case 70: Skills->TasteIDTarget(s,Player,pi); break;
	case 71: if (Iready) ContainerEmptyTarget1(ps,pi); break;
	case 72: if (Iready) ContainerEmptyTarget2(ps,pi); break;
	case 75: if (Iready) Targ->TargIdTarget(s,pi); break;
	case 76: AxeTarget(ps,pt); break;
	case 77: Skills->DetectHidden(s,Player); break;
	case 78: placeTownHouseTarget(ps,pt); break;
	case 79: Skills->ProvocationTarget1(s,pc); break;
	case 80: Skills->ProvocationTarget2(s,pc); break;
	case 81: Skills->EnticementTarget1(s,pc); break;
	case 82: Skills->EnticementTarget2(s,pc); break;
	case 83: Boats->PlaceShipYard(s,Player,targpos); break;
	case 84: if (Iready) Boats->AddShipPart(s,Player,pi); break;
		
	case 86: Targ->SwordTarget(ps,pt); break;
//	case 87: if (Iready) OldMagic->SbOpenContainer(s,pi); break; // command doesnt make sens with new spell-storing system
	case 88: Targ->SetDirTarget(s,pc,pi); break;
	case 89: ItemTarget(ps,pt); break;//ObjPrivTarget
	case 90: ItemTarget(ps,pt); break;//More2Target
		
	case 100: OldMagic->NewCastSpell(s,pt->Tserial); break;	// we now have this as our new spell targeting location

	// the following is for new magic (thrawn)
	case 101:
		cSpell* spell;
		spell = Player->GetCastingSpell();
		if (!spell) return;

		if ((spell->GetTargetType()&MAP_TARGET)==MAP_TARGET) // FIRST see whether we need a map target
		{
			Magic->FinnishCasting(Player,&targpos,MAP_TARGET); // targeted an item
		}
		else if (Cready && ((spell->GetTargetType()&CHAR_TARGET)==CHAR_TARGET) ) 
		{
			Magic->FinnishCasting(Player,pc,CHAR_TARGET); // targeted a char and need a char
		}
		else if (Iready && ((spell->GetTargetType()&ITEM_TARGET)==ITEM_TARGET) )
		{
			Magic->FinnishCasting(Player,pi,ITEM_TARGET); // targeted an item and need an item
		}
		else sysmessage(s,"That is not a valid target for this spell!"); // invalid target for the spell

		break;
	// end magic stuff

	case 105: if (Cready) Targ->xSpecialBankTarget(s,pc); break;//AntiChrist
	case 106: if (Cready) Targ->NpcAITarget(s,pc); break;
	case 107: if (Cready) Targ->xBankTarget(s,pc); break;
	case 108: Skills->AlchemyTarget(s,pi); break;
	case 109: Skills->BottleTarget(s,Player,pi); break;
	case 110: if (Iready) Targ->DupeTarget(ps,pi); break;
	case 111: ItemTarget(ps,pt); break;//MovableTarget
	case 112: if (Cready) Targ->SellStuffTarget(s,pc); break;
	case 113: Targ->ManaTarget(s,pc); break;
	case 114: Targ->StaminaTarget(s,pc); break;
	case 115: if (Cready) Targ->GmOpenTarget(s,pc); break;
	case 116: if (Cready) Targ->MakeShopTarget(s,pc); break;
	case 117: if (Cready) Targ->FollowTarget(s,pc); break;
	case 118: if (Cready) Targ->AttackTarget(s,pc); break;
	case 119: if (Cready) Targ->TransferTarget(s,pc); break;
	case 120: if (Cready) Targ->GuardTarget(s,pc); break;
	case 121: if (Cready) Targ->BuyShopTarget(s,pc); break;
	case 122: ItemTarget(ps,pt); break;//SetValueTarget
	case 123: ItemTarget(ps,pt); break;//SetRestockTarget
	case 124: Targ->FetchTarget(s); break;
		
	case 126: if (Cready) Targ->JailTarget(s,pc); break;
	case 127: Targ->ReleaseTarget(s,pc); break;
	case 128: Skills->CreateBandageTarget(s,Player,pi); break;
	case 129: ItemTarget(ps,pt); break;//SetAmount2Target
	case 130: Skills->HealingSkillTarget(s,Player,pc); break;
	case 131: if (Cready) Targ->permHideTarget(s,pc,Player); break; /* not used */
	case 132: if (Cready) Targ->unHideTarget(s,pc,Player); break; /* not used */
	case 133: ItemTarget(ps,pt); break;//SetWipeTarget
	case 134: Skills->Carpentry(s,Player,pi); break;
	case 135: if (Cready) Targ->SetSpeechTarget(s,pc); break;
	case 136: Targ->XTeleport(s,Player,0); break;
		
	case 150: if (Cready) SetSpAttackTarget(s,pc); break;
	case 151: if (Cready) Targ->FullStatsTarget(s,pc); break;
	case 152: Skills->BeggingTarget(s,Player,pc); break;
	case 153: Skills->AnimalLoreTarget(s,Player,pc); break;
	case 154: Skills->ForensicsTarget(s,Player,pi); break;
	case 155: target(s, 156, "What item do you want to poison?");	break;
	case 156: Skills->PoisoningTarget(s,Player,pi); break;
		
	case 160: if (Iready) Skills->Inscribe(s,pi); break;
		
	case 162: Skills->LockPick(s,Player,pi); break;
		
	case 164: Skills->Wheel(s,Player,pi, YARN); break;
	case 165: Skills->Loom(s,Player,pi); break;
	case 166: Skills->Wheel(s,Player,pi, THREAD); break;
	case 167: Skills->Tailoring(s,Player,pi); break;
		
	case 170: if (Iready) Targ->LoadCannon(s,Player,pi); break;
	case 171: OldMagic->BuildCannon(s,Player); break;
	case 172: Skills->Fletching(s,Player,pi); break;
	case 173: Skills->MakeDough(s,Player,pi); break;
	case 174: Skills->MakePizza(s,Player,pi); break;
	case 175: if (Cready) Targ->SetPoisonTarget(s,pc); break;
	case 176: if (Cready) Targ->SetPoisonedTarget(s,pc); break;
	case 177: if (Cready) Targ->SetSpaDelayTarget(s,pc); break;
	case 178: if (Cready) Targ->SetAdvObjTarget(s,pc); break;
	case 179: if (Cready) SetInvulFlag(ps,pc); break;
	case 180: if (Iready) Skills->Tinkering(s,Player,pi); break;
	case 181: Skills->PoisoningTarget(s,Player,pi); break;
		
	case 183: Skills->TinkerAxel(s,pi); break;
	case 184: Skills->TinkerAwg(s,pi); break;
	case 185: Skills->TinkerClock(s,pi); break;
	case 186: vialtarget(s,Player,pt->Tserial); break;
		
	case 198: Tiling(s,pt); break;
	case 199: Targ->Wiping(s,pt); break;
		
	case 200: Commands->SetItemTrigger(s,pi); break;
	case 201: Commands->SetNPCTrigger(s,pc); break;
	case 202: if (Iready) Commands->SetTriggerType(s,pi); break;
	case 203: Commands->SetTriggerWord(s,pc); break;
	case 204: if (Iready) triggertarget(s,pi); break; // Fixed by Magius(CHE)
	case 205: Skills->StealingTarget(s,Player,pt->Tserial); break;
	case 206: if (Cready) Targ->CanTrainTarget(s,pc); break;
	case 207: ExpPotionTarget(s,Player,pt); break;
	case 209: if (Cready) Targ->SetSplitTarget(s,pc); break;
	case 210: if (Cready) Targ->SetSplitChanceTarget(s,pc); break;
	case 212: Commands->Possess(s); break;
	case 213: Skills->PickPocketTarget(s); break;
	case 214: TeleStuff1(s,Player,pt); break;
	case 215: TeleStuff2(s,Player,pt); break;
		
	case 220: Guilds->Recruit(s,pc); break;
	case 221: Guilds->TargetWar(s,pc); break;
	case 223: if (Cready) Targ->SquelchTarg(s,pc); break;//Squelch
	case 224: if (Iready) Targ->PlVBuy(s,Player,pi); break;//PlayerVendors
	case 225: if (Cready) Targ->Priv3XTarget(s,pc); break; // SETPRIV3 +/- target
	case 226: if (Cready) Targ->ShowPriv3Target(s,pc); break; // SHOWPRIV3
	case 227: if (Cready) Targ->HouseOwnerTarget(s,Player,pc); break; // cj aug11/99
	case 228: if (Cready) Targ->HouseEjectTarget(s,Player,pc); break; // cj aug11/99
	case 229: if (Cready) Targ->HouseBanTarget(s,Player,pc); break; // cj aug12/99
	case 230: if (Cready) Targ->HouseFriendTarget(s,Player,pc); break; // cj aug 12/99
	case 231: if (Cready) Targ->HouseUnlistTarget(s,pc); break; // cj aug 12/99
	case 232: Targ->HouseLockdown(s,Player,pi); break; // Abaddon 17th December 1999
	case 233: Targ->HouseRelease(s,Player,pi); break; // Abaddon 17th December 1999
	case 234: Targ->HouseSecureDown(s,Player,pi); break; // Ripper
	case 235: Targ->BanTarg(s); break;
	case 236: Skills->RepairTarget(s,Player,pi); break; //Ripper..Repairing item
	case 237: Skills->SmeltItemTarget(s,Player,pi); break; //Ripper..Smelting item
		//taken from 6904t2(5/10/99) - AntiChrist
	case 240: if (Cready) Targ->SetMurderCount(s,pc); break; // Abaddon 13 Sept 1999
		
	case 245: if (!Cready && !Iready) buildHousePass2(s,addx[s]);	break;
	case 246: if (!Cready && !Iready) AddTarget(s,pt); break;
		
	case 247: if (Cready) Targ->ShowSkillTarget(s,pc);break;
	//case 248: Targ->MenuPrivTarg(s);break; // menupriv target
	case 249: if (Iready) Targ->UnglowTarget(s,Player,pi);break; // unglow
	case 250: if (Cready) Priv3Target(s,pc); break; // meta gm target
	case 251: Targ->NewXTarget(s,pc,pi); break; // NEWX
	case 252: Targ->NewYTarget(s,pc,pi); break; // NEWY
	case 253: Targ->IncXTarget(s,pc,pi); break; // INCX
	case 254: Targ->IncYTarget(s,pc,pi); break; // INCY
	case 255: Targ->GlowTarget(s,Player,pi); break; // glow
		
	case 259: if (Cready) Targ->RevertSwapTarget(s,pc);break;
	case 260: { cInczTarget		T(ps);	T.process(); } break;
	case 261: if (Iready) SetiTarget(ps,pi);break;
	case 262: if (Cready) CloneNPCTarget(s, pc); break;
	case 263: if (Cready) PermActTarget(s, pc); break;
	case 264: if (Cready) XsayTarget(ps, pc);break;
		
	case 299: TilingScript(s, pt); break;
	case 300: KillNpcs(s,pt); break;
	case 301: SwapNpcsTarget(s, pt); break;
	case 302: if (Cready) MakeNpcItem(ps, pc); break;

	case 350: if (Cready) PartySystem->RequestAuthorization(ps,pc); break;
	case 351: if (Cready) PartySystem->RemoveMemberTarg(ps,pc); break;
		
	default:
		LogErrorVar("Fallout of switch statement, multitarget(), value=(%i)",pt->Tnum);
	}

	Player->target1serial = pt->Tserial;	// remember what they targeted
}

