/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

/////////////////////////////////////////
/// rewrite: LB Sept 2002
/////////////////////////////////////////

///////////////////////////////////////////////////////
////
//// this class covers access to tiledata.mul 
//// (hardcoded properties of tiles (items+land tiles)
////
//////////////////////////////////////////////////////

#include "tiledata.h"
#include "assert.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "teleport.cpp"

// lets read the data from tiledata.mul and make a 1:1 copy in RAM
tileInfo::tileInfo(std::string basedir)
{
	FILE * vf=NULL;
	FILE * tf=NULL;
	FILE * ai=NULL;
	bool verdata;
	std::string verDir, tileDir, aiDir, t;
	int a,b,r;
	unsigned long int pos;

	error = 0;

	verDir  = basedir + "verdata.mul";
	tileDir = basedir + "tiledata.mul";

	t = "\n\n-----------------------------------------------------------\n\nLoading " + tileDir +"\n";
	Console.send(t.c_str());

	tf = fopen(tileDir.c_str(), "rb");
	if (tf==NULL)
	{
		t = "fatal error: " + tileDir + " not found, quitting\n";
		Console.send(t.c_str());
		error=1;
		return;
	}

	assert ( sizeof(landData)==26 || sizeof(landData)==28); 
	assert ( sizeof(tileData)==37 || sizeof(tileData)==40); 

	unsigned int ui;
	unsigned char c;
	unsigned short sh;
	unsigned char n[22];

	//// note: loading could be done faster and with fewer code
	//// but because it loads here in <1s and I don't want to retest,  and it only adds to startup time, not runtime, 
	//// I don't bother optimizing it

	/////////////////////
	// load landdata
	//////////////////////
	for (a=0; a<0x4000; a++)
	{
		pos = ((a>>5)+1)*4 + a*26;
		b = fseek(tf, pos, SEEK_SET);

		if (b!=0)
		{
		   t = "fatal error seeking into: " + tileDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}

		r = fread( &ui, sizeof(ui), 1, tf);	
		if (r!=1)
		{
		   t = "fatal error reading1: " + tileDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}
		
		land[a].flags=ui;

		r = fread( &sh, sizeof(sh), 1, tf);	
		if (r!=1)
		{
		   t = "fatal error reading2: " + tileDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}

		land[a].textureId=sh;

		r = fread(&n, 20, sizeof(unsigned char), tf);	
		if (r!=1)
		{
		   t = "fatal error reading3: " + tileDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}

		memcpy(&(land[a].name), &n, 20);
	}

	/////////////////////
	// load tiledata
	/////////////////////
	for (a=0; a<0x4000; a++)
	{
		pos = 0x00068800 + ((a>>5)+1)*4 + a*37;
		b = fseek(tf, pos, SEEK_SET);

		if (b!=0)
		{
		   t = "fatal error seeking into: " + tileDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}

		r = fread( &ui, sizeof(ui), 1, tf);	
		if (r!=1)
		{
		   t = "fatal error reading1: " + tileDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}
		tile[a].flags=ui;

		r = fread( &c, sizeof(c), 1, tf);	
		if (r!=1)
		{
		   t = "fatal error reading2: " + tileDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}
		tile[a].weight=c;

		r = fread( &c, sizeof(c), 1, tf);	
		if (r!=1)
		{
		   t = "fatal error reading3: " + tileDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}
		tile[a].quality=c;

		r = fread( &sh, sizeof(sh), 1, tf);	
		if (r!=1)
		{
		   t = "fatal error reading4: " + tileDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}
		tile[a].uk1=sh;

		r = fread( &c, sizeof(c), 1, tf);	
		if (r!=1)
		{
		   t = "fatal error reading5: " + tileDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}
		tile[a].uk2=c;

		r = fread( &c, sizeof(c), 1, tf);	
		if (r!=1)
		{
		   t = "fatal error reading6: " + tileDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}
		tile[a].quantity=c;

		r = fread( &sh, sizeof(sh), 1, tf);	
		if (r!=1)
		{
		   t = "fatal error reading7: " + tileDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}
		tile[a].appearance=sh;

		r = fread( &c, sizeof(c), 1, tf);	
		if (r!=1)
		{
		   t = "fatal error reading8: " + tileDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}
		tile[a].uk3=c;

		r = fread( &c, sizeof(c), 1, tf);	
		if (r!=1)
		{
		   t = "fatal error reading9: " + tileDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}
		tile[a].hue=c;

		r = fread( &c, sizeof(c), 1, tf);	
		if (r!=1)
		{
		   t = "fatal error reading10: " + tileDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}
		tile[a].uk4=c;

		r = fread( &c, sizeof(c), 1, tf);	
		if (r!=1)
		{
		   t = "fatal error reading11: " + tileDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}
		tile[a].value=c;

		r = fread( &c, sizeof(c), 1, tf);	
		if (r!=1)
		{
		   t = "fatal error reading12: " + tileDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}
		tile[a].height=c;

		r = fread(&n, 20, sizeof(unsigned char), tf);	
		if (r!=1)
		{
		   t = "fatal error reading13: " + tileDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}
		memcpy(&(tile[a].name), &n, 20);
	}

	/// open artidx.mul
	aiDir = basedir + "artidx.mul";

	ai = fopen(aiDir.c_str(), "rb");
	if (ai==NULL)
	{
		t = "fatal error: " + aiDir + " not found, quitting\n";
		Console.send(t.c_str());
		error=1;
		return;
	}

	int lnum=0, tnum=0;
	unsigned int v;
	unsigned long int p;
	bool val, val2;

	for (a=0; a<0x4000; a++)
	{
		// check if land tile valid via artidx
		p = a*12;

		b = fseek(ai, p, SEEK_SET);
		if (b!=0)
		{
		   t = "fatal error seeking into: " + aiDir + ", quitting\n";Console.send(t.c_str());
		   error=1;
		   return;
		}

		r = fread(&v, sizeof(int), 1, ai);	
		if (r!=1)
		{
		   t = "fatal error reading19: " + aiDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}

		if (v==0xffffffff) val=false; else val=true;

		// same for tiles

		p = (a+0x4000)*12;

		b = fseek(ai, p, SEEK_SET);
		if (b!=0)
		{
		   t = "fatal error seeking into: " + aiDir + ", quitting\n";Console.send(t.c_str());
		   error=1;
		   return;
		}

		r = fread(&v, sizeof(int), 1, ai);	
		if (r!=1)
		{
		   t = "fatal error reading19: " + aiDir + ", quitting\n";Console.send(t.c_str());
		   error=1;return;
		}
	
		if (v==0xffffffff) val2=false; else val2=true;
		
		if (val)
		{
			lnum++;
			land[a].invalid=false;
		} else land[a].invalid=true;

		if (val2)
		{
			tnum++;
			tile[a].invalid=false;
		} else tile[a].invalid=true;	
	}

	sprintf(temp,"number of land-tiles   : %i\nnumber of static-tiles : %i\n", lnum, tnum);
	Console.send(temp);

	/*
	for (a=0; a<0x4000; a++)
	{
		if (!tile[a].invalid)
		{
			if (this->isTileBreakLos(a)) LogErrorVar("Break-LOS bit:%04x\n",a);
		}		
	}*/



	///////////////////////////////////////////
	// do we have verdata.mul present ???
	// if yes, overwrite 
	///////////////////////////////////////////
	vf = fopen(verDir.c_str(), "rb");
	if (vf==NULL) verdata=false; else verdata=true;
	
	if (verdata)
	{
		Console.send("Verdata found\n");
		int numpatch;
		int r;
		
		r = fread(&numpatch, sizeof(int), 1, vf);
		if (r!=1) 
		{
			Console.send("but invalid, not using it\n");		
			return;
		}
		
		b=0;
		int d;
		int type, block, extra, size, position;
		for (a=0; a<numpatch; a++)
		{
			pos = 4+a*20;

			r = fseek(vf, pos, SEEK_SET);
			if (r!=0)
			{
			   t = "error seeking1: " + verDir + "\n";Console.send(t.c_str());		  			  
		       return;
			}

			r = fread(&type, sizeof(int), 1, vf);
			if (r!=1)
			{
				t = "error reading1: " + verDir + "\n";Console.send(t.c_str());
				return;
			}
			
			if (type==0x1e) // 0x1e: tiledata patch magic number
			{		
				b++;			
				
				r = fread(&block, sizeof(int),1, vf);
				if (r!=1)
				{
					t = "error reading2: " + verDir + "\n";Console.send(t.c_str());		  			  
					return;
				}
				
				r = fread(&position, sizeof(int),1, vf);
				if (r!=1)
				{
					t = "error reading3: " + verDir + "\n";Console.send(t.c_str());		  			  
					return;
				}
				
				r = fread(&size, sizeof(int),1, vf);
				if (r!=1)
				{
					t = "error reading4: " + verDir + "\n";Console.send(t.c_str());		  			  
					return;
				}
				
				r = fread(&extra, sizeof(int),1, vf);
				if (r!=1)
				{
					t = "error reading4: " + verDir + "\n";Console.send(t.c_str());		  			  
					return;
				}
				
				//printf("b:%i pos:%i sz:%i v:%i ",(block-512)*32, position, size, extra);
				
				assert ( (size==1188 && block>511) || (size==836 && block<=511) );
				
				if (size==(32*37+4) && block>511) // tiledata patch
				{
					block-=512;
					block=block*32;
					
					//printf(" :%s ",tile[block].name);
					
					r = fseek(vf, position+4, SEEK_SET); // skip over first redundant 4 block bytes
					
					if (r!=0)
					{
						t = "error seeking2: " + verDir + "\n";Console.send(t.c_str());		  			  
						return;
					}
					
					for(d=0; d<32; d++) // for unknown reason OSI only patches in blocks of 32 tiles
					{
						r = fread( &ui, sizeof(ui), 1, vf);	
						if (r!=1)
						{
							t = "error reading1: " + verDir + "\n";Console.send(t.c_str());
							return;
						}
						tile[block+d].flags=ui;
						
						r = fread( &c, sizeof(c), 1, vf);	
						if (r!=1)
						{
							t = "error reading2: " + verDir + "\n";Console.send(t.c_str());
							return;
						}
						tile[block+d].weight=c;
						
						r = fread( &c, sizeof(c), 1, vf);	
						if (r!=1)
						{
							t = "error reading3: " + verDir + "\n";Console.send(t.c_str());
							return;
						}
						tile[block+d].quality=c;
						
						r = fread( &sh, sizeof(sh), 1, vf);	
						if (r!=1)
						{
							t = "error reading4: " + verDir + "\n";Console.send(t.c_str());
							return;
						}
						tile[block+d].uk1=sh;
						
						r = fread( &c, sizeof(c), 1, vf);	
						if (r!=1)
						{
							t = "error reading5: " + verDir + "\n";Console.send(t.c_str());
							return;
						}
						tile[block+d].uk2=c;
						
						r = fread( &c, sizeof(c), 1, vf);	
						if (r!=1)
						{
							t = "error reading6: " + verDir + "\n";Console.send(t.c_str());
							return;
						}
						tile[block+d].quantity=c;
						
						r = fread( &sh, sizeof(sh), 1, vf);	
						if (r!=1)
						{
							t = "fatal error reading7: " + verDir + "\n";Console.send(t.c_str());
							return;
						}
						tile[block+d].appearance=sh;
						
						r = fread( &c, sizeof(c), 1, vf);	
						if (r!=1)
						{
							t = "error reading8: " + verDir + "\n";Console.send(t.c_str());
							return;
						}
						tile[block+d].uk3=c;
						
						r = fread( &c, sizeof(c), 1, vf);	
						if (r!=1)
						{
							t = "error reading9: " + verDir + "\n";Console.send(t.c_str());
							return;
						}
						tile[block+d].hue=c;
						
						r = fread( &c, sizeof(c), 1, vf);	
						if (r!=1)
						{
							t = "error reading10: " + verDir + "\n";Console.send(t.c_str());
							return;
						}
						tile[block+d].uk4=c;
						
						r = fread( &c, sizeof(c), 1, vf);	
						if (r!=1)
						{
							t = "error reading11: " + verDir + "\n";Console.send(t.c_str());
							return;
						}
						tile[block+d].value=c;
						
						r = fread( &c, sizeof(c), 1, vf);	
						if (r!=1)
						{
							t = "error reading12: " + verDir + "\n";Console.send(t.c_str());
							return;
						}
						tile[block+d].height=c;
						
						r = fread(&n, 20, sizeof(unsigned char), vf);	
						if (r!=1)
						{
							t = "error reading13: " + verDir + "\n";Console.send(t.c_str());
							return;
						}
						memcpy(&(tile[block+d].name), &n, 20);				
				  } // end of block loop
				  
				  //printf(" :%s \n",tile[block].name);
			  }
			  else if (size==(32*26+4)  && block<=511) // land patch
			  {
				  block=block*32;		
				  
				  r = fseek(vf, position+4, SEEK_SET); // skip over first redundant 4 block bytes
				  if (r!=0)
				  {
					  t = "error seeking2: " + verDir + "\n";Console.send(t.c_str());		  			  
					  return;
				  }
				  
				  for(d=0; d<32; d++)
				  {
					  r = fread( &ui, sizeof(ui), 1, vf);	
					  if (r!=1)
					  {
						  t = "error reading1: " + verDir + "\n";Console.send(t.c_str());
						  return;
					  }
					  land[block+d].flags=ui;
					  
					  r = fread( &sh, sizeof(sh), 1, vf);	
					  if (r!=1)
					  {
						  t = "error reading2: " + verDir + "\n";Console.send(t.c_str());
						  return;
					  }
					  land[block+d].textureId=sh;
					  
					  r = fread(&n, 20, sizeof(unsigned char), vf);	
					  if (r!=1)
					  {
						  t = "error reading3: " + verDir + "\n";Console.send(t.c_str());
						  return;
					  }
					  memcpy(&(land[block+d].name), &n, 20);
				  } // end block loop
			  }
			  else
			  {
				  t = "error: invalid tile patch data in verdata\n";Console.send(t.c_str());
				  return;
			  }
			  
			} else 
			{
				fseek(vf, 16, SEEK_CUR); // skip over next 16 bytes if type mismatching
			}
		}
		sprintf(temp, "%i verdata tiledata patches applied\n",b);
		Console.send(temp);
	}
	
	Console.send("Done\n");
	
	/*
	int num=100;
	printf("#%i %s %08x %i\n", num, land[num].name, land[num].flags, ((num>>5)+1)*4 + num*26);
	printf("#%i %s %08x %i\n", num, tile[num].name, tile[num].flags, 0x00068800 + ((num>>5)+1)*4 + num*37);
	*/
	
	if (ai) fclose(ai);
	if (vf) fclose(vf);
	if (tf) fclose(tf);
}
