/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

#if !defined(__TILEDATA_H__)
#define __TILEDATA_H__

#include "platform.h"
#include "typedefs.h"
#include "wolfpack.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "tiledata.h"


// client: only makes sense client side
// server: coulde be used client side XOR server side
// server,used: we are currently using this bit in server
// server w/o used means: could be evaluated, but currently isn't
// anyway, only use those that have an explicit method like TileInfo::isWet(...)

#define	FLOOR			0x00000001 // background (server, used)
#define	EQUIPABLEA		0x00000002 // weapon (client)
#define	NONBLOCKING		0x00000004 // transparent (client)
#define	LIQUID			0x00000008 // translucent (client)
#define	WALL			0x00000010 // (server)
#define DAMAGEING		0x00000020 // (server)
#define	BLOCKING		0x00000040 // (server, used)
#define	WATER			0x00000080 // (server, used)

#define	UNKNOWNA		0x00000100
#define	PLATFORM		0x00000200 // surface, (server,used)
#define	CLIMBABLE		0x00000400 // ladder/staircase !!!! (bridge??), server(used)
#define	STACKABLE		0x00000800 // generic as well (server,used)
#define	WALL_WINDOW		0x00001000 // server
#define	BREAK_LOS		0x00002000 // no shoot! (server)
#define	ARTICLE_A		0x00004000 // server (used)
#define	ARTICLE_AN		0x00008000 // server (used)

#define	DESCRIPTION		0x00010000 // internal (beards, corpses,... ) client
#define	SEE_THROUGH		0x00020000 // foliage !! (client)
#define	MATERIAL		0x00040000 // partial hue ! client
#define	UNKNOWNC		0x00080000 // 
#define	ISAMAP			0x00100000 // client
#define	CONTAINER		0x00200000 // client
#define	EQUIPABLEB		0x00400000 // client
#define	LIGHTSOURCE		0x00800000 // served,used

#define	ANIMATION		0x01000000 // client
#define	UNKNOWND		0x02000000 // no diagonal !
#define	UNKNOWNE		0x04000000 //
#define	BODYITEM		0x08000000 // armor, client
#define	ROOF			0x10000000 // server
#define	DOOR			0x20000000 // server (used)
#define	STAIRSBACK		0x40000000 // stairback (server)
#define	STAIRSRIGHT		0x80000000 // stairright  (stairback/stairright -> use 2 bits as stairs direction, N,S,W,E) (server)

struct landData
{
   unsigned int flags;
   unsigned short textureId;
   unsigned char name[20];
   bool invalid;
};

struct tileData
{
	// storing unknows intentionaly, because just a few kb waste & perhaps someone figures them out later
	// packing not worth troubles as well. (1.1MB versus 1.0MB)
	unsigned int   flags;
	unsigned char  weight;
	unsigned char  quality;
    unsigned short uk1;
	unsigned char  uk2;
    unsigned char  quantity;
	unsigned short appearance;
    unsigned char  uk3;
	unsigned char  hue;
	unsigned char  uk4;
	unsigned char  value;
	signed char    height;
	unsigned char  name[20];
	bool invalid;
};

class tileInfo
{
   private:
	tileData tile[0x4000];
	landData land[0x4000];
	int error;

	inline bool checkLandnum(unsigned short ln)
	{
		if (ln>=0x4000)
		{
		   LogErrorVar("Invalid Seekland# %i\n",ln);
		   return false;
		}
		   
		if (land[ln].invalid)
		{
			if (SrvParms->logmode&2)
			{
				sprintf(temp, "Empty Seekland tile# %i\n",ln);
				Console.send(temp);
				LogWarning(temp);
			}
			return false;
		}
		
		return true;
	}

	inline bool checkTilenum(unsigned short tn)
	{
	   	if (tn>=0x4000)
		{
		   LogErrorVar("Invalid Seekland# %i\n",tn);
		   return false;
		}
		   
		if (tile[tn].invalid)
		{
			if (SrvParms->logmode&2)
			{
				sprintf(temp, "Empty Seekland tile# %i\n",tn);
				Console.send(temp);
				LogWarning(temp);
			}
			return false;
		}
		return true;
	}

   public:

	   tileInfo::tileInfo(std::string basedir);
	   tileInfo::~tileInfo(void) { return; }
	   int getError(void) { return error; }

	   /// all data at once getters ... (not recommanded, but there might be situations where you need it)
	   /// in particular needed in translation phase from old code to new once.
	   const landData* tileInfo::SeekLand(unsigned short landnum) 
	   {
		   if (landnum>=0x4000)
		   {			 
			   sprintf(temp, "Invalid SeekLand# %i\n", landnum);
			   LogError(temp);
			   return &land[0];
		   }
		   
		   if (land[landnum].invalid)
		   {
			   if (SrvParms->logmode&2)
			   {
				   sprintf(temp, "Empty SeekLand# %i\n", landnum);
				   Console.send(temp);
				   LogWarning(temp);
			   }
		   }

		   return &land[landnum]; 
	   }

	   const tileData* tileInfo::SeekTile(unsigned short tilenum)
	   {
		   if (tilenum>=0x4000)
		   {
			   sprintf(temp,"Invalid SeekTile# %i\n",tilenum);
			   //LogError(temp);
			   return &tile[0];
		   }
		   
		   if (tile[tilenum].invalid)
		   {
			   if (SrvParms->logmode&2)
			   {
				   sprintf(temp,"Empty SeekTile# %i\n",tilenum);			  
				   Console.send(temp);
				   LogWarning(temp);
			   }
		   }
		   
		   return &tile[tilenum]; 
	   }
	   
	   // recommanded

	   inline bool isTileOk(unsigned short tn) { if (tn>=0x4000) return false; if (tile[tn].invalid) return false; return true; }

	   /// WET
	   inline bool isLandWet(unsigned short landnum) { if (checkLandnum(landnum)) return isWet(land[landnum].flags); else return false; }
	   inline bool isTileWet(unsigned short tilenum) { if (checkTilenum(tilenum)) return isWet(tile[tilenum].flags); else return false; }
	   inline bool isWet(unsigned int flag)          { if (flag&WATER) return true; else return false; }
	   
	   ///STACKABLE	  
	   inline bool isTileStackable(unsigned short tilenum) { if (checkTilenum(tilenum)) return isStackable(tile[tilenum].flags); else return false; }
	   inline bool isStackable(unsigned int flag)          { if (flag&STACKABLE) return true; else return false; }

	   ///an
	   inline bool isLandAn(unsigned short landnum)  { if (checkLandnum(landnum)) return isAn(land[landnum].flags); else return false; }
	   inline bool isTileAn(unsigned short tilenum)  { if (checkTilenum(tilenum)) return isAn(tile[tilenum].flags); else return false; }
	   inline bool isAn(unsigned int flag)           { if (flag&ARTICLE_AN) return true; else return false; }

	   ///a
	   inline bool isLandA(unsigned short landnum)  { if (checkLandnum(landnum)) return isA(land[landnum].flags); else return false; }
	   inline bool isTileA(unsigned short tilenum)  { if (checkTilenum(tilenum)) return isA(tile[tilenum].flags); else return false; }
	   inline bool isA(unsigned int flag)           { if (flag&ARTICLE_A) return true; else return false; }

	   /// stair-direction
	   inline signed char TileStairsDir(unsigned short tilenum)  { if (checkTilenum(tilenum)) return StairsDir(tile[tilenum].flags); else return false; }
	   inline signed char StairsDir(unsigned int flag)           
	   { 
		   char d,e;
		   if (flag&STAIRSBACK)  d=1;  else d=0; 
		   if (flag&STAIRSRIGHT) e=1;  else e=0;   
		   if      (d==0 && e==0) return -1; // no direction / stairs
		   else if (d==0 && e==1) return 0;
		   else if (d==1 && e==0) return 1;
		   else if (d==1 && e==1) return 2;
		   return -1;
	   }

	   /// floor
	   inline bool isLandFloor(unsigned short landnum)  { if (checkLandnum(landnum)) return isFloor(land[landnum].flags); else return false; }
	   inline bool isTileFloor(unsigned short tilenum)  { if (checkTilenum(tilenum)) return isFloor(tile[tilenum].flags); else return false; }
	   inline bool isFloor(unsigned int flag)           { if (flag&FLOOR) return true; else return false; }

	   /// block
	   inline bool isLandBlocking(unsigned short landnum)  { if (checkLandnum(landnum)) return isBlocking(land[landnum].flags); else return false; }
	   inline bool isTileBlocking(unsigned short tilenum)  { if (checkTilenum(tilenum)) return isBlocking(tile[tilenum].flags); else return false; }
	   inline bool isBlocking(unsigned int flag)           { if (flag&BLOCKING) return true; else return false; }

	   /// climbable
	   inline bool isLandClimbable(unsigned short landnum)  { if (checkLandnum(landnum)) return isClimbable(land[landnum].flags); else return false; }
	   inline bool isTileClimbable(unsigned short tilenum)  { if (checkTilenum(tilenum)) return isClimbable(tile[tilenum].flags); else return false; }
	   inline bool isClimbable(unsigned int flag)           { if (flag&CLIMBABLE) return true; else return false; }

	   /// Door
	   inline bool isLandDoor(unsigned short landnum)  { if (checkLandnum(landnum)) return isDoor(land[landnum].flags); else return false; }
	   inline bool isTileDoor(unsigned short tilenum)  { if (checkTilenum(tilenum)) return isDoor(tile[tilenum].flags); else return false; }
	   inline bool isDoor(unsigned int flag)           { if (flag&DOOR) return true; else return false; }

	   /// Platform
	   inline bool isLandPlatform(unsigned short landnum)  { if (checkLandnum(landnum)) return isPlatform(land[landnum].flags); else return false; }
	   inline bool isTilePlatform(unsigned short tilenum)  { if (checkTilenum(tilenum)) return isPlatform(tile[tilenum].flags); else return false; }
	   inline bool isPlatform(unsigned int flag)           { if (flag&PLATFORM) return true; else return false; }	  

	   /// damageing
	   inline bool isLandDamaging(unsigned short landnum)  { if (checkLandnum(landnum)) return isDamaging(land[landnum].flags); else return false; }
	   inline bool isTileDamaging(unsigned short tilenum)  { if (checkTilenum(tilenum)) return isDamaging(tile[tilenum].flags); else return false; }
	   inline bool isDamaging(unsigned int flag)           { if (flag&DAMAGEING) return true; else return false; }

	   /// wall
	   inline bool isLandWall(unsigned short landnum)  { if (checkLandnum(landnum)) return isWall(land[landnum].flags); else return false; }
	   inline bool isTileWall(unsigned short tilenum)  { if (checkTilenum(tilenum)) return isWall(tile[tilenum].flags); else return false; }
	   inline bool isWall(unsigned int flag)           { if (flag&WALL) return true; else return false; }

	   /// break LOS
	   inline bool isLandBreakLos(unsigned short landnum)  { if (checkLandnum(landnum)) return isBreakLos(land[landnum].flags); else return false; }
	   inline bool isTileBreakLos(unsigned short tilenum)  { if (checkTilenum(tilenum)) return isBreakLos(tile[tilenum].flags); else return false; }
	   inline bool isBreakLos(unsigned int flag)           { if (flag&BREAK_LOS) return true; else return false; }

	   /// roof
	   inline bool isLandRoof(unsigned short landnum)  { if (checkLandnum(landnum)) return isRoof(land[landnum].flags); else return false; }
	   inline bool isTileRoof(unsigned short tilenum)  { if (checkTilenum(tilenum)) return isRoof(tile[tilenum].flags); else return false; }
	   inline bool isRoof(unsigned int flag)           { if (flag&ROOF) return true; else return false; }

	   /// window
	   inline bool isLandWindow(unsigned short landnum)  { if (checkLandnum(landnum)) return isWindow(land[landnum].flags); else return false; }
	   inline bool isTileWindow(unsigned short tilenum)  { if (checkTilenum(tilenum)) return isWindow(tile[tilenum].flags); else return false; }
	   inline bool isWindow(unsigned int flag)           { if (flag&WALL_WINDOW) return true; else return false; }
		///LightSource
	   inline bool isLightSourceItem(short id) { return isLightSource(SeekTile(id)->flags);    }
	   inline bool isLightSource(unsigned int flag)		{ return (flag&LIGHTSOURCE ? true:false); }

};

#endif
