/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

// *******************************************************************
// trigger.cpp (Trigger processing module)
//
// The trigger is activated here.  When the users doubleclicks on a triggered
// item or a triggered NPC, this routine will load the trigger from the
// triggers.scp (for dynamic), ntrigrs.scp (for NPCs) file or wtrigrs.scp
// (static world objects) and process the tokens for the trigger.
// NOTE: By using this source you are agreeing to all the terms of the GNU
// agreement.  Any use of this code, sniplets of this code, concepts of
// this code or ideas derived from this code requires that any program(s)
// and source containing any of the above mentioned states of the code, must
// be released in its entirety.
//
// DO NOT MODIFY THIS CODE UNLESS you are sure you know what your doing.
// The trigger system is not designed for making tokens that do just one
// thing.  Remember, if it can't be used for multiple things, it shouldnt
// be here. Do not distribute your changes unless its either by consent of
// the author(Zadius) or if the change and/or addition is by popular
// demand and it has been thouroughly tested.
// 
// Zadius 11-4-98 (linkage@xmission.com)
// *******************************************************************


#include "trigger.h"

#include "Client.h"
#include "gumps.h"
#include "magic_new.h"
#include "mapstuff.h"
#include "network.h"
#include "regions.h"
#include "skills.h"
#include "SndPkg.h"
#include "teleport.h"
#include "tiledata.h"
#include "walking.h"
#include "weight.h"

#include "wolfpack.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "trigger.cpp"

class cTriggerContext   // to store data that is passed between trigger-tags 
{ 
public: 
	P_ITEM EnvItem; 
	P_ITEM TrigItem; 
	P_CHAR TrigChar; 
	P_ITEM NeedItem; 
	P_ITEM SpawnedItem; 
	P_CHAR Spawned;      // npc spawned by NADD 
	int coloring; 
	int FollowupTrigger; 
	short memColor; 
	char cmsg[512];      // completed trigger message 
	char fmsg[512];      // trigger failure message 
	char tempname2[512]; 
	char needname[100];   // name of the NEEDed item 
	
	void Init() 
	{ 
		EnvItem = NULL; 
		TrigItem = NULL; 
		TrigChar = NULL; 
		NeedItem = NULL; 
		SpawnedItem = NULL; 
		Spawned = NULL; 
		coloring = -1; 
		FollowupTrigger = 0; 
		memColor = 0; 
		cmsg[0] = 0; 
		fmsg[0] = 0; 
		tempname2[0] = 0; 
		needname[0] = 0; 
	} 
};


////////////// 
// name:   handle_ADVANCE 
// purpose:   handles the ADVANCE tag which fires just like the player had walked into an advancement gate. 
// history:   written makes the advance gate system more flexible (sayro 2003-06-21) 
// 
static void handle_ADVANCE(UOXSOCKET const ts, P_CHAR pc_ts, Script* Scp, cTriggerContext& TC) 
{ 
	cline = &script2[0]; 
	splitline(); 
	scpMark m=Scp->Suspend(); 
	advancementobjects(pc_ts, makenumber(0), makenumber(1)); 
	Scp->Resume(m); 
	strcpy(script1, "DUMMY"); 
}

//////////////
// name:	handle_BOLT
// purpose:	handles the BOLT tag ie. displays a given effect (bolt effect is default)
// history:	by Sayro, Jan 2003)
//
static void handle_BOLT(UOXSOCKET const ts, P_CHAR pc_ts, cTriggerContext& TC) 
{
	cline = &script2[0] ;
	splitline();
	if (tnum)
	{ // if any ID were given
		int boltamount = makenumber(2);
		if (tnum<=2) boltamount=333;
		for (int j=0;j<=boltamount;j++) 
		{
			bolteffect2(pc_ts,hexnumber(0), hexnumber(1));
		}
	} else
	{ // if no ID were given
		bolteffect(pc_ts, true);
		soundeffect2(pc_ts, 0x0029);
	}
}

//////////////
// name:	handle_EVDUR
// purpose:	handles the EVDUR tag ie. changes the HP of the envoking item
// history:	cut from both triggerwitem and triggernpcs because they were (almost) identical (Duke, 8.4.2003)
//
static void handle_EVDUR(UOXSOCKET const ts, cTriggerContext& TC) 
{
	if (!TC.EnvItem) return;
	
	cline = &script2[0];
	splitline();
	int j = makenumber(0);
	int p = makenumber(1);
	if (p <= 0)
		p = 100;
	
	int c = TC.EnvItem->hp;
	if (TC.EnvItem->maxhp>0)
	{
		if ((rand()%(100)) + 1 <= p)
		{
			if ((c >= TC.EnvItem->maxhp) && j>0)
			{
				sprintf(temp,"Your %s is already totally repaired!", TC.tempname2);
				sysmessage(ts, temp);
			}
			TC.EnvItem->hp += j;
			if (TC.EnvItem->hp >= TC.EnvItem->maxhp)
				TC.EnvItem->hp = TC.EnvItem->maxhp;
			if (TC.EnvItem->hp - c>0)
			{
				if (strlen(TC.cmsg))
					sysmessage(ts, TC.cmsg);
				else 
				{
					float total = (float) TC.EnvItem->hp/TC.EnvItem->maxhp;
					sprintf(temp,"Your %s is now repaired! [%.1f%%]", TC.tempname2, total*100);
					sysmessage(ts, temp);
				}
			}
			else if (TC.EnvItem->hp - c < 0)
			{
				if (strlen(TC.fmsg))
					sysmessage(ts, TC.fmsg);
				else 
				{
					sprintf(temp,"Your %s appears to be more ruined than before!", TC.tempname2);
					sysmessage(ts, temp);
				}
			}
			if (TC.EnvItem->hp <= 0)
			{
				sprintf(temp,"Your %s was too old and it has been destroyed!", TC.tempname2);
				sysmessage(ts, temp);
				TC.EnvItem->ReduceAmount(1);
			}
		}
	}
	else 
	{
		sprintf(temp,"You can't repair %s !", TC.tempname2);
		sysmessage(ts, temp);
	}
}

//////////////
// name:	handle_EVMAXDUR
// purpose:	handles the EVMAXDUR tag ie. changes the durability of the envoking item
// history:	cut from both triggerwitem and triggernpcs because they were (almost) identical (Duke, 8.4.2003)
//
static void handle_EVMAXDUR(UOXSOCKET const ts, cTriggerContext& TC) 
{
	if (!TC.EnvItem) return;	// no item to change durability

	P_ITEM pi = TC.EnvItem;
	cline = &script2[0];
	splitline();
	int j = makenumber(0);
	int p = makenumber(1);
	if (p <= 0)
		p = 100;

	if (pi->maxhp>0)
	{
		if ((rand()%(100)) + 1 <= p)
		{
			pi->maxhp += j; // Magius(CHE) 
			if (pi->hp >= pi->maxhp)
				pi->hp = pi->maxhp;
			if (str2num(script2) >= 0)
			{
				if (strlen(TC.cmsg))
					sysmessage(ts, TC.cmsg);
				else 
				{
					sprintf(temp,"You increased the maximum durability of your %s !", TC.tempname2);
					sysmessage(ts, temp);
				}
			}
			else 
			{
				if (strlen(TC.fmsg))
					sysmessage(ts, TC.fmsg);
				else 
				{
					sprintf(temp,"Your %s appears to be not resistant as before!", TC.tempname2);
					sysmessage(ts, temp);
				}
			}
			if (pi->maxhp <= 0)
			{
				sprintf(temp,"Your %s was too old and it has been destroyed!", TC.tempname2);
				sysmessage(ts, temp);
				pi->ReduceAmount(1);
			}
		}
	}
	else 
	{
		sprintf(temp,"You can't increase durability of %s !", TC.tempname2);
		sysmessage(ts, temp);
	}
}

////////////// 
// name:   handle_IADD 
// purpose:   handles the IADD tag ie. adds an item 
// history:   cut from both triggerwitem and triggernpcs because they were (almost) identical (duke, 8.4.2001) 
//      Rewritten to allow atLocation & relativeToPlayer (Sayro, 2003-07-18) 
// remarks: many other tags seem to be identical... 
// 
static void handle_IADD(UOXSOCKET const ts, P_CHAR pc_ts, Script* Scp, cTriggerContext& TC) 
{ 
	enum addways {onGround, inBackPack, atLocation, relativeToPlayer}; 
	cline = &script2[0]; 
	splitline(); 
	unsigned int ItemNum = makenumber(0); 
	addways addHow = (addways)makenumber(1); 
	int x = makenumber(2), y = makenumber(3); 
	signed char z = makenumber(4); 
	if (ItemNum<1) 
	{ 
		sysmessage(ts, "Contact GM: Invalid Item Nr in IADD"); 
		return; 
	} 
	if (addHow<0 || addHow>3) 
	{ 
		sysmessage(ts, "Contact GM: Invalid IADD Type"); 
		return; 
	} 
	if (addHow==onGround) 
	{ 
		x = pc_ts->pos.x; 
		y = pc_ts->pos.y; 
		GetXYfromDir(pc_ts->dir, &x, &y); 
		z = pc_ts->pos.z; 
	} 
	else if (addHow==relativeToPlayer) 
	{ 
		x += pc_ts->pos.x; 
		y += pc_ts->pos.y; 
		z += pc_ts->pos.z; 
	} 
	
	scpMark m=Scp->Suspend(); 
	// first create the item on the ground 
	P_ITEM iItem=Items->CreateScriptItem(ts, ItemNum, 0); 
	// then shove it where it should go 
	
	if (iItem!=NULL) 
	{ 
		if (addHow != inBackPack) 
			iItem->MoveTo(x,y,z); 
		else 
		{ 
			P_ITEM iPack = Packitem(pc_ts); 
			iPack->AddItem(iItem); 
			Weight->NewCalc(pc_ts); 
			statwindow(ts, pc_ts); 
		} 
		if (TC.coloring>-1) // Added colormem token here! by Magius(CHE)  
		{ 
			iItem->setColor(TC.memColor); 
		} 
		if (addHow != inBackPack) iItem->refresh(); // if in packpack it already got refreshs ! 
	} 
	// end addons 
	TC.SpawnedItem=iItem; 
	
	Scp->Resume(m); 
	strcpy(script1, "DUMMY"); 
}

////////////// 
// name:	handle_IFRACE 
// purpose: handles the IFRACE tag ie. checks players race 
// history: written for race based triggers & craftables systems (sayro 2003-06-17) 
// 
static int handle_IFRACE(UOXSOCKET const ts, P_CHAR pc_ts, Script* Scp, cTriggerContext& TC) 
{ 
	cline = &script2[0] ; 
	splitline(); 
	unsigned int race = makenumber(0); 
	int followUpTrigger = makenumber(1); 
	if (tnum>1 && followUpTrigger<1) 
	{ 
		sysmessage(ts, "Contact GM: Invalid Trigger Nr in IFRACE"); 
		return 2; 
	} 
	if ( (race >= 0 && race != pc_ts->getRace()) || 
		(race < 0  && (unsigned)::abs((int)race) == pc_ts->getRace()) ) // note: the sligtly pervert (unsinged)::(int) constructon is necassairy to compile it under gcc 
	{ 
		if (tnum==1) // only message+fail if not a list of cases with followups 
		{ 
			if (strlen(TC.fmsg)) 
				sysmessage(ts, TC.fmsg); 
			else 
				sysmessage(ts, "Your race have never been able to do that."); 
			return 2; 
		} else return 1; 
	}
	else
	{ 
		if (tnum>1) 
		{ 
			if (TC.TrigItem) 
			{ 
				pc_ts->targtrig = followUpTrigger;
				Scp->Close(); 
				triggerwitem(ts, TC.TrigItem, 1); 
				return 3; 
			} 
			if (TC.TrigChar) 
			{ 
				TC.FollowupTrigger = followUpTrigger;
				Scp->Close(); 
				triggernpc(ts, TC.TrigChar, 2); 
				return 3; 
			} 
		} else return 1; 
	} 
	return 1; 
}

////////////// 
// name: handle_IFREGION 
// purpose: handles the IFREGION tag ie. multiple options of checks for what region the player is in 
// history: written to try out the minetriggers, thus we can check if they are in a certain mine (sayro 2003-07-07) 
// 
static int handle_IFREGION(UOXSOCKET const ts, P_CHAR pc_ts, Script* Scp, cTriggerContext& TC) 
{ 
	cline = &script2[0] ; 
	splitline(); 
	int region = makenumber(0); 
	int followUpTrigger = makenumber(1); 
	int inregion = calcRegionFromXY(pc_ts->pos); 
	if (tnum>1 && followUpTrigger<1) 
	{ 
		sysmessage(ts, "Contact GM: Invalid Trigger Nr in IFREGION"); 
		return 2; 
	} 
	if ( (region >= 0 && region != inregion) || 
		(region < 0  && abs(region) == inregion) ) 
	{ 
		if (tnum==1) // only message+fail if not a list of cases with followups 
		{ 
			if (strlen(TC.fmsg)) 
				sysmessage(ts, TC.fmsg); 
			else 
				sysmessage(ts, "Can't be done here."); 
			return 2; 
		}
		else return 1; 
	}
	else
	{ 
		if (tnum>1) 
		{ 
			if (TC.TrigItem) 
			{ 
				pc_ts->targtrig = followUpTrigger; 
				Scp->Close(); 
				triggerwitem(ts, TC.TrigItem, 1); 
				return 3; 
			} 
			if (TC.TrigChar) 
			{ 
				TC.FollowupTrigger = followUpTrigger; 
				Scp->Close(); 
				triggernpc(ts, TC.TrigChar, 2); 
				return 3; 
			} 
		}
		else return 1; 
	} 
	return 1; 
}

//////////////
// name:   handle_MAKEPLAYERCRIMINAL
// purpose:   handles the MAKEPLAYERCRIMINAL tag
// history:   by Sayro, 2003-08-19
//
static void handle_MAKEPLAYERCRIMINAL(UOXSOCKET const ts, P_CHAR pc_ts)
{
	cline = &script2[0];
	splitline();
	criminal(pc_ts);
	if (tnum) // if any custom timelimit is given
		pc_ts->crimflag = (makenumber(0)*MY_CLOCKS_PER_SEC)+uiCurrentTime;
}

//////////////
// name:	handle_NEED
// purpose:	handles the NEED tag ie. checks if the player has an item of suitable ID in his backpack
// history:	cut from both triggerwitem and triggernpcs because they were (almost) identical (duke, 28.4.2003)
//
static int handle_NEED(UOXSOCKET const ts, P_CHAR pc_ts, cTriggerContext& TC) 
{
	P_ITEM Pack = pc_ts->getBackpack();
	if (Pack)
	{
		int ci=0,loopexit=0;
		P_ITEM pin;
		while ( ((pin=ContainerSearch(Pack->serial,&ci)) != NULL) && (++loopexit < MAXLOOPS) )
		{
			char idstr[20];
			sprintf(idstr, "x%x", pin->id());
			if (strstr(script2, idstr))
			{
				TC.NeedItem = pin;
				if (pin->name[0] != '#')	// Get Temporary Name of the NEED Item - Magius(CHE) 
					strcpy(TC.needname, pin->name);
				else 
				{
					const tileData *tile = TileData->SeekTile(pin->id());
					strcpy(TC.needname, (char*)tile->name);
				}
				break;
			}
		}
	}
	
	if (!TC.NeedItem)
	{
		if (strlen(TC.fmsg))
			sysmessage(ts, TC.fmsg); // Added by Magius(CHE)
		else 
			sysmessage(ts, "You don't have the neccessary supplies to do that.");
		return 2;	// abort the trigger
	}
	return 1;
}

//////////////
// name:	handle_PUT
// purpose:	the PUT tag teleports the player to the place given
// history:	cut from both triggerwitem and triggernpcs because they were (almost) identical (Duke, 13.9.2003)
//
static int handle_PUT(P_CHAR pc_ts, Script* Scp)
{
	cline = &script2[0];
	splitline();
	
	scpMark m=Scp->Suspend();	// teleTo might open scripts
	cCoord newPos(makenumber(0),makenumber(1),makenumber(2),DEFAULT_MAP);
	pc_ts->teleTo(newPos);
	Scp->Resume(m);
	strcpy(script1, "DUMMY");
	return 1;
}

//////////////
// name:	handle_REQ
// purpose:	handles the REQ tag ie. checks if the trigger was envoked by an item of the given ID
// history:	cut from both triggerwitem and triggernpcs because they were (almost) identical (Duke, 28.4.2003)
//
static int handle_REQ(UOXSOCKET const ts, P_CHAR pc_ts, cTriggerContext& TC) 
{
	if (TC.EnvItem)
	{
		char idstr[20];
		sprintf(idstr, "x%x", TC.EnvItem->id());
		if (strstr(script2, idstr))
		{
			if (TC.EnvItem->name[0] != '#') // Get Temporany Name of the REQUIRED Item - Magius(CHE)
				strcpy(TC.tempname2, TC.EnvItem->name);
			else
			{
				const tileData *tile = TileData->SeekTile(TC.EnvItem->id());
				strcpy(TC.tempname2, (char*)tile->name);
			}
			return 1;
		}
	}
	sysmessage(ts, "That didn't seem to work.");
	return 2;
}

//////////////
// name:	handle_SKL
// purpose:	handles the SKL tag ie. does math on the players skill
// history:	cut from both triggerwitem and triggernpcs because they were (almost) identical (Duke, 18.4.2004)
//
static int handle_SKL(UOXSOCKET const ts, P_CHAR pc_ts) 
{
	cline = &script2[0];
	splitline();
	int p = makenumber(0);
	int j = makenumber(1);
	
	if (j)
	{
		pc_ts->chgRealSkill(p, j);
		SkillManager->AdvanceStats(pc_ts, p);
	}
	else 
	{
		SkillManager->AdvanceSkill(pc_ts, p, 1);
	}
	SkillManager->updateSkillLevel(pc_ts, p);
	updateskillPacket(ts, p);
	return 1;
}

//////////////
// name:	handle_USEUP
// purpose:	handles the USEUP tag ie. checks if the trigger was envoked by an item of the given ID
// history:	cut from both triggerwitem and triggernpcs because they were (almost) identical (Duke, 16.6.2003)
//
static int handle_USEUP(UOXSOCKET const ts, P_CHAR pc_ts, cTriggerContext& TC) 
{
	if (!TC.NeedItem)
	{
		P_ITEM Pack = Packitem(pc_ts);
		if (Pack)
		{
			int ci=0,loopexit=0;
			P_ITEM pin;
			while ( ((pin=ContainerSearch(Pack->serial,&ci)) != NULL) && (++loopexit < MAXLOOPS) )
			{
				char idstr[20];
				sprintf(idstr, "x%x", pin->id());
				if (strstr(script2, idstr))
				{
					TC.NeedItem = pin;
					break;
				}
			}
		}
	}
	if (!TC.NeedItem)
	{
		sysmessage(ts, "It appears as though you have insufficient supplies to make that with.");
		return 2;
	}
	else
	{
		TC.NeedItem->ReduceAmount(1);
		return 1;
	}
}

/////////////
// name:	handle_CommonTags
// purpose: handle the tags that are supported in both ntriggers and triggers.scp
// history:	by Duke, 27.3.2003
//
int handle_CommonTags(UOXSOCKET const ts, P_CHAR pc_ts, cTriggerContext& TC, Script* Scp)
{
	
	if (!strcmp("ADVANCE", script1))	// ADVANCE #advancenr #Always(0/1) 
	{ 
		handle_ADVANCE(ts,pc_ts,Scp,TC); 
		return 1; 
	} 
	else if (!strcmp("BOLT", script1))  // BOLT #ID #ID #Amount
	{
		handle_BOLT(ts,pc_ts,TC);
		return 1;
	}
	else if (!strcmp("EVDUR", script1))  // Math on Evoked item HP ---- Rewrite by Magius(CHE) 
	{
		handle_EVDUR(ts,TC);
		return 1;
	}
	else if (!strcmp("EVMAXDUR", script1))  // Math on Evoked item MAXHP ---- Rewrite by Magius(CHE) 
	{
		handle_EVMAXDUR(ts,TC);
		return 1;
	}
	else if (!strcmp("IADD", script1))  // Add item in front of player //if 2nd param. is 1, add item into player's backpack - AntiChrist (with autostack)
	{
		handle_IADD(ts, pc_ts, Scp, TC);
		return 1;
	}
	else if (!strcmp("IFRACE", script1)) // Check if player is a certain race - Sayro
	{
		return handle_IFRACE(ts, pc_ts, Scp, TC);
	}
	else if (!strcmp("IFREGION", script1)) // Check if player is in a certain region - Sayro
	{
		return handle_IFREGION(ts, pc_ts, Scp, TC);
	}
	else if (!strcmp("MAKEPLAYERCRIMINAL", script1)) // makes player criminal, for srvparam or custom time - Sayro
	{
		handle_MAKEPLAYERCRIMINAL(ts, pc_ts);
		return 1;
	}
	else if (!strcmp("NEED", script1))  // The item here is required and will be removed
	{
		return handle_NEED(ts,pc_ts,TC);
	}
	else if (!strcmp("PUT", script1))	// teleport player
	{
		return handle_PUT(pc_ts,Scp);
	}
	else if (!strcmp("REQ", script1))	// Check if envoked by certain item.
	{
		return handle_REQ(ts,pc_ts,TC);
	}
	else if (!strcmp("SKL", script1))	// do math on skill
	{
		return handle_SKL(ts,pc_ts);
	}
	else if (!strcmp("TPLOADGUMP", script1)) // sends the tithingpoints gump
	{
		Magic->ShowTithingGump(pc_ts);	// show the tithing gump
		return 1;
	}
	else if (!strcmp("USEUP", script1))	// Check for and delete 'needed' item.
	{
		return handle_USEUP(ts,pc_ts,TC);
	}
	else if (!strcmp("WEBLAUNCH", script1))	// Launches browser and sends to scripted URL. - Sayro
	{
		weblaunch(ts, script2);
		return 1;
	}
	return 0;	// not handled
}

// This routine processes both Static and Dynamic items
// ts: socket
void triggerwitem(UOXSOCKET const ts, P_ITEM Ti, int ttype)
{
	cTriggerContext TC;
	TC.Init();

	char sect[512], buff[512];
	unsigned int i, uiCompleted = 0;
	int tl;
	int p, j, c=0, r;
	int trig = 0;
	int x1 = 0, y1 = 0, x2 = 0, y2 = 0, z2 = 0, dx, dy;
	
	// Addons by Magius(CHE)
	const tileData *tile;
	char tempname[512], tempstr[512];
	tempname[0] = 0;
	tempstr[0] = 0;
	float total; // Magius(CHE) 
	char dismsg[512]; // Magius(CHE) 
	dismsg[0] = 0; // Magius(CHE) 
	int loopexit = 0;
	
	if (ts < 0)
		return; // Magius crash fix
	
	P_ITEM pi = NULL;	// on error return
	if (Ti)
	{
		TC.TrigItem=pi=Ti;	// there may be some cases where Ti=NULL is valid (Duke)
	}

	P_CHAR pc_ts = MAKE_CHARREF_LR(currchar(ts));
	TC.EnvItem = FindItemBySerial(pc_ts->dclickedserial);
	if (!TC.TrigItem) 
		TC.TrigItem = FindItemBySerial(pc_ts->dclickedserial); 
	
	Script* Scp = NULL;
	if (ttype == 1)
	{
		if (pc_ts->targtrig)
			trig = pc_ts->targtrig;
		else
		{
			if (pi)
				trig = pi->trigger;
		}
		pc_ts->targtrig = 0;

		Scp=findSection(trigger_script,"TRG",trig);		// open file and find section
		if (!Scp) return;
	}
	else
	{
		Scp=i_scripts[wtrigger_script];
		Scp->Open();
		sprintf(buff, "x%x%x", buffer[ts][0x11], buffer[ts][0x12]);
		if (!Scp->isin(buff))
		{
			strcpy(sect, "You cant think of a way to use that.");
			sysmessage(ts, sect);
			Scp->Close();
			return;
		}
	}
	
	// Get Temporany Name of the Item (Magius(CHE))
	if (pi) // crashfix LB
	{
		if (pi->name[0] != '#')
			strcpy(tempname, pi->name);
		else
		{
			tile = TileData->SeekTile(pi->id());
			strcpy(tempname,(char*)tile->name);
		}
		
		if (pi->disabled>uiCurrentTime) // Added by Magius(CHE) 
		{
			if (pi->TrigInf && strlen(pi->TrigInf->disabledmsg)>0)
				strcpy(temp, pi->TrigInf->disabledmsg);// AntiChrist
			else 
				strcpy(temp, "That doesnt seem to work right now.");
			sysmessage(ts, temp);
			Scp->Close();
			return;
		}
	}
	// End addons by magius
	
	while (Scp->NextLineSplitted() && (++loopexit < MAXLOOPS))
	{
		int rc = handle_CommonTags(ts,pc_ts,TC,Scp);
		if (rc==1) continue;	// handled !!
		if (rc==2)
		{
			Scp->Close();		// abort the trigger
			return;
		}
		if (rc==3) return;		// aborted and already closed

		switch (script1[0])
		{
		case 'A':
			if (!strcmp("ACT", script1))  // Make player perform an action
			{
				cline = &script2[0];
				splitline();
				int act = hexnumber(0);// AntiChrist
				// AntiChrist - auto-convert code of animation
				// that automatically check if player is on a horse
				switch (act)
				{
				case 0x1A:// Mining-Gravedigging
				case 0x0B:
					if (pc_ts->onhorse)
						act = 0x1A;
					else 
						act = 0x0b;
					break;
				case 0x1C:// LumberJacking-Bowcraft
				case 0x0D:
					if (pc_ts->onhorse)
						act = 0x1C;
					else 
						act = 0x0D;
					break;
				case 0x1D:// Swordtarget
					// case 0x0D:
					if (pc_ts->onhorse)
						act = 0x1D;
					else 
						act = 0x0D;
					break;
				case 0x0A:// Fist Fighting
					if (pc_ts->onhorse)
						act = 0x1A;
					else 
						act = 0x0A;
					break;
				case 0x0E:// Smelting irons
					if (pc_ts->onhorse)
						act = 0x1C;
					else 
						act = 0x0E;
					break;
				case 0x09:// Working ingots
					if (pc_ts->onhorse)
						act = 0x1A;
					else 
						act = 0x09;
					break;
				case 0x14:// These can be done only if not onhorse
				case 0x22:
					if (pc_ts->onhorse)
						act = 0x00;
					break;
				default:
					break;
				}
				if (act>0)
					pc_ts->DoAction(act);
			}
			break;
		case 'C':
			if (!strcmp("CMSG", script1))  // Set Token Completed message
			{
				if (uiCompleted)
				{
					strcpy(TC.cmsg, script2);
					if (strlen(TC.cmsg))
						sysmessage(ts, TC.cmsg);
				}
			}
			else if (!strcmp("COLOR", script1))  // Set the color check by Magius(CHE) 
			{
				cline = &script2[0];
				splitline();
				unsigned short color = (hexnumber(0)<<8)+hexnumber(1);
				j = makenumber(2);
				if (pi)
				{// AntiChrist
					if (color != pi->color())
					{
						if (strlen(TC.fmsg))
							sysmessage(ts, TC.fmsg);
						else
						{
							sprintf(temp, "You need another type of %s!", tempname);
							sysmessage(ts, temp);
						}
						Scp->Close();
						return;
					}
					if (j>0)
					{
						pc_ts->targtrig = j;
						Scp->Close();
						triggerwitem(ts, pi, 1);
						return;
					}
				}
			}
			else if (!strcmp("CALCUSES", script1))  // Calculate an items uses
			{
				if (pi)
				{
					if (pi->tuses)
					{
						pi->tuses += str2num(script2);
						Scp->Close();
						return;
					}
				}
			}
			else if (!strcmp("CSKL", script1))	// Make a check on the players skill
			{
				cline = &script2[0];
				splitline();
				unsigned int skill = makenumber(0);
				i = (rand()%1000) + 1;
				// Taur 69.02 added to get some chance of
				// skill gain on failure
				if (i > pc_ts->getRealSkill(skill))
				{
					SkillManager->AdvanceSkill(pc_ts, skill, 0);
					SkillManager->updateSkillLevel(pc_ts, skill);
					updateskillPacket(ts, skill);
					if (strlen(TC.fmsg))
						sysmessage(ts, TC.fmsg); // by Magius(CHE) 
					else 
					{
						strcpy(temp, "You fail in your attempt..."); // by Magius(CHE) 
						sysmessage(ts, temp); // by Magius(CHE) 
					}
					Scp->Close();
					return;
				} // Taur end 69.02 change
				j = makenumber(1);
				if (j>0)
				{
					// Magius(CHE) 
					pc_ts->targtrig = j;
					Scp->Close();
					triggerwitem(ts, pi, 1);
					return;
				}
			}
			break;
		case 'D':
			if (!strcmp("DEX", script1))  // Do math on players dexterity
			{
				j = str2num(script2);
				if (j < 0)
				{
					sysmessage(ts, "You are getting clumsier!");
				}
				else 
				{
					sysmessage(ts, "You are getting more agile!");
				}
				pc_ts->chgRealDex(j);
			}
			else if (!strcmp("DUR", script1))  // Math on item HP ---- rewrite by Magius(CHE) 
			{
				cline = &script2[0];
				splitline();
				j = makenumber(0);
				p = makenumber(1);
				if (p <= 0)
					p = 100;
				if (pi)
				{
					c = pi->hp;
					if (pi->maxhp>0)
					{
						if ((rand()%(100)) + 1 <= p)
						{
							if ((c >= pi->maxhp) && j>0)
							{
								sprintf(tempstr, "Your %s is already totally repaired!", tempname);
								sysmessage(ts, tempstr);
							}
							pi->hp += j;
							if (pi->hp >= pi->maxhp)
								pi->hp = pi->maxhp;
							if (pi->hp - c>0)
							{
								if (strlen(TC.cmsg))
									sysmessage(ts, TC.cmsg);
								else 
								{
									total = (float) pi->hp/pi->maxhp; // Magius(CHE) 
									sprintf(tempstr, "Your %s is now repaired! [%.1f%%]", tempname, total*100);  // Magius(CHE) 
									sysmessage(ts, tempstr);
								}
							}
							else if (pi->hp - c < 0)
							{
								if (strlen(TC.fmsg))
									sysmessage(ts, TC.fmsg);
								else 
								{
									sprintf(tempstr, "Your %s appears to be more ruined than before!", tempname);
									sysmessage(ts, tempstr);
								}
							}
							if (pi->hp <= 0)
							{
								sprintf(tempstr, "Your %s was too old and it has been destroyed!", tempname);
								sysmessage(ts, tempstr);
								pi->ReduceAmount(1);
							}
						}
					}
					else 
					{
						sprintf(tempstr, "You can't repair %s !", tempname);
						sysmessage(ts, tempstr);
					}
				}
			}
			else if (!strcmp("DISABLE", script1))  // Disable Item for n seconds
			{
				if (pi)
				{
					pi->disabled = (uiCurrentTime +(MY_CLOCKS_PER_SEC*str2num(script2)));
					strcpy(pi->GetTrigInf()->disabledmsg, dismsg); // Added by Magius(CHE) 
				}
			}
			else if (!strcmp("DISABLEMSG", script1))  // Disable Item Message --- by Magius(CHE) 
			{
				strcpy(dismsg, script2);
			}
			break;
		case 'E':
			if (!strcmp("EMT", script1))  // Player says something when trigger is activated
			{
				strcpy(sect, script2);
				for (i = 0; i < now; i++)
				{
					if (perm[i])
					{
					  if (inrange1pl(pc_ts, currchar(i)))
					  {
						tl = 44 + strlen(sect) + 1;
					
						unsigned char talk[14];
						LongToCharPtr(pc_ts->serial,&talk[3]);
						ShortToCharPtr(pc_ts->id(),&talk[7]);
						talk[9] = 0; // Type
						ShortToCharPtr(pc_ts->saycolor, &talk[10]);
						talk[12] = 0;
						talk[13] = pc_ts->fonttype;

						XStartPkt(0x1C, tl);
						XAddPkt(&talk[0+3], 14-3);
						XAddPkt((unsigned char*)&pc_ts->name[0], 30);
						XAddPkt((unsigned char*)&sect[0], strlen(sect) + 1);
						XFinishPkt(i);
					  }
					}
				}
			}
			else if (!strcmp("END_TRIGGER", script1))  // End execution of a trigger
			{
				unsigned int uiTempi = str2num(script2);
				if ((uiTempi && uiCompleted) ||(!uiTempi && !uiCompleted))
				{
					Scp->Close();
					return;
				}
			}
			break;
		case 'F':
			if (!strcmp("FMSG", script1))  // Set fail message
			{
				strcpy(TC.fmsg,script2);
			}
			else if (!strcmp("FAME", script1))  // Do math on the players fame - Magius(CHE)
			{
				cline = &script2[0];
				splitline();
				j = makenumber(0);
				c = pc_ts->fame;
				if (j>0 && c >= 10000)
					sysmessage(ts, "You are already the most glorious person in this world!");
				if (j < 0 && c <=-10000)
					sysmessage(ts, "You are already the most unknown person in this world!");
				pc_ts->fame += j;
				if (pc_ts->fame>10000)
					pc_ts->fame = 10000;
				if (pc_ts->fame < -10000)
					pc_ts->fame=-10000;
				j = pc_ts->fame - c;
				if (j>0)
				{
					if (j < 25)
						sysmessage(ts, "You have gained a little fame!");
					else if (j < 75)
						sysmessage(ts, "You have gained some fame!");
					else if (j < 100)
						sysmessage(ts, "You have gained a lot of fame!");
					else 
						sysmessage(ts, "You have gained a huge amount of fame!");
				}
				else if (j < 0)
				{
					if (abs(j) < 25)
						sysmessage(ts, "You have lost a little fame!");
					else if (abs(j) < 75)
						sysmessage(ts, "You have lost some fame!");
					else if (abs(j) < 100)
						sysmessage(ts, "You have lost a lot of fame!");
					else 
						sysmessage(ts, "You have lost a huge amount of fame!");
				}
				// refresh title
			}
			break;
		case 'H':
			if (!strcmp("HUNGER", script1))  // Do math on players hunger from 0 to 6 - Magius(CHE)
			{
				j = str2num(script2);
				pc_ts->hunger += j;
				if (pc_ts->hunger>6)
					pc_ts->hunger = 6;
				if (pc_ts->hunger < 1)
					pc_ts->hunger = 1;
				switch (pc_ts->hunger)
				{
				case 0:  
					sysmessage(ts, "You eat the food, but are still extremely hungry.");
					break;
				case 1:  
					sysmessage(ts, "You eat the food, but are still extremely hungry.");
					break;
				case 2:  
					sysmessage(ts, "After eating the food, you feel much less hungry.");
					break;
				case 3:  
					sysmessage(ts, "You eat the food, and begin to feel more satiated.");
					break;
				case 4:  
					sysmessage(ts, "You feel quite full after consuming the food.");
					break;
				case 5:  
					sysmessage(ts, "You are nearly stuffed, but manage to eat the food.");
					break;
				case 6:  
					sysmessage(ts, "You are simply too full to eat any more!");
					break;
				}
			}
			else if (!strcmp("HEA", script1))  // Do math on players health
			{
				j = str2num(script2);
				if (j < 0 && pc_ts->Hp() < abs(j) && !pc_ts->isInvul())
				{
					deathstuff(pc_ts);
					Scp->Close();
					return;
				}
				pc_ts->increaseHp(j);
			}
			break;
		case 'I':
			if (!strcmp("IFHUNGER", script1))  // If hunger meets a certain criteria - Magius(CHE)
			{
				j = str2num(script2);
				if (j >= 0)
				{
					if (!(pc_ts->hunger >= j))
					{
						if (strlen(TC.fmsg))
							sysmessage(ts, TC.fmsg);
						Scp->Close();
						return;
					}
				}
				else 
				{
					if (!(pc_ts->hunger <= abs(j)))
					{
						if (strlen(TC.fmsg))
							sysmessage(ts, TC.fmsg);
						Scp->Close();
						return;
					}
				}
			}
			else if (!strcmp("IFKARMA", script1))  // If karma meets a certain criteria - Magius(CHE)
			{
				char opt;
				gettokennum(script2, 0);
				opt = gettokenstr[0];
				gettokennum(script2, 1);
				j = str2num(gettokenstr);
				// Console.send("Script2: %s\nOPT: %c\n%i %c %i\n",script2,opt,chars[p].karma,opt,j);
				if (opt == '>')
				{
					if (!(pc_ts->karma >= j))
					{
						if (strlen(TC.fmsg))
							sysmessage(ts, TC.fmsg);
						else 
							sysmessage(ts, "You need more Karma to do that!");
						Scp->Close();
						return;
					}
				}
				else if (opt == '<')
				{
					if (!(pc_ts->karma <= j))
					{
						if (strlen(TC.fmsg))
							sysmessage(ts, TC.fmsg);
						else 
							sysmessage(ts, "You need less Karma to do that!");
						Scp->Close();
						return;
					}
				}
				else 
					Console.send("TRIGGER.SCP:Triggerwitem() Syntax Error in IFKARMA token.\n   IFKARMA <opt> <value>\n    <opt>= '<' or '>'\n    <value> range from '-10000' to '10000'.\n");
			}
			else if (!strcmp("IDADD", script1))  // Add item in front of player by ID
			{
				unsigned int low = 0, high = 0;
				cline = &script2[0];
				splitline();
				
				low = makenumber(2);
				high = (tnum == 4) ? makenumber(3) : low;
				r = RandomNum(low, high);
				// AntiChrist
				scpMark m=Scp->Suspend();
				
				short id = (hexnumber(0)<<8)+hexnumber(1);
				P_ITEM iC = Items->SpawnItem(ts,pc_ts, r, "#", 1, id, 0, 1, 1);
				if (!iC) return;
				
				// Added colormem token here! by Magius(CHE) 
				if (c>-1 && TC.coloring>-1)
				{
					iC->setColor(TC.memColor);
					iC->refresh();
				}
				// end addons	
				TC.SpawnedItem = iC;
				Scp->Resume(m);
			}
			else if (!strcmp("IDFX", script1))  // Makes an effect at players by ID
			{
				unsigned char effect[29];
				cCoord Tpos;
				for (i = 0; i < 29; i++)
				{
					effect[i] = 0;
				}
				if (pi)
				{
					P_ITEM Pack = Packitem(pc_ts);
					if (Pack)
					{
						if (Pack->Contains(pi))
						{
							Scp->Close();
							return;
						}
						else
						{
							Tpos = pi->pos;
						}
					}
				}
				else 
				{
					Tpos.x = (buffer[ts][11] << 8) + buffer[ts][12];
					Tpos.y = (buffer[ts][13] << 8) + buffer[ts][14];
					Tpos.z = (buffer[ts][15] << 8) + buffer[ts][16];
				}
				cline = &script2[0];
				splitline();
				effect[0] = 0x70; // Effect message
				effect[1] = 0x00; // Moving effect
				LongToCharPtr(pc_ts->serial,&effect[2]);
				//        effect[6]=pc_ts->ser1;
				//        effect[7]=pc_ts->ser2;
				//        effect[8]=pc_ts->ser3;
				//        effect[9]=pc_ts->ser4;
				effect[10] = hexnumber(0);// Object id of the effect
				effect[11] = hexnumber(1);
				PosToPtrXYZ(pc_ts->pos,&effect[12]);
				PosToPtrXYZ(Tpos,&effect[17]);
				effect[22] = 0x09;
				effect[23] = 0x06; // 0 is really long.  1 is the shortest.
				effect[24] = 0; // This value is unknown
				effect[25] = 0; // This value is unknown
				effect[26] = 1; // This value is unknown
				effect[27] = 0x01; // This value is used for moving effects that explode on impact.
				
				SndToAllInRange(pc_ts, effect, 28);
			}
			else if (!strcmp("INT", script1))  // Do math on players intelligence
			{
				j = str2num(script2);
				if (j < 0)
					sysmessage(ts, "Your mind is growing weaker!");
				else 
					sysmessage(ts, "Your mind is getting stronger!");
				pc_ts->chgRealInt(j);
			}
			else if (!strcmp("IFOWNER", script1))  // If item is owned by triggerer
			{
				if (pi)
				{
					if (!pc_ts->Owns(pi))
					{
						sysmessage(ts, "You do not own that.");
						Scp->Close();
						return;
					}
				}
			}
			else if (!strcmp("IFSKL", script1))  // If skill meets a certain criteria
			{
				cline = &script2[0];
				splitline();
				i = makenumber(0);
				j = makenumber(1);
				if (j >= 0)
				{
					if (pc_ts->getRealSkill(i) < j)
					{
						if (strlen(TC.fmsg))
							sysmessage(ts, TC.fmsg);
						Scp->Close();
						return;
					}
				}
				else 
				{
					if (pc_ts->getRealSkill(i) > abs(j))
					{
						if (strlen(TC.fmsg))
							sysmessage(ts, TC.fmsg);
						Scp->Close();
						return;
					}
				}
			}
			else if (!strcmp("IFFAME", script1))  // If karma meets a certain criteria - Magius(CHE)
			{
				char opt;
				gettokennum(script2, 0);
				opt = gettokenstr[0];
				gettokennum(script2, 1);
				j = str2num(gettokenstr);
				if (opt == '>')
				{
					if (!(pc_ts->fame >= j))
					{
						if (strlen(TC.fmsg))
							sysmessage(ts, TC.fmsg);
						else 
							sysmessage(ts, "You need more Fame to do that!");
						Scp->Close();
						return;
					}
				}
				else if (opt == '<')
				{
					if (!(pc_ts->fame <= abs(j)))
					{
						if (strlen(TC.fmsg))
							sysmessage(ts, TC.fmsg);
						else 
							sysmessage(ts, "You need less Fame to do that!");
						Scp->Close();
						return;
					}
				}
				else 
					Console.send("TRIGGER.SCP:Triggerwitem() Syntax Error in IFFAME token.\n   IFFAME <opt> <value>\n    <opt>= '<' or '>'\n    <value> range from '-10000' to '10000'.\n");
			}
			else if (!strcmp("IFREQ", script1)) // Check Envoked ID, if true JUMP to another trigger (in trigger.scp)
			{
				cline = &script2[0];
				splitline();
				if (TC.EnvItem)
				{
					sprintf(sect, "x%x", TC.EnvItem->id());
					if (strstr((char*)comm[0], sect))
					{
						pc_ts->targtrig = makenumber(1);
						Scp->Close();
						triggerwitem(ts, pi, 1);
						return;
					}
				}
			}
			break;
					case 'K':
						if (!strcmp("KARMA", script1))  // Do math on the players karma - Magius(CHE)
						{
							cline = &script2[0];
							splitline();
							j = makenumber(0);
							c = pc_ts->karma;
							if (j>0 && c >= 10000)
								sysmessage(ts, "You are already the most honest person in this world!");
							if (j < 0 && c <=-10000)
								sysmessage(ts, "You are already the most evil person in this world!");
							pc_ts->karma += j;
							if (pc_ts->karma>10000)
								pc_ts->karma = 10000;
							if (pc_ts->karma < -10000)
								pc_ts->karma=-10000;
							j = pc_ts->karma - c;
							if (j>0)
							{
								if (j < 25)
									sysmessage(ts, "You have gained a little karma!");
								else if (j < 75)
									sysmessage(ts, "You have gained some karma!");
								else if (j < 100)
									sysmessage(ts, "You have gained a lot of karma!");
								else 
									sysmessage(ts, "You have gained a huge amount of karma!");
							}
							else if (j < 0)
							{
								if (abs(j) < 25)
									sysmessage(ts, "You have lost a little karma!");
								else if (abs(j) < 75)
									sysmessage(ts, "You have lost some karma!");
								else if (abs(j) < 100)
									sysmessage(ts, "You have lost a lot of karma!");
								else 
									sysmessage(ts, "You have lost a huge amount of karma!");
							}
						}
						break;
					case 'M':
						if (!strcmp("MAXDUR", script1))  // Math on item MAXHP ---- rewrite by Magius(CHE) 
						{
							cline = &script2[0];
							splitline();
							j = makenumber(0);
							p = makenumber(1);
							if (p <= 0)
								p = 100;
							if (pi)
							{
								if (pi->maxhp>0)
								{
									if ((rand()%(100)) + 1 <= p)
									{
										pi->maxhp += j;
										if (pi->hp >= pi->maxhp)
											pi->hp = pi->maxhp;
										if (str2num(script2) >= 0)
										{
											if (strlen(TC.cmsg))
												sysmessage(ts, TC.cmsg);
											else 
											{
												sprintf(tempstr, "You increased the maximum durability of your %s !", tempname);
												sysmessage(ts, tempstr);
											}
										}
										else 
										{
											if (strlen(TC.fmsg))
												sysmessage(ts, TC.fmsg);
											else 
											{
												sprintf(tempstr, "Your %s appears to be more ruined than before!", tempname);
												sysmessage(ts, tempstr);
											}
										}
										if (pi->maxhp <= 0)
										{
											sprintf(tempstr, "Your %s was too old and it has been destoryed!", tempname);
											sysmessage(ts, tempstr);
											pi->ReduceAmount(1);
										}
									}
								}
								else 
								{
									sprintf(tempstr, "You can't increase durability of %s !", tempname);
									sysmessage(ts, tempstr);
								}
							}
						}
						else if (!strcmp("MAN", script1))  // Do math on players mana
						{
							j = str2num(script2);
							if (j < 0 && pc_ts->mn < abs(j))
							{
								sysmessage(ts, "Your mind is too tired to do that.");
								Scp->Close();
								return;
							}
							pc_ts->mn += j;
							if (pc_ts->mn>pc_ts->effInt())
								pc_ts->mn = pc_ts->effInt();
							updatestats(pc_ts, 1);
						}
						else if (!strcmp("MAKE", script1))  // Give user the make menu
						{
							// itemmenu(ts,str2num(script2));
							cline = &script2[0];
							splitline();
							itemmake[ts].Mat1id = ((hexnumber(2)) << 8) + hexnumber(3);
							itemmake[ts].has = pc_ts->CountItems(itemmake[ts].Mat1id); 
							itemmake[ts].has2 = pc_ts->CountItems(itemmake[ts].Mat2id);
							itemmake[ts].coloring = TC.coloring; // Magius(CHE) 
							if (TC.coloring>-1)
							{
								itemmake[ts].newColor = TC.memColor;
							}
							scpMark m=Scp->Suspend();
							
							Skills->MakeMenu(ts, makenumber(0), makenumber(1));
							
							Scp->Resume(m);
							strcpy(script1, "DUMMY");
						}
						else if (!strcmp("MISC", script1))  // Perform a miscellaneous function
						{
							if (!strcmp("bank", strlwr(script2)))
								openbank(ts, pc_ts);
						}
						else if (!strcmp("MSG", script1))  // Display a message when trigger is activated
						{
							strcpy(sect,script2);
							sysmessage(ts, sect);
						}
						else if (!strcmp("MEMCOLOR", script1))  // Store the item color in memory by Magius(CHE) 
						{
							if (!strcmp(script2, "EMPTY"))
							{
								TC.coloring=-1;
								Scp->Close();
								return;
							}
							P_ITEM iP = pi;
							if (!strcmp(script2, "REQ"))
								iP = TC.EnvItem;
							if (!strcmp(script2, "NEED"))
								iP = TC.NeedItem;
							if (iP!=NULL)
							{
								TC.memColor = iP->color();
								TC.coloring = 1;
							}
							else
							{
								Console.send("Error in Trigger script.\n Check MEMCOLOR %s in SECTION ", script2);
								if (ttype == 1)
									Console.send("%i.\n", trig);
								else 
									Console.send("...x%x%x...\n", buffer[ts][0x11], buffer[ts][0x12]);
								Console.send(" Make sure that it was after %s.\n", script2);
							}
						}
						break;
					case 'N':
						if (!strcmp("NEWTYPE", script1))  // Set active item type
						{
							if (TC.SpawnedItem)
								TC.SpawnedItem->setType(str2num(script2));
							else
							{
								if (pi)
									pi->setType(str2num(script2));
							}
						}
						else if (!strcmp("NADD", script1))  // Add a NPC at given location - AntiChrist
						{// Usage: NADD <npc_number> <life_in_seconds>
							uiCompleted = 0;
							cline = &script2[0];
							splitline();
							unsigned int uiNpcNum = makenumber(0);
//							unsigned int uiTimer = makenumber(1);
							
							if (pi)
							{
								P_ITEM Pack = Packitem(pc_ts);
								if (Pack!=NULL)
								if (Pack->Contains(pi))
								{
									triggerx = pc_ts->pos.x + 1;
									triggery = pc_ts->pos.y + 1;
									triggerz = pc_ts->pos.z;
								}
								else 
								{
									triggerx = pi->pos.x;
									triggery = pi->pos.y;
									triggerz = pi->pos.z;
								}
							}
							else
							{
								x2 = (buffer[ts][11] << 8) + buffer[ts][12];
								y2 = (buffer[ts][13] << 8) + buffer[ts][14];
								z2 = (buffer[ts][15] << 8) + buffer[ts][16];
								triggerx = x2;
								triggery = y2;
								triggerz = z2;
							}
							scpMark m=Scp->Suspend();

							TC.Spawned = Npcs->AddRespawnNPC(pi, uiNpcNum);
							
							/*
							if (uiTimer>0)
							{// if we have a timer
							TC.Spawned->summontimer = uiCurrentTime +(uiTimer*MY_CLOCKS_PER_SEC);
							}
							Commented out by Correa, reason: Bug fix. Should it be a new feature? If so,
							it's not parsing the second param when it doesn't exist, so uiTimer == uiNpcNum.
							*/
							
							triggerx = 0;
							Scp->Resume(m);
							strcpy(script1, "DUMMY");
						}
						else if (!strcmp("NEWBIEITEM", script1))  // Set item newbie
						{
							if (pi)
								pi->newbieOn();
						}
						else if (!strcmp("NEWOWNER", script1))  // Set ownership of item
						{
							if (TC.SpawnedItem)
							{
								TC.SpawnedItem->SetOwnSerial(pc_ts->serial);
							}
							if (TC.Spawned)
							{
								TC.Spawned->SetOwnSerial(pc_ts->serial);
							}
						}
						else if (!strcmp("NEWSPEECH", script1))  // Give the new npc a new spech -- MAgius(CHE) 
						{
							if (TC.Spawned)
							{
								TC.Spawned->speech = str2num(script2);
							}
						}
						else if (!strcmp("NEEDCOLOR", script1))  // Set the color check on NEEDED item by Magius(CHE) 
						{
							cline = &script2[0];
							splitline();
							unsigned short color = (hexnumber(0)<<8)+hexnumber(1);
							j = makenumber(2);
							if (TC.NeedItem)
							{// AntiChrist
								if (color != TC.NeedItem->color())
								{
									if (strlen(TC.fmsg))
										sysmessage(ts, TC.fmsg);
									else
									{
										sprintf(temp, "You need another type of %s!", TC.needname);
										sysmessage(ts, temp);
									}
									Scp->Close();
									return;
								}
							}
							
							if (j>0)
							{
								pc_ts->targtrig = j;
								Scp->Close();
								triggerwitem(ts, pi, 1);
								return;
							}
						}
						else if (!strcmp("NEEDDUR", script1))	// Math on NEED item HP (only if NEEDITEM>-1)---- rewrite by Magius(CHE) 
						{
							cline = &script2[0];
							splitline();
							j = makenumber(0);
							p = makenumber(1);
							if (p <= 0)
								p = 100;
							if (TC.NeedItem)
							{
								c = TC.NeedItem->hp;
								if (TC.NeedItem->maxhp>0)
								{
									if ((rand()%(100)) + 1 <= p)
									{
										if (c >= TC.NeedItem->maxhp && j>0)
										{
											sprintf(tempstr, "Your %s is already totally repaired!", TC.needname);
											sysmessage(ts, tempstr);
										}
										TC.NeedItem->hp += j;
										if (TC.NeedItem->hp >= TC.NeedItem->maxhp)
											TC.NeedItem->hp = TC.NeedItem->maxhp;
										if (TC.NeedItem->hp - c>0)
										{
											if (strlen(TC.cmsg))
												sysmessage(ts, TC.cmsg);
											else 
											{
												total = (float) TC.NeedItem->hp/TC.NeedItem->maxhp; // Magius(CHE) 
												sprintf(tempstr, "Your %s is now repaired! [%.1f%%]", TC.needname, total*100);  // Magius(CHE) 
												sysmessage(ts, tempstr);
											}
										}
										else if (TC.NeedItem->hp - c < 0)
										{
											if (strlen(TC.fmsg))
												sysmessage(ts, TC.fmsg);
											else 
											{
												sprintf(tempstr, "Your %s appears to be more ruined than before!", TC.needname);
												sysmessage(ts, tempstr);
											}
										}
										if (TC.NeedItem->hp <= 0)
										{
											sprintf(tempstr, "Your %s was too old and it has been destroyed!", TC.needname);
											sysmessage(ts, tempstr);
											TC.NeedItem->ReduceAmount(1);
										}
									}
								}
								else 
								{
									sprintf(tempstr, "You can't repair %s !", TC.needname);
									sysmessage(ts, tempstr);
								}
							}
						}
						else if (!strcmp("NEEDMAXDUR", script1))	// Math on NEED item HP (only if NEEDITEM>-1) ---- rewrite by Magius(CHE) 
						{
							cline = &script2[0];
							splitline();
							j = makenumber(0);
							p = makenumber(1);
							if (p <= 0)
								p = 100;
							if (TC.NeedItem)
							{
								if (TC.NeedItem->maxhp>0)
								{
									if ((rand()%(100)) + 1 <= p)
									{
										TC.NeedItem->maxhp += j;
										if (TC.NeedItem->hp >= TC.NeedItem->maxhp)
											TC.NeedItem->hp = TC.NeedItem->maxhp;
										if (str2num(script2) >= 0)
										{
											if (strlen(TC.cmsg))
												sysmessage(ts, TC.cmsg);
											else 
											{
												sprintf(tempstr, "You increased the maximum durability of your %s !", TC.needname);
												sysmessage(ts, tempstr);
											}
										}
										else 
										{
											if (strlen(TC.fmsg))
												sysmessage(ts, TC.fmsg);
											else 
											{
												sprintf(tempstr, "Your %s appears to be more ruined than before!", TC.needname);
												sysmessage(ts, tempstr);
											}
										}
										if (TC.NeedItem->maxhp <= 0)
										{
											sprintf(tempstr, "Your %s was too old and it has been destoryed!", TC.needname);
											sysmessage(ts, tempstr);
											TC.NeedItem->ReduceAmount(1);
										}
									}
								}
								else 
								{
									sprintf(tempstr, "You can't increase durability of %s !", TC.needname);
									sysmessage(ts, tempstr);
								}
							}
						}
						else if (!strcmp("NEWNAME", script1))	// Give the new item/npc a name
						{
							if (TC.SpawnedItem)
							{
								strcpy(TC.SpawnedItem->name, script2);
							}
							if (TC.Spawned)
							{
								strcpy(TC.Spawned->name, script2);
							}
						}
						else if (!strcmp("NEWTRIG", script1))  // Give the new item/npc a dynamic trigger number
						{
							if (TC.SpawnedItem)
							{
								TC.SpawnedItem->trigger = str2num(script2);
							}
							if (TC.Spawned)
							{
								TC.Spawned->trigger = str2num(script2);
							}
						}
						else if (!strcmp("NEWWORD", script1))  // Give the new npc a trigger word
						{
							if (TC.Spawned)
							{
								strcpy(TC.Spawned->trigword, script2);
							}
						}
						break;
					case 'O':
						if (!strcmp("OFFEMT", script1))  // Player says something when trigger is activated
						{
							if (pi->trigon)
							{
								strcpy(sect, script2);
								for (i = 0; i < now; i++)
								{
									if (inrange1pl(pc_ts, currchar(i)) && perm[i])
									{
										tl = 44 + strlen(sect) + 1;

										unsigned char talk[14];									

										LongToCharPtr(pc_ts->serial,&talk[3]);
										ShortToCharPtr(pc_ts->id(),&talk[7]);
										talk[9] = 0; // Type
										ShortToCharPtr(pc_ts->saycolor, &talk[10]);
										talk[12] = 0;
										talk[13] = pc_ts->fonttype;
										XStartPkt(0x1C, tl);
										XAddPkt(&talk[0+3], 14-3);
										XAddPkt((unsigned char*)&pc_ts->name[0], 30);
										XAddPkt((unsigned char*)&sect[0], strlen(sect) + 1);
										XFinishPkt(i);
									}
								}
							}
						}
						else if (!strcmp("ONEMT", script1))  // Player says something when trigger is activated
						{
							if (!pi->trigon)
							{
								strcpy(sect, script2);
								for (i = 0; i < now; i++)
								{
									if (inrange1pl(pc_ts, currchar(i)) && perm[i])
									{
										tl = 44 + strlen(sect) + 1;

										unsigned char talk[14];		
									
										LongToCharPtr(pc_ts->serial,&talk[3]);
										ShortToCharPtr(pc_ts->id(),&talk[7]);
										talk[9] = 0; // Type
										ShortToCharPtr(pc_ts->saycolor, &talk[10]);
										talk[12] = 0;
										talk[13] = pc_ts->fonttype;
										
										XStartPkt(0x1C, tl);
										XAddPkt(&talk[0+3], 14-3);
										XAddPkt((unsigned char*)&pc_ts->name[0], 30);
										XAddPkt((unsigned char*)&sect[0], strlen(sect) + 1);
										XFinishPkt(i);
									}
								}
							}
						}
						else if (!strcmp("OFFMSG", script1))// -Frazurbluu- New script command (type 2)
						{
							if (pi->trigon)
							{
							strcpy(sect,script2);
							sysmessage(ts, sect);
							}
						}
						else if (!strcmp("ONMSG", script1))// -Frazurbluu- New script command (type 2)
						{
							if (!pi->trigon)
							{
							strcpy(sect,script2);
							sysmessage(ts, sect);
							}
						}
						else if (!strcmp("OUTRANGE", script1))  // Player is in range of an item (only if this item is out of backpack) -- Magius(CHE) 
						{
							cline = &script2[0];
							splitline();
							// comm[0] contains the id of the out-items!
							p = makenumber(1);
							
							bool found = false;
							PC_ITEM pim;
							int loopexit=0;
							RegGrid3x3Iterator4Items rg3ii(pc_ts->pos);
							while((pim = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
							{
								if (iteminrange(pc_ts, pim, p))
								{
									sprintf(sect, "x%x", pim->id());
									if (strstr((char*)comm[0], sect))
									{// if it's the item we want
										found = true;
									}
								}
							}
							
							if (!found)
							{
								if (strlen(TC.fmsg))
									sysmessage(ts, TC.fmsg);
								else
								{
									strcpy(sect, "You are not close enough to work on that.");
									sysmessage(ts, sect);
								}
								Scp->Close();
								return;
							}
						}
						else if (!strcmp("OPENGUMP", script1))
						{
							scpMark m=Scp->Suspend(); // gumps->menu opens scripts !!

							Gumps->Menu(ts, str2num(script2));

							Scp->Resume(m);
							strcpy(script1, "DUMMY");
						}
						
						break;
					case 'R':
						if (!strcmp("RANDOM_ITEM", script1))  //%chance to IADD item
						{
							uiCompleted = 0;
							cline = &script2[0];
							splitline();
							unsigned int uiItemNum = makenumber(0);
							unsigned int uiChance = makenumber(1);
							unsigned int InBackpack = makenumber(2);
							
							if (uiChance >=(unsigned)((rand()%100) + 1))
							{
								strcpy(script1, "IADD");
								sprintf(script2, "%i %i", uiItemNum, InBackpack);
								uiCompleted = 1;
							}
							else if (strlen(TC.fmsg))
								sysmessage(ts, TC.fmsg); // Added by Magius(CHE) 
						}
						else if (!strcmp("RANDOM_NPC", script1))  //%chance to NADD item
						{
							uiCompleted = 0;
							cline = &script2[0];
							splitline();
							unsigned int uiItemNum = makenumber(0);
							unsigned int uiChance = makenumber(1);
							
							if (uiChance >=(unsigned)((rand()%100) + 1))
							{
								strcpy(script1, "NADD");
								sprintf(script2, "%i", uiItemNum);
								uiCompleted = 1;
							}
							else if (strlen(TC.fmsg))
								sysmessage(ts, TC.fmsg);// Added by Magius(CHE) 
						}
						else if (!strcmp("REQCOLOR", script1))  // Set the color check on REQUIRED item by Magius(CHE) 
						{
							cline = &script2[0];
							splitline();
							unsigned short color = (hexnumber(0)<<8)+hexnumber(1);
							j = makenumber(2);
							if (TC.EnvItem)
							{// AntiChrist
								if (color != TC.EnvItem->color())
								{
									if (strlen(TC.fmsg))
										sysmessage(ts, TC.fmsg);
									else
									{
										sprintf(temp, "You need another type of %s!", TC.tempname2);
										sysmessage(ts, temp);
									}
									Scp->Close();
									return;
								}
							}
							
							if (j>0)
							{
								pc_ts->targtrig = j;
								Scp->Close();
								triggerwitem(ts, pi, 1);
								return;
							}
						}
						else if (!strcmp("RANGE", script1))  // Player is in range
						{
							x1 = pc_ts->pos.x;
							y1 = pc_ts->pos.y;
							if (pi)
							{
								x2 = pi->pos.x;
								y2 = pi->pos.y;
								z2 = pi->pos.z;
								P_ITEM Pack = Packitem(pc_ts);
								if (Pack)
								{
									if (!Pack->Contains(pi))
									{
										dx = abs(x1 - x2);
										dy = abs(y1 - y2);
										if ((dx>str2num(script2)) ||(dy>str2num(script2)))
										{
											// Magius(CHE) 
											if (strlen(TC.fmsg))
												sysmessage(ts, TC.fmsg);
											else
											{
												strcpy(sect, "You are not close enough to use that.");
												sysmessage(ts, sect);
											}
											// End MAgius Fix
											Scp->Close();
											return;
										}
									}
								}
							}
							else 
							{
								x2 = (buffer[ts][11] << 8) + buffer[ts][12];
								y2 = (buffer[ts][13] << 8) + buffer[ts][14];
								z2 = (buffer[ts][15] << 8) + buffer[ts][16];
								dx = abs(x1 - x2);
								dy = abs(y1 - y2);
								if ((dx>str2num(script2)) ||(dy>str2num(script2)))
								{
									// Magius(CHE) 
									if (strlen(TC.fmsg))
										sysmessage(ts, TC.fmsg);
									else 
									{
										strcpy(sect, "You are not close enough to use that.");
										sysmessage(ts, sect);
									}
									// End MAgius Fix
									Scp->Close();
									return;
								}
							}
						}
						else if (!strcmp("RAND", script1))  // Does a random check
						{
							cline = &script2[0];
							splitline();
							if (makenumber(0) != 0)
								p = (rand()%(makenumber(0))) + 1;
							else 
								p = 1;
							if (p>makenumber(1))
							{
								if (strlen(TC.fmsg))
									sysmessage(ts, TC.fmsg);
								Scp->Close();
								return;
							}
						}
						else if (!strcmp("REMOVE", script1))  // Remove item after triggered
						{
							if (pi)
								pi->ReduceAmount(1);
						}
						else if (!strcmp("RNDUSES", script1))  // Randomly set an items uses
						{
							if (pi)
							{
								if (!pi->tuses)
								{
									cline = &script2[0];
									splitline();
									if (makenumber(1) != 0)
										pi->tuses = (rand()%(makenumber(1))) + makenumber(0);
									else 
										pi->tuses = makenumber(0);
								}
							}
						}
						break;
					case 'S':
						if (!strcmp("SETUSES", script1))  // Set an items uses
						{
							if (pi)
								pi->tuses = str2num(script2);
						}
						else if (!strcmp("STAM", script1))  // Do math on players stamina
						{
							j = str2num(script2);
							if (j < 0 && pc_ts->Stm() < abs(j))
							{
								sysmessage(ts, "You are too tired to do that.");
								Scp->Close();
								return;
							}
							pc_ts->changeStm(j);
						}
						else if (!strcmp("STR", script1))	// Do math on players strength
						{
							j = str2num(script2);
							if (j < 0)
							{
								sysmessage(ts, "You lost strength!");
							}
							else 
							{
								sysmessage(ts, "You are getting stronger!");
							}
							pc_ts->chgRealStr(j);
						}
						else if (!strcmp("SND", script1))  // Play a sound when trigger is activated
						{
							cline = &script2[0];
							splitline();
							soundeffect(ts, hexnumber(0), hexnumber(1));
						}
						else if (!strcmp("SETEVID", script1))  // Set envoked items id to new id
						{
							if (TC.EnvItem)
							{
								cline = &script2[0];
								splitline();
								TC.EnvItem->setId((hexnumber(0)<<8)+hexnumber(1));		
								TC.EnvItem->refresh();
								TC.SpawnedItem = TC.EnvItem;
							}
						}
						else if (!strcmp("SETOWNER", script1))  // Set ownership of item
						{
							if (pi)
								pi->SetOwnSerial(pc_ts->serial);
						}
						else if (!strcmp("SETTRG", script1))  // Set items trigger to new trigger
						{
							if (pi)
								pi->trigger = str2num(script2);
						}
						else if (!strcmp("SETID", script1))  // Set items id to new id
						{
							if (pi)
							{
								cline = &script2[0];
								splitline();
								pi->setId((hexnumber(0)<<8)+hexnumber(1));				
								pi->refresh();
								TC.SpawnedItem = pi;
							}
						}
						break;
					case 'T':
						if (!strcmp("TARG", script1))  // Give a targeter with trigger number --- Fixed By Magius(CHE)
						{
							target(ts, 204, "Select a target");
							pc_ts->targtrig = str2num(script2);
						}
						else if (!strcmp("TRONSK", script1)) // Trigger On Skill
						{
							cline = &script2[0];
							splitline();
							p = makenumber(0);
							j = makenumber(1);
							if (pi->trigon) // -Frazurbluu- beginning of skill raising items
							{
								int sk=SkillManager->getAllSkillsValue();
								if ((p>-1) && (p<sk))
								{
									pc_ts->chgRealSkill(p, -j);
									SkillManager->updateSkillLevel(pc_ts, p);
									updateskillPacket(ts, p);
									pi->trigon=0;
								}
								else
								{
									if (p==STR)// strength
									{
										pc_ts->chgRealStr(-1 * j);
										updatestats(pc_ts, p);
									}
									if (p==DEX)// dexterity
									{
										pc_ts->chgRealDex(-1 * j);
										updatestats(pc_ts, p);
									}
									if (p==INTELL)// intelligence
									{
										pc_ts->chgRealInt(-1 * j);
										updatestats(pc_ts, p);
									}
									pi->trigon=0;
								}
							}
							else
							{
								int sk=SkillManager->getAllSkillsValue();
								if ((p>-1) && (p<sk))
								{
									pc_ts->chgRealSkill(p,j);
									SkillManager->updateSkillLevel(pc_ts, p);
									updateskillPacket(ts, p);
									pi->trigon=1;
								}
								else
								{
									if (p==STR)// strength
									{
										pc_ts->chgRealStr(j);
										updatestats(pc_ts, p);
									}
									if (p==DEX)// dexterity
									{
										pc_ts->chgRealDex(j);
										updatestats(pc_ts, p);
									}
									if (p==INTELL)// intelligence
									{
										pc_ts->chgRealInt(j);
										updatestats(pc_ts, p);
									}
									pi->trigon=1;
								}
							}
						}
						Scp->Close();
						return;			// Fraz, are you sure ??? (Duke, 20.10.01)
						break;
					default:
						break;
			}
		}
	Scp->Close();
}

//**************************************************************************
// This routine processes tokens for the NPC triggers.
// ts: socket!!!! (Player fire the trigger)- AntiChrist
// ti: no-socket!!!! (NPC triggered) - Magius(CHE)
void triggernpc(UOXSOCKET ts, P_CHAR Tchar, int ttype) // Changed by Magius(CHE) 
{
	if (!Tchar) return;
	cTriggerContext TC;
	TC.Init();

	char sect[512];
	signed int j;
	int p, c=0;
	unsigned int i;
	
	// Added by Magius(CHE) 
	int tl;
	char buff[512], dismsg[512];
	int trig=-1, loopexit = 0;
	buff[0] = 0;
	dismsg[0] = 0;
	
	if (ts < 0)
		return; // Magius crash fix

	P_CHAR pc_ts = MAKE_CHARREF_LR(currchar(ts));
	//TC.EnvItem = FindItemBySerial(pc_ts->dclickedserial);
	TC.TrigChar = Tchar;

	Script* Scp = NULL;
	if (ttype == 1)
	{
		trig = Tchar->trigger;
		Scp=findSection(ntrigger_script,"TRG",trig);		// open file and find section
		if (!Scp) return;
	}
	else if (ttype == 2) 
	{ 
		trig = TC.FollowupTrigger; 
		Scp=findSection(ntrigger_script,"TRG",trig);		// open file and find section 
		if (!Scp) return; 
	} 
	
	else
	{
		Scp=i_scripts[wtrigger_script];
		Scp->Open();
		sprintf(buff, "x%x%x", buffer[ts][0x11], buffer[ts][0x12]);
		if (!Scp->isin(buff))
		{
			strcpy(sect, "You cant think of a way to use that.");
			sysmessage(ts, sect);
			Scp->Close();
			return;
		}
	}
	if (Tchar->disabled>uiCurrentTime) // Added by Magius(CHE) 
	{
		if (!Tchar->disabledmsg[0] == 0x0)
			strcpy(temp, Tchar->disabledmsg);
		else 
			strcpy(temp, "You cant work now! Wait more time!");
		sysmessage(ts, temp);
		Scp->Close();
		return;
	}

	while (Scp->NextLineSplitted() && (++loopexit < MAXLOOPS))
	{
		int rc = handle_CommonTags(ts,pc_ts,TC,Scp);
		if (rc==1) continue;	// handled !!
		if (rc==2)
		{
			Scp->Close();		// abort the trigger
			return;
		}
		if (rc==3) return;		// aborted and already closed

		switch (script1[0])
		{
		case 'A':
			if (!strcmp("ACT", script1))  // Make player perform an action
			{
				cline = &script2[0];
				splitline();
				pc_ts->DoAction(hexnumber(0));
			}
			break;
		case 'C':
			if (!strcmp("CSKL", script1))  // Make a check on the players skill -- Rewrite by Magius(CHE) 
			{
				cline = &script2[0];
				splitline();
				unsigned int skill = makenumber(0);
				i = (rand()%1000) + 1;
				// Taur 69.02 added to get some chance of
				// skill gain on failure
				if (i > pc_ts->getRealSkill(skill))
				{
					SkillManager->AdvanceSkill(pc_ts, skill, 0);
					SkillManager->updateSkillLevel(pc_ts, skill);
					updateskillPacket(ts, skill);
					if (strlen(TC.fmsg))
						sysmessage(ts, TC.fmsg); // by Magius(CHE) 
					else 
					{
						strcpy(temp, "You fail in your attempt..."); // by Magius(CHE) 
						sysmessage(ts, temp); // by Magius(CHE) 
					}
					Scp->Close();
					return;
				} // Taur end 69.02 change
				j = makenumber(1);
				if (j>0)
				{
					// Magius(CHE) 
					Scp->Close();
					triggernpc(ts, Tchar, 1);
					return;
				}
			}
			break;
		case 'D':
			if (!strcmp("DEX", script1))  // Do math on players dexterity
			{
				j = str2num(script2);
				if (j < 0)
				{
					sysmessage(ts, "You are getting clumsier!");
				}
				else 
				{
					sysmessage(ts, "You are getting more agile!");
				}
				pc_ts->chgRealDex(j);
			}
			else if (!strcmp("DISABLE", script1))  // Disable NPC for n seconds
			{
				Tchar->disabled = (uiCurrentTime +(MY_CLOCKS_PER_SEC*str2num(script2)));
				strcpy(Tchar->disabledmsg, dismsg); // Added by Magius(CHE) 
			}
			else if (!strcmp("DISABLEMSG", script1))  // Disable NPC Message --- by Magius(CHE) 
			{
				strcpy(dismsg, script2);
			}
			break;
		case 'E':
			if (!strcmp("EMT", script1))  // Player says something when trigger is activated -- Changed by Magius(CHE) 
			{
				strcpy(sect, script2);
				for (i = 0; i < now; i++)
				{
					if (perm[i])
					{
						if (inrange1pl(pc_ts, currchar(i)))
						{
							tl = 44 + strlen(sect) + 1;
							
							unsigned char talk[14];					
							
							LongToCharPtr(pc_ts->serial,&talk[3]);
							ShortToCharPtr(pc_ts->id(),&talk[7]);
							talk[9] = 0; // Type
							ShortToCharPtr(pc_ts->saycolor, &talk[10]);
							talk[12] = 0;
							talk[13] = pc_ts->fonttype;
							
							XStartPkt(0x1C,tl);
							XAddPkt(&talk[0+3], 14-3);
							XAddPkt((unsigned char*)&pc_ts->name[0], 30);
							XAddPkt((unsigned char*)&sect[0], strlen(sect) + 1);
							XFinishPkt(i);
						}
					}
				}
			}
			break;
		case 'F':
			if (!strcmp("FAME", script1))  // Do math on the players fame - Magius(CHE)
			{
				cline = &script2[0];
				splitline();
				j = makenumber(0);
				c = pc_ts->fame;
				if (j>0 && c >= 10000)
					sysmessage(ts, "You are already the most gloriosus person in this world!");
				if (j < 0 && c <=-10000)
					sysmessage(ts, "You are already the most unknown person in this world!");
				pc_ts->fame += j;
				if (pc_ts->fame>10000)
					pc_ts->fame = 10000;
				if (pc_ts->fame < -10000)
					pc_ts->fame=-10000;
				j = pc_ts->fame - c;
				if (j>0)
				{
					if (j < 25)
						sysmessage(ts, "You have gained a little fame!");
					else if (j < 75)
						sysmessage(ts, "You have gained some fame!");
					else if (j < 100)
						sysmessage(ts, "You have gained a lot of fame!");
					else 
						sysmessage(ts, "You have gained a huge amount of fame!");
				}
				
				else if (j < 0)
				{
					if (abs(j) < 25)
						sysmessage(ts, "You have lost a little fame!");
					else if (abs(j) < 75)
						sysmessage(ts, "You have lost some fame!");
					else if (abs(j) < 100)
						sysmessage(ts, "You have lost a lot of fame!");
					else 
						sysmessage(ts, "You have lost a huge amount of fame!");
				}
				
				
				// refresh title
			}
			else if (!strcmp("FMSG", script1))  // Set fail message
			{
				strcpy(TC.fmsg, script2);
			}
			break;
		case 'H':
			if (!strcmp("HEA", script1))  // Do math on players health
			{
				j = str2num(script2);
				
				if (j < 0 && pc_ts->Hp() < abs(j) && !pc_ts->isInvul())
				{
					Scp->Close();
					deathstuff(pc_ts);
					return;
				}
				pc_ts->increaseHp(j);
			}
			else if (!strcmp("HUNGER", script1))  // Do math on players hunger from 0 to 6 - Magius(CHE)
			{
				j = str2num(script2);
				pc_ts->hunger += j;
				if (pc_ts->hunger>6)
					pc_ts->hunger = 6;
				if (pc_ts->hunger < 1)
					pc_ts->hunger = 1;
				switch (pc_ts->hunger)
				{
				case 0:	
					sysmessage(ts, "You eat the food, but are still extremely hungry.");
					break;
				case 1: 
					sysmessage(ts, "You eat the food, but are still extremely hungry.");
					break;
				case 2: 
					sysmessage(ts, "After eating the food, you feel much less hungry.");
					break;
				case 3: 
					sysmessage(ts, "You eat the food, and begin to feel more satiated.");
					break;
				case 4: 
					sysmessage(ts, "You feel quite full after consuming the food.");
					break;
				case 5: 
					sysmessage(ts, "You are nearly stuffed, but manage to eat the food.");
					break;
				case 6: 
					sysmessage(ts, "You are simply too full to eat any more!");
					break;
				}
			}
			break;
		case 'I':
			if (!strcmp("IFREQ", script1))
			{
				// Check Envoked ID, if true JUMP to another trigger (in ntrig.scp)
				cline = &script2[0];
				splitline();
				j = makenumber(1);
				if (TC.EnvItem)
				{
					sprintf(sect, "x%x", TC.EnvItem->id());
					if (strstr((char*)comm[0], sect))
					{
						Scp->Close();
						triggernpc(ts, Tchar, 1);
						return;
					}
				}
			}
			else if (!strcmp("IFKARMA", script1))  // If karma meets a certain criteria - Magius(CHE)
			{
				char opt;
				gettokennum(script2, 0);
				opt = gettokenstr[0];
				gettokennum(script2, 1);
				j = str2num(gettokenstr);
				if (opt == '>')
				{
					if (!(pc_ts->karma >= j))
					{
						if (strlen(TC.fmsg))
							sysmessage(ts, TC.fmsg);
						else 
							sysmessage(ts, "You need more Karma to do that!");
						Scp->Close();
						return;
					}
				}
				else if (opt == '<')
				{
					if (!(pc_ts->karma <= j))
					{
						if (strlen(TC.fmsg))
							sysmessage(ts, TC.fmsg);
						else 
							sysmessage(ts, "You need less Karma to do that!");
						Scp->Close();
						return;
					}
				}
				else 
					Console.send("TRIGGER.SCP:Triggerwitem() Syntax Error in IFKARMA token.\n   IFKARMA <opt> <value>\n    <opt>= '<' or '>'\n    <value> range from '-10000' to '10000'.\n");
			}
			else if (!strcmp("IFFAME", script1))  // If karma meets a certain criteria - Magius(CHE)
			{
				char opt;
				gettokennum(script2, 0);
				opt = gettokenstr[0];
				gettokennum(script2, 1);
				j = str2num(gettokenstr);
				if (opt == '>')
				{
					if (!(pc_ts->fame >= j))
					{
						if (strlen(TC.fmsg))
							sysmessage(ts, TC.fmsg);
						else 
							sysmessage(ts, "You need more Fame to do that!");
						Scp->Close();
						return;
					}
				}
				else if (opt == '<')
				{
					if (!(pc_ts->fame <= abs(j)))
						
					{
						if (strlen(TC.fmsg))
							sysmessage(ts, TC.fmsg);
						else 
							sysmessage(ts, "You need less Fame to do that!");
						Scp->Close();
						return;
					}
				}
				else 
					Console.send("TRIGGER.SCP:Triggerwitem() Syntax Error in IFFAME token.\n   IFFAME <opt> <value>\n    <opt>= '<' or '>'\n    <value> range from '-10000' to '10000'.\n");
			}
			else if (!strcmp("IFHUNGER", script1))  // If hunger meets a certain criteria - Magius(CHE)
			{
				j = str2num(script2);
				if (j >= 0)
				{
					if (!(pc_ts->hunger >= j))
					{
						if (strlen(TC.fmsg))
							sysmessage(ts, TC.fmsg);
						Scp->Close();
						return;
					}
				}
				else 
				{
					if (!(pc_ts->hunger <= abs(j)))
						
					{
						if (strlen(TC.fmsg))
							sysmessage(ts, TC.fmsg);
						Scp->Close();
						return;
					}
				}
			}
			else if (!strcmp("IFOWNER", script1))  // If character is owned by triggerer
			{
				if (!pc_ts->Owns(Tchar))
				{
					sysmessage(ts, "You do not own this creature.");
					Scp->Close();
					return;
				}
			}
			else if (!strcmp("IDADD", script1))  // Add item in player pack by ID
			{
				cline = &script2[0];
				splitline();
				scpMark m=Scp->Suspend();
				short id = (hexnumber(0)<<8)+hexnumber(1);
				P_ITEM iC = Items->SpawnItem(ts, pc_ts, 1, "#", 1, id, 0, 1, 1);
				if (!iC) return;
				// Added colormem token here! by Magius(CHE) 
				if (c>-1 && TC.coloring>-1)
				{
					iC->setColor(TC.memColor);
					iC->refresh();
				}
				// end addons	
				Scp->Resume(m);
			}
			else if (!strcmp("IDFX", script1))	// Makes an effect at players by ID
			{
				unsigned char effect[29];
				for (i = 0; i < 29; i++)
				{
					effect[i] = 0;
				}
				cline = &script2[0];
				splitline();
				effect[0] = 0x70; // Effect message
				effect[1] = 0x00; // Moving effect
				LongToCharPtr(Tchar->serial,&effect[2]);
				LongToCharPtr(pc_ts->serial,&effect[6]);
				effect[10] = hexnumber(0);// Object id of the effect
				effect[11] = hexnumber(1);
				PosToPtrXYZ(Tchar->pos,&effect[12]);
				PosToPtrXYZ(pc_ts->pos,&effect[17]);
				effect[22] = 0x09;
				effect[23] = 0x06; // 0 is really long.  1 is the shortest.
				effect[24] = 0; // This value is unknown
				effect[25] = 0; // This value is unknown
				effect[26] = 1; // This value is unknown
				effect[27] = 0x00; // This value is used for moving effects that explode on impact.
				
				SndToAllInRange(pc_ts, effect, 28);
			}
			else if (!strcmp("INT", script1))	// Do math on players intelligence
			{
				j = str2num(script2);
				if (j < 0)
				{
					sysmessage(ts, "Your mind is growing weaker!");
				}
				else 
				{
					sysmessage(ts, "Your mind is getting stronger!");
				}
				pc_ts->chgRealInt(j);
			}
			break;
					case 'K':
						if (!strcmp("KARMA", script1))  // Do math on the players karma - Magius(CHE)
						{
							cline = &script2[0];
							splitline();
							j = makenumber(0);
							c = pc_ts->karma;
							if (j>0 && c >= 10000)
								sysmessage(ts, "You are already the most honest person in this world!");
							if (j < 0 && c <=-10000)
								sysmessage(ts, "You are already the most evil person in this world!");
							pc_ts->karma += j;
							if (pc_ts->karma>10000)
								pc_ts->karma = 10000;
							if (pc_ts->karma < -10000)
								pc_ts->karma=-10000;
							j = pc_ts->karma - c;
							if (j>0)
							{
								if (j < 25)
									sysmessage(ts, "You have gained a little karma!");
								else if (j < 75)
									sysmessage(ts, "You have gained some karma!");
								else if (j < 100)
									sysmessage(ts, "You have gained a lot of karma!");
								else 
									sysmessage(ts, "You have gained a huge amount of karma!");
							}
							else if (j < 0)
							{
								if (abs(j) < 25)
									sysmessage(ts, "You have lost a little karma!");
								else if (abs(j) < 75)
									sysmessage(ts, "You have lost some karma!");
								else if (abs(j) < 100)
									sysmessage(ts, "You have lost a lot of karma!");
								
								else 
									sysmessage(ts, "You have lost a huge amount of karma!");
							}
						}
						break;
					case 'M':
						if (!strcmp("MAKE", script1))  // Give user the make menu
						{
							cline = &script2[0];
							splitline();
							itemmake[ts].Mat1id = ((hexnumber(2)) << 8) + hexnumber(3);
							itemmake[ts].has = pc_ts->CountItems(itemmake[ts].Mat1id); 
							itemmake[ts].has2 = pc_ts->CountItems(itemmake[ts].Mat2id);
							itemmake[ts].coloring = TC.coloring; // Magius(CHE) 
							if (TC.coloring>-1)
							{
								itemmake[ts].newColor = TC.memColor;
							}
							scpMark m=Scp->Suspend();

							Skills->MakeMenu(ts, makenumber(0), makenumber(1));
							Scp->Resume(m);
							strcpy(script1, "DUMMY");
						}
						else if (!strcmp("MISC", script1))  // Perform a miscellaneous function
						{
							if (!strcmp("bank", strlwr(script2)))
								openbank(ts, pc_ts);
							if (!strcmp("ware", strlwr(script2)))
							{ 
								scpMark m=Scp->Suspend(); 
								openspecialbank(ts, pc_ts);// AntiChrist 
								Scp->Resume(m); 
								strcpy(script1, "DUMMY"); 
							} 
							if (!strcmp("balance", strlwr(script2)))
							{
								sprintf(sect, "You have %i gp in your bank account!", pc_ts->CountBankGold());
								npctalk(ts, Tchar, sect, 0);
							}
						}
						else if (!strcmp("MSG", script1))  // Display a message when trigger is activated
						{
							if (!pc_ts->npc)
							{
								sysmessage(ts, script2);
							}
						}
						else if (!strcmp("MAN", script1))	// Do math on players mana
						{
							j = str2num(script2);
							if (j < 0 && pc_ts->mn < abs(j))
								
							{
								sysmessage(ts, "Your mind is too tired to do that.");
								Scp->Close();
								return;
							}
							pc_ts->mn += j;
							if (pc_ts->mn > pc_ts->effInt())
								pc_ts->mn = pc_ts->effInt();
							updatestats(pc_ts, 1);
						}
						else if (!strcmp("MEMCOLOR", script1))	// Store the item color in memory by Magius(CHE) 
						{
							P_ITEM iNeed = NULL;
							if (!strcmp(script2, "EMPTY"))
							{
								TC.coloring=-1;
								Scp->Close();
								return;
							}
							else if (!strcmp(script2, "REQ"))
								iNeed = TC.EnvItem;
							else if (!strcmp(script2, "NEED"))
								iNeed = TC.NeedItem;
							if (iNeed)
							{
								TC.memColor = iNeed->color();
								TC.coloring = 1;
							}
							else 
							{
								Console.send("Error in Trigger script.\n Check MEMCOLOR %s in SECTION ", script2);
								if (ttype == 1)
									Console.send("%i.\n", trig);
								else 
									Console.send("...x%x%x...\n", buffer[ts][0x11], buffer[ts][0x12]);
								Console.send(" Make sure that it was after %s.\n", script2);
							}
						}
						break;
					case 'N':
						if (!strcmp("NPCACT", script1))		// Make player perform an action
						{
							cline = &script2[0];
							splitline();
							Tchar->DoAction(hexnumber(0));
						}
						else if (!strcmp("NADD", script1))	// Add a NPC at given location - AntiChrist -- Fixed here by Magius(CHE) 
						{// Usage: NADD <npc_number> <life_in_seconds>
							// uiCompleted=0;
							cline = &script2[0];
							splitline();
							unsigned int uiNpcNum = makenumber(0);
							unsigned int uiTimer = makenumber(1);
							/*unsigned int uiTamed = makenumber(2); */
							
							triggerx = pc_ts->pos.x + 1;
							triggery = pc_ts->pos.y + 1;
							triggerz = pc_ts->pos.z;
							
							scpMark m=Scp->Suspend();

							TC.Spawned = Npcs->AddNPC(-1, NULL, uiNpcNum, triggerx, triggery,triggerz);	
							
							if (uiTimer>0)
							{// if we have a timer
								TC.Spawned->summontimer = uiCurrentTime +(uiTimer*MY_CLOCKS_PER_SEC);
							}
							
							triggerx = 0;
							Scp->Resume(m);
							strcpy(script1, "DUMMY");
						}
						else if (!strcmp("NEWOWNER", script1))  // Set ownership of NPC
						{
							if (TC.SpawnedItem)
							{// item
								TC.SpawnedItem->SetOwnSerial(pc_ts->serial);
							}
							if (TC.Spawned)// char
							{
								TC.Spawned->SetOwnSerial(pc_ts->serial);
							}
						}
						else if (!strcmp("NEEDCOLOR", script1))  // Set the color check on NEEDED item by Magius(CHE) 
						{
							cline = &script2[0];
							splitline();
							unsigned short color = (hexnumber(0)<<8)+hexnumber(1);
							j = makenumber(2);
							if (!TC.NeedItem)
							{
								Scp->Close();
								return;
							}
							if (color != TC.NeedItem->color())
							{
								if (strlen(TC.fmsg))
									sysmessage(ts, TC.fmsg);
								else 
								{
									sprintf(temp,"You need another type of %s!", TC.needname);
									sysmessage(ts, temp);
								}
								Scp->Close();
								return;
							}
							if (j>0)
							{
								pc_ts->targtrig = j;
								Scp->Close();
								triggerwitem(ts, TC.NeedItem, 1);
								return;
							}
						}
						else if (!strcmp("NEWTYPE", script1))  // Set active item type
						{
							if (TC.SpawnedItem)
								TC.SpawnedItem->setType(str2num(script2));
						}
						else if (!strcmp("NEWNAME", script1))  // Give the new item/npc a name
						{
							if (TC.SpawnedItem)
							{
								strcpy(TC.SpawnedItem->name, script2);
							}
							if (TC.Spawned)
							{
								strcpy(TC.Spawned->name, script2);
							}
						}
						else if (!strcmp("NEWTRIG", script1))  // Give the new item/npc a dynamic trigger number
						{
							if (TC.SpawnedItem)
							{
								TC.SpawnedItem->trigger = str2num(script2);
							}
							if (TC.Spawned)
							{
								TC.Spawned->trigger = str2num(script2);
							}
						}
						else if (!strcmp("NEWWORD", script1))  // Give the new npc a triggerword
						{
							if (TC.Spawned)
							{
								strcpy(TC.Spawned->trigword, script2);
							}
						}
						else if (!strcmp("NEWSPEECH", script1))  // Give the new npc a new spech -- MAgius(CHE) 
						{
							if (TC.Spawned)
							{
								TC.Spawned->speech = str2num(script2);
							}
						}
						break;
					case 'O':
						if (!strcmp("OPENGUMP", script1))
						{
							scpMark m=Scp->Suspend(); // gumps->menu opens scripts !!

							Gumps->Menu(ts, str2num(script2));

							Scp->Resume(m);
							strcpy(script1, "DUMMY");		
						}
						break;
						
					case 'R':
						if (!strcmp("REQCOLOR", script1))	// Set the color check on REQUIRED item by Magius(CHE) 
						{
							cline = &script2[0];
							splitline();
							unsigned short color = (hexnumber(0)<<8)+hexnumber(1);
							j = makenumber(2);
							if (TC.EnvItem)
							{
								Scp->Close();
								return;
							}
							if (color != TC.EnvItem->color())
							{
								if (strlen(TC.fmsg))
									sysmessage(ts, TC.fmsg);
								else 
								{
									sprintf(temp,"You need another type of %s!", TC.tempname2);
									sysmessage(ts, temp);
								}
								Scp->Close();
								return;
							}
							if (j>0)
							{
								pc_ts->targtrig = j;
								Scp->Close();
								triggerwitem(ts, TC.EnvItem, 1);
								return;
							}
						}
						else if (!strcmp("RAND", script1))  // Does a random check
						{
							cline = &script2[0];
							splitline();
							if (makenumber(0) != 0)
								p = (rand()%(makenumber(0))) + 1;
							else 
								p = 1;
							
							if (p>makenumber(1))
							{
								if (strlen(TC.fmsg))
									sysmessage(ts, TC.fmsg);
								Scp->Close();
								return;
							}
						}
						else if (!strcmp("REMOVENPC", script1)) // Simply deletes NPC 
						{ 
							Npcs->DeleteChar(Tchar); 
						} 
						break;
					case 'S':
						if (!strcmp("SND", script1))  // Play a sound when trigger is activated
						{
							cline = &script2[0];
							splitline();
							soundeffect2(Tchar, (hexnumber(0)<<8)+ hexnumber(1));
						}
						else if (!strcmp("SETTRG", script1))	// Set npcs trigger to new trigger
						{
							Tchar->trigger = str2num(script2);
						}
						else if (!strcmp("SPEECH", script1))	// Set the triggered npc a new spech -- Magius(CHE) 
						{
							Tchar->speech = str2num(script2);
						}
						// End NPC Triggers
						else if (!strcmp("SETID", script1))	// Set chars id to new id
						{
							cline = &script2[0];
							splitline();
							short id=(hexnumber(0)<<8)+hexnumber(1);
							Tchar->setId(id);
							Tchar->setXid(id);
							P_CHAR pc;
							AllPlayersInRangeIterator api(Tchar);
							for (pc = api.First(); pc != NULL; pc = api.Next())
							{
								updatechar(Tchar);
							}
						}
						else if (!strcmp("STAM", script1))	// Do math on players stamina
						{
							j = str2num(script2);
							
							if (j < 0 && pc_ts->Stm() < abs(j))
							{
								sysmessage(ts, "You are too tired to do that.");
								Scp->Close();
								return;
							}
							pc_ts->changeStm(j);
						}
						else if (!strcmp("STR", script1))	// Do math on players strength
						{
							j = str2num(script2);
							if (j < 0)
							{
								sysmessage(ts, "You lost strength!");
							}
							else 
							{
								sysmessage(ts, "You are getting stronger!");
							}
							pc_ts->chgRealStr(j);
						}
						else if (!strcmp("SETWORD", script1))	// Sets the trigger word of an NPC
						{
							strcpy(Tchar->trigword, script2);
						}
						else if (!strcmp("SETOWNER", script1))  // Set ownership of NPC
						{
							Tchar->SetOwnSerial(pc_ts->serial);
						}
						else if (!strcmp("SETNPCAI", script1))	// Set npcs AI
						{
							Tchar->npcaitype = hstr2num(script2);
						}
						break;
					case 'T':
						if (!strcmp("TALK", script1))	// the triggered NPC now talking! --- by Magius(CHE) 
						{
							if (strlen(script2)>0)
								npctalk(ts, Tchar, script2, 0);
						}
						break;
					default:
						break;
				}
		}
	Scp->Close();
}

int checkenvoke(short id)
{
	FILE *envokefile;
	char buf[1024];
	
	if (!(envokefile = fopen("envoke.scp", "r")))
	{
		fprintf(stderr, "Cannot open envoke.scp: %s", strerror(errno));
		exit(1);
	}
	int loopexit = 0;
	while ((fgets(buf, sizeof(buf), envokefile))  && (++loopexit < MAXLOOPS))
	{
		sprintf(temp, "x%xx", id);
		sprintf(temp2, "x%x ", id);
		if ((strstr(buf, temp) > 0) || (strstr(buf, temp2) > 0))
		{
			fclose(envokefile);
			return 1;
		}
	}
	fclose(envokefile);
	
	return 0;
}
