/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

#ifndef __VERINFO_H__
#define __VERINFO_H__

//System Includes

#include <string>

using namespace std;

///// do not chang those

#define WPV_RELEASE    0x01
#define WPV_INERNAL    0x02 
#define WPV_BETA       0x04
#define WPV_SPECIAL    0x08
#define WPV_MAPILSH    0x10 //  if not set BRIT map else ILSH
#define WPV_REQXWOLF   0x20
#define WPV_GUI        0x40
#define WPV_NONDEVTEAM 0x80

#define WOLF      0x0
#define NOXWIZARD 0x1
#define AVANOON   0x2
#define UOX3      0x3 // if they add wip one day, 
#define LONEWOLF  0x4 

/// end of do not change

/////////////////////////////////////////////////////////////////////////////////////
/// purpose of 32 bit version encoding: external tools that use WIP like X-WOLF 
/// being able to detect LoneWolf version+variant correctly
////////////////////////////////////////////////////////////////////////////////////

/*

  LoneWolf Version: - 32 bit encoding ( vernumber in version struct )

  g5                 g4                g3             g2               g1         
u u u u       u u u u u u u u     u u u u u u    u u u u u u u    u u u u u u u
(28-31)           (20-27)           (14-19)         (7-13)             (0-6)


g1 (bit 0-6)    ->  major version, 0...127

g2 (bit 7-13)   ->  subVersion, 0..126, 127: no subversion

g3 (bit 14-19)  ->  subsubverison, 0..62, 63: no subsubverison

g4: flags

0x01 -> supported / official release
0x02 -> internal
0x04 -> beta
0x08 -> special
0x10 -> ILSHENAR or BRITANNIA MAP SERVER ? (0x10 -> ILSH, else BRIT)
0x20 -> requires X-Wolf if windows version of server
0x40 -> GUI version
0x80 -> nonofficial (if not set -> official LoneWolf team version, if set non official team version)

of course ALL can be combined.

g5: LoneWolf derivates

0 -> Wolfpack
1 -> noxwizard
2 -> avanoon
3 -> UOX3 if it adds WIP in future. not likely though :>
4 -> Official LONEWOLF

5..15: open

*/

/// note: no code for offical encoding yet, but will be added soon, LB 30-Sept 2001


struct wpversion_32bit
{
	unsigned char major_version;
    unsigned char sub_version;
	unsigned char subsub_version;
	unsigned char flags;
	unsigned char derivate;
	wpversion_32bit() : flags(0) { ; }

};


// changed all the #defines dealing with verison stuff to a struct, LB
struct wp_version_info
{
    public:

	   unsigned int   vernumber;
	   wpversion_32bit verstruct;

	   string verstring; 
	   string productstring;
	   string betareleasestring;
	   string clientsupportedstring;
	   string timezonestring;
	   string compiledbystring;
	   string codersstring;
	   string emailstring;

	   wp_version_info()
	   {
		   /////////////////////////////////////

		   verstruct.major_version  = 13;
		   verstruct.sub_version    = 0;
		   verstruct.subsub_version = 10;
//		   verstruct.subsub_version = 63; // no subsub

		   verstruct.flags    = WPV_BETA ;
//		   verstruct.flags    = WPV_RELEASE ;
		   verstruct.derivate = LONEWOLF ;
		  
           vernumber = to_32bitver(verstruct);		   
		  
		   /// code for auto generation of verstring, productstring and betareleasestring coming soon (LB) ... LB soon (tr)
//		   verstring = "12.9 release";		// LoneWolf version 

		   verstring = "13.0.10 D7";	// LoneWolf version 

		   productstring = "LoneWolf";		// selfexplainatory, isnt't it :)
//		   betareleasestring = "Release";	// release status
		   betareleasestring = "Beta";		// release status
//		   betareleasestring = "Alpha";		// release status

		   //////////////////////////////////////////////////

		   clientsupportedstring = "4.0.0e";

		   // SUPPORTED_CLIENT is part of the client version control system.
           // this is NOT necassairily the lastest client.
           // don't change it unless you are sure that its packets structure is in synch with network.cpp etc.
           // until I (LB) don't leave the wolfpack project, I want to be the only(!) one who
           // changes that value. thx for your understanding =:)		 

		   timezonestring = "GMT+1";   // your timezone

		   #ifdef __unix__
             compiledbystring = "Usul" ;  // person that compiled linux version
             codersstring = "Duke, Lord Binary, Opik, punt & Thrawn" ;
           #else
             compiledbystring = "Duke" ; // person that compiled windows version
             codersstring = "Duke, Lord Binary, Opik & Thrawn" ;
           #endif

           emailstring = " ";		  
	   }

	   virtual ~wp_version_info() { ; }

	   private:
       unsigned int to_32bitver(wpversion_32bit data)
	   {
	       unsigned int result, r1,r2,r3,r4,r5;	
           r1 =  ( (unsigned int) ( data.major_version  & 0x7f ) ) << 0;
	       r2 =  ( (unsigned int) ( data.sub_version    & 0x3f ) ) << 7;
	       r3 =  ( (unsigned int) ( data.subsub_version & 0x7f ) ) << 13;
	       r4 =  ( (unsigned int) ( data.flags          & 0xff ) ) << 20;   
           r5 =  ( (unsigned int) ( data.derivate       & 0x0f ) ) << 28;		
	       result = r1+r2+r3+r4+r5 ;
	       return result;
	   }


       wpversion_32bit from_32bitver( unsigned long int indata)
	   {
	       wpversion_32bit wpv32;
	       wpv32.major_version  = (unsigned char) ( (indata & 0x0000007f) >> 0);
	       wpv32.sub_version    = (unsigned char) ( (indata & 0x00001fc0) >> 7);
	       wpv32.subsub_version = (unsigned char) ( (indata & 0x000fe000) >> 13);
	       wpv32.flags          = (unsigned char) ( (indata & 0x0ff00000) >> 20);
	       wpv32.derivate       = (unsigned char) ( (indata & 0xf0000000) >> 28);
	       return wpv32;
	   }

};


#endif
