/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

#include "walking.h"

#include <cmath>
#include <time.h>
#include "boats.h"
#include "Client.h"
#include "guilds.h"
#include "house.h"
#include "magic.h"
#include "mapstuff.h"
#include "msgboard.h"
#include "npcinfo.h"
#include "regions.h"
#include "skills.h"
#include "SndPkg.h"
#include "sregions.h"
#include "teleport.h"
#include "TmpEff.h"
#include "trigger.h"
#include "weight.h"
#include "multi.h"
#include "tiledata.h"
#include "mapS.h"
#include "network.h"

#include "wolfpack.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "walking.cpp"

int xycount;
#define XYMAX 5000 // Maximum items LoneWolf can handle on one X/Y square
CArray<XYMAX, unitile_st>xyblock("xyblock");


///////////////
// Name:	getLeftDir, getRightDir
// history:	by Duke, 14.7.2001
// Purpose:	calculates the adjacent direction
//
int getLeftDir(int dir)	// counterclockwise
{
	dir &= 7;	// make sure it's valid
	return dir==0 ? 7 : dir-1;
}
int getRightDir(int dir)	// clockwise
{
	dir &= 7;
	return dir==7 ? 0 : dir+1;
}

///////////////
// Name:	getXYfromDir
// history:	by Duke, 14.7.2001
// Purpose:	calculates and changes the given coords one step into the given direction
//
void GetXYfromDir(int dir, int *x, int *y)
{
	switch(dir&0x07)
	{
	case 0: (*y)--;				break;
	case 1: (*x)++; (*y)--;		break;
	case 2: (*x)++;				break;
	case 3: (*x)++; (*y)++;		break;
	case 4: (*y)++;				break;
	case 5: (*x)--; (*y)++;		break;
	case 6: (*x)--;				break;
	case 7: (*x)--; (*y)--;		break;
	}
}

void GetValidXYfromDir(int dir, int *x, int *y)
{
	GetXYfromDir(dir,x,y);
	////////// out of bounds/world checks (LB) ///////////

	int MapNum=0; // get world # , needs to be changed if multip map support gets in
	int max_x = AllMaps->GetMapWidth(MapNum)  * 8;
    int max_y = AllMaps->GetMapHeight(MapNum) * 8;

	if ((*y)<0) (*y)=0; else if ((*y)>=max_y) (*y) = max_y;
	if ((*x)<0) (*x)=0; else if ((*x)>=max_x) (*x) = max_x;
	// note: though item/char->pos.x/y UNSIGNED, here it got signedness back via (signed) int. why we need to check <0
}

///////////////
// Name:	WalkHandle
// history:	cut from walking() by Duke, 27.10.2000
// Purpose:	dummy
//
bool WalkHandle(UOXSOCKET s, CHARACTER c)
{
	return true;
}

///////////////
// Name:	WalkHandleAllowance
// history:	cut from walking() by Duke, 27.10.2000
// Purpose:	checks if the Char is allowed to move at all i.e. not frozwn, overloaded...
//
bool WalkHandleAllowance(UOXSOCKET s, P_CHAR pc, int sequence)
{
	if (!pc) return false;
	
	if (pc->stablemaster_serial>0 && s<0) return false; // shouldnt be called for stabled pets, just to be on the safe side

	if (s!=-1)
	{
		if (walksequence[s]+1!=sequence && sequence!=256)
		{
			deny(s, pc, sequence);
			return false;
		}
	}

	if (!pc->dead && !pc->npc && !pc->isGMorCounselor())
	{
		if (!Weight->CheckWeight(pc, s) || pc->Stm() < 2) // this has to executes, even if s==-1, thus that many !=-1's
		{
			if (s!=-1)
			{
				sysmessage(s, "You are too fatigued to move.");
				walksequence[s]=-1;
				teleport(pc, pc->pos); // old pos = currpos
				deny(s,pc,sequence); // !!!
				return false;
			}
		}
	}

	// dont put a if s!=-1 here !!!
	if (pc->IsFrozen()) // lord binary !!! 
	{			
		teleport(pc, pc->pos); // old pos = currpos
		if (s>-1)
		{
			if (pc->IsCasting())
				sysmessage(s, "You cannot move while casting.");
		    else
				sysmessage(s, "You are frozen and cannot move.");
			deny(s,pc,sequence);
		}
		return false;
	} 
	return true;
}

///////////////
// Name:	WalkHandleRunning
// history:	cut from walking() by Duke, 27.10.2000
// Purpose:	handles running, stealth etc.
//
bool WalkHandleRunning(UOXSOCKET s, P_CHAR pc, int dir)
{
	if (!pc) return true;
	if (dir&0x80) 
	{ //AntiChrist -- if running
		//AntiChrist - we have to unhide always if running
		pc->unhide();
		//Running stamina - AntiChrist
		pc->regen2=uiCurrentTime+(SrvParms->staminarate*MY_CLOCKS_PER_SEC); //Don't regenerate stamina while running
		pc->running++;

		// Horse Stamina loss bug Fixed by blackwind. 
		// Without loss of stamina players will be able to runaway for ever..
		if (!pc->dead)
		{
			if (((!pc->onhorse && pc->running > SrvParms->runningstaminasteps)*2) ||
				(pc->onhorse && pc->running > SrvParms->runningstaminasteps*2*2) )
			{ //The first *2 it's because i noticed that a step(animation) correspond to 2 walking calls
				pc->running=0;
				pc->reduceStm(1);
			}
		}
	}
	else 
	{
		pc->running=0;
		if (pc->stealth>-1) 
		{ //AntiChrist - Stealth
			if (pc->stealth > 0 &&		// give them at least one step, regardless of skill (Duke)
				pc->stealth > (SrvParms->maxstealthsteps*pc->Skill(STEALTH))/1000)
			{
				pc->unhide();
			}
			pc->stealth++;
		}
		else
		{
			pc->unhide();
		}
	}
	return true;
}

///////////////
// Name:	WalkCollectBlockers
// history:	cut from walking() by Duke, 20.11.2000
// Purpose:	Collects Landscape plus static and dynamic items in an array
//
// Remark LB 14'th nov 2000: will rewrite this stuff when I have time again for wp-project...
// dont delete the rem'd out clConsole.send's in meanwhile, thx !
//
void WalkCollectBlockers(UOXSOCKET s, cCoord& tpos)
{
	xycount=0;

	int mapid = 0;
	signed char mapz = Map->AverageMapElevation(tpos, mapid);
	if (mapz != illegal_z)
	{
		const landData *land;
		land = TileData->SeekLand(mapid);
		
		xyblock[xycount].type=0;
		xyblock[xycount].basez = mapz;
		xyblock[xycount].id = mapid;
		xyblock[xycount].flags=land->flags;
		xyblock[xycount].height=0;
		xyblock[xycount].weight=255;

		//printf("0 %s %s %08x %04x\n", land->name, pc->name, land->flags, mapid);

		xycount++;
	}

	PC_ITEM pi;
	int loopexit=0;
	RegGrid3x3Iterator4Items rg3ii(tpos);
	while((pi = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (!pi->isMulti()) // no multi
		{
			if (pi->pos.IsSameXY(tpos))
			{
				const tileData *tile;
				tile = TileData->SeekTile(pi->id());
				xyblock[xycount].type=1;
				xyblock[xycount].basez=pi->pos.z;
				xyblock[xycount].id=pi->id();
				xyblock[xycount].flags=tile->flags;			
				xyblock[xycount].height=tile->height;
				xyblock[xycount].weight=tile->weight;

				//printf("1 %s %08x %04x\n", pc->name, tile->flags, pi->id());

				xycount++;
			}
		}
		else // if multi tile
		{
			if (inRange(pi->pos, tpos, BUILDRANGE))
			{					
				int LoopExit=0;
	            singleMultiPart mult;
	            assert(pi->id()>=0x4000);

	            Multi->multiIteratorStart(pi->id()-0x4000);			
	            while (Multi->multiIteratorNext(mult) && ++LoopExit<MAXLOOPS)				
				{					
					if (mult.visibility && (pi->pos.x+mult.x == tpos.x) && (pi->pos.y+mult.y == tpos.y))
					{
						const tileData *tile;
						tile = TileData->SeekTile(mult.tile);				
						xyblock[xycount].type=2;
						xyblock[xycount].basez=mult.z+pi->pos.z;
						xyblock[xycount].id=mult.tile;
						xyblock[xycount].flags=tile->flags;					
						xyblock[xycount].height=tile->height;
						xyblock[xycount].weight=255;

						//printf("2:%s %s %i %08x %04x\n",tile->name, pc->name, mult.visibility, tile->flags, mult.tile);

						xycount++;
					}
				}
			}
		}
	}

	AllMaps->mapStaticIteratorStart(tpos);
	loopexit = 0;
	cellData stat;
	while ( AllMaps->mapStaticIteratorNext(stat) && (++loopexit < MAXLOOPS))
	{	
		const tileData *tile;
		tile = TileData->SeekTile(stat.TID);
		xyblock[xycount].type=2;
		xyblock[xycount].basez=stat.z;
		xyblock[xycount].id=stat.TID;
		xyblock[xycount].flags=tile->flags;	
		xyblock[xycount].height=tile->height;
		xyblock[xycount].weight=255;

		//printf("3:%s %s %08x %04x\n",tile->name, pc->name, tile->flags, stat.TID);

		xycount++;
	}								
}

///////////////
// Name:	WalkEvaluateBlockers
// history:	cut from walking() by Duke, 20.11.2000
// Purpose:	Decides if something in the array blocks the walker
//
// Remark LB 14'th nov 2000: will rewrite this stuff when I have time again for wp-project...
// dont delete the rem'd out clConsole.send's in meanwhile, thx !
//
void WalkEvaluateBlockers(UOXSOCKET s, P_CHAR pc, signed char *pz, signed char *pdispz)
{
	if (!pc) return;
	signed char z, oldz, seekz,dispz=0;
	int num;
	int i;
	z=-128;
	oldz=pc->pos.z;	
	int loopexit=0;
		
	char gmbody;
	if (pc->isGM()) gmbody=1; else gmbody=0;
	if (pc->dead) gmbody=1; else gmbody=0;//ghosts can walk trhou doors
	// NOTE: this is clearly a bug (for living GMs gmbody is 0), but the fix enables a different bug (Duke, 14.9.03)

	do
	{
		seekz=127;
		num=-1;
		for (i=0;i<xycount;i++)
		{
			if (xyblock[i].basez+xyblock[i].height < seekz)
			{
				num=i;
				seekz=xyblock[i].basez+xyblock[i].height;
			}
		}
		
		if (num==-1)
		{
#ifdef DEBUG
			clConsole.send("(walking) Error?\n");
#endif
			xycount=0;
			z=-128;
		}
		else
		{
			unitile_st& Blocker = xyblock[num];
			if (Blocker.type!=0)
			{
				if (Blocker.height==0) Blocker.height++;
				if (Blocker.id!=1 && Blocker.basez<=oldz+MaxZstep)
				{
					if (TileData->isWet(Blocker.flags) && !(pc->isGM() || pc->dead))
					{
						//clConsole.send("A\n");
						z=-128;
					}
					else
					{
						if (gmbody)	//ghosts and GMs can walk through doors
						{
							if ( (Blocker.weight==255 && !pc->CanMoveAll()) || Blocker.type==2 )
								if ( Blocker.weight==255 || Blocker.type==2 )
								{
									if (TileData->isClimbable(Blocker.flags))
									{													
										if (Blocker.basez<oldz+MaxZstep)
										{
											//clConsole.send("x3: seekz: %i\n",seekz);
											z=Blocker.basez+Blocker.height;
											dispz=Blocker.basez+(Blocker.height/2);
											
										}
										else
										{
											if (pc->isGM() || pc->dead || TileData->isDoor(Blocker.flags))
											{
												//clConsole.send("x4: seekz: %i\n",seekz);
												dispz=z=Blocker.basez;
											}
											else
											{
												//clConsole.send("B\n");
												z=-128;
											}
										}
									}
									else
									{
										if (Blocker.basez+Blocker.height<oldz+MaxZstep)
										{
											//clConsole.send("x1: seekz: %i\n",seekz);
											dispz=z=Blocker.basez+Blocker.height;
										}
										else
										{
											if (pc->isGM() || pc->dead || TileData->isDoor(Blocker.flags))
											{
												//clConsole.send("x2: seekz: %i\n",seekz);
												dispz=z=Blocker.basez;
											}
											else
											{
												z=-128;
											}
										}
									}
								}
						}
						else
						{
							//clConsole.send("Blocker.flags=%08x, pc->priv);
							if (TileData->isBlocking(Blocker.flags) /*&& (!(Blocker.flag1&1)))*/ && !pc->isGM()) // lb, castle walk bugfix without sideffects with very.
								
							{
								//clConsole.send("D (ID=%x)\n", Blocker.id);
								if (Blocker.basez+Blocker.height>pc->pos.z)
									z=-128;
							}
							else
							{
								if (TileData->isClimbable(Blocker.flags))
								{
									//clConsole.send("ladder-z: %i height: %i\n", Blocker.basez,Blocker.height);
									if (Blocker.basez < oldz+MaxZstep)
									{
										//clConsole.send("ladder-z2: %i height: %i seekz: %i\n", Blocker.basez,Blocker.height,seekz);
										z=Blocker.basez+Blocker.height;
										dispz=Blocker.basez+(Blocker.height/2);
									}
									else
									{													
										if (pc->isGM())
											
										{
											//clConsole.send("x4: seekz: %i\n",seekz);
											
											dispz=z=Blocker.basez;
										}
										else
										{
											//clConsole.send("E\n");
											z=-128;
										}
									}
								}
								else
								{
									if (Blocker.basez+Blocker.height < oldz+MaxZstep)
									{
										//clConsole.send("x5: seekz: %i base: %i height: %i oldz: %i\n",seekz, Blocker.basez, Blocker.height, oldz);
										dispz=z=Blocker.basez+Blocker.height;
									}
									else
									{
										if (pc->isGM())
										{
											// clConsole.send("x6: seekz: %i\n",seekz);
											dispz=z=Blocker.basez;
										}
										else
										{
											if (TileData->isPlatform(Blocker.flags))
											{
												//clConsole.send("F: %08x\n", Blocker.flags);
												z=-128;
											}
										}
									}
								}
							}
						}
					}
				}
			}
			else
			{
				if ((TileData->isWet(Blocker.flags) || TileData->isBlocking(Blocker.flags)) && !pc->isGM())
				{
					//clConsole.send("G\n");
					if (s==-1) z=-128; // changed from c to s, LB 16.9.99
				}
				else
				{
					if (z==-128 || Blocker.basez+Blocker.height < oldz+MaxZstep)
					{
						//clConsole.send("x7: seekz: %i\n",seekz);
						dispz=z=Blocker.basez;
					}
				}
			}
			
			if (xycount==0) break;
			memcpy(&xyblock[num],&xyblock[xycount-1],sizeof(unitile_st));
			
			xycount--;
		}
	}
	while (xycount>0  && (++loopexit < MAXLOOPS) );
	
	*pz=z; *pdispz=dispz;
}

///////////////
// Name:	WalkHandleBlocking
// history:	cut from walking() by Duke, 27.10.2000
// Purpose:	Handles a 'real move' if the Char is not only changing direction
//
// Remark LB 14'th nov 2000: will rewrite this stuff when I have time again for wp-project...
// dont delete the rem'd out clConsole.send's in meanwhile, thx !

bool WalkHandleBlocking(UOXSOCKET s, P_CHAR pc, int sequence, int dir, int oldx, int oldy)
{
	if (!pc) return false;
	if (pc->npc)
		pc->setNextMoveTime(16); //reset move timer
	
	switch(dir&0x0F)
	{
	case '\x00' : pc->pos.y--;
		break;
	case '\x01' : { pc->pos.x++; pc->pos.y--; }
		break;
	case '\x02' : pc->pos.x++;
		break;
	case '\x03' : { pc->pos.x++; pc->pos.y++; }
		break;
	case '\x04' : pc->pos.y++;
		break;
	case '\x05' : { pc->pos.x--; pc->pos.y++; }
		break;
	case '\x06' : pc->pos.x--;
		break;
	case '\x07' : { pc->pos.x--; pc->pos.y--; }
		break;
	default:
		Console.send("ERROR: Fallout of switch statement without default in walking()\n"); //Morrolan
		Console.send("Caused by character %s. dir: %i dir&0x0f: %i dir-passed : %i dp&0x0f : %i\n",pc->name,pc->dir,pc->dir&0x0f,dir,dir&0x0f);
		// sysmessage(calcSocketFromChar(c), "Dont desparate, I'm working on that damned fu**** bug, Lord Binary");
		// pc->dir=0;
		// teleport(c);
		if (s!=-1) deny(s, pc, sequence); // lb, crashfix
		return false;
	}
	
	WalkCollectBlockers(s, pc->pos);
	
	signed char z, dispz=0;

	WalkEvaluateBlockers(s, pc, &z, &dispz);
	
	// check if player is banned from a house - crackerjack 8/12/99
	int j;

	if (pc->npc==0) // this is also called for npcs .. LB
	{
		PC_ITEM House = findHouseAt(pc->pos);
		if (House) 
		{
			int sx, sy, ex, ey;		
			j=on_hlist(House, pc->serial);
			
			if (j==H_BAN) 
			{
				Map->MultiArea(House,&sx,&sy,&ex,&ey);
				if (s!=-1) sysmessage(s, "You are banned from that location.");
				cCoord old_pos = pc->pos;
				pc->pos.x=ex;
				pc->pos.y=ey+1;
				teleport(pc, old_pos);
				if (s!=-1) deny(s,pc,sequence);
				return false;
			}
			else if ((j==H_FRIEND || pc->Owns(House)) && isTent(House->id()))
			{
				// do this only for tents because they have no doors
				// house refreshment code moved to dooruse()
				Map->MultiArea(House,&sx,&sy,&ex,&ey);
				if (pc->pos.x==ex || pc->pos.x==sx ||
					pc->pos.y==ey || pc->pos.y==sy )	// ONLY when they step on the border of the rectangle
					refreshhouse(s,pc,House->unconst());
			}
			
		} // end of is_multi
	} // end of is player

	if (pc->multis != -1)	// if they were on a boat
	{
		if (findBoatAt(pc->pos)==NULL)	// if they're no longer on a boat...
			pc->SetMultiSerial(-1);
	}
	
	if (z==-128)
	{
		pc->pos.x=oldx;
		pc->pos.y=oldy;
		if (s!=-1) deny(s, pc, sequence);
		return false;
	}
	
	int nowx2,nowy2;
	
	//Char mapRegions
	
	nowx2=pc->pos.x;
	nowy2=pc->pos.y;
	pc->pos.x=oldx;
	pc->pos.y=oldy; // we have to remove it with OLD x,y ... LB, very important
	cCoord nPos(nowx2,nowy2,z,pc->pos.map);
	pc->MoveTo(nPos);
	
	return true;
}

///////////////
// Name:	WalkSendToPlayers
// history:	cut from walking() by Duke, 27.10.2000
// Purpose:	shows the char at the new position to other players in range
//
bool WalkSendToPlayers(UOXSOCKET s, P_CHAR pc, int dir, int oldx, int oldy, int newx, int newy)
{
	if (!pc) return true;

	int map=0; // change if multimap code implemented
	cCoord old_pos(oldx,oldy,0, map);
	cCoord new_pos(newx,newy,0, map);

	int dist_old, dist_new;
	
	#define VISRANGE_EUCLIDEAN 18

	P_CHAR Watcher;	
	AllPlayersInRangeIterator api(pc);
	for (Watcher = api.First(); Watcher != NULL; Watcher = api.Next())
	{
		UOXSOCKET si = api.getSocket();
		{
			dist_old = Watcher->pos.distance(old_pos);
			dist_new = Watcher->pos.distance(new_pos);

			//printf("dist from %s to %s old:%i new:%i\n", pc->name, Watcher->name, dist_old, dist_new);
		
		    /*	if (
				(((abs(newx - Watcher->pos.x) == VISRANGE) || (abs(newy - Watcher->pos.y) == VISRANGE)) &&
				((abs(oldx - Watcher->pos.x)>VISRANGE) || (abs(oldy - Watcher->pos.y)>VISRANGE))) ||
				((abs(newx - Watcher->pos.x) == VISRANGE) && (abs(newy - Watcher->pos.y) == VISRANGE))
				)
		    */		
           
			if (      (dist_old>VISRANGE_EUCLIDEAN)  && (dist_new<=VISRANGE_EUCLIDEAN) ) // mobile ENTERS vis-range, send 0x78
			{
				if (!Watcher->isSameAs(pc)) impowncreate(si, pc, 1);
			}
			else if ( (dist_old<=VISRANGE_EUCLIDEAN) &&	(dist_new>VISRANGE_EUCLIDEAN) )	// mobile leaves vis-range, send 0x1d	
			{
				if (!Watcher->isSameAs(pc)) SndRemoveNpc(si, pc);
			}
			else if ( (dist_old>VISRANGE_EUCLIDEAN)  &&	(dist_new>VISRANGE_EUCLIDEAN) ) // old and new pos outside vis-range, do nada
			{
				return true;
			}
			else if ((dist_old<=VISRANGE_EUCLIDEAN)  &&	(dist_new<=VISRANGE_EUCLIDEAN) ) // mobile stays within vis-range, send 0x77
			{
				bool sendit;						
				if ((pc->isHidden() /*|| (pc->dead && !pc->war && !sockchar_dead)*/)
					&& !pc->isSameAs(Watcher) && !Watcher->isGM() )
					sendit = false;
				else 
					sendit = true; 
				// clConsole.send("notyourself %i walker-hidden: %i sendit %i send-to-priv %i \n",c!=currchar[si],pc->hidden,sendit,Watcher->priv&0x01);
				
				if (sendit)
				{
					unsigned char extmove[17]={0,};

					LongToCharPtr(pc->serial,&extmove[1]);
					ShortToCharPtr(pc->id(),&extmove[5]);
					PosToPtrXY(pc->pos,&extmove[7]);
					extmove[11] = pc->dispz;
					extmove[12] = dir;
					
					// running stuff
					if (pc->npc && pc->war) // Skyfire
					{
						extmove[12] = dir | 0x80;
					}
					if (pc->npc && pc->IsFollowing())
					{
						extmove[12] = dir | 0x80;
					}
					
					//** Lb's flying creatures stuff ***/
					
					if (pc->npc)
					{
						int b, cr, d;
						b = pc->id();
						if (b<2048 && b>-1) cr = NpcInfo_canItFly(b); // can it fly ?
						else cr = 0;
						
						if (cr == 1)
						{
							if (pc->fly_steps>0)
							{
								pc->fly_steps--;
								extmove[12] |= 0x80; // run mode = fly for that ones that can fly
							}
							else
							{
								unsigned int fly_p=18; // flying probability = 1/fly_p each step (if it doesnt fly)
								unsigned char fly_steps_max=27;

								if (fly_p != 0)
									d = rand()%fly_p;
								else 
									d = 0;
								if (d == 0) 
								{
									if (fly_steps_max != 0)
										pc->fly_steps = (rand()%fly_steps_max) + 2; 
									else 
										pc->fly_steps += 2;
								}
							}
						}
					}
					
					ShortToCharPtr(pc->skin, &extmove[13]);
					
					if (pc->war)
						extmove[15] = 0x40;
					else 
						extmove[15] = 0x00;
					if (pc->isHidden())
						extmove[15] |= 0x80;
					if (pc->dead && !pc->war)
						extmove[15] |= 0x80; // Ripper
					if (pc->Poisoned())
						extmove[15] |= 0x04; // AntiChrist -- thnx to SpaceDog
					int guild;
					guild = Guilds->Compare(pc, Watcher);
					if (guild == 1)// Same guild (Green)
						extmove[16] = 2;
					else if (guild == 2) // Enemy guild.. set to orange
						extmove[16] = 5;
					else
					{
						if (pc->isMurderer())
							extmove[16] = 6;		// If a bad, show as red.
						else if (pc->isInnocent())
							extmove[16] = 1;		// If a good, show as blue.
						else if (pc->flag == 0x08)
							extmove[16] = 2;		// green (guilds)
						else if (pc->flag == 0x10)
							extmove[16] = 5;		// orange (guilds)
						else  
							extmove[16] = 3;		// grey
					}
					if (!Watcher->isSameAs(pc)) 
					{
						XStartPkt(0x77, 17);
						XAddPkt(&extmove[0+1], 17-1);
						XFinishPkt(si);
					}
				} // end of it sendit
			}
		}
	}
	return true;
}

///////////////
// Name:	WalkHandleCharsAtNewPos
// history:	cut from walking() by Duke, 27.10.2000
// Purpose:	sends the newly visible Chars to the screen and checks for shoving
//
bool WalkHandleCharsAtNewPos(UOXSOCKET s, P_CHAR pc, int oldx, int oldy, int newx, int newy)
{
	if (!pc) return true;

	int map=0; // change if multimap code implemented
	cCoord old_pos(oldx,oldy,0, map);
	cCoord new_pos(newx,newy,0, map);

	int dist_old, dist_new;
	
	#define VISRANGE_EUCLIDEAN 18

	cChar* pc_i;
	int loopexit=0;
	RegGrid3x3Iterator4Chars rg3ic(pc->pos);
	while((pc_i = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{			
		if (pc_i->stablemaster_serial<=0) // dont send stabled pets to let them appear invisible
		{
			if ((pc_i->InGame()||pc_i->npc)||(pc->isGM()))//Let GMs see logged out players
			{
				dist_old = pc_i->pos.distance(old_pos);
			    dist_new = pc_i->pos.distance(new_pos);
				/*
				if (
					(((abs(newx-pc_i->pos.x)==VISRANGE)||(abs(newy-pc_i->pos.y)==VISRANGE)) &&
					((abs(oldx-pc_i->pos.x)>VISRANGE)||(abs(oldy-pc_i->pos.y)>VISRANGE))) ||
					((abs(newx-pc_i->pos.x)==VISRANGE)&& (abs(newy-pc_i->pos.y)==VISRANGE))
					)
				{
					impowncreate(s, pc_i, 1);
				}
				*/

			if (        (dist_old>VISRANGE_EUCLIDEAN)  && (dist_new<=VISRANGE_EUCLIDEAN) ) // (not moving) mobile ENTERS vis-range of moving player, send 0x78
			{
				impowncreate(s, pc_i, 1);
			}
			else if   ( (dist_old<=VISRANGE_EUCLIDEAN) && (dist_new>VISRANGE_EUCLIDEAN) )	// (not moving) mobile leaves vis-range of moving player, send 0x1d	
			{
				 SndRemoveNpc(s, pc_i);
			}
		}
	
		if (oldx == newx && oldy == newy)	// just turning ?
		continue;						    //  no multiple shoving

		if (!(
				pc->id()==0x03DB ||
				pc->id()==0x0192 ||
				pc->id()==0x0193 ||
				pc->isGMorCounselor()
				))
			{
				if (!pc_i->isSameAs(pc) && (pc_i->InGame() || (pc_i->npc && !pc_i->isDummy()))) //cant collide with dummys (thrawn)
				{
					if (pc_i->pos == pc->pos)
					{
						if (pc_i->isHidden() && !pc_i->dead && !pc_i->isInvul() && !pc_i->isGM())
						{
							if (s!=-1) sysmessage(s, "Being perfectly rested, you shoved something invisible out of the way.");
							pc->reduceStm(4);
						}
						else if (!pc_i->isHidden() && !pc_i->dead && !pc_i->isInvul() && !pc_i->isGM()) // ripper..GMs and ghosts dont get shoved.)
						{
							if (s!=-1)	
							{
								sprintf(temp,"Being perfectly rested, you shove %s out of the way.", pc_i->name);
								sysmessage(s,temp);
							}
							pc->reduceStm(4);
						}
						else if (!pc_i->isGMorCounselor() && !pc_i->isInvul())//A normal player (No priv1(Not a gm))
						{
							if (s!=-1) sysmessage(s, "Being perfectly rested, you shove something invisible out of the way.");
							pc->reduceStm(4);
						}
					}
				}
			}
		}
	}
	return true;
}

///////////////
// Name:	WalkHandleItemsAtNewPos
// history:	cut from walking() by Duke, 27.10.2000
// Purpose:	sends the newly visible items to the screen and checks for item effects
//
bool WalkHandleItemsAtNewPos(UOXSOCKET s, P_CHAR pc, int oldx, int oldy, int newx, int newy)
{
	int start_x, scan_x,scan_y;

	if (!pc) return true;
	
	PC_ITEM pi;
	int loopexit=0;
	RegGrid3x3Iterator4Items rg3ii(pc->pos);
	while((pi = rg3ii.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (pc->pos.IsSameXY(pi->pos) && pc->pos.z>=pi->pos.z && pc->pos.z<=pi->pos.z+5)
		{
			OldMagic->CheckFieldEffect(pc, pi);
			
			if (pi->trigger!=0)
			{
				if (pi->trigtype==1 && !pc->dead)
				{
					if (pi->disabled<=uiCurrentTime)//AntiChrist
					{
						triggerwitem(s,pi->unconst(),1);  //When player steps on a trigger
					}
				}
			}
		}
		
		if ( pi->id()>=0x407C && pi->id()<=0x407E )
		{
			int di=dist(pi->pos, pc->pos);
			if (di<=BUILDRANGE && di>=VISRANGE)								
			{			
				senditem(s, pi);
			}
		}	
		
		
		// Item-resend-rewritten by Lord Binary, 4.Nov.1999
		// the old code send about 500 items per new step (if screen is full of items)
		// though there are only max 32 that are really new.
		// this was a bandwidth waste and causzed client freezes and slowdown in extreme cases
		// plz dont touch, its VERY WELL tested.
		// it could be written a bit more compact, but that has no  influence on performance
		// so again, dont touch :)							
		
		switch (pc->dir)
		{
		case 7: // NW
			start_x=pc->pos.x-14;
			scan_y=pc->pos.y+2;								      
			
			for (scan_x=start_x;scan_x<=start_x+15;scan_x++) // find all items that have to be resend
			{
				if (pi->pos.x==scan_x || pi->pos.x==scan_x+1)
				{
					if (pi->pos.y==scan_y)
					{
						senditem(s,pi);
					}
				}
				scan_y--;
			}
			break;
			
		case 3: // SE
			start_x=pc->pos.x-2;
			scan_y=pc->pos.y+14;
			
			for (scan_x=start_x;scan_x<=start_x+15;scan_x++) // find all items that have to be resend
			{
				if (pi->pos.x==scan_x || pi->pos.x==scan_x+1)
				{
					if (pi->pos.y==scan_y)
					{		
						senditem(s,pi);
					}
				}
				scan_y--;
			}									   
			break;
			
		case 5: // SW
			start_x=pc->pos.x-14;
			scan_y=pc->pos.y+2;
			
			for (scan_x=start_x;scan_x<=start_x+15;scan_x++) // find all items that have to be resend
			{
				if (pi->pos.x==scan_x)
				{
					if (pi->pos.y==scan_y || pi->pos.y==scan_y+1)
					{											  
						senditem(s,pi);
					}
				}
				scan_y++;
			}
			break;
			
			
		case 1: // NE
			start_x=pc->pos.x+2;
			scan_y=pc->pos.y-14;
			
			for (scan_x=start_x;scan_x<=start_x+15;scan_x++) // find all items that have to be resend
			{
				if (pi->pos.x==scan_x)
				{
					if (pi->pos.y==scan_y || pi->pos.y==scan_y+1)
					{		  
						senditem(s,pi);
					}
				}
				scan_y++;
			}
			
			break;
			
		case 4: //S
			start_x=pc->pos.x-14;
			scan_y=pc->pos.y+2;
			
			for (scan_x=start_x;scan_x<=start_x+15;scan_x++) // find all items that have to be resend
			{
				if (pi->pos.IsSameXY(scan_x,scan_y))
					senditem(s,pi);
				scan_y++;
			}
			
			start_x=pc->pos.x-2;
			scan_y=pc->pos.y+14;
			
			for (scan_x=start_x;scan_x<=start_x+15;scan_x++) // find all items that have to be resend
			{
				if (pi->pos.IsSameXY(scan_x,scan_y))
					senditem(s,pi);
				scan_y--;
			}		
			
			break;
			
		case 0: // N
			start_x=pc->pos.x-14;
			scan_y=pc->pos.y+2;
			
			for (scan_x=start_x;scan_x<=start_x+15;scan_x++) // find all items that have to be resend
			{
				if (pi->pos.IsSameXY(scan_x,scan_y))
					senditem(s,pi);
				scan_y--;
			}
			
			start_x=pc->pos.x+2;
			scan_y=pc->pos.y-14;
			
			for (scan_x=start_x;scan_x<=start_x+15;scan_x++) // find all items that have to be resend
			{
				if (pi->pos.IsSameXY(scan_x,scan_y))
					senditem(s,pi);
				scan_y++;
			}
			break;
			
		case 2: // E
			start_x=pc->pos.x+2;
			scan_y=pc->pos.y-14;
			
			for (scan_x=start_x;scan_x<=start_x+15;scan_x++) // find all items that have to be resend
			{
				if (pi->pos.IsSameXY(scan_x,scan_y))
					senditem(s,pi);
				scan_y++;
			}
			
			start_x=pc->pos.x-2;
			scan_y=pc->pos.y+14;
			
			for (scan_x=start_x;scan_x<=start_x+15;scan_x++) // find all items that have to be resend
			{
				if (pi->pos.IsSameXY(scan_x,scan_y))
					senditem(s,pi);
				scan_y--;
			}									   
			break;
			
		case 6:
			start_x=pc->pos.x-14;
			scan_y=pc->pos.y+2;
			
			for (scan_x=start_x;scan_x<=start_x+15;scan_x++) // find all items that have to be resend
			{
				if (pi->pos.IsSameXY(scan_x,scan_y))
					senditem(s,pi);
				scan_y++;
			}
			
			start_x=pc->pos.x-14;
			scan_y=pc->pos.y+2;
			
			for (scan_x=start_x;scan_x<=start_x+15;scan_x++) // find all items that have to be resend
			{
				if (pi->pos.IsSameXY(scan_x,scan_y))
					senditem(s,pi);
				scan_y--;
			}
			
			break;
			
		default: // if dir is screwed for some resason send away the items for all directins
			int di = dist(pi->pos, pc->pos);
			if (di==11 || di==12)								
			{								       
				senditem(s, pi);
			}
		} // end of switch
		
		// end of LB's new item-resend							 
	}
	return true;
}


void WalkingHandleRainSnow(UOXSOCKET s, P_CHAR pc)
{
	if (!pc) return;
	
	/********* LB's no rain & snow in buildings stuff ***********/
	if (!pc->npc && pc->InGame()) // check for being in buildings (for weather) only for PC's
	{
		if (wtype!=0) // check only neccasairy if it rains or snows ...
		{
			int y=pc->calcSocket();
			int j=indungeon(pc); // dung-check
			int i=Map->StaticTop(pc->pos,pc->pos.z); // static check
			
			// dynamics-check
			int x=Map->DynamicElevation(pc->pos,pc->pos.z);
			if (x!=-127) if (Boats->GetBoat(y)) x=-127; // check for dynamic buildings except boats
			if (x==1 || x==0) x=-127; // 1 seems to be the multi-borders
			// bugfix LB
			
			int kk=noweather[s];    
			if (j || i || x!=-127 ) noweather[s]=1; else noweather[s]=0; // no rain & snow in static buildings+dungeons;
			if (kk-noweather[s]!=0) /*weather(y,0);*/ j=j; // iff outside-inside changes resend weather ...
			// needs to be de-rem'd if weather is available again
		}
	}
}


void WalkingHandleGlowingItems(P_CHAR pc)
{
	int i;
	if (pc == NULL)
		return;
	if (pc->InGame())
	{
		int serial,serhash,ci;
		serial=pc->serial;
		serhash=serial%HASHMAX;
		for (ci=0;ci<glowsp[serhash].max;ci++)
		{
			i=glowsp[serhash].pointer[ci];
			P_ITEM iItem = MAKE_ITEM_REF(i);
			if (iItem!=NULL)
			{
				pc->glowHalo(iItem);
			}
		}
	}
}


unsigned char calcWalkingNotority(P_CHAR Player)
{
	if (Player->isHidden()) return 0; // not sure, copied from old wrong code, have to check into this

	/*
	notoriety:   
	
	0 = invalid/across server line
    1 = innocent (blue)
    2 = guilded/ally (green)
    3 = attackable but not criminal (gray)
    4 = criminal (gray)
    5 = enemy (orange)
    6 = murderer (red)
    7 = unknown use (translucent (like 0x4000 hue))
	*/

	return 1;
}

void walking(P_CHAR pc, int dir, int sequence)
{	
	int newx, newy;
	
	UOXSOCKET s=pc->calcSocket();

	if (!WalkHandleAllowance(s,pc,sequence))		// check sequence, frozen, weight etc.
	{		
		return;
	}

	WalkHandleRunning(s,pc,dir);

	int oldx=pc->pos.x;
	int oldy=pc->pos.y;

	if ((dir&0x0F)==pc->dir)
	{
		if (!WalkHandleBlocking(s,pc,sequence,dir, oldx, oldy))
		{			
			return;
		}
	}

	if (s!=-1)
	{
		unsigned char walkok[4]="\x22\x00\x01";
		walkok[1]=sequence;
		walkok[2]=calcWalkingNotority(pc);
		
		XStartPkt(0x22, 3);
		XAddPkt(&walkok[0+1], 3-1);
		XFinishPkt(s);
		
		walksequence[s]=sequence;
		if (walksequence[s]>=255) walksequence[s]=1; // initially starts with 0 but after first 255 always 1's following 255's (not 0)
		
		static int fw2=0;
		
		if (SrvParms->antifastwalk)
		{
			unsigned char addkey[9]={0xbf,0x0,0x9,0x0,0x2,0x0,0x0,0x0,0x0};
			LongToCharPtr(0x88888800+(fw2), addkey+5);
			XStartPkt(0xBF, 9);
			XAddPkt(&addkey[0+3], 9-3);
			XFinishPkt(s);
			fw2++;
		}
	}
	
	newx=pc->pos.x;
	newy=pc->pos.y;

	WalkSendToPlayers(s,pc,dir, oldx, oldy, newx, newy);

	if (dir>-1 && (dir&0x0F)<8) { pc->dir=(dir&0x0F);}
	else { LogErrorVar("dir-screwed : %i",dir); }

	if (s!=-1)
	{
		WalkHandleCharsAtNewPos(s, pc, oldx, oldy, newx, newy);
		WalkHandleItemsAtNewPos(s, pc, oldx, oldy, newx, newy);
	}
  
	if (!pc->pos.IsSameXY(oldx,oldy))
	{
        if (pc->npc==0 || pc->questType || pc->tamed)
            objTeleporters(pc); 
		teleporters(pc);
	}

	// WalkingHandleRainSnow(s,c); // while rain and snow are disabled its a waste of CPU cycles

	WalkingHandleGlowingItems(pc);
    
    OldMagic->GateCollision(pc);

    checkregion(pc); 
	
	if (Skills->GetCombatSkill(pc) == ARCHERY)  // -Frazurbluu- add in changes for archery skill, and dexterity
    {                                        //  possibly weapon speed?? maybe not, cause crossbows notta running shooting   
		if (pc->HasTarget())
        {
            if (pc->timeout>= uiCurrentTime)
               pc->timeout= uiCurrentTime + (3*CLOCKS_PER_SEC);

		}
    } 
}


void walking2(P_CHAR pc) // Only for switching to combat mode
{
	if (!pc) return;
	unsigned int i;
	int sendit;

	for (i = 0; i < now; i++)
	{
		if (perm[i])
		{
			P_CHAR Watcher = MAKE_CHAR_REF(currchar(i));
			if (!Watcher) continue;
			if (inrange1pl(pc, currchar(i)))
			{
				if ((pc->isHidden() || (pc->dead && !pc->war)) && !Watcher->isSameAs(pc) && !Watcher->isGM())
				{
					if (!Watcher->dead)
					{
						SndRemoveNpc(i, pc);
						sendit = 0;
					}
					else
						sendit = 1;
				}
				else 
					sendit = 1; // LB 9-12-99 client 1.26.2 fix
				
				if (sendit)
				{
					unsigned char extmove[17]={0,};

					LongToCharPtr(pc->serial,&extmove[1]);
					ShortToCharPtr(pc->id(),&extmove[5]);
					PosToPtrXY(pc->pos,&extmove[7]);
					extmove[11] = pc->dispz;
					extmove[12] = pc->dir&0x7F;
					
					// running stuff
					int dir = pc->dir;
					if (pc->npc && pc->war) // Skyfire
					{
						extmove[12] = dir | 0x80;
						XStartPkt(0x77, 17);
						XAddPkt(&extmove[0+1], 17-1);
						XFinishPkt(i);
					}
					if (pc->npc && pc->IsFollowing())
					{
						extmove[12] = dir | 0x80;
						XStartPkt(0x77, 17);
						XAddPkt(&extmove[0+1], 17-1);
						XFinishPkt(i);
					}
					
					ShortToCharPtr(pc->skin, &extmove[13]);
					
					if (pc->war)
						extmove[15] = 0x40;
					else 
						extmove[15] = 0x00;
					if (pc->isHidden())
						extmove[15] |= 0x80;
					if (pc->Poisoned())
						extmove[15] |= 0x04; // AntiChrist -- thnx to SpaceDog
					if (pc->kills >= 4)
						extmove[16] = 6; // ripper
					int guild;
					guild = Guilds->Compare(pc, Watcher);
					if (guild == 1)// Same guild (Green)
						extmove[16] = 2;
					else if (guild == 2) // Enemy guild.. set to orange
						extmove[16] = 5;
					else 
					{
						if (pc->isMurderer())      // show red
							extmove[16] = 6;
						else if (pc->isInnocent()) // show blue
							extmove[16] = 1;
						else if (pc->flag == 0x08) // show green
							extmove[16] = 2;
						else if (pc->flag == 0x10) // show orange
							extmove[16] = 5;
						else 
							extmove[16] = 3;            // show grey
					}
					// end of if sendit
					
					if (!pc->war) // we have to execute this no matter if invisble or not LB
					{
						pc->setAttacker(NULL);
						pc->ResetTarget();
					}
					
					if (sendit)
					{
						XStartPkt(0x77, 17);
						XAddPkt(&extmove[0+1], 17-1);
						XFinishPkt(i);						
					}
				}
			}
		}
	}
}

int npcSelectDir(P_CHAR pc_i, int j)
{
	if (pc_i->blocked)
	{
		short x=0;
		if (j/2.0!=j/2)
			x=1;
		if (pc_i->blocked<=2)
			j =pc_i->dir2;	/* =(j-2-x)%8; //works better  ????*/
		else
		{
			if (rand()%2) j=pc_i->dir2 = static_cast<char>((j-2-x)%8);
			else j=pc_i->dir2 = static_cast<char>((j+2+x)%8);
		}
	}
	if (j<0)
		j=rand()%8;
	return j;
}

int npcSelectDirWarOld(P_CHAR pc_i, int j)
{
	if (pc_i->blocked)
	{
		short x=0;
		if (j/2.0!=j/2)
			x=1;
		if (pc_i->blocked<=2)
			j =pc_i->dir2 =(j-2-x)%8; //works better  ????
		else
		{
			if (rand()%2) j=pc_i->dir2=(j-2-x)%8;
			else j=pc_i->dir2=(j+2+x)%8;
		}
	}
	if (j<0)
		j=rand()%8;
	return j;
}

class cWalkingAI
{
private:
	short results;
	int startX, startY;
	int targetX, targetY;
	int zz;
	P_CHAR Npc;
public:
	void Init(P_CHAR pNpc, int desiredDir, int tx, int ty)
	{
		results = 0;
		startX = pNpc->pos.x;
		startY = pNpc->pos.y;
		targetX = tx;
		targetY = ty;
		zz = pNpc->pos.z;
		Npc = pNpc;
	}

	short dirBitChecked(int dir) { int i=(1<<dir);
	return i; }
	//return (1<<dir); }
	short dirBitBlocked(int dir) { return (1<<(dir+8)); }

	bool TileBlocks(int dir)		// check if tile blocks
	{
		if (results & dirBitChecked(dir))	// avoid double checking
			return (results & dirBitBlocked(dir)) ? true : false;
		int chkX = startX;
		int chkY = startY;
		GetValidXYfromDir(dir,&chkX,&chkY);		// get coords of the location we want to walk
		results |= dirBitChecked(dir);
		if (!validNPCMove(chkX, chkY, zz, Npc))
			results |= dirBitBlocked(dir);
		return (results & dirBitBlocked(dir)) ? true : false;
	}

	bool DirBlocks(int dir)		// check if we can reach tile, return true if blocked
	{
		if (TileBlocks(dir))
			return true;		// the tile itself is blocking
		if (!(dir & 1))			// square ie. N,S,W or E
			return false;		// perfect! we can just go where we want to go
		else					// diagonal ie. NE,SE,SW or SE ??
		{						// We'll have to check the adjacent tiles too
			if (!TileBlocks(getLeftDir(dir)))
				return false;
			if (!TileBlocks(getRightDir(dir)))
				return false;
		}
		return true;
	}
	
	double dist(int xa, int ya, int xb, int yb) // Distance between position a and b
	{
		return hypot( abs(xa-xb), abs(ya-yb) ); // hypot is POSIX standard
	}
	
	double distToTarget(int dir)
	{
		int chkX1 = startX;
		int chkY1 = startY;
		GetValidXYfromDir(dir,&chkX1,&chkY1);		// get coords of the location we want to check
		return dist(startX,startY,chkX1,chkY1);
	}

	int ClosestDir(int dir1, int dir2)
	{
		if ( distToTarget(dir1) < distToTarget(dir2) )
			return dir1;
		else
			return dir2;
	}

	int PreferredDir(int dir1, int dir2)
	{
		int dir1stTry = ClosestDir(dir1,dir2);
		if (!DirBlocks(dir1stTry))
			return dir1stTry;
		int dir2ndTry = (dir1stTry==dir1 ? dir2 : dir1);
		if (!DirBlocks(dir2ndTry))
			return dir2ndTry;
		return -1;
	}

	int SelectDirWar(int desiredDir)
	{
		int dir = -1;
		// step 1: let's see if we can just go where we want to go
		if (!DirBlocks(desiredDir))
			return desiredDir;				// perfect!

		
		// step 2: the direct way is blocked, find another way...
		int dirLeft1  = getLeftDir (desiredDir);
		int dirRight1 = getRightDir(desiredDir);

		dir = PreferredDir(dirLeft1,dirRight1);
		if (dir != -1)
			return dir;

		
		// step 3: let's try to go sideways...
		int dirLeft2  = getLeftDir (dirLeft1);
		int dirRight2 = getRightDir(dirRight1);

		dir = PreferredDir(dirLeft2,dirRight2);
		if (dir != -1)
			return dir;

		
		// step 4: let's try to go backwards...
		int dirLeft3  = getLeftDir (dirLeft2);
		int dirRight3 = getRightDir(dirRight2);

		dir = PreferredDir(dirLeft3,dirRight3);
		if (dir != -1)
			return dir;


		// step 5: only one place left to go...
		int opposite  = getLeftDir (dirLeft3);
		if (!DirBlocks(opposite))
			return opposite;

		return -1;				// we are totally blocked !
	}
	
};

//NEW NPCMOVEMENT ZIPPY CODE STARTS HERE -- AntiChrist meging codes --
void npcMovement2(unsigned int currenttime, P_CHAR pc_i)//Lag fix
{
	if (!pc_i) return;
	int j, dnpctime;
	
	dnpctime=0;
    if (pc_i->npc && (pc_i->npcmovetime<=currenttime||(overflow)))
    {
		if (pc_i->war && pc_i->npcWander != 5) 
		{ // a very simple pathfinding algorithm
			P_CHAR Attacker = pc_i->getAttacker();
			if (Attacker)
			{
				if ((pc_i->dist(Attacker)>1 || chardir(pc_i, Attacker)!=pc_i->dir))
				{
					if ( (!Attacker->npc && Attacker->InGame()) || Attacker->npc)				
					{
						j=chardir(pc_i, Attacker);
#if 0
						// this code is disabled because there is a bug (?) in validNPCMove() calling DoesStaticBlock()
						// eg. 'brambles' are considered 'non-blocking' but the client doesn't let you go there !
						// thus the new code called here is running into the same probs as the old code :(
						// I have no clue yet :(  (Duke 18.7.01)
						cWalkingAI wai;
						wai.Init(pc_i, j, Attacker->x, Attacker->y);
						j=wai.SelectDirWar(j);
						if (j==-1) return;	// totally blocked. Don't walk.
#else
						j=npcSelectDirWarOld(pc_i,j);
#endif

						npcwalk(pc_i,j%8,0);
					}
				}
			}
		} 
		
		else
		{
			switch(pc_i->npcWander)
			{
			case 0: // No movement
				break;
			case 1: // Follow the follow target
				{
				P_CHAR ToFollow = pc_i->GetFollowTarget();
				if (!ToFollow)
				{
					pc_i->npcWander=4;	// wander freely again
					return;
				}

				if (ToFollow->dead || ToFollow->disabled>uiCurrentTime) break; //antichrist

				// Dupois - Added April 4, 1999
				// Has the Escortee reached the destination ??
				if (pc_i->questDestRegion==pc_i->region)
				{
					// Pay the Escortee and free the NPC
					MsgBoardQuestEscortArrive( pc_i, ToFollow->calcSocket() );
				}
				// End - Dupois
				
				if (pc_i->dist(ToFollow) >1
					|| chardir(pc_i, ToFollow)!=pc_i->dir)
				{
					j=chardir(pc_i, ToFollow);
					j=npcSelectDir(pc_i,j);
					dnpctime=1;
					npcwalk(pc_i,j%8,0);
				}
				break;
				}
			case 2: // Wander freely, in a defined circle
				j=rand()%40;
				if (j<8 || j>32) dnpctime=1;
				if (j>7 && j<33) // Let's move in the same direction lots of the time.  Looks nicer.
					j=pc_i->dir;
				npcwalk(pc_i,j%8,2); // lb bugfix, was 0
				break;
			case 3: // Wander freely, within a defined box
				j=rand()%40;
				if (j<8 || j>32) dnpctime=1;
				if (j>7 && j<33) // Let's move in the same direction lots of the time.  Looks nicer.
					j=pc_i->dir;
				npcwalk(pc_i,j%8,1);
				break;
			case 4:  // wander freely, avoiding obstacles
				
				j=rand()%40;
				if (j<8 || j>32) dnpctime=1;
				if (j>7 && j<33) // Let's move in the same direction lots of the time.  Looks nicer.
					j=pc_i->dir;
				npcwalk(pc_i,j%8,0); 
				
				break;
			case 5: //FLEE!!!!!!
				{
					P_CHAR pc_k = pc_i->GetTarget();
					if (!pc_k) break;
					if (pc_i->dist(pc_k)<15)
					{
						j=(chardir(pc_i, pc_k)+4)%8;
						j=npcSelectDir(pc_i,j);
						npcwalk(pc_i,j%8,0);
					}
					break;
				}
			default:
				Console.send("ERROR: Fallout of switch statement without default [%i]. walking.cpp, npcMovement2()\n",pc_i->npcWander); //Morrolan
			}
		}

		//AntiChrist
		if (pc_i->npcWander==1)
			pc_i->npcmovetime=(unsigned int)(currenttime+double(NPCSPEED*MY_CLOCKS_PER_SEC*(dnpctime)/4)); //reset move timer
		else
			pc_i->npcmovetime=(unsigned int)(currenttime+double(NPCSPEED*MY_CLOCKS_PER_SEC*(dnpctime))); //reset move timer
	}
}

int checkBoundingBox(cCoord& chkPos, int fx1, int fy1, int fz1, int fx2, int fy2)
{
	if (chkPos.x >= ((fx1<fx2)?fx1:fx2) && chkPos.x <= ((fx1<fx2)?fx2:fx1))
		if (chkPos.y >= ((fy1<fy2)?fy1:fy2) && chkPos.y <= ((fy1<fy2)?fy2:fy1))
			if (fz1==-1 || abs(fz1-Map->Height(chkPos, fz1))<=5)
				return 1;
			return 0;
}

int checkBoundingCircle(cCoord& chkPos, int fx1, int fy1, int fz1, int radius)
{
	if ( (chkPos.x-fx1)*(chkPos.x-fx1) + (chkPos.y-fy1)*(chkPos.y-fy1) <= radius * radius)
		if (fz1==-1 || abs(fz1-Map->Height(chkPos, fz1))<=5)
			return 1;
		return 0;
}

int checkBounds(P_CHAR pc, int newX, int newY, int type)
{
	cCoord chkPos(newX,newY,pc->pos.z,pc->pos.map);
	int move=0;
	switch (type)
	{
	case 0: move=1;break;
	case 1: move=checkBoundingBox(chkPos, pc->fx1, pc->fy1, pc->fz1, pc->fx2, pc->fy2);break;
	case 2: move=checkBoundingCircle(chkPos, pc->fx1, pc->fy1, pc->fz1, pc->fx2);break;
	default: move=0;	// invalid type given
	}
	return move;
}

// Restructured & bugfixed by LB 17 Septemer 1999
// Thx to JM for help.

void npcwalk(P_CHAR pc_i, int j, int type)   //type is npcwalk mode (0 for normal, 1 for box, 2 for circle)
{
	int cr,k;
	signed char z;
	char v1,v2,v3,valid,move;

	if (!pc_i) return;

	z = pc_i->pos.z;
	//Bug Fix -- Zippy
	if (pc_i->IsFrozen()) return;//Frozen - Don't send them al the way to walking to check this, just do it here.

	//clConsole.send("  npcw-2 c-dir: %i j-dor: %i\n",pc_i->dir,j);


	//clConsole.send("callwe-w: %i t: %i dir: %i\n",caller,type,j);

	/////////// LB's flying creatures stuff, flying animation if they stand still ///////
  
	short b=pc_i->id();
	if (b<2048 && b>-1) cr=NpcInfo_canItFly(b); // can it fly ?  
	else cr=0;
	if (cr==1)
	{
		if (pc_i->fly_steps>0)
		{
			if (rand()%3==0) pc_i->DoAction(0x13); // flying animation
		} 
	}
	
	/////////////////////////////////////////////////////////////////////////////////////

	//clConsole.send("name: %s j-dir:%i, c-dir:%i type:%i\n",pc_i->name,j,pc_i->dir,type);

	k=pc_i->dir;
	if (k==j)  // If we're moving, not changing direction
	{
		int newX = pc_i->pos.x;
		int newY = pc_i->pos.y;
		GetValidXYfromDir(pc_i->dir,&newX,&newY);	// get coords of the location we want to walk

		switch(j) // Switch, based on the direction we're going to move.
		{
		case 0: // North
		case 2:
		case 4:
		case 6:
			valid=validNPCMove(newX, newY, z, pc_i);
			if (valid)
			{
				move=checkBounds(pc_i, newX, newY, type);
				if (move)
					walking(pc_i,j,256);
			}
			break;
			
		case 1: // Northeast
		case 3:
		case 5:
		case 7:
			v1 = validNPCMove(newX, newY, z, pc_i); // check northeast
			if (!v1) valid=0;	// if it blocks -> no move 
			else				// if not, check if both, N & E are blocking. 
								// if so it blocks too, otherwise its ok.
			{
				int leftX = pc_i->pos.x;
				int leftY = pc_i->pos.y;
				GetValidXYfromDir(getLeftDir(pc_i->dir),&leftX,&leftY);	// get coords of the left location we want to pass
				
				v2 =  validNPCMove(leftX, leftY, z, pc_i);
				if (v2) valid=1; 
				else
				{ 
					int rightX = pc_i->pos.x;
					int rightY = pc_i->pos.y;
					GetValidXYfromDir(getRightDir(pc_i->dir),&rightX,&rightY);	// get coords of the right location we want to pass

					v3= validNPCMove(rightX, rightY, z, pc_i);
					if (v3) valid=1; else valid=0;
				}
			}
			
			  // LB remark: NW-block AND (N-block OR W-block) is the same more compact, BUT ...
			  // could be slower, depending on the compilers optimization.
              // I dont trust msvc, thus the "hand-optimization

			//clConsole.send("val: %i v1%i v2%i v3%i\n",valid,v1,v2,v3);

			if (valid)
			{
				move=checkBounds(pc_i, newX, newY, type);
				if (move)
				{ /*clConsole.send("calling w\n");*/walking(pc_i,j,256);}
			}
			break;
		}
	}
	else 
	{        
		//clConsole.send("changing dir?\n");
		walking(pc_i, j, 256);
	} 
}

int validNPCMove(int x, int y, signed char z, P_CHAR pc_s)
{
	if (!pc_s) return 0;

	cCoord newPos(x,y,z,pc_s->pos.map);
    pc_s->blocked++;
	cItem* pi;
	int loopexit=0;

	RegCellIterator4Items rcii(newPos);
	while((pi = rcii.Next()) != NULL && (++loopexit < MAXLOOPS) )
	{
		if (!pi->pos.IsSameXY(x,y)) continue;
		if (pi->isMulti()) continue;

		const tileData *tile;
		tile = TileData->SeekTile(pi->id());
		if (pi->pos.z+tile->height>z+1 && pi->pos.z<z+MaxZstep)
		{
			// bugfix found by JustMichael, moved by crackerjack
			// 8/2/99 makes code run faster too - one less loop :)
			if (pi->id()==0x3946 || pi->id()==0x3956) return 0;
			if (pi->id()<=0x03E2) return 0;
			if (pi->id()>0x0854 && pi->id()<0x0866) return 0;
			
			if (pi->isType(ITY_Door))
			{
				if (pc_s->npc && (strlen(pc_s->title) > 0 || pc_s->npcaitype != 0))
				{                            
					// clConsole.send("doors!!!\n");
					dooruse(-1, pi);
				}                                   
				pc_s->blocked=0;
				return 0;
			}
		}
    }

	// experimental check for bad spawning/walking places, not optimized in any way (Duke, 3.9.01)
	int mapid = 0;
#if 0
	map_st map = Map->SeekMap0( x, y );
	mapid = map.id;
#else
	signed char mapz = Map->AverageMapElevation(newPos, mapid);	// just to get the map-ID
	if (mapz != illegal_z)
#endif

	{
		if ((mapid >= 0x25A && mapid <= 0x261) ||	// cave wall
			(mapid >= 0x266 && mapid <= 0x26D) ||	// cave wall
			(mapid >= 0x2BC && mapid <= 0x2CB) )	// cave wall
			return 0;
		if ( mapid >= 0x0A8 && mapid <= 0x0AB) 	// water (ocean ?)
			return 0;
//		land_st land;
//		Map->SeekLand(mapid, &land);
	}
	
	
	// check if other NPC's block
	if (speed.antinpcaggregation)
	{	  
		cChar* pc;
		int loopexit=0;
		RegGrid3x3Iterator4Chars rg3ic(newPos);
		while((pc = rg3ic.Next()) != NULL && (++loopexit < MAXLOOPS) )
		{
			if (pc->npc && pc->serial!=pc_s->serial) // no self-avoidance ;-)
			{             
				int p = newPos.distance(pc->pos);		
				if (p<=0) return 0;       // walking to fellow npc, that's blocking for sure 
				else 
				{
					int pepper=rand()%100; // kind of avoidance, with small probability they wont move even if no DIRECT collision but within radius <=3
					if (pepper>=97 && p==3) return 0;
					if (pepper>=91 && p==2) return 0;
					if (pepper>=87 && p==1) return 0;				
				}
			}
		}
	}
	
	
    // see if the map says its ok to move here
    if (Map->CanMonsterMoveHere(newPos, z))
    {
		pc_s->blocked = 0;
		return 1;
    }

				 
    return 0;
}

bool canNpcStandHere(short x, short y);
bool canNpcStepHere (short x, short y, int capabilities, signed char oldZ);
bool canNpcSpawnHere(short x, short y, int capabilities);
