/////////////////////////////////////////
//
//	This software is LICENSED material !!
//	see LoneWolf.cpp for details.
//
//

// WorldMain.cpp: implementation of the CWorldMain class.
//
//////////////////////////////////////////////////////////////////////

#include "worldmain.h"

#include "archive.h"
#include "boats.h"
#include "guilds.h"
#include "mapstuff.h"
#include "regions.h"
#include "skills.h"
#include "SndPkg.h"
#include "sregions.h"
#include "TmpEff.h"
#include "utilsys.h"
#include "multi.h"
#include "mapS.h"
#include "network.h"

#include "wolfpack.h"

#include "debug.h"
#undef  DBGFILE
#define DBGFILE "worldmain.cpp"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CWorldMain::CWorldMain()
{
	save_counter = 0;
	state = 0;
	announce(false);
	LastSaveTime=uiCurrentTime;
	iWsc = cWsc = NULL;
	Cur = 0;
	isSaving = false;
}

CWorldMain::~CWorldMain()
{
	if (iWsc)
		fclose(iWsc);
	if (cWsc)
		fclose(cWsc);
}

void CWorldMain::CheckSaving()
{
	switch (state)
	{
	case 0:
		if (LastSaveTime+((SrvParms->saveinterval-10)*MY_CLOCKS_PER_SEC) <= uiCurrentTime)
		{
			sysbroadcast("World will be saved in 10 seconds..");
			state = 1;
		}
		break;
	case 1:
		if (LastSaveTime+(SrvParms->saveinterval*MY_CLOCKS_PER_SEC) <= uiCurrentTime)
		{
			if ( announce() )
			{
				sysbroadcast("World data saving....");
				Console.send("\nWorldsave Started!\n" );
				Console.send("items: %i / %i\n",Items->GetCount() , Items->GetCapacity() );
			}
			state = 2;
			SaveStartTime = uiCurrentTime;
		}
		break;
	case 2:
		this->savenewworld(0);
		if (!this->Saving() )	// indicates the end of the saving process
		{
			state = 3;
			SaveEndTime = uiCurrentTime;
		}
		break;
	case 3:
		LastSaveTime = uiCurrentTime;
		state = 0;
		if ( announce() )
		{
			char tt[99];
			sprintf(tt,"Worldsave Done! (%i ms)\n", SaveEndTime-SaveStartTime);
			sysbroadcast(tt);
			Console.send("Worldsave Done!\n");
		}	

		break;
	}
}


void loadchar() // Load a character from WSC
{
	unsigned long k;
	int i ;
	int j, loops=0;

	P_CHAR pc = Npcs->MakeBaseChar(false);	// false coz we don't need a new serial here
	if (!pc) return;

	pc->hungertime=uiCurrentTime+(SrvParms->hungerrate*MY_CLOCKS_PER_SEC/2);	// avoid loss of one hungerpoint for each restart (Duke, 6.6.2001)

	do
	{
		readw2();
		int rc = Npcs->ApplyCommonTags(pc);				// tags common to scp, wsc and monstergate
		if (rc) {if (rc==1) continue; else return;}

		switch(script1[0])
		{
		case 'A':
			if (!strcmp(script1, "ACCOUNT"))				{ pc->account=str2num(script2); }	
			else if (!strcmp(script1, "ATT"))				{ pc->att=str2num(script2);}
			else if (!strcmp(script1, "ADVOBJ"))			{ pc->advobj=str2num(script2);}
		break;

		case 'B':
			if (!strcmp(script1, "BODY"))					{ pc->setId(str2num(script2));}
		break;

		case 'C':
			if (!strcmp(script1, "CREATIONDAY"))			{ pc->creationday = str2num(script2); }
			if (pc->creationday>2000) 
			{
				sprintf(temp, "%s: Converted old (invalid) CREATEDAY value to new one (today)\n", pc->name);
				Console.send(temp);
				pc->creationday=getPlatformDay();
			}

			else if (!strcmp(script1, "CARVE"))				pc->carve=str2num(script2);
			else if (!strcmp(script1, "CANTRAIN"))			pc->cantrain = true; 
			else if (!strcmp(script1, "CLONED"))			pc->isclone = true;
		break;

		case 'D':
			if (!strcmp(script1, "DISPZ"))					pc->dispz=str2num(script2);
			else if (!strcmp(script1, "DIR"))				pc->dir=str2num(script2);
			else if (!strcmp(script1, "DEXTERITY"))			pc->setDex(str2num(script2));
			else if (!strcmp(script1, "DEXTERITY2"))		pc->setDecDex(str2num(script2));
			else if (!strcmp(script1, "DEXTERITYLIMIT"))	pc->setDexLimit(str2num(script2));
			else if (!strcmp(script1, "DEATHS"))			pc->deaths=str2num(script2);
			else if (!strcmp(script1, "DEAD"))				pc->dead = true;	// dead only appears in file if it's true.
			else if (!strcmp(script1, "DEF"))				pc->def=str2num(script2);
			else if (!strcmp(script1, "DISABLED"))			pc->disabled=uiCurrentTime+(str2num(script2)*MY_CLOCKS_PER_SEC);
			else if (!strcmp(script1, "DISABLEMSG"))		strncpy(pc->disabledmsg,script2,255);
			else if (!strcmp(script1, "DOPPELGAENGERSERIAL")) pc->doppelgaenger_serial=str2num(script2);
			
		break;

		case 'E':
			if (!strcmp(script1, "EMOTE"))				pc->emotecolor=str2num(script2);
		break;

		case 'F':
				 if (!strcmp(script1, "FIXEDLIGHT"))	pc->fixedlight=str2num(script2);
			else if (!strcmp(script1, "FOLLOW"))		pc->ftarg=str2num(script2);	// temporarily store the serial
			else if (!strcmp(script1, "FONT"))			pc->fonttype=str2num(script2);
			else if (!strcmp(script1, "FLEEAT"))		pc->fleeat=str2num(script2);
			else if (!strcmp(script1, "FX1"))			pc->fx1=str2num(script2);
			else if (!strcmp(script1, "FY1"))			pc->fy1=str2num(script2);
			else if (!strcmp(script1, "FZ1"))			pc->fz1=str2num(script2);
			else if (!strcmp(script1, "FX2"))			pc->fx2=str2num(script2);
			else if (!strcmp(script1, "FY2"))			pc->fy2=str2num(script2);
			else if (!strcmp(script1, "FOOD"))			pc->SetFood(str2num(script2));
		break;

		case 'G': 
			if (!strcmp(script1,"GUILD TYPE") || !strcmp(script1,"GUILDTYPE")) 
			{ 
				i = str2num(script2);	// Get the guild type 
				if (i<-1||i>MAX_GUILDTYPE) 
					i=-1;     // Default to no guild type 
				pc->GuildType=i; 
			} 
			else if (!strcmp(script1,"GUILD TRAITOR") || !strcmp(script1,"GUILDTRAITOR")) 
			{ 
				if (!strcmp(script2, "YES")) 
					pc->GuildTraitor=true; // yes this character has switch guilds 
				else 
					pc->GuildTraitor=false; 
			} 
			else if (!strcmp(script1, "GUILDTOGGLE"))   pc->guildtoggle=str2num(script2);
			else if (!strcmp(script1, "GUILDNUMBER"))   pc->guildnumber=str2num(script2);
			else if (!strcmp(script1, "GMRESTRICT"))    pc->gmrestrict=str2num(script2); 
		    else if (!strcmp(script1, "GUILDTITLE"))    strcpy(pc->guildtitle,script2);
		    else if (!strcmp(script1, "GUILDFEALTY"))   pc->guildfealty=str2num(script2);
			else if (!strcmp(script1, "GMMOVEEFF"))	   pc->gmMoveEff=str2num(script2);	// Reads in the Gm movement effect int //Aldur 
		break;

		case 'H':
			if (!strcmp(script1, "HITPOINTS"))			  pc->setHp(str2num(script2));
			else if (!strcmp(script1, "HIDDEN"))			  pc->hidden=str2num(script2);
			else if (!strcmp(script1, "HUNGER"))			  pc->setHunger(str2num(script2));
			else if (!strcmp(script1, "HOLDGOLD"))		  pc->holdg=str2num(script2);
		break;

		case 'I':
				 if (!strcmp(script1, "INTELLIGENCE"))	  pc->setInt(str2num(script2));
			else if (!strcmp(script1, "INTELLIGENCE2"))	  pc->setDecInt(str2num(script2));
			else if (!strcmp(script1, "INTELLIGENCELIMIT"))pc->setIntLimit(str2num(script2));
		break;

		case 'J': 
			if (!strcmp(script1, "JAILTIMER"))			  pc->jailtimer.setSecs(str2num(script2));
			else if (!strcmp(script1, "JAILCELL"))		  { pc->cell=str2num(script2); jails[pc->cell].occupied = true;} 
		break;

		case 'K':
			if (!strcmp(script1, "KILLS"))				  pc->kills=str2num(script2);
		break;

		case 'M':		
			 if (!strcmp(script1, "MANA"))					pc->mn=str2num(script2);
			 else if (!strcmp(script1, "MAP"))				pc->pos.map=str2num(script2);
		break;

		case 'N':
			if (!strcmp(script1, "NPC"))				  pc->npc=str2num(script2);
			else if (!strcmp(script1, "NPCWANDER"))		  pc->npcWander=str2num(script2);
			else if (!strcmp(script1, "NOTRAIN"))		  pc->cantrain=false;
			else if (!strcmp(script1, "NPCTYPE"))		  pc->npc_type=str2num(script2);
			else if (!strcmp(script1, "NPCAITYPE"))		  pc->setAi(str2num(script2));
		break;

		case 'O':
			if (!strcmp(script1, "OLDNPCWANDER"))		  pc->oldnpcWander=str2num(script2);
			else if (!strcmp(script1, "OLDX"))			  jails[pc->cell].oldpos.x=str2num(script2);
			else if (!strcmp(script1, "OLDY"))			  jails[pc->cell].oldpos.y=str2num(script2);
			else if (!strcmp(script1, "OLDZ"))			  jails[pc->cell].oldpos.z=str2num(script2);
			else if (!strcmp(script1, "OWN") || !strcmp(script1, "OWNER"))	// for backward compatibility
			{
				pc->SetOwnSerial(str2num(script2));
			}
		break;

		case 'P':
			if (!strcmp(script1, "PRIV"))					pc->setPriv(str2num(script2));
			else if (!strcmp(script1, "PRIV2"))				pc->SetPriv2(str2num(script2));
			else if (!strcmp(script1, "PRIV4"))				pc->SetPriv4(str2num(script2));
			else if (!strcmp(script1, "PRIV3a"))			pc->priv3[0]=str2num(script2);
			else if (!strcmp(script1, "PRIV3b"))			pc->priv3[1]=str2num(script2);
			else if (!strcmp(script1, "PRIV3c"))			pc->priv3[2]=str2num(script2);
			else if (!strcmp(script1, "PRIV3d"))			pc->priv3[3]=str2num(script2);
			else if (!strcmp(script1, "PRIV3e"))			pc->priv3[4]=str2num(script2);
			else if (!strcmp(script1, "PRIV3f"))			pc->priv3[5]=str2num(script2);
			else if (!strcmp(script1, "PRIV3g"))			pc->priv3[6]=str2num(script2);
			else if (!strcmp(script1, "POISONED"))			pc->poisoned=str2num(script2);
		break;

		case 'Q':
			if (!strcmp(script1, "QUESTTYPE"))				pc->questType=str2num(script2);
			else if (!strcmp(script1, "QUESTDESTREGION"))	pc->questDestRegion=str2num(script2);
			else if (!strcmp(script1, "QUESTORIGREGION"))	pc->questOrigRegion=str2num(script2);
			else if (!strcmp(script1, "QUESTBOUNTYPOSTSERIAL")) pc->questBountyPostSerial=str2num(script2);
			else if (!strcmp(script1, "QUESTBOUNTYREWARD")) pc->questBountyReward=str2num(script2);
			else if (!strcmp(script1, "QSF"))				pc->qsf=str2num(script2);
		break;

		case 'R':
			if (!strcmp(script1, "RACE"))					pc->setRace(str2num(script2));
			else if (!strcmp(script1, "ROBE"))				pc->robe = str2num(script2);
			else if (!strcmp(script1, "REATTACKAT"))		pc->reattackat=str2num(script2);
		// no cells have been marked as -1, but no cell=0
		// with that line its not necassairy to take it manually out.
		break;

		case 'S':
			if (!strcmp(script1, "SERIAL"))
			{
				unsigned int i = str2num(script2);
				Npcs->DetermineHighestSerial(i);
				pc->SetSerial(i);
			}
			else if (!strcmp(script1, "SAY"))				pc->saycolor = static_cast<UI16>(str2num(script2));
			else if (!strcmp(script1, "STRENGTH"))			pc->setStr(str2num(script2));
			else if (!strcmp(script1, "STRENGTH2"))			pc->setDecStr(str2num(script2));
			else if (!strcmp(script1, "STRENGTHLIMIT"))		pc->setStrLimit(str2num(script2));
			else if (!strcmp(script1, "STAMINA"))			pc->setStm(str2num(script2));
			else if (!strcmp(script1, "SUMMONREMAININGSECONDS"))
			{//AntiChrist - let's restore remaining seconds!
				i=str2num(script2);
				pc->summontimer=uiCurrentTime+(i*MY_CLOCKS_PER_SEC);
			}
			else if (!strcmp(script1, "SHOP"))			  pc->shop=str2num(script2);
			else if (!strcmp(script1, "SPEECH"))			  pc->speech=str2num(script2);
			else if (script1[0]=='S' && script1[1]=='K' && script1[2]=='I' &&
				script1[3]=='L' && script1[4]=='L')
			{
				pc->setRealSkill(j=str2num(&script1[5]), str2num(script2));
				SkillManager->updateSkillLevel(pc, j);
			}
			else if (!strcmp(script1, "SKIN"))			  pc->skin = static_cast<UI16>(str2num(script2));
			else if (!strcmp(script1, "SPATTACK"))		  pc->spattack=str2num(script2);
			else if (!strcmp(script1, "SPADELAY"))		  pc->spadelay=str2num(script2);
			else if (!strcmp(script1, "SPLIT"))			  pc->split=str2num(script2);
			else if (!strcmp(script1, "SPLITCHANCE"))	  pc->splitchnc=str2num(script2);
			else if (!strncmp(script1, "SKL", 3 ))		// for skill locking
			{
				pc->lockSkill[j=str2num(&script1[3])]=str2num(script2);
			}
			else if (!strcmp(script1, "SPAWNREGION"))		pc->spawnregion = IncSpawnCount(str2num(script2));
			else if (!strcmp(script1, "SCRIPTNUM"))         pc->scriptnum=str2num(script2);
			else if (!strcmp(script1, "SPAWN"))				
			{
				bool ok = pc->SetSpawnSerial(str2num(script2));
				if (!ok) 
				{
					sprintf(temp, "Invalid Spawn serial:%i NPC-serial:%i name:%s, reset spawn value to -1\n", str2num(script2), pc->serial, pc->name);
					LogError(temp);
					pc->SetSpawnSerial(-1);
				}

			}
			else if (!strcmp(script1, "STABLEMASTER"))
			{
				pc->stablemaster_serial=str2num(script2);
				if (pc->stablemaster_serial>0)
					stablesp.insert(pc->stablemaster_serial, pc->serial);
				//setptr(&stablesp[pc->stablemaster_serial%HASHMAX], x);
			}
			else if (!strcmp(script1, "SUMMONED"))			pc->summoned=true;
		break;

		case 'T':
			if (!strcmp(script1, "TAMING"))					pc->totame=str2num(script2);
			else if (!strcmp(script1, "TOTAME"))			pc->totame=str2num(script2);
			else if (!strcmp(script1, "TAMEDAMOUNT"))       pc->tamedamount=str2num(script2);
			else if (!strcmp(script1, "TRIGGER"))			pc->trigger=str2num(script2);
			else if (!strcmp(script1, "TRIGWORD"))			strcpy(pc->trigword,script2);
			else if (!strcmp(script1, "TIME_UNUSED"))		pc->time_unused=str2num(script2);
		break;

		case 'X':
			if (!strcmp(script1, "X"))						pc->pos.x=str2num(script2);
			else if (!strcmp(script1, "XBODY"))				pc->setXid(str2num(script2));
			else if (!strcmp(script1, "XSKIN"))				pc->xskin = static_cast<UI16>(str2num(script2));
		break;

		case 'W':
			if (!strcmp(script1, "WAR"))					pc->war=str2num(script2);
		break;

		case 'Y':
			if (!strcmp(script1, "Y"))						pc->pos.y=str2num(script2);
		break;

		case 'Z':
			if (!strcmp(script1, "Z"))						pc->dispz=pc->pos.z=str2num(script2);
		break;
		}
		loops++;
	}
	while (strcmp(script1, "}") && loops<=200);
	if (pc->npc && pc->war) pc->war=0;
	
	
	////////////////////////////////////////////////////////////////////
	
	// LB autocorrect for same faulty previous code
	int zeta;
	int sk=SkillManager->getAllSkillsValue();
	for (zeta = 0;zeta<sk;zeta++) if (pc->lockSkill[zeta]!=0 && pc->lockSkill[zeta]!=1 && pc->lockSkill[zeta]!=2) pc->lockSkill[zeta]=0;
	
	//AntiChrist bugfix for hiding
	pc->RemovePermaFrozen(); // unhide - AntiChrist
	pc->hidden = 0;
	pc->stealth = -1;
	
	//AntiChrist bugfix for magic reflect
	pc->RemoveMagicReflect();
	
	
	pc->region = calcRegionFromXY(pc->pos); //LB bugfix
	
	pc->antispamtimer=0;//LB - AntiSpam -
	pc->antiguardstimer=0; //AntiChrist - AntiSpam for "GUARDS" call - to avoid (laggy) guards multi spawn
	// lord binarys body/skin autocorrection code
	k = pc->id();
	if (k<=0x3e1)
	{
		unsigned short color = pc->skin;
		if (((color&0x4000) && (k >=0x0190 && k<=0x03e1)) || color==0x8000)
		{
			if (color!=0xf000)
			{
				pc->skin = pc->xskin = 0xF000;
				Console.send("char/player: %s : %i correted problematic skin hue\n",pc->name,pc->serial);
			}
		}
	} else	// client crashing body --> delete if non player esle put onlx a warning on server screen
		// we dont want to delete that char, dont we ?
	{
		if (pc->account==-1)
		{
			Npcs->DeleteChar(pc);
		}
		else
		{
			pc->setId(0x0190);
			Console.send("player: %s with bugged body-value detected, restored to male shape\n",pc->name);
		}
	}
	
	//a = mapRegions->AddItem(x+CharacterOffset);
	if (pc->stablemaster_serial == 0)
	{ 
		mapRegions->Add(pc); 
	} 
	
	// begin of meta gm stuff
	
	for (int u=0;u<7;u++)
	{
		if (!pc->npc)
		{
			if (pc->priv3[u]==0) // dont overwrite alreday saved settings
			{
				if (!pc->isGMorCounselor()) pc->priv3[u]=metagm[2][u]; //normal player defaults
				if (pc->isCounselor()) pc->priv3[u]=metagm[1][u]; // couscelor defaults
				if (pc->isGM()) pc->priv3[u]=metagm[0][u]; // gm defaults
				if (pc->account==0) pc->priv3[u]=0xffffffff;
			}
		}
	}
	
	
	int MapNum = 0; // of course, this needs to replaced with real map num when multi maps implemented
	
	int max_x = AllMaps->GetMapWidth(MapNum)  * 8;
	int max_y = AllMaps->GetMapHeight(MapNum) * 8;
		
	if (pc->pos.x>=max_x || pc->pos.y>=max_y)
	{
		if (pc->account==-1)
		{	
			sprintf(temp, "INVALID COORDS !!!! maxX:%i maxY:%i x:%i y:%i name:%s\n", max_x, max_y, pc->pos.x, pc->pos.y, pc->name);
			LogError(temp);
			Npcs->DeleteChar(pc); //character in an invalid location
		} 
		else
		{
			sprintf(temp, "INVALID COORDS2 !!!! maxX:%i maxY:%i x:%i y:%i name:%s\n", max_x, max_y, pc->pos.x, pc->pos.y, pc->name);
			LogError(temp);
			cCoord nPos(900,300,30,DEFAULT_MAP);
			pc->MoveTo(nPos); //player in an invalid location
		}	  
	}
	
	setcharflag(pc);//AntiChrist
}

void loaditem() // Load an item from WSC
{
	unsigned long int i;
	int loops=0;
	char bad=0;

	const P_ITEM pi=Items->MakeBaseItem(false);
	if (!pi) return;

	/*pi->id1='\x0F';
	pi->id2='\xA6'; */

	// who did this why ?
	// 0xfa6 is a client crasher casue it doesnt exist..
	// changed to 1, LB

	pi->setId(0x0001);


	do
	{
		readw2();
		int rc = Items->ApplyCommonTags(pi);				// tags used in both scp and wsc
		if (rc) {if (rc==1) continue; else bad=1;}

		rc = Items->ApplyWscTags(pi);						// tags used only in wsc
		if (rc) {if (rc==1) continue; else bad=1;}

		switch (script1[0])
		{
		case 'C':
			if (!strcmp(script1, "CONT"))				pi->SetContSerial(str2num(script2));
			break;
		case 'Q':
			if (!strcmp(script1, "QSF"))				pi->qsf=str2num(script2);
			break;
		case 'L':
			if (!strcmp(script1, "LOCKDOWNMAX"))		pi->lockdownmax=str2num(script2);
			break;
		case 'O':
			if (!strcmp(script1, "OWNER"))				pi->SetOwnSerial(str2num(script2));
			break;
		case 'S':
			if (!strcmp(script1, "SERIAL"))
			{
				i=str2num(script2);
				Items->DetermineHighestSerial(i);
				pi->setSerial(i);
			}
			else if (!strcmp(script1, "SECUREMAX"))		pi->securemax=str2num(script2);
			else if (!strcmp(script1, "SPAWN"))			pi->SetSpawnSerial(str2num(script2));
			else if (!strcmp(script1, "SPAWNREGION"))	// new spawnregion recognition, LB
			{
				pi->spawnregion=IncSpawnCount(str2num(script2));
			}
			break;
		default:
			break;
		}
		loops++;
	}
	while (strcmp(script1, "}") && loops<=200);

	pi->timeused_last=getNormalizedTime();

	//add item weight if item doesn't have it yet
	if (pi->weight<=0) // LB, changed from 29 to 0
	{
		pi->weight=pi->getWeight();
	}
	
	if (pi->maxhp==0) pi->maxhp=pi->hp;
	// Tauriel adding region pointers
	
	if (pi->isInWorld())
	{
		int MapNum = 0; // of course, this needs to replaced with real map num when multi maps implemented
        int max_x = AllMaps->GetMapWidth(MapNum)  * 8;
        int max_y = AllMaps->GetMapHeight(MapNum) * 8;

		//printf("maxx:%i maxy:%i\n",max_x, max_y);
	
		mapRegions->Add(pi); // it reurns 1 if invalid, if invalid it DOESNT get added !!!
		if (pi->pos.x>=max_x || pi->pos.y>=max_y)	// lord bianry
		{
			sprintf(temp, "\nINVALID COORDS3 !!!! maxX:%i maxY:%i x:%i y:%i name:%s\n", max_x, max_y, pi->pos.x, pi->pos.y, pi->name);
			LogError(temp);
			Items->DeleItem(pi);	//these are invalid locations, delete them!
		}
	}
	if (bad) 
	{		
		Items->DeleItem(pi);
	}
}

void CWorldMain::loadnewworld() // Load world from WOLFPACK.WSC
{
	char outper[4];	// Magius(CHE) (1)
	unsigned int i=0;
	unsigned int percent=0,a=0,pred=0,maxm=0; // Magius(CHE) (1)
	*outper='\0';

	Console.send("Loading World, Building map Regions, checking Item weight...\n");

	wscfile=fopen("wppcs.wsc","r");
	if (!wscfile)
	{
		Console.send("WARNING: wppcs.wsc not found, creating new one with next worldsave\n");
		Npcs->AllocateMemory(679);	
	}
	else
	{
		//Get number for initial character memory needed ->
		int cnum=0;
		readw3();
		if (!strcmp(script1, "INITMEM")) cnum=str2num(script2) + 5000; // large initial memory as a q&d workaround for realloc prob (Duke)
		maxm=cnum; // Magius(CHE) (1)

		Npcs->AllocateMemory(cnum);			
	
		bool s=false;
		readw3();
		                                                        
        if (!strcmp(script1, "SECTION")) s=true; 

		Console.send("Loading characters ");	// MAgius(CHE) (1)
		int loopexit=0;
		do
		{
			if (!s) readw3(); else s=false;
			if (!strcmp(script1, "SECTION"))
			{
				if (!strcmp(script2, "CHARACTER"))
				{
					loadchar();
					a++;
					pred=percent; //Magius(CHE) (1)
					if (maxm==0) maxm++;
					percent=(int) (a*100)/(maxm);
					if (strlen(outper)>0)
					{ //Magius(CHE) (1)
						for (i=1;i<=strlen(outper)+1;i++) Console.send("\b");
						*outper='\0';
					}
				}
			}
			if (percent>pred) // Changed by MAgius(CHE) (1)
			{
				numtostr(percent,outper);
				Console.send("%s%%",outper);
				pred=percent;
			}
		} while (strcmp(script1,"EOF") && !feof(wscfile) && (++loopexit < MAXLOOPS*10) );
		fclose(wscfile);
		if (strlen(outper)>0)
		{ //Magius(CHE) (1)
			for (i=1;i<=strlen(outper)+1;i++) Console.send("\b");
			*outper='\0';
		}
		Console.send("Done.\n"); // Magius(CHE)
	}

	
	wscfile=fopen("wpitems.wsc", "r");
	if (!wscfile)
	{
		Console.send("ERROR: wpitems.wsc not found. No items will be loaded.\n");
		Items->AllocateMemory(2157);
	}
	else
	{
	   //Get number for initial character memory needed ->
			int inum=0;
			readw3();
			if (!strcmp(script1, "INITMEM"))
				inum=str2num(script2) + 5000; // large initial memory as a q&d workaround for realloc prob (Duke)
			maxm=inum; // Magius(CHE) (1)

			Items->AllocateMemory(inum);

			if (SrvParms->trade_system==1) Console.send("Loading items ");	// AntiChrist - sorry magius, but it's better to see in this way i think
			else Console.send("Loading items ");	// MAgius(CHE) (2)
			a=0; // Magius(CHE) (2)
			int loopexit=0;
			do {
				readw3();
				if (!strcmp(script1, "SECTION"))
				{
					if (!strcmp(script2, "WORLDITEM"))
					{
						loaditem();
					}
					else if (!strcmp(script2, "GUILD"))
						Guilds->Read(str2num(script3));
					a++;
					pred=percent; //Magius(CHE) (1)
					if (maxm==0) maxm++;
					percent=(int) (a*100)/(maxm);
					if (strlen(outper)>0) { //Magius(CHE) (1)
						for (i=1;i<=strlen(outper)+1;i++) Console.send("\b");
						*outper='\0';
					}
				}
				if (percent>pred) // Changed by MAgius(CHE) (1)
				{
					numtostr(percent,outper);
					Console.send("%s%%",outper);
					pred=percent;
				}
			} while (strcmp(script1, "EOF") && !feof(wscfile) && (++loopexit < MAXLOOPS*10) );
			fclose(wscfile);
		}
		if (strlen(outper)>0) { //Magius(CHE) (1)
			for (i=1;i<=strlen(outper)+1;i++) Console.send("\b");
			*outper='\0';

		}
		Console.send("Done.\n"); // Magius(CHE)
		Console.send("World Loaded.\n"); // Changed by MAgius(CHE) (1)
		return;	
}

/////////////////
// Name:	savenewworld
// Purpose:	Save current world state. Stores all values in an easily
//			readable script file "*.wsc". This stores all world items
//			and NPC/PC character information for a given shard
// History:	by Unknown
//
void CWorldMain::savenewworld(char x)
{
	static unsigned long ocCount, oiCount;

	worldsaved=true;

	AllTmpEff->Off();

	if ( !Saving() )
	{
		SrvParms->Save();
		if (SrvParms->server_log) savelog("Server data save\n","server.log");
		// Dupois - If shard op has defined an ARCHIVEPATH in SERVER.SCP
		//          then archive the WSC to the directory specified.
		// Added Oct 29, 1998
		if (strlen(SrvParms->archivepath)>1)
		{
			save_counter++; // LB - added world backup rate
			if (( save_counter % SrvParms->backup_save_ratio ) == 0 ) fileArchive("wppcs.wsc", "wpitems.wsc", SrvParms->archivepath);
		}
		// End - Dupois

		cWsc=fopen("wppcs.wsc", "w");
		if (cWsc == NULL) 
		{
			Console.send("Fatal Error, couldn't open wppcs.wsc for writing. Check file permissions,quitting\n");
			printf("Fatal Error, couldn't open wppcs.wsc for writing. Check file permissions,quitting\n");
			exit(-1);
		}
	
		fprintf(cWsc, "// LoneWolf World Script (WSC)[TEXT]\n");
		fprintf(cWsc, "// Generated by %s Version %s\n",  wp_version.betareleasestring.c_str() , wp_version.verstring.c_str() );
		fprintf(cWsc, "// This file replaces the function of the *.UOX files in pre-0.50 versions[TEXT]\n");
		fprintf(cWsc, "// LoneWolf CHARACTERS.\n");
		fprintf(cWsc, "INITMEM %i //Do NOT edit this line!\n", Npcs->GetCount());

		fprintf(cWsc, "\n\n");

		iWsc=fopen("wpitems.wsc", "w");
		if (iWsc == NULL)
		{
			Console.send("Fatal Error, couldn't open wpitems.wsc for writing. Check file permissions,quitting.\n");
			printf("Fatal Error, couldn't open wpitems.wsc for writing. Check file permissions,quitting.\n");
			exit(-2);
		}
	
		fprintf(iWsc, "// LoneWolf World Script (WSC)[TEXT]\n");
		fprintf(iWsc, "// Generated by %s Version %s\n",  wp_version.betareleasestring.c_str() , wp_version.verstring.c_str() );
		fprintf(iWsc, "// This file replaces the function of the *.UOX files in pre-0.27 versions[TEXT]\n");
		fprintf(iWsc, "// LoneWolf items and Guild information.\n");
		fprintf(iWsc, "INITMEM %i //Do NOT edit this line!\n",Items->GetCount()); // lb bugfix, cut& paste bug, was charcount ...	
		fprintf(iWsc, "\n\n"); 	

		oiCount = Items->GetCount();
		ocCount = Npcs->GetCount();
		Cur = 0;
		isSaving = true;
	}

	Max = max(oiCount, ocCount);

	P_ITEM piDefault = new cItem;	// create an item with the default values
	piDefault->Init(0);
	static int moditems = 0;			// # of items with archive bit on

	for ( ; Cur<Max ; Cur++)
	{
		if ( Cur < ocCount )
			SaveChar( Cur );
		if ( Cur < oiCount )
		{
			P_ITEM pi = MAKE_ITEM_REF(Cur);
			if (!pi) continue;
			if (pi->HasChanged())
				moditems++;
			SaveItem(pi, piDefault);
		}
	}
	if ( Cur == max(oiCount, ocCount) )
	{ //Done saving
		fprintf(cWsc,"\nEOF\n\n");
		fclose(cWsc);

		Guilds->Write( iWsc );
		fprintf(iWsc, "EOF\n\n");
		fclose(iWsc);

		Cur = Max = ocCount = oiCount = 0;
		isSaving = false;

		iWsc = cWsc = NULL;
	}

	AllTmpEff->On();
	uiCurrentTime = getNormalizedTime();

	Traffic->doStatistics(); // give out traffic/packets statistics each worldsave
	char chg[222];
	sprintf(chg,"Saved %i modified items\n",moditems);
//	Console.send(chg);

	delete piDefault; // (small) memory leak fix !!!

}

int CWorldMain::announce()
{
	return DisplayWorldSaves;
}

void CWorldMain::announce(int choice)
{
	if (choice<1)
		DisplayWorldSaves=0;
	else
		DisplayWorldSaves=1;
}

bool CWorldMain::Saving( void )
{
	return isSaving;
}

void CWorldMain::SaveChar( CHARACTER i )
{
	int j;
	P_CHAR pc = MAKE_CHAR_REF(i);
	if (!pc) return;
	P_CHAR Default = new cChar;
	Default->Init(false);
	
	char valid=1;
	if (!SrvParms->savespawns && pc->spawnregion>0) valid=0;
	if (valid)
	{
		fprintf(cWsc, "SECTION CHARACTER %i\n", i);
		fprintf(cWsc, "{\n");
		fprintf(cWsc, "SERIAL %i\n", pc->serial);
		//AntiChrist - special incognito related stuff - 12/99
		if (pc->incognito)
		{//save original name
			fprintf(cWsc, "NAME %s\n", pc->orgname);
		} else
		{
			fprintf(cWsc, "NAME %s\n", pc->name);
		}
		
		fprintf(cWsc, "TITLE %s\n", pc->title);
		if (pc->account > -1)
		{
			fprintf(cWsc, "ACCOUNT %i\n", pc->account);
			fprintf(cWsc, "CREATIONDAY %i\n", pc->creationday);
		}

		if (pc->gmMoveEff	!= Default->gmMoveEff)		fprintf(cWsc, "GMMOVEEFF %i\n", pc->gmMoveEff);
		if (pc->GuildType	!= Default->GuildType)
			fprintf(cWsc,"GUILDTYPE %i\n",pc->GuildType);
		else
			fprintf(cWsc,"GUILDTYPE -1\n");	//	Make sure that it gets written in oldchars made before this fix
		if (pc->GuildTraitor)
			fprintf(cWsc,"GUILDTRAITOR YES\n");
		else
			fprintf(cWsc,"GUILDTRAITOR NO\n");	//	Make sure that it gets written to for old chars made before this fix		
		if (pc->qsf   != Default->qsf)					fprintf(cWsc, "QSF %i\n", pc->qsf);
		if (pc->pos.x != Default->pos.x)				fprintf(cWsc, "X %i\n", pc->pos.x);
		if (pc->pos.y != Default->pos.y)				fprintf(cWsc, "Y %i\n", pc->pos.y);
		if (pc->pos.z != Default->pos.z)				fprintf(cWsc, "Z %i\n", pc->pos.z);
		if (pc->pos.map != Default->pos.map)			fprintf(cWsc, "MAP %i\n", pc->pos.map);
		if (pc->dispz != Default->dispz)				fprintf(cWsc, "DISPZ %i\n", pc->dispz);
		if (pc->cell  != Default->cell)
		{
			fprintf(cWsc, "JAILCELL %i\n", pc->cell);
			fprintf(cWsc, "OLDX %i\n", jails[pc->cell].oldpos.x);
			fprintf(cWsc, "OLDY %i\n", jails[pc->cell].oldpos.y);
			fprintf(cWsc, "OLDZ %i\n", jails[pc->cell].oldpos.z);
		}
		if (pc->dir != Default->dir)					fprintf(cWsc, "DIR %i\n", pc->dir);
		if (pc->getRace() != Default->getRace())		fprintf(cWsc, "RACE %i\n", pc->getRace());
		//AntiChrist - incognito and polymorph spell special stuff - 12/99
		if (pc->incognito || pc->polymorph)
		{//if under incognito spell, don't save BODY but the original XBODY
			if (pc->xid())
				fprintf(cWsc, "BODY %i\n", pc->xid());
		} else
		{//else backup body normally
			if (pc->id() != Default->id())
				fprintf(cWsc, "BODY %i\n", pc->id());
		}
		if (pc->xid())
			fprintf(cWsc, "XBODY %i\n", pc->xid());
		//AntiChrist - incognito spell special stuff - 12/99
		if (pc->incognito)
		{//if under incognito spell, don't save SKIN but the original XSKIN
			if (pc->xskin)
				fprintf(cWsc, "SKIN %i\n", pc->xskin);
		} else
		{//else backup skin normally
			if (pc->skin)
				fprintf(cWsc, "SKIN %i\n", pc->skin);
		}
		
		if (pc->xskin)
			fprintf(cWsc, "XSKIN %i\n", pc->xskin);
		if (pc->getPriv()	!= Default->getPriv())			fprintf(cWsc, "PRIV %i\n", pc->getPriv());
		if (pc->GetPriv2()	!= Default->GetPriv2())			fprintf(cWsc, "PRIV2 %i\n", pc->GetPriv2());
		if (pc->GetPriv4()	!= Default->GetPriv4())			fprintf(cWsc, "PRIV4 %i\n", pc->GetPriv4());
		
		// beginn of meta-gm save
		if (pc->priv3[0] != Default->priv3[0])				fprintf(cWsc, "PRIV3a %i\n", pc->priv3[0]);
		if (pc->priv3[1] != Default->priv3[1])				fprintf(cWsc, "PRIV3b %i\n", pc->priv3[1]);
		if (pc->priv3[2] != Default->priv3[2])				fprintf(cWsc, "PRIV3c %i\n", pc->priv3[2]);
		if (pc->priv3[3] != Default->priv3[3])				fprintf(cWsc, "PRIV3d %i\n", pc->priv3[3]);
		if (pc->priv3[4] != Default->priv3[4])				fprintf(cWsc, "PRIV3e %i\n", pc->priv3[4]);
		if (pc->priv3[5] != Default->priv3[5])				fprintf(cWsc, "PRIV3f %i\n", pc->priv3[5]);
		if (pc->priv3[6] != Default->priv3[6])				fprintf(cWsc, "PRIV3g %i\n", pc->priv3[6]);
		// end of meta-gm save
		
		if (pc->stablemaster_serial != Default->stablemaster_serial)	    fprintf(cWsc, "STABLEMASTER %i\n", pc->stablemaster_serial);
		if (pc->summoned)	fprintf(cWsc, "SUMMONED\n");
		if (pc->npc_type	!= Default->npc_type)			fprintf(cWsc, "NPCTYPE %i\n", pc->npc_type);
		if (pc->time_unused != Default->time_unused)		fprintf(cWsc, "TIME_UNUSED %lu\n", pc->time_unused);
		
		if (pc->fonttype	!= Default->fonttype)			fprintf(cWsc, "FONT %i\n", pc->fonttype);
		if (pc->saycolor	!= Default->saycolor)			fprintf(cWsc, "SAY %i\n", pc->saycolor);
		if (pc->emotecolor	!= Default->emotecolor)			fprintf(cWsc, "EMOTE %i\n", pc->emotecolor);
		if (pc->realStr()	!= Default->realStr())			fprintf(cWsc, "STRENGTH %i\n", pc->realStr());
		if (pc->decStr()	!= Default->decStr())			fprintf(cWsc, "STRENGTH2 %i\n", pc->decStr());
		if (pc->StrLimit()	!= Default->StrLimit())			fprintf(cWsc, "STRENGTHLIMIT %i\n", pc->StrLimit());
		if (pc->realDex()	!= Default->realDex())			fprintf(cWsc, "DEXTERITY %i\n", pc->realDex());
		if (pc->decDex()	!= Default->decDex())			fprintf(cWsc, "DEXTERITY2 %i\n", pc->decDex());
		if (pc->DexLimit()	!= Default->DexLimit())			fprintf(cWsc, "DEXTERITYLIMIT %i\n", pc->DexLimit());
		if (pc->realInt()	!= Default->realInt())			fprintf(cWsc, "INTELLIGENCE %i\n", pc->realInt());
		if (pc->decInt()	!= Default->decInt())			fprintf(cWsc, "INTELLIGENCE2 %i\n", pc->decInt());
		if (pc->IntLimit()	!= Default->IntLimit())			fprintf(cWsc, "INTELLIGENCELIMIT %i\n", pc->IntLimit());
		if (pc->Hp()		!= Default->Hp())				fprintf(cWsc, "HITPOINTS %i\n", pc->Hp());
		if (pc->spawnregion != Default->spawnregion)		fprintf(cWsc, "SPAWNREGION %i\n", pc->spawnregion);
		if (pc->Stm()		!= Default->Stm())				fprintf(cWsc, "STAMINA %i\n", pc->Stm());
		if (pc->mn			!= Default->mn)					fprintf(cWsc, "MANA %i\n", pc->mn);
		if (pc->npc			!= Default->npc)				fprintf(cWsc, "NPC %i\n", pc->npc);
		if (pc->holdg		!= Default->holdg)				fprintf(cWsc, "HOLDGOLD %i\n", pc->holdg); // bugfix lb, holdgold value never saved !!!
		if (pc->scriptnum	!= Default->scriptnum)			fprintf(cWsc, "SCRIPTNUM %i\n", pc->scriptnum); 
		
		if (pc->shop		!= Default->shop)				fprintf(cWsc, "SHOP %i\n", pc->shop);
		if (pc->ownserial	!= Default->ownserial)			fprintf(cWsc, "OWN %i\n", pc->ownserial);
		if (pc->robe		!= Default->robe)				fprintf(cWsc, "ROBE %i\n", pc->robe);
		if (pc->karma		!= Default->karma)				fprintf(cWsc, "KARMA %i\n", pc->karma);
		if (pc->fame		!= Default->fame)				fprintf(cWsc, "FAME %i\n", pc->fame);
		if (pc->kills		!= Default->kills)				fprintf(cWsc, "KILLS %i\n", pc->kills);
		if (pc->deaths		!= Default->deaths)				fprintf(cWsc, "DEATHS %i\n", pc->deaths);
		if (pc->dead		!= Default->dead)				fprintf(cWsc, "DEAD 1\n");
		if (pc->fixedlight	!= Default->fixedlight)			fprintf(cWsc, "FIXEDLIGHT %i\n", pc->fixedlight);
		if (pc->speech		!= Default->speech)				fprintf(cWsc, "SPEECH %i\n", pc->speech);
		if (pc->trigger		!= Default->trigger)			fprintf(cWsc, "TRIGGER %i\n", pc->trigger);
		if (strcmp(pc->trigword, Default->trigword))		fprintf(cWsc, "TRIGWORD %s\n", pc->trigword);
		if (strcmp(pc->disabledmsg, Default->disabledmsg))	fprintf(cWsc, "DISABLEMSG %s\n", pc->disabledmsg); // Added by Magius(CHE) 

	    if (pc->isclone)                                    fprintf(cWsc, "CLONED\n");
				
		for (j=0;j<ALLSKILLSMAX;j++)  // note: all skills, even if in non aos mode intentional, to avoid losing 3 new aos skills when switch back and forth aos mode
		{
			// Don't save the default value given by initchar
			if (pc->getRealSkill(j) != Default->getRealSkill(j)) 
				fprintf(cWsc, "SKILL%i %i\n", j, pc->getRealSkill(j));
			
			if ( pc->lockSkill[j] != Default->lockSkill[j] ) 
				fprintf(cWsc, "SKL%i %i\n", j, pc->lockSkill[j] );
		}

		if (!pc->cantrain)
			fprintf(cWsc, "NOTRAIN\n");
		else
			fprintf(cWsc, "CANTRAIN\n");
	
		if (pc->tamedamount != Default->tamedamount)		fprintf(cWsc, "TAMEDAMOUNT %i\n", pc->tamedamount);
		if (pc->att			!= Default->att)				fprintf(cWsc, "ATT %i\n", pc->att);
		if (pc->def			!= Default->def)				fprintf(cWsc, "DEF %i\n", pc->def);
		if (pc->lodamage	!= Default->lodamage)			fprintf(cWsc, "LODAMAGE %i\n", pc->lodamage);
		if (pc->hidamage	!= Default->hidamage)			fprintf(cWsc, "HIDAMAGE %i\n", pc->hidamage);
		if (pc->war			!= Default->war)				fprintf(cWsc, "WAR %i\n", pc->war);
		if (pc->npcWander	!= Default->npcWander)			fprintf(cWsc, "NPCWANDER %i\n", pc->npcWander);
		if (pc->oldnpcWander!= Default->oldnpcWander)		fprintf(cWsc, "OLDNPCWANDER %i\n", pc->oldnpcWander);
		if (pc->carve		!= Default->carve)				fprintf(cWsc, "CARVE %i\n", pc->carve);
		if (pc->fx1			!= Default->fx1)				fprintf(cWsc, "FX1 %i\n", pc->fx1);
		if (pc->fy1			!= Default->fy1)				fprintf(cWsc, "FY1 %i\n", pc->fy1);
		if (pc->fz1			!= Default->fz1)				fprintf(cWsc, "FZ1 %i\n", pc->fz1);
		if (pc->fx2			!= Default->fx2)				fprintf(cWsc, "FX2 %i\n", pc->fx2);
		if (pc->fy2			!= Default->fy2)				fprintf(cWsc, "FY2 %i\n", pc->fy2);
		if (pc->spawnserial != Default->spawnserial)		fprintf(cWsc, "SPAWN %i\n", pc->spawnserial);
		if (pc->hidden		!= Default->hidden)				fprintf(cWsc, "HIDDEN %i\n", pc->hidden);
		if (pc->Hunger()	!= Default->Hunger())			fprintf(cWsc, "HUNGER %i\n", pc->Hunger());
		if (pc->npcaitype	!= Default->npcaitype)			fprintf(cWsc, "NPCAITYPE %i\n", pc->npcaitype);
		if (pc->spattack	!= Default->spattack)			fprintf(cWsc, "SPATTACK %i\n", pc->spattack);
		if (pc->spadelay	!= Default->spadelay)			fprintf(cWsc, "SPADELAY %i\n", pc->spadelay);
		if (pc->totame		!= Default->totame)				fprintf(cWsc, "TOTAME %i\n", pc->totame);
		if (pc->summontimer != Default->summontimer)		fprintf(cWsc, "SUMMONREMAININGSECONDS %i\n", (pc->summontimer/MY_CLOCKS_PER_SEC));
		
		if (pc->advobj		!= Default->advobj)				fprintf(cWsc, "ADVOBJ %i\n", pc->advobj);
		if (pc->Poison()	!= Default->Poison())			fprintf(cWsc, "POISON %i\n", pc->Poison());
		if (pc->Poisoned()	!= Default->Poisoned())			fprintf(cWsc, "POISONED %i\n", pc->Poisoned());
		if (pc->fleeat		!= Default->fleeat)				fprintf(cWsc, "FLEEAT %i\n", pc->fleeat);
		if (pc->reattackat	!= Default->reattackat)			fprintf(cWsc, "REATTACKAT %i\n", pc->reattackat);
		if (pc->split		!= Default->split)				fprintf(cWsc, "SPLIT %i\n", pc->split);
		if (pc->splitchnc	!= Default->splitchnc)			fprintf(cWsc, "SPLITCHANCE %i\n", pc->splitchnc);
		// Begin of Guild related things (DasRaetsel)
		if (pc->guildtoggle != Default->guildtoggle)		fprintf(cWsc, "GUILDTOGGLE %i\n", pc->guildtoggle);  
		if (pc->guildnumber != Default->guildnumber)		fprintf(cWsc, "GUILDNUMBER %i\n", pc->guildnumber);  
		if (strcmp(pc->guildtitle, Default->guildtitle))	fprintf(cWsc, "GUILDTITLE %s\n", pc->guildtitle);  
		if (pc->guildfealty != Default->guildfealty)		fprintf(cWsc, "GUILDFEALTY %i\n", pc->guildfealty);  
		// Dupois - Escort quests
		if (pc->questType != Default->questType)				fprintf(cWsc, "QUESTTYPE %i\n", pc->questType);  
		if (pc->questDestRegion != Default->questDestRegion)	fprintf(cWsc, "QUESTDESTREGION %i\n", pc->questDestRegion);  
		if (pc->questOrigRegion != Default->questOrigRegion)	fprintf(cWsc, "QUESTORIGREGION %i\n", pc->questOrigRegion);
		if (pc->questBountyPostSerial != Default->questBountyPostSerial)		fprintf(cWsc, "QUESTBOUNTYPOSTSERIAL %i\n", pc->questBountyPostSerial);
		if (pc->questBountyReward != Default->questBountyReward)				fprintf(cWsc, "QUESTBOUNTYREWARD %i\n", pc->questBountyReward);
		if (pc->jailtimer.getRemainingSecs())				fprintf(cWsc, "JAILTIMER %i\n", pc->jailtimer.getRemainingSecs()); 
		if (pc->gmrestrict != Default->gmrestrict)			fprintf(cWsc, "GMRESTRICT %i\n", pc->gmrestrict); 

		if (pc->doppelgaenger_serial != Default->doppelgaenger_serial)		fprintf(cWsc, "DOPPELGAENGERSERIAL %i\n", pc->doppelgaenger_serial); 
		if (pc->GetFood() != Default->GetFood())			fprintf(cWsc, "FOOD %i\n", pc->GetFood()); 

		P_CHAR Follow = pc->GetFollowTarget();
		if (Follow)	fprintf(cWsc, "FOLLOW %i\n", Follow->serial);

		fprintf(cWsc, "}\n\n");	
	}

	delete Default;
	Default = NULL;
}

#if 0
	#define save_int(a,b) fprintf(iWsc,"%s %i\n",a,b)
	#define save_str(a,b) fprintf(iWsc,"%s %s\n",a,b)
	#define save_txt(txt) fprintf(iWsc,"%s\n",txt)
	#define save_sect
#else
//	char *sp;
	#define save_int(a,b)	      sp+=sprintf(sp,"%s %i\n",a,b)
    #define save_unsignedint(a,b) sp+=sprintf(sp,"%s %lu\n",a,b)
	#define save_str(a,b)	      sp+=sprintf(sp,"%s %s\n",a,b)
	#define save_txt(txt)	      sp+=sprintf(sp,"%s\n",txt)
	//#define save_sect()		fputs(sectbuf,iWsc)
	#define save_sect()		      save_sect2(sectbuf,iWsc)

	void save_sect2(char *sectbuf,FILE *iWsc)
	{
		fputs(sectbuf,iWsc);
	}

#endif

/*
static void decay1(P_ITEM pi, int i)
{
	long serial, serhash, j;
	int k, ci;
	if (pi->corpse==1)
	{
		serial=pi->serial;
		serhash=serial%HASHMAX;
		for (ci=0;ci<contsp[serhash].max;ci++)
		{
			j=contsp[serhash].pointer[ci];
			if ( j != -1 )
			{
				P_ITEM pi_j=MAKE_ITEM_REF(i);
				if (!pi_j) continue;
				if (pi->Contains(pi_j) &&
					pi_j->layer!=0x0B && pi_j->layer!=0x10)
				{
					pi_j->SetContSerial(-1);
					pi_j->MoveTo(pi->pos);
					for (k=0;k<now;k++)
					{
						if (perm[k] && inrange2(k, pi_j))
						{
							senditem(k,pi_j);
						}
					}
				}
				if ((pi->Contains(pi_j)) &&
					(pi_j->layer==0x0B || pi_j->layer==0x10))
				{
					Items->DeleItem(pi_j);
				}
			}
		}
	}
	else
	{
		if ( pi->multis == -1 )
		{
			if ( !findMulti( pi->pos ) )
			{
				Items->DeleItem(pi);
			}
		}
	}
}
*/

////////////
// name:	swapDragInfo
// pupose:	exchange current and OLD Position of an item
// history:	by Duke, 17.10.01
//
void swapDragInfo(P_ITEM pi)
{
	cCoord tmpPos;
	int tmpSer;
	signed char tmpLayer;

	tmpPos=pi->pos;
	tmpSer=pi->contserial;
	tmpLayer=pi->layer;

	pi->pos=pi->oldpos;
	pi->contserial=pi->oldcontserial;
	pi->layer=pi->oldlayer;

	pi->oldpos=tmpPos;
	pi->oldcontserial=tmpSer;
	pi->oldlayer=tmpLayer;
}

void CWorldMain::SaveTags(PC_ITEM pi, P_ITEM Default, char* &sp)
{
	save_str("NAME", pi->name);
	save_int("ID", pi->id());
	
	if (strcmp(pi->name2,Default->name2))			{save_str("NAME2",		pi->name2);}
	if (pi->pos.x		!= Default->pos.x)			{save_int("X",			pi->pos.x);}
	if (pi->pos.y		!= Default->pos.y)			{save_int("Y",			pi->pos.y);}
	if (pi->pos.z		!= Default->pos.z)			{save_int("Z",			pi->pos.z);}
	if (pi->pos.map		!= Default->pos.map)		{save_int("MAP",		pi->pos.map);}
	if (pi->color()		!= Default->color())		{save_int("COLOR",		pi->color());}
	if (pi->layer		!= Default->layer)			{save_int("LAYER",		pi->layer);}
	if (pi->itmhand		!= Default->itmhand)		{save_int("ITEMHAND",	pi->itmhand);}
	if (pi->type		!= Default->type)			{save_int("TYPE",		pi->type);}
	if (pi->type2		!= Default->type2)			{save_int("TYPE2",		pi->type2);}
	if (pi->offspell	!= Default->offspell)		{save_int("OFFSPELL",	pi->offspell);}
	if (pi->getMore1_4()!= Default->getMore1_4())	{save_int("MORE",		pi->getMore1_4());}
	if (pi->getMoreB1_4()!=Default->getMoreB1_4())	{save_int("MORE2",		pi->getMoreB1_4());}
	if (pi->morex		!= Default->morex)			{save_int("MOREX",		pi->morex);}
	if (pi->morey		!= Default->morey)			{save_int("MOREY",		pi->morey);}
	if (pi->morez		!= Default->morez)			{save_int("MOREZ",		pi->morez);}
	if (pi->amount		!= Default->amount)			{save_int("AMOUNT",		pi->amount);}
	if (pi->pileable	!= Default->pileable)		{save_int("PILEABLE",	pi->pileable);}
	if (pi->doordir		!= Default->doordir)		{save_int("DOORFLAG",	pi->doordir);}
	if (pi->dye			!= Default->dye)			{save_int("DYEABLE",	pi->dye);}
	if (pi->decaytimer.IsRunning())					{save_int("DECAYTIME",	pi->decaytimer.getRemainingSecs()*MY_CLOCKS_PER_SEC);}
	if (pi->corpse		!= Default->corpse)			{save_int("CORPSE",		pi->corpse);}
	if (pi->att			!= Default->att)			{save_int("ATT",		pi->att);}
	if (pi->def			!= Default->def)			{save_int("DEF",		pi->def);}
	if (pi->hidamage	!= Default->hidamage)		{save_int("HIDAMAGE",	pi->hidamage);}
	if (pi->lodamage	!= Default->lodamage)		{save_int("LODAMAGE",	pi->lodamage);}
	if (pi->racehate	!= Default->racehate)		{save_int("RACEHATE",	pi->racehate);}
	if (pi->st			!= Default->st)				{save_int("ST",			pi->st);}
	if (pi->time_unused != Default->time_unused)	{save_unsignedint("TIME_UNUSED",pi->time_unused);} 
	if (pi->weight		!= Default->weight)			{save_int("WEIGHT",		pi->weight);}
	if (pi->hp			!= Default->hp)				{save_int("HP",			pi->hp);}
	if (pi->maxhp		!= Default->maxhp)			{save_int("MAXHP",		pi->maxhp );}
	if (pi->rank		!= Default->rank)			{save_int("RANK",		pi->rank );}
	if (pi->st2			!= Default->st2)			{save_int("ST2",		pi->st2);}
	if (pi->dx			!= Default->dx)				{save_int("DX",			pi->dx);}
	if (pi->dx2			!= Default->dx2)			{save_int("DX2",		pi->dx2);}
	if (pi->in			!= Default->in)				{save_int("IN",			pi->in);}
	if (pi->in2			!= Default->in2)			{save_int("IN2",		pi->in2);}
	if (pi->trigon		!= Default->trigon)			{save_int("TRIGON",		pi->trigon);}//-Frazurbluu- part of the ACC replacement
	if (pi->spd			!= Default->spd)			{save_int("SPD",		pi->spd);}
	if (pi->Poisoned()	!= Default->Poisoned())		{save_int("POISONED",	pi->Poisoned());}
	if (pi->wipe		!= Default->wipe)			{save_int("WIPE",		pi->wipe);}
	if (pi->getMovability()	!= Default->getMovability()){save_int("MOVABLE",pi->getMovability());}
	if (pi->visible		!= Default->visible)		{save_int("VISIBLE",	pi->visible);}
	if (pi->dir			!= Default->dir)			{save_int("DIR",		pi->dir);}
	if (pi->getPriv()	!= Default->getPriv())		{save_int("PRIV",		pi->getPriv());}
	if (pi->value		!= Default->value)			{save_int("VALUE",		pi->value);}
	if (pi->restock		!= Default->restock)		{save_int("RESTOCK",	pi->restock);}
	if (pi->trigger		!= Default->trigger)		{save_int("TRIGGER",	pi->trigger);}
	if (pi->trigtype	!= Default->trigtype)		{save_int("TRIGTYPE",	pi->trigtype);}
	if (pi->tuses		!= Default->tuses)			{save_int("USES",		pi->tuses);}
	if (pi->good		!= Default->good)			{save_int("GOOD",		pi->good);} // Magius(CHE)
	if (pi->secureIt	!= Default->secureIt)		{save_int("SECUREIT",	pi->secureIt);}
	if (pi->glow		!= Default->glow)			{save_int("GLOW",		pi->glow);}
	if (pi->glow_color	!= Default->glow_color)		{save_int("GLOWBC",		pi->glow_color);}
	if (pi->glow_effect != Default->glow_effect)	{save_int("GLOWTYPE",	pi->glow_effect);}
	if (strlen(pi->desc)>0)							{save_str("DESC",		pi->desc);}	// save out our vendor description
	if (pi->lockdownmax != Default->lockdownmax)	{save_int("LOCKDOWNMAX",pi->lockdownmax);}
	if (pi->securemax	!= Default->securemax)		{save_int("SECUREMAX",	pi->securemax);}
}

void CWorldMain::SaveItem(P_ITEM pi, P_ITEM Default)
{
	if (!pi) return;

	if (pi->decays() && pi->isInWorld())
	{
		if (!pi->decaytimer.IsRunning())
		{
			pi->startDecay();
		}
		else if (pi->decaytimer.hasExpired())
		{
		//	decay1(pi,i);
		}
	}

	char sectbuf[4000];
	if (!SrvParms->savespawns && pi->spawnregion>0) // if it was spawned by spawregions AND save spawnregions set to 0,
		return;		// dont save
	if (pi->flags.isBeeingDragged == true)	// dragged items need to be saved at OLD position
	{
		swapDragInfo(pi);
	}

	int MapNum=0; // get world # , needs to be changed if multi map support gets in
	int max_x = AllMaps->GetMapWidth(MapNum) * 8;
	int max_y = AllMaps->GetMapHeight(MapNum) * 8;

	char valid=1;
	if (pi->isInWorld())
	{
		if (pi->pos.y>=max_y) valid=0;
		if (pi->pos.x>=max_x) valid=0;
	}
	
	if (valid)
	{
		char* sp=sectbuf;
		save_int("SECTION WORLDITEM", pi->GetIndex());
		save_txt("{");
		save_int("SERIAL", pi->serial);
		SaveTags(pi,Default,sp);

		if (strlen(pi->creator)>0)						{save_str("CREATOR",	pi->creator );} // by Magius(CHE)
		if (pi->madewith	!= Default->madewith)		{save_int("SK_MADE",	pi->madewith );} // by Magius(CHE)
		if (pi->qsf			!= Default->qsf)			{save_int("QSF",		pi->qsf);}
		if (pi->contserial	!= Default->contserial)		{save_int("CONT",		pi->contserial);}
		if (pi->ownserial	!= Default->ownserial)		{save_int("OWNER",		pi->ownserial);}
		if (pi->spawnserial != -1)						{save_int("SPAWN",		pi->spawnserial);}
		if (pi->disabled	!= Default->disabled)		
		{
			int remaining = pi->disabled <= uiCurrentTime ? 0 : (pi->disabled-uiCurrentTime)/MY_CLOCKS_PER_SEC;
			save_int("DISABLED",remaining);
		}
		if (pi->spawnregion	!= Default->spawnregion)	{save_int("SPAWNREGION",pi->spawnregion);}
		
		save_txt("}\n");
		save_sect();
	}

	if (pi->flags.isBeeingDragged == true)	// dragged items need to be saved at OLD position
	{
		swapDragInfo(pi);		// swap it back
	}
}

